@Bean
public FilterChainProxy springSecurityFilterChain() throws Exception {
    BasicAuthenticationEntryPoint entryPoint = new BasicAuthenticationEntryPoint();
    entryPoint.setRealmName ( "AppName Realm" );
    List<AccessDecisionVoter> voters = Arrays.<AccessDecisionVoter>asList ( new RoleVoter(), new WebExpressionVoter() );
    AccessDecisionManager accessDecisionManager = new AffirmativeBased ( voters );
    SecurityExpressionHandler<FilterInvocation> securityExpressionHandler = new DefaultWebSecurityExpressionHandler();
    UserDetailsByNameServiceWrapper<PreAuthenticatedAuthenticationToken> authenticationUserDetailsService = new UserDetailsByNameServiceWrapper<PreAuthenticatedAuthenticationToken> ( authUserDetailService );
    authenticationUserDetailsService.afterPropertiesSet();
    PreAuthenticatedAuthenticationProvider preAuthenticatedAuthenticationProvider = new PreAuthenticatedAuthenticationProvider();
    preAuthenticatedAuthenticationProvider.setPreAuthenticatedUserDetailsService ( authenticationUserDetailsService );
    preAuthenticatedAuthenticationProvider.afterPropertiesSet();
    List<AuthenticationProvider> providers = Arrays.<AuthenticationProvider>asList ( preAuthenticatedAuthenticationProvider );
    AuthenticationManager authenticationManager = new ProviderManager ( providers );
    HttpSessionSecurityContextRepository httpSessionSecurityContextRepository = new HttpSessionSecurityContextRepository();
    SessionRegistry sessionRegistry = new SessionRegistryImpl();
    ConcurrentSessionControlStrategy concurrentSessionControlStrategy = new ConcurrentSessionControlStrategy ( sessionRegistry );
    ConcurrentSessionFilter concurrentSessionFilter = new ConcurrentSessionFilter ( sessionRegistry );
    concurrentSessionFilter.afterPropertiesSet();
    SecurityContextPersistenceFilter securityContextPersistenceFilter = new SecurityContextPersistenceFilter ( httpSessionSecurityContextRepository );
    X509AuthenticationFilter x509AuthenticationFilter = new X509AuthenticationFilter();
    x509AuthenticationFilter.setAuthenticationManager ( authenticationManager );
    x509AuthenticationFilter.afterPropertiesSet();
    RequestCacheAwareFilter requestCacheAwareFilter = new RequestCacheAwareFilter();
    SecurityContextHolderAwareRequestFilter securityContextHolderAwareRequestFilter = new SecurityContextHolderAwareRequestFilter();
    SessionManagementFilter sessionManagementFilter = new SessionManagementFilter ( httpSessionSecurityContextRepository, concurrentSessionControlStrategy );
    ExceptionTranslationFilter exceptionTranslationFilter = new ExceptionTranslationFilter ( entryPoint );
    exceptionTranslationFilter.setAccessDeniedHandler ( new AccessDeniedHandlerImpl() );
    exceptionTranslationFilter.afterPropertiesSet();
    FilterSecurityInterceptor filterSecurityInterceptor = new FilterSecurityInterceptor();
    filterSecurityInterceptor.setAuthenticationManager ( authenticationManager );
    filterSecurityInterceptor.setAccessDecisionManager ( accessDecisionManager );
    LinkedHashMap<RequestMatcher, Collection<ConfigAttribute>> map = new LinkedHashMap<RequestMatcher, Collection<ConfigAttribute>>();
    map.put ( new AntPathRequestMatcher ( "/**" ), Arrays.<ConfigAttribute>asList ( new SecurityConfig ( "isAuthenticated()" ) ) );
    ExpressionBasedFilterInvocationSecurityMetadataSource ms = new ExpressionBasedFilterInvocationSecurityMetadataSource ( map, securityExpressionHandler );
    filterSecurityInterceptor.setSecurityMetadataSource ( ms );
    filterSecurityInterceptor.afterPropertiesSet();
    SecurityFilterChain chain = new DefaultSecurityFilterChain ( new AntPathRequestMatcher ( "/**" ),
            concurrentSessionFilter,
            securityContextPersistenceFilter,
            x509AuthenticationFilter,
            requestCacheAwareFilter,
            securityContextHolderAwareRequestFilter,
            sessionManagementFilter,
            exceptionTranslationFilter,
            filterSecurityInterceptor );
    return new FilterChainProxy ( chain );
}
