public class StackOverflow_10037379 {
    private static Logger sLogger = Logger.getLogger ( StackOverflow_10037379.class.getName() );
    public static class CLIPoolableObjectFactory extends BasePoolableObjectFactory<Process> {
        private String mProcessToRun;
        public CLIPoolableObjectFactory ( String processToRun ) {
            mProcessToRun = processToRun;
        }
        @Override
        public Process makeObject() throws Exception {
            ProcessBuilder builder = new ProcessBuilder();
            builder.redirectError ( Redirect.INHERIT );
            builder.redirectOutput ( Redirect.INHERIT );
            builder.redirectInput ( Redirect.PIPE );
            builder.command ( mProcessToRun );
            return builder.start();
        }
        @Override
        public boolean validateObject ( Process process ) {
            try {
                process.exitValue();
                return false;
            } catch ( IllegalThreadStateException ex ) {
                return true;
            }
        }
        @Override
        public void destroyObject ( Process process ) throws Exception {
            process.destroy();
        }
        @Override
        public void passivateObject ( Process process ) throws Exception {
        }
    }
    public static class CLIWorkItem implements Runnable {
        private ObjectPool<Process> mPool;
        private String mWork;
        public CLIWorkItem ( ObjectPool<Process> pool, String work ) {
            mPool = pool;
            mWork = work;
        }
        @Override
        public void run() {
            Process workProcess = null;
            try {
                workProcess = mPool.borrowObject();
                OutputStream os = workProcess.getOutputStream();
                os.write ( mWork.getBytes ( Charset.forName ( "UTF-8" ) ) );
                os.flush();
                Thread.sleep ( 100 );
            } catch ( Exception ex ) {
                sLogger.log ( Level.SEVERE, null, ex );
            } finally {
                if ( workProcess != null ) {
                    try {
                        mPool.returnObject ( workProcess );
                    } catch ( Exception ex ) {
                        sLogger.log ( Level.SEVERE, null, ex );
                    }
                }
            }
        }
    }
    public static void main ( String[] args ) throws Exception {
        ObjectPool<Process> pool =
            new GenericObjectPool<> (
            new CLIPoolableObjectFactory ( "mock_php.exe" ), 5 );
        BlockingQueue<Runnable> queue =
            new ArrayBlockingQueue<> ( 100, true );
        ThreadPoolExecutor executor =
            new ThreadPoolExecutor ( 20, 20, 1, TimeUnit.HOURS, queue );
        executor.execute ( new CLIWorkItem ( pool, "Message 1\r\n" ) );
        executor.execute ( new CLIWorkItem ( pool, "Message 2\r\n" ) );
        executor.execute ( new CLIWorkItem ( pool, "Message 3\r\n" ) );
        executor.execute ( new CLIWorkItem ( pool, "Message 4\r\n" ) );
        executor.execute ( new CLIWorkItem ( pool, "Message 5\r\n" ) );
        executor.execute ( new CLIWorkItem ( pool, "Message 6\r\n" ) );
        executor.execute ( new CLIWorkItem ( pool, "Message 7\r\n" ) );
        executor.execute ( new CLIWorkItem ( pool, "Message 8\r\n" ) );
        executor.execute ( new CLIWorkItem ( pool, "Message 9\r\n" ) );
        executor.execute ( new CLIWorkItem ( pool, "Message 10\r\n" ) );
        executor.execute ( new CLIWorkItem ( pool, "Message 11\r\n" ) );
        executor.shutdown();
        executor.awaitTermination ( 4000, TimeUnit.HOURS );
        pool.close();
    }
}
