public class ProfileImageReplacedElementFactory implements ReplacedElementFactory {
    private final ReplacedElementFactory superFactory;
    public ProfileImageReplacedElementFactory ( ReplacedElementFactory superFactory ) {
        this.superFactory = superFactory;
    }
    @Override
    public ReplacedElement createReplacedElement ( LayoutContext layoutContext, BlockBox blockBox,
            UserAgentCallback userAgentCallback, int cssWidth, int cssHeight ) {
        Element element = blockBox.getElement();
        if ( element == null ) {
            return null;
        }
        String nodeName = element.getNodeName();
        String className = element.getAttribute ( "class" );
        if ( "div".equals ( nodeName ) && className.contains ( "profile_picture" ) ) {
            InputStream input = null;
            try {
                input = ...;
                byte[] bytes = IOUtils.toByteArray ( input );
                Image image = Image.getInstance ( bytes );
                FSImage fsImage = new ITextFSImage ( image );
                if ( fsImage != null ) {
                    if ( ( cssWidth != -1 ) || ( cssHeight != -1 ) ) {
                        fsImage.scale ( cssWidth, cssHeight );
                    }
                    return new ITextImageElement ( fsImage );
                }
            } catch ( IOException e ) {
                getLogger().error ( ExceptionUtils.getStackTrace ( e ) );
            } catch ( BadElementException e ) {
                getLogger().error ( ExceptionUtils.getStackTrace ( e ) );
            } finally {
                IOUtils.closeQuietly ( input );
            }
        }
        return superFactory.createReplacedElement ( layoutContext, blockBox, userAgentCallback, cssWidth, cssHeight );
    }
    @Override
    public void reset() {
        superFactory.reset();
    }
    @Override
    public void remove ( Element e ) {
        superFactory.remove ( e );
    }
    @Override
    public void setFormSubmissionListener ( FormSubmissionListener listener ) {
        superFactory.setFormSubmissionListener ( listener );
    }
}
