private ExecutorService newPool ( int poolSize,
                                  String poolName, final int priority, int capacity ) {
    int cpu = Runtime.getRuntime().availableProcessors();
    ExecutorService result = null;
    if ( poolSize != 0 ) {
        if ( poolSize == -1 ) {
            poolSize = cpu;
        }
        if ( capacity <= 0 ) {
            capacity = Integer.MAX_VALUE;
        }
        result = new ThreadPoolExecutor ( poolSize, poolSize,
                                          120, TimeUnit.MINUTES,
                                          new LinkedBlockingQueue<Runnable> ( capacity ),
        new ThreadFactory() {
            @Override
            public Thread newThread ( Runnable runnable ) {
                Thread t = new Thread ( runnable );
                t.setPriority ( priority );
                return t;
            }
        }, new RejectedExecutionHandler() {
            @Override
            public void rejectedExecution ( Runnable r,
                                            ThreadPoolExecutor executor ) {
                if ( !executor.isShutdown() ) {
                    try {
                        executor.getQueue().put ( r );
                    } catch ( InterruptedException ex ) {
                    }
                }
            }
        } );
    }
    return result;
}
