public class CommandExecutor {
    public static final int NUMBER_OF_RUNS = 2;
    public static void main ( String[] args ) {
        System.out.println ( "CommandExecutor.main Testing Begin" );
        String command_01 = "cmd.exe /C dir";
        File dir_01 = new File ( "C:\\" );
        System.out.println ( "CommandExecutor.main Testing. Input: " );
        System.out.println ( "CommandExecutor.main Testing. command: " + command_01 );
        System.out.println ( "CommandExecutor.main Testing. dir: " + dir_01 );
        CommandExecutor ce_01 = new CommandExecutor ( command_01, null, dir_01 );
        ce_01.exec();
        System.out.println ( "CommandExecutor.main Testing. Output:" );
        System.out.println ( "  exitValue: " + ce_01.getExitValue() );
        System.out.println ( "  stdout: " + ce_01.getStdout() );
        System.out.println ( "  stderr: " + ce_01.getStderr() );
        String command_02 = "cmd.exe /C dirs";
        File dir_02 = new File ( "C:\\" );
        System.out.println ( "CommandExecutor.main Testing. Input: " );
        System.out.println ( "CommandExecutor.main Testing. command: " + command_02 );
        System.out.println ( "CommandExecutor.main Testing. dir: " + dir_02 );
        CommandExecutor ce_02 = new CommandExecutor ( command_02, null, dir_02 );
        ce_02.exec();
        System.out.println ( "CommandExecutor.main Testing. Output:" );
        System.out.println ( "  exitValue: " + ce_02.getExitValue() );
        System.out.println ( "  stdout: " + ce_02.getStdout() );
        System.out.println ( "  stderr: " + ce_02.getStderr() );
        System.out.println ( "CommandExecutor.main Testing End" );
    }
    protected String command;
    protected String[] env;
    protected File dir;
    protected boolean executed = false;
    protected int exitValue;
    protected Process process;
    protected List<String> stderr;
    protected Thread stderrReader;
    protected List<String> stdout;
    protected Thread stdoutReader;
    public CommandExecutor ( String command, String[] env, File dir ) {
        this.command = command;
        this.env = env;
        this.dir = dir;
    }
    protected void connectStderrReader() {
        stderr = new ArrayList<String>();
        final InputStream stream = process.getErrorStream();
        final BufferedReader reader =
            new BufferedReader ( new InputStreamReader ( stream ) );
        stderrReader = new Thread ( new Runnable() {
            public void run() {
                String nextLine = null;
                try {
                    while ( ( nextLine = reader.readLine() ) != null ) {
                        stderr.add ( nextLine );
                    }
                } catch ( IOException e ) {
                    System.out.println (
                        "CommandExecutor.connectStderrReader() error in reader thread" );
                    e.printStackTrace ( System.out );
                }
            }
        } );
        stderrReader.start();
    }
    protected void connectStdoutReader() {
        stdout = new ArrayList<String>();
        final InputStream stream = process.getInputStream();
        final BufferedReader reader =
            new BufferedReader ( new InputStreamReader ( stream ) );
        stdoutReader = new Thread ( new Runnable() {
            public void run() {
                String nextLine = null;
                try {
                    while ( ( nextLine = reader.readLine() ) != null ) {
                        stdout.add ( nextLine );
                    }
                } catch ( IOException e ) {
                    System.out.println (
                        "CommandExecutor.connectStdoutReader() error in reader thread" );
                    e.printStackTrace ( System.out );
                }
            }
        } );
        stdoutReader.start();
    }
    protected void createProcess() {
        try {
            process = Runtime.getRuntime().exec ( command, env, dir );
        } catch ( IOException e ) {
            System.out.println ( "CommandExecutor.exec() error in process creation. Exception: " + e );
            e.printStackTrace ( System.out );
        }
    }
    public synchronized void exec() {
        if ( !executed ) {
            createProcess();
            connectStdoutReader();
            connectStderrReader();
            waitForProcess();
            joinThreads();
            exitValue = process.exitValue();
            executed = true;
        }
    }
    public String getCommand() {
        return command;
    }
    public File getDir() {
        return dir;
    }
    public String[] getEnv() {
        return env;
    }
    public int getExitValue() {
        return exitValue;
    }
    public List<String> getStderr() {
        return stderr;
    }
    public List<String> getStdout() {
        return stdout;
    }
    public boolean isExecuted() {
        return executed;
    }
    protected void joinThreads() {
        try {
            stderrReader.join();
            stdoutReader.join();
        } catch ( InterruptedException e ) {
            System.out.println ( "CommandExecutor.joinThreads() error. Exception: " );
            e.printStackTrace ( System.out );
        }
    }
    @Override
    public synchronized String toString() {
        StringBuilder result = new StringBuilder();
        result.append ( "CommandExecutor:" );
        result.append ( "  command: " + command );
        result.append ( "  env: " + Arrays.deepToString ( env ) );
        result.append ( "  dir: " + dir );
        result.append ( "  executed: " + executed );
        result.append ( "  exitValue: " + exitValue );
        result.append ( "  stdout: " + stdout );
        result.append ( "  stderr: " + stderr );
        return result.toString();
    }
    protected void waitForProcess() {
        int numberOfRuns = 0;
        boolean terminated = false;
        while ( ( !terminated ) && ( numberOfRuns < NUMBER_OF_RUNS ) ) {
            try {
                process.waitFor();
                terminated = true;
            } catch ( InterruptedException e ) {
                System.out.println ( "CommandExecutor.waitForProcess() error" );
                e.printStackTrace ( System.out );
                numberOfRuns++;
            }
        }
    }
}
