public class Test {
    public static void main ( String[] args ) {
        String[][] data = {
            {"Toyota", "Gas", "Compact", "Corolla"},
            {"Toyota", "Gas", "Compact", "Camry"},
            {"Toyota", "Hybrid", "Compact", "Prius"},
            {"Honda", "Gas", "Compact", "Civic"}
        };
        TreeManager treeManager = new TreeManager();
        for ( String[] row : data ) {
            List<String> path = new ArrayList<String>();
            for ( String item : row ) {
                path.add ( item );
                treeManager.addData ( treeManager, path );
            }
        }
        treeManager.getData ( data[0] ).putValue ( "MPG", 38 );
        treeManager.getData ( data[1] ).putValue ( "MPG", 28 );
        Gson gson = new Gson();
        System.out.println ( gson.toJson ( treeManager ) );
    }
    public static abstract class TreeItem implements Iterable<TreeItem> {
        private Map<String, TreeItem> children;
        private String name;
        public TreeItem() {
            children = new HashMap<String, TreeItem>();
        }
        public String getName() {
            return name;
        }
        public void setName ( String name ) {
            this.name = name;
        }
        public void addChild ( String key, TreeItem data ) {
            children.put ( key, data );
        }
        public TreeItem getChild ( String key ) {
            return children.get ( key );
        }
        public boolean hasChild ( String key ) {
            return children.containsKey ( key );
        }
        @Override
        public Iterator<TreeItem> iterator() {
            return children.values().iterator();
        }
    }
    public static class TreeManager extends TreeItem {
        public void addData ( List<String> path ) {
            addData ( this, path );
        }
        private void addData ( TreeItem parent, List<String> path ) {
            String data = path.get ( 0 );
            if ( path.size() == 1 ) {
                if ( !parent.hasChild ( data ) ) {
                    Group group = new Group();
                    group.setName ( data );
                    parent.addChild ( data, group );
                }
            } else {
                addData ( parent.getChild ( data ), path.subList ( 1, path.size() ) );
            }
        }
        public Group getData ( String[] path ) {
            return ( Group ) getData ( this, Arrays.asList ( path ) );
        }
        public Group getData ( List<String> path ) {
            return ( Group ) getData ( this, path );
        }
        private TreeItem getData ( TreeItem parent, List<String> path ) {
            if ( parent == null || path.size() == 0 ) {
                throw new IllegalArgumentException ( "Invalid path specified in getData, remainder: "
                                                     + Arrays.toString ( path.toArray() ) );
            }
            String data = path.get ( 0 );
            if ( path.size() == 1 ) {
                return parent.getChild ( data );
            } else {
                return getData ( parent.getChild ( data ), path.subList ( 1, path.size() ) );
            }
        }
    }
    public static class Group extends TreeItem {
        private Map<String, Object> properties;
        public Object getValue ( Object key ) {
            return properties.get ( key );
        }
        public Object putValue ( String key, Object value ) {
            return properties.put ( key, value );
        }
        public Group () {
            super();
            properties = new HashMap<String, Object>();
        }
    }
}
