import java.util.concurrent.locks.ReadWriteLock;
import java.util.concurrent.locks.ReentrantReadWriteLock;
public abstract class PauseableThread implements Runnable {
    private final ReadWriteLock pause = new ReentrantReadWriteLock();
    private volatile boolean cancelled = false;
    private Exception thrown = null;
    @Override
    public void run() {
        try {
            while ( !cancelled ) {
                blockIfPaused();
                step();
            }
        } catch ( Exception ex ) {
            thrown = ex;
        }
    }
    private void blockIfPaused() throws InterruptedException {
        try {
            pause.writeLock().lockInterruptibly();
        } finally {
            pause.writeLock().unlock();
        }
    }
    public void pause() {
        pause.readLock().lock();
    }
    public void resume() {
        pause.readLock().unlock();
    }
    public void cancel() {
        cancelled = true;
    }
    public void start() {
        new Thread ( this ).start();
    }
    public Exception getThrown() {
        return thrown;
    }
    public abstract void step() throws Exception;
    public static PauseableThread make ( Stepper stepper ) {
        StepperThread pauseableStepper = new StepperThread ( stepper );
        return pauseableStepper;
    }
    public interface Stepper {
        public void step() throws Exception;
    }
    private static class StepperThread extends PauseableThread {
        private final Stepper stepper;
        StepperThread ( Stepper stepper ) {
            this.stepper = stepper;
        }
        @Override
        public void step() throws Exception {
            stepper.step();
        }
    }
    static int n = 0;
    public static void main ( String[] args ) throws InterruptedException {
        try {
            Stepper s = new Stepper() {
                @Override
                public void step() throws Exception {
                    n += 1;
                    Thread.sleep ( 10 );
                }
            };
            PauseableThread t = PauseableThread.make ( s );
            t.start();
            Thread.sleep ( 1000 );
            t.pause();
            System.out.println ( "Paused: " + n );
            Thread.sleep ( 1000 );
            System.out.println ( "Resuminng: " + n );
            t.resume();
            Thread.sleep ( 1000 );
            t.cancel();
        } catch ( Exception e ) {
        }
    }
}
