import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.SwingUtilities;
import javax.swing.Timer;
public class AnimatedGraphics extends JComponent implements ActionListener {
    Color startColor = Color.RED;
    Color endColor = Color.GREEN;
    Color currentColor = startColor;
    int animationDuration = 2000;
    long animStartTime;
    public AnimatedGraphics() {
        Timer timer = new Timer ( 30, this );
        timer.setInitialDelay ( 1000 );
        animStartTime = 1000 + System.nanoTime() / 1000000;
        timer.start();
    }
    public void paintComponent ( Graphics g ) {
        g.setColor ( getBackground() );
        g.fillRect ( 0, 0, getWidth(), getHeight() );
        g.setColor ( currentColor );
        g.fillOval ( 0, 0, getWidth(), getHeight() );
    }
    public void actionPerformed ( ActionEvent ae ) {
        long currentTime = System.nanoTime() / 1000000;
        long totalTime = currentTime - animStartTime;
        if ( totalTime > animationDuration ) {
            animStartTime = currentTime;
        }
        float fraction = ( float ) totalTime / animationDuration;
        fraction = Math.min ( 1.0f, fraction );
        int red = ( int ) ( fraction * endColor.getRed() +
                            ( 1 - fraction ) * startColor.getRed() );
        int green = ( int ) ( fraction * endColor.getGreen() +
                              ( 1 - fraction ) * startColor.getGreen() );
        int blue = ( int ) ( fraction * endColor.getBlue() +
                             ( 1 - fraction ) * startColor.getBlue() );
        currentColor = new Color ( red, green, blue );
        repaint();
    }
    private static void createAndShowGUI() {
        JFrame f = new JFrame ( "Animated Graphics" );
        f.setDefaultCloseOperation ( JFrame.EXIT_ON_CLOSE );
        f.setSize ( 200, 200 );
        f.add ( new AnimatedGraphics() );
        f.setVisible ( true );
    }
    public static void main ( String args[] ) {
        Runnable doCreateAndShowGUI = new Runnable() {
            public void run() {
                createAndShowGUI();
            }
        };
        SwingUtilities.invokeLater ( doCreateAndShowGUI );
    }
}
