import java.lang.reflect.Field;
import org.junit.Assert;
import org.junit.Test;
public class PojoTransactionTest {
    public static class PojoTransaction<T> {
        private T source;
        private T target;
        public PojoTransaction ( T source ) {
            try {
                this.source = source;
                this.target = ( T ) source.getClass().newInstance();
                copyState ( source, target );
            } catch ( Exception e ) {
                throw new RuntimeException ( "Failed to create PojoTransaction", e );
            }
        }
        private void copyState ( T from, T to ) throws IllegalAccessException {
            for ( Field f : from.getClass().getDeclaredFields() ) {
                f.setAccessible ( true );
                f.set ( to, f.get ( from ) );
            }
        }
        public T getTransactionTarget() {
            return target;
        }
        public void commit() {
            try {
                copyState ( target, source );
            } catch ( Exception e ) {
                throw new RuntimeException ( "Failed to change state of original object", e );
            }
        }
    }
    public static class TestData {
        private String strValue = "TEST";
        private int intValue = 1;
        private float floatValue = 3.1415f;
        public String getStrValue() {
            return strValue;
        }
        public void setStrValue ( String strValue ) {
            this.strValue = strValue;
        }
        public int getIntValue() {
            return intValue;
        }
        public void setIntValue ( int intValue ) {
            this.intValue = intValue;
        }
        public float getFloatValue() {
            return floatValue;
        }
        public void setFloatValue ( float floatValue ) {
            this.floatValue = floatValue;
        }
    }
    @Test
    public void testTransaction() {
        TestData orig = new TestData();
        PojoTransaction<TestData> tx = new PojoTransaction<TestData> ( orig );
        TestData target = tx.getTransactionTarget();
        target.setFloatValue ( 1.0f );
        target.setIntValue ( 5 );
        target.setStrValue ( "Another string" );
        Assert.assertEquals ( 1, orig.getIntValue() );
        Assert.assertEquals ( 3.1415f, orig.getFloatValue(), 0.001f );
        Assert.assertEquals ( "TEST", orig.getStrValue() );
        tx.commit();
        Assert.assertEquals ( 5, orig.getIntValue() );
        Assert.assertEquals ( 1.0f, orig.getFloatValue(), 0.001f );
        Assert.assertEquals ( "Another string", orig.getStrValue() );
    }
}
