public class MyFrame extends javax.swing.JFrame {
    private EventList<Person> eventList = new BasicEventList<Person>();
    private EventSelectionModel<Person> selectionModel;
    public MyFrame() {
        initComponents();
        loadData();
        configureTable();
    }
    private void loadData() {
        eventList.add ( new Person ( "Richard", "Dawkins" ) );
        eventList.add ( new Person ( "Sam", "Harris" ) );
        eventList.add ( new Person ( "Christopher", "Hitchens" ) );
        eventList.add ( new Person ( "Daniel", "Dennett" ) );
    }
    private void configureTable() {
        String[] headers = new String[] {"Firstname", "Lastname"};
        String[] properties = new String[] {"firstname", "lastname"};
        TextFilterator<Person> personTextFilterator = new TextFilterator<Person>() {
            @Override
            public void getFilterStrings ( List list, Person p ) {
                list.add ( p.getFirstname() );
                list.add ( p.getLastname() );
            }
        };
        MatcherEditor<Person> textMatcherEditor = new TextComponentMatcherEditor<Person> ( txtFilter, personTextFilterator );
        FilterList<Person> filterList = new FilterList<Person> ( eventList, textMatcherEditor );
        TableFormat tf = GlazedLists.tableFormat ( properties, headers );
        EventTableModel<Person> model = new EventTableModel<Person> ( filterList, tf );
        selectionModel = new EventSelectionModel<Person> ( filterList );
        tblNames.setSelectionModel ( selectionModel );
        tblNames.setModel ( model );
    }
    @SuppressWarnings ( "unchecked" )
    private void initComponents() {
        jLabel1 = new javax.swing.JLabel();
        txtFilter = new javax.swing.JTextField();
        jScrollPane1 = new javax.swing.JScrollPane();
        tblNames = new javax.swing.JTable();
        btnDelete = new javax.swing.JButton();
        btnReload = new javax.swing.JButton();
        setDefaultCloseOperation ( javax.swing.WindowConstants.EXIT_ON_CLOSE );
        setTitle ( "GlazedLists test" );
        jLabel1.setText ( "Filter" );
        tblNames.setModel ( new javax.swing.table.DefaultTableModel (
        new Object [][] {
            {null, null, null, null},
            {null, null, null, null},
            {null, null, null, null},
            {null, null, null, null}
        },
        new String [] {
            "Title 1", "Title 2", "Title 3", "Title 4"
        }
                            ) );
        jScrollPane1.setViewportView ( tblNames );
        btnDelete.setText ( "Delete" );
        btnDelete.addActionListener ( new java.awt.event.ActionListener() {
            public void actionPerformed ( java.awt.event.ActionEvent evt ) {
                btnDeleteActionPerformed ( evt );
            }
        } );
        btnReload.setText ( "Reload data" );
        btnReload.addActionListener ( new java.awt.event.ActionListener() {
            public void actionPerformed ( java.awt.event.ActionEvent evt ) {
                btnReloadActionPerformed ( evt );
            }
        } );
        javax.swing.GroupLayout layout = new javax.swing.GroupLayout ( getContentPane() );
        getContentPane().setLayout ( layout );
        layout.setHorizontalGroup (
            layout.createParallelGroup ( org.jdesktop.layout.GroupLayout.LEADING )
            .add ( layout.createSequentialGroup()
                   .addContainerGap()
                   .add ( layout.createParallelGroup ( org.jdesktop.layout.GroupLayout.LEADING )
                          .add ( layout.createSequentialGroup()
                                 .add ( jLabel1 )
                                 .addPreferredGap ( org.jdesktop.layout.LayoutStyle.RELATED )
                                 .add ( txtFilter ) )
                          .add ( jScrollPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 388, Short.MAX_VALUE )
                          .add ( org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                                 .add ( btnReload )
                                 .addPreferredGap ( org.jdesktop.layout.LayoutStyle.RELATED, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE )
                                 .add ( btnDelete ) ) )
                   .addContainerGap() )
        );
        layout.setVerticalGroup (
            layout.createParallelGroup ( org.jdesktop.layout.GroupLayout.LEADING )
            .add ( layout.createSequentialGroup()
                   .addContainerGap()
                   .add ( layout.createParallelGroup ( org.jdesktop.layout.GroupLayout.BASELINE )
                          .add ( jLabel1 )
                          .add ( txtFilter, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE ) )
                   .addPreferredGap ( org.jdesktop.layout.LayoutStyle.UNRELATED )
                   .add ( jScrollPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 240, Short.MAX_VALUE )
                   .addPreferredGap ( org.jdesktop.layout.LayoutStyle.RELATED )
                   .add ( layout.createParallelGroup ( org.jdesktop.layout.GroupLayout.BASELINE )
                          .add ( btnDelete )
                          .add ( btnReload ) )
                   .addContainerGap() )
        );
        pack();
    }
    private void btnDeleteActionPerformed ( java.awt.event.ActionEvent evt ) {
        if ( !selectionModel.isSelectionEmpty() ) {
            eventList.removeAll ( selectionModel.getSelected() );
        }
    }
    private void btnReloadActionPerformed ( java.awt.event.ActionEvent evt ) {
        eventList.clear();
        loadData();
    }
    public static void main ( String args[] ) {
        java.awt.EventQueue.invokeLater ( new Runnable() {
            @Override
            public void run() {
                new MyFrame().setVisible ( true );
            }
        } );
    }
    private javax.swing.JButton btnDelete;
    private javax.swing.JButton btnReload;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JTable tblNames;
    private javax.swing.JTextField txtFilter;
}
