public class Period implements Serializable {
    private static final long serialVersionUID = 1L;
    private Calendar          start;
    private Calendar          end;
    private Calendar          startClone;
    private Calendar          endClone;
    private long              durationMillis;
    private long              startMillis;
    private long              endMillis;
    private long              MAX              = 604800000;
    private long              DAY              = 86400000;
    private long              SIX              = 518400000;
    private ArrayList<Week>   weeks;
    public Period ( Calendar start, Calendar end ) {
        this.start = start;
        this.end = end;
        startClone = ( Calendar ) start.clone();
        endClone = ( Calendar ) end.clone();
        startMillis = start.getTimeInMillis();
        endMillis = end.getTimeInMillis();
        durationMillis = endMillis - startMillis;
        int startWeek = start.get ( Calendar.WEEK_OF_YEAR );
        int endWeek = end.get ( Calendar.WEEK_OF_YEAR );
        weeks = new ArrayList<Week>();
    }
    public ArrayList<Week> makeWeeks() {
        Calendar borderLine;
        if ( ( durationMillis < MAX ) || ( endMillis < startMillis ) ) {
            return null;
        }
        Calendar endHead = Calendar.getInstance();
        if ( start.get ( Calendar.DAY_OF_WEEK ) != Calendar.MONDAY ) {
            int days = 0;
            start = adjustCalendar ( start );
            days = getDaysToMonday ( start.get ( Calendar.DAY_OF_WEEK ) );
            endHead = ( Calendar ) start.clone();
            endHead.add ( Calendar.DAY_OF_YEAR, days - 1 );
            weeks.add ( new Week ( start, endHead, days, calcCompletion ( days ), setWeekNumber ( start ) ) );
            Calendar vector = endHead;
            vector.add ( Calendar.DAY_OF_YEAR, 1 );
            borderLine = vector;
        } else {
            start = adjustCalendar ( start );
            endHead = ( Calendar ) start.clone();
            borderLine = ( Calendar ) endHead.clone();
        }
        long endMillis = 0;
        while ( ( ( ( borderLine.getTimeInMillis() + SIX ) ) < ( end.getTimeInMillis() + 1 ) ) ) {
            Calendar endNext = Calendar.getInstance();
            Calendar begin = Calendar.getInstance();
            begin = ( Calendar ) borderLine.clone();
            endNext = ( Calendar ) borderLine.clone();
            endNext.add ( Calendar.DAY_OF_YEAR, 6 );
            weeks.add ( new Week ( begin, endNext, 7, true, setWeekNumber ( begin ) ) );
            borderLine.add ( Calendar.DAY_OF_YEAR, 7 );
            endMillis = endNext.getTimeInMillis();
        }
        Calendar tail = Calendar.getInstance();
        tail.setTimeInMillis ( endMillis );
        tail.add ( Calendar.DAY_OF_YEAR, 1 );
        if ( ! ( tail.get ( Calendar.DAY_OF_YEAR ) >= end.get ( Calendar.DAY_OF_YEAR ) ) ) {
            tail = adjustCalendar ( tail );
            int daysTail = getDaysToTail ( end.get ( Calendar.DAY_OF_WEEK ) );
            if ( daysTail != 0 ) {
                weeks.add ( new Week ( tail, end, daysTail, calcCompletion ( daysTail ), setWeekNumber ( end ) ) );
            }
        }
        return weeks;
    }
    private double setWeekNumber ( Calendar cal ) {
        Integer week = cal.get ( Calendar.WEEK_OF_YEAR );
        Double clone = week.doubleValue();
        return clone;
    }
    private int getDaysToMonday ( int currentDay ) {
        switch ( currentDay ) {
        case 1:
            return 1;
        case 2:
            return 0;
        case 3:
            return 6;
        case 4:
            return 5;
        case 5:
            return 4;
        case 6:
            return 3;
        case 7:
            return 2;
        }
        return 0;
    }
    private int getDaysToTail ( int tailDay ) {
        switch ( tailDay ) {
        case 1:
            return 0;
        case 2:
            return 1;
        case 3:
            return 2;
        case 4:
            return 3;
        case 5:
            return 4;
        case 6:
            return 5;
        case 7:
            return 6;
        }
        return 0;
    }
    public int totalWeeks() {
        return weeks.size();
    }
    public boolean isAtleastWeek() {
        long diff = endClone.getTimeInMillis() - startClone.getTimeInMillis();
        if ( diff < MAX ) {
            return false;
        }
        return true;
    }
    public String getDates ( Double weekNumber ) {
        int weekNumberInt = weekNumber.intValue();
        String vector = new String();
        for ( Week week : weeks ) {
            if ( week.getWeekNumber() == weekNumberInt ) {
                vector += week.getStart().get ( Calendar.DAY_OF_MONTH ) + "/";
                vector += ( week.getStart().get ( Calendar.MONTH ) + 1 ) + "/";
                vector += week.getStart().get ( Calendar.YEAR ) + "-";
                vector += ( week.getEnd().get ( Calendar.DAY_OF_MONTH ) ) + "/";
                vector += ( week.getEnd().get ( Calendar.MONTH ) + 1 ) + "/";
                vector += week.getEnd().get ( Calendar.YEAR );
            }
        }
        return vector;
    }
    public Week getWeek ( Double weekNumber ) {
        int weekNumberInt = weekNumber.intValue();
        for ( Week week : weeks ) {
            if ( weekNumberInt == week.getWeekNumber() ) {
                return week;
            }
        }
        return null;
    }
    private Calendar adjustCalendar ( Calendar nu ) {
        nu.set ( Calendar.HOUR, 0 );
        nu.set ( Calendar.MINUTE, 0 );
        nu.set ( Calendar.SECOND, 0 );
        nu.set ( Calendar.MILLISECOND, 0 );
        return nu;
    }
    private boolean calcCompletion ( int days ) {
        if ( days < 4 ) {
            return false;
        }
        return true;
    }
    public boolean hasWeekNumber ( Double weekNumber ) {
        for ( Week week : weeks ) {
            if ( weekNumber.intValue() == week.getWeekNumber() ) {
                return true;
            }
        }
        return false;
    }
    public double getHighestWeekNumber() {
        return weeks.get ( weeks.size() - 1 ).getWeekNumber();
    }
    private String showCalValues ( Calendar cal ) {
        return "Day of year " + cal.get ( Calendar.DAY_OF_YEAR ) + " Day of month: " + cal.get ( Calendar.DAY_OF_MONTH )
               + " Day of week: " + cal.get ( Calendar.DAY_OF_WEEK ) + " month " + cal.get ( Calendar.MONTH ) + " date: "
               + cal.get ( Calendar.HOUR_OF_DAY ) + ":" + cal.get ( Calendar.MINUTE ) + ":" + cal.get ( Calendar.MILLISECOND )
               + " week number: " + cal.get ( Calendar.WEEK_OF_YEAR ) + " millis: " + cal.getTimeInMillis();
    }
    private void showWeeks() {
        for ( Week w : weeks ) {
            System.out.println ( w.getWeekNumber() + " start millis: " + w.getStart().getTimeInMillis() );
        }
    }
}
