public class AuthenticationTokenProcessingFilter extends GenericFilterBean {
    @Autowired UserService userService;
    @Autowired TokenUtils tokenUtils;
    AuthenticationManager authManager;
    public AuthenticationTokenProcessingFilter ( AuthenticationManager authManager ) {
        this.authManager = authManager;
    }
    @Override
    public void doFilter ( ServletRequest request, ServletResponse response,
                           FilterChain chain ) throws IOException, ServletException {
        @SuppressWarnings ( "unchecked" )
        Map<String, String[]> parms = request.getParameterMap();
        if ( parms.containsKey ( "token" ) ) {
            String token = parms.get ( "token" ) [0];
            if ( tokenUtils.validate ( token ) ) {
                UserDetails userDetails = tokenUtils.getUserFromToken ( token );
                UsernamePasswordAuthenticationToken authentication =
                    new UsernamePasswordAuthenticationToken ( userDetails.getUsername(), userDetails.getPassword() );
                authentication.setDetails ( new WebAuthenticationDetailsSource().buildDetails ( ( HttpServletRequest ) request ) );
                SecurityContextHolder.getContext().setAuthentication ( authManager.authenticate ( authentication ) );
            }
        }
        chain.doFilter ( request, response );
    }
}
