import javax.swing.DefaultCellEditor;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.util.ArrayList;
public class TableRenderDemo extends JPanel {
    private static final long serialVersionUID = 1L;
    private boolean DEBUG = false;
    public TableRenderDemo() {
        super ( new GridLayout ( 1, 0 ) );
        JTable table = new JTable ( new MyTableModel() );
        table.setPreferredScrollableViewportSize ( new Dimension ( 500, 70 ) );
        table.setFillsViewportHeight ( true );
        JScrollPane scrollPane = new JScrollPane ( table );
        initColumnSizes ( table );
        setUpSportColumn ( table, table.getColumnModel().getColumn ( 2 ) );
        add ( scrollPane );
    }
    private void initColumnSizes ( JTable table ) {
        MyTableModel model = ( MyTableModel ) table.getModel();
        TableColumn column = null;
        Component comp = null;
        int headerWidth = 0;
        int cellWidth = 0;
        Object[] longValues = model.longValues;
        TableCellRenderer headerRenderer = table.getTableHeader().getDefaultRenderer();
        for ( int i = 0; i < 5; i++ ) {
            column = table.getColumnModel().getColumn ( i );
            comp = headerRenderer.getTableCellRendererComponent ( null, column.getHeaderValue(), false, false, 0, 0 );
            headerWidth = comp.getPreferredSize().width;
            comp = table.getDefaultRenderer ( model.getColumnClass ( i ) ).getTableCellRendererComponent ( table, longValues[i], false, false, 0, i );
            cellWidth = comp.getPreferredSize().width;
            if ( DEBUG ) {
                System.out.println ( "Initializing width of column " + i + ". " + "headerWidth = " + headerWidth + "; cellWidth = " + cellWidth );
            }
            column.setPreferredWidth ( Math.max ( headerWidth, cellWidth ) );
        }
    }
    public void setUpSportColumn ( JTable table, TableColumn sportColumn ) {
        ArrayList<String> listSomeString = new ArrayList<String>();
        listSomeString.add ( "Snowboarding" );
        listSomeString.add ( "Rowing" );
        listSomeString.add ( "Knitting" );
        listSomeString.add ( "Speed reading" );
        listSomeString.add ( "Pool" );
        listSomeString.add ( "None of the above" );
        Java2sAutoComboBox comboBox = new Java2sAutoComboBox ( listSomeString );
        comboBox.setDataList ( listSomeString );
        comboBox.setMaximumRowCount ( 3 );
        sportColumn.setCellEditor ( new DefaultCellEditor ( comboBox ) );
        DefaultTableCellRenderer renderer = new DefaultTableCellRenderer();
        renderer.setToolTipText ( "Click for combo box" );
        sportColumn.setCellRenderer ( renderer );
    }
    class MyTableModel extends AbstractTableModel {
        private static final long serialVersionUID = 1L;
        private String[] columnNames = {"First Name", "Last Name", "Sport", "# of Years", "Vegetarian"};
        private Object[][] data = {{"Kathy", "Smith", "Snowboarding", new Integer ( 5 ), false},
            {"John", "Doe", "Rowing", new Integer ( 3 ), true}, {"Sue", "Black", "Knitting", new Integer ( 2 ), false},
            {"Jane", "White", "Speed reading", new Integer ( 20 ), true}, {"Joe", "Brown", "Pool", new Integer ( 10 ), false}
        };
        public final Object[] longValues = {"Jane", "Kathy", "None of the above", new Integer ( 20 ), Boolean.TRUE};
        @Override
        public int getColumnCount() {
            return columnNames.length;
        }
        @Override
        public int getRowCount() {
            return data.length;
        }
        @Override
        public String getColumnName ( int col ) {
            return columnNames[col];
        }
        @Override
        public Object getValueAt ( int row, int col ) {
            return data[row][col];
        }
        @Override
        public Class<?> getColumnClass ( int c ) {
            return getValueAt ( 0, c ).getClass();
        }
        @Override
        public boolean isCellEditable ( int row, int col ) {
            if ( col < 2 ) {
                return false;
            } else {
                return true;
            }
        }
        @Override
        public void setValueAt ( Object value, int row, int col ) {
            if ( DEBUG ) {
                System.out.println ( "Setting value at " + row + "," + col
                                     + " to " + value
                                     + " (an instance of "
                                     + value.getClass() + ")" );
            }
            data[row][col] = value;
            fireTableCellUpdated ( row, col );
            if ( DEBUG ) {
                System.out.println ( "New value of data:" );
                printDebugData();
            }
        }
        private void printDebugData() {
            int numRows = getRowCount();
            int numCols = getColumnCount();
            for ( int i = 0; i < numRows; i++ ) {
                System.out.print ( "    row " + i + ":" );
                for ( int j = 0; j < numCols; j++ ) {
                    System.out.print ( "  " + data[i][j] );
                }
                System.out.println();
            }
            System.out.println ( "--------------------------" );
        }
    }
    private static void createAndShowGUI() {
        JFrame frame = new JFrame ( "TableRenderDemo" );
        frame.setDefaultCloseOperation ( JFrame.EXIT_ON_CLOSE );
        TableRenderDemo newContentPane = new TableRenderDemo();
        newContentPane.setOpaque ( true );
        frame.setContentPane ( newContentPane );
        frame.pack();
        frame.setVisible ( true );
    }
    public static void main ( String[] args ) {
        javax.swing.SwingUtilities.invokeLater ( new Runnable() {
            @Override
            public void run() {
                createAndShowGUI();
            }
        } );
    }
}
