public class ContextSourceAndJpaTransactionManager extends JpaTransactionManager {
    private static final long serialVersionUID = 1L;
    private ContextSourceTransactionManagerDelegate ldapManagerDelegate =
        new ContextSourceTransactionManagerDelegate();
    protected boolean isExistingTransaction ( Object transaction ) {
        ContextSourceAndJpaTransactionObject actualTransactionObject =
            ( ContextSourceAndJpaTransactionObject ) transaction;
        return super.isExistingTransaction ( actualTransactionObject
                                             .getJpaTransactionObject() );
    }
    protected Object doGetTransaction() throws TransactionException {
        Object dataSourceTransactionObject = super.doGetTransaction();
        Object contextSourceTransactionObject =
            ldapManagerDelegate.doGetTransaction();
        return new ContextSourceAndJpaTransactionObject (
                   contextSourceTransactionObject, dataSourceTransactionObject
               );
    }
    protected void doBegin ( Object transaction, TransactionDefinition definition )
    throws TransactionException {
        ContextSourceAndJpaTransactionObject actualTransactionObject =
            ( ContextSourceAndJpaTransactionObject ) transaction;
        super.doBegin ( actualTransactionObject.getJpaTransactionObject(),
                        definition );
        ldapManagerDelegate.doBegin (
            actualTransactionObject.getLdapTransactionObject(),
            definition
        );
    }
    protected void doCleanupAfterCompletion ( Object transaction ) {
        ContextSourceAndJpaTransactionObject actualTransactionObject =
            ( ContextSourceAndJpaTransactionObject ) transaction;
        super.doCleanupAfterCompletion ( actualTransactionObject
                                         .getJpaTransactionObject() );
        ldapManagerDelegate.doCleanupAfterCompletion ( actualTransactionObject
                .getLdapTransactionObject() );
    }
    protected void doCommit ( DefaultTransactionStatus status )
    throws TransactionException {
        ContextSourceAndJpaTransactionObject actualTransactionObject =
            ( ContextSourceAndJpaTransactionObject ) status.getTransaction();
        try {
            super.doCommit ( new DefaultTransactionStatus (
                                 actualTransactionObject.getJpaTransactionObject(),
                                 status.isNewTransaction(),
                                 status.isNewSynchronization(),
                                 status.isReadOnly(),
                                 status.isDebug(),
                                 status.getSuspendedResources() )
                           );
        } catch ( TransactionException ex ) {
            if ( isRollbackOnCommitFailure() ) {
                logger.debug ( "Failed to commit db resource, rethrowing", ex );
                throw ex;
            } else {
                logger.warn (
                    "Failed to commit and resource is rollbackOnCommit not set -"
                    + " proceeding to commit ldap resource." );
            }
        }
        ldapManagerDelegate.doCommit ( new DefaultTransactionStatus (
                                           actualTransactionObject.getLdapTransactionObject(),
                                           status.isNewTransaction(),
                                           status.isNewSynchronization(),
                                           status.isReadOnly(),
                                           status.isDebug(),
                                           status.getSuspendedResources() )
                                     );
    }
    protected void doRollback ( DefaultTransactionStatus status ) throws TransactionException {
        ContextSourceAndJpaTransactionObject actualTransactionObject =
            ( ContextSourceAndJpaTransactionObject ) status.getTransaction();
        super.doRollback ( new DefaultTransactionStatus (
                               actualTransactionObject.getJpaTransactionObject(),
                               status.isNewTransaction(),
                               status.isNewSynchronization(),
                               status.isReadOnly(),
                               status.isDebug(),
                               status.getSuspendedResources() )
                         );
        ldapManagerDelegate.doRollback ( new DefaultTransactionStatus (
                                             actualTransactionObject.getLdapTransactionObject(),
                                             status.isNewTransaction(),
                                             status.isNewSynchronization(),
                                             status.isReadOnly(),
                                             status.isDebug(),
                                             status.getSuspendedResources() )
                                       );
    }
    @SuppressWarnings ( "UnusedDeclaration" )
    public ContextSource getContextSource() {
        return ldapManagerDelegate.getContextSource();
    }
    public void setContextSource ( ContextSource contextSource ) {
        ldapManagerDelegate.setContextSource ( contextSource );
    }
    @SuppressWarnings ( "UnusedDeclaration" )
    protected void setRenamingStrategy ( TempEntryRenamingStrategy renamingStrategy ) {
        ldapManagerDelegate.setRenamingStrategy ( renamingStrategy );
    }
    private final static class ContextSourceAndJpaTransactionObject {
        private Object ldapTransactionObject;
        private Object jpaTransactionObject;
        public ContextSourceAndJpaTransactionObject (
            Object ldapTransactionObject, Object jpaTransactionObject ) {
            this.ldapTransactionObject = ldapTransactionObject;
            this.jpaTransactionObject = jpaTransactionObject;
        }
        public Object getJpaTransactionObject() {
            return jpaTransactionObject;
        }
        public Object getLdapTransactionObject() {
            return ldapTransactionObject;
        }
    }
    protected Object doSuspend ( Object transaction ) throws TransactionException {
        throw new TransactionSuspensionNotSupportedException (
            "Transaction manager [" + getClass().getName()
            + "] does not support transaction suspension" );
    }
    protected void doResume ( Object transaction, Object suspendedResources )
    throws TransactionException {
        throw new TransactionSuspensionNotSupportedException (
            "Transaction manager [" + getClass().getName()
            + "] does not support transaction suspension" );
    }
    @Override
    protected void doSetRollbackOnly ( DefaultTransactionStatus status ) {
        super.doSetRollbackOnly (
            new DefaultTransactionStatus (
                ( ( ContextSourceAndJpaTransactionObject ) status.getTransaction() )
                .getJpaTransactionObject(),
                status.isNewTransaction(),
                status.isNewSynchronization(),
                status.isReadOnly(),
                status.isDebug(),
                status.getSuspendedResources() )
        );
    }
}
