import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import sun.awt.SunToolkit;
public class TestMenu {
    private static int delay = 500;
    private static JMenu[] menus = new JMenu[5];
    private static Dimension[] parentSizes;
    private static Robot robot;
    private static SunToolkit toolkit;
    public static void main ( String[] args ) throws Exception {
        robot = new Robot();
        toolkit = ( SunToolkit ) Toolkit.getDefaultToolkit();
        parentSizes = new Dimension[menus.length];
        createGUI();
        moveAndClickCenter ( menus[0] );
        for ( int index = 0; index < menus.length; index++ ) {
            parentSizes[index] = getDropDownSize ( index );
        }
        Component item = menus[menus.length - 1].getMenuComponent ( menus[menus.length - 1].getMenuComponentCount() - 1 );
        moveAndClickCenter ( item );
        boolean bug = false;
        moveAndClickCenter ( menus[menus.length - 1] );
        for ( int index = menus.length - 1; index >= 0; index-- ) {
            Dimension currentSize = getDropDownSize ( index );
            System.out.print ( "old: " + parentSizes[index] + ", new: " + currentSize );
            if ( !parentSizes[index].equals ( currentSize ) ) {
                bug = true;
                System.out.println ( " ERROR" );
            } else {
                System.out.println();
            }
        }
        if ( bug ) {
            throw new RuntimeException ( "JMenu drop down size is changed for no reason." );
        }
    }
    private static Dimension getDropDownSize ( int index ) throws Exception {
        moveToCenter ( menus[index] );
        return menus[index].getMenuComponent ( 0 ).getParent().getSize();
    }
    private static void click() throws Exception {
        robot.mousePress ( InputEvent.BUTTON1_MASK );
        robot.mouseRelease ( InputEvent.BUTTON1_MASK );
        syncAndDelay();
    }
    private static void createGUI() throws Exception {
        SwingUtilities.invokeAndWait ( new Runnable() {
            @Override
            public void run() {
                UIManager.LookAndFeelInfo[] infos = UIManager.getInstalledLookAndFeels();
                for ( final UIManager.LookAndFeelInfo info : infos ) {
                    if ( info.getName().toLowerCase().indexOf ( "metal" ) >= 0 ) {
                        if ( !UIManager.getLookAndFeel().getName().equals ( info.getName() ) ) {
                            try {
                                UIManager.setLookAndFeel ( info.getClassName() );
                                System.out.println ( "Attempt to set look and feel to " + info.getName() );
                            } catch ( Exception e ) {
                                e.printStackTrace();
                            }
                        } else {
                            System.out.println ( "Metal look and feel is the default" );
                        }
                        break;
                    }
                }
                System.out.println ( "Testing with " + UIManager.getLookAndFeel().getName() );
                JFrame frame = new JFrame ( "A frame" );
                frame.setJMenuBar ( new JMenuBar() );
                for ( int menuIndex = 0; menuIndex < menus.length; menuIndex++ ) {
                    menus[menuIndex] = new JMenu ( "Menu " + menuIndex );
                    frame.getJMenuBar().add ( menus[menuIndex] );
                    for ( int itemIndex = 0; itemIndex <= menus.length - menuIndex; itemIndex++ ) {
                        JMenuItem item = new JMenuItem ( "Menu " + menuIndex + " item " + itemIndex );
                        menus[menuIndex].add ( item );
                    }
                }
                frame.setDefaultCloseOperation ( JFrame.EXIT_ON_CLOSE );
                frame.pack();
                frame.setLocationRelativeTo ( null );
                frame.setVisible ( true );
            }
        } );
        syncAndDelay();
    }
    private static void moveAndClickCenter ( Component c ) throws Exception {
        moveToCenter ( c );
        click();
    }
    private static void moveToCenter ( final Component c ) throws Exception {
        final Point cp = new Point();
        SwingUtilities.invokeAndWait ( new Runnable() {
            @Override
            public void run() {
                Point p = new Point ( c.getWidth() / 2, c.getHeight() / 2 );
                SwingUtilities.convertPointToScreen ( p, c );
                cp.setLocation ( p );
            }
        } );
        robot.mouseMove ( cp.x, cp.y );
        syncAndDelay();
    }
    private static void syncAndDelay() throws Exception {
        if ( delay > 0 ) {
            Thread.sleep ( delay );
        }
        toolkit.realSync();
    }
    private TestMenu() {
    }
}
