import java.io.IOException;
import java.io.PrintStream;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
public class SysStreamsLogger {
    private static Logger sysOutLogger = LoggerFactory.getLogger ( "SYSOUT" );
    private static Logger sysErrLogger = LoggerFactory.getLogger ( "SYSERR" );
    public static final PrintStream sysout = System.out;
    public static final PrintStream syserr = System.err;
    protected static final String LINE_SEPERATOR = System.getProperty ( "line.separator" );
    public static void bindSystemStreams() {
        System.setOut ( new PrintStream ( new LoggingOutputStream ( sysOutLogger, false ), true ) );
        System.setErr ( new PrintStream ( new LoggingOutputStream ( sysErrLogger, true ), true ) );
    }
    public static void unbindSystemStreams() {
        System.setOut ( sysout );
        System.setErr ( syserr );
    }
    private static class LoggingOutputStream extends java.io.OutputStream {
        protected Logger log;
        protected boolean isError;
        protected boolean hasBeenClosed = false;
        protected byte[] buf;
        protected int count;
        private int bufLength;
        public static final int DEFAULT_BUFFER_LENGTH = 2048;
        private LoggingOutputStream() {
        }
        public LoggingOutputStream ( Logger log, boolean isError ) throws IllegalArgumentException {
            if ( log == null ) {
                throw new IllegalArgumentException ( "log == null" );
            }
            this.isError = isError;
            this.log = log;
            bufLength = DEFAULT_BUFFER_LENGTH;
            buf = new byte[DEFAULT_BUFFER_LENGTH];
            count = 0;
        }
        @Override
        public void close() {
            flush();
            hasBeenClosed = true;
        }
        @Override
        public void write ( final int b ) throws IOException {
            if ( hasBeenClosed ) {
                throw new IOException ( "The stream has been closed." );
            }
            if ( b == 0 ) {
                return;
            }
            if ( count == bufLength ) {
                final int newBufLength = bufLength + DEFAULT_BUFFER_LENGTH;
                final byte[] newBuf = new byte[newBufLength];
                System.arraycopy ( buf, 0, newBuf, 0, bufLength );
                buf = newBuf;
                bufLength = newBufLength;
            }
            buf[count] = ( byte ) b;
            count++;
        }
        @Override
        public void flush() {
            if ( count == 0 ) {
                return;
            }
            if ( count == LINE_SEPERATOR.length() ) {
                if ( ( ( char ) buf[0] ) == LINE_SEPERATOR.charAt ( 0 ) && ( ( count == 1 ) ||
                        ( ( count == 2 ) && ( ( char ) buf[1] ) == LINE_SEPERATOR.charAt ( 1 ) ) ) ) {
                    reset();
                    return;
                }
            }
            final byte[] theBytes = new byte[count];
            System.arraycopy ( buf, 0, theBytes, 0, count );
            if ( isError ) {
                log.error ( new String ( theBytes ) );
            } else {
                log.info ( new String ( theBytes ) );
            }
            reset();
        }
        private void reset() {
            count = 0;
        }
    }
}
