public abstract class AjaxTimerFormSubmitBehavior extends AbstractAjaxTimerBehavior {
    private Form<?> __form;
    private boolean defaultProcessing = true;
    public AjaxTimerFormSubmitBehavior ( Duration updateInterval ) {
        this ( null, updateInterval );
    }
    public AjaxTimerFormSubmitBehavior ( Form<?> form, Duration updateInterval ) {
        super ( updateInterval );
        __form = form;
        if ( form != null ) {
            form.setOutputMarkupId ( true );
        }
    }
    @Override
    protected void onTimer ( final AjaxRequestTarget target ) {
        getForm().getRootForm().onFormSubmitted ( new IFormSubmitter() {
            public Form<?> getForm() {
                return AjaxTimerFormSubmitBehavior.this.getForm();
            }
            public boolean getDefaultFormProcessing() {
                return AjaxTimerFormSubmitBehavior.this.getDefaultProcessing();
            }
            public void onSubmit() {
                AjaxTimerFormSubmitBehavior.this.onSubmit ( target );
            }
            public void onError() {
                AjaxTimerFormSubmitBehavior.this.onError ( target );
            }
        } );
    }
    public final Form<?> getForm() {
        if ( __form == null ) {
            __form = findForm();
            if ( __form == null ) {
                throw new IllegalStateException (
                    "form was not specified in the constructor and cannot "
                    + "be found in the hierarchy of the component this behavior "
                    + "is attached to: Component="
                    + getComponent().toString ( false ) );
            }
        }
        return __form;
    }
    public boolean getDefaultProcessing() {
        return defaultProcessing;
    }
    protected Form<?> findForm() {
        Component component = getComponent();
        if ( component instanceof Form<?> ) {
            return ( Form<?> ) component;
        } else {
            return component.findParent ( Form.class );
        }
    }
    protected abstract void onSubmit ( AjaxRequestTarget target );
    protected abstract void onError ( AjaxRequestTarget target );
}
