public class AndroidImage {
    private Bitmap image;
    private String formatName;
    private int width;
    private int height;
    protected int[] colorArray;
    public AndroidImage ( Bitmap img ) {
        this.image =  img;
        this.formatName = "jpg";
        this.width = img.getWidth();
        this.height = img.getHeight();
        updateColorArray();
    }
    public void clearImage ( int color ) {
        for ( int y = 0; y < height; y++ ) {
            for ( int x = 0; x < width; x++ ) {
                image.setPixel ( x, y, color );
            }
        }
    }
    private void updateColorArray() {
        colorArray = new int[width * height];
        image.getPixels ( colorArray, 0, width, 0, 0, width, height );
        int r, g, b;
        for ( int y = 0; y < height; y++ ) {
            for ( int x = 0; x < width; x++ ) {
                int index = y * width + x;
                r = ( colorArray[index] >> 16 ) & 0xff;
                g = ( colorArray[index] >> 8 ) & 0xff;
                b = colorArray[index] & 0xff;
                colorArray[index] = 0xff000000 | ( r << 16 ) | ( g << 8 ) | b;
            }
        }
    }
    public void setPixelColor ( int x, int y, int color ) {
        colorArray[ ( ( y * image.getWidth() + x ) )] = color;
        image.setPixel ( x, y, color );
    }
    public int getPixelColor ( int x, int y ) {
        return colorArray[y * width + x];
    }
    public void setPixelColor ( int x, int y, int c0, int c1, int c2 ) {
        colorArray[ ( ( y * image.getWidth() + x ) )] = ( 255 << 24 ) + ( c0 << 16 ) + ( c1 << 8 ) + c2;
        image.setPixel ( x, y, colorArray[ ( ( y * image.getWidth() + x ) )] );
    }
    public int getRComponent ( int x, int y ) {
        return ( getColorArray() [ ( ( y * width + x ) )] & 0x00FF0000 ) >>> 16;
    }
    public int getGComponent ( int x, int y ) {
        return ( getColorArray() [ ( ( y * width + x ) )] & 0x0000FF00 ) >>> 8;
    }
    public int getBComponent ( int x, int y ) {
        return ( getColorArray() [ ( ( y * width + x ) )] & 0x000000FF );
    }
    public void rotate ( int rotateDegrees ) {
        Matrix mtx = new Matrix();
        mtx.postRotate ( rotateDegrees );
        image = Bitmap.createBitmap ( image, 0, 0, width, height, mtx, true );
        width = image.getWidth();
        height = image.getHeight();
        updateColorArray();
    }
    public Bitmap getImage() {
        return image;
    }
    public void setImage ( Bitmap image ) {
        this.image = image;
    }
    public String getFormatName() {
        return formatName;
    }
    public void setFormatName ( String formatName ) {
        this.formatName = formatName;
    }
    public int getWidth() {
        return width;
    }
    public void setWidth ( int width ) {
        this.width = width;
    }
    public int getHeight() {
        return height;
    }
    public void setHeight ( int height ) {
        this.height = height;
    }
    public int[] getColorArray() {
        return colorArray;
    }
    public void setColorArray ( int[] colorArray ) {
        this.colorArray = colorArray;
    }
}
