import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.LayoutManager;
import java.awt.Rectangle;
import java.util.Hashtable;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.Scrollable;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
public class TestTextArea {
    private void initUI() {
        JFrame frame = new JFrame ( "test" );
        class ScrollablePanel extends JPanel implements Scrollable {
            @Override
            public Dimension getPreferredScrollableViewportSize() {
                return getPreferredSize();
            }
            @Override
            public int getScrollableUnitIncrement ( Rectangle visibleRect, int orientation, int direction ) {
                switch ( orientation ) {
                case SwingConstants.VERTICAL:
                    return visibleRect.height / 10;
                case SwingConstants.HORIZONTAL:
                    return visibleRect.width / 10;
                default:
                    throw new IllegalArgumentException ( "Invalid orientation: " + orientation );
                }
            }
            @Override
            public int getScrollableBlockIncrement ( Rectangle visibleRect, int orientation, int direction ) {
                switch ( orientation ) {
                case SwingConstants.VERTICAL:
                    return visibleRect.height;
                case SwingConstants.HORIZONTAL:
                    return visibleRect.width;
                default:
                    throw new IllegalArgumentException ( "Invalid orientation: " + orientation );
                }
            }
            @Override
            public boolean getScrollableTracksViewportWidth() {
                return true;
            }
            @Override
            public boolean getScrollableTracksViewportHeight() {
                return false;
            }
        }
        JPanel listPanel = new ScrollablePanel();
        listPanel.setLayout ( new VerticalLayout ( 5, VerticalLayout.BOTH ) );
        JPanel mainPanel = new JPanel ( new GridLayout() );
        JScrollPane sp = new JScrollPane();
        sp.setHorizontalScrollBarPolicy ( javax.swing.ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER );
        sp.setViewportView ( listPanel );
        mainPanel.add ( sp );
        listPanel.add ( new MyPanel() );
        listPanel.add ( new MyPanel() );
        listPanel.add ( new MyPanel() );
        listPanel.add ( new MyPanel() );
        listPanel.add ( new MyPanel() );
        frame.setDefaultCloseOperation ( JFrame.EXIT_ON_CLOSE );
        frame.add ( mainPanel );
        frame.pack();
        frame.setVisible ( true );
    }
    public static void main ( String[] args ) {
        SwingUtilities.invokeLater ( new Runnable() {
            @Override
            public void run() {
                new TestTextArea().initUI();
            }
        } );
    }
    class MyPanel extends JPanel {
        public MyPanel() {
            this.setLayout ( new BorderLayout() );
            JTextArea ta = new JTextArea();
            ta.setText ( "Hello world Hello world Hello world Hello world " + "Hello world Hello world Hello world Hello world "
                         + "Hello world Hello world Hello world Hello world " + "Hello world Hello world Hello world Hello world " );
            ta.setColumns ( 20 );
            ta.setEditable ( false );
            ta.setLineWrap ( true );
            ta.setRows ( 5 );
            ta.setWrapStyleWord ( true );
            this.add ( ta, BorderLayout.CENTER );
        }
    }
    public static class VerticalLayout implements LayoutManager {
        public final static int CENTER = 0;
        public final static int RIGHT = 1;
        public final static int LEFT = 2;
        public final static int BOTH = 3;
        public final static int TOP = 1;
        public final static int BOTTOM = 2;
        private int vgap;
        private int alignment;
        private int anchor;
        private Hashtable comps;
        public VerticalLayout() {
            this ( 5, CENTER, TOP );
        }
        public VerticalLayout ( int vgap ) {
            this ( vgap, CENTER, TOP );
        }
        public VerticalLayout ( int vgap, int alignment ) {
            this ( vgap, alignment, TOP );
        }
        public VerticalLayout ( int vgap, int alignment, int anchor ) {
            this.vgap = vgap;
            this.alignment = alignment;
            this.anchor = anchor;
        }
        private Dimension layoutSize ( Container parent, boolean minimum ) {
            Dimension dim = new Dimension ( 0, 0 );
            Dimension d;
            synchronized ( parent.getTreeLock() ) {
                int n = parent.getComponentCount();
                for ( int i = 0; i < n; i++ ) {
                    Component c = parent.getComponent ( i );
                    if ( c.isVisible() ) {
                        d = minimum ? c.getMinimumSize() : c.getPreferredSize();
                        dim.width = Math.max ( dim.width, d.width );
                        dim.height += d.height;
                        if ( i > 0 ) {
                            dim.height += vgap;
                        }
                    }
                }
            }
            Insets insets = parent.getInsets();
            dim.width += insets.left + insets.right;
            dim.height += insets.top + insets.bottom + vgap + vgap;
            return dim;
        }
        @Override
        public void layoutContainer ( Container parent ) {
            Insets insets = parent.getInsets();
            synchronized ( parent.getTreeLock() ) {
                int n = parent.getComponentCount();
                Dimension pd = parent.getSize();
                int y = 0;
                for ( int i = 0; i < n; i++ ) {
                    Component c = parent.getComponent ( i );
                    Dimension d = c.getPreferredSize();
                    y += d.height + vgap;
                }
                y -= vgap;
                if ( anchor == TOP ) {
                    y = insets.top;
                } else if ( anchor == CENTER ) {
                    y = ( pd.height - y ) / 2;
                } else {
                    y = pd.height - y - insets.bottom;
                }
                for ( int i = 0; i < n; i++ ) {
                    Component c = parent.getComponent ( i );
                    Dimension d = c.getPreferredSize();
                    int x = insets.left;
                    int wid = d.width;
                    if ( alignment == CENTER ) {
                        x = ( pd.width - d.width ) / 2;
                    } else if ( alignment == RIGHT ) {
                        x = pd.width - d.width - insets.right;
                    } else if ( alignment == BOTH ) {
                        wid = pd.width - insets.left - insets.right;
                    }
                    c.setBounds ( x, y, wid, d.height );
                    y += d.height + vgap;
                }
            }
        }
        @Override
        public Dimension minimumLayoutSize ( Container parent ) {
            return layoutSize ( parent, false );
        }
        @Override
        public Dimension preferredLayoutSize ( Container parent ) {
            return layoutSize ( parent, false );
        }
        @Override
        public void addLayoutComponent ( String name, Component comp ) {
        }
        @Override
        public void removeLayoutComponent ( Component comp ) {
        }
        @Override
        public String toString() {
            return getClass().getName() + "[vgap=" + vgap + " align=" + alignment + " anchor=" + anchor + "]";
        }
    }
}
