import java.util.HashMap;
import java.util.HashSet;
public class Multiset<E> extends HashSet<E> {
    private static final long   serialVersionUID = -9013417064272046980L;
    private final HashMap<E, Integer> multiplicities  = new HashMap<E, Integer>();
    public boolean add ( E element ) {
        return add ( element, 1 );
    }
    private boolean add ( E element, int copies ) {
        if ( !contains ( element ) ) {
            multiplicities.put ( element, 0 );
        }
        int n = multiplicities.get ( element );
        multiplicities.put ( element, n + copies );
        return super.add ( element );
    }
    public boolean addAll ( Multiset<E> that ) {
        boolean flag = false;
        for ( E element : that ) {
            flag = add ( element, that.multiplicity ( element ) );
        }
        return flag;
    }
    public Multiset<E> setView() {
        Multiset<E> set = new Multiset<E>();
        for ( E o : multiplicities.keySet() ) {
            set.add ( o );
        }
        return set;
    }
    public Multiset<E> union ( Multiset<E> that ) {
        HashSet<E> both = new HashSet<E>();
        both.addAll ( this );
        both.addAll ( that );
        Multiset<E> union = new Multiset<E>();
        for ( E element : both ) {
            union.add ( element, Math.max ( this.multiplicity ( element ), that.multiplicity ( element ) ) );
        }
        return union;
    }
    public Multiset<E> intersect ( Multiset<E> that ) {
        Multiset<E> intersection = new Multiset<E>();
        final Multiset<E> other = that.setView();
        for ( E element : this.setView() )
            if ( other.contains ( element ) ) {
                intersection.add ( element, Math.min ( this.multiplicity ( element ), that.multiplicity ( element ) ) );
            }
        return intersection;
    }
    public int multiplicity ( E element ) {
        return contains ( element ) ? multiplicities.get ( element ) : 0;
    }
    public int cardinality() {
        int card = 0;
        for ( Integer n : multiplicities.values() ) {
            card += n;
        }
        return card;
    }
    public int similarityOfMultisets ( Multiset<E> that ) {
        return union ( that ).cardinality() - intersect ( that ).cardinality();
    }
}
