import java.io.*;
import java.net.URI;
import java.util.Deque;
import java.util.Enumeration;
import java.util.LinkedList;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipOutputStream;
public class Main {
    public static void main ( String[] args ) throws IOException {
        File testJar = new File ( "test.jar" );
        File testDir = new File ( "test" );
        File jarPack = new File ( "packed_test.jar" );
        unpack ( testJar, testDir );
        jar ( testDir, jarPack );
    }
    public static void jar ( File directory, File zipfile ) throws IOException {
        URI base = directory.toURI();
        Deque<File> queue = new LinkedList<>();
        queue.push ( directory );
        OutputStream out = new FileOutputStream ( zipfile );
        Closeable res = out;
        try {
            ZipOutputStream zout = new ZipOutputStream ( out );
            res = zout;
            while ( !queue.isEmpty() ) {
                directory = queue.pop();
                for ( File kid : directory.listFiles() ) {
                    String name = base.relativize ( kid.toURI() ).getPath();
                    if ( kid.isDirectory() ) {
                        queue.push ( kid );
                        name = name.endsWith ( "/" ) ? name : name + "/";
                        zout.putNextEntry ( new ZipEntry ( name ) );
                    } else {
                        zout.putNextEntry ( new ZipEntry ( name ) );
                        copy ( kid, zout );
                        zout.closeEntry();
                    }
                }
            }
        } finally {
            res.close();
        }
    }
    private static void copy ( File file, OutputStream out ) throws IOException {
        try ( InputStream in = new FileInputStream ( file ) ) {
            byte[] buffer = new byte[1024];
            while ( true ) {
                int readCount = in.read ( buffer );
                if ( readCount < 0 ) {
                    break;
                }
                out.write ( buffer, 0, readCount );
            }
        }
    }
    public static void unpack ( File zip, File extractTo ) throws IOException {
        ZipFile archive = new ZipFile ( zip );
        Enumeration e = archive.entries();
        while ( e.hasMoreElements() ) {
            ZipEntry entry = ( ZipEntry ) e.nextElement();
            File file = new File ( extractTo, entry.getName() );
            if ( entry.isDirectory() && !file.exists() ) {
                file.mkdirs();
            } else {
                if ( !file.getParentFile().exists() ) {
                    file.getParentFile().mkdirs();
                }
                BufferedOutputStream out;
                try ( InputStream in = archive.getInputStream ( entry ) ) {
                    out = new BufferedOutputStream (
                        new FileOutputStream ( file ) );
                    byte[] buffer = new byte[8192];
                    int read;
                    while ( -1 != ( read = in.read ( buffer ) ) ) {
                        out.write ( buffer, 0, read );
                    }
                }
                out.close();
            }
        }
    }
}
