import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.security.InvalidKeyException;
import java.security.KeyFactory;
import java.security.NoSuchAlgorithmException;
import java.security.PublicKey;
import java.security.Signature;
import java.security.SignatureException;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.X509EncodedKeySpec;
import org.bouncycastle.util.io.pem.PemReader;
import org.castor.util.Base64Decoder;
import fr.paris.lutece.portal.service.util.AppLogService;
public final class PayboxUtil {
    private static final String CHARSET = "utf-8";
    private static final String ENCRYPTION_ALGORITHM = "RSA";
    private static final String HASH_ENCRYPTION_ALGORITHM = "SHA1withRSA";
    private PayboxUtil( ) {
    }
    public static boolean checkSign ( String message, String sign, String keyPath ) {
        boolean ret = false;
        try {
            ret = PayboxUtil.verify ( message, sign, PayboxUtil.getKey ( keyPath ) );
        } catch ( final FileNotFoundException e ) {
            AppLogService.error ( e );
        } catch ( final IOException e ) {
            AppLogService.error ( e );
        } catch ( final NoSuchAlgorithmException e ) {
            AppLogService.error ( e );
        } catch ( final InvalidKeySpecException e ) {
            AppLogService.error ( e );
        } catch ( final InvalidKeyException e ) {
            AppLogService.error ( e );
        } catch ( final SignatureException e ) {
            AppLogService.error ( e );
        }
        return ret;
    }
    private static PublicKey getKey ( String keyPath )
    throws NoSuchAlgorithmException, IOException, InvalidKeySpecException {
        final KeyFactory keyFactory = KeyFactory.getInstance ( PayboxUtil.ENCRYPTION_ALGORITHM );
        final PemReader reader = new PemReader ( new FileReader ( keyPath ) );
        final byte[] pubKey = reader.readPemObject( ).getContent( );
        final X509EncodedKeySpec publicKeySpec = new X509EncodedKeySpec ( pubKey );
        return keyFactory.generatePublic ( publicKeySpec );
    }
    private static boolean verify ( String message, String sign, PublicKey publicKey )
    throws NoSuchAlgorithmException, InvalidKeyException, SignatureException, UnsupportedEncodingException {
        final Signature sig = Signature.getInstance ( PayboxUtil.HASH_ENCRYPTION_ALGORITHM );
        sig.initVerify ( publicKey );
        sig.update ( message.getBytes ( PayboxUtil.CHARSET ) );
        final byte[] bytes = Base64Decoder.decode ( URLDecoder.decode ( sign, PayboxUtil.CHARSET ) );
        return sig.verify ( bytes );
    }
}
