import java.util.*;
import java.io.*;
class RuntimeStreamer extends Thread {
    InputStream is;
    String lines;
    RuntimeStreamer ( InputStream is ) {
        this.is = is;
        this.lines = "";
    }
    public String contents() {
        return this.lines;
    }
    public void run() {
        try {
            InputStreamReader isr = new InputStreamReader ( is );
            BufferedReader    br  = new BufferedReader ( isr );
            String line = null;
            while ( ( line = br.readLine() ) != null ) {
                this.lines += line + "\n";
            }
        } catch ( IOException ioe ) {
            ioe.printStackTrace();
        }
    }
    public static String execute ( String[] cmdArray ) {
        try {
            Runtime runtime = Runtime.getRuntime();
            Process proc = runtime.exec ( cmdArray );
            RuntimeStreamer outputStreamer = new RuntimeStreamer ( proc.getInputStream() );
            RuntimeStreamer errorStreamer  = new RuntimeStreamer ( proc.getErrorStream() );
            outputStreamer.start();
            errorStreamer.start();
            proc.waitFor();
            return outputStreamer.contents() + errorStreamer.contents();
        } catch ( Throwable t ) {
            t.printStackTrace();
        }
        return null;
    }
    public static String execute ( String cmd ) {
        String[] cmdArray = { cmd };
        return RuntimeStreamer.execute ( cmdArray );
    }
}
class JavaInfo {
    public String  path;
    public String  version;
    public boolean is64bits;
    public JavaInfo ( String javaPath ) {
        String versionInfo = RuntimeStreamer.execute ( new String[] { javaPath, "-version" } );
        String[] tokens = versionInfo.split ( "\"" );
        if ( tokens.length < 2 ) {
            this.version = "Unkown";
        } else {
            this.version = tokens[1];
        }
        this.is64bits = versionInfo.toUpperCase().contains ( "64-BIT" );
        this.path     = javaPath;
    }
    public String toString() {
        return this.path + ":\n  Version: " + this.version + "\n  Bitness: " + ( this.is64bits ? "64-bits" : "32-bits" );
    }
}
public class JavaFinder {
    private static List<String> searchRegistry ( String key, int wow64, List<String> previous ) {
        List<String> result = previous;
        try {
            List<String> entries = WinRegistry.readStringSubKeys ( WinRegistry.HKEY_LOCAL_MACHINE, key, wow64 );
            for ( int i = 0; entries != null && i < entries.size(); i++ ) {
                String val = WinRegistry.readString ( WinRegistry.HKEY_LOCAL_MACHINE, key + "\\" + entries.get ( i ), "JavaHome", wow64 );
                if ( !result.contains ( val + "\\bin\\java.exe" ) ) {
                    result.add ( val + "\\bin\\java.exe" );
                }
            }
        } catch ( Throwable t ) {
            t.printStackTrace();
        }
        return result;
    }
    public static List<JavaInfo> findJavas() {
        List<String> javaExecs = new ArrayList<String>();
        javaExecs = JavaFinder.searchRegistry ( "SOFTWARE\\JavaSoft\\Java Runtime Environment", WinRegistry.KEY_WOW64_32KEY, javaExecs );
        javaExecs = JavaFinder.searchRegistry ( "SOFTWARE\\JavaSoft\\Java Runtime Environment", WinRegistry.KEY_WOW64_64KEY, javaExecs );
        javaExecs = JavaFinder.searchRegistry ( "SOFTWARE\\JavaSoft\\Java Development Kit",     WinRegistry.KEY_WOW64_32KEY, javaExecs );
        javaExecs = JavaFinder.searchRegistry ( "SOFTWARE\\JavaSoft\\Java Development Kit",     WinRegistry.KEY_WOW64_64KEY, javaExecs );
        javaExecs.add ( System.getenv ( "WINDIR" ) + "\\system32\\java.exe" );
        javaExecs.add ( System.getenv ( "WINDIR" ) + "\\SysWOW64\\java.exe" );
        List<JavaInfo> result = new ArrayList<JavaInfo>();
        for ( String javaPath : javaExecs ) {
            if ( ! ( new File ( javaPath ).exists() ) ) {
                continue;
            }
            result.add ( new JavaInfo ( javaPath ) );
        }
        return result;
    }
    public static String getOSBitnessJava() {
        String arch      = System.getenv ( "PROCESSOR_ARCHITECTURE" );
        String wow64Arch = System.getenv ( "PROCESSOR_ARCHITEW6432" );
        boolean isOS64 = arch.endsWith ( "64" ) || ( wow64Arch != null && wow64Arch.endsWith ( "64" ) );
        List<JavaInfo> javas = JavaFinder.findJavas();
        for ( int i = 0; i < javas.size(); i++ ) {
            if ( javas.get ( i ).is64bits == isOS64 ) {
                return javas.get ( i ).path;
            }
        }
        return null;
    }
    public static void main ( String [] args ) {
        List<JavaInfo> javas = JavaFinder.findJavas();
        for ( int i = 0; i < javas.size(); i++ ) {
            System.out.println ( "\n" + javas.get ( i ) );
        }
    }
}
