import java.io.FileOutputStream;
import java.io.IOException;
import java.security.AlgorithmParameters;
import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.SecureRandom;
import java.security.Security;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.InvalidParameterSpecException;
import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.EncryptedPrivateKeyInfo;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.SecretKey;
import javax.crypto.SecretKeyFactory;
import javax.crypto.spec.PBEKeySpec;
import javax.crypto.spec.PBEParameterSpec;
import org.bouncycastle.asn1.bc.BCObjectIdentifiers;
public class EncodePKCS8 {
    public static void main ( String[] args ) throws NoSuchAlgorithmException,
               InvalidKeySpecException, NoSuchPaddingException, InvalidKeyException,
               InvalidAlgorithmParameterException, IllegalBlockSizeException, BadPaddingException,
        InvalidParameterSpecException, IOException, NoSuchProviderException {
        Security.addProvider ( new org.bouncycastle.jce.provider.BouncyCastleProvider() );
        String password = "Very long and complex password";
        KeyPairGenerator keyPairGenerator = KeyPairGenerator.getInstance ( "RSA" );
        keyPairGenerator.initialize ( 2048 );
        KeyPair keyPair = keyPairGenerator.genKeyPair();
        byte[] encryptedPkcs8 = encryptPrivateKey ( password, keyPair );
        FileOutputStream fos = new FileOutputStream ( "privkey.p8" );
        fos.write ( encryptedPkcs8 );
        fos.close();
        return;
    }
    private static byte[] encryptPrivateKey ( String password, KeyPair keyPair )
    throws InvalidKeyException, NoSuchAlgorithmException, NoSuchProviderException,
        InvalidKeySpecException, NoSuchPaddingException,
        InvalidAlgorithmParameterException, IllegalBlockSizeException,
        BadPaddingException, InvalidParameterSpecException, IOException {
        int count = 100000;
        return encryptPrivateKey ( password, keyPair, count );
    }
    private static byte[] encryptPrivateKey ( String password,
            KeyPair keyPair, int count ) throws NoSuchAlgorithmException,
        NoSuchProviderException, InvalidKeySpecException,
        NoSuchPaddingException, InvalidKeyException,
        InvalidAlgorithmParameterException, IllegalBlockSizeException,
        BadPaddingException, InvalidParameterSpecException, IOException {
        byte[] encodedprivkey = keyPair.getPrivate().getEncoded();
        String encAlg = BCObjectIdentifiers.bc_pbe_sha256_pkcs12_aes256_cbc.getId();
        SecureRandom random = new SecureRandom();
        byte[] salt = new byte[16];
        random.nextBytes ( salt );
        PBEParameterSpec pbeParamSpec = new PBEParameterSpec ( salt, count );
        PBEKeySpec pbeKeySpec = new PBEKeySpec ( password.toCharArray() );
        SecretKeyFactory keyFac = SecretKeyFactory.getInstance ( encAlg, "BC" );
        SecretKey pbeKey = keyFac.generateSecret ( pbeKeySpec );
        Cipher pbeCipher = Cipher.getInstance ( encAlg, "BC" );
        pbeCipher.init ( Cipher.ENCRYPT_MODE, pbeKey, pbeParamSpec );
        byte[] ciphertext = pbeCipher.doFinal ( encodedprivkey );
        AlgorithmParameters algparms = AlgorithmParameters.getInstance ( encAlg, "BC" );
        algparms.init ( pbeParamSpec );
        EncryptedPrivateKeyInfo encinfo = new EncryptedPrivateKeyInfo ( algparms, ciphertext );
        byte[] encryptedPkcs8 = encinfo.getEncoded();
        return encryptedPkcs8;
    }
}
