public class MutableTableColumnModel extends DefaultTableColumnModel {
    private List<TableColumn> allTableColumns;
    public MutableTableColumnModel() {
        allTableColumns = new ArrayList<TableColumn> ( 10 );
    }
    public void setColumnVisible ( int index, boolean visible ) {
        setColumnVisible ( getColumnByModelIndex ( index ), visible );
    }
    public void setColumnVisible ( TableColumn column, boolean visible ) {
        if ( column != null ) {
            if ( !visible ) {
                if ( isColumnVisible ( column ) ) {
                    super.removeColumn ( column );
                }
            } else {
                int noVisibleColumns = tableColumns.size();
                int noInvisibleColumns = allTableColumns.size();
                int visibleIndex = 0;
                for ( int invisibleIndex = 0; invisibleIndex < noInvisibleColumns; ++invisibleIndex ) {
                    TableColumn visibleColumn = ( visibleIndex < noVisibleColumns ? ( TableColumn ) tableColumns.get ( visibleIndex ) : null );
                    TableColumn testColumn = ( TableColumn ) allTableColumns.get ( invisibleIndex );
                    if ( testColumn == column ) {
                        if ( visibleColumn != column ) {
                            super.addColumn ( column );
                            super.moveColumn ( tableColumns.size() - 1, visibleIndex );
                        }
                        return;
                    }
                    if ( testColumn == visibleColumn ) {
                        ++visibleIndex;
                    }
                }
            }
        }
    }
    public void showAllColumns() {
        int noColumns = allTableColumns.size();
        for ( int columnIndex = 0; columnIndex < noColumns; ++columnIndex ) {
            TableColumn visibleColumn = ( columnIndex < tableColumns.size() ? ( TableColumn ) tableColumns.get ( columnIndex ) : null );
            TableColumn invisibleColumn = ( TableColumn ) allTableColumns.get ( columnIndex );
            if ( visibleColumn != invisibleColumn ) {
                super.addColumn ( invisibleColumn );
                super.moveColumn ( tableColumns.size() - 1, columnIndex );
            }
        }
    }
    public TableColumn getColumnByModelIndex ( int modelColumnIndex ) {
        for ( int columnIndex = 0; columnIndex < allTableColumns.size(); ++columnIndex ) {
            TableColumn column = ( TableColumn ) allTableColumns.get ( columnIndex );
            if ( column.getModelIndex() == modelColumnIndex ) {
                return column;
            }
        }
        return null;
    }
    public boolean isColumnVisible ( TableColumn aColumn ) {
        return ( tableColumns.indexOf ( aColumn ) >= 0 );
    }
    public void addColumn ( TableColumn column ) {
        allTableColumns.add ( column );
        super.addColumn ( column );
    }
    public void removeColumn ( TableColumn column ) {
        int allColumnsIndex = allTableColumns.indexOf ( column );
        if ( allColumnsIndex != -1 ) {
            allTableColumns.remove ( allColumnsIndex );
        }
        super.removeColumn ( column );
    }
    public void moveColumn ( int oldIndex, int newIndex ) {
        if ( ( oldIndex < 0 ) || ( oldIndex >= getColumnCount() )
                || ( newIndex < 0 ) || ( newIndex >= getColumnCount() ) ) {
            throw new IllegalArgumentException ( "moveColumn() - Index out of range" );
        }
        TableColumn fromColumn = ( TableColumn ) tableColumns.get ( oldIndex );
        TableColumn toColumn = ( TableColumn ) tableColumns.get ( newIndex );
        int allColumnsOldIndex = allTableColumns.indexOf ( fromColumn );
        int allColumnsNewIndex = allTableColumns.indexOf ( toColumn );
        if ( oldIndex != newIndex ) {
            allTableColumns.remove ( allColumnsOldIndex );
            allTableColumns.add ( allColumnsNewIndex, fromColumn );
        }
        super.moveColumn ( oldIndex, newIndex );
    }
    public int getColumnCount ( boolean onlyVisible ) {
        List<TableColumn> columns = ( onlyVisible ? tableColumns : allTableColumns );
        return columns.size();
    }
    public Iterator<TableColumn> getColumns ( boolean onlyVisible ) {
        List<TableColumn> columns = ( onlyVisible ? tableColumns : allTableColumns );
        return columns.iterator();
    }
    public int getColumnIndex ( Object identifier, boolean onlyVisible ) {
        if ( identifier == null ) {
            throw new IllegalArgumentException ( "Identifier is null" );
        }
        List columns = ( onlyVisible ? tableColumns : allTableColumns );
        int noColumns = columns.size();
        TableColumn column;
        for ( int columnIndex = 0; columnIndex < noColumns; ++columnIndex ) {
            column = ( TableColumn ) columns.get ( columnIndex );
            if ( identifier.equals ( column.getIdentifier() ) ) {
                return columnIndex;
            }
        }
        throw new IllegalArgumentException ( "Identifier not found" );
    }
    public TableColumn getColumn ( int columnIndex, boolean onlyVisible ) {
        return ( TableColumn ) tableColumns.elementAt ( columnIndex );
    }
    public TableColumn[] getAllColumns() {
        return allTableColumns.toArray ( new TableColumn[allTableColumns.size()] );
    }
    public TableColumn[] getHiddenColumns() {
        ArrayList<TableColumn> lstColumns = new ArrayList<TableColumn> ( 10 );
        for ( TableColumn column : getAllColumns() ) {
            if ( !isColumnVisible ( column ) ) {
                lstColumns.add ( column );
            }
        }
        return lstColumns.toArray ( new TableColumn[lstColumns.size()] );
    }
}
