package wag.contribution.addons;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.inject.Inject;
import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.e4.ui.model.application.MApplication;
import org.eclipse.e4.ui.model.application.ui.MElementContainer;
import org.eclipse.e4.ui.model.application.ui.MUIElement;
import org.eclipse.e4.ui.model.application.ui.advanced.MPlaceholder;
import org.eclipse.e4.ui.workbench.modeling.EModelService;
import org.osgi.service.event.Event;
import org.osgi.service.event.EventHandler;
public class LayoutSorter {
    @Inject private IEventBroker broker;
    private EventHandler handler;
    private static List<String> PART_IDS = Arrays.asList ( new String[] {
                                               "wag.aView", "wag.bView", "org.eclipse.ui.editorss"
                                           } );
    @PostConstruct
    void hookListeners ( final MApplication application,
                         final EModelService service ) {
        if ( handler == null ) {
            handler = new EventHandler() {
                @Override
                public void handleEvent ( Event event ) {
                    try {
                        sort ( application, service );
                        broker.unsubscribe ( handler );
                    } catch ( Exception e ) {
                    }
                }
            };
            broker.subscribe ( "org/osgi/framework/ServiceEvent/MODIFIED",
                               handler );
        }
    }
    private void sort ( MApplication application, EModelService service ) {
        List<MPlaceholder> placeholders = service.findElements ( application,
                                          null, MPlaceholder.class, null );
        for ( int i = placeholders.size() - 1; i > -1; i-- ) {
            if ( !PART_IDS.contains ( placeholders.get ( i ).getElementId() ) ) {
                placeholders.remove ( i );
            }
        }
        List<MElementContainer<MUIElement>> parents = new ArrayList<> (
            placeholders.size() );
        for ( MPlaceholder placeholder : placeholders ) {
            parents.add ( placeholder.getParent() );
        }
        MElementContainer<MUIElement> targetParent = null;
        for ( MElementContainer<MUIElement> parent : parents ) {
            for ( MUIElement child : parent.getChildren() ) {
                if ( parents.contains ( child ) ) {
                    continue;
                }
                targetParent = parent;
            }
        }
        if ( targetParent != null ) {
            for ( int i = 0; i < placeholders.size(); i++ ) {
                if ( targetParent != placeholders.get ( i ).getParent() ) {
                    service.move ( placeholders.get ( i ), targetParent, i );
                }
            }
        }
    }
    @PreDestroy
    void unhookListeners() {
        if ( handler != null ) {
            broker.unsubscribe ( handler );
        }
    }
}
