public static DetachedCriteria createDetachedCriteria ( final Class<?> type, final String identifierPropertyName,
        final Map<String, Criterion> filters ) {
    final DetachedCriteria criteria = DetachedCriteria.forClass ( type );
    final TreeNode<Entry<String, Criterion>> rootNode = HibernateUtils2.createTree ( filters.entrySet(),
            identifierPropertyName );
    final Iterator<TreeNode<Entry<String, Criterion>>> it = rootNode.getChildren().iterator();
    while ( it.hasNext() ) {
        HibernateUtils.addRestrictions ( criteria, it.next() );
    }
    return criteria;
}
public static <T extends Object> TreeNode<Entry<String, T>> createTree ( final Set<Entry<String, T>> filters,
        final String identifierPropertyName ) {
    final Iterator<Entry<String, Object>> it = filters.iterator();
    final TreeNode<Entry<String, Object>> rootNode = new TreeNode<Entry<String, Object>> (
        new SimpleEntry<String, Object> ( "root", null ) );
    while ( it.hasNext() ) {
        final Entry<String, Object> entry = it.next();
        final String key = entry.getKey();
        String[] props;
        if ( key.contains ( "." ) ) {
            props = key.split ( "\\." );
            if ( !StringUtils.isBlank ( identifierPropertyName ) ) {
                int propsTempLength = props.length - 1;
                if ( props[propsTempLength].equals ( identifierPropertyName ) ) {
                    props = Arrays.copyOf ( props, propsTempLength );
                    propsTempLength--;
                    props[propsTempLength] = props[propsTempLength] + "." + identifierPropertyName;
                }
            }
            if ( props.length > 1 && props[0].equals ( "this" ) ) {
                props[0] = "this." + props[1];
                props = ArrayUtils.remove ( props, 1 );
            }
        } else
            props = new String[] {
                key
            };
        TreeNode<Entry<String, Object>> currNode = rootNode;
        for ( int i = 0; i < props.length; i++ ) {
            Object valueAdd;
            if ( i != props.length - 1 ) {
                valueAdd = null;
            } else {
                valueAdd = entry.getValue();
            }
            final TreeNode<Entry<String, Object>> childTempNode = new TreeNode<Entry<String, Object>> (
                new SimpleEntry<String, Object> ( props[i], valueAdd ) );
            TreeNode<Entry<String, Object>> childNode = currNode.getChild ( childTempNode.getElement() );
            if ( childNode == null ) {
                childNode = childTempNode;
                currNode.addChild ( childNode );
            }
            currNode = childNode;
        }
    }
    return rootNode;
}
public static void addRestrictions ( final DetachedCriteria criteria,
                                     final TreeNode<Entry<String, Criterion>> treeNode ) {
    if ( treeNode.getChildren().size() == 0 ) {
        criteria.add ( treeNode.getElement().getValue() );
    } else {
        final DetachedCriteria subCriteria = criteria.createCriteria ( treeNode.getElement().getKey() );
        final Iterator<TreeNode<Entry<String, Criterion>>> it = treeNode.getChildren().iterator();
        while ( it.hasNext() ) {
            HibernateUtils.addRestrictions ( subCriteria, it.next() );
        }
    }
}
public class TreeNode<T> {
    private final T element;
    private final Set<TreeNode<T>> childrens;
    public TreeNode ( final T element ) {
        if ( element == null ) {
            throw new IllegalArgumentException ( "Element cannot be null" );
        }
        this.element = element;
        this.childrens = new HashSet<TreeNode<T>>();
    }
    public void addChildren ( final TreeNode<T> children ) {
        this.childrens.add ( children );
    }
    public TreeNode<T> getChildren ( final TreeNode<T> children ) {
        final Iterator<TreeNode<T>> it = this.childrens.iterator();
        TreeNode<T> next = null;
        while ( it.hasNext() ) {
            next = it.next();
            if ( next.equals ( children ) ) {
                return next;
            }
        }
        return null;
    }
    public T getElement() {
        return this.element;
    }
    public Set<TreeNode<T>> getChildrens() {
        return this.childrens;
    }
    @Override
    public boolean equals ( final Object obj ) {
        if ( this == obj ) {
            return true;
        }
        if ( obj != null && obj instanceof TreeNode ) {
            final TreeNode<?> treeNode = ( TreeNode<?> ) obj;
            return this.element.equals ( treeNode.element );
        } else {
            return false;
        }
    }
    @Override
    public int hashCode() {
        int hash = 1;
        hash = hash * 17 + this.element.hashCode();
        return hash;
    }
}
