private static void createViewer ( Composite parent ) {
    viewer = new TableViewer ( parent, SWT.MULTI | SWT.H_SCROLL
                               | SWT.V_SCROLL | SWT.FULL_SELECTION | SWT.BORDER );
    createColumns ( parent, viewer );
    final Table table = viewer.getTable();
    table.setHeaderVisible ( true );
    table.setLinesVisible ( true );
    viewer.setContentProvider ( new ArrayContentProvider() );
    viewer.setInput ( ModelProvider.INSTANCE.getPersons() );
    GridData gridData = new GridData();
    gridData.verticalAlignment = GridData.FILL;
    gridData.horizontalSpan = 2;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    gridData.horizontalAlignment = GridData.FILL;
    viewer.getControl().setLayoutData ( gridData );
    comparator = new PersonViewerComparator();
    viewer.setComparator ( comparator );
}
private static void createColumns ( final Composite parent,
                                    final TableViewer viewer ) {
    String[] titles = { "First name", "Last name", "Gender", "Married" };
    int[] bounds = { 100, 100, 100, 100 };
    TableViewerColumn col = createTableViewerColumn ( titles[0], bounds[0], 0 );
    col.setLabelProvider ( new ColumnLabelProvider() {
        @Override
        public String getText ( Object element ) {
            Person p = ( Person ) element;
            return p.getFirstName();
        }
    } );
    col.setEditingSupport ( new CheckLengthEditingSupport ( viewer ) );
    col = createTableViewerColumn ( titles[1], bounds[1], 1 );
    col.setLabelProvider ( new ColumnLabelProvider() {
        @Override
        public String getText ( Object element ) {
            Person p = ( Person ) element;
            return p.getLastName();
        }
    } );
    col = createTableViewerColumn ( titles[2], bounds[2], 2 );
    col.setLabelProvider ( new ColumnLabelProvider() {
        @Override
        public String getText ( Object element ) {
            Person p = ( Person ) element;
            return p.getGender();
        }
    } );
    col = createTableViewerColumn ( titles[3], bounds[3], 3 );
    col.setLabelProvider ( new ColumnLabelProvider() {
        @Override
        public String getText ( Object element ) {
            return "";
        }
        @Override
        public Image getImage ( Object element ) {
            if ( ( ( Person ) element ).isMarried() ) {
                return CHECKED;
            } else {
                return UNCHECKED;
            }
        }
    } );
}
static class CheckLengthEditingSupport extends EditingSupport {
    private final TableViewer viewer;
    private TextCellEditor editor = null;
    public FirstNameEditingSupport ( TableViewer viewer ) {
        super ( viewer );
        this.viewer = viewer;
        editor = new TextCellEditor ( viewer.getTable() );
        Text text = ( Text ) editor.getControl();
        text.setTextLimit ( 3 );
    }
    @Override
    protected CellEditor getCellEditor ( Object element ) {
        return editor;
    }
    @Override
    protected boolean canEdit ( Object element ) {
        return true;
    }
    @Override
    protected Object getValue ( Object element ) {
        return ( ( YourObject ) element ).getName();
    }
    @Override
    protected void setValue ( Object element, Object value ) {
        ( ( YourObject ) element ).setName ( String.valueOf ( value ) );
        viewer.update ( element, null );
    }
}
