public class SpriteSheetAnimation extends GameObjectTemplate {
    int animationSheetLength;
    int animationSheetHeight;
    int columns;
    int rows;
    Rect srcRect;
    int left;
    int bottom;
    int lengthOfSpriteInSheet;
    int heightOfSpriteInSheet;
    boolean repeatAnimation;
    boolean animationCompleted;
    boolean animationStarted;
    Bitmap bitmap;
    public SpriteSheetAnimation ( WorldTemplate world, Bitmap animationSheet, int amountOfRows, int amountOfColumns ) {
        super ( world );
        this.animationSheetLength = animationSheet.getWidth();
        this.animationSheetHeight = animationSheet.getHeight();
        this.columns = amountOfColumns;
        this.rows = amountOfRows;
        this.lengthOfSpriteInSheet = this.animationSheetLength / this.columns;
        this.heightOfSpriteInSheet = this.animationSheetHeight / this.rows;
        srcRect = new Rect();
        left = 0;
        bottom = this.heightOfSpriteInSheet;
        repeatAnimation = false;
        animationCompleted = false;
        animationStarted = false;
        this.bitmap = animationSheet;
        this.drawShapesOfSprites = true;
        this.gameObjectShape.addRectangle ( new Rect ( 0, 0, 0 + this.heightOfSpriteInSheet, 0 + this.lengthOfSpriteInSheet ) );
    }
    @Override
    public void defineGameObjectPositionShape() {
        this.gameObjectShape.addRectangle ( new Rect() );
    }
    public Rect getSrcRect() {
        return srcRect;
    }
    public Rect getDstRect() {
        return this.getGameObjectPositionRect();
    }
    public void repeatAnimation ( boolean repetition ) {
        this.repeatAnimation = repetition;
    }
    public boolean isAnimationFinished() {
        return animationCompleted;
    }
    public boolean hasBeenStarted() {
        return animationStarted;
    }
    public void render() {
        world.game.getGraphics().getCanvasGameScreen().drawBitmap ( this.bitmap, this.getSrcRect(), this.getGameObjectPositionRect(), null );
    }
    @Override
    public void update() {
        if ( !animationStarted ) {
            animationStarted = true;
        }
        srcRect.set ( left, ( bottom - this.heightOfSpriteInSheet ), ( left + this.lengthOfSpriteInSheet ), bottom );
        left = left + this.lengthOfSpriteInSheet;
        if ( left == this.animationSheetLength && bottom == this.animationSheetHeight ) {
            if ( repeatAnimation == true ) {
                left = 0;
                bottom = 0;
            } else {
                animationCompleted = true;
            }
        }
        if ( left == this.animationSheetLength ) {
            bottom = bottom + this.heightOfSpriteInSheet;
            left = 0;
        }
    }
}
