abstract class ResizePanel extends Panel implements HasResizeHandlers {
    private static Element glassElem = null;
    private final Element[] elements = new Element[1];
    private final EventBus resizeHandlerManager = new SimpleEventBus();
    private boolean isResizing = false;
    private final Element splitElem;
    private final Widget[] widgets = new Widget[2];
    ResizePanel ( final Element mainElem, final Element splitElem, final Element headElem ) {
        this.setElement ( mainElem );
        this.splitElem = splitElem;
        this.elements[0] = headElem;
        this.sinkEvents ( Event.MOUSEEVENTS | Event.ONLOSECAPTURE | Event.TOUCHEVENTS | Event.ONTOUCHCANCEL );
        if ( ResizePanel.glassElem == null ) {
            ResizePanel.glassElem = DOM.createDiv();
            ResizePanel.glassElem.getStyle()
            .setProperty ( "position", "absolute" );
            ResizePanel.glassElem.getStyle().setProperty ( "top", "0px" );
            ResizePanel.glassElem.getStyle().setProperty ( "left", "0px" );
            ResizePanel.glassElem.getStyle().setProperty ( "margin", "0px" );
            ResizePanel.glassElem.getStyle().setProperty ( "padding", "0px" );
            ResizePanel.glassElem.getStyle().setProperty ( "border", "0px" );
            ResizePanel.glassElem.getStyle().setProperty ( "background", "white" );
            ResizePanel.glassElem.getStyle().setProperty ( "opacity", "0.0" );
            ResizePanel.glassElem.getStyle().setProperty ( "filter",
                    "alpha(opacity=0)" );
            ResizePanel.glassElem.setId ( "glassElementId" );
        }
    }
    static final int getOffsetHeight ( final Element elem ) {
        return DOM.getElementPropertyInt ( elem, "offsetHeight" );
    }
    static final int getOffsetWidth ( final Element elem ) {
        return DOM.getElementPropertyInt ( elem, "offsetWidth" );
    }
    static final Element preventBoxStyles ( final Element elem ) {
        DOM.setIntStyleAttribute ( elem, "padding", 0 );
        DOM.setIntStyleAttribute ( elem, "margin", 0 );
        DOM.setStyleAttribute ( elem, "border", "none" );
        return elem;
    }
    static void setBottom ( final Element elem, final String size ) {
        DOM.setStyleAttribute ( elem, "bottom", size );
    }
    static final void setClassname ( final Element elem, final String className ) {
        DOM.setElementProperty ( elem, "className", className );
    }
    static final void setHeight ( final Element elem, final String height ) {
        DOM.setStyleAttribute ( elem, "height", height );
    }
    static final void setLeft ( final Element elem, final String left ) {
        DOM.setStyleAttribute ( elem, "left", left );
    }
    static final void setRight ( final Element elem, final String right ) {
        DOM.setStyleAttribute ( elem, "right", right );
    }
    static final void setTop ( final Element elem, final String top ) {
        DOM.setStyleAttribute ( elem, "top", top );
    }
    static final void setWidth ( final Element elem, final String width ) {
        DOM.setStyleAttribute ( elem, "width", width );
    }
    public Element getSplitElement() {
        return this.splitElem;
    }
    public EventBus getResizeHandlerManager() {
        return resizeHandlerManager;
    }
    public boolean isResizing() {
        return this.isResizing;
    }
    static void addAbsolutePositoning ( final Element elem ) {
        DOM.setStyleAttribute ( elem, "position", "absolute" );
    }
    static final void addClipping ( final Element elem ) {
        DOM.setStyleAttribute ( elem, "overflow", "hidden" );
    }
    static final void addScrolling ( final Element elem ) {
        DOM.setStyleAttribute ( elem, "overflow", "auto" );
    }
    static final void expandToFitParentUsingCssOffsets ( final Element elem ) {
        final String zeroSize = "0px";
        ResizePanel.addAbsolutePositoning ( elem );
        ResizePanel.setLeft ( elem, zeroSize );
        ResizePanel.setRight ( elem, zeroSize );
        ResizePanel.setTop ( elem, zeroSize );
        ResizePanel.setBottom ( elem, zeroSize );
    }
    static final void expandToFitParentUsingPercentages ( final Element elem ) {
        final String zeroSize = "0px";
        final String fullSize = "100%";
        ResizePanel.addAbsolutePositoning ( elem );
        ResizePanel.setTop ( elem, zeroSize );
        ResizePanel.setLeft ( elem, zeroSize );
        ResizePanel.setWidth ( elem, fullSize );
        ResizePanel.setHeight ( elem, fullSize );
    }
    @Override
    public void add ( final Widget widget ) {
        boolean canAddWidget = false;
        for ( int index = 0; index < this.widgets.length; index++ ) {
            if ( this.getWidget ( index ) == null ) {
                this.setWidget ( index, widget );
                canAddWidget = true;
                break;
            }
        }
        if ( !canAddWidget ) {
            throw new IllegalStateException (
                "A Resizable panel can only contain two Widgets." );
        }
    }
    public HandlerRegistration addResizeHandler ( final ResizeEventHandler handler ) {
        return this.resizeHandlerManager.addHandler ( ResizeEvent.TYPE, handler );
    }
    @Override
    public Iterator<Widget> iterator() {
        return ResizableControlIterator
               .createWidgetIterator ( this, this.widgets );
    }
    @Override
    public void onBrowserEvent ( final Event event ) {
        switch ( DOM.eventGetType ( event ) ) {
        case Event.ONMOUSEDOWN:
            final Element target = DOM.eventGetTarget ( event );
            if ( DOM.isOrHasChild ( this.splitElem, target ) ) {
                this.startResizingFrom ( DOM.eventGetClientX ( event ) - this.getAbsoluteLeft(), DOM.eventGetClientY ( event ) - this.getAbsoluteTop() );
                DOM.setCapture ( this.getElement() );
                DOM.eventPreventDefault ( event );
            }
            break;
        case Event.ONMOUSEUP:
            if ( this.isResizing() ) {
                DOM.releaseCapture ( this.getElement() );
                this.splitPositionSetDone();
                if ( this.isResizing() ) {
                    this.stopResizing();
                }
            }
            break;
        case Event.ONMOUSEMOVE:
            if ( this.isResizing() ) {
                assert DOM.getCaptureElement() != null;
                this.onSplitterResize ( DOM.eventGetClientX ( event ) - this.getAbsoluteLeft(), DOM.eventGetClientY ( event ) - this.getAbsoluteTop() );
                DOM.eventPreventDefault ( event );
            }
            break;
        case Event.ONLOSECAPTURE:
            if ( this.isResizing() ) {
                this.stopResizing();
            }
            break;
        case Event.ONTOUCHSTART:
            final Element touchableTarget = DOM.eventGetTarget ( event );
            if ( DOM.isOrHasChild ( this.splitElem, touchableTarget ) ) {
                this.startResizingFrom ( ( event.getTouches().get ( 0 ).getClientX() - this.getAbsoluteLeft() ), ( event.getTouches().get ( 0 ).getClientY() - this.getAbsoluteTop() ) );
                DOM.setCapture ( this.getElement() );
                event.preventDefault();
            }
            break;
        case Event.ONTOUCHMOVE:
            if ( this.isResizing() ) {
                assert DOM.getCaptureElement() != null;
                this.onSplitterResize ( ( event.getTouches().get ( 0 ).getClientX() - this.getAbsoluteLeft() ), ( event.getTouches().get ( 0 ).getClientY() - this.getAbsoluteTop() ) );
                event.preventDefault();
            }
            break;
        case Event.ONTOUCHEND:
            if ( this.isResizing() ) {
                DOM.releaseCapture ( this.getElement() );
                this.splitPositionSetDone();
                if ( this.isResizing() ) {
                    this.stopResizing();
                }
            }
            break;
        case Event.ONTOUCHCANCEL:
            if ( this.isResizing() ) {
                this.stopResizing();
            }
            break;
        default:
            break;
        }
        super.onBrowserEvent ( event );
    }
    @Override
    public boolean remove ( final Widget widget ) {
        if ( widget != null ) {
            for ( int index = 0; index < this.widgets.length; index++ ) {
                if ( ( this.widgets[index] == widget ) ) {
                    this.setWidget ( index, null );
                    return true;
                }
            }
        }
        return false;
    }
    public abstract void setSplitPosition ( String size );
    public abstract void splitPositionSetDone();
    abstract void onSplitterResize ( int x, int y );
    abstract void onSplitterResizeStarted ( int x, int y );
    protected Element getElement ( final int index ) {
        return this.elements[index];
    }
    protected Widget getWidget ( final int index ) {
        return this.widgets[index];
    }
    @Override
    protected void onEnsureDebugId ( final String baseId ) {
        super.onEnsureDebugId ( baseId );
        UIObject.ensureDebugId ( this.splitElem, baseId, "splitter" );
    }
    protected final void setWidget ( final int index, final Widget widget ) {
        final Widget oldWidget = this.widgets[index];
        if ( oldWidget == widget ) {
            return;
        }
        if ( widget != null ) {
            widget.removeFromParent();
        }
        if ( oldWidget != null ) {
            try {
                this.orphan ( oldWidget );
            } finally {
                DOM.removeChild ( this.elements[0], oldWidget.getElement() );
                this.widgets[index] = null;
            }
        }
        this.widgets[index] = widget;
        if ( widget != null ) {
            DOM.appendChild ( this.elements[0], widget.getElement() );
            this.adopt ( widget );
        }
    }
    public void startResizingFrom ( final int x, final int y ) {
        this.isResizing = true;
        this.onSplitterResizeStarted ( x, y );
        final int height = RootPanel.getBodyElement().getScrollHeight() - 1;
        final int width = RootPanel.getBodyElement().getScrollWidth() - 1;
        ResizePanel.glassElem.getStyle().setProperty ( "height", height + "px" );
        ResizePanel.glassElem.getStyle().setProperty ( "width", width + "px" );
        RootPanel.getBodyElement().appendChild ( ResizePanel.glassElem );
    }
    void stopResizing() {
        this.isResizing = false;
        try {
            RootPanel.getBodyElement().removeChild ( ResizePanel.glassElem );
        } catch ( final Exception e ) {
            GWT.log ( "Got error removing the glassElem: " + e );
        }
    }
}
