public static class ButtonDialogEditor extends AbstractCellEditor
    implements TableCellEditor {
    public final int lastColumn;
    public final int firstColumn;
    private int row;
    private DefaultTableModel model;
    private JDialog dialog;
    private boolean committed;
    private JButton editingComponent;
    public ButtonDialogEditor ( int firstColumn, int lastColumn ) {
        this.firstColumn = firstColumn;
        this.lastColumn = lastColumn;
        model = new DefaultTableModel ( 1, lastColumn - firstColumn + 1 );
        JTable table = new JTable ( model );
        table.putClientProperty ( "terminateEditOnFocusLost", true );
        dialog = new JDialog();
        dialog.setModal ( true );
        dialog.add ( new JScrollPane ( table ) );
        dialog.add ( new JButton ( createOkAction() ), BorderLayout.SOUTH );
        dialog.pack();
        editingComponent = new JButton ( createShowDialogAction() );
    }
    public Object getValueAt ( int column ) {
        return model.getValueAt ( 0, column - firstColumn );
    }
    public int getModelRow() {
        return row;
    }
    private Action createShowDialogAction() {
        Action action = new AbstractAction() {
            @Override
            public void actionPerformed ( ActionEvent e ) {
                committed = false;
                dialog.setVisible ( true );
                if ( committed ) {
                    fireEditingStopped();
                } else {
                    fireEditingCanceled();
                }
            }
        };
        return action;
    }
    private Action createOkAction() {
        Action action = new AbstractAction ( "OK" ) {
            @Override
            public void actionPerformed ( ActionEvent e ) {
                dialog.setVisible ( false );
                committed = true;
            }
        };
        return action;
    }
    @Override
    public Component getTableCellEditorComponent ( JTable table,
            Object value, boolean isSelected, int row, int column ) {
        editingComponent.setText ( value != null ? value.toString() : "" );
        prepareDialog ( table, row );
        return editingComponent;
    }
    private void prepareDialog ( JTable table, int row ) {
        this.row = table.convertRowIndexToModel ( row );
        for ( int i = firstColumn; i <= lastColumn; i++ ) {
            model.setValueAt ( table.getModel().getValueAt ( this.row, i ), 0, i - firstColumn );
        }
    }
    @Override
    public Object getCellEditorValue() {
        return editingComponent.getText();
    }
}
public static class ButtonRenderer implements TableCellRenderer {
    JButton button = new JButton();
    @Override
    public Component getTableCellRendererComponent ( JTable table,
            Object value, boolean isSelected, boolean hasFocus, int row,
            int column ) {
        button.setText ( value != null ? value.toString() : "" );
        return button;
    }
}
final DefaultTableModel model = new DefaultTableModel ( 0, 3 ) {
    @Override
    public boolean isCellEditable ( int row, int column ) {
        return column == 0;
    }
};
for ( int i = 0; i < 20; i++ ) {
    model.addRow ( new Object[] {"Edit", i} );
}
JTable table = new JTable ( model );
table.getColumnModel().getColumn ( 0 ).setCellRenderer ( new ButtonRenderer() );
final ButtonDialogEditor cellEditor = new ButtonDialogEditor ( 1, model.getColumnCount() - 1 );
CellEditorListener l = new CellEditorListener() {
    @Override
    public void editingStopped ( ChangeEvent e ) {
        for ( int i = cellEditor.firstColumn; i <= cellEditor.lastColumn; i++ ) {
            model.setValueAt ( cellEditor.getValueAt ( i ), cellEditor.getModelRow(), i );
        }
    }
    @Override
    public void editingCanceled ( ChangeEvent e ) {
    }
};
cellEditor.addCellEditorListener ( l );
table.getColumnModel().getColumn ( 0 ).setCellEditor (
    cellEditor );
