public class PropertyHolder {
    private final ImmutableMap<PropertyKey<?>, ?> storage;
    @SuppressWarnings ( "unchecked" )
    public <T extends Serializable> T get ( PropertyKey<T> key ) {
        return ( T ) storage.get ( key );
    }
    public <T> PropertyHolder put ( PropertyKey<T> key, T value ) {
        Preconditions.checkNotNull ( key, "PropertyKey cannot be null" );
        Preconditions.checkNotNull ( value, "Value for key %s is null",
                                     key );
        Preconditions.checkArgument ( key.checkType ( value ),
                                      "Property \"%s\" was given "
                                      + "value of a wrong type \"%s\"", key, value );
        return new PropertyHolder ( filterOutKey ( key )
                                    .put ( key, value ).build() );
    }
    private <T> Builder<PropertyKey<? extends Serializable>, Serializable> filterOutKey ( PropertyKey<T> key ) {
        Builder<PropertyKey<? extends Serializable>, Serializable> builder = ImmutableMap
                .<PropertyKey<? extends Serializable>, Serializable> builder();
        for ( Entry<PropertyKey<? extends Serializable>, Serializable> entry : this.storage.entrySet() ) {
            if ( !entry.getKey().equals ( key ) ) {
                builder.put ( entry );
            }
        }
        return builder;
    }
    ... rest of the class
    }
