package ndd;
public class RadiusBox {
    static final Double kEarthRadiusMiles = 3956.0;
    public static RadiusBox create ( Double latitude, Double longitude, Double radiusInMiles ) {
        double lat;
        double dlon;
        double dLatInRads = latitude * ( Math.PI / 180.0 );
        double dLongInRads = longitude * ( Math.PI / 180.0 );
        double dDistInRad = radiusInMiles / kEarthRadiusMiles;
        RadiusBox box = new RadiusBox ();
        box.setRadius ( radiusInMiles );
        box.setTopLine ( ( dLatInRads + dDistInRad ) * ( 180.0 / Math.PI ) );
        box.setBottomLine ( ( dLatInRads - dDistInRad ) * ( 180.0 / Math.PI ) );
        lat = Math.asin ( Math.sin ( dLatInRads ) * Math.cos ( dDistInRad ) );
        dlon = Math.atan2 ( Math.sin ( Math.PI / 2.0 ) * Math.sin ( dDistInRad ) * Math.cos ( dLatInRads ), Math.cos ( dDistInRad ) - Math.sin ( dLatInRads ) * Math.sin ( lat ) );
        box.setRightLine ( ( ( ( dLongInRads + dlon + Math.PI ) % ( 2.0 * Math.PI ) ) - Math.PI ) * ( 180.0 / Math.PI ) );
        dlon = Math.atan2 ( Math.sin ( 3.0 * Math.PI / 2.0 ) * Math.sin ( dDistInRad ) * Math.cos ( dLatInRads ), Math.cos ( dDistInRad ) - Math.sin ( dLatInRads ) * Math.sin ( lat ) );
        box.setLeftLine ( ( ( ( dLongInRads + dlon + Math.PI ) % ( 2.0 * Math.PI ) ) - Math.PI ) * ( 180.0 / Math.PI ) );
        return box;
    }
    private double bottomLine;
    private double topLine;
    private double leftLine;
    private double rightLine;
    private double radius;
    public double getBottomLine() {
        return bottomLine;
    }
    public void setBottomLine ( double bottomLine ) {
        this.bottomLine = bottomLine;
    }
    public double getTopLine() {
        return topLine;
    }
    public void setTopLine ( double topLine ) {
        this.topLine = topLine;
    }
    public double getLeftLine() {
        return leftLine;
    }
    public void setLeftLine ( double leftLine ) {
        this.leftLine = leftLine;
    }
    public double getRightLine() {
        return rightLine;
    }
    public void setRightLine ( double rightLine ) {
        this.rightLine = rightLine;
    }
    public double getRadius() {
        return radius;
    }
    public void setRadius ( double radius ) {
        this.radius = radius;
    }
    @Override
    public String toString() {
        return "RadiusBox [bottomLine=" + bottomLine + ", leftLine=" + leftLine
               + ", radius=" + radius + ", rightLine=" + rightLine
               + ", topLine=" + topLine + "]";
    }
    public String getSqlQuery() {
        StringBuilder sb = new StringBuilder();
        sb.append ( "SELECT * FROM ZIP_CODES WHERE " );
        sb.append ( "LATITUDE >= " );
        sb.append ( bottomLine );
        sb.append ( " AND LATITUDE <= " );
        sb.append ( topLine );
        sb.append ( " AND LONGITUDE >= " );
        sb.append ( leftLine );
        sb.append ( " AND LONGITUDE <= " );
        sb.append ( rightLine );
        sb.append ( " ORDER BY CITY, STATE, ZIP" );
        return sb.toString();
    }
}
