import java.io.*;
import java.util.*;
import javafx.application.Platform;
import javafx.beans.value.*;
import javafx.embed.swing.JFXPanel;
import javafx.event.*;
import javafx.geometry.Pos;
import javafx.scene.Scene;
import javafx.scene.control.*;
import javafx.scene.layout.*;
import javafx.scene.media.*;
import javafx.util.Duration;
import javax.swing.*;
public class JavaFXVideoPlayerLaunchedFromSwing {
    private static void initAndShowGUI() {
        JFrame frame = new JFrame ( "FX" );
        final JFXPanel fxPanel = new JFXPanel();
        frame.add ( fxPanel );
        frame.setBounds ( 200, 100, 800, 250 );
        frame.setDefaultCloseOperation ( WindowConstants.EXIT_ON_CLOSE );
        frame.setVisible ( true );
        Platform.runLater ( new Runnable() {
            @Override public void run() {
                initFX ( fxPanel );
            }
        } );
    }
    private static void initFX ( JFXPanel fxPanel ) {
        Scene scene = new SceneGenerator().createScene();
        fxPanel.setScene ( scene );
    }
    public static void main ( String[] args ) {
        SwingUtilities.invokeLater ( new Runnable() {
            @Override public void run() {
                initAndShowGUI();
            }
        } );
    }
}
class SceneGenerator {
    final Label currentlyPlaying = new Label();
    final ProgressBar progress = new ProgressBar();
    private ChangeListener<Duration> progressChangeListener;
    public Scene createScene() {
        final StackPane layout = new StackPane();
        final File dir = new File ( "C:\\Users\\Public\\Music\\Sample Music" );
        if ( !dir.exists() || !dir.isDirectory() ) {
            System.out.println ( "Cannot find video source directory: " + dir );
            Platform.exit();
            return null;
        }
        final List<MediaPlayer> players = new ArrayList<MediaPlayer>();
        for ( String file : dir.list ( new FilenameFilter() {
        @Override public boolean accept ( File dir, String name ) {
                return name.endsWith ( ".mp3" );
            }
        } ) ) players.add ( createPlayer ( "file:///" + ( dir + "\\" + file ).replace ( "\\", "/" ).replaceAll ( " ", "%20" ) ) );
        if ( players.isEmpty() ) {
            System.out.println ( "No audio found in " + dir );
            Platform.exit();
            return null;
        }
        final MediaView mediaView = new MediaView ( players.get ( 0 ) );
        final Button skip = new Button ( "Skip" );
        final Button play = new Button ( "Pause" );
        for ( int i = 0; i < players.size(); i++ ) {
            final MediaPlayer player     = players.get ( i );
            final MediaPlayer nextPlayer = players.get ( ( i + 1 ) % players.size() );
            player.setOnEndOfMedia ( new Runnable() {
                @Override public void run() {
                    player.currentTimeProperty().removeListener ( progressChangeListener );
                    mediaView.setMediaPlayer ( nextPlayer );
                    nextPlayer.play();
                }
            } );
        }
        skip.setOnAction ( new EventHandler<ActionEvent>() {
            @Override public void handle ( ActionEvent actionEvent ) {
                final MediaPlayer curPlayer = mediaView.getMediaPlayer();
                MediaPlayer nextPlayer = players.get ( ( players.indexOf ( curPlayer ) + 1 ) % players.size() );
                mediaView.setMediaPlayer ( nextPlayer );
                curPlayer.currentTimeProperty().removeListener ( progressChangeListener );
                curPlayer.stop();
                nextPlayer.play();
            }
        } );
        play.setOnAction ( new EventHandler<ActionEvent>() {
            @Override public void handle ( ActionEvent actionEvent ) {
                if ( "Pause".equals ( play.getText() ) ) {
                    mediaView.getMediaPlayer().pause();
                    play.setText ( "Play" );
                } else {
                    mediaView.getMediaPlayer().play();
                    play.setText ( "Pause" );
                }
            }
        } );
        mediaView.mediaPlayerProperty().addListener ( new ChangeListener<MediaPlayer>() {
            @Override public void changed ( ObservableValue<? extends MediaPlayer> observableValue, MediaPlayer oldPlayer, MediaPlayer newPlayer ) {
                setCurrentlyPlaying ( newPlayer );
            }
        } );
        mediaView.setMediaPlayer ( players.get ( 0 ) );
        mediaView.getMediaPlayer().play();
        setCurrentlyPlaying ( mediaView.getMediaPlayer() );
        Button invisiblePause = new Button ( "Pause" );
        invisiblePause.setVisible ( false );
        play.prefHeightProperty().bind ( invisiblePause.heightProperty() );
        play.prefWidthProperty().bind ( invisiblePause.widthProperty() );
        layout.setStyle ( "-fx-background-color: cornsilk; -fx-font-size: 20; -fx-padding: 20; -fx-alignment: center;" );
        layout.getChildren().addAll (
            invisiblePause,
            VBoxBuilder.create().spacing ( 10 ).alignment ( Pos.CENTER ).children (
                currentlyPlaying,
                mediaView,
                HBoxBuilder.create().spacing ( 10 ).alignment ( Pos.CENTER ).children ( skip, play, progress ).build()
            ).build()
        );
        progress.setMaxWidth ( Double.MAX_VALUE );
        HBox.setHgrow ( progress, Priority.ALWAYS );
        return new Scene ( layout, 800, 600 );
    }
    private void setCurrentlyPlaying ( final MediaPlayer newPlayer ) {
        progress.setProgress ( 0 );
        progressChangeListener = new ChangeListener<Duration>() {
            @Override public void changed ( ObservableValue<? extends Duration> observableValue, Duration oldValue, Duration newValue ) {
                progress.setProgress ( 1.0 * newPlayer.getCurrentTime().toMillis() / newPlayer.getTotalDuration().toMillis() );
            }
        };
        newPlayer.currentTimeProperty().addListener ( progressChangeListener );
        String source = newPlayer.getMedia().getSource();
        source = source.substring ( 0, source.length() - ".mp4".length() );
        source = source.substring ( source.lastIndexOf ( "/" ) + 1 ).replaceAll ( "%20", " " );
        currentlyPlaying.setText ( "Now Playing: " + source );
    }
    private MediaPlayer createPlayer ( String aMediaSrc ) {
        System.out.println ( "Creating player for: " + aMediaSrc );
        final MediaPlayer player = new MediaPlayer ( new Media ( aMediaSrc ) );
        player.setOnError ( new Runnable() {
            @Override public void run() {
                System.out.println ( "Media error occurred: " + player.getError() );
            }
        } );
        return player;
    }
}
