public interface Command {
    public void execute ( String[] arguments );
}
public interface ValuedCommand<T> extends Command {
    public T evaluate ( String[] arguments );
}
public abstract class AbstractCommand<T> implements ValuedCommand<T> {
    public void execute ( String[] arguments ) {
        evaluate ( arguments );
    }
}
public class Commands {
    private Commands() {}
    public static void execute ( Command cmd, String... arguments ) {
        cmd.execute ( arguments );
    }
    public static <T> void execute ( ValuedCommand<T> cmd, String... arguments ) {
        return cmd.evaluate ( arguments );
    }
    public static ValuedCommand<?> asValuedCommand ( Command cmd ) {
        return new VoidCommand ( cmd );
    }
    private static class VoidCommand extends AbstractCommand<Void> {
        private final Command cmd;
        public VoidCommand ( Command actual ) {
            cmd = actual;
        }
        public Void evaluate ( String[] arguments ) {
            cmd.execute ( arguments );
            return null;
        }
    }
}
