public class RestExample_AddTagsToTestCase {
    public static void main ( String[] args ) throws URISyntaxException, IOException {
        String rallyURL = "https://rally1.rallydev.com";
        String wsapiVersion = "1.34";
        String applicationName = "RestExample_AddTagsToTestCase";
        String userName = "user@company.com";
        String userPassword = "password";
        RallyRestApi restApi = new RallyRestApi (
            new URI ( rallyURL ),
            userName,
            userPassword );
        restApi.setWsapiVersion ( wsapiVersion );
        restApi.setApplicationName ( applicationName );
        String testerUserName = "tester@company.com";
        String myWorkspace = "My Workspace";
        String myProject = "My Project";
        String existTestCaseFormattedID = "TC172";
        String tagNameAlpine = "Alpine";
        String tagNameMontane = "Montane";
        QueryRequest userRequest = new QueryRequest ( "User" );
        userRequest.setFetch ( new Fetch ( "UserName", "Subscription", "DisplayName" ) );
        userRequest.setQueryFilter ( new QueryFilter ( "UserName", "=", testerUserName ) );
        QueryResponse userQueryResponse = restApi.query ( userRequest );
        JsonArray userQueryResults = userQueryResponse.getResults();
        JsonElement userQueryElement = userQueryResults.get ( 0 );
        JsonObject userQueryObject = userQueryElement.getAsJsonObject();
        String userRef = userQueryObject.get ( "_ref" ).toString();
        QueryRequest workspaceRequest = new QueryRequest ( "Workspace" );
        workspaceRequest.setFetch ( new Fetch ( "Name", "Owner", "Projects" ) );
        workspaceRequest.setQueryFilter ( new QueryFilter ( "Name", "=", myWorkspace ) );
        QueryResponse workspaceQueryResponse = restApi.query ( workspaceRequest );
        String workspaceRef = workspaceQueryResponse.getResults().get ( 0 ).getAsJsonObject().get ( "_ref" ).toString();
        QueryRequest projectRequest = new QueryRequest ( "Project" );
        projectRequest.setFetch ( new Fetch ( "Name", "Owner", "Projects" ) );
        projectRequest.setQueryFilter ( new QueryFilter ( "Name", "=", myProject ) );
        QueryResponse projectQueryResponse = restApi.query ( projectRequest );
        String projectRef = projectQueryResponse.getResults().get ( 0 ).getAsJsonObject().get ( "_ref" ).toString();
        QueryRequest tagRequest = new QueryRequest ( "Tag" );
        tagRequest.setFetch ( new Fetch ( "Name" ) );
        tagRequest.setQueryFilter ( new QueryFilter ( "Name", "=", tagNameAlpine ) );
        QueryResponse tagQueryResponse = restApi.query ( tagRequest );
        JsonObject tagAlpineJsonObject = tagQueryResponse.getResults().get ( 0 ).getAsJsonObject();
        JsonElement tagAlpine = tagQueryResponse.getResults().get ( 0 ).getAsJsonObject();
        String tagAlpineRef = tagAlpineJsonObject.get ( "_ref" ).toString();
        tagRequest.setQueryFilter ( new QueryFilter ( "Name", "=", tagNameMontane ) );
        tagQueryResponse = restApi.query ( tagRequest );
        JsonObject tagMontaneJsonObject = tagQueryResponse.getResults().get ( 0 ).getAsJsonObject();
        JsonElement tagMontane = tagQueryResponse.getResults().get ( 0 ).getAsJsonObject();
        String tagMontaneRef = tagMontaneJsonObject.get ( "_ref" ).toString();
        QueryRequest  existTestCaseRequest = new QueryRequest ( "TestCase" );
        existTestCaseRequest.setFetch ( new Fetch ( "FormattedID", "Name", "Tags" ) );
        existTestCaseRequest.setQueryFilter ( new QueryFilter ( "FormattedID", "=", existTestCaseFormattedID ) );
        QueryResponse testCaseQueryResponse = restApi.query ( existTestCaseRequest );
        JsonObject existTestCaseJsonObject = testCaseQueryResponse.getResults().get ( 0 ).getAsJsonObject();
        String existTestCaseRef = testCaseQueryResponse.getResults().get ( 0 ).getAsJsonObject().get ( "_ref" ).toString();
        JsonArray existTagsJsonArray = ( JsonArray ) testCaseQueryResponse.getResults().get ( 0 ).getAsJsonObject().get ( "Tags" );
        try {
            existTagsJsonArray.add ( tagMontane );
            System.out.println ( "\nUpdating Existing Test Case's tags..." );
            JsonObject updateExistTestCase = new JsonObject();
            updateExistTestCase.add ( "Tags", existTagsJsonArray );
            UpdateRequest updateExistTestCaseRequest = new UpdateRequest ( existTestCaseRef, updateExistTestCase );
            UpdateResponse updateExistTestCaseResponse = restApi.update ( updateExistTestCaseRequest );
            if ( updateExistTestCaseResponse.wasSuccessful() ) {
                System.out.println ( "Updated Test Case with new Tags" );
            } else {
                String[] updateExistTestCaseErrors;
                updateExistTestCaseErrors = updateExistTestCaseResponse.getErrors();
                System.out.println ( "Error occurred updating Test Case: " );
                for ( int i = 0; i < updateExistTestCaseErrors.length; i++ ) {
                    System.out.println ( updateExistTestCaseErrors[i] );
                }
            }
        } catch ( Exception e ) {
            System.out.println ( "Exception occurred while updating Tags on existing Test Case: " );
            e.printStackTrace();
        }
        try {
            System.out.println ( "Creating Test Case..." );
            JsonObject newTestCase = new JsonObject();
            newTestCase.addProperty ( "Name", "TC0400-1200: SeleniumGroup004" );
            newTestCase.addProperty ( "Method", "Automated" );
            newTestCase.addProperty ( "Type", "Performance" );
            newTestCase.addProperty ( "Workspace", workspaceRef );
            newTestCase.addProperty ( "Project", projectRef );
            newTestCase.addProperty ( "Owner", userRef );
            CreateRequest createRequest = new CreateRequest ( "testcase", newTestCase );
            CreateResponse createResponse = restApi.create ( createRequest );
            if ( createResponse.wasSuccessful() ) {
                System.out.println ( String.format ( "Created %s", createResponse.getObject().get ( "_ref" ).getAsString() ) );
                String newTestCaseRef = Ref.getRelativeRef ( createResponse.getObject().get ( "_ref" ).getAsString() );
                System.out.println ( String.format ( "\nReading Test Case %s...", newTestCaseRef ) );
                GetRequest getRequest = new GetRequest ( newTestCaseRef );
                getRequest.setFetch ( new Fetch ( "FormattedID", "Name", "Tags" ) );
                GetResponse getResponse = restApi.get ( getRequest );
                JsonObject obj = getResponse.getObject();
                System.out.println ( String.format ( "Read Test Case. FormattedID = %s, Name = %s",
                                                     obj.get ( "FormattedID" ).getAsString(), obj.get ( "Name" ).getAsString() ) );
                JsonArray newTagsJsonArray = new JsonArray();
                newTagsJsonArray.add ( tagAlpine );
                newTagsJsonArray.add ( tagMontane );
                System.out.println ( "\nUpdating Test Case tags..." );
                JsonObject updatedTestCase = new JsonObject();
                updatedTestCase.add ( "Tags", newTagsJsonArray );
                UpdateRequest updateRequest = new UpdateRequest ( newTestCaseRef, updatedTestCase );
                UpdateResponse updateResponse = restApi.update ( updateRequest );
                if ( updateResponse.wasSuccessful() ) {
                    System.out.println ( "Updated Test Case with new Tags" );
                } else {
                    String[] updateErrors;
                    updateErrors = updateResponse.getErrors();
                    System.out.println ( "Error occurred updating Test Case: " );
                    for ( int i = 0; i < updateErrors.length; i++ ) {
                        System.out.println ( updateErrors[i] );
                    }
                }
            } else {
                String[] createErrors;
                createErrors = createResponse.getErrors();
                System.out.println ( "Error occurred creating Test Case: " );
                for ( int i = 0; i < createErrors.length; i++ ) {
                    System.out.println ( createErrors[i] );
                }
            }
        } finally {
            restApi.close();
        }
    }
}
