public class EightQueensSolver {
    final int[] queensPositions;
    public EightQueensSolver ( int boardSize ) {
        if ( boardSize < 4 ) {
            throw new RuntimeException ( "Can not solve boards of size < 4" );
        }
        queensPositions = new int[boardSize];
        Arrays.fill ( queensPositions, -1 );
    }
    boolean solve() {
        return solve ( 0 );
    }
    boolean solve ( int row ) {
        for ( int col = 0; col < queensPositions.length; col++ ) {
            if ( isSafe ( row, col ) ) {
                queensPositions[row] = col;
                if ( row == queensPositions.length - 1 ) {
                    return true;
                }
                if ( solve ( row + 1 ) ) {
                    return true;
                } else {
                    queensPositions[row] = -1;
                }
            }
        }
        return false;
    }
    boolean isSafe ( int row, int col ) {
        for ( int r = 0; r < row; r++ ) {
            int c = queensPositions[r];
            if ( c == col || Math.abs ( c - col ) == Math.abs ( r - row ) ) {
                return false;
            }
        }
        return true;
    }
    void printBoard() {
        for ( int row = 0; row < queensPositions.length; row++ ) {
            for ( int col = 0; col < queensPositions.length; col++ ) {
                if ( queensPositions[row] == col ) {
                    System.out.print ( "Q," );
                } else {
                    System.out.print ( " ," );
                }
            }
            System.out.println();
        }
    }
    int getQueenColumnPosition ( int row ) {
        return queensPositions[row];
    }
}
