public class PaintingExample {
    public PaintingExample() {
        JFrame frame = new JFrame ( "Painting on Image" );
        frame.setDefaultCloseOperation ( JFrame.EXIT_ON_CLOSE );
        frame.setLayout ( new BorderLayout() );
        frame.add ( new PaintSurface() );
        frame.pack();
        frame.setLocationByPlatform ( true );
        frame.setVisible ( true );
    }
    public static void main ( String... args ) {
        SwingUtilities.invokeLater ( new Runnable() {
            public void run() {
                new PaintingExample();
            }
        } );
    }
    protected class PaintSurface extends JPanel {
        private JTextArea renderer;
        private BufferedImage background;
        public PaintSurface() {
            try {
                background = ImageIO.read ( new File ( "path/to/your/background/image" ) );
            } catch ( IOException ex ) {
                ex.printStackTrace();
            }
            setLayout ( null );
            setFocusable ( true );
            addMouseListener ( new MouseAdapter() {
                @Override
                public void mouseClicked ( MouseEvent e ) {
                    if ( renderer.getParent() == null ) {
                        renderer.setText ( "Text" );
                        renderer.setLocation ( e.getPoint() );
                        add ( renderer );
                        renderer.requestFocusInWindow();
                        renderer.selectAll();
                        repaint();
                    } else {
                        requestFocusInWindow();
                    }
                }
            } );
            renderer = new JTextArea();
            renderer.setBorder ( new LineBorder ( Color.RED ) );
            renderer.setOpaque ( false );
            renderer.getDocument().addDocumentListener ( new DocumentListener() {
                protected void updateSize() {
                    renderer.setSize ( renderer.getPreferredSize() );
                }
                @Override
                public void insertUpdate ( DocumentEvent e ) {
                    updateSize();
                }
                @Override
                public void removeUpdate ( DocumentEvent e ) {
                    updateSize();
                }
                @Override
                public void changedUpdate ( DocumentEvent e ) {
                    updateSize();
                }
            } );
            renderer.addFocusListener ( new FocusAdapter() {
                @Override
                public void focusLost ( FocusEvent e ) {
                    remove ( renderer );
                    repaint();
                }
            } );
        }
        @Override
        public Dimension getPreferredSize() {
            Dimension size = super.getPreferredSize();
            if ( background != null ) {
                size = new Dimension ( background.getWidth(), background.getHeight() );
            }
            return size;
        }
        @Override
        protected void paintComponent ( Graphics g ) {
            super.paintComponent ( g );
            if ( background != null ) {
                g.drawImage ( background, 0, 0, this );
            }
        }
    }
}
