public class AllLocales {
    private static final Locale[] priorityLocales = {
        Locale.US,
        Locale.UK
    };
    private static class MyLocale implements Comparable<MyLocale> {
        private final Locale me;
        public MyLocale ( Locale me ) {
            this.me = me;
        }
        public String getCountry() {
            return me.getCountry();
        }
        @Override
        public int compareTo ( MyLocale it ) {
            if ( getCountry().equals ( it.getCountry() ) ) {
                return 0;
            }
            for ( int i = 0; i < priorityLocales.length; i++ ) {
                Locale priority = priorityLocales[i];
                if ( getCountry().equals ( priority.getCountry() ) ) {
                    return -1;
                }
                if ( it.getCountry().equals ( priority.getCountry() ) ) {
                    return 1;
                }
            }
            return getCountry().compareTo ( it.getCountry() );
        }
    }
    public static List<String> listAll() {
        Set<MyLocale> byLocale = new TreeSet();
        for ( Locale locale : Locale.getAvailableLocales() ) {
            final String isoCountry = locale.getDisplayCountry();
            if ( isoCountry.length() > 0 ) {
                byLocale.add ( new MyLocale ( locale ) );
            }
        }
        ArrayList<String> list = new ArrayList<>();
        for ( MyLocale l : byLocale ) {
            list.add ( l.getCountry() );
        }
        return list;
    }
    public static void main ( String[] args ) throws InterruptedException {
        List<String> locales = listAll();
        System.out.println ( locales );
    }
}
