public static class JImagePanel extends JPanel implements Scrollable {
    private BufferedImage image;
    public JImagePanel ( BufferedImage image ) {
        this.image = image;
    }
    @Override
    protected void paintComponent ( Graphics g ) {
        super.paintComponent ( g );
        BufferedImage scaled =
            GraphicsUtilities.createCompatibleImage ( getWidth(), getHeight() );
        Graphics2D g2 = scaled.createGraphics();
        g2.drawImage ( image, 0, 0, getWidth(), getHeight(), null );
        g.drawImage ( scaled, 0, 0, this );
        g2.dispose();
    }
    @Override
    public Dimension getPreferredSize() {
        Dimension size = getImageSize();
        if ( !getScrollableTracksViewportWidth() ) {
            size.width = getMinimumSize().width;
        }
        if ( !getScrollableTracksViewportHeight() ) {
            size.height = getMinimumSize().height;
        }
        return size;
    }
    @Override
    public Dimension getMinimumSize() {
        Dimension min = getImageSize();
        min.height /= 2;
        min.width /= 2;
        return min;
    }
    @Override
    public Dimension getPreferredScrollableViewportSize() {
        return getImageSize();
    }
    protected Dimension getImageSize() {
        return new Dimension ( image.getWidth(), image.getHeight() );
    }
    @Override
    public boolean getScrollableTracksViewportWidth() {
        return getParent() instanceof JViewport
               && getParent().getWidth() >= getMinimumSize().width;
    }
    @Override
    public boolean getScrollableTracksViewportWidth() {
        return getParent() instanceof JViewport
               && getParent().getWidth() >= getMinimumSize().width;
    }
    @Override
    public boolean getScrollableTracksViewportHeight() {
        return getParent() instanceof JViewport
               && getParent().getHeight() >= getMinimumSize().height;
    }
    @Override
    public int getScrollableUnitIncrement ( Rectangle visibleRect,
                                            int orientation, int direction ) {
        return 10;
    }
    @Override
    public int getScrollableBlockIncrement ( Rectangle visibleRect,
            int orientation, int direction ) {
        return 100;
    }
}
