package mydatastore.collections.concurrent;
import java.util.Iterator;
import java.util.ListIterator;
public class SecureArrayList_Test {
    private static SecureArrayList<String> statList = new SecureArrayList<>();
    public static void main ( String[] args ) {
        accessExamples();
    }
    private static void accessExamples() {
        final SecureArrayList<String> list = getList();
        try {
            list.lockWrite();
            list.add ( "banana" );
            list.add ( "test" );
        } finally {
            list.unlockWrite();
        }
        String val = list.get ( 0 );
        int lastIndex = -1;
        try {
            list.lockRead();
            String search = "test";
            if ( list.contains ( search ) ) {
                lastIndex = list.lastIndexOf ( search );
            }
        } finally {
            list.unlockRead();
        }
        try {
            list.lockWrite();
            int index = list.indexOf ( "test" );
            if ( index != -1 ) {
                String newVal = "banana";
                list.add ( index + 1, newVal );
            }
        } finally {
            list.unlockWrite();
        }
        System.out.println ( "First output: " );
        try {
            list.lockRead();
            for ( Iterator<String> it = list.iterator(); it.hasNext(); ) {
                String string = it.next();
                System.out.println ( string );
            }
        } finally {
            list.unlockRead();
        }
        System.out.println ( "------" );
        try {
            list.lockWrite();
            boolean firstAdd = true;
            for ( ListIterator<String> it = list.listIterator(); it.hasNext(); ) {
                int index = it.nextIndex();
                String string = it.next();
                switch ( string ) {
                case "banana":
                    it.remove();
                    break;
                case "test":
                    if ( firstAdd ) {
                        it.add ( "whatever" );
                        firstAdd = false;
                    }
                    break;
                }
                if ( index == 2 ) {
                    list.set ( index - 1, "pretty senseless data and operations but just to show "
                               + "what's possible" );
                }
            }
        } finally {
            list.unlockWrite();
        }
        System.out.println ( "Complete last output: " );
        try {
            list.lockRead();
            for ( String string : list ) {
                System.out.println ( string );
            }
        } finally {
            list.unlockRead();
        }
        System.out.println ( "------" );
        String lastElement = null;
        try {
            list.lockRead();
            int size = list.size();
            lastElement = list.get ( size - 1 );
        } finally {
            list.unlockRead();
        }
        System.out.println ( "Last element: " + lastElement );
    }
    private static void mechanismTest_1() {
        SecureArrayList<String> list = getList();
        try {
            System.out.print ( "fus, " );
            list.lockRead();
            System.out.print ( "roh, " );
            list.lockWrite();
            System.out.println ( "dah!" );
        } finally {
            System.out.println ( "dah?" );
            list.unlockRead();
            list.unlockWrite();
        }
    }
    private static void mechanismTest_2() {
        SecureArrayList<String> list = getList();
        try {
            System.out.print ( "fus, " );
            list.lockWrite();
            System.out.print ( "roh, " );
            list.lockRead();
            System.out.println ( "dah!" );
        } finally {
            list.unlockRead();
            list.unlockWrite();
        }
    }
    private static SecureArrayList<String> getList() {
        return statList;
    }
}
