public class InterruptingExecutorService extends ThreadPoolExecutor {
    private volatile FutureTask<?> currentFuture;
    public InterruptingExecutorService ( boolean daemon ) {
        super ( 0, 1, 1000L, TimeUnit.MILLISECONDS,
                new LinkedBlockingQueue<Runnable>(),
                daemon ? new DaemonThreadFactory() : Executors.defaultThreadFactory() );
    }
    public static class DaemonThreadFactory implements ThreadFactory {
        ThreadFactory delegate = Executors.defaultThreadFactory();
        @Override
        public Thread newThread ( Runnable r ) {
            Thread t = delegate.newThread ( r );
            t.setDaemon ( true );
            return t;
        }
    }
    private void cancelCurrentFuture() {
        Iterator<Runnable> it = getQueue().iterator();
        while ( it.hasNext() ) {
            FutureTask<?> task = ( FutureTask<?> ) it.next();
            task.cancel ( true );
            it.remove();
        }
        FutureTask<?> currentFuture = this.currentFuture;
        if ( currentFuture != null ) {
            currentFuture.cancel ( true );
        }
    }
    @Override
    public void execute ( Runnable command ) {
        if ( command == null ) {
            throw new NullPointerException();
        }
        cancelCurrentFuture();
        if ( ! ( command instanceof FutureTask ) ) {
            command = newTaskFor ( command, null );
        }
        super.execute ( command );
    }
    @Override
    protected void beforeExecute ( Thread t, Runnable r ) {
        this.currentFuture = ( FutureTask<?> ) r;
    }
    @Override
    protected void afterExecute ( Runnable r, Throwable t ) {
        this.currentFuture = null;
    }
}
