package demo;
import demo.model.User;
import demo.security.Authenticator;
import java.util.logging.Level;
import java.util.logging.Logger;
import javafx.application.Application;
import javafx.fxml.FXMLLoader;
import javafx.fxml.JavaFXBuilderFactory;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.stage.Stage;
public class App extends Application {
    private Stage stage;
    private User loggedUser;
    private static App instance;
    public App() {
        instance = this;
    }
    public static App getInstance() {
        return instance;
    }
    public static void main ( String[] args ) {
        launch ( args );
    }
    @Override public void start ( Stage primaryStage ) {
        try {
            stage = primaryStage;
            gotoLogin();
            primaryStage.show();
        } catch ( Exception ex ) {
            Logger.getLogger ( App.class.getName() ).log ( Level.SEVERE, null, ex );
        }
    }
    public User getLoggedUser() {
        return loggedUser;
    }
    public boolean userLogging ( String userId, String password ) {
        if ( Authenticator.validate ( userId, password ) ) {
            loggedUser = User.of ( userId );
            gotoProfile();
            return true;
        } else {
            return false;
        }
    }
    public void userLogout() {
        loggedUser = null;
        gotoLogin();
    }
    private void gotoProfile() {
        try {
            replaceSceneContent ( "profile.fxml" );
        } catch ( Exception ex ) {
            Logger.getLogger ( App.class.getName() ).log ( Level.SEVERE, null, ex );
        }
    }
    private void gotoLogin() {
        try {
            replaceSceneContent ( "login.fxml" );
        } catch ( Exception ex ) {
            Logger.getLogger ( App.class.getName() ).log ( Level.SEVERE, null, ex );
        }
    }
    private Parent replaceSceneContent ( String fxml ) throws Exception {
        Parent page = ( Parent ) FXMLLoader.load ( App.class.getResource ( fxml ), null, new JavaFXBuilderFactory() );
        Scene scene = stage.getScene();
        if ( scene == null ) {
            scene = new Scene ( page, 700, 450 );
            scene.getStylesheets().add ( App.class.getResource ( "demo.css" ).toExternalForm() );
            stage.setScene ( scene );
        } else {
            stage.getScene().setRoot ( page );
        }
        stage.sizeToScene();
        return page;
    }
}
