public class TestTable01 extends JPanel {
    private JTable mainTable;
    public TestTable01() {
        super ( new GridLayout ( 1, 0 ) );
        String[] columnNames = {"First Name",
                                "Last Name",
                                "Sport",
                                "# of Years",
                                "Vegetarian"
                               };
        Object[][] data = {
            {
                "Kathy", "Smith",
                "Snowboarding", new Integer ( 5 ), new Boolean ( false )
            },
            {
                "John", "Doe",
                "Rowing", new Integer ( 3 ), new Boolean ( true )
            },
            {
                "Sue", "Black",
                "Knitting", new Integer ( 2 ), new Boolean ( false )
            },
            {
                "Jane", "White",
                "Speed reading", new Integer ( 20 ), new Boolean ( true )
            },
            {
                "Joe", "Brown",
                "Pool", new Integer ( 10 ), new Boolean ( false )
            }
        };
        mainTable = new JTable ( data, columnNames );
        mainTable.setAutoResizeMode ( JTable.AUTO_RESIZE_OFF );
        mainTable.setPreferredScrollableViewportSize ( new Dimension ( 500, 70 ) );
        mainTable.setFillsViewportHeight ( true );
        update();
        JScrollPane scrollPane = new JScrollPane ( mainTable );
        add ( scrollPane );
    }
    private static void createAndShowGUI() {
        JFrame frame = new JFrame ( "SimpleTableDemo" );
        frame.setDefaultCloseOperation ( JFrame.EXIT_ON_CLOSE );
        TestTable01 newContentPane = new TestTable01();
        newContentPane.setOpaque ( true );
        frame.setContentPane ( newContentPane );
        frame.pack();
        frame.setVisible ( true );
    }
    private void update() {
        System.out.println ( "updating" );
        adjustJTableRowSizes ( mainTable );
        for ( int i = 0; i < mainTable.getColumnCount(); i++ ) {
            adjustColumnSizes ( mainTable, i, 2 );
        }
    }
    private void adjustJTableRowSizes ( JTable jTable ) {
        for ( int row = 0; row < jTable.getRowCount(); row++ ) {
            int maxHeight = 0;
            for ( int column = 0; column < jTable.getColumnCount(); column++ ) {
                TableCellRenderer cellRenderer = jTable.getCellRenderer ( row, column );
                Object valueAt = jTable.getValueAt ( row, column );
                Component tableCellRendererComponent = cellRenderer.getTableCellRendererComponent ( jTable, valueAt, false, false, row, column );
                int heightPreferable = tableCellRendererComponent.getPreferredSize().height;
                maxHeight = Math.max ( heightPreferable, maxHeight );
            }
            jTable.setRowHeight ( row, maxHeight );
        }
    }
    public void adjustColumnSizes ( JTable table, int column, int margin ) {
        DefaultTableColumnModel colModel = ( DefaultTableColumnModel ) table.getColumnModel();
        TableColumn col = colModel.getColumn ( column );
        int width;
        TableCellRenderer renderer = col.getHeaderRenderer();
        if ( renderer == null ) {
            renderer = table.getTableHeader().getDefaultRenderer();
        }
        Component comp = renderer.getTableCellRendererComponent ( table, col.getHeaderValue(), false, false, 0, 0 );
        width = comp.getPreferredSize().width;
        for ( int r = 0; r < table.getRowCount(); r++ ) {
            renderer = table.getCellRenderer ( r, column );
            comp = renderer.getTableCellRendererComponent ( table, table.getValueAt ( r, column ), false, false, r, column );
            int currentWidth = comp.getPreferredSize().width;
            width = Math.max ( width, currentWidth );
        }
        width += 2 * margin;
        col.setPreferredWidth ( width );
        col.setWidth ( width );
    }
    public static void main ( String[] args ) {
        javax.swing.SwingUtilities.invokeLater ( new Runnable() {
            public void run() {
                createAndShowGUI();
            }
        } );
    }
}
