package x.y.z;
import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D;
import java.util.Date;
import java.util.List;
import org.jfree.chart.axis.AxisState;
import org.jfree.chart.axis.DateAxis;
import org.jfree.chart.axis.DateTick;
import org.jfree.chart.axis.DateTickUnit;
import org.jfree.chart.axis.TickType;
import org.jfree.chart.axis.ValueTick;
import org.jfree.text.TextUtilities;
import org.jfree.ui.RectangleEdge;
public class AlignedDateAxis extends DateAxis {
    private TickLabelPosition tickLabelPosition = TickLabelPosition.DEFAULT_VALUE;
    static private final long serialVersionUID = 1;
    public enum TickLabelPosition {
        INTERVAL_START,
        INTERVAL_MIDDLE;
        static public final TickLabelPosition DEFAULT_VALUE = INTERVAL_START;
    }
    public AlignedDateAxis() {
        super();
    }
    public AlignedDateAxis ( TickLabelPosition tickLabelPos ) {
        this();
        this.setTickLabelPosition ( tickLabelPos );
    }
    public TickLabelPosition getTickLabelPosition() {
        return this.tickLabelPosition;
    }
    public void setTickLabelPosition ( TickLabelPosition value ) {
        this.tickLabelPosition = value;
    }
    @Override
    protected float[] calculateAnchorPoint ( ValueTick tick, double cursor, Rectangle2D dataArea, RectangleEdge edge ) {
        float[] resultAnchor = super.calculateAnchorPoint ( tick, cursor, dataArea, edge );
        Date tickDate = ( tick instanceof DateTick ) ?
                        ( ( DateTick ) tick ).getDate() :
                        new Date ( ( long ) tick.getValue() );
        double labelShift;
        switch ( this.getTickLabelPosition() ) {
        case INTERVAL_MIDDLE:
            DateTickUnit unit = this.getTickUnit();
            Date nextTickDate = unit.addToDate ( tickDate, this.getTimeZone() );
            double nextTickVal = this.valueToJava2D ( nextTickDate.getTime(), dataArea, edge );
            labelShift = ( nextTickVal - resultAnchor[0] ) / 2;
            break;
        case INTERVAL_START:
        default:
            labelShift = 0;
            break;
        }
        if ( RectangleEdge.isTopOrBottom ( edge ) ) {
            resultAnchor[0] += labelShift;
        } else if ( RectangleEdge.isLeftOrRight ( edge ) ) {
            resultAnchor[1] += labelShift;
        }
        return resultAnchor;
    }
    @Override
    protected AxisState drawTickMarksAndLabels ( Graphics2D g2, double cursor, Rectangle2D plotArea, Rectangle2D dataArea, RectangleEdge edge ) {
        AxisState state = new AxisState ( cursor );
        if ( this.isAxisLineVisible() ) {
            this.drawAxisLine ( g2, cursor, dataArea, edge );
        }
        List<DateTick> ticks = this.refreshTicks ( g2, state, dataArea, edge );
        state.setTicks ( ticks );
        g2.setFont ( this.getTickLabelFont() );
        for ( DateTick tick : ticks ) {
            if ( this.isTickLabelsVisible() ) {
                g2.setPaint ( this.getTickLabelPaint() );
                float anchorPoint[] = this.calculateAnchorPoint ( tick, cursor, dataArea, edge );
                Shape labelBounds = TextUtilities.calculateRotatedStringBounds ( tick.getText(), g2, anchorPoint[0], anchorPoint[1], tick.getTextAnchor(), tick.getAngle(), tick.getRotationAnchor() );
                double labelEdge = ( RectangleEdge.isTopOrBottom ( edge ) ) ? ( labelBounds.getBounds2D().getMaxX() ) : ( labelBounds.getBounds2D().getMaxY() );
                double dataAreaBound = ( RectangleEdge.isTopOrBottom ( edge ) ) ? ( dataArea.getMaxX() ) : ( dataArea.getMaxY() );
                if ( ( labelEdge - 5 ) <= dataAreaBound ) {
                    TextUtilities.drawRotatedString ( tick.getText(), g2, anchorPoint[0], anchorPoint[1], tick.getTextAnchor(), tick.getAngle(), tick.getRotationAnchor() );
                }
            }
            if ( ( this.isTickMarksVisible() && tick.getTickType().equals ( TickType.MAJOR ) ) || ( this.isMinorTickMarksVisible() && tick.getTickType().equals ( TickType.MINOR ) ) ) {
                double ol = tick.getTickType().equals ( TickType.MINOR ) ? this.getMinorTickMarkOutsideLength() : this.getTickMarkOutsideLength();
                double il = tick.getTickType().equals ( TickType.MINOR ) ? this.getMinorTickMarkInsideLength() : this.getTickMarkInsideLength();
                float tickVal = ( float ) this.valueToJava2D ( tick.getValue(), dataArea, edge );
                Line2D mark = null;
                g2.setStroke ( this.getTickMarkStroke() );
                g2.setPaint ( this.getTickMarkPaint() );
                if ( edge == RectangleEdge.LEFT ) {
                    mark = new Line2D.Double ( cursor - ol, tickVal, cursor + il, tickVal );
                } else if ( edge == RectangleEdge.RIGHT ) {
                    mark = new Line2D.Double ( cursor + ol, tickVal, cursor - il, tickVal );
                } else if ( edge == RectangleEdge.TOP ) {
                    mark = new Line2D.Double ( tickVal, cursor - ol, tickVal, cursor + il );
                } else if ( edge == RectangleEdge.BOTTOM ) {
                    mark = new Line2D.Double ( tickVal, cursor + ol, tickVal, cursor - il );
                }
                g2.draw ( mark );
            }
        }
        if ( this. isTickLabelsVisible() ) {
            double used = 0.0;
            if ( edge == RectangleEdge.LEFT ) {
                used += this.findMaximumTickLabelWidth ( ticks, g2, plotArea, this.isVerticalTickLabels() );
                state.cursorLeft ( used );
            } else if ( edge == RectangleEdge.RIGHT ) {
                used = this.findMaximumTickLabelWidth ( ticks, g2, plotArea, this.isVerticalTickLabels() );
                state.cursorRight ( used );
            } else if ( edge == RectangleEdge.TOP ) {
                used = this.findMaximumTickLabelHeight ( ticks, g2, plotArea, this.isVerticalTickLabels() );
                state.cursorUp ( used );
            } else if ( edge == RectangleEdge.BOTTOM ) {
                used = this.findMaximumTickLabelHeight ( ticks, g2, plotArea, this.isVerticalTickLabels() );
                state.cursorDown ( used );
            }
        }
        return state;
    }
}
