public static class PropertyParser {
    public static class ParsedParameters {
        private final Set<String> literals;
        private final Set<String> parameters;
        private ParsedParameters() {
            this.parameters = new HashSet<String>();
            this.literals = new HashSet<String>();
        }
        private void addLiteral ( String string ) {
            this.literals.add ( string );
        }
        private void addParameter ( String string ) {
            this.parameters.add ( string );
        }
        public Set<String> getLiterals() {
            return this.literals;
        }
        public Set<String> getParameters() {
            return this.parameters;
        }
    }
    private static final String BAD_FIELD_CHAR =
        "Illegal character detected for field parameter: %c";
    public static ParsedParameters parseParametersAndLiterals ( String string )
    throws IllegalArgumentException {
        if ( ( string == null ) || string.isEmpty() ) {
            return new ParsedParameters();
        }
        ParsedParameters result = new ParsedParameters();
        StringBuffer param = null;
        Character badChar = null;
        char c;
        char p = '^';
        boolean close = false;
        boolean lQuote = false;
        boolean open = false;
        int l = string.length();
        for ( int i = 0; i < l; ++i ) {
            c = string.charAt ( i );
            if ( !lQuote && ( p == '\'' ) && ( c == '$' ) ) {
                lQuote = true;
            } else if ( ( p == '$' ) && ( c == '{' ) ) {
                param = new StringBuffer();
                open = true;
                badChar = null;
            } else if ( open
                        && ( ( ( c == '}' ) && ( !lQuote || ( ( 1 + i ) == l ) ) ) || ( lQuote && ( p == '}' ) ) ) ) {
                open = false;
                close = true;
            } else if ( open ) {
                boolean validCharacter = Character.isLetterOrDigit ( c ) || ( c == '_' );
                if ( validCharacter || ( lQuote && ( c != '}' ) ) ) {
                    param.append ( c );
                }
                if ( !validCharacter && ( c != '}' ) ) {
                    badChar = c;
                }
            }
            if ( close ) {
                if ( ( badChar != null ) && ! ( lQuote && ( p == '}' ) && ( c == '\'' ) ) ) {
                    throw new IllegalArgumentException ( String.format ( BAD_FIELD_CHAR, badChar ) );
                } else if ( c != '\'' ) {
                    if ( param.length() > 0 ) {
                        result.addParameter ( param.toString() );
                    }
                } else {
                    result.addLiteral ( param.toString() );
                }
                lQuote = false;
                close = false;
                badChar = null;
            }
            p = c;
        }
        return result;
    }
}
