package com.test;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import javax.imageio.ImageIO;
import com.xuggle.mediatool.IMediaReader;
import com.xuggle.mediatool.MediaListenerAdapter;
import com.xuggle.mediatool.ToolFactory;
import com.xuggle.mediatool.event.IVideoPictureEvent;
import com.xuggle.xuggler.Global;
public class VideoThumbnailsExample {
    public static final double SECONDS_BETWEEN_FRAMES = 1;
    private static final String inputFilename = "/Users/ismail/practice/vlc/sample.3gp";
    private static final String outputFilePrefix = "/Users/ismail/practice/vlc/";
    private static int mVideoStreamIndex = -1;
    private static long mLastPtsWrite = Global.NO_PTS;
    public static final long MICRO_SECONDS_BETWEEN_FRAMES = ( long ) ( Global.DEFAULT_PTS_PER_SECOND * SECONDS_BETWEEN_FRAMES );
    public static void main ( String[] args ) {
        long startTime = System.currentTimeMillis();
        long stopTime = 0L;
        IMediaReader mediaReader = ToolFactory.makeReader ( inputFilename );
        try {
            mediaReader
            .setBufferedImageTypeToGenerate ( BufferedImage.TYPE_3BYTE_BGR );
            ImageSnapListener isListener = new ImageSnapListener();
            mediaReader.addListener ( isListener );
            while ( !isListener.isImageGrabbed() ) {
                mediaReader.readPacket();
            }
            stopTime = System.currentTimeMillis();
        } catch ( Exception ex ) {
            ex.printStackTrace();
        }
        System.out.println ( "Total Time: " + ( stopTime - startTime ) );
    }
    private static class ImageSnapListener extends MediaListenerAdapter {
        public boolean imageGrabbed = false;
        public void onVideoPicture ( IVideoPictureEvent event ) {
            if ( event.getStreamIndex() != mVideoStreamIndex ) {
                if ( mVideoStreamIndex == -1 ) {
                    mVideoStreamIndex = event.getStreamIndex();
                } else {
                    return;
                }
            }
            if ( mLastPtsWrite == Global.NO_PTS )
                mLastPtsWrite = event.getTimeStamp()
                                - MICRO_SECONDS_BETWEEN_FRAMES;
            if ( event.getTimeStamp() - mLastPtsWrite >= MICRO_SECONDS_BETWEEN_FRAMES ) {
                String outputFilename = dumpImageToFile ( event.getImage() );
                this.imageGrabbed = true;
                double seconds = ( ( double ) event.getTimeStamp() )
                                 / Global.DEFAULT_PTS_PER_SECOND;
                System.out.printf ( "at elapsed time of %6.3f seconds wrote: %s\n", seconds, outputFilename );
                mLastPtsWrite += MICRO_SECONDS_BETWEEN_FRAMES;
            }
        }
        private String dumpImageToFile ( BufferedImage image ) {
            try {
                String outputFilename = outputFilePrefix
                                        + System.currentTimeMillis() + ".jpg";
                System.out.println ( "Thumbnail image name is going to be : =====>" + outputFilename );
                ImageIO.write ( image, "jpg", new File ( outputFilename ) );
                return outputFilename;
            } catch ( IOException e ) {
                e.printStackTrace();
                return null;
            }
        }
        public boolean isImageGrabbed() {
            return imageGrabbed;
        }
    }
}
