import java.awt.*;
import java.awt.event.*;
import java.util.Date;
import javax.swing.*;
import javax.swing.event.*;
public class ListDemo extends JPanel implements ListSelectionListener {
    private JList list;
    private DefaultListModel listModel;
    private static final String hireString = "Hire";
    private static final String fireString = "Fire";
    private JButton fireButton;
    private JTextField employeeName;
    private javax.swing.Timer timer = null;
    private int delay = 3;
    private int count = 0;
    public ListDemo() {
        super ( new BorderLayout() );
        listModel = new DefaultListModel();
        listModel.addElement ( "Jane Doe" );
        listModel.addElement ( "John Smith" );
        listModel.addElement ( "Kathy Green" );
        list = new JList ( listModel );
        list.setSelectionMode ( ListSelectionModel.SINGLE_SELECTION );
        list.setSelectedIndex ( 0 );
        list.addListSelectionListener ( this );
        list.setVisibleRowCount ( 5 );
        JScrollPane listScrollPane = new JScrollPane ( list );
        JButton hireButton = new JButton ( hireString );
        HireListener hireListener = new HireListener ( hireButton );
        hireButton.setActionCommand ( hireString );
        hireButton.addActionListener ( hireListener );
        hireButton.setEnabled ( false );
        fireButton = new JButton ( fireString );
        fireButton.setActionCommand ( fireString );
        fireButton.addActionListener ( new FireListener() );
        employeeName = new JTextField ( 10 );
        employeeName.addActionListener ( hireListener );
        employeeName.getDocument().addDocumentListener ( hireListener );
        String name = listModel.getElementAt ( list.getSelectedIndex() ).toString();
        JPanel buttonPane = new JPanel();
        buttonPane.setLayout ( new BoxLayout ( buttonPane, BoxLayout.LINE_AXIS ) );
        buttonPane.add ( fireButton );
        buttonPane.add ( Box.createHorizontalStrut ( 5 ) );
        buttonPane.add ( new JSeparator ( SwingConstants.VERTICAL ) );
        buttonPane.add ( Box.createHorizontalStrut ( 5 ) );
        buttonPane.add ( employeeName );
        buttonPane.add ( hireButton );
        buttonPane.setBorder ( BorderFactory.createEmptyBorder ( 5, 5, 5, 5 ) );
        add ( listScrollPane, BorderLayout.CENTER );
        add ( buttonPane, BorderLayout.PAGE_END );
        start();
        System.out.println ( new Date() );
    }
    private void start() {
        timer = new javax.swing.Timer ( delay * 10, updateCol() );
        timer.start();
    }
    public Action updateCol() {
        return new AbstractAction ( "text load action" ) {
            private static final long serialVersionUID = 1L;
            @Override
            public void actionPerformed ( ActionEvent e ) {
                listModel.addElement ( "Jane Doe" );
                listModel.addElement ( "John Smith" );
                listModel.addElement ( "Kathy Green" );
                list.ensureIndexIsVisible ( listModel.getSize() - 1 );
                count++;
                if ( count >= 500 ) {
                    timer.stop();
                    System.out.println ( "update cycle completed" );
                    System.out.println ( new Date() );
                }
            }
        };
    }
    class FireListener implements ActionListener {
        @Override
        public void actionPerformed ( ActionEvent e ) {
            int index = list.getSelectedIndex();
            listModel.remove ( index );
            int size = listModel.getSize();
            if ( size == 0 ) {
                fireButton.setEnabled ( false );
            } else {
                if ( index == listModel.getSize() ) {
                    index--;
                }
                list.setSelectedIndex ( index );
                list.ensureIndexIsVisible ( index );
            }
        }
    }
    class HireListener implements ActionListener, DocumentListener {
        private boolean alreadyEnabled = false;
        private JButton button;
        HireListener ( JButton button ) {
            this.button = button;
        }
        @Override
        public void actionPerformed ( ActionEvent e ) {
            String name = employeeName.getText();
            if ( name.isEmpty() || alreadyInList ( name ) ) {
                Toolkit.getDefaultToolkit().beep();
                employeeName.requestFocusInWindow();
                employeeName.selectAll();
                return;
            }
            int index = list.getSelectedIndex();
            if ( index == -1 ) {
                index = 0;
            } else {
                index++;
            }
            listModel.insertElementAt ( employeeName.getText(), index );
            employeeName.requestFocusInWindow();
            employeeName.setText ( "" );
            list.setSelectedIndex ( index );
            list.ensureIndexIsVisible ( index );
        }
        protected boolean alreadyInList ( String name ) {
            return listModel.contains ( name );
        }
        @Override
        public void insertUpdate ( DocumentEvent e ) {
            enableButton();
        }
        @Override
        public void removeUpdate ( DocumentEvent e ) {
            handleEmptyTextField ( e );
        }
        @Override
        public void changedUpdate ( DocumentEvent e ) {
            if ( !handleEmptyTextField ( e ) ) {
                enableButton();
            }
        }
        private void enableButton() {
            if ( !alreadyEnabled ) {
                button.setEnabled ( true );
            }
        }
        private boolean handleEmptyTextField ( DocumentEvent e ) {
            if ( e.getDocument().getLength() <= 0 ) {
                button.setEnabled ( false );
                alreadyEnabled = false;
                return true;
            }
            return false;
        }
    }
    @Override
    public void valueChanged ( ListSelectionEvent e ) {
        if ( e.getValueIsAdjusting() == false ) {
            if ( list.getSelectedIndex() == -1 ) {
                fireButton.setEnabled ( false );
            } else {
                fireButton.setEnabled ( true );
            }
        }
    }
    private static void createAndShowGUI() {
        JFrame frame = new JFrame ( "ListDemo" );
        frame.setDefaultCloseOperation ( JFrame.EXIT_ON_CLOSE );
        JComponent newContentPane = new ListDemo();
        newContentPane.setOpaque ( true );
        frame.setContentPane ( newContentPane );
        frame.pack();
        frame.setVisible ( true );
    }
    public static void main ( String[] args ) {
        javax.swing.SwingUtilities.invokeLater ( new Runnable() {
            @Override
            public void run() {
                createAndShowGUI();
            }
        } );
    }
}
