public class PrintView extends FrameView implements Printable, ActionListener {
    public PrintView ( SingleFrameApplication app ) throws PrinterException {
        super ( app );
        initComponents();
        jButton1.addActionListener ( this );
        ResourceMap resourceMap = getResourceMap();
        int messageTimeout = resourceMap.getInteger ( "StatusBar.messageTimeout" );
        messageTimer = new Timer ( messageTimeout, new ActionListener() {
            public void actionPerformed ( ActionEvent e ) {
                statusMessageLabel.setText ( "" );
            }
        } );
        messageTimer.setRepeats ( false );
        int busyAnimationRate = resourceMap.getInteger ( "StatusBar.busyAnimationRate" );
        for ( int i = 0; i < busyIcons.length; i++ ) {
            busyIcons[i] = resourceMap.getIcon ( "StatusBar.busyIcons[" + i + "]" );
        }
        busyIconTimer = new Timer ( busyAnimationRate, new ActionListener() {
            public void actionPerformed ( ActionEvent e ) {
                busyIconIndex = ( busyIconIndex + 1 ) % busyIcons.length;
                statusAnimationLabel.setIcon ( busyIcons[busyIconIndex] );
            }
        } );
        idleIcon = resourceMap.getIcon ( "StatusBar.idleIcon" );
        statusAnimationLabel.setIcon ( idleIcon );
        progressBar.setVisible ( false );
        TaskMonitor taskMonitor = new TaskMonitor ( getApplication().getContext() );
        taskMonitor.addPropertyChangeListener ( new java.beans.PropertyChangeListener() {
            public void propertyChange ( java.beans.PropertyChangeEvent evt ) {
                String propertyName = evt.getPropertyName();
                if ( "started".equals ( propertyName ) ) {
                    if ( !busyIconTimer.isRunning() ) {
                        statusAnimationLabel.setIcon ( busyIcons[0] );
                        busyIconIndex = 0;
                        busyIconTimer.start();
                    }
                    progressBar.setVisible ( true );
                    progressBar.setIndeterminate ( true );
                } else if ( "done".equals ( propertyName ) ) {
                    busyIconTimer.stop();
                    statusAnimationLabel.setIcon ( idleIcon );
                    progressBar.setVisible ( false );
                    progressBar.setValue ( 0 );
                } else if ( "message".equals ( propertyName ) ) {
                    String text = ( String ) ( evt.getNewValue() );
                    statusMessageLabel.setText ( ( text == null ) ? "" : text );
                    messageTimer.restart();
                } else if ( "progress".equals ( propertyName ) ) {
                    int value = ( Integer ) ( evt.getNewValue() );
                    progressBar.setVisible ( true );
                    progressBar.setIndeterminate ( false );
                    progressBar.setValue ( value );
                }
            }
        } );
    }
    @Action
    public void showAboutBox() {
        if ( aboutBox == null ) {
            JFrame mainFrame = PrintApp.getApplication().getMainFrame();
            aboutBox = new PrintAboutBox ( mainFrame );
            aboutBox.setLocationRelativeTo ( mainFrame );
        }
        PrintApp.getApplication().show ( aboutBox );
    }
    @SuppressWarnings ( "unchecked" )
    private void initComponents() {
        mainPanel = new javax.swing.JPanel();
        jButton1 = new javax.swing.JButton();
        jPanel1 = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        jComboBox1 = new javax.swing.JComboBox();
        jSlider1 = new javax.swing.JSlider();
        jProgressBar1 = new javax.swing.JProgressBar();
        jPasswordField1 = new javax.swing.JPasswordField();
        menuBar = new javax.swing.JMenuBar();
        javax.swing.JMenu fileMenu = new javax.swing.JMenu();
        javax.swing.JMenuItem exitMenuItem = new javax.swing.JMenuItem();
        javax.swing.JMenu helpMenu = new javax.swing.JMenu();
        javax.swing.JMenuItem aboutMenuItem = new javax.swing.JMenuItem();
        statusPanel = new javax.swing.JPanel();
        javax.swing.JSeparator statusPanelSeparator = new javax.swing.JSeparator();
        statusMessageLabel = new javax.swing.JLabel();
        statusAnimationLabel = new javax.swing.JLabel();
        progressBar = new javax.swing.JProgressBar();
        mainPanel.setName ( "mainPanel" );
        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance ( print.PrintApp.class ).getContext().getResourceMap ( PrintView.class );
        jButton1.setText ( resourceMap.getString ( "jButton1.text" ) );
        jButton1.setName ( "jButton1" );
        jPanel1.setBorder ( new javax.swing.border.MatteBorder ( null ) );
        jPanel1.setName ( "jPanel1" );
        jLabel1.setText ( resourceMap.getString ( "jLabel1.text" ) );
        jLabel1.setName ( "jLabel1" );
        jComboBox1.setModel ( new javax.swing.DefaultComboBoxModel ( new String[] { "1", "2", "3", "4" } ) );
        jComboBox1.setName ( "jComboBox1" );
        jSlider1.setName ( "jSlider1" );
        jProgressBar1.setName ( "jProgressBar1" );
        jPasswordField1.setText ( resourceMap.getString ( "jPasswordField1.text" ) );
        jPasswordField1.setName ( "jPasswordField1" );
        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout ( jPanel1 );
        jPanel1.setLayout ( jPanel1Layout );
        jPanel1Layout.setHorizontalGroup (
            jPanel1Layout.createParallelGroup ( javax.swing.GroupLayout.Alignment.LEADING )
            .addGroup ( jPanel1Layout.createSequentialGroup()
                        .addGap ( 371, 371, 371 )
                        .addComponent ( jLabel1 )
                        .addPreferredGap ( javax.swing.LayoutStyle.ComponentPlacement.RELATED, 280, Short.MAX_VALUE )
                        .addComponent ( jComboBox1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE )
                        .addGap ( 133, 133, 133 ) )
            .addGroup ( javax.swing.GroupLayout.Alignment.TRAILING, jPanel1Layout.createSequentialGroup()
                        .addContainerGap ( 233, Short.MAX_VALUE )
                        .addComponent ( jProgressBar1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE )
                        .addGap ( 185, 185, 185 )
                        .addComponent ( jSlider1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE )
                        .addGap ( 97, 97, 97 ) )
            .addGroup ( javax.swing.GroupLayout.Alignment.TRAILING, jPanel1Layout.createSequentialGroup()
                        .addContainerGap ( 486, Short.MAX_VALUE )
                        .addComponent ( jPasswordField1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE )
                        .addGap ( 264, 264, 264 ) )
        );
        jPanel1Layout.setVerticalGroup (
            jPanel1Layout.createParallelGroup ( javax.swing.GroupLayout.Alignment.LEADING )
            .addGroup ( jPanel1Layout.createSequentialGroup()
                        .addGap ( 49, 49, 49 )
                        .addGroup ( jPanel1Layout.createParallelGroup ( javax.swing.GroupLayout.Alignment.TRAILING )
                                    .addComponent ( jComboBox1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE )
                                    .addComponent ( jLabel1 ) )
                        .addGap ( 62, 62, 62 )
                        .addGroup ( jPanel1Layout.createParallelGroup ( javax.swing.GroupLayout.Alignment.TRAILING )
                                    .addComponent ( jSlider1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE )
                                    .addComponent ( jProgressBar1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE ) )
                        .addPreferredGap ( javax.swing.LayoutStyle.ComponentPlacement.RELATED, 60, Short.MAX_VALUE )
                        .addComponent ( jPasswordField1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE )
                        .addGap ( 32, 32, 32 ) )
        );
        javax.swing.GroupLayout mainPanelLayout = new javax.swing.GroupLayout ( mainPanel );
        mainPanel.setLayout ( mainPanelLayout );
        mainPanelLayout.setHorizontalGroup (
            mainPanelLayout.createParallelGroup ( javax.swing.GroupLayout.Alignment.LEADING )
            .addGroup ( mainPanelLayout.createSequentialGroup()
                        .addGap ( 446, 446, 446 )
                        .addComponent ( jButton1 )
                        .addContainerGap ( 466, Short.MAX_VALUE ) )
            .addGroup ( javax.swing.GroupLayout.Alignment.TRAILING, mainPanelLayout.createSequentialGroup()
                        .addContainerGap ( 66, Short.MAX_VALUE )
                        .addComponent ( jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE )
                        .addGap ( 46, 46, 46 ) )
        );
        mainPanelLayout.setVerticalGroup (
            mainPanelLayout.createParallelGroup ( javax.swing.GroupLayout.Alignment.LEADING )
            .addGroup ( mainPanelLayout.createSequentialGroup()
                        .addGap ( 107, 107, 107 )
                        .addComponent ( jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE )
                        .addPreferredGap ( javax.swing.LayoutStyle.ComponentPlacement.RELATED, 30, Short.MAX_VALUE )
                        .addComponent ( jButton1 )
                        .addGap ( 56, 56, 56 ) )
        );
        menuBar.setName ( "menuBar" );
        fileMenu.setText ( resourceMap.getString ( "fileMenu.text" ) );
        fileMenu.setName ( "fileMenu" );
        javax.swing.ActionMap actionMap = org.jdesktop.application.Application.getInstance ( print.PrintApp.class ).getContext().getActionMap ( PrintView.class, this );
        exitMenuItem.setAction ( actionMap.get ( "quit" ) );
        exitMenuItem.setName ( "exitMenuItem" );
        fileMenu.add ( exitMenuItem );
        menuBar.add ( fileMenu );
        helpMenu.setText ( resourceMap.getString ( "helpMenu.text" ) );
        helpMenu.setName ( "helpMenu" );
        aboutMenuItem.setAction ( actionMap.get ( "showAboutBox" ) );
        aboutMenuItem.setName ( "aboutMenuItem" );
        helpMenu.add ( aboutMenuItem );
        menuBar.add ( helpMenu );
        statusPanel.setName ( "statusPanel" );
        statusPanelSeparator.setName ( "statusPanelSeparator" );
        statusMessageLabel.setName ( "statusMessageLabel" );
        statusAnimationLabel.setHorizontalAlignment ( javax.swing.SwingConstants.LEFT );
        statusAnimationLabel.setName ( "statusAnimationLabel" );
        progressBar.setName ( "progressBar" );
        javax.swing.GroupLayout statusPanelLayout = new javax.swing.GroupLayout ( statusPanel );
        statusPanel.setLayout ( statusPanelLayout );
        statusPanelLayout.setHorizontalGroup (
            statusPanelLayout.createParallelGroup ( javax.swing.GroupLayout.Alignment.LEADING )
            .addComponent ( statusPanelSeparator, javax.swing.GroupLayout.DEFAULT_SIZE, 975, Short.MAX_VALUE )
            .addGroup ( statusPanelLayout.createSequentialGroup()
                        .addContainerGap()
                        .addComponent ( statusMessageLabel )
                        .addPreferredGap ( javax.swing.LayoutStyle.ComponentPlacement.RELATED, 805, Short.MAX_VALUE )
                        .addComponent ( progressBar, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE )
                        .addPreferredGap ( javax.swing.LayoutStyle.ComponentPlacement.RELATED )
                        .addComponent ( statusAnimationLabel )
                        .addContainerGap() )
        );
        statusPanelLayout.setVerticalGroup (
            statusPanelLayout.createParallelGroup ( javax.swing.GroupLayout.Alignment.LEADING )
            .addGroup ( statusPanelLayout.createSequentialGroup()
                        .addComponent ( statusPanelSeparator, javax.swing.GroupLayout.PREFERRED_SIZE, 2, javax.swing.GroupLayout.PREFERRED_SIZE )
                        .addPreferredGap ( javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE )
                        .addGroup ( statusPanelLayout.createParallelGroup ( javax.swing.GroupLayout.Alignment.BASELINE )
                                    .addComponent ( statusMessageLabel )
                                    .addComponent ( statusAnimationLabel )
                                    .addComponent ( progressBar, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE ) )
                        .addGap ( 3, 3, 3 ) )
        );
        setComponent ( mainPanel );
        setMenuBar ( menuBar );
        setStatusBar ( statusPanel );
    }
    private javax.swing.JButton jButton1;
    private javax.swing.JComboBox jComboBox1;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPasswordField jPasswordField1;
    private javax.swing.JProgressBar jProgressBar1;
    private javax.swing.JSlider jSlider1;
    private javax.swing.JPanel mainPanel;
    private javax.swing.JMenuBar menuBar;
    private javax.swing.JProgressBar progressBar;
    private javax.swing.JLabel statusAnimationLabel;
    private javax.swing.JLabel statusMessageLabel;
    private javax.swing.JPanel statusPanel;
    private final Timer messageTimer;
    private final Timer busyIconTimer;
    private final Icon idleIcon;
    private final Icon[] busyIcons = new Icon[15];
    private int busyIconIndex = 0;
    private JDialog aboutBox;
    public void actionPerformed ( ActionEvent e ) {
        if ( e.getActionCommand().equals ( "PRINT" ) ) {
            PrinterJob job = PrinterJob.getPrinterJob();
            job.setPrintable ( this );
            boolean ok = job.printDialog();
            if ( ok ) {
                try {
                    job.print();
                } catch ( PrinterException ex ) {
                }
            }
        }
    }
    public int print ( Graphics g, PageFormat pf, int page ) throws
        PrinterException {
        if ( page > 0 ) {
            return NO_SUCH_PAGE;
        }
        Graphics2D g2d = ( Graphics2D ) g;
        g2d.translate ( pf.getImageableX(), pf.getImageableY() );
        jPanel1.paint ( g2d );
        jPanel1.repaint();
        return PAGE_EXISTS;
    }
}
