public class TestGame {
    public static void main ( String[] args ) throws InterruptedException {
        new TestGame();
    }
    public TestGame() {
        EventQueue.invokeLater ( new Runnable() {
            @Override
            public void run() {
                try {
                    UIManager.setLookAndFeel ( UIManager.getSystemLookAndFeelClassName() );
                } catch ( ClassNotFoundException ex ) {
                } catch ( InstantiationException ex ) {
                } catch ( IllegalAccessException ex ) {
                } catch ( UnsupportedLookAndFeelException ex ) {
                }
                Game game = new Game();
                JFrame frame = new JFrame ( "Test" );
                frame.setDefaultCloseOperation ( JFrame.EXIT_ON_CLOSE );
                frame.setLayout ( new BorderLayout() );
                frame.add ( game );
                frame.pack();
                frame.setLocationRelativeTo ( null );
                frame.setVisible ( true );
            }
        } );
    }
    public class GameThread extends Thread {
        private Game game;
        public GameThread ( Game game ) {
            setDaemon ( false );
            this.game = game;
        }
        @Override
        public void run() {
            while ( true ) {
                game.move();
                try {
                    long startedAt = System.currentTimeMillis();
                    SwingUtilities.invokeAndWait ( new Runnable() {
                        @Override
                        public void run() {
                            game.repaint();
                        }
                    } );
                    long completedAt = System.currentTimeMillis();
                    long sleepFor = 30 - ( completedAt - startedAt );
                    if ( sleepFor < 0 ) {
                        sleepFor = 30;
                    }
                    Thread.sleep ( sleepFor );
                } catch ( Exception exp ) {
                    exp.printStackTrace();
                }
            }
        }
    }
    public class Game extends JPanel {
        Square square = new Square ( this );
        Ground ground = new Ground ( this );
        public Game() {
            setFocusable ( true );
            InputMap im = getInputMap ( WHEN_IN_FOCUSED_WINDOW );
            ActionMap am = getActionMap();
            im.put ( KeyStroke.getKeyStroke ( KeyEvent.VK_LEFT, 0, false ), "press-left" );
            im.put ( KeyStroke.getKeyStroke ( KeyEvent.VK_RIGHT, 0, false ), "press-right" );
            im.put ( KeyStroke.getKeyStroke ( KeyEvent.VK_DOWN, 0, false ), "press-down" );
            im.put ( KeyStroke.getKeyStroke ( KeyEvent.VK_UP, 0, false ), "press-up" );
            im.put ( KeyStroke.getKeyStroke ( KeyEvent.VK_LEFT, 0, true ), "release-left" );
            im.put ( KeyStroke.getKeyStroke ( KeyEvent.VK_RIGHT, 0, true ), "release-right" );
            im.put ( KeyStroke.getKeyStroke ( KeyEvent.VK_DOWN, 0, true ), "release-down" );
            im.put ( KeyStroke.getKeyStroke ( KeyEvent.VK_UP, 0, true ), "release-up" );
            am.put ( "press-left", new PressLeftAction ( square ) );
            am.put ( "press-right", new PressRightAction ( square ) );
            am.put ( "press-down", new PressDownAction ( square ) );
            am.put ( "press-up", new PressUpAction ( square ) );
            am.put ( "release-left", new ReleaseLeftAction ( square ) );
            am.put ( "release-right", new ReleaseRightAction ( square ) );
            am.put ( "release-down", new ReleaseDownAction ( square ) );
            am.put ( "release-up", new ReleaseUpAction ( square ) );
            new GameThread ( this ).start();
        }
        public void move() {
            square.move();
        }
        @Override
        public Dimension getPreferredSize() {
            return new Dimension ( 400, 400 );
        }
        @Override
        protected void paintComponent ( Graphics g ) {
            super.paintComponent ( g );
            Graphics2D g2d = ( Graphics2D ) g.create();
            g2d.setRenderingHint ( RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON );
            square.paint ( g2d );
            ground.paint ( g2d );
            g2d.dispose();
        }
    }
    public class Square {
        Square square;
        private int x, xa;
        private int y;
        private int ya;
        private Game game;
        private int fistX, fistY;
        private int d = 60;
        private int wide;
        private boolean onGround;
        public Square ( Game game ) {
            this.game = game;
            x = 100;
            y = 100;
            xa = 0;
            ya = 0;
            onGround = false;
            wide = game.getWidth();
        }
        public void move() {
            y += ya;
            x += xa;
            if ( x < 0 ) {
                x = 0;
            } else if ( x + 30 > game.getWidth() ) {
                x = game.getWidth() - 30;
            }
            if ( y < 0 ) {
                y = 0;
            } else if ( collision() ) {
                onGround = true;
                y = game.ground.getBounds().y - d;
            }
        }
        public void paint ( Graphics2D g ) {
            g.setColor ( Color.RED );
            System.out.println ( x + "x" + ( y - d ) );
            g.fillRoundRect ( x, y, 30, d, 10, 10 );
        }
        private boolean collision() {
            return game.ground.getBounds().intersects ( getBounds() );
        }
        public Rectangle getBounds() {
            return new Rectangle ( x, y, 30, 60 );
        }
    }
    public class Ground {
        private Game game;
        public Ground ( Game game ) {
            this.game = game;
        }
        public void paint ( Graphics2D g ) {
            g.setColor ( Color.BLACK );
            g.fillRect ( 0, game.getHeight() - 30, game.getWidth(), 30 );
        }
        public Rectangle getBounds() {
            return new Rectangle ( 0, game.getHeight() - 30, game.getWidth(), 30 );
        }
    }
    public abstract class AbstractSquareAction extends AbstractAction {
        private Square square;
        public AbstractSquareAction ( Square square ) {
            this.square = square;
        }
        public Square getSquare() {
            return square;
        }
    }
    public class PressLeftAction extends AbstractSquareAction {
        public PressLeftAction ( Square square ) {
            super ( square );
        }
        @Override
        public void actionPerformed ( ActionEvent e ) {
            getSquare().xa = -3;
            System.out.println ( "pressLeft" );
        }
    }
    public class PressRightAction extends AbstractSquareAction {
        public PressRightAction ( Square square ) {
            super ( square );
        }
        @Override
        public void actionPerformed ( ActionEvent e ) {
            getSquare().xa = 3;
        }
    }
    public class PressDownAction extends AbstractSquareAction {
        public PressDownAction ( Square square ) {
            super ( square );
        }
        @Override
        public void actionPerformed ( ActionEvent e ) {
            getSquare().ya = 30;
        }
    }
    public class PressUpAction extends AbstractSquareAction {
        public PressUpAction ( Square square ) {
            super ( square );
        }
        @Override
        public void actionPerformed ( ActionEvent e ) {
            getSquare().ya -= 30;
        }
    }
    public class ReleaseLeftAction extends AbstractSquareAction {
        public ReleaseLeftAction ( Square square ) {
            super ( square );
        }
        @Override
        public void actionPerformed ( ActionEvent e ) {
            getSquare().xa = 0;
        }
    }
    public class ReleaseRightAction extends AbstractSquareAction {
        public ReleaseRightAction ( Square square ) {
            super ( square );
        }
        @Override
        public void actionPerformed ( ActionEvent e ) {
            getSquare().xa = 0;
        }
    }
    public class ReleaseDownAction extends AbstractSquareAction {
        public ReleaseDownAction ( Square square ) {
            super ( square );
        }
        @Override
        public void actionPerformed ( ActionEvent e ) {
            getSquare().ya = 0;
        }
    }
    public class ReleaseUpAction extends AbstractSquareAction {
        public ReleaseUpAction ( Square square ) {
            super ( square );
        }
        @Override
        public void actionPerformed ( ActionEvent e ) {
            getSquare().ya = 0;
        }
    }
}
