public class TestRoundButton {
    public static void main ( String[] args ) {
        new TestRoundButton();
    }
    public TestRoundButton() {
        EventQueue.invokeLater ( new Runnable() {
            @Override
            public void run() {
                try {
                    UIManager.setLookAndFeel ( UIManager.getSystemLookAndFeelClassName() );
                } catch ( ClassNotFoundException ex ) {
                } catch ( InstantiationException ex ) {
                } catch ( IllegalAccessException ex ) {
                } catch ( UnsupportedLookAndFeelException ex ) {
                }
                JFrame frame = new JFrame();
                frame.setDefaultCloseOperation ( JFrame.EXIT_ON_CLOSE );
                frame.getContentPane().setBackground ( Color.RED );
                frame.setLayout ( new GridBagLayout() );
                frame.add ( new RoundButton ( ":)" ) );
                frame.pack();
                frame.setLocationRelativeTo ( null );
                frame.setVisible ( true );
            }
        } );
    }
    public class RoundButton extends JPanel {
        ActionListener actionListener;
        String label;
        protected boolean pressed = false;
        private MouseListener listener;
        public RoundButton() {
            this ( "" );
        }
        @Override
        public void addNotify() {
            super.addNotify();
            if ( listener == null ) {
                listener = new MouseHandler();
                addMouseListener ( listener );
            }
        }
        @Override
        public void removeNotify() {
            removeMouseListener ( listener );
            super.removeNotify();
        }
        public RoundButton ( String label ) {
            this.label = label;
            setOpaque ( false );
        }
        public String getLabel() {
            return label;
        }
        public void setLabel ( String label ) {
            this.label = label;
            invalidate();
            repaint();
        }
        @Override
        protected void paintComponent ( Graphics g ) {
            super.paintComponent ( g );
            int s = Math.min ( getSize().width - 1, getSize().height - 1 );
            Graphics2D g2d = ( Graphics2D ) g.create();
            g2d.setRenderingHint ( RenderingHints.KEY_ALPHA_INTERPOLATION, RenderingHints.VALUE_ALPHA_INTERPOLATION_QUALITY );
            g2d.setRenderingHint ( RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON );
            g2d.setRenderingHint ( RenderingHints.KEY_COLOR_RENDERING, RenderingHints.VALUE_COLOR_RENDER_QUALITY );
            g2d.setRenderingHint ( RenderingHints.KEY_DITHERING, RenderingHints.VALUE_DITHER_ENABLE );
            g2d.setRenderingHint ( RenderingHints.KEY_FRACTIONALMETRICS, RenderingHints.VALUE_FRACTIONALMETRICS_ON );
            g2d.setRenderingHint ( RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BILINEAR );
            g2d.setRenderingHint ( RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY );
            g2d.setRenderingHint ( RenderingHints.KEY_STROKE_CONTROL, RenderingHints.VALUE_STROKE_PURE );
            Color highlight = getBackground();
            if ( pressed ) {
                highlight = highlight.darker();
            }
            Color darklight = highlight.darker();
            LinearGradientPaint lgp = new LinearGradientPaint (
                new Point ( 0, 0 ),
                new Point ( 0, s ),
                new float[] {0, 1f},
                new Color[] {highlight, darklight} );
            Ellipse2D shape = new Ellipse2D.Float ( 0, 0, s, s );
            g2d.setPaint ( lgp );
            g2d.fill ( shape );
            g2d.setColor ( getBackground().darker().darker().darker() );
            g2d.draw ( shape );
            Font f = getFont();
            if ( f != null ) {
                FontMetrics fm = getFontMetrics ( getFont() );
                g2d.setColor ( getForeground() );
                g2d.drawString ( label,
                                 s / 2 - fm.stringWidth ( label ) / 2,
                                 s / 2 + fm.getMaxDescent() );
            }
            g2d.dispose();
        }
        public Dimension getPreferredSize() {
            Font f = getFont();
            if ( f != null ) {
                FontMetrics fm = getFontMetrics ( getFont() );
                int max = Math.max ( fm.stringWidth ( label ) + 40, fm.getHeight() + 40 );
                return new Dimension ( max, max );
            } else {
                return new Dimension ( 100, 100 );
            }
        }
        public Dimension getMinimumSize() {
            return new Dimension ( 100, 100 );
        }
        public void addActionListener ( ActionListener listener ) {
            listenerList.add ( ActionListener.class, listener );
        }
        public void removeActionListener ( ActionListener listener ) {
            listenerList.add ( ActionListener.class, listener );
        }
        public boolean contains ( int x, int y ) {
            int mx = getSize().width / 2;
            int my = getSize().height / 2;
            return ( ( ( mx - x ) * ( mx - x ) + ( my - y ) * ( my - y ) ) <= mx * mx );
        }
        public class MouseHandler extends MouseAdapter {
            @Override
            public void mousePressed ( MouseEvent e ) {
                pressed = true;
                repaint();
            }
            @Override
            public void mouseReleased ( MouseEvent e ) {
                pressed = false;
                repaint();
            }
        }
    }
}
