import java.io.IOException;
import java.io.InputStream;
import static org.apache.commons.io.IOUtils.toByteArray;
import static org.apache.commons.io.IOUtils.closeQuietly;
...
public class MyClassLoader
    extends ClassLoader {
    MyClassLoaderListener listener;
    MyClassLoader ( ClassLoader parent, MyClassLoaderListener listener ) {
        super ( parent );
        this.listener = listener;
    }
    @Override
    protected Class<?> loadClass ( String name, boolean resolve )
    throws ClassNotFoundException {
        if ( name.startsWith ( "java." ) ) {
            return super.loadClass ( name, resolve );
        } else {
            Class<?> c = findLoadedClass ( name );
            if ( c != null ) {
                return c;
            }
            InputStream in = null;
            try {
                in = getParent().getResourceAsStream ( name.replaceAll ( "\\.", "/" ) + ".class" );
                if ( in == null ) {
                    throw new ClassNotFoundException ( "Could not find " + name );
                }
                byte[] cBytes = toByteArray ( in );
                c = defineClass ( name, cBytes, 0, cBytes.length );
                if ( resolve ) {
                    resolveClass ( c );
                }
                if ( listener != null ) {
                    listener.classLoaded ( c );
                }
                return c;
            } catch ( IOException e ) {
                throw new ClassNotFoundException ( "Could not load " + name, e );
            } finally {
                closeQuietly ( in );
            }
        }
    }
}
