public class ParallelizedParameterized extends Parameterized {
    private static class ThreadPoolScheduler implements RunnerScheduler {
        private ExecutorService executor;
        public ThreadPoolScheduler() {
            String threads = System.getProperty ( "junit.parallel.threads", "16" );
            int numThreads = Integer.parseInt ( threads );
            executor = Executors.newFixedThreadPool ( numThreads );
        }
        @Override
        public void finished() {
            executor.shutdown();
            try {
                executor.awaitTermination ( 10, TimeUnit.MINUTES );
            } catch ( InterruptedException exc ) {
                throw new RuntimeException ( exc );
            }
        }
        @Override
        public void schedule ( Runnable childStatement ) {
            executor.submit ( childStatement );
        }
    }
    public ParallelizedParameterized ( Class klass ) throws Throwable {
        super ( klass );
        setScheduler ( new ThreadPoolScheduler() );
    }
}
