void xor ( uint16_t * data, uint16_t * data2, size_t count, uint16_t * out ) {
    uint8_t i;
    for ( i = 0; i < count; i++ ) {
        out[i] = data[i] ^ data2[i];
    }
}
+ ( NSMutableData * ) incrementCtrWithIv: ( NSData * ) iv increment: ( uint16_t ) increment {
    NSMutableData * ctrBytes = [[NSMutableData alloc] initWithCapacity:kCCBlockSizeAES128];
    NSMutableArray * arrayOfBytes = [[NSMutableArray alloc] initWithCapacity:8];
    bool carry = true;
    for ( int k = 7; k >= 0 ; k-- ) {
        uint16_t ctr = 0;
        [iv getBytes: & ctr range:NSMakeRange ( ( NSUInteger ) ( 8 + k ), 1 )];
        if ( carry ) {
            if ( ( ctr + increment ) >  255 ) {
                ctr = ( uint16_t ) ( ctr + increment ) % 256;
                increment -= ctr;
            }  else {
                ctr += increment;
                carry = false;
            }
        }
        [arrayOfBytes addObject:[NSData dataWithBytes: & ctr length:1]];
    }
    for ( int k = 7; k >= 0; k-- ) {
        [ctrBytes appendData:[arrayOfBytes objectAtIndex: ( NSUInteger ) k]];
    }
    return ctrBytes;
}
+ ( NSData * ) symmetricCipher: ( NSData * ) data key: ( NSData * ) symmetricKey context: ( CCOperation ) encryptOrDecrypt {
    NSMutableData * cipherOrPlainText = [[NSMutableData alloc] initWithCapacity: ( [data length] + kCCBlockSizeAES128 )];
    NSData * iv = [[data subdataWithRange:NSMakeRange ( 0, kCCBlockSizeAES128 )] copy];
    NSUInteger dataLength = data.length;
    uint8_t start = 0;
    if ( encryptOrDecrypt == kCCDecrypt ) {
        start += kCCBlockSizeAES128;
        dataLength -= kCCBlockSizeAES128;
    } else {
        iv = [self generateIv];
    }
    NSData * incrementalIv = [iv copy];
    size_t blocks = ( ( dataLength + kCCBlockSizeAES128 ) / kCCBlockSizeAES128 );
    for ( uint16_t j = 0; j < blocks; j++ ) {
        size_t realLengthRemaining = dataLength - j * kCCBlockSizeAES128;
        size_t actualLength = ( size_t ) ( realLengthRemaining >  kCCBlockSizeAES128 ? kCCBlockSizeAES128 : realLengthRemaining );
        if ( realLengthRemaining > 0 ) {
            if ( 0 != j ) {
                NSData * ctrBytes = [Crypto incrementCtrWithIv:incrementalIv increment:1];
                NSMutableData * ctrIv = [[iv subdataWithRange:NSMakeRange ( 0, 8 )] mutableCopy];
                [ctrIv appendData:ctrBytes];
                incrementalIv = ctrIv;
            }
            CCCryptorRef cryptor = NULL;
            uint8_t * bufferPtr = NULL;
            size_t remainingBytes = 0;
            size_t movedBytes = 0;
            size_t bufferPtrSize = 0;
            size_t totalBytesWritten = 0;
            LOGGING_FACILITY ( data != nil, @"PlainText object cannot be nil." );
            LOGGING_FACILITY ( symmetricKey != nil, @"Symmetric key object cannot be nil." );
            LOGGING_FACILITY ( [symmetricKey length] == kCCKeySizeAES256, @"Disjoint choices for key size." );
            CCCryptorStatus status = CCCryptorCreateWithMode ( kCCEncrypt, kCCModeECB, kCCAlgorithmAES128, ccNoPadding,
                                     nil, [symmetricKey bytes], kCCKeySizeAES256, NULL, 0, 0, 0, & cryptor );
            LOGGING_FACILITY1 ( status == kCCSuccess, @"Problem creating the context, status == %d.", status );
            bufferPtrSize = CCCryptorGetOutputLength ( cryptor, kCCBlockSizeAES128, true );
            bufferPtr = malloc ( bufferPtrSize * sizeof ( uint8_t ) );
            memset ( ( void * ) bufferPtr, 0x0, bufferPtrSize );
            uint8_t * ptr = bufferPtr;
            remainingBytes = bufferPtrSize;
            status = CCCryptorUpdate ( cryptor,
                                       [incrementalIv bytes],
                                       kCCBlockSizeAES128,
                                       ptr,
                                       remainingBytes,
                                       & movedBytes
                                     );
            LOGGING_FACILITY1 ( status == kCCSuccess, @"Problem with CCCryptorUpdate, status == %d.", status );
            ptr += movedBytes;
            remainingBytes -= movedBytes;
            totalBytesWritten += movedBytes;
            status = CCCryptorFinal ( cryptor,
                                      ptr,
                                      remainingBytes,
                                      & movedBytes
                                    );
            totalBytesWritten += movedBytes;
            if ( cryptor ) {
                ( void ) CCCryptorRelease ( cryptor );
                cryptor = NULL;
            }
            NSData * out = [NSData dataWithBytes:bufferPtr length: ( NSUInteger ) totalBytesWritten];
            if ( bufferPtr ) {
                free ( bufferPtr );
            }
            NSData * input = [data subdataWithRange:NSMakeRange ( start + j * kCCBlockSizeAES128, actualLength )];
            uint16_t xord[kCCBlockSizeAES128] = {0};
            xor ( [input bytes], [out bytes], actualLength, xord );
            LOGGING_FACILITY1 ( status == kCCSuccess, @"Problem with encipherment status == %d", status );
            NSData * blockData = [NSData dataWithBytes:xord length: ( NSUInteger ) actualLength];
            NSString * decryptedText = [[NSString alloc] initWithData:blockData encoding:NSASCIIStringEncoding];
            [cipherOrPlainText appendData:blockData];
            [self wipeData:blockData];
        }
    }
    if ( encryptOrDecrypt == kCCEncrypt ) {
        NSMutableData * cipherText = [[NSMutableData alloc] initWithData:iv];
        [cipherText appendData:cipherOrPlainText];
        cipherOrPlainText = cipherText;
    }
    [self wipeData:iv];
    [self wipeData:incrementalIv];
    return cipherOrPlainText;
}
+ ( void ) wipeData: ( NSData * ) data {
    memset ( [data bytes], 0, [data length] );
}
