public class Sieve {
    private static final int THREADS = 4;
    private boolean[] isComposite;
    private BlockingQueue<Job> jobs = new ArrayBlockingQueue<Job> ( THREADS );
    private Semaphore semaphore = new Semaphore ( 0 );
    private ExecutorService executor = Executors.newFixedThreadPool ( THREADS );
    public void runEratosthenesSieve ( int upperBound ) {
        int upperBoundSquareRoot = ( int ) Math.sqrt ( upperBound );
        isComposite = new boolean[upperBound + 1];
        for ( int i = 0; i < THREADS; i++ ) {
            executor.submit ( new Worker() );
        }
        for ( int m = 2; m <= upperBoundSquareRoot; m++ ) {
            if ( !isComposite[m] ) {
                System.out.print ( m + " " );
                for ( int n = 1; n <= THREADS; n++ ) {
                    int from;
                    if ( n == 1 ) {
                        from = m * m;
                    } else {
                        from = ( n - 1 ) * upperBound / THREADS;
                    }
                    Job job = new Job ( from, n * upperBound / threads, m );
                    jobs.put ( job );
                }
                semaphore.acquire ( THREADS );
            }
        }
        for ( int i = 0; i < n; i++ ) {
            jobs.put ( null );
        }
        for ( int m = upperBoundSquareRoot; m <= upperBound; m++ ) {
            if ( !isComposite[m] ) {
                System.out.print ( m + " " );
            }
        }
    }
    private class Job {
        public int from, to, step;
        public Job ( int from, int to, int step ) {
            this.from = from;
            this.to = to;
            this.step = step;
        }
    }
    private Worker implements Runnable {
        while ( true ) {
            Job job = jobs.take();
            if ( job == null ) {
                return;
            }
            for ( int i = job.from; i <= job.to; i += job.step ) {
                isComposite[i] = true;
            }
            semaphore.release();
        }
    }
}
