import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.EventQueue;
import java.awt.Font;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.MouseEvent;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.event.MouseInputAdapter;
public class Main extends JFrame {
    private JTextArea textArea;
    private JScrollPane textAreaScroll;
    private JPanel glassPane;
    public Main() {
        initComponents();
        initListeners();
        for ( int i = 0; i < 20; i++ ) {
            addLine ( i + " Hello" );
        }
    }
    public void addLine ( String line ) {
        textArea.append ( "\n> " + line );
        textArea.setCaretPosition ( textArea.getDocument().getLength() );
    }
    private void initListeners() {
        GlassPaneListener gpl = new GlassPaneListener ( textAreaScroll.getVerticalScrollBar(), this );
        glassPane.addMouseMotionListener ( gpl );
        glassPane.addMouseListener ( gpl );
    }
    private void initComponents() {
        try {
            UIManager.setLookAndFeel ( UIManager.getSystemLookAndFeelClassName() );
        } catch ( UnsupportedLookAndFeelException | ClassNotFoundException | InstantiationException | IllegalAccessException e ) {
        }
        textAreaScroll = new JScrollPane();
        textArea = new JTextArea();
        setDefaultCloseOperation ( javax.swing.WindowConstants.EXIT_ON_CLOSE );
        setUndecorated ( true );
        setAlwaysOnTop ( true );
        setAutoRequestFocus ( false );
        setBackground ( new Color ( 130, 210, 255, 130 ) );
        setFocusCycleRoot ( false );
        setFocusable ( false );
        setFocusableWindowState ( false );
        setName ( "main" );
        setOpacity ( 0.4f );
        setResizable ( false );
        textAreaScroll.setBorder ( null );
        textAreaScroll.setFocusable ( false );
        textAreaScroll.setRequestFocusEnabled ( false );
        textArea.setEditable ( false );
        textArea.setBackground ( new Color ( 0, 0, 0 ) );
        textArea.setColumns ( 20 );
        textArea.setFont ( new Font ( "Consolas", 0, 14 ) );
        textArea.setForeground ( new Color ( 255, 255, 255 ) );
        textArea.setLineWrap ( true );
        textArea.setRows ( 5 );
        textArea.setText ( "> Hello world!\n> another line!" );
        textArea.setBorder ( null );
        textArea.setFocusable ( false );
        textArea.setRequestFocusEnabled ( false );
        textAreaScroll.setViewportView ( textArea );
        textAreaScroll.setPreferredSize ( new Dimension ( 200, 200 ) );
        javax.swing.GroupLayout layout = new javax.swing.GroupLayout ( getContentPane() );
        getContentPane().setLayout ( layout );
        layout.setHorizontalGroup (
            layout.createParallelGroup ( javax.swing.GroupLayout.Alignment.LEADING )
            .addComponent ( textAreaScroll, javax.swing.GroupLayout.DEFAULT_SIZE, 400, Short.MAX_VALUE ) );
        layout.setVerticalGroup (
            layout.createParallelGroup ( javax.swing.GroupLayout.Alignment.LEADING )
            .addComponent ( textAreaScroll, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 214, Short.MAX_VALUE ) );
        glassPane = new JPanel();
        glassPane.setOpaque ( false );
        setGlassPane ( glassPane );
        pack();
        setVisible ( true );
        glassPane.setVisible ( true );
    }
    public static void main ( String args[] ) {
        EventQueue.invokeLater ( new Runnable() {
            @Override
            public void run() {
                new Main();
            }
        } );
    }
}
class GlassPaneListener extends MouseInputAdapter {
    private int posX = 0;
    private int posY = 0;
    Toolkit toolkit;
    private final Container contentPane;
    private final Component textAreaScroll;
    private final Component glassPane;
    private final JFrame frame;
    private boolean wasClickOnInterestedComponent = false;
    public GlassPaneListener ( Component textAreaScroll, JFrame frame ) {
        toolkit = Toolkit.getDefaultToolkit();
        this.textAreaScroll = textAreaScroll;
        this.frame = frame;
        this.glassPane = frame.getGlassPane();
        this.contentPane = frame.getContentPane();
    }
    @Override
    public void mouseDragged ( MouseEvent e ) {
        if ( !redispatchMouseEvent ( e ) ) {
            frame.setLocation ( e.getXOnScreen() - posX, e.getYOnScreen() - posY );
        }
    }
    @Override
    public void mousePressed ( MouseEvent e ) {
        if ( !redispatchMouseEvent ( e ) ) {
            posX = e.getX();
            posY = e.getY();
        }
    }
    @Override
    public void mouseReleased ( MouseEvent me ) {
        wasClickOnInterestedComponent = false;
    }
    private boolean redispatchMouseEvent ( MouseEvent e ) {
        Point glassPanePoint = e.getPoint();
        Container container = contentPane;
        Point containerPoint = SwingUtilities.convertPoint ( glassPane, glassPanePoint, contentPane );
        Component component = SwingUtilities.getDeepestComponentAt ( container, containerPoint.x,
                              containerPoint.y );
        if ( ( component != null ) && ( component.equals ( textAreaScroll ) ) || wasClickOnInterestedComponent ) {
            wasClickOnInterestedComponent = true;
            Point componentPoint = SwingUtilities.convertPoint ( glassPane, glassPanePoint, component );
            component.dispatchEvent ( new MouseEvent ( component, e.getID(), e.getWhen(), e.getModifiers(),
                                      componentPoint.x, componentPoint.y, e.getClickCount(), e.isPopupTrigger() ) );
            return true;
        } else {
            return false;
        }
    }
}
