public class IncludeProjectDependenciesComponentConfigurator extends AbstractComponentConfigurator {
    public void configureComponent ( Object component, PlexusConfiguration configuration,
                                     ExpressionEvaluator expressionEvaluator, ClassRealm containerRealm,
                                     ConfigurationListener listener )
    throws ComponentConfigurationException {
        addProjectDependenciesToClassRealm ( expressionEvaluator, containerRealm );
        converterLookup.registerConverter ( new ClassRealmConverter ( containerRealm ) );
        ObjectWithFieldsConverter converter = new ObjectWithFieldsConverter();
        converter.processConfiguration ( converterLookup, component, containerRealm.getClassLoader(), configuration,
                                         expressionEvaluator, listener );
    }
    private void addProjectDependenciesToClassRealm ( ExpressionEvaluator expressionEvaluator, ClassRealm containerRealm ) throws ComponentConfigurationException {
        List<String> runtimeClasspathElements;
        try {
            runtimeClasspathElements = ( List<String> ) expressionEvaluator.evaluate ( "${project.runtimeClasspathElements}" );
        } catch ( ExpressionEvaluationException e ) {
            throw new ComponentConfigurationException ( "There was a problem evaluating: ${project.runtimeClasspathElements}", e );
        }
        final URL[] urls = buildURLs ( runtimeClasspathElements );
        for ( URL url : urls ) {
            containerRealm.addConstituent ( url );
        }
    }
    private URL[] buildURLs ( List<String> runtimeClasspathElements ) throws ComponentConfigurationException {
        List<URL> urls = new ArrayList<URL> ( runtimeClasspathElements.size() );
        for ( String element : runtimeClasspathElements ) {
            try {
                final URL url = new File ( element ).toURI().toURL();
                urls.add ( url );
            } catch ( MalformedURLException e ) {
                throw new ComponentConfigurationException ( "Unable to access project dependency: " + element, e );
            }
        }
        return urls.toArray ( new URL[urls.size()] );
    }
}
