import java.awt.AWTEvent;
import java.awt.AWTEventMulticaster;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
public class Test {
    public Test() {
        initComponents();
    }
    private void initComponents() {
        final JFrame frame = new JFrame();
        frame.setDefaultCloseOperation ( JFrame.EXIT_ON_CLOSE );
        final JTextField tf = new JTextField ( "" );
        RoundedButton rb = new RoundedButton ( "Go" );
        rb.setBackground ( Color.yellow );
        rb.addActionListener ( new ActionListener() {
            @Override
            public void actionPerformed ( ActionEvent ae ) {
                JOptionPane.showMessageDialog ( frame, "You said: " + tf.getText() );
            }
        } );
        frame.add ( tf, BorderLayout.NORTH );
        frame.add ( rb );
        frame.pack();
        frame.setVisible ( true );
    }
    public static void main ( String[] args ) {
        SwingUtilities.invokeLater ( new Runnable() {
            @Override
            public void run() {
                new Test();
            }
        } );
    }
}
class RoundedButton extends Component {
    ActionListener actionListener;
    String label;
    protected boolean pressed = false;
    public RoundedButton() {
        this ( "" );
    }
    public RoundedButton ( String label ) {
        this.label = label;
        enableEvents ( AWTEvent.MOUSE_EVENT_MASK );
    }
    public String getLabel() {
        return label;
    }
    public void setLabel ( String label ) {
        this.label = label;
        invalidate();
        repaint();
    }
    @Override
    public void paint ( Graphics g ) {
        if ( pressed ) {
            g.setColor ( getBackground().darker().darker() );
        } else {
            g.setColor ( getBackground() );
        }
        g.fillRoundRect ( 0, 0, getWidth() - 1, getHeight() - 1, 20, 20 );
        g.setColor ( getBackground().darker().darker().darker() );
        g.drawRoundRect ( 0, 0, getWidth() - 1, getHeight() - 1, 20, 20 );
        Font f = getFont();
        if ( f != null ) {
            FontMetrics fm = getFontMetrics ( getFont() );
            g.setColor ( getForeground() );
            g.drawString ( label, getWidth() / 2 - fm.stringWidth ( label ) / 2, getHeight() / 2 + fm.getMaxDescent() );
        }
    }
    @Override
    public Dimension getPreferredSize() {
        Font f = getFont();
        if ( f != null ) {
            FontMetrics fm = getFontMetrics ( getFont() );
            int max = Math.max ( fm.stringWidth ( label ) + 40, fm.getHeight() + 40 );
            return new Dimension ( max, max );
        } else {
            return new Dimension ( 100, 100 );
        }
    }
    @Override
    public Dimension getMinimumSize() {
        return new Dimension ( 100, 100 );
    }
    public void addActionListener ( ActionListener listener ) {
        actionListener = AWTEventMulticaster.add ( actionListener, listener );
        enableEvents ( AWTEvent.MOUSE_EVENT_MASK );
    }
    public void removeActionListener ( ActionListener listener ) {
        actionListener = AWTEventMulticaster.remove ( actionListener, listener );
    }
    @Override
    public boolean contains ( int x, int y ) {
        int mx = getSize().width / 2;
        int my = getSize().height / 2;
        return ( ( ( mx - x ) * ( mx - x ) + ( my - y ) * ( my - y ) ) <= mx * mx );
    }
    @Override
    public void processMouseEvent ( MouseEvent e ) {
        Graphics g;
        switch ( e.getID() ) {
        case MouseEvent.MOUSE_PRESSED:
            pressed = true;
            repaint();
            break;
        case MouseEvent.MOUSE_RELEASED:
            if ( actionListener != null ) {
                actionListener.actionPerformed ( new ActionEvent (
                                                     this, ActionEvent.ACTION_PERFORMED, label ) );
            }
            if ( pressed == true ) {
                pressed = false;
                repaint();
            }
            break;
        case MouseEvent.MOUSE_ENTERED:
            break;
        case MouseEvent.MOUSE_EXITED:
            if ( pressed == true ) {
                pressed = false;
                repaint();
            }
            break;
        }
        super.processMouseEvent ( e );
    }
}
