import java.awt.Desktop;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.*;
import javax.swing.JTextField;
import javax.swing.JPanel;
import javax.swing.JOptionPane;
import javax.swing.border.MatteBorder;
import javax.swing.border.Border;
import java.net.URI;
import java.io.File;
public class LinkLabel
    extends JTextField
    implements MouseListener, FocusListener, ActionListener {
    private URI target;
    public Color standardColor = new Color ( 0, 0, 255 );
    public Color hoverColor = new Color ( 255, 0, 0 );
    public Color activeColor = new Color ( 128, 0, 128 );
    public Color transparent = new Color ( 0, 0, 0, 0 );
    public boolean underlineVisible = true;
    private Border activeBorder;
    private Border hoverBorder;
    private Border standardBorder;
    public LinkLabel ( URI target ) {
        this ( target, target.toString() );
    }
    public LinkLabel ( URI target, String text ) {
        super ( text );
        this.target = target;
    }
    public void setActiveColor ( Color active ) {
        activeColor = active;
    }
    public void setHoverColor ( Color hover ) {
        hoverColor = hover;
    }
    public void setStandardColor ( Color standard ) {
        standardColor = standard;
    }
    public void setUnderlineVisible ( boolean underlineVisible ) {
        this.underlineVisible = underlineVisible;
    }
    public void init() {
        this.addMouseListener ( this );
        this.addFocusListener ( this );
        this.addActionListener ( this );
        setToolTipText ( target.toString() );
        if ( underlineVisible ) {
            activeBorder = new MatteBorder ( 0, 0, 1, 0, activeColor );
            hoverBorder = new MatteBorder ( 0, 0, 1, 0, hoverColor );
            standardBorder = new MatteBorder ( 0, 0, 1, 0, transparent );
        } else {
            activeBorder = new MatteBorder ( 0, 0, 0, 0, activeColor );
            hoverBorder = new MatteBorder ( 0, 0, 0, 0, hoverColor );
            standardBorder = new MatteBorder ( 0, 0, 0, 0, transparent );
        }
        setEditable ( false );
        setForeground ( standardColor );
        setBorder ( standardBorder );
        setCursor ( new Cursor ( Cursor.HAND_CURSOR ) );
    }
    public void browse() {
        setForeground ( activeColor );
        setBorder ( activeBorder );
        try {
            Desktop.getDesktop().browse ( target );
        } catch ( Exception e ) {
            e.printStackTrace();
        }
        setForeground ( standardColor );
        setBorder ( standardBorder );
    }
    public void actionPerformed ( ActionEvent ae ) {
        browse();
    }
    public void mouseClicked ( MouseEvent me ) {
        browse();
    }
    public void mouseEntered ( MouseEvent me ) {
        setForeground ( hoverColor );
        setBorder ( hoverBorder );
    }
    public void mouseExited ( MouseEvent me ) {
        setForeground ( standardColor );
        setBorder ( standardBorder );
    }
    public void mouseReleased ( MouseEvent me ) {}
    public void mousePressed ( MouseEvent me ) {}
    public void focusLost ( FocusEvent fe ) {
        setForeground ( standardColor );
        setBorder ( standardBorder );
    }
    public void focusGained ( FocusEvent fe ) {
        setForeground ( hoverColor );
        setBorder ( hoverBorder );
    }
    public static void main ( String[] args ) throws Exception {
        JPanel p = new JPanel ( new GridLayout ( 0, 1 ) );
        File f = new File ( ".", "LinkLabel.java" );
        f = new File ( f.getCanonicalPath() );
        URI uriFile = f.toURI();
        LinkLabel linkLabelFile = new LinkLabel ( uriFile );
        linkLabelFile.init();
        p.add ( linkLabelFile );
        LinkLabel linkLabelWeb = new LinkLabel (
            new URI ( "http://sscce.org/" ),
            "SSCCE" );
        linkLabelWeb.setStandardColor ( new Color ( 0, 128, 0 ) );
        linkLabelWeb.setHoverColor ( new Color ( 222, 128, 0 ) );
        linkLabelWeb.init();
        p.add ( linkLabelWeb );
        LinkLabel linkLabelConstrain = new LinkLabel (
            new URI ( "http://sdnshare.sun.com/" ),
            "SDN Share" );
        linkLabelConstrain.init();
        JPanel labelConstrain = new JPanel ( new BorderLayout() );
        labelConstrain.add ( linkLabelConstrain, BorderLayout.EAST );
        p.add ( labelConstrain );
        LinkLabel linkLabelNoUnderline = new LinkLabel (
            new URI ( "http://java.net/" ),
            "java.net" );
        linkLabelNoUnderline.setUnderlineVisible ( false );
        linkLabelNoUnderline.
        setHorizontalAlignment ( JTextField.CENTER );
        linkLabelNoUnderline.init();
        p.add ( linkLabelNoUnderline );
        JOptionPane.showMessageDialog ( null, p );
    }
}
