public class DropAjaxBehavior extends AbstractAjaxBehavior {
    private static final Logger LOG = LoggerFactory.getLogger ( DropAjaxBehavior.class );
    @Override
    public final void onRequest() {
        LOG.debug ( "Received request" );
        final RequestCycle requestCycle = RequestCycle.get();
        processRequest ( requestCycle );
        sendResponse ( requestCycle );
    }
    private void processRequest ( RequestCycle requestCycle ) {
        final WebRequest wr = ( WebRequest ) requestCycle.getRequest();
        final HttpServletRequest hsr = ( HttpServletRequest ) wr.getContainerRequest();
        try {
            final byte[] data = new byte[hsr.getContentLength()];
            ByteStreams.readFully ( hsr.getInputStream(), data );
            final String[] base64Data = new String ( data ).split ( ";" );
            final String fileName = base64Data[0].substring ( base64Data[0].indexOf ( ':' ) + 1, base64Data[0].length() );
            final String dataType = base64Data[1].substring ( base64Data[1].indexOf ( ':' ) + 1, base64Data[1].length() );
            final String binaryData = base64Data[2].substring ( base64Data[2].indexOf ( ',' ) + 1, base64Data[2].length() );
            if ( dataType.contains ( "image" ) ) {
                final byte[] image = DatatypeConverter.parseBase64Binary ( binaryData );
                DatabaseQuery.addImage ( image, fileName );
            }
        } catch ( IOException ioe ) {
            LOG.error ( "IO error while reading HttpServletRequest: ", ioe );
        }
    }
    private void sendResponse ( RequestCycle requestCycle ) {
        requestCycle.scheduleRequestHandlerAfterCurrent ( new TextRequestHandler ( "text/html", "UTF-8", "done" ) );
    }
    @Override
    protected final void onComponentTag ( ComponentTag tag ) {
        tag.put ( "my:dropcontainer.callback", getCallbackUrl().toString() );
    }
}
