public class GameLogic {
    public GameLogic() {
        initComponents();
    }
    private void initComponents() {
        JFrame frame = new JFrame ( "Game Test" );
        frame.setDefaultCloseOperation ( JFrame.EXIT_ON_CLOSE );
        final GamePanel gp = new GamePanel ( 600, 500 );
        JPanel buttonPanel = new JPanel();
        buttonPanel.setOpaque ( false );
        final JButton startButton = new JButton ( "Start" );
        final JButton pauseButton = new JButton ( "Pause" );
        final JButton stopButton = new JButton ( "Stop" );
        pauseButton.setEnabled ( false );
        stopButton.setEnabled ( false );
        startButton.addActionListener ( new ActionListener() {
            @Override
            public void actionPerformed ( ActionEvent ae ) {
                gp.clearEntities();
                ArrayList<BufferedImage> ballEntityImages = new ArrayList<>();
                ArrayList<Long> ballEntityTimings = new ArrayList<>();
                ballEntityImages.add ( createColouredImage ( "white", 50, 50, true ) );
                ballEntityTimings.add ( 500l );
                Entity ballEntity = new Entity ( gp.getWidth() / 2, gp.getHeight() / 2, ballEntityImages, ballEntityTimings );
                ballEntity.RIGHT = true;
                ArrayList<BufferedImage> advEntityImages = new ArrayList<>();
                ArrayList<Long> advEntityTimings = new ArrayList<>();
                advEntityImages.add ( createColouredImage ( "orange", 10, 100, false ) );
                advEntityTimings.add ( 500l );
                advEntityImages.add ( createColouredImage ( "blue", 10, 100, false ) );
                advEntityTimings.add ( 500l );
                AdvancedSpritesEntity player1Entity = new AdvancedSpritesEntity ( 0, 100, advEntityImages, advEntityTimings );
                ArrayList<BufferedImage> entityImages = new ArrayList<>();
                ArrayList<Long> entityTimings = new ArrayList<>();
                entityImages.add ( createColouredImage ( "red", 10, 100, false ) );
                entityTimings.add ( 500l );
                Entity player2Entity = new Entity ( gp.getWidth() - 10, 200, entityImages, entityTimings );
                gp.addEntity ( player1Entity );
                gp.addEntity ( player2Entity );
                gp.addEntity ( ballEntity );
                GameKeyBindings gameKeyBindings = new GameKeyBindings ( gp, player1Entity, player2Entity );
                GamePanel.running.set ( true );
                runGameLoop ( gp );
                startButton.setEnabled ( false );
                pauseButton.setEnabled ( true );
                stopButton.setEnabled ( true );
            }
        } );
        pauseButton.addActionListener ( new ActionListener() {
            @Override
            public void actionPerformed ( ActionEvent ae ) {
                boolean b = GamePanel.paused.get();
                GamePanel.paused.set ( !b );
                if ( pauseButton.getText().equals ( "Pause" ) ) {
                    pauseButton.setText ( "Un-pause" );
                } else {
                    pauseButton.setText ( "Pause" );
                }
            }
        } );
        stopButton.addActionListener ( new ActionListener() {
            @Override
            public void actionPerformed ( ActionEvent ae ) {
                GamePanel.running.set ( false );
                GamePanel.paused.set ( false );
                if ( !pauseButton.getText().equals ( "Pause" ) ) {
                    pauseButton.setText ( "Pause" );
                }
                startButton.setEnabled ( true );
                pauseButton.setEnabled ( false );
                stopButton.setEnabled ( false );
            }
        } );
        buttonPanel.add ( startButton );
        buttonPanel.add ( pauseButton );
        buttonPanel.add ( stopButton );
        frame.add ( gp );
        frame.add ( buttonPanel, BorderLayout.SOUTH );
        frame.pack();
        frame.setLocationRelativeTo ( null );
        frame.setVisible ( true );
    }
    public static BufferedImage createColouredImage ( String color, int w, int h, boolean circular ) {
        BufferedImage img = new BufferedImage ( w, h, BufferedImage.TYPE_INT_ARGB );
        Graphics2D g2 = img.createGraphics();
        switch ( color.toLowerCase() ) {
        case "green":
            g2.setColor ( Color.GREEN );
            break;
        case "magenta":
            g2.setColor ( Color.MAGENTA );
            break;
        case "red":
            g2.setColor ( Color.RED );
            break;
        case "yellow":
            g2.setColor ( Color.YELLOW );
            break;
        case "blue":
            g2.setColor ( Color.BLUE );
            break;
        case "orange":
            g2.setColor ( Color.ORANGE );
            break;
        case "cyan":
            g2.setColor ( Color.CYAN );
            break;
        case "gray":
            g2.setColor ( Color.GRAY );
            break;
        default:
            g2.setColor ( Color.WHITE );
            break;
        }
        if ( !circular ) {
            g2.fillRect ( 0, 0, img.getWidth(), img.getHeight() );
        } else {
            g2.fillOval ( 0, 0, img.getWidth(), img.getHeight() );
        }
        g2.dispose();
        return img;
    }
    private void runGameLoop ( final GamePanel gp ) {
        Thread loop = new Thread ( new Runnable() {
            @Override
            public void run() {
                gp.gameLoop();
            }
        } );
        loop.start();
    }
    public static void main ( String[] args ) {
        SwingUtilities.invokeLater ( new Runnable() {
            @Override
            public void run() {
                try {
                    for ( UIManager.LookAndFeelInfo info : UIManager.getInstalledLookAndFeels() ) {
                        if ( "Nimbus".equals ( info.getName() ) ) {
                            UIManager.setLookAndFeel ( info.getClassName() );
                            break;
                        }
                    }
                } catch ( ClassNotFoundException | InstantiationException | IllegalAccessException | UnsupportedLookAndFeelException ex ) {
                }
                GameLogic gameLogic = new GameLogic();
            }
        } );
    }
}
class GameKeyBindings {
    public GameKeyBindings ( JComponent gp, final Entity entity, final Entity entity2 ) {
        gp.getInputMap ( JComponent.WHEN_IN_FOCUSED_WINDOW ).put ( KeyStroke.getKeyStroke ( KeyEvent.VK_W, 0, false ), "W pressed" );
        gp.getActionMap().put ( "W pressed", new AbstractAction() {
            @Override
            public void actionPerformed ( ActionEvent ae ) {
                entity.UP = true;
            }
        } );
        gp.getInputMap ( JComponent.WHEN_IN_FOCUSED_WINDOW ).put ( KeyStroke.getKeyStroke ( KeyEvent.VK_W, 0, true ), "W released" );
        gp.getActionMap().put ( "W released", new AbstractAction() {
            @Override
            public void actionPerformed ( ActionEvent ae ) {
                entity.UP = false;
            }
        } );
        gp.getInputMap ( JComponent.WHEN_IN_FOCUSED_WINDOW ).put ( KeyStroke.getKeyStroke ( KeyEvent.VK_S, 0, false ), "S pressed" );
        gp.getActionMap().put ( "S pressed", new AbstractAction() {
            @Override
            public void actionPerformed ( ActionEvent ae ) {
                entity.DOWN = true;
            }
        } );
        gp.getInputMap ( JComponent.WHEN_IN_FOCUSED_WINDOW ).put ( KeyStroke.getKeyStroke ( KeyEvent.VK_S, 0, true ), "S released" );
        gp.getActionMap().put ( "S released", new AbstractAction() {
            @Override
            public void actionPerformed ( ActionEvent ae ) {
                entity.DOWN = false;
            }
        } );
        gp.getInputMap ( JComponent.WHEN_IN_FOCUSED_WINDOW ).put ( KeyStroke.getKeyStroke ( KeyEvent.VK_DOWN, 0, false ), "down pressed" );
        gp.getActionMap().put ( "down pressed", new AbstractAction() {
            @Override
            public void actionPerformed ( ActionEvent ae ) {
                entity2.DOWN = true;
            }
        } );
        gp.getInputMap ( JComponent.WHEN_IN_FOCUSED_WINDOW ).put ( KeyStroke.getKeyStroke ( KeyEvent.VK_DOWN, 0, true ), "down released" );
        gp.getActionMap().put ( "down released", new AbstractAction() {
            @Override
            public void actionPerformed ( ActionEvent ae ) {
                entity2.DOWN = false;
            }
        } );
        gp.getInputMap ( JComponent.WHEN_IN_FOCUSED_WINDOW ).put ( KeyStroke.getKeyStroke ( KeyEvent.VK_UP, 0, false ), "up pressed" );
        gp.getActionMap().put ( "up pressed", new AbstractAction() {
            @Override
            public void actionPerformed ( ActionEvent ae ) {
                entity2.UP = true;
            }
        } );
        gp.getInputMap ( JComponent.WHEN_IN_FOCUSED_WINDOW ).put ( KeyStroke.getKeyStroke ( KeyEvent.VK_UP, 0, true ), "up released" );
        gp.getActionMap().put ( "up released", new AbstractAction() {
            @Override
            public void actionPerformed ( ActionEvent ae ) {
                entity2.UP = false;
            }
        } );
    }
}
class AdvancedSpritesEntity extends Entity {
    public AdvancedSpritesEntity ( int x, int y, ArrayList<BufferedImage> images, ArrayList<Long> timings ) {
        super ( x, y, images, timings );
    }
    void setAnimation ( ArrayList<BufferedImage> images, ArrayList<Long> timings ) {
        reset();
        setFrames ( images, timings );
    }
}
class Entity extends Animator {
    private int speed = 5;
    public boolean UP = false, DOWN = false, LEFT = false, RIGHT = false, visible;
    private Rectangle2D.Double rect;
    public Entity ( int x, int y, ArrayList<BufferedImage> images, ArrayList<Long> timings ) {
        super ( images, timings );
        UP = false;
        DOWN = false;
        LEFT = false;
        RIGHT = false;
        visible = true;
        rect = new Rectangle2D.Double ( x, y, getCurrentImage().getWidth(), getCurrentImage().getHeight() );
    }
    public void setSpeed ( int speed ) {
        this.speed = speed;
    }
    public int getSpeed() {
        return speed;
    }
    public boolean isVisible() {
        return visible;
    }
    public void setVisible ( boolean visible ) {
        this.visible = visible;
    }
    public HashSet<String> getMask ( Entity e ) {
        HashSet<String> mask = new HashSet<>();
        int pixel, a;
        BufferedImage bi = e.getCurrentImage();
        for ( int i = 0; i < bi.getWidth(); i++ ) {
            for ( int j = 0; j < bi.getHeight(); j++ ) {
                pixel = bi.getRGB ( i, j );
                a = ( pixel >> 24 ) & 0xff;
                if ( a != 0 ) {
                    mask.add ( ( e.getX() + i ) + "," + ( e.getY() - j ) );
                }
            }
        }
        return mask;
    }
    public boolean checkPerPixelCollision ( Entity b ) {
        int ax1 = ( int ) getX();
        int ay1 = ( int ) getY();
        int ax2 = ax1 + ( int ) getWidth();
        int ay2 = ay1 + ( int ) getHeight();
        int bx1 = ( int ) b.getX();
        int by1 = ( int ) b.getY();
        int bx2 = bx1 + ( int ) b.getWidth();
        int by2 = by1 + ( int ) b.getHeight();
        if ( by2 < ay1 || ay2 < by1 || bx2 < ax1 || ax2 < bx1 ) {
            return false;
        } else {
            HashSet<String> maskPlayer1 = getMask ( this );
            HashSet<String> maskPlayer2 = getMask ( b );
            maskPlayer1.retainAll ( maskPlayer2 );
            if ( maskPlayer1.size() > 0 ) {
                return true;
            }
        }
        return false;
    }
    public void move() {
        if ( UP ) {
            rect.y -= speed;
        }
        if ( DOWN ) {
            rect.y += speed;
        }
        if ( LEFT ) {
            rect.x -= speed;
        }
        if ( RIGHT ) {
            rect.x += speed;
        }
    }
    @Override
    public void update ( long elapsedTime ) {
        super.update ( elapsedTime );
        getWidth();
        getHeight();
    }
    public boolean intersects ( Entity e ) {
        return rect.intersects ( e.rect );
    }
    public double getX() {
        return rect.x;
    }
    public double getY() {
        return rect.y;
    }
    public double getWidth() {
        if ( getCurrentImage() == null ) {
            return rect.width = 0;
        }
        return rect.width = getCurrentImage().getWidth();
    }
    public double getHeight() {
        if ( getCurrentImage() == null ) {
            return rect.height = 0;
        }
        return rect.height = getCurrentImage().getHeight();
    }
}
class Animator {
    private ArrayList<BufferedImage> frames;
    private ArrayList<Long> timings;
    private int currIndex;
    private long animationTime;
    private long totalAnimationDuration;
    private AtomicBoolean done;
    public Animator ( ArrayList<BufferedImage> frames, ArrayList< Long> timings ) {
        currIndex = 0;
        animationTime = 0;
        totalAnimationDuration = 0;
        done = new AtomicBoolean ( false );
        this.frames = new ArrayList<>();
        this.timings = new ArrayList<>();
        setFrames ( frames, timings );
    }
    public boolean isDone() {
        return done.get();
    }
    public void reset() {
        totalAnimationDuration = 0;
        done.getAndSet ( false );
    }
    public void update ( long elapsedTime ) {
        if ( frames.size() > 1 ) {
            animationTime += elapsedTime;
            if ( animationTime >= totalAnimationDuration ) {
                animationTime = animationTime % totalAnimationDuration;
                currIndex = 0;
                done.getAndSet ( true );
            }
            while ( animationTime > timings.get ( currIndex ) ) {
                currIndex++;
            }
        }
    }
    public BufferedImage getCurrentImage() {
        if ( frames.isEmpty() ) {
            return null;
        } else {
            try {
                return frames.get ( currIndex );
            } catch ( Exception ex ) {
                currIndex = 0;
                return frames.get ( currIndex );
            }
        }
    }
    public void setFrames ( ArrayList<BufferedImage> frames, ArrayList< Long> timings ) {
        if ( frames == null || timings == null ) {
            return;
        }
        this.frames = frames;
        this.timings.clear();
        for ( long animTime : timings ) {
            totalAnimationDuration += animTime;
            this.timings.add ( totalAnimationDuration );
        }
    }
}
class GamePanel extends JPanel {
    private final static RenderingHints textRenderHints = new RenderingHints ( RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON );
    private final static RenderingHints imageRenderHints = new RenderingHints ( RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON );
    private final static RenderingHints colorRenderHints = new RenderingHints ( RenderingHints.KEY_COLOR_RENDERING, RenderingHints.VALUE_COLOR_RENDER_QUALITY );
    private final static RenderingHints interpolationRenderHints = new RenderingHints ( RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BICUBIC );
    private final static RenderingHints renderHints = new RenderingHints ( RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY );
    public static AtomicBoolean running = new AtomicBoolean ( false ), paused = new AtomicBoolean ( false );
    private int width, height, frameCount = 0, fps = 0;
    private ArrayList<Entity> entities = new ArrayList<>();
    private final Random random = new Random();
    public GamePanel ( int w, int h ) {
        super ( true );
        setIgnoreRepaint ( true );
        width = w;
        height = h;
    }
    @Override
    public Dimension getPreferredSize() {
        return new Dimension ( width, height );
    }
    public void addEntity ( Entity e ) {
        entities.add ( e );
    }
    void clearEntities() {
        entities.clear();
    }
    public void gameLoop() {
        final double GAME_HERTZ = 30.0;
        final double TIME_BETWEEN_UPDATES = 1000000000 / GAME_HERTZ;
        final int MAX_UPDATES_BEFORE_RENDER = 5;
        double lastUpdateTime = System.nanoTime();
        double lastRenderTime = System.nanoTime();
        final double TARGET_FPS = 60;
        final double TARGET_TIME_BETWEEN_RENDERS = 1000000000 / TARGET_FPS;
        int lastSecondTime = ( int ) ( lastUpdateTime / 1000000000 );
        long currTime = System.currentTimeMillis();
        while ( running.get() ) {
            if ( !paused.get() ) {
                double now = System.nanoTime();
                long elapsedTime = System.currentTimeMillis() - currTime;
                currTime += elapsedTime;
                int updateCount = 0;
                while ( now - lastUpdateTime > TIME_BETWEEN_UPDATES && updateCount < MAX_UPDATES_BEFORE_RENDER ) {
                    updateGame ( elapsedTime );
                    lastUpdateTime += TIME_BETWEEN_UPDATES;
                    updateCount++;
                }
                if ( now - lastUpdateTime > TIME_BETWEEN_UPDATES ) {
                    lastUpdateTime = now - TIME_BETWEEN_UPDATES;
                }
                drawGame();
                lastRenderTime = now;
                int thisSecond = ( int ) ( lastUpdateTime / 1000000000 );
                if ( thisSecond > lastSecondTime ) {
                    fps = frameCount;
                    frameCount = 0;
                    lastSecondTime = thisSecond;
                }
                while ( now - lastRenderTime < TARGET_TIME_BETWEEN_RENDERS && now - lastUpdateTime < TIME_BETWEEN_UPDATES ) {
                    Thread.yield();
                    try {
                        Thread.sleep ( 1 );
                    } catch ( Exception e ) {
                    }
                    now = System.nanoTime();
                }
            }
        }
        fps = 0;
    }
    private void drawGame() {
        repaint();
    }
    private void updateGame ( long elapsedTime ) {
        updateEntityMovements ( elapsedTime );
        checkForCollisions();
    }
    private void checkForCollisions() {
        if ( entities.get ( 0 ).intersects ( entities.get ( 2 ) ) || entities.get ( 1 ).intersects ( entities.get ( 2 ) ) ) {
            if ( entities.get ( 2 ).LEFT ) {
                entities.get ( 2 ).RIGHT = true;
                entities.get ( 2 ).LEFT = false;
            } else {
                entities.get ( 2 ).LEFT = true;
                entities.get ( 2 ).RIGHT = false;
            }
            System.out.println ( "Intersecting" );
        }
    }
    private void updateEntityMovements ( long elapsedTime ) {
        for ( Entity e : entities ) {
            e.update ( elapsedTime );
            e.move();
        }
    }
    @Override
    protected void paintComponent ( Graphics grphcs ) {
        super.paintComponent ( grphcs );
        Graphics2D g2d = ( Graphics2D ) grphcs;
        applyRenderHints ( g2d );
        drawBackground ( g2d );
        drawEntitiesToScreen ( g2d );
        drawFpsCounter ( g2d );
        frameCount++;
    }
    public static void applyRenderHints ( Graphics2D g2d ) {
        g2d.setRenderingHints ( textRenderHints );
        g2d.setRenderingHints ( imageRenderHints );
        g2d.setRenderingHints ( colorRenderHints );
        g2d.setRenderingHints ( interpolationRenderHints );
        g2d.setRenderingHints ( renderHints );
    }
    private void drawEntitiesToScreen ( Graphics2D g2d ) {
        for ( Entity e : entities ) {
            if ( e.isVisible() ) {
                g2d.drawImage ( e.getCurrentImage(), ( int ) e.getX(), ( int ) e.getY(), null );
            }
        }
    }
    private void drawFpsCounter ( Graphics2D g2d ) {
        g2d.setColor ( Color.WHITE );
        g2d.drawString ( "FPS: " + fps, 5, 10 );
    }
    private void drawBackground ( Graphics2D g2d ) {
        g2d.setColor ( Color.BLACK );
        g2d.fillRect ( 0, 0, getWidth(), getHeight() );
        g2d.setColor ( Color.BLACK );
        for ( int i = 0; i < 128; i++ ) {
            g2d.setColor ( new Color ( random.nextInt ( 256 ), random.nextInt ( 256 ), random.nextInt ( 256 ) ) );
            g2d.drawLine ( getWidth() / 2, getHeight() / 2, random.nextInt ( getWidth() ), random.nextInt ( getHeight() ) );
        }
    }
}
