import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.GraphicsEnvironment;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Window;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.imageio.ImageIO;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;
public class DragButtons {
    ArrayList<JButton> buttons = new ArrayList<>();
    public DragButtons() {
        initComponents();
    }
    public static void main ( String[] args ) {
        SwingUtilities.invokeLater ( new Runnable() {
            @Override
            public void run() {
                new DragButtons();
            }
        } );
    }
    private void initComponents() {
        JFrame frame = new JFrame();
        frame.setDefaultCloseOperation ( JFrame.EXIT_ON_CLOSE );
        final JPanel panel = new JPanel ( new GridLayout ( 2, 2 ) );
        ComponentMover cm = new ComponentMover() {
            @Override
            public void mouseReleased ( MouseEvent e ) {
                super.mouseReleased ( e );
                HashMap<Integer, JButton> collisions = new HashMap<>();
                JButton draggedButton = ( JButton ) e.getSource();
                for ( JButton btn : buttons ) {
                    if ( btn != draggedButton ) {
                        int col = checkPerPixelCollision ( draggedButton, btn );
                        System.out.println ( "Button " + btn.getText() );
                        System.out.println ( col );
                        collisions.put ( col, btn );
                    }
                }
                int maxCollisions = 0;
                JButton intersectingButton = null;
                for ( Map.Entry<Integer, JButton> entry : collisions.entrySet() ) {
                    Integer collisionCount = entry.getKey();
                    JButton button = entry.getValue();
                    if ( collisionCount > maxCollisions ) {
                        maxCollisions = collisionCount;
                        intersectingButton = button;
                    }
                }
                boolean reLayout = false;
                if ( maxCollisions > 0 ) {
                    System.out.println ( "Button " + draggedButton.getText() + " is intersecting more of Button " + intersectingButton.getText() );
                    System.out.println ( "Collisions: " + maxCollisions );
                    reLayout = true;
                } else {
                    System.out.println ( "No change made" );
                    reLayout = false;
                }
                ArrayList<JButton> tmpButtons = ( ArrayList<JButton> ) buttons.clone();
                if ( reLayout ) {
                    buttons.clear();
                    for ( JButton b : tmpButtons ) {
                        if ( b == intersectingButton ) {
                            buttons.add ( draggedButton );
                        } else if ( b == draggedButton ) {
                            buttons.add ( intersectingButton );
                        } else {
                            buttons.add ( b );
                        }
                    }
                    panel.removeAll();
                    for ( JButton btn : buttons ) {
                        panel.add ( btn );
                    }
                    panel.revalidate();
                    panel.repaint();
                }
            }
        };
        for ( int i = 0; i < 4; i++ ) {
            JButton b = new JButton ( String.valueOf ( i + 1 ) );
            buttons.add ( b );
            panel.add ( b );
            cm.registerComponent ( b );
        }
        frame.add ( panel );
        frame.pack();
        frame.setVisible ( true );
    }
    public HashSet<String> getMask ( JButton e ) {
        HashSet<String> mask = new HashSet<>();
        int pixel, a;
        BufferedImage bi = null;
        try {
            bi = componentToImage ( e, e.getBounds() );
        } catch ( IOException ex ) {
            Logger.getLogger ( DragButtons.class.getName() ).log ( Level.SEVERE, null, ex );
        }
        for ( int i = 0; i < bi.getWidth(); i++ ) {
            for ( int j = 0; j < bi.getHeight(); j++ ) {
                pixel = bi.getRGB ( i, j );
                a = ( pixel >> 24 ) & 0xff;
                if ( a != 0 ) {
                    mask.add ( ( e.getX() + i ) + "," + ( e.getY() - j ) );
                }
            }
        }
        return mask;
    }
    public static BufferedImage componentToImage ( Component component, Rectangle region ) throws IOException {
        BufferedImage img = new BufferedImage ( component.getWidth(), component.getHeight(), BufferedImage.TYPE_INT_ARGB_PRE );
        Graphics g = img.getGraphics();
        g.setColor ( component.getForeground() );
        g.setFont ( component.getFont() );
        component.paintAll ( g );
        ImageIO.write ( img, "png", new File ( "c:/saved.png" ) );
        return img;
    }
    public int checkPerPixelCollision ( JButton b, JButton b2 ) {
        int ax1 = ( int ) b2.getX();
        int ay1 = ( int ) b2.getY();
        int ax2 = ax1 + ( int ) b2.getWidth();
        int ay2 = ay1 + ( int ) b2.getHeight();
        int bx1 = ( int ) b.getX();
        int by1 = ( int ) b.getY();
        int bx2 = bx1 + ( int ) b.getWidth();
        int by2 = by1 + ( int ) b.getHeight();
        if ( by2 < ay1 || ay2 < by1 || bx2 < ax1 || ax2 < bx1 ) {
            return 0;
        } else {
            HashSet<String> maskPlayer1 = getMask ( b2 );
            HashSet<String> maskPlayer2 = getMask ( b );
            maskPlayer1.retainAll ( maskPlayer2 );
            if ( maskPlayer1.size() > 0 ) {
                return maskPlayer1.size();
            }
        }
        return 0;
    }
}
class ComponentMover extends MouseAdapter {
    private Insets dragInsets = new Insets ( 0, 0, 0, 0 );
    private Dimension snapSize = new Dimension ( 1, 1 );
    private Insets edgeInsets = new Insets ( 0, 0, 0, 0 );
    private boolean changeCursor = true;
    private boolean autoLayout = false;
    private Class destinationClass;
    private Component destinationComponent;
    private Component destination;
    private Component source;
    private Point pressed;
    private Point location;
    private Cursor originalCursor;
    private boolean autoscrolls;
    private boolean potentialDrag;
    public ComponentMover() {
    }
    public ComponentMover ( Class destinationClass, Component... components ) {
        this.destinationClass = destinationClass;
        registerComponent ( components );
    }
    public ComponentMover ( Component destinationComponent, Component... components ) {
        this.destinationComponent = destinationComponent;
        registerComponent ( components );
    }
    public boolean isAutoLayout() {
        return autoLayout;
    }
    public void setAutoLayout ( boolean autoLayout ) {
        this.autoLayout = autoLayout;
    }
    public boolean isChangeCursor() {
        return changeCursor;
    }
    public void setChangeCursor ( boolean changeCursor ) {
        this.changeCursor = changeCursor;
    }
    public Insets getDragInsets() {
        return dragInsets;
    }
    public void setDragInsets ( Insets dragInsets ) {
        this.dragInsets = dragInsets;
    }
    public Insets getEdgeInsets() {
        return edgeInsets;
    }
    public void setEdgeInsets ( Insets edgeInsets ) {
        this.edgeInsets = edgeInsets;
    }
    public void deregisterComponent ( Component... components ) {
        for ( Component component : components ) {
            component.removeMouseListener ( this );
        }
    }
    public void registerComponent ( Component... components ) {
        for ( Component component : components ) {
            component.addMouseListener ( this );
        }
    }
    public Dimension getSnapSize() {
        return snapSize;
    }
    public void setSnapSize ( Dimension snapSize ) {
        if ( snapSize.width < 1
                || snapSize.height < 1 ) {
            throw new IllegalArgumentException ( "Snap sizes must be greater than 0" );
        }
        this.snapSize = snapSize;
    }
    @Override
    public void mousePressed ( MouseEvent e ) {
        source = e.getComponent();
        int width = source.getSize().width - dragInsets.left - dragInsets.right;
        int height = source.getSize().height - dragInsets.top - dragInsets.bottom;
        Rectangle r = new Rectangle ( dragInsets.left, dragInsets.top, width, height );
        if ( r.contains ( e.getPoint() ) ) {
            setupForDragging ( e );
        }
    }
    private void setupForDragging ( MouseEvent e ) {
        source.addMouseMotionListener ( this );
        potentialDrag = true;
        if ( destinationComponent != null ) {
            destination = destinationComponent;
        } else if ( destinationClass == null ) {
            destination = source;
        } else {
            destination = SwingUtilities.getAncestorOfClass ( destinationClass, source );
        }
        pressed = e.getLocationOnScreen();
        location = destination.getLocation();
        if ( changeCursor ) {
            originalCursor = source.getCursor();
            source.setCursor ( Cursor.getPredefinedCursor ( Cursor.MOVE_CURSOR ) );
        }
        if ( destination instanceof JComponent ) {
            JComponent jc = ( JComponent ) destination;
            autoscrolls = jc.getAutoscrolls();
            jc.setAutoscrolls ( false );
        }
    }
    @Override
    public void mouseDragged ( MouseEvent e ) {
        Point dragged = e.getLocationOnScreen();
        int dragX = getDragDistance ( dragged.x, pressed.x, snapSize.width );
        int dragY = getDragDistance ( dragged.y, pressed.y, snapSize.height );
        int locationX = location.x + dragX;
        int locationY = location.y + dragY;
        while ( locationX < edgeInsets.left ) {
            locationX += snapSize.width;
        }
        while ( locationY < edgeInsets.top ) {
            locationY += snapSize.height;
        }
        Dimension d = getBoundingSize ( destination );
        while ( locationX + destination.getSize().width + edgeInsets.right > d.width ) {
            locationX -= snapSize.width;
        }
        while ( locationY + destination.getSize().height + edgeInsets.bottom > d.height ) {
            locationY -= snapSize.height;
        }
        destination.setLocation ( locationX, locationY );
    }
    private int getDragDistance ( int larger, int smaller, int snapSize ) {
        int halfway = snapSize / 2;
        int drag = larger - smaller;
        drag += ( drag < 0 ) ? -halfway : halfway;
        drag = ( drag / snapSize ) * snapSize;
        return drag;
    }
    private Dimension getBoundingSize ( Component source ) {
        if ( source instanceof Window ) {
            GraphicsEnvironment env = GraphicsEnvironment.getLocalGraphicsEnvironment();
            Rectangle bounds = env.getMaximumWindowBounds();
            return new Dimension ( bounds.width, bounds.height );
        } else {
            return source.getParent().getSize();
        }
    }
    @Override
    public void mouseReleased ( MouseEvent e ) {
        super.mouseReleased ( e );
        if ( !potentialDrag ) {
            return;
        }
        source.removeMouseMotionListener ( this );
        potentialDrag = false;
        if ( changeCursor ) {
            source.setCursor ( originalCursor );
        }
        if ( destination instanceof JComponent ) {
            ( ( JComponent ) destination ).setAutoscrolls ( autoscrolls );
        }
        if ( autoLayout ) {
            if ( destination instanceof JComponent ) {
                ( ( JComponent ) destination ).revalidate();
            } else {
                destination.revalidate();
            }
            destination.repaint();
        }
    }
}
