private static final String ESCAPED_FRAGMENT_FORMAT1 = "_escaped_fragment_=";
private static final int ESCAPED_FRAGMENT_LENGTH1 = ESCAPED_FRAGMENT_FORMAT1.length();
private static final String ESCAPED_FRAGMENT_FORMAT2 = "&" + ESCAPED_FRAGMENT_FORMAT1;
private static final int ESCAPED_FRAGMENT_LENGTH2 = ESCAPED_FRAGMENT_FORMAT2.length();
private class SyncAllAjaxController extends NicelyResynchronizingAjaxController {
    private static final long serialVersionUID = 1L;
    @Override
    public boolean processSynchron ( HtmlPage page, WebRequest request, boolean async ) {
        return true;
    }
}
private WebClient webClient = null;
private static final long _pumpEventLoopTimeoutMillis = 200;
private static final long _jsTimeoutMillis = 200;
private static final long _pageWaitMillis = 100;
final int _maxLoopChecks = 2;
public void destroy() {
    if ( webClient != null ) {
        webClient.closeAllWindows();
    }
}
public void doFilter ( ServletRequest request, ServletResponse response, FilterChain filterChain )
throws IOException, ServletException {
    final HttpServletRequest httpRequest = ( HttpServletRequest ) request;
    final String requestURI = httpRequest.getRequestURI();
    final String queryString = httpRequest.getQueryString();
    final HttpServletResponse httpResponse = ( HttpServletResponse ) response;
    if ( ( queryString != null ) && ( queryString.contains ( ESCAPED_FRAGMENT_FORMAT1 ) ) ) {
        final String domain = httpRequest.getServerName();
        final int port = httpRequest.getServerPort();
        final String urlStringWithHashFragment = requestURI + rewriteQueryString ( queryString );
        final String protocol = httpRequest.getProtocol();
        final URL urlWithHashFragment = new URL ( protocol, domain, port, urlStringWithHashFragment );
        final WebRequest webRequest = new WebRequest ( urlWithHashFragment );
        webClient = new WebClient ( BrowserVersion.FIREFOX_3_6 );
        webClient.getCache().clear();
        webClient.setJavaScriptEnabled ( true );
        webClient.setThrowExceptionOnScriptError ( false );
        webClient.setRedirectEnabled ( false );
        webClient.setAjaxController ( new SyncAllAjaxController() );
        webClient.setCssErrorHandler ( new SilentCssErrorHandler() );
        if ( _logger.isInfoEnabled() ) {
            _logger.info ( "HtmlUnit starting webClient.getPage(webRequest) where webRequest = " + webRequest.toString() );
        }
        final HtmlPage page = webClient.getPage ( webRequest );
        webClient.getJavaScriptEngine().pumpEventLoop ( _pumpEventLoopTimeoutMillis );
        int waitForBackgroundJavaScript = webClient.waitForBackgroundJavaScript ( _jsTimeoutMillis );
        int loopCount = 0;
        while ( waitForBackgroundJavaScript > 0 && loopCount < _maxLoopChecks ) {
            ++loopCount;
            waitForBackgroundJavaScript = webClient.waitForBackgroundJavaScript ( _jsTimeoutMillis );
            if ( waitForBackgroundJavaScript == 0 ) {
                if ( _logger.isTraceEnabled() ) {
                    _logger.trace ( "HtmlUnit exits background javascript at loop counter " + loopCount );
                }
                break;
            }
            synchronized ( page ) {
                if ( _logger.isTraceEnabled() ) {
                    _logger.trace ( "HtmlUnit waits for background javascript at loop counter " + loopCount );
                }
                try {
                    page.wait ( _pageWaitMillis );
                } catch ( InterruptedException e ) {
                    _logger.error ( "HtmlUnit ERROR on page.wait at loop counter " + loopCount );
                    e.printStackTrace();
                }
            }
        }
        webClient.getAjaxController().processSynchron ( page, webRequest, false );
        if ( webClient.getJavaScriptEngine().isScriptRunning() ) {
            _logger.warn ( "HtmlUnit webClient.getJavaScriptEngine().shutdownJavaScriptExecutor()" );
            webClient.getJavaScriptEngine().shutdownJavaScriptExecutor();
        }
        final String staticSnapshotHtml = page.asXml();
        httpResponse.setContentType ( "text/html;charset=UTF-8" );
        final PrintWriter out = httpResponse.getWriter();
        out.println ( "<hr />" );
        out.println ( "<center><h3>Page non-interactive pour le crawler." );
        out.println ( "La page interactive est: <a href=\""
                      + urlWithHashFragment
                      + "\">"
                      + urlWithHashFragment + "</a></h3></center>" );
        out.println ( "<hr />" );
        out.println ( staticSnapshotHtml );
        webClient.closeAllWindows();
        out.println ( "" );
        out.flush();
        out.close();
        if ( _logger.isInfoEnabled() ) {
            _logger.info ( "HtmlUnit completed webClient.getPage(webRequest) where webRequest = " + webRequest.toString() );
        }
    } else {
        if ( requestURI.contains ( ".nocache." ) ) {
            final Date now = new Date();
            httpResponse.setDateHeader ( "Date", now.getTime() );
            httpResponse.setDateHeader ( "Expires", now.getTime() - 86400000L );
            httpResponse.setHeader ( "Pragma", "no-cache" );
            httpResponse.setHeader ( "Cache-control", "no-cache, no-store, must-revalidate" );
        }
        filterChain.doFilter ( request, response );
    }
}
private static String rewriteQueryString ( String queryString )
throws UnsupportedEncodingException {
    int index = queryString.indexOf ( ESCAPED_FRAGMENT_FORMAT2 );
    int length = ESCAPED_FRAGMENT_LENGTH2;
    if ( index == -1 ) {
        index = queryString.indexOf ( ESCAPED_FRAGMENT_FORMAT1 );
        length = ESCAPED_FRAGMENT_LENGTH1;
    }
    if ( index != -1 ) {
        final StringBuilder queryStringSb = new StringBuilder();
        if ( index > 0 ) {
            queryStringSb.append ( "?" );
            queryStringSb.append ( queryString.substring ( 0, index ) );
        }
        queryStringSb.append ( "#!" );
        final String token2Decode = queryString.substring ( index + length, queryString.length() );
        final String tokenDecoded = URLDecoder.decode ( token2Decode, "UTF-8" );
        queryStringSb.append ( tokenDecoded );
        return queryStringSb.toString();
    }
    return queryString;
}
