public class PrintDocument {
    public void printText ( String text ) throws PrintException, IOException {
        PrintService service = PrintServiceLookup.lookupDefaultPrintService();
        InputStream is = new ByteArrayInputStream ( text.getBytes ( "UTF8" ) );
        PrintRequestAttributeSet  pras = new HashPrintRequestAttributeSet();
        pras.add ( new Copies ( 1 ) );
        DocFlavor flavor = DocFlavor.INPUT_STREAM.AUTOSENSE;
        Doc doc = new SimpleDoc ( is, flavor, null );
        DocPrintJob job = service.createPrintJob();
        PrintJobWatcher pjw = new PrintJobWatcher ( job );
        job.print ( doc, pras );
        pjw.waitForDone();
        is.close();
    }
}
class PrintJobWatcher {
    boolean done = false;
    PrintJobWatcher ( DocPrintJob job ) {
        job.addPrintJobListener ( new PrintJobAdapter() {
            public void printJobCanceled ( PrintJobEvent pje ) {
                allDone();
            }
            public void printJobCompleted ( PrintJobEvent pje ) {
                allDone();
            }
            public void printJobFailed ( PrintJobEvent pje ) {
                allDone();
            }
            public void printJobNoMoreEvents ( PrintJobEvent pje ) {
                allDone();
            }
            void allDone() {
                synchronized ( PrintJobWatcher.this ) {
                    done = true;
                    System.out.println ( "Printing document is done ..." );
                    PrintJobWatcher.this.notify();
                }
            }
        } );
    }
    public synchronized void waitForDone() {
        try {
            while ( !done ) {
                wait();
            }
        } catch ( InterruptedException e ) {
        }
    }
}
