public class VideoPlayerActivity extends Activity implements
    OnBufferingUpdateListener, OnCompletionListener,
    OnPreparedListener, OnVideoSizeChangedListener, SurfaceHolder.Callback {
    private static final String TAG = "MediaPlayerDemo";
    private int mVideoWidth;
    private int mVideoHeight;
    private MediaPlayer mMediaPlayer;
    private SurfaceView mPreview;
    private SurfaceHolder holder;
    private String path;
    private static final int STREAM_VIDEO = 5;
    private boolean mIsVideoSizeKnown = false;
    private boolean mIsVideoReadyToBePlayed = false;
    @Override
    public void onCreate ( Bundle icicle ) {
        super.onCreate ( icicle );
        setContentView ( R.layout.main );
        mPreview = ( SurfaceView ) findViewById ( R.id.VideoView );
        holder = mPreview.getHolder();
        holder.addCallback ( this );
        holder.setType ( SurfaceHolder.SURFACE_TYPE_PUSH_BUFFERS );
    }
    private void playVideo ( Integer Media ) {
        doCleanUp();
        try {
            path = "http://commonsware.com/misc/test2.3gp";
            mMediaPlayer = new MediaPlayer();
            mMediaPlayer.setDataSource ( path );
            mMediaPlayer.setDisplay ( holder );
            mMediaPlayer.prepare();
            mMediaPlayer.setOnBufferingUpdateListener ( this );
            mMediaPlayer.setOnCompletionListener ( this );
            mMediaPlayer.setOnPreparedListener ( this );
            mMediaPlayer.setOnVideoSizeChangedListener ( this );
        } catch ( Exception e ) {
            Log.e ( TAG, "error: " + e.getMessage(), e );
        }
    }
    public void onBufferingUpdate ( MediaPlayer arg0, int percent ) {
        Log.d ( TAG, "onBufferingUpdate percent:" + percent );
    }
    public void onCompletion ( MediaPlayer arg0 ) {
        Log.d ( TAG, "onCompletion called" );
    }
    public void onVideoSizeChanged ( MediaPlayer mp, int width, int height ) {
        Log.v ( TAG, "onVideoSizeChanged called" );
        if ( width == 0 || height == 0 ) {
            Log.e ( TAG, "invalid video width(" + width + ") or height(" + height + ")" );
            return;
        }
        mIsVideoSizeKnown = true;
        mVideoWidth = width;
        mVideoHeight = height;
        if ( mIsVideoReadyToBePlayed && mIsVideoSizeKnown ) {
            startVideoPlayback();
        }
    }
    public void onPrepared ( MediaPlayer mediaplayer ) {
        Log.d ( TAG, "onPrepared called" );
        mIsVideoReadyToBePlayed = true;
        if ( mIsVideoReadyToBePlayed && mIsVideoSizeKnown ) {
            startVideoPlayback();
        }
    }
    public void surfaceChanged ( SurfaceHolder surfaceholder, int i, int j, int k ) {
        Log.d ( TAG, "surfaceChanged called" );
    }
    public void surfaceDestroyed ( SurfaceHolder surfaceholder ) {
        Log.d ( TAG, "surfaceDestroyed called" );
    }
    public void surfaceCreated ( SurfaceHolder holder ) {
        Log.d ( TAG, "surfaceCreated called" );
        playVideo ( STREAM_VIDEO );
    }
    @Override
    protected void onPause() {
        super.onPause();
        releaseMediaPlayer();
        doCleanUp();
    }
    @Override
    protected void onDestroy() {
        super.onDestroy();
        releaseMediaPlayer();
        doCleanUp();
    }
    private void releaseMediaPlayer() {
        if ( mMediaPlayer != null ) {
            mMediaPlayer.release();
            mMediaPlayer = null;
        }
    }
    private void doCleanUp() {
        mVideoWidth = 0;
        mVideoHeight = 0;
        mIsVideoReadyToBePlayed = false;
        mIsVideoSizeKnown = false;
    }
    private void startVideoPlayback() {
        Log.v ( TAG, "startVideoPlayback" );
        holder.setFixedSize ( mVideoWidth, mVideoHeight );
        mMediaPlayer.start();
    }
}
