public class SequenceEncoder {
    private SeekableByteChannel ch;
    private Picture toEncode;
    private RgbToYuv420 transform;
    private H264Encoder encoder;
    private ArrayList<ByteBuffer> spsList;
    private ArrayList<ByteBuffer> ppsList;
    private CompressedTrack outTrack;
    private ByteBuffer _out;
    private int frameNo;
    private MP4Muxer muxer;
    public SequenceEncoder ( File out ) throws IOException {
        this.ch = NIOUtils.writableFileChannel ( out );
        transform = new RgbToYuv420 ( 0, 0 );
        muxer = new MP4Muxer ( ch, Brand.MP4 );
        outTrack = muxer.addTrackForCompressed ( TrackType.VIDEO, 25 );
        _out = ByteBuffer.allocate ( 1920 * 1080 * 6 );
        encoder = new H264Encoder();
        spsList = new ArrayList<ByteBuffer>();
        ppsList = new ArrayList<ByteBuffer>();
    }
    public void encodeImage ( BufferedImage bi ) throws IOException {
        if ( toEncode == null ) {
            toEncode = Picture.create ( bi.getWidth(), bi.getHeight(), ColorSpace.YUV420 );
        }
        for ( int i = 0; i < 3; i++ ) {
            Arrays.fill ( toEncode.getData() [i], 0 );
        }
        transform.transform ( AWTUtil.fromBufferedImage ( bi ), toEncode );
        _out.clear();
        ByteBuffer result = encoder.encodeFrame ( _out, toEncode );
        spsList.clear();
        ppsList.clear();
        H264Utils.encodeMOVPacket ( result, spsList, ppsList );
        outTrack.addFrame ( new MP4Packet ( result, frameNo, 25, 1, frameNo, true, null, frameNo, 0 ) );
        frameNo++;
    }
    public void finish() throws IOException {
        outTrack.addSampleEntry ( H264Utils.createMOVSampleEntry ( spsList, ppsList ) );
        muxer.writeHeader();
        NIOUtils.closeQuietly ( ch );
    }
    public static void main ( String[] args ) throws IOException {
        SequenceEncoder encoder = new SequenceEncoder ( new File ( "video.mp4" ) );
        for ( int i = 1; i < 100; i++ ) {
            BufferedImage bi = ImageIO.read ( new File ( String.format ( "folder/img%08d.png", i ) ) );
            encoder.encodeImage ( bi );
        }
        encoder.finish();
    }
}
