public class OverlayTextArea {
    public static void main ( String[] args ) {
        new OverlayTextArea();
    }
    public OverlayTextArea() {
        EventQueue.invokeLater ( new Runnable() {
            @Override
            public void run() {
                try {
                    UIManager.setLookAndFeel ( UIManager.getSystemLookAndFeelClassName() );
                } catch ( Exception ex ) {
                }
                JFrame frame = new JFrame ( "Testing" );
                frame.setDefaultCloseOperation ( JFrame.EXIT_ON_CLOSE );
                frame.setContentPane ( new ImagePane() );
                frame.setLayout ( new BorderLayout() );
                frame.add ( new TransparentTextArea() );
                frame.pack();
                frame.setLocationRelativeTo ( null );
                frame.setVisible ( true );
            }
        } );
    }
    public class TransparentTextArea extends JTextArea {
        public TransparentTextArea() {
            setOpaque ( false );
            setBorder ( new CompoundBorder ( new EmptyBorder ( 10, 10, 10, 10 ), new LineBorder ( Color.LIGHT_GRAY ) ) );
        }
        @Override
        protected void paintComponent ( Graphics g ) {
            g.setColor ( new Color ( 255, 255, 255, 128 ) );
            Insets insets = getInsets();
            int x = insets.left;
            int y = insets.top;
            int width = getWidth() - ( insets.left + insets.right );
            int height = getHeight() - ( insets.top + insets.bottom );
            g.fillRect ( x, y, width, height );
            super.paintComponent ( g );
        }
    }
    public class ImagePane extends JPanel {
        private BufferedImage background;
        public ImagePane() {
            try {
                background = ImageIO.read ( new File ( "/path/to/background.img" ) );
            } catch ( IOException ex ) {
                ex.printStackTrace();
            }
        }
        @Override
        public Dimension getPreferredSize() {
            return background == null ? super.getPreferredSize() : new Dimension ( background.getWidth(), background.getHeight() );
        }
        @Override
        protected void paintComponent ( Graphics g ) {
            super.paintComponent ( g );
            if ( background != null ) {
                int x = ( getWidth() - background.getWidth() ) / 2;
                int y = ( getHeight() - background.getHeight() ) / 2;
                g.drawImage ( background, x, y, this );
            }
        }
    }
}
