import com.actionbarsherlock.widget.SearchView;
import android.app.SearchManager;
import android.app.SearchableInfo;
import android.content.ContentResolver;
import android.content.Context;
import android.database.Cursor;
import android.net.Uri;
import android.support.v4.widget.CursorAdapter;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;
public final class DrugsSearchAdapter extends CursorAdapter {
    private static final int QUERY_LIMIT = 50;
    private LayoutInflater inflater;
    private SearchView searchView;
    private SearchableInfo searchable;
    public DrugsSearchAdapter ( Context context, SearchableInfo info, SearchView searchView ) {
        super ( context, null, CursorAdapter.FLAG_REGISTER_CONTENT_OBSERVER );
        this.searchable = info;
        this.searchView = searchView;
        this.inflater = LayoutInflater.from ( context );
    }
    @Override
    public void bindView ( View v, Context context, Cursor c ) {
        String name = c.getString ( c.getColumnIndex ( SearchManager.SUGGEST_COLUMN_TEXT_1 ) );
        TextView namet = ( TextView ) v.findViewById ( R.id.list_item_drug_name );
        namet.setText ( name );
        String man = c.getString ( c.getColumnIndex ( SearchManager.SUGGEST_COLUMN_TEXT_2 ) );
        TextView manuf = ( TextView ) v.findViewById ( R.id.list_item_drug_manufacturer );
        manuf.setText ( man );
    }
    @Override
    public View newView ( Context arg0, Cursor arg1, ViewGroup arg2 ) {
        return this.inflater.inflate ( R.layout.list_item_drug_search, null );
    }
    @Override
    public Cursor runQueryOnBackgroundThread ( CharSequence constraint ) {
        String query = ( constraint == null ) ? "" : constraint.toString();
        Cursor cursor = null;
        if ( searchView.getVisibility() != View.VISIBLE
                || searchView.getWindowVisibility() != View.VISIBLE ) {
            return null;
        }
        try {
            cursor = getSuggestions ( searchable, query, QUERY_LIMIT );
            if ( cursor != null ) {
                cursor.getCount();
                return cursor;
            }
        } catch ( RuntimeException e ) {
        }
        return null;
    }
    public Cursor getSuggestions ( SearchableInfo searchable, String query, int limit ) {
        if ( searchable == null ) {
            return null;
        }
        String authority = searchable.getSuggestAuthority();
        if ( authority == null ) {
            return null;
        }
        Uri.Builder uriBuilder = new Uri.Builder()
        .scheme ( ContentResolver.SCHEME_CONTENT )
        .authority ( authority )
        .query ( "" )
        .fragment ( "" );
        final String contentPath = searchable.getSuggestPath();
        if ( contentPath != null ) {
            uriBuilder.appendEncodedPath ( contentPath );
        }
        uriBuilder.appendPath ( SearchManager.SUGGEST_URI_PATH_QUERY );
        String selection = searchable.getSuggestSelection();
        String[] selArgs = null;
        if ( selection != null ) {
            selArgs = new String[] { query };
        } else {
            uriBuilder.appendPath ( query );
        }
        if ( limit > 0 ) {
            uriBuilder.appendQueryParameter ( "limit", String.valueOf ( limit ) );
        }
        Uri uri = uriBuilder.build();
        return mContext.getContentResolver().query ( uri, null, selection, selArgs, null );
    }
}
