public class AbsFigureTest {
    public static final String defaultTestPath = "unittest.tmp";
    private List<FigureGeneral> inputFigures;
    private List<FigureGeneral> outputFigures;
    private DeepCompare deepCompare;
    @Before
    public void setUp() throws FigureGeneralFilesFoundException,
        FigureGeneralFormatException {
        inputFigures = absFigure.generateRandomFigures ( 5 );
        absFigure.write ( defaultTestPath, inputFigures );
        outputFigures = absFigure.figureReader ( defaultTestPath );
        deepCompare = new DeepCompare();
    }
    @After
    public void releaseResources() {
        new File ( defaultTestPath ).delete();
    }
    @Test
    public void assertEquals_FigureGeneralLists_ReturnZero() {
        for ( int i = 0; i < inputFigures.size(); i++ ) {
            FigureGeneral inputValue = inputFigures.get ( i );
            FigureGeneral outputValue = outputFigures.get ( i );
            if ( deepCompare.compare ( inputValue, outputValue ) != 0 ) {
                throw new RuntimeException ( "not equal" );
            }
        }
    }
    @Test
    public void testDeepCompRectangle_EqualResult_ReturnZero() {
        FigureGeneral oneFigure = new Rectangle ( 2.0, 2.0, "rectangle" );
        FigureGeneral twoFigure = new Rectangle ( 2.0, 2.0, "rectangle" );
        int result = deepCompare.compare ( oneFigure, twoFigure );
        assertTrue ( "expected to be equal", result == 0 );
    }
    @Test
    public void testDeepCompTriangle_EqualResult_ReturnZero() {
        FigureGeneral oneFigure = new Triangle ( 2.0, 2.0, "rectangle" );
        FigureGeneral twoFigure = new Triangle ( 2.0, 2.0, "rectangle" );
        int result = deepCompare.compare ( oneFigure, twoFigure );
        assertTrue ( "expected to be equal", result == 0 );
    }
    @Test
    public void testDeepComp_GreaterThan_ReturnOne() {
        FigureGeneral oneFigure = new Triangle ( 2.0, 2.0, "triangle" );
        FigureGeneral twoFigure = new Rectangle ( 1.0, 1.0, "rectangle" );
        int result = deepCompare.compare ( oneFigure, twoFigure );
        assertTrue ( "expected to be greater than", result >= 1 );
        if ( deepCompare.compare ( oneFigure, twoFigure ) != 0 ) {
            throw new RuntimeException ( "not equal" );
        }
    }
}
