package org.jdesktop.swingx.plaf.basic;
import java.util.Calendar;
import javax.swing.JComponent;
import org.jdesktop.swingx.JXMonthView;
import org.jdesktop.swingx.decorator.CompoundHighlighter;
import org.jdesktop.swingx.decorator.Highlighter;
public class DemoCalendarRenderingHandler extends BasicCalendarRenderingHandler {
    private DemoCalendarAdapter adapter;
    private CompoundHighlighter demoHighlighter;
    public void setHighlighters ( Highlighter... highlighters ) {
        getHighlighter().setHighlighters ( highlighters );
    }
    public void addHighlighters ( Highlighter... highlighters ) {
        for ( Highlighter highlighter : highlighters ) {
            getHighlighter().addHighlighter ( highlighter );
        }
    }
    @Override
    public JComponent prepareRenderingComponent ( JXMonthView monthView,
            Calendar calendar, CalendarState dayState ) {
        JComponent component = super.prepareRenderingComponent ( monthView, calendar, dayState );
        return ( JComponent ) getHighlighter().highlight (
                   component, getCalendarAdapter ( monthView, calendar, dayState ) );
    }
    protected CompoundHighlighter getHighlighter() {
        if ( demoHighlighter == null ) {
            demoHighlighter = new CompoundHighlighter();
        }
        return demoHighlighter;
    }
    protected DemoCalendarAdapter getCalendarAdapter ( JXMonthView monthView,
            Calendar calendar, CalendarState dayState ) {
        if ( adapter == null ) {
            adapter = new DemoCalendarAdapter ( monthView );
        }
        return adapter.install ( calendar, dayState );
    }
    public static class DemoCalendarAdapter extends CalendarAdapter {
        public DemoCalendarAdapter ( JXMonthView component ) {
            super ( component );
        }
        @Override
        public DemoCalendarAdapter install ( Calendar calendar, CalendarState dayState ) {
            return ( DemoCalendarAdapter ) super.install ( calendar, dayState );
        }
        @Override
        public boolean isSelected() {
            if ( !isDayInMonth() ) {
                return false;
            }
            return super.isSelected();
        }
        public Calendar getCalendar() {
            return ( Calendar ) calendar.clone();
        }
        public boolean isDayInMonth() {
            return ( CalendarState.IN_MONTH == getCalendarState() )
                   || ( CalendarState.TODAY == getCalendarState() );
        }
    }
