package com.quartetfs.pivot.sandbox.postprocessor.impl;
import java.util.Timer;
import java.util.TimerTask;
import com.quartetfs.biz.pivot.IActivePivotSession;
import com.quartetfs.biz.pivot.query.aggregates.IAggregatesContinuousQueryEngine;
import com.quartetfs.biz.pivot.query.aggregates.impl.AStream;
import com.quartetfs.fwk.QuartetExtendedPluginValue;
import com.quartetfs.fwk.types.impl.ExtendedPluginInjector;
@QuartetExtendedPluginValue ( interfaceName = "com.quartetfs.biz.pivot.query.aggregates.IStream", key = TickingStream.PLUGIN_KEY )
public class TickingStream extends AStream<Void> {
    private static final long serialVersionUID = 1L;
    public static final String PLUGIN_KEY = "TICKING";
    protected static final long DEFAULT_PERIOD = 1000;
    protected long period = DEFAULT_PERIOD;
    protected final TimerTask sendEventTask;
    protected Timer tickingTimer;
    public TickingStream ( IAggregatesContinuousQueryEngine engine,
                           IActivePivotSession session ) {
        super ( engine, session );
        sendEventTask = new TimerTask() {
            @Override
            public void run() {
                sendEvent ( null );
            }
        };
        rescheduleTask();
    }
    protected void rescheduleTask() {
        if ( tickingTimer != null ) {
            tickingTimer.cancel();
        }
        tickingTimer = new Timer();
        tickingTimer.schedule ( sendEventTask, 0, period );
    }
    public void setPeriod ( long period ) {
        if ( period <= 0 ) {
            throw new IllegalArgumentException ( "Non-positive period." );
        }
        this.period = period;
        rescheduleTask();
    }
    @Override
    public Class<Void> getEventType() {
        return Void.class;
    }
    @Override
    public String getType() {
        return PLUGIN_KEY;
    }
}
