public class TestAnimatedWindow {
    public static void main ( String[] args ) {
        new TestAnimatedWindow();
    }
    public TestAnimatedWindow() {
        EventQueue.invokeLater ( new Runnable() {
            @Override
            public void run() {
                try {
                    UIManager.setLookAndFeel ( UIManager.getSystemLookAndFeelClassName() );
                } catch ( ClassNotFoundException | InstantiationException | IllegalAccessException | UnsupportedLookAndFeelException ex ) {
                }
                JFrame frame = new JFrame ( "Testing" );
                frame.setDefaultCloseOperation ( JFrame.EXIT_ON_CLOSE );
                frame.setLayout ( new BorderLayout() );
                frame.add ( new AnimatedPane() );
                frame.pack();
                frame.setLocationRelativeTo ( null );
                frame.setVisible ( true );
            }
        } );
    }
    public class AnimatedPane extends JPanel {
        private GridBagConstraints gbc = new GridBagConstraints();
        private Timer anim;
        private long startTime;
        private int duration = 1000;
        private Rectangle startBounds, targetBounds;
        private JPanel top;
        private JPanel bottom;
        private JSlider durationSlider;
        private JLabel durationLabel;
        private JPanel content;
        public AnimatedPane() {
            top = new JPanel();
            bottom = new JPanel ( new GridBagLayout() );
            content = new JPanel ( new GridBagLayout() );
            durationLabel = new JLabel ( "1000" );
            durationSlider = new JSlider ( 125, 5000 );
            durationSlider.setMajorTickSpacing ( 1000 );
            durationSlider.setMinorTickSpacing ( 500 );
            durationSlider.setPaintTicks ( true );
            durationSlider.setValue ( 1000 );
            GridBagConstraints gbcBottom = new GridBagConstraints();
            gbcBottom.weightx = 1;
            gbcBottom.fill = GridBagConstraints.HORIZONTAL;
            bottom.add ( durationSlider, gbcBottom );
            gbcBottom.weightx = 0;
            gbcBottom.fill = GridBagConstraints.NONE;
            bottom.add ( durationLabel, gbcBottom );
            durationSlider.addChangeListener ( new ChangeListener() {
                @Override
                public void stateChanged ( ChangeEvent e ) {
                    int value = durationSlider.getValue();
                    durationLabel.setText ( Integer.toString ( value ) );
                }
            } );
            setLayout ( new BorderLayout() );
            gbc.gridwidth = GridBagConstraints.REMAINDER;
            JButton grow = new JButton ( "Grow" );
            top.add ( grow, gbc );
            add ( top, BorderLayout.NORTH );
            add ( content );
            add ( bottom, BorderLayout.SOUTH );
            grow.addActionListener ( new ActionListener() {
                @Override
                public void actionPerformed ( ActionEvent e ) {
                    anim.stop();
                    Window window = SwingUtilities.windowForComponent ( AnimatedPane.this );
                    startBounds = window.getBounds();
                    JPanel panel = new JPanel ( new GridBagLayout() );
                    panel.add ( new JLabel ( "Boo" ) );
                    panel.setBorder ( new LineBorder ( Color.RED ) );
                    content.add ( panel, gbc );
                    targetBounds = new Rectangle();
                    Insets insets = window.getInsets();
                    Dimension size = getPreferredSize();
                    size.width += insets.left + insets.right;
                    size.height += insets.top + insets.bottom;
                    targetBounds.setSize ( size );
                    Rectangle viewBounds = getScreenViewableBounds();
                    Point p = new Point();
                    p.x = ( ( viewBounds.width - size.width ) / 2 ) + viewBounds.x;
                    p.y = ( ( viewBounds.height - size.height ) / 2 ) + viewBounds.y;
                    targetBounds.setLocation ( p );
                    if ( targetBounds.width % 2 != 0 ) {
                        targetBounds.width++;
                    }
                    if ( targetBounds.height % 2 != 0 ) {
                        targetBounds.height++;
                    }
                    startTime = System.currentTimeMillis();
                    duration = durationSlider.getValue();
                    anim.restart();
                }
            } );
            anim = new Timer ( 40, new ActionListener() {
                @Override
                public void actionPerformed ( ActionEvent e ) {
                    long now = System.currentTimeMillis();
                    long diff = now - startTime;
                    Window window = SwingUtilities.getWindowAncestor ( AnimatedPane.this );
                    Rectangle bounds = targetBounds;
                    if ( diff < duration ) {
                        double progress = ( double ) diff / ( double ) duration;
                        bounds = calculateProgress ( startBounds, targetBounds, progress );
                        if ( bounds.width % 2 != 0 ) {
                            bounds.width++;
                        }
                        if ( bounds.height % 2 != 0 ) {
                            bounds.height++;
                        }
                    } else {
                        ( ( Timer ) e.getSource() ).stop();
                    }
                    window.setBounds ( bounds );
                }
            } );
            anim.setRepeats ( true );
            anim.setCoalesce ( true );
            anim.setInitialDelay ( 0 );
        }
    }
    public static Rectangle calculateProgress ( Rectangle startBounds, Rectangle targetBounds, double progress ) {
        Rectangle bounds = new Rectangle();
        if ( startBounds != null && targetBounds != null ) {
            bounds.setLocation ( calculateProgress ( startBounds.getLocation(), targetBounds.getLocation(), progress ) );
            bounds.setSize ( calculateProgress ( startBounds.getSize(), targetBounds.getSize(), progress ) );
        }
        return bounds;
    }
    public static Dimension calculateProgress ( Dimension startSize, Dimension targetSize, double progress ) {
        Dimension size = new Dimension();
        if ( startSize != null && targetSize != null ) {
            size.width = calculateProgress ( startSize.width, targetSize.width, progress );
            size.height = calculateProgress ( startSize.height, targetSize.height, progress );
        }
        return size;
    }
    public static Point calculateProgress ( Point startPoint, Point targetPoint, double progress ) {
        Point point = new Point();
        if ( startPoint != null && targetPoint != null ) {
            point.x = calculateProgress ( startPoint.x, targetPoint.x, progress );
            point.y = calculateProgress ( startPoint.y, targetPoint.y, progress );
        }
        return point;
    }
    public static int calculateProgress ( int startValue, int endValue, double fraction ) {
        int value = 0;
        int distance = endValue - startValue;
        value = ( int ) ( ( float ) distance * fraction );
        value += startValue;
        return value;
    }
    public static Rectangle getScreenViewableBounds() {
        GraphicsEnvironment ge = GraphicsEnvironment.getLocalGraphicsEnvironment();
        GraphicsDevice gd = ge.getDefaultScreenDevice();
        return getScreenViewableBounds ( gd );
    }
    public static Rectangle getScreenViewableBounds ( GraphicsDevice gd ) {
        Rectangle bounds = new Rectangle ( 0, 0, 0, 0 );
        if ( gd != null ) {
            GraphicsConfiguration gc = gd.getDefaultConfiguration();
            bounds = gc.getBounds();
            Insets insets = Toolkit.getDefaultToolkit().getScreenInsets ( gc );
            bounds.x += insets.left;
            bounds.y += insets.top;
            bounds.width -= ( insets.left + insets.right );
            bounds.height -= ( insets.top + insets.bottom );
        }
        return bounds;
    }
}
