public class MainActivity extends Activity {
    private BluetoothAdapter mBluetoothAdapter;
    private ArrayAdapter<String> mPairedDevicesArrayAdapter;
    private ListView PairedListView;
    private BluetoothService mBluetoothService = null;
    public void onCreate ( Bundle savedInstanceState ) {
        super.onCreate ( savedInstanceState );
        setContentView ( R.layout.layout );
        mBluetoothAdapter = BluetoothAdapter.getDefaultAdapter();
        mPairedDevicesArrayAdapter = new ArrayAdapter<String> ( this, R.layout.device_name );
        PairedListView = ( ListView ) findViewById ( R.id.listView_paired_devices );
        PairedListView.setAdapter ( mPairedDevicesArrayAdapter );
        PairedListView.setOnItemClickListener ( this );
        getPairedDevices();
    }
    private void getPairedDevices() {
        Set<BluetoothDevice> pairedDevices = mBluetoothAdapter.getBondedDevices();
        if ( pairedDevices.size() > 0 ) {
            for ( BluetoothDevice device : pairedDevices ) {
                mPairedDevicesArrayAdapter.add ( device.getName() + "\n" + device.getAddress() );
            }
        } else {
            String noDevices = getResources().getText ( R.string.title_none_paired ).toString();
            mPairedDevicesArrayAdapter.add ( noDevices );
        }
    }
}
@Override
public void onItemClick ( AdapterView<?> av, View v, int arg2, long arg3 ) {
    mBluetoothAdapter.cancelDiscovery();
    String info = ( ( TextView ) v ).getText().toString();
    String adress = info.substring ( info.length() - 17 );
    BluetoothDevice device = mBluetoothAdapter.getRemoteDevice ( adress );
    mBluetoothService.connect ( device );
}
public class BluetoothService {
    private static final UUID MY_UUID = UUID.fromString ( "00001101-0000-1000-8000-00805F9B34FB" );
    private ConnectThread mConnectThread;
    private ConnectedThread mConnectedThread;
    private BluetoothAdapter mBluetoothAdapter;
    public synchronized void connect ( BluetoothDevice device ) {
        mConnectThread = new ConnectThread ( device );
        mConnectThread.start();
    }
    private class ConnectThread extends Thread {
        private final BluetoothSocket mBluetoothSocket;
        private final BluetoothDevice mBluetoothDevice;
        public ConnectThread ( BluetoothDevice device ) {
            mBluetoothDevice = device;
            BluetoothSocket tmp = null;
            try {
                tmp = device.createInsecureRfcommSocketToServiceRecord ( MY_UUID );
            } catch ( IOException e ) {
            }
            mBluetoothSocket = tmp;
        }
        public void run() {
            mBluetoothAdapter = BluetoothAdapter.getDefaultAdapter();
            mBluetoothAdapter.cancelDiscovery();
            try {
                mBluetoothSocket.connect();
            } catch ( IOException e ) {
                try {
                    mBluetoothSocket.close();
                    setState ( STATE_CONNECTION_FAILED );
                } catch ( IOException e1 ) {
                    e1.printStackTrace();
                }
            }
            synchronized ( BluetoothService.this ) {
                mConnectThread = null;
            }
            if ( mBluetoothSocket.isConnected() ) {
                connected ( mBluetoothSocket, mBluetoothDevice );
            }
        }
        public void cancel() {
            try {
                mBluetoothSocket.close();
            } catch ( IOException e ) {
                e.printStackTrace();
            }
        }
    }
}
