public class IQCameraView extends ViewGroup implements SurfaceHolder.Callback {
    public static final int CAMERA_ERROR = 0;
    public static final int CAMERA_RESULT = 1;
    private final String TAG = "Preview";
    private SurfaceView mSurfaceView;
    private SurfaceHolder mHolder;
    private Size mPreviewSize;
    private List<Size> mSupportedPreviewSizes;
    private Camera mCamera;
    private boolean mCameraActive;
    private IQCameraCallback mCallback;
    public void setmCallback ( IQCameraCallback mCallback ) {
        this.mCallback = mCallback;
    }
    public static boolean checkCameraHardware ( Context context ) {
        if ( context.getPackageManager().hasSystemFeature (
                    PackageManager.FEATURE_CAMERA ) ) {
            return true;
        } else {
            return false;
        }
    }
    public IQCameraView ( Context context ) {
        super ( context );
        mSurfaceView = new SurfaceView ( context );
        addView ( mSurfaceView );
        mHolder = mSurfaceView.getHolder();
        mHolder.addCallback ( this );
        mHolder.setType ( SurfaceHolder.SURFACE_TYPE_PUSH_BUFFERS );
    }
    public void setCamera ( Camera camera ) {
        mCamera = camera;
        if ( mCamera != null ) {
            mSupportedPreviewSizes = mCamera.getParameters()
                                     .getSupportedPreviewSizes();
            requestLayout();
        }
    }
    public void takePicture() {
        if ( null != mCamera && mCameraActive ) {
            mCamera.takePicture ( null, null, new IQPhotoHandler ( getContext(),
                                  this ) );
        }
    }
    public void onImageCapture ( final String result ) {
        if ( null != mCallback ) {
            mCallback.onCameraCallback ( CAMERA_RESULT, result );
        }
        mCamera.startPreview();
    }
    @Override
    protected void onMeasure ( int widthMeasureSpec, int heightMeasureSpec ) {
        final int width = resolveSize ( getSuggestedMinimumWidth(),
                                        widthMeasureSpec );
        final int height = resolveSize ( getSuggestedMinimumHeight(),
                                         heightMeasureSpec );
        setMeasuredDimension ( width, height );
        if ( mSupportedPreviewSizes != null ) {
            mPreviewSize = getPreviewSize ( mSupportedPreviewSizes, width, height );
        }
    }
    @Override
    protected void onLayout ( boolean changed, int l, int t, int r, int b ) {
        if ( changed && getChildCount() > 0 ) {
            final View child = getChildAt ( 0 );
            final int width = r - l;
            final int height = b - t;
            int previewWidth = width;
            int previewHeight = height;
            if ( width * previewHeight > height * previewWidth ) {
                final int scaledChildWidth = previewWidth * height
                                             / previewHeight;
                child.layout ( ( width - scaledChildWidth ) / 2, 0,
                               ( width + scaledChildWidth ) / 2, height );
            } else {
                final int scaledChildHeight = previewHeight * width
                                              / previewWidth;
                child.layout ( 0, ( height - scaledChildHeight ) / 2, width,
                               ( height + scaledChildHeight ) / 2 );
            }
        }
    }
    public void releaseCamera() {
        if ( null != mCamera && mCameraActive ) {
            mCamera.stopPreview();
            mCamera.release();
            mCamera = null;
        }
        mCameraActive = false;
    }
    public void surfaceCreated ( SurfaceHolder holder ) {
        try {
            if ( mCamera != null ) {
                mCamera.setPreviewDisplay ( holder );
            }
        } catch ( IOException e ) {
            Log.e ( TAG, "IOException caused by setPreviewDisplay()", e );
        }
    }
    public void surfaceDestroyed ( SurfaceHolder holder ) {
        releaseCamera();
    }
    public void surfaceChanged ( SurfaceHolder holder, int format, int w, int h ) {
        Camera.Parameters parameters = mCamera.getParameters();
        parameters.setPreviewSize ( mPreviewSize.width, mPreviewSize.height );
        parameters.setPictureSize ( mPreviewSize.width, mPreviewSize.height );
        parameters.setPictureFormat ( PixelFormat.JPEG );
        mCamera.setDisplayOrientation ( 90 );
        requestLayout();
        try {
            mCamera.setParameters ( parameters );
        } catch ( Exception e ) {
            System.out.print ( "hih" );
        }
        mCamera.startPreview();
        mCameraActive = true;
    }
    private Size getPreviewSize ( List<Size> sizes, int w, int h ) {
        final double ASPECT_TOLERANCE = 0.1;
        double targetRatio = ( double ) w / h;
        if ( sizes == null ) {
            return null;
        }
        Size optimalSize = null;
        double minDiff = Double.MAX_VALUE;
        int targetHeight = h;
        for ( Size size : sizes ) {
            double ratio = ( double ) size.width / size.height;
            if ( Math.abs ( ratio - targetRatio ) > ASPECT_TOLERANCE ) {
                continue;
            }
            if ( Math.abs ( size.height - targetHeight ) < minDiff ) {
                optimalSize = size;
                minDiff = Math.abs ( size.height - targetHeight );
            }
        }
        if ( optimalSize == null ) {
            minDiff = Double.MAX_VALUE;
            for ( Size size : sizes ) {
                if ( Math.abs ( size.height - targetHeight ) < minDiff ) {
                    optimalSize = size;
                    minDiff = Math.abs ( size.height - targetHeight );
                }
            }
        }
        return optimalSize;
    }
    public static interface IQCameraCallback {
        public void onCameraCallback ( final int type, final Object param );
    }
}
