@WebServlet ( "/userProfileImageServlet" )
public class UserProfileImageServlet extends HttpServlet {
    protected void doGet ( HttpServletRequest request, HttpServletResponse response ) throws ServletException, IOException {
        Long userProfileId = Long.valueOf ( request.getParameter ( "id" ) );
        Connection connection = null;
        PreparedStatement statement = null;
        ResultSet resultSet = null;
        try {
            connection = database.getConnection();
            statement = connection.prepareStatement ( "SELECT image, imageFileName, LENGTH(image) AS imageContentLength FROM userProfile WHERE id=?" );
            statement.setLong ( 1, userProfileId );
            resultSet = statement.executeQuery();
            if ( resultSet.next() ) {
                response.setContentType ( getServletContext().getMimeType ( resultSet.getString ( "imageFileName" ) ) );
                response.setContentLength ( resultSet.getInt ( "imageContentLength" ) );
                response.setHeader ( "Content-Disposition", "inline;filename=\"" + resultSet.getString ( "imageFileName" ) + "\"" );
                ReadableByteChannel input = null;
                WritableByteChannel output = null;
                try {
                    input = Channels.newChannel ( resource.getInputStream() );
                    output = Channels.newChannel ( externalContext.getResponseOutputStream() );
                    for ( ByteBuffer buffer = ByteBuffer.allocateDirect ( 10240 ); input.read ( buffer ) != -1; buffer.clear() ) {
                        output.write ( ( ByteBuffer ) buffer.flip() );
                    }
                } finally {
                    if ( output != null ) try {
                            output.close();
                        } catch ( IOException ignore ) {}
                    if ( input != null ) try {
                            input.close();
                        } catch ( IOException ignore ) {}
                }
            } else {
                response.sendError ( HttpServletResponse.SC_NOT_FOUND );
            }
        } catch ( SQLException e ) {
            throw new ServletException ( "Something failed at SQL/DB level.", e );
        } finally {
            if ( resultSet != null ) try {
                    resultSet.close();
                } catch ( SQLException logOrIgnore ) {}
            if ( statement != null ) try {
                    statement.close();
                } catch ( SQLException logOrIgnore ) {}
            if ( connection != null ) try {
                    connection.close();
                } catch ( SQLException logOrIgnore ) {}
        }
    }
}
