import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JTextPane;
import javax.swing.SwingUtilities;
import javax.swing.Timer;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledDocument;
public class KaraokeTest {
    private int[] timingsArray = {1000, 1000, 9000, 1000, 1000, 1000, 1000, 1000, 1000, 1000};
    private String[] individualWordsToHighlight = {" \nHello\n", " world\n", " Hello", " world", " Hello", " world", " Hello", " world", " Hello", " world"};
    private int count = 0;
    private final JTextPane jtp = new JTextPane();
    private final JButton startButton = new JButton ( "Start" );
    private final JFrame frame = new JFrame();
    final ArrayList<String> chars = new ArrayList<>();
    final ArrayList<Long> charsTiming = new ArrayList<>();
    public KaraokeTest() {
        initComponents();
    }
    private void initComponents() {
        frame.setDefaultCloseOperation ( JFrame.EXIT_ON_CLOSE );
        frame.setResizable ( false );
        for ( String s : individualWordsToHighlight ) {
            String tmp = jtp.getText();
            jtp.setText ( tmp + s );
        }
        jtp.setEditable ( false );
        startButton.addActionListener ( new ActionListener() {
            @Override
            public void actionPerformed ( ActionEvent ae ) {
                startButton.setEnabled ( false );
                count = 0;
                charsTiming.clear();
                chars.clear();
                for ( String s : individualWordsToHighlight ) {
                    for ( int i = 0; i < s.length(); i++ ) {
                        chars.add ( String.valueOf ( s.charAt ( i ) ) );
                    }
                }
                for ( int x = 0; x < timingsArray.length; x++ ) {
                    for ( int i = 0; i < individualWordsToHighlight[x].length(); i++ ) {
                        individualWordsToHighlight[x] = individualWordsToHighlight[x].replace ( "\n", " " ).replace ( "\r", " " );
                        charsTiming.add ( ( long ) ( timingsArray[x] / individualWordsToHighlight[x].trim().length() ) );
                    }
                }
                Timer t = new Timer ( 1, new AbstractAction() {
                    long startTime = 0;
                    long acum = 0;
                    long timeItTookTotal = 0;
                    long dif = 0, timeItTook = 0, timeToTake = 0;
                    int delay = 0;
                    @Override
                    public void actionPerformed ( ActionEvent ae ) {
                        if ( count < charsTiming.size() ) {
                            if ( count == 0 ) {
                                startTime = System.nanoTime();
                                System.out.println ( "Started: " + startTime );
                            }
                            timeToTake = charsTiming.get ( count );
                            acum += timeToTake;
                            highlightNextWord();
                            timeItTook = ( acum - ( ( System.nanoTime() - startTime ) / 1000000 ) );
                            timeItTookTotal += timeItTook;
                            System.out.println ( "Time the char should take: " + timeToTake );
                            System.out.println ( "Time it took: " + timeItTook );
                            dif = ( timeToTake - timeItTook );
                            System.out.println ( "Difference: " + dif );
                            delay = ( int ) ( timeToTake - dif );
                            if ( delay < 1 ) {
                                delay = 1;
                            }
                            ( ( Timer ) ae.getSource() ).setInitialDelay ( ( int ) timeToTake );
                            ( ( Timer ) ae.getSource() ).restart();
                        } else {
                            long timeStopped = System.nanoTime();
                            System.out.println ( "Stopped: " + timeStopped );
                            System.out.println ( "Time it should take in total: " + acum );
                            System.out.println ( "Time it took using accumulator of time taken for each letter: " + timeItTookTotal
                                                 + "\nDifference: " + ( acum - timeItTookTotal ) );
                            long timeItTookUsingNanoTime = ( ( timeStopped - startTime ) / 1000000 );
                            System.out.println ( "Time it took using difference (endTime-startTime): " + timeItTookUsingNanoTime
                                                 + "\nDifference: " + ( acum - timeItTookUsingNanoTime ) );
                            reset();
                            ( ( Timer ) ae.getSource() ).stop();
                        }
                        count++;
                    }
                } );
                t.setRepeats ( false );
                t.start();
            }
        } );
        frame.add ( jtp, BorderLayout.CENTER );
        frame.add ( startButton, BorderLayout.SOUTH );
        frame.pack();
        frame.setVisible ( true );
    }
    private void reset() {
        startButton.setEnabled ( true );
        jtp.setText ( "" );
        for ( String s : individualWordsToHighlight ) {
            String tmp = jtp.getText();
            jtp.setText ( tmp + s );
        }
        JOptionPane.showMessageDialog ( frame, "Done" );
    }
    private void highlightNextWord() {
        int sp = 0;
        for ( int i = 0; i < count + 1; i++ ) {
            sp += 1;
        }
        while ( chars.get ( sp - 1 ).equals ( " " ) ) {
            sp += 1;
            count++;
        }
        Style style = jtp.addStyle ( "RED", null );
        StyleConstants.setForeground ( style, Color.RED );
        ( ( StyledDocument ) jtp.getDocument() ).setCharacterAttributes ( 0, sp, style, true );
    }
    public static void main ( String[] args ) {
        SwingUtilities.invokeLater ( new Runnable() {
            @Override
            public void run() {
                new KaraokeTest();
            }
        } );
    }
}
