class MyIntFilter extends DocumentFilter {
    private int maxLength = 0;
    public void setMaxLength ( int maxLength ) {
        this.maxLength = maxLength;
    }
    public int getMaxLength() {
        return maxLength;
    }
    public void insertString ( FilterBypass fb, int offset, String string,
                               AttributeSet attr ) throws BadLocationException {
        Document doc = fb.getDocument();
        StringBuilder sb = new StringBuilder();
        sb.append ( doc.getText ( 0, doc.getLength() ) );
        sb.insert ( offset, string );
        if ( maxLength > 0 && doc.getLength() + string.length() <= maxLength ) {
            if ( test ( sb.toString() ) ) {
                super.insertString ( fb, offset, string, attr );
            } else {
            }
        }
    }
    private boolean test ( String text ) {
        try {
            Integer.parseInt ( text );
            return true;
        } catch ( NumberFormatException e ) {
            return false;
        }
    }
    @Override
    public void replace ( FilterBypass fb, int offset, int length, String text,
                          AttributeSet attrs ) throws BadLocationException {
        Document doc = fb.getDocument();
        StringBuilder sb = new StringBuilder ( 2 );
        sb.append ( doc.getText ( 0, doc.getLength() ) );
        sb.replace ( offset, offset + length, text );
        if ( test ( sb.toString() ) ) {
            if ( sb.length() > maxLength ) {
                length = sb.length() - maxLength;
                if ( length > 0 ) {
                    text = text.substring ( 0, length );
                    super.replace ( fb, offset, length, text, attrs );
                }
            }
        } else {
        }
    }
    @Override
    public void remove ( FilterBypass fb, int offset, int length )
    throws BadLocationException {
        Document doc = fb.getDocument();
        StringBuilder sb = new StringBuilder ( 2 );
        sb.append ( doc.getText ( 0, doc.getLength() ) );
        sb.delete ( offset, offset + length );
        if ( test ( sb.toString() ) ) {
            super.remove ( fb, offset, length );
        } else {
        }
    }
}
