import java.awt.AlphaComposite;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.RenderingHints;
import java.awt.geom.RoundRectangle2D;
import java.awt.image.BufferedImage;
import java.net.URL;
import javax.imageio.ImageIO;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;
public class ShadowTest {
    private JFrame frame;
    public ShadowTest() {
        initComponents();
    }
    public static void main ( String[] args ) {
        SwingUtilities.invokeLater ( new Runnable() {
            @Override
            public void run() {
                new ShadowTest();
            }
        } );
    }
    private void initComponents() {
        frame = new JFrame();
        frame.setTitle ( "Example" );
        frame.setDefaultCloseOperation ( JFrame.EXIT_ON_CLOSE );
        frame.setResizable ( false );
        frame.setLayout ( new GridBagLayout() );
        GridBagConstraints gc = new GridBagConstraints();
        gc.fill = GridBagConstraints.HORIZONTAL;
        gc.insets = new Insets ( 10, 10, 10, 10 );
        frame.add ( new RoundedPanel(), gc );
        frame.pack();
        frame.setVisible ( true );
    }
}
class RoundedPanel extends JPanel {
    protected int strokeSize = 1;
    protected Color shadowColor = Color.black;
    protected boolean shady = true;
    protected boolean highQuality = true;
    protected Dimension arcs = new Dimension ( 20, 20 );
    protected int shadowGap = 10;
    protected int shadowOffset = 4;
    protected int shadowAlpha = 150;
    int width = 300, height = 300;
    BufferedImage image;
    BufferedImage roundedImage;
    public RoundedPanel() {
        super();
        setOpaque ( false );
        try {
            image = ImageIO.read ( new URL ( "http://www.wallpaperpimper.com/wallpaper/Anime_&_Manga/Naruto/Naruto-Vs-Sasuke-1-D6UGW3NLAZ-800x600.jpg" ) );
        } catch ( Exception ex ) {
            ex.printStackTrace();
        }
        roundedImage = makeRoundedCorner ( image, 20 );
        width = roundedImage.getWidth() + arcs.width / 2;
        height = roundedImage.getHeight() + arcs.height / 2;
    }
    public static BufferedImage makeRoundedCorner ( BufferedImage image, int cornerRadius ) {
        int w = image.getWidth();
        int h = image.getHeight();
        BufferedImage output = new BufferedImage ( w, h, BufferedImage.TYPE_INT_ARGB );
        Graphics2D g2 = output.createGraphics();
        g2.setComposite ( AlphaComposite.Src );
        g2.setRenderingHint ( RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON );
        g2.setColor ( Color.WHITE );
        g2.fill ( new RoundRectangle2D.Float ( 0, 0, w, h, cornerRadius, cornerRadius ) );
        g2.setComposite ( AlphaComposite.SrcAtop );
        g2.drawImage ( image, 0, 0, null );
        g2.dispose();
        return output;
    }
    @Override
    protected void paintComponent ( Graphics g ) {
        super.paintComponent ( g );
        Color shadowColorA = new Color ( shadowColor.getRed(),
                                         shadowColor.getGreen(), shadowColor.getBlue(), shadowAlpha );
        Graphics2D graphics = ( Graphics2D ) g;
        if ( highQuality ) {
            graphics.setRenderingHint ( RenderingHints.KEY_ANTIALIASING,
                                        RenderingHints.VALUE_ANTIALIAS_ON );
        }
        if ( shady ) {
            graphics.setColor ( shadowColorA );
            graphics.fillRoundRect (
                shadowOffset,
                shadowOffset,
                width - strokeSize - shadowOffset,
                height - strokeSize - shadowOffset,
                arcs.width, arcs.height );
        } else {
            shadowGap = 1;
        }
        graphics.setColor ( getBackground() );
        graphics.fillRoundRect ( 0, 0, width - shadowGap,
                                 height - shadowGap, arcs.width, arcs.height );
        graphics.setColor ( getForeground() );
        graphics.setStroke ( new BasicStroke ( strokeSize ) );
        graphics.drawRoundRect ( 0, 0, width - shadowGap,
                                 height - shadowGap, arcs.width, arcs.height );
        graphics.drawImage ( roundedImage, 0, 0, this );
    }
    @Override
    public Dimension getPreferredSize() {
        return new Dimension ( width, height );
    }
}
