import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.IOException;
import java.util.ArrayList;
import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.DropMode;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.TransferHandler;
public class TestListReorder extends JPanel {
    ArrayListTransferHandler arrayListHandler;
    public TestListReorder() {
        arrayListHandler = new ArrayListTransferHandler();
        JList list1, list2;
        DefaultListModel list1Model = new DefaultListModel();
        list1Model.addElement ( "0 (list 1)" );
        list1Model.addElement ( "1 (list 1)" );
        list1Model.addElement ( "2 (list 1)" );
        list1Model.addElement ( "3 (list 1)" );
        list1Model.addElement ( "4 (list 1)" );
        list1Model.addElement ( "5 (list 1)" );
        list1Model.addElement ( "6 (list 1)" );
        list1 = new JList ( list1Model );
        list1.setDropMode ( DropMode.USE_SELECTION );
        list1.setSelectionMode ( ListSelectionModel.SINGLE_INTERVAL_SELECTION );
        list1.setTransferHandler ( arrayListHandler );
        list1.setDragEnabled ( true );
        JScrollPane list1View = new JScrollPane ( list1 );
        list1View.setPreferredSize ( new Dimension ( 200, 100 ) );
        JPanel panel1 = new JPanel();
        panel1.setLayout ( new BorderLayout() );
        panel1.add ( list1View, BorderLayout.CENTER );
        panel1.setBorder ( BorderFactory.createEmptyBorder ( 5, 5, 5, 5 ) );
        setLayout ( new BorderLayout() );
        add ( panel1, BorderLayout.LINE_START );
        setBorder ( BorderFactory.createEmptyBorder ( 20, 20, 20, 20 ) );
    }
    private static void createAndShowGUI() {
        JFrame.setDefaultLookAndFeelDecorated ( true );
        JFrame frame = new JFrame ( "DragListDemo" );
        frame.setDefaultCloseOperation ( JFrame.EXIT_ON_CLOSE );
        TestListReorder demo = new TestListReorder();
        frame.setContentPane ( demo );
        frame.pack();
        frame.setVisible ( true );
    }
    public static void main ( String[] args ) {
        javax.swing.SwingUtilities.invokeLater ( new Runnable() {
            public void run() {
                createAndShowGUI();
            }
        } );
    }
}
class ArrayListTransferHandler extends TransferHandler {
    DataFlavor localArrayListFlavor, serialArrayListFlavor;
    String localArrayListType = DataFlavor.javaJVMLocalObjectMimeType
                                + ";class=java.util.ArrayList";
    JList source = null;
    int[] indices = null;
    int addIndex = -1;
    int addCount = 0;
    public ArrayListTransferHandler() {
        try {
            localArrayListFlavor = new DataFlavor ( localArrayListType );
        } catch ( ClassNotFoundException e ) {
            System.out
            .println ( "ArrayListTransferHandler: unable to create data flavor" );
        }
        serialArrayListFlavor = new DataFlavor ( ArrayList.class, "ArrayList" );
    }
    public boolean importData ( JComponent c, Transferable t ) {
        JList target = null;
        ArrayList alist = null;
        if ( !canImport ( c, t.getTransferDataFlavors() ) ) {
            return false;
        }
        try {
            target = ( JList ) c;
            if ( hasLocalArrayListFlavor ( t.getTransferDataFlavors() ) ) {
                alist = ( ArrayList ) t.getTransferData ( localArrayListFlavor );
            } else if ( hasSerialArrayListFlavor ( t.getTransferDataFlavors() ) ) {
                alist = ( ArrayList ) t.getTransferData ( serialArrayListFlavor );
            } else {
                return false;
            }
        } catch ( UnsupportedFlavorException ufe ) {
            System.out.println ( "importData: unsupported data flavor" );
            return false;
        } catch ( IOException ioe ) {
            System.out.println ( "importData: I/O exception" );
            return false;
        }
        int index = target.getSelectedIndex();
        if ( source.equals ( target ) ) {
            if ( indices != null && index >= indices[0] - 1
                    && index <= indices[indices.length - 1] ) {
                indices = null;
                return true;
            }
        }
        DefaultListModel listModel = ( DefaultListModel ) target.getModel();
        int max = listModel.getSize();
        if ( index < 0 ) {
            index = max;
        } else {
            index++;
            if ( index > max ) {
                index = max;
            }
        }
        addIndex = index;
        addCount = alist.size();
        for ( int i = 0; i < alist.size(); i++ ) {
            listModel.add ( index++, alist.get ( i ) );
        }
        return true;
    }
    protected void exportDone ( JComponent c, Transferable data, int action ) {
        if ( ( action == MOVE ) && ( indices != null ) ) {
            DefaultListModel model = ( DefaultListModel ) source.getModel();
            if ( addCount > 0 ) {
                for ( int i = 0; i < indices.length; i++ ) {
                    if ( indices[i] > addIndex ) {
                        indices[i] += addCount;
                    }
                }
            }
            for ( int i = indices.length - 1; i >= 0; i-- ) {
                model.remove ( indices[i] );
            }
        }
        indices = null;
        addIndex = -1;
        addCount = 0;
    }
    private boolean hasLocalArrayListFlavor ( DataFlavor[] flavors ) {
        if ( localArrayListFlavor == null ) {
            return false;
        }
        for ( int i = 0; i < flavors.length; i++ ) {
            if ( flavors[i].equals ( localArrayListFlavor ) ) {
                return true;
            }
        }
        return false;
    }
    private boolean hasSerialArrayListFlavor ( DataFlavor[] flavors ) {
        if ( serialArrayListFlavor == null ) {
            return false;
        }
        for ( int i = 0; i < flavors.length; i++ ) {
            if ( flavors[i].equals ( serialArrayListFlavor ) ) {
                return true;
            }
        }
        return false;
    }
    public boolean canImport ( JComponent c, DataFlavor[] flavors ) {
        if ( hasLocalArrayListFlavor ( flavors ) ) {
            return true;
        }
        if ( hasSerialArrayListFlavor ( flavors ) ) {
            return true;
        }
        return false;
    }
    protected Transferable createTransferable ( JComponent c ) {
        if ( c instanceof JList ) {
            source = ( JList ) c;
            indices = source.getSelectedIndices();
            Object[] values = source.getSelectedValues();
            if ( values == null || values.length == 0 ) {
                return null;
            }
            ArrayList alist = new ArrayList ( values.length );
            for ( int i = 0; i < values.length; i++ ) {
                Object o = values[i];
                String str = o.toString();
                if ( str == null ) {
                    str = "";
                }
                alist.add ( str );
            }
            return new ArrayListTransferable ( alist );
        }
        return null;
    }
    public int getSourceActions ( JComponent c ) {
        return COPY_OR_MOVE;
    }
    public class ArrayListTransferable implements Transferable {
        ArrayList data;
        public ArrayListTransferable ( ArrayList alist ) {
            data = alist;
        }
        public Object getTransferData ( DataFlavor flavor )
        throws UnsupportedFlavorException {
            if ( !isDataFlavorSupported ( flavor ) ) {
                throw new UnsupportedFlavorException ( flavor );
            }
            return data;
        }
        public DataFlavor[] getTransferDataFlavors() {
            return new DataFlavor[] {localArrayListFlavor,
                                     serialArrayListFlavor
                                    };
        }
        public boolean isDataFlavorSupported ( DataFlavor flavor ) {
            if ( localArrayListFlavor.equals ( flavor ) ) {
                return true;
            }
            if ( serialArrayListFlavor.equals ( flavor ) ) {
                return true;
            }
            return false;
        }
    }
}
