package com.quartetfs.biz.pivot.aggfun.impl;
import java.util.Arrays;
import com.quartetfs.fwk.IClone;
public class DoubleVector implements IClone<DoubleVector> {
    protected final double[] data;
    private DoubleVector ( double[] data ) {
        this.data = data;
    }
    public static DoubleVector compress ( double[] raw ) {
        final int length = raw.length;
        int bucketCount = 0;
        while ( ( bucketCount << 6 ) < length ) {
            bucketCount++;
        }
        int nonZeroes = 0;
        for ( int i = 0; i < length; i++ ) {
            nonZeroes += raw[i] == 0.0 ? 0 : 1;
        }
        final double[] data = new double[1 + bucketCount + nonZeroes];
        data[0] = Double.longBitsToDouble ( ( long ) length );
        nonZeroes = 0;
        for ( int b = 0; b < bucketCount; b++ ) {
            data[1 + b] = Double.longBitsToDouble ( 0L );
            final int from = b << 6;
            final int to = Math.min ( length, ( b + 1 ) << 6 );
            for ( int i = from; i < to; i++ ) {
                double value = raw[i];
                if ( value != 0.0 ) {
                    int bucketIdx = i >>> 6;
                    int shift = i & 0x3F;
                    final long bit = 1L << shift;
                    long bucket = Double.doubleToLongBits ( data[1 + bucketIdx] );
                    bucket = bucket | bit;
                    data[1 + bucketIdx] = Double.longBitsToDouble ( bucket );
                    data[1 + bucketCount + nonZeroes++] = value;
                }
            }
        }
        return new DoubleVector ( data );
    }
    public DoubleVector clone() {
        return new DoubleVector ( data.clone() );
    }
    public int length() {
        return ( int ) Double.doubleToLongBits ( data[0] );
    }
    public DoubleVector add ( DoubleVector other ) {
        return add ( other, false );
    }
    public DoubleVector add ( DoubleVector other, boolean negative ) {
        final int length = ( int ) Double.doubleToLongBits ( this.data[0] );
        if ( ( int ) Double.doubleToLongBits ( other.data[0] ) != length ) {
            throw new IllegalArgumentException ( "Cannot aggregate vectors of different lengths." );
        }
        int bucketCount = 0;
        while ( ( bucketCount << 6 ) < length ) {
            bucketCount++;
        }
        int nonZeroes = 0;
        for ( int b = 0; b < bucketCount; b++ ) {
            nonZeroes += Long.bitCount ( Double.doubleToLongBits ( this.data[1 + b] ) | Double.doubleToLongBits ( other.data[1 + b] ) );
        }
        final double[] result = new double[1 + bucketCount + nonZeroes];
        result[0] = Double.longBitsToDouble ( length );
        for ( int b = 0; b < bucketCount; b++ ) {
            result[1 + b] = Double.longBitsToDouble ( Double.doubleToLongBits ( this.data[1 + b] ) | Double.doubleToLongBits ( other.data[1 + b] ) );
        }
        int a = 0;
        int b = 0;
        int c = 0;
        for ( int i = 0; i < length; i++ ) {
            final int bucketIdx = i >>> 6;
            final int shift = i & 0x3F;
            final long bucketA = Double.doubleToLongBits ( this.data[bucketIdx + 1] );
            final long bucketB = Double.doubleToLongBits ( other.data[bucketIdx + 1] );
            long bitA = ( bucketA >>> shift ) & 0x1L;
            long bitB = ( bucketB >>> shift ) & 0x1L;
            double valueA = bitA == 0L ? 0.0 : this.data[1 + bucketCount + a++];
            double valueB = bitB == 0L ? 0.0 : other.data[1 + bucketCount + b++];
            if ( bitA != 0L || bitB != 0L ) {
                result[1 + bucketCount + c++] = valueA + ( negative ? -valueB : valueB );
            }
        }
        return new DoubleVector ( result );
    }
    public double[] content() {
        final int length = ( int ) Double.doubleToLongBits ( data[0] );
        int bucketCount = 0;
        while ( ( bucketCount << 6 ) < length ) {
            bucketCount++;
        }
        final double[] content = new double[length];
        int nonZeroes = 0;
        for ( int i = 0; i < length; i++ ) {
            final int bucketIdx = i >>> 6;
            final int shift = i & 0x3F;
            final long bucket = Double.doubleToLongBits ( data[bucketIdx + 1] );
            if ( ( ( bucket >>> shift ) & 0x1L ) != 0L ) {
                content[i] = data[1 + bucketCount + nonZeroes++];
            }
        }
        return content;
    }
    public double compressionRatio() {
        long originalSize = 16L + 8 * Double.doubleToLongBits ( data[0] );
        long compressedSize = 16L + 8L + 16L + 8 * data.length;
        return 0.01 * ( 100L * compressedSize / originalSize );
    }
    @Override
    public String toString() {
        return Arrays.toString ( content() );
    }
    protected static String binaryString ( final long b ) {
        final String binary = Long.toBinaryString ( b );
        final int length = binary.length();
        final StringBuffer buffer = new StringBuffer();
        if ( binary.length() <= 64 ) {
            for ( int i = 0; i < ( 64 - length ); i++ ) {
                buffer.append ( '0' );
            }
            buffer.append ( binary );
            return buffer.toString();
        } else {
            return binary.substring ( length - 64 );
        }
    }
    public static void main ( String[] args ) {
        double[] v1 = new double[] { 0.0,   0.0, 2.0, 0.0, 0.0, 5.0, 0.0, 6.0, 7.0, 8.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,  0.0 };
        double[] v2 = new double[] { 10.0, 10.0, 8.0, 0.0, 0.0, 5.0, 0.0, 4.0, 3.0, 2.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 10.0 };
        double[] sum = v1.clone();
        for ( int i = 0; i < sum.length; i++ ) {
            sum[i] += v2[i];
        }
        DoubleVector c1 = DoubleVector.compress ( v1 );
        DoubleVector c2 = DoubleVector.compress ( v2 );
        DoubleVector cSum = c1.add ( c2 );
        System.out.println ( Arrays.toString ( v1 )  + " -> " + c1   + " (" + c1.compressionRatio()   * 100 + "%)" );
        System.out.println ( Arrays.toString ( v2 )  + " -> " + c2   + " (" + c2.compressionRatio()   * 100 + "%)" );
        System.out.println ( Arrays.toString ( sum ) + " -> " + cSum + " (" + cSum.compressionRatio() * 100 + "%)" );
    }
}
