@Override
public void startEditingAtPath ( TreePath path ) {
    super.startEditingAtPath ( path );
    if ( isEditing() ) {
        updateEditorListener();
        updateEditorRemover();
    }
}
private void updateEditorListener() {
    if ( editorListener == null ) {
        editorListener = new CellEditorListener() {
            @Override
            public void editingCanceled ( ChangeEvent e ) {
                terminated ( e );
            }
            private void terminated ( ChangeEvent e ) {
                analyseFocus();
                ( ( CellEditor ) e.getSource() ).removeCellEditorListener ( editorListener );
            }
            @Override
            public void editingStopped ( ChangeEvent e ) {
                terminated ( e );
            }
        };
    }
    getCellEditor().addCellEditorListener ( editorListener );
}
protected void analyseFocus() {
    if ( isFocusOwnerDescending() ) {
        requestFocusInWindow();
    }
}
private boolean isFocusOwnerDescending() {
    if ( !isEditing() ) {
        return false;
    }
    Component focusOwner =
        KeyboardFocusManager.getCurrentKeyboardFocusManager().getFocusOwner();
    if ( focusOwner == null ) {
        return false;
    }
    if ( SwingXUtilities.isDescendingFrom ( focusOwner, this ) ) {
        return true;
    }
    Component permanent =
        KeyboardFocusManager.getCurrentKeyboardFocusManager().getPermanentFocusOwner();
    return SwingXUtilities.isDescendingFrom ( permanent, this );
}
@Override
public void removeNotify() {
    if ( editorRemover != null ) {
        editorRemover.release();
        editorRemover = null;
    }
    super.removeNotify();
}
private void updateEditorRemover() {
    if ( editorRemover == null ) {
        editorRemover = new CellEditorRemover();
    }
    editorRemover.updateKeyboardFocusManager();
}
public class CellEditorRemover implements PropertyChangeListener {
    KeyboardFocusManager focusManager;
    public CellEditorRemover() {
        updateKeyboardFocusManager();
    }
    public void updateKeyboardFocusManager() {
        KeyboardFocusManager current = KeyboardFocusManager.getCurrentKeyboardFocusManager();
        setKeyboardFocusManager ( current );
    }
    public void release() {
        setKeyboardFocusManager ( null );
    }
    private void setKeyboardFocusManager ( KeyboardFocusManager current ) {
        if ( focusManager == current ) {
            return;
        }
        KeyboardFocusManager old = focusManager;
        if ( old != null ) {
            old.removePropertyChangeListener ( "permanentFocusOwner", this );
        }
        focusManager = current;
        if ( focusManager != null ) {
            focusManager.addPropertyChangeListener ( "permanentFocusOwner",
                    this );
        }
    }
    @Override
    public void propertyChange ( PropertyChangeEvent ev ) {
        if ( !isEditing() ) {
            return;
        }
        Component c = focusManager.getPermanentFocusOwner();
        JXTree tree = JXTree.this;
        while ( c != null ) {
            if ( c instanceof JPopupMenu ) {
                c = ( ( JPopupMenu ) c ).getInvoker();
            } else {
                if ( c == tree ) {
                    return;
                } else if ( ( c instanceof Window ) ||
                            ( c instanceof Applet && c.getParent() == null ) ) {
                    if ( c == SwingUtilities.getRoot ( tree ) ) {
                        if ( tree.getInvokesStopCellEditing() ) {
                            tree.stopEditing();
                        }
                        if ( tree.isEditing() ) {
                            tree.cancelEditing();
                        }
                    }
                    break;
                }
                c = c.getParent();
            }
        }
    }
}
