import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.EventQueue;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Paint;
import java.awt.Point;
import java.awt.RadialGradientPaint;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import javax.imageio.ImageIO;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
public class MouseCover {
    public static void main ( String[] args ) {
        new MouseCover();
    }
    public MouseCover() {
        EventQueue.invokeLater ( new Runnable() {
            @Override
            public void run() {
                try {
                    UIManager.setLookAndFeel ( UIManager.getSystemLookAndFeelClassName() );
                } catch ( ClassNotFoundException | InstantiationException | IllegalAccessException | UnsupportedLookAndFeelException ex ) {
                }
                JFrame frame = new JFrame ( "Testing" );
                frame.setDefaultCloseOperation ( JFrame.EXIT_ON_CLOSE );
                frame.setLayout ( new BorderLayout() );
                frame.add ( new TestPane() );
                frame.pack();
                frame.setLocationRelativeTo ( null );
                frame.setVisible ( true );
            }
        } );
    }
    public static class TestPane extends JPanel {
        public static final int RADIUS = 200;
        private Point mousePoint = null;
        private BufferedImage background;
        public TestPane() {
            MouseAdapter mouseHandler = new MouseAdapter() {
                @Override
                public void mouseMoved ( MouseEvent e ) {
                    mousePoint = e.getPoint();
                    repaint();
                }
                @Override
                public void mouseExited ( MouseEvent e ) {
                    mousePoint = null;
                    repaint();
                }
            };
            addMouseMotionListener ( mouseHandler );
            addMouseListener ( mouseHandler );
            try {
                background = ImageIO.read ( ... );
            } catch ( IOException ex ) {
                ex.printStackTrace();
            }
        }
        @Override
        public Dimension getPreferredSize() {
            return background == null ? new Dimension ( 200, 200 ) : new Dimension ( background.getWidth(), background.getHeight() );
        }
        @Override
        protected void paintComponent ( Graphics g ) {
            super.paintComponent ( g );
            Graphics2D g2d = ( Graphics2D ) g.create();
            if ( background != null ) {
                int x = ( getWidth() - background.getWidth() ) / 2;
                int y = ( getHeight() - background.getHeight() ) / 2;
                g2d.drawImage ( background, x, y, this );
            }
            Paint paint = Color.BLACK;
            if ( mousePoint != null ) {
                paint = new RadialGradientPaint (
                    mousePoint,
                    RADIUS,
                    new float[] {0, 1f},
                new Color[] {new Color ( 0, 0, 0, 0 ), new Color ( 0, 0, 0, 255 ) } );
            }
            g2d.setPaint ( paint );
            g2d.fillRect ( 0, 0, getWidth(), getHeight() );
            g2d.dispose();
        }
    }
}
