import java.io.IOException;
import java.io.ObjectInputStream;
import javax.swing.text.BadLocationException;
import javax.swing.text.Caret;
import javax.swing.text.Element;
import org.fife.ui.rsyntaxtextarea.RSyntaxDocument;
import org.fife.ui.rsyntaxtextarea.RSyntaxTextArea;
import org.fife.ui.rsyntaxtextarea.RSyntaxUtilities;
import org.fife.ui.rsyntaxtextarea.templates.*;
public class ArtemisCodeTemplate extends StaticCodeTemplate {
    private static final long serialVersionUID = 1;
    private String beforeCaret;
    private String afterCaret;
    private transient int firstBeforeNewline;
    private transient int firstAfterNewline;
    private static final String EMPTY_STRING        = "";
    private String placeholder;
    public ArtemisCodeTemplate() {
    }
    public ArtemisCodeTemplate ( String id, String beforeCaret, String placeholder, String afterCaret ) {
        super ( id, beforeCaret, afterCaret );
        setPlaceholder ( placeholder );
    }
    public String getPlaceholder() {
        return placeholder;
    }
    public void setPlaceholder ( String text ) {
        this.placeholder = text;
    }
    private String getAfterTextIndented ( String indent ) {
        return getTextIndented ( getAfterCaretText(), firstAfterNewline, indent );
    }
    private String getBeforeTextIndented ( String indent ) {
        return getTextIndented ( getBeforeCaretText(), firstBeforeNewline, indent );
    }
    private String getTextIndented ( String text, int firstNewline, String indent ) {
        if ( firstNewline == -1 ) {
            return text;
        }
        int pos = 0;
        int old = firstNewline + 1;
        StringBuffer sb = new StringBuffer ( text.substring ( 0, old ) );
        sb.append ( indent );
        while ( ( pos = text.indexOf ( '\n', old ) ) > -1 ) {
            sb.append ( text.substring ( old, pos + 1 ) );
            sb.append ( indent );
            old = pos + 1;
        }
        if ( old < text.length() ) {
            sb.append ( text.substring ( old ) );
        }
        return sb.toString();
    }
    public void invoke ( RSyntaxTextArea textArea ) throws BadLocationException {
        Caret c = textArea.getCaret();
        int dot = c.getDot();
        int mark = c.getMark();
        int p0 = Math.min ( dot, mark );
        int p1 = Math.max ( dot, mark );
        RSyntaxDocument doc = ( RSyntaxDocument ) textArea.getDocument();
        Element map = doc.getDefaultRootElement();
        int lineNum = map.getElementIndex ( dot );
        Element line = map.getElement ( lineNum );
        int start = line.getStartOffset();
        int end = line.getEndOffset() - 1;
        String s = textArea.getText ( start, end - start );
        int len = s.length();
        int endWS = 0;
        while ( endWS < len && RSyntaxUtilities.isWhitespace ( s.charAt ( endWS ) ) ) {
            endWS++;
        }
        s = s.substring ( 0, endWS );
        p0 -= getID().length();
        String beforeText = getBeforeTextIndented ( s );
        String afterText = getAfterTextIndented ( s );
        doc.replace ( p0, p1 - p0, beforeText + placeholder + afterText, null );
        textArea.setCaretPosition ( p0 + beforeText.length() );
        textArea.moveCaretPosition ( p0 + beforeText.length() + placeholder.length() );
    }
    private void readObject ( ObjectInputStream in ) throws ClassNotFoundException,
        IOException  {
        in.defaultReadObject();
        setBeforeCaretText ( this.beforeCaret );
        setAfterCaretText ( this.afterCaret );
    }
    public String toString() {
        return "[ArtemisCodeTemplate: id=" + getID() +
               ", text=" + getBeforeCaretText() + "|" +
               getPlaceholder() + "|" + getAfterCaretText() + "]";
    }
}
