public abstract class StoppableRunnable implements Runnable {
    private volatile boolean stopWork;;
    private boolean done;
    public final void run() {
        setup();
        while ( !stopWork && !done ) {
            doUnitOfWork();
        }
        cleanup();
    }
    public void stop() {
        stopWork = true;
    }
    protected void done() {
        done = true;
    }
    protected void setup() { }
    protected void cleanup() { }
    protected abstract void doUnitOfWork();
}
