public class FunctionPlotter implements EntryPoint {
    public static class ArbitraryFunctionData {
        private double xValue;
        private Map<String, Double> yValues = new HashMap<String, Double>();
        public double get ( String key ) {
            return yValues.get ( key );
        }
        public void put ( String key, double yValue ) {
            yValues.put ( key, yValue );
        }
        public double getXValue() {
            return xValue;
        }
        public void setxValue ( double xValue ) {
            this.xValue = xValue;
        }
    }
    public interface AFDProperties extends PropertyAccess<ArbitraryFunctionData> {
        @Path ( "xValue" )
        ModelKeyProvider<ArbitraryFunctionData> key();
        ValueProvider<ArbitraryFunctionData, Double> xValue();
    }
    public static abstract class ArbitraryFunction implements ValueProvider<ArbitraryFunctionData, Double> {
        private final String field;
        public ArbitraryFunction ( String field ) {
            this.field = field;
        }
        public abstract Double f ( Double x );
        @Override
        public Double getValue ( ArbitraryFunctionData object ) {
            return object.get ( field );
        }
        @Override
        public void setValue ( ArbitraryFunctionData object, Double value ) {
            object.put ( field, value );
        }
        @Override
        public String getPath() {
            return field;
        }
    }
    @Override
    public void onModuleLoad() {
        Viewport vp = new Viewport();
        Set<ArbitraryFunction> functions = new HashSet<ArbitraryFunction>();
        ArbitraryFunction f1 = new ArbitraryFunction ( "f1" ) {
            @Override
            public Double f ( Double x ) {
                return Math.sin ( x );
            }
        };
        functions.add ( f1 );
        AFDProperties props = GWT.create ( AFDProperties.class );
        ListStore<ArbitraryFunctionData> store = new ListStore<ArbitraryFunctionData> ( props.key() );
        for ( Double x = 0.0; x <= 2 * Math.PI; x = x + 0.1 ) {
            ArbitraryFunctionData d = new ArbitraryFunctionData();
            d.setxValue ( x );
            for ( ArbitraryFunction func : functions ) {
                d.put ( func.getPath(), func.f ( x ) );
            }
            store.add ( d );
        }
        Chart<ArbitraryFunctionData> chart = new Chart<ArbitraryFunctionData>();
        chart.setStore ( store );
        NumericAxis<ArbitraryFunctionData> verticalAxis = new NumericAxis<ArbitraryFunctionData>();
        verticalAxis.setPosition ( Position.LEFT );
        verticalAxis.addField ( f1 );
        verticalAxis.setSteps ( 2 );
        verticalAxis.setMinorTickSteps ( 5 );
        chart.addAxis ( verticalAxis );
        NumericAxis<ArbitraryFunctionData> horizAxis = new NumericAxis<ArbitraryFunctionData>();
        horizAxis.setPosition ( Position.BOTTOM );
        horizAxis.addField ( props.xValue() );
        horizAxis.setSteps ( 2 );
        horizAxis.setMinorTickSteps ( 5 );
        chart.addAxis ( horizAxis );
        for ( ArbitraryFunction func : functions ) {
            LineSeries<ArbitraryFunctionData> line = new LineSeries<ArbitraryFunctionData>();
            line.setXAxisPosition ( Position.BOTTOM );
            line.setXField ( props.xValue() );
            line.setYAxisPosition ( Position.LEFT );
            line.setYField ( func );
            line.setStroke ( RGB.GRAY );
            line.setStrokeWidth ( 2 );
            chart.addSeries ( line );
        }
        vp.setWidget ( chart );
        RootPanel.get().add ( vp );
    }
}
