public class FunctionPlotter implements EntryPoint {
    public abstract static class Function implements ValueProvider<Double, Double> {
        private final String name;
        private final Color color;
        public Function ( String name, Color color ) {
            this.name = name;
            this.color = color;
        }
        @Override
        public abstract Double getValue ( Double object );
        @Override
        public String getPath() {
            return name;
        }
        @Override
        public void setValue ( Double object, Double value ) {
        }
        public Color getColor() {
            return color;
        }
    }
    @Override
    public void onModuleLoad() {
        Viewport vp = new Viewport();
        Set<Function> functions = new HashSet<Function>();
        Function f1 = new Function ( "f1", RGB.RED ) {
            @Override
            public Double getValue ( Double x ) {
                return Math.sin ( x );
            }
        };
        functions.add ( f1 );
        Function f2 = new Function ( "f2", RGB.BLACK ) {
            @Override
            public Double getValue ( Double x ) {
                return Math.cos ( x );
            }
        };
        functions.add ( f2 );
        ListStore<Double> store = new ListStore<Double> ( new ModelKeyProvider<Double>() {
            @Override
            public String getKey ( Double item ) {
                return item.toString();
            }
        } );
        for ( Double x = 0.0; x <= 2 * Math.PI; x = x + 0.1 ) {
            store.add ( x );
        }
        Chart<Double> chart = new Chart<Double>();
        chart.setStore ( store );
        NumericAxis<Double> verticalAxis = new NumericAxis<Double>();
        verticalAxis.setPosition ( Position.LEFT );
        for ( Function func : functions ) {
            verticalAxis.addField ( func );
        }
        verticalAxis.setSteps ( 2 );
        verticalAxis.setMinorTickSteps ( 5 );
        chart.addAxis ( verticalAxis );
        NumericAxis<Double> horizAxis = new NumericAxis<Double>();
        horizAxis.setPosition ( Position.BOTTOM );
        horizAxis.addField ( new IdentityValueProvider<Double>() );
        horizAxis.setSteps ( 2 );
        horizAxis.setMinorTickSteps ( 5 );
        chart.addAxis ( horizAxis );
        for ( Function func : functions ) {
            LineSeries<Double> line = new LineSeries<Double>();
            line.setXAxisPosition ( Position.BOTTOM );
            line.setXField ( new IdentityValueProvider<Double>() );
            line.setYAxisPosition ( Position.LEFT );
            line.setYField ( func );
            line.setStroke ( func.getColor() );
            line.setStrokeWidth ( 2 );
            chart.addSeries ( line );
        }
        vp.setWidget ( chart );
        RootPanel.get().add ( vp );
    }
}
