public Long insertRows ( String projectId,
                         String datasetId,
                         String tableId,
                         InputStream schema,
                         AbstractInputStreamContent data ) {
    try {
        ObjectMapper mapper = new ObjectMapper();
        List<TableFieldSchema> schemaFields = mapper.readValue ( schema, new TypeReference<List<TableFieldSchema>>() {} );
        TableSchema tableSchema = new TableSchema().setFields ( schemaFields );
        TableReference tableReference = new TableReference()
        .setProjectId ( projectId )
        .setDatasetId ( datasetId )
        .setTableId ( tableId );
        JobConfigurationLoad loadConfig = new JobConfigurationLoad()
        .setDestinationTable ( tableReference )
        .setSchema ( tableSchema )
        .setSourceFormat ( "NEWLINE_DELIMITED_JSON" )
        .setCreateDisposition ( "CREATE_IF_NEEDED" )
        .setWriteDisposition ( "WRITE_APPEND" );
        Job loadJob = new Job()
        .setJobReference (
            new JobReference()
            .setJobId ( Joiner.on ( "-" ).join ( "INSERT", projectId, datasetId,
                        tableId, DateTime.now().toString ( "dd-MM-yyyy_HH-mm-ss-SSS" ) ) )
            .setProjectId ( projectId ) )
        .setConfiguration ( new JobConfiguration().setLoad ( loadConfig ) );
        Job createTableJob = bigquery.jobs().insert ( projectId, loadJob, data ).execute();
        String jobId = createTableJob.getJobReference().getJobId();
        createTableJob = waitForJob ( projectId, createTableJob );
        Long rowCount = createTableJob != null ? createTableJob.getStatistics().getLoad().getOutputRows() : 0l;
        log.info ( "{} rows inserted in table '{}' (dataset: '{}', project: '{}')", rowCount, tableId, datasetId, projectId );
        return rowCount;
    } catch ( IOException e ) {
        throw Throwables.propagate ( e );
    }
}
