public class TerminalService {
    private static Specification<Terminal> hasProperties ( final Map<String, String> properties ) {
        return new Specification<Terminal>() {
            @Override
            public Predicate toPredicate ( Root<Terminal> root, CriteriaQuery<?> query, CriteriaBuilder builder ) {
                Subquery<TerminalProperty> subQuery = query.subquery ( TerminalProperty.class );
                Root propertyRoot = subQuery.from ( TerminalProperty.class );
                subQuery.select ( propertyRoot.get ( "terminal.id" ) );
                Predicate whereClause = null;
                for ( Map.Entry<String, String> entry : properties.entrySet() ) {
                    Predicate predicate = builder.and ( builder.equal ( propertyRoot.get ( "key" ),
                                                        entry.getKey() ), builder.equal ( propertyRoot.get ( "value" ), entry.getValue() ) );
                    if ( whereClause == null ) {
                        whereClause = predicate;
                    } else {
                        whereClause = builder.or ( whereClause, predicate );
                    }
                }
                subQuery.where ( whereClause );
                subQuery.groupBy ( propertyRoot.get ( "terminal.id" ) );
                subQuery.having ( builder.equal ( builder.count ( propertyRoot ), properties.size() ) );
                return query.where ( builder.in ( root.get ( "id" ) ).value ( subQuery ) ).getRestriction();
            }
        };
    }
    @Autowired
    private TerminalRepository terminalRepository;
    public Iterable<Terminal> findTerminalsWith ( Map<String, String> properties ) {
        return terminalRepository.findAll ( hasProperties ( properties ) );
    }
}
