import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.Arrays;
import javax.swing.JOptionPane;
public class PhoneBook extends Frame implements ActionListener, ItemListener {
    private static final long serialVersionUID = 1L;
    MenuItem newMI, openMI, saveMI, saveAsMI, exitMI;
    MenuItem searchMI, deleteMI, updateMI, newEntryMI, sortMI;
    String fileName;
    List entries;
    TextField lastName, firstName, phoneNumber;
    public PhoneBook() {
        super ( "White Pages" );
        setLayout ( new BorderLayout() );
        MenuBar menubar = new MenuBar();
        setMenuBar ( menubar );
        Menu fileMenu = new Menu ( "File" );
        menubar.add ( fileMenu );
        newMI = fileMenu.add ( new MenuItem ( "New" ) );
        newMI.addActionListener ( this );
        openMI = fileMenu.add ( new MenuItem ( "Open" ) );
        openMI.addActionListener ( this );
        fileMenu.addSeparator();
        saveMI = fileMenu.add ( new MenuItem ( "Save" ) );
        saveAsMI = fileMenu.add ( new MenuItem ( "Save As ..." ) );
        fileMenu.addSeparator();
        exitMI = fileMenu.add ( new MenuItem ( "Exit" ) );
        exitMI.addActionListener ( this );
        Menu editMenu = new Menu ( "Edit" );
        menubar.add ( editMenu );
        updateMI = editMenu.add ( new MenuItem ( "Update" ) );
        updateMI.addActionListener ( this );
        newEntryMI = editMenu.add ( new MenuItem ( "New Entry" ) );
        newEntryMI.addActionListener ( this );
        deleteMI = editMenu.add ( new MenuItem ( "Delete" ) );
        deleteMI.addActionListener ( this );
        editMenu.addSeparator();
        searchMI = editMenu.add ( new MenuItem ( "Search" ) );
        searchMI.addActionListener ( this );
        sortMI = editMenu.add ( new MenuItem ( "Sort" ) );
        sortMI.addActionListener ( this );
        Panel listPanel = new Panel ( new BorderLayout() );
        add ( listPanel, BorderLayout.CENTER );
        Label label = new Label ( "Name List", Label.LEFT );
        listPanel.add ( label, BorderLayout.NORTH );
        entries = new List();
        entries.addItemListener ( this );
        listPanel.add ( entries, BorderLayout.CENTER );
        Panel panel = new Panel ( new BorderLayout() );
        add ( panel, BorderLayout.WEST );
        Panel editPanel = new Panel ( new GridLayout ( 6, 1 ) );
        panel.add ( editPanel, BorderLayout.NORTH );
        label = new Label ( "Last Name", Label.LEFT );
        editPanel.add ( label );
        lastName = new TextField();
        editPanel.add ( lastName );
        label = new Label ( "First Name", Label.LEFT );
        editPanel.add ( label );
        firstName = new TextField();
        editPanel.add ( firstName );
        label = new Label ( "Phone Number", Label.LEFT );
        editPanel.add ( label );
        phoneNumber = new TextField();
        editPanel.add ( phoneNumber );
    }
    public void actionPerformed ( ActionEvent event ) {
        Object source = event.getSource();
        if ( source == newMI ) {
            entries.removeAll();
            fileName = null;
            display ( -1 );
            setTitle ( "White Pages" );
        } else if ( source == openMI ) {
            doOpen();
        } else if ( source == exitMI ) {
            System.exit ( 0 );
        } else if ( source == updateMI ) {
            int index = entries.getSelectedIndex();
            String entry = String.format ( "%s %s %s", lastName.getText().trim(),
                                           firstName.getText().trim(), phoneNumber.getText().trim() );
            if ( index < 0 ) {
                entries.add ( entry );
                entries.select ( entries.getItemCount() - 1 );
            } else {
                entries.replaceItem ( entry, index );
                entries.select ( index );
            }
        } else if ( source == newEntryMI ) {
            entries.select ( -1 );
            display ( -1 );
        } else if ( source == searchMI ) {
            String searchName = JOptionPane.showInputDialog ( this,
                                "Please enter a name (last first) to search:" );
            System.out.println ( "Name to search: " + searchName );
        } else if ( source == deleteMI ) {
            int index = entries.getSelectedIndex();
            if ( index >= 0 ) {
                entries.remove ( index );
            }
        } else if ( source == sortMI ) {
            String[] sortedEntries = entries.getItems();
            Arrays.sort ( sortedEntries );
            entries.removeAll();
            for ( String entry : sortedEntries ) {
                entries.add ( entry );
            }
        }
    }
    public void itemStateChanged ( ItemEvent event ) {
        display ( entries.getSelectedIndex() );
    }
    private void doOpen() {
        FileDialog fDialog = new FileDialog ( this, "Open ...", FileDialog.LOAD );
        fDialog.setVisible ( true );
        String file = fDialog.getFile();
        if ( file == null ) {
            return;
        }
        fileName = fDialog.getDirectory() + file;
        FileReader reader;
        try {
            reader = new FileReader ( fileName );
        } catch ( FileNotFoundException ex ) {
            JOptionPane.showMessageDialog ( this, "File Not Found: " + fileName,
                                            "Error", JOptionPane.ERROR_MESSAGE );
            doOpen();
            return;
        }
        BufferedReader bReader = new BufferedReader ( reader );
        entries.removeAll();
        try {
            int index;
            String textLine = bReader.readLine();
            while ( textLine != null ) {
                index = textLine.indexOf ( ( int ) ',' );
                if ( index > 0 ) {
                    String name = textLine.substring ( 0, index );
                    String number = textLine.substring ( index + 1 );
                    entries.add ( String.format ( "%s %s", name.trim(),
                                                  number.trim() ) );
                }
                textLine = bReader.readLine();
            }
            bReader.close();
            reader.close();
        } catch ( IOException ioe ) {
            JOptionPane.showMessageDialog ( this,
                                            "Error reading file: " + ioe.toString(), "Error",
                                            JOptionPane.ERROR_MESSAGE );
            return;
        }
        setTitle ( "White Pages: " + file );
        entries.select ( 0 );
        display ( 0 );
    }
    private void display ( int index ) {
        if ( index < 0 ) {
            lastName.setText ( "" );
            firstName.setText ( "" );
            phoneNumber.setText ( "" );
        } else {
            String[] fields = entries.getItem ( index ).split ( "(,|\\s)" );
            if ( fields.length > 0 ) {
                lastName.setText ( fields[0] );
            }
            if ( fields.length > 1 ) {
                firstName.setText ( fields[1] );
            }
            if ( fields.length > 2 ) {
                phoneNumber.setText ( fields[2] );
            }
        }
    }
    public static void main ( String[] argv ) {
        System.out.println ( "Creating window ... " );
        PhoneBook frame = new PhoneBook();
        Dimension size = Toolkit.getDefaultToolkit().getScreenSize();
        frame.setSize ( size.width / 2, size.height / 2 );
        frame.setLocation ( 100, 100 );
        System.out.println ( "Your Screen Size: " + size.width + " (width) x "
                             + size.height + " (height)" );
        frame.addWindowListener ( new WindowAdapter() {
            public void windowClosing ( WindowEvent e ) {
                System.exit ( 0 );
            }
        } );
        frame.setVisible ( true );
    }
}
