import javax.swing.*;
class SplashTest {
    static boolean useThread = false;
    public static void main ( String[] args ) {
        if ( args.length > 0 ) {
            useThread = new Boolean ( args[0] );
        }
        SwingUtilities.invokeLater ( new Runnable() {
            @Override
            public void run() {
                SplashFrame splashFrame = new SplashFrame();
                splashFrame.setVisible ( true );
                new AppFrame ( splashFrame ).setVisible ( true );
            }
        } );
    }
    private static class BaseFrame extends JFrame {
        public BaseFrame() {
            setDefaultCloseOperation ( JFrame.DISPOSE_ON_CLOSE );
            setSize ( 200, 200 );
            setTitle ( getClass().getSimpleName() );
        }
    }
    private static class SplashFrame extends BaseFrame implements AppFrameWorkListener {
        JLabel status;
        public SplashFrame() {
            setLocation ( 0, 0 );
            status = new JLabel ( "Splash Frame" );
            getContentPane().add ( status );
        }
        public void appFrameWorkStart() {
            status.setText ( "Work started" );
        }
        public void appFrameWorkProgress ( long timeElapsed ) {
            status.setText ( "Work has taken " + timeElapsed + "ms so far" );
        }
        public void appFrameWorkDone() {
            setVisible ( false );
            dispose();
        }
    }
    private static class AppFrame extends BaseFrame {
        JLabel status;
        AppFrameWorkListener listener;
        public AppFrame ( AppFrameWorkListener listener ) {
            setLocation ( 200, 200 );
            status = new JLabel ( "App Frame" );
            getContentPane().add ( status );
            this.listener = listener;
            if ( useThread ) {
                new Thread ( new Runnable() {
                    @Override
                    public void run() {
                        doLotsOfWork ( 4 );
                    }
                } ).start();
            } else {
                doLotsOfWork ( 4 );
                onWorkDone();
            }
        }
        private void doLotsOfWork ( int workLengthSeconds ) {
            invokeOnWorkStartOnEDT();
            long start = System.currentTimeMillis();
            long workLengthMs = workLengthSeconds * 1000;
            while ( System.currentTimeMillis() - start < workLengthMs ) {
                long innerStart = System.currentTimeMillis();
                while ( System.currentTimeMillis() - innerStart < 250 );
                invokeOnWorkProgressOnEDT ( System.currentTimeMillis() - start );
            }
            invokeOnWorkDoneOnEDT();
        }
        private void invokeOnWorkStartOnEDT() {
            SwingUtilities.invokeLater ( new Runnable() {
                @Override
                public void run() {
                    onWorkStart();
                }
            } );
        }
        private void invokeOnWorkProgressOnEDT ( final long timeElapsed ) {
            SwingUtilities.invokeLater ( new Runnable() {
                @Override
                public void run() {
                    onWorkProgress ( timeElapsed );
                }
            } );
        }
        private void invokeOnWorkDoneOnEDT() {
            SwingUtilities.invokeLater ( new Runnable() {
                @Override
                public void run() {
                    onWorkDone();
                }
            } );
        }
        private void onWorkStart() {
            status.setText ( "Work Started" );
            if ( null != listener ) {
                listener.appFrameWorkStart();
            }
        }
        private void onWorkProgress ( long timeElapsed ) {
            status.setText ( "Work has taken " + timeElapsed + "ms so far" );
            if ( null != listener ) {
                listener.appFrameWorkProgress ( timeElapsed );
            }
        }
        private void onWorkDone() {
            status.setText ( "Work Done" );
            if ( null != listener ) {
                listener.appFrameWorkDone();
            }
        }
    }
    interface AppFrameWorkListener {
        public void appFrameWorkDone();
        public void appFrameWorkStart();
        public void appFrameWorkProgress ( long timeElapsed );
    }
}
