import io.netty.bootstrap.ServerBootstrap;
import io.netty.channel.ChannelFuture;
import io.netty.channel.ChannelInitializer;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.SocketChannel;
import io.netty.channel.socket.nio.NioServerSocketChannel;
import io.netty.handler.codec.LineBasedFrameDecoder;
import io.netty.handler.codec.string.StringDecoder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import java.nio.charset.Charset;
public class MultiHandlerServer {
    private static final Logger logger = LoggerFactory.getLogger ( MultiHandlerServer.class );
    final int port;
    public MultiHandlerServer ( final int port ) {
        this.port = port;
    }
    public void run() throws InterruptedException {
        final NioEventLoopGroup bossGroup = new NioEventLoopGroup();
        final NioEventLoopGroup workerGroup = new NioEventLoopGroup();
        try {
            final ServerBootstrap serverBootstrap = new ServerBootstrap()
            .group ( bossGroup, workerGroup )
            .channel ( NioServerSocketChannel.class )
            .childHandler ( new ChannelInitializer<SocketChannel>() {
                @Override
                protected void initChannel ( SocketChannel ch ) throws Exception {
                    ch.pipeline().addLast (
                        new LineBasedFrameDecoder ( 8192 ),
                        new StringDecoder ( Charset.forName ( "UTF-8" ) ),
                        new MultiHandler01(), new MultiHandler02() );
                }
            } );
            final ChannelFuture future = serverBootstrap.bind ( port ).sync();
            future.channel().closeFuture().sync();
        } finally {
            bossGroup.shutdownGracefully();
            workerGroup.shutdownGracefully();
        }
    }
    public static void main ( String[] args ) throws InterruptedException {
        final MultiHandlerServer client = new MultiHandlerServer ( 8080 );
        client.run();
    }
}
