class Mp3FileXuggler {
    private boolean DEBUG = true;
    private String _sInputFileName;
    private IContainer _inputContainer;
    private int _iBitRate;
    private IPacket _packet;
    private int _iAudioStreamId;
    private IStreamCoder _audioCoder;
    private int _iSampleBufferSize;
    private int _iInputSampleRate;
    private static SourceDataLine mLine;
    private int DECODED_AUDIO_SECOND_SIZE = 176375;
    private int _bytesPerPacket;
    private byte[] _residualBuffer;
    public Mp3FileXuggler ( String sFileName ) throws UnsuportedSampleRateException {
        this._sInputFileName = sFileName;
        this._inputContainer = IContainer.make();
        this._iSampleBufferSize = 18432;
        this._residualBuffer = null;
        if ( this._inputContainer.open ( this._sInputFileName, IContainer.Type.READ, null ) < 0 ) {
            throw new IllegalArgumentException ( "Could not read the file: " + this._sInputFileName );
        }
        int iNumStreams = this._inputContainer.getNumStreams();
        this._iBitRate = this._inputContainer.getBitRate();
        if ( DEBUG ) {
            System.out.println ( "Bitrate: " + this._iBitRate );
        }
        this._iAudioStreamId = -1;
        this._audioCoder = null;
        boolean bFound = false;
        int i = 0;
        while ( i < iNumStreams && bFound == false ) {
            IStream stream = this._inputContainer.getStream ( i );
            IStreamCoder coder = stream.getStreamCoder();
            if ( coder.getCodecType() == ICodec.Type.CODEC_TYPE_AUDIO ) {
                this._iAudioStreamId = i;
                this._audioCoder = coder;
                this._iInputSampleRate = coder.getSampleRate();
                bFound = true;
            }
            ++i;
        }
        if ( this._iAudioStreamId == -1 ) {
            throw new RuntimeException ( "Could not find audio stream in container: " + this._sInputFileName );
        }
        if ( this._audioCoder.open ( null, null ) < 0 ) {
            throw new RuntimeException ( "could not open audio decoder for container: " + this._sInputFileName );
        }
        this._packet = IPacket.make();
        this._inputContainer.readNextPacket ( this._packet );
        switch ( this._iInputSampleRate ) {
        case 22050:
            this._bytesPerPacket = 2304;
            break;
        case 44100:
            this._bytesPerPacket = 4608;
            break;
        }
    }
    public byte[] getSamples() {
        byte[] rawBytes = null;
        while ( this._inputContainer.readNextPacket ( this._packet ) >= 0 ) {
            if ( this._packet.getStreamIndex() == this._iAudioStreamId ) {
                IAudioSamples samples = IAudioSamples.make ( this._iSampleBufferSize, this._audioCoder.getChannels() );
                int iCurrentOffset = 0;
                while ( iCurrentOffset < this._packet.getSize() ) {
                    int iBytesDecoded = this._audioCoder.decodeAudio ( samples, this._packet, iCurrentOffset );
                    iCurrentOffset += iBytesDecoded;
                    if ( samples.isComplete() ) {
                        rawBytes = samples.getData().getByteArray ( 0, samples.getSize() );
                    }
                }
                return rawBytes;
            } else {
                do {} while ( false );
            }
        }
        return rawBytes;
    }
