public class ModalInternalPanel {
    public static  Object showInternalDialog ( Component parentComponent, Container container, HasReturnValue hasReturnValue, String title )  {
        JInternalFrame frame = createInternalFrame ( parentComponent, title, container );
        startModal ( frame );
        if ( hasReturnValue != null ) {
            return hasReturnValue.getReturnValue();
        } else {
            return null;
        }
    }
    public static JInternalFrame createInternalFrame ( Component parentComponent, String title, Container container ) throws RuntimeException  {
        JLayeredPane toUse = JOptionPane.getDesktopPaneForComponent ( parentComponent );
        if ( toUse == null ) {
            toUse = JLayeredPane.getLayeredPaneAbove ( parentComponent );
        }
        if ( toUse == null ) {
            throw new RuntimeException ( "parentComponent does not have a valid parent" );
        }
        JInternalFrame frame = new JInternalFrame ( title );
        frame.setContentPane ( container );
        frame.setClosable ( true );
        toUse.add ( frame );
        frame.setLayer ( JLayeredPane.MODAL_LAYER );
        frame.pack();
        frame.setVisible ( true );
        return frame;
    }
    private static void startModal ( JInternalFrame f ) {
        JPanel modalInterceptor = new JPanel();
        modalInterceptor.setOpaque ( false );
        JLayeredPane lp = JLayeredPane.getLayeredPaneAbove ( f );
        lp.setLayer ( modalInterceptor, JLayeredPane.MODAL_LAYER.intValue() );
        modalInterceptor.setBounds ( 0, 0, lp.getWidth(), lp.getHeight() );
        modalInterceptor.addMouseListener ( new MouseAdapter() {} );
        modalInterceptor.addMouseMotionListener ( new MouseMotionAdapter() {} );
        lp.add ( modalInterceptor );
        f.toFront();
        EventQueue queue = Toolkit.getDefaultToolkit().getSystemEventQueue();
        try {
            while ( ! f.isClosed() )       {
                if ( EventQueue.isDispatchThread() )    {
                    AWTEvent ev = queue.getNextEvent();
                    if ( ev instanceof ActiveEvent ) {
                        ( ( ActiveEvent ) ev ).dispatch();
                    } else if ( ev.getSource() instanceof Component ) {
                        ( ( Component ) ev.getSource() ).dispatchEvent ( ev );
                    } else if ( ev.getSource() instanceof MenuComponent ) {
                        ( ( MenuComponent ) ev.getSource() ).dispatchEvent ( ev );
                    }
                } else  {
                    Thread.yield();
                }
            }
        } catch ( InterruptedException ex ) {
        } finally {
            lp.remove ( modalInterceptor );
            Container parent = f.getParent();
            if ( parent != null ) {
                parent.remove ( f );
            }
        }
    }
}
