import java.awt.*;
import java.awt.event.*;
import java.util.ArrayList;
import java.util.List;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.*;
import javax.swing.table.*;
import javax.swing.UIManager.LookAndFeelInfo;
public class TestJTableMultiline2 {
    public JComponent makeUI() {
        String[] columnNames = {"A", "B", "C"};
        Object[][] data = {
            {"A0, Line1\nA0, Line2\nA0, Line3", "B0, Line1\nB0, Line2", "C0, Line1"},
            {"A1, Line1", "B1, Line1\nB1, Line2", "C1, Line1"},
            {"A2, Line1", "B2, Line1", "C2, Line1"}
        };
        DefaultTableModel model = new DefaultTableModel ( data, columnNames ) {
            @Override public Class<?> getColumnClass ( int column ) {
                return String.class;
            }
            @Override public boolean isCellEditable ( int row, int column ) {
                return false;
            }
        };
        JTable table = new JTable ( model );
        table.setAutoCreateRowSorter ( true );
        MultiLineTableCellRenderer r = new MultiLineTableCellRenderer();
        UIDefaults d = new UIDefaults();
        d.put ( "TextArea.borderPainter", new Painter() {
            @Override public void paint ( Graphics2D g, Object o, int w, int h ) {}
        } );
        r.putClientProperty ( "Nimbus.Overrides", d );
        r.putClientProperty ( "Nimbus.Overrides.InheritDefaults", false );
        table.setDefaultRenderer ( String.class, r );
        return new JScrollPane ( table );
    }
    public static void main ( String[] args ) {
        EventQueue.invokeLater ( new Runnable() {
            @Override public void run() {
                createAndShowGUI();
            }
        } );
    }
    public static void createAndShowGUI() {
        try {
            for ( LookAndFeelInfo info : UIManager.getInstalledLookAndFeels() ) {
                if ( "Nimbus".equals ( info.getName() ) ) {
                    UIManager.setLookAndFeel ( info.getClassName() );
                    break;
                }
            }
        } catch ( Exception e ) {
            e.printStackTrace();
        }
        JFrame f = new JFrame ( "Multi-Line Cell Example 2" );
        f.setDefaultCloseOperation ( WindowConstants.EXIT_ON_CLOSE );
        f.getContentPane().add ( new TestJTableMultiline2().makeUI() );
        f.setSize ( 320, 240 );
        f.setLocationRelativeTo ( null );
        f.setVisible ( true );
    }
}
class MultiLineTableCellRenderer extends JTextArea implements TableCellRenderer {
    private List<List<Integer>> rowColHeight = new ArrayList<List<Integer>>();
    public MultiLineTableCellRenderer() {
        super();
        setLineWrap ( true );
        setWrapStyleWord ( true );
        setOpaque ( true );
    }
    @Override
    public Component getTableCellRendererComponent (
        JTable table, Object value, boolean isSelected,
        boolean hasFocus, int row, int column ) {
        setFont ( table.getFont() );
        setText ( value != null ? value.toString() : "" );
        setMargin ( new Insets ( 2, 5, 2, 5 ) );
        if ( isSelected ) {
            setForeground ( table.getSelectionForeground() );
            setBackground ( table.getSelectionBackground() );
        } else {
            setForeground ( table.getForeground() );
            setBackground ( table.getBackground() );
        }
        adjustRowHeight ( table, row, column );
        return this;
    }
    private void adjustRowHeight ( JTable table, int row, int column ) {
        int cWidth = table.getCellRect ( row, column, false ).width;
        setSize ( new Dimension ( cWidth, 1000 ) );
        int prefH = getPreferredSize().height;
        while ( rowColHeight.size() <= row ) {
            rowColHeight.add ( new ArrayList<Integer> ( column ) );
        }
        List<Integer> colHeights = rowColHeight.get ( row );
        while ( colHeights.size() <= column ) {
            colHeights.add ( 0 );
        }
        colHeights.set ( column, prefH );
        int maxH = prefH;
        for ( Integer colHeight : colHeights ) {
            if ( colHeight > maxH ) {
                maxH = colHeight;
            }
        }
        if ( table.getRowHeight ( row ) != maxH ) {
            table.setRowHeight ( row, maxH );
        }
    }
}
