public static void main ( String[] args ) {
    final Display display = new Display();
    final Shell shell = new Shell ( display );
    shell.setLayout ( new FillLayout() );
    TableViewer viewer = new TableViewer ( shell );
    for ( int i = 0; i < 4; i++ ) {
        TableViewerColumn column = new TableViewerColumn ( viewer, SWT.NONE );
        column.setLabelProvider ( new TrimLabelProvider() );
        column.getColumn().setText ( "Column " + i );
    }
    viewer.setContentProvider ( new ArrayContentProvider() );
    viewer.setInput ( DataModelProvider.getInstance().getItems() );
    Table table = viewer.getTable();
    table.setHeaderVisible ( true );
    table.setLinesVisible ( true );
    for ( TableColumn column : table.getColumns() ) {
        column.pack();
    }
    shell.pack();
    shell.open();
    while ( !shell.isDisposed() ) {
        if ( !display.readAndDispatch() ) {
            display.sleep();
        }
    }
    display.dispose();
}
private static class DataModelProvider {
    private static DataModelProvider    INSTANCE;
    private List<TableRow>              items;
    public static DataModelProvider getInstance() {
        if ( INSTANCE == null ) {
            INSTANCE = new DataModelProvider();
        }
        return INSTANCE;
    }
    private DataModelProvider() {
        items = new ArrayList<TableRow>();
        items.add ( new TableRow ( new String[] { "first", "second", "third", "fourth" } ) );
        items.add ( new TableRow ( new String[] { "first", "second second second second second second second second", "third", "fourth" } ) );
    }
    public void addItem ( TableRow item ) {
        items.add ( item );
    }
    public List<TableRow> getItems() {
        return items;
    }
}
private static class TableRow {
    private String[]    data;
    public TableRow ( String[] input ) {
        data = input;
    }
    public String getData ( int index ) {
        if ( index < 0 || index > data.length - 1 ) {
            throw new IllegalArgumentException ( "Invalid index: " + index + ". Minimum: 0, Maximum: " + ( data.length - 1 ) );
        }
        return data[index];
    }
}
private static class TrimLabelProvider extends CellLabelProvider {
    @Override
    public void update ( ViewerCell cell ) {
        Object element = cell.getElement();
        if ( element instanceof TableRow ) {
            int columnIndex = cell.getColumnIndex();
            TableRow row = ( TableRow ) cell.getElement();
            cell.setText ( shortenText ( row.getData ( columnIndex ) ) );
        }
    }
    private String shortenText ( String text ) {
        int value = 20;
        if ( text.length() > value ) {
            int index = text.indexOf ( " ", value );
            if ( index != -1 ) {
                return text.substring ( 0, index ) + " [...]";
            }
        }
        return text;
    }
}
