abstract class LoopEvaluator<T> {
    public boolean eval ( List<T> list ) {
        for ( T element : list ) {
            if ( this.evalElement ( element ) ) {
                return true;
            }
        }
        return false;
    }
    abstract boolean evalElement ( T element );
}
class LookForLeaders<T extends Member> extends LoopEvaluator<T> {
    @override
    public bool evalElement ( T element ) {
        return element.getRole().equals ( "Leader" );
    }
}
class LookForTwoNewJoiners extends LoopEvaluator<T> {
    private int newJoiners;
    @override
    public boolean eval ( List<T> list ) {
        this.newJoiners = 0;
        return super.eval ( list );
    }
    @override
    public boolean evalElement ( T element ) {
        DateTime aMonthAgo = DateTime.now().minusMonths ( 1 );
        if ( member.startingDate().isAfter ( aMonthAgo ) ) {
            this.newJoiners++;
        }
        return newJoiners == 2;
    }
}
