import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import javax.swing.*;
import javax.swing.event.MouseInputAdapter;
class TestCanvas extends JComponent {
    float x0 = 150f, y0 = 150f;
    float xa = 200f, ya = 150f;
    float xd = 150f, yd =  50f;
    float dist0 ( float x, float y ) {
        return ( float ) Math.sqrt ( sqr ( x - x0 ) + sqr ( y - y0 ) );
    }
    float angle0 ( float x, float y ) {
        return ( float ) Math.toDegrees ( Math.atan2 ( y0 - y, x - x0 ) );
    }
    @Override
    protected void paintComponent ( Graphics g0 ) {
        Graphics2D g = ( Graphics2D ) g0;
        dot ( g, x0, y0 );
        float ra = dist0 ( xa, ya );
        float rd = dist0 ( xd, yd );
        if ( ra == 0 || rd == 0 ) {
            return;
        }
        float aa = angle0 ( xa, ya );
        float ad = angle0 ( xd, yd );
        g.draw ( new Arc2D.Float ( x0 - ra, y0 - ra,
                                   2 * ra, 2 * ra,
                                   aa, angleDiff ( aa, ad ),
                                   Arc2D.OPEN ) );
        dot ( g, xa, ya );
        float xb = x0 + ( xd - x0 ) * ra / rd;
        float yb = y0 + ( yd - y0 ) * ra / rd;
        dot ( g, xb, yb );
    }
    static void dot ( Graphics2D g, float x, float y ) {
        final int rad = 2;
        g.fill ( new Ellipse2D.Float ( x - rad, y - rad, 2 * rad, 2 * rad ) );
    }
    static float sqr ( float x ) {
        return x * x;
    }
    static float angleDiff ( float a, float b ) {
        float d = b - a;
        while ( d >= 180f ) {
            d -= 360f;
        }
        while ( d < -180f ) {
            d += 360f;
        }
        return d;
    }
    TestCanvas() {
        addMouseListener ( mouseListener );
        addMouseMotionListener ( mouseListener );
    }
    MouseInputAdapter mouseListener = new MouseInputAdapter() {
        boolean mouseDown = false;
        @Override
        public void mousePressed ( MouseEvent e ) {
            if ( e.getButton() == MouseEvent.BUTTON1 ) {
                mouseDown = true;
                xa = xd = e.getX();
                ya = yd = e.getY();
                repaint();
            }
        }
        @Override
        public void mouseReleased ( MouseEvent e ) {
            if ( e.getButton() == MouseEvent.BUTTON1 ) {
                mouseDown = false;
            }
        }
        @Override
        public void mouseDragged ( MouseEvent e ) {
            if ( mouseDown ) {
                xd = e.getX();
                yd = e.getY();
                repaint();
            }
        }
    };
}
public class Test extends JFrame {
    public Test() {
        setSize ( 400, 400 );
        setLocationRelativeTo ( null );
        setDefaultCloseOperation ( JFrame.EXIT_ON_CLOSE );
        getContentPane().add ( new TestCanvas() );
    }
    public static void main ( String[] args ) {
        SwingUtilities.invokeLater ( new Runnable() {
            @Override
            public void run() {
                new Test().setVisible ( true );
            }
        } );
    }
}
