import java.io.File;
import java.sql.Connection;
import java.util.List;
import com.northconcepts.datapipeline.core.DataReader;
import com.northconcepts.datapipeline.core.DataWriter;
import com.northconcepts.datapipeline.core.FieldList;
import com.northconcepts.datapipeline.core.Record;
import com.northconcepts.datapipeline.core.RecordList;
import com.northconcepts.datapipeline.csv.CSVReader;
import com.northconcepts.datapipeline.csv.CSVWriter;
import com.northconcepts.datapipeline.job.JobTemplate;
import com.northconcepts.datapipeline.transform.TransformingReader;
import com.northconcepts.datapipeline.transform.lookup.JdbcLookup;
import com.northconcepts.datapipeline.transform.lookup.Lookup;
import com.northconcepts.datapipeline.transform.lookup.LookupTransformer;
public class CompareOracleToCsv {
    public static void main ( String[] args ) {
        Connection connection = null;
        String lookupSql = "SELECT * FROM NSA_AL.POLVEHICLE PV" +
                           " WHERE PV.VIN=?" +
                           " AND PV.POLICYID=?" +
                           " AND PV.SEGEFFDATE<=TO_DATE(?,'MM/DD/YYYY HH24:MI')" +
                           " AND PV.SEGEXPDATE>=TO_DATE(?,'MM/DD/YYYY HH24:MI')" +
                           " AND PV.SEGSTATUS= '1'";
        DataReader reader = new CSVReader ( new File ( "input.csv" ) )
        .setFieldNamesInFirstRow ( true );
        reader = new TransformingReader ( reader )
        .add ( new LookupTransformer2 (
                   new FieldList ( "vin", "policyNumber", "verificationDate", "verificationDate" ),
                   new JdbcLookup ( connection, lookupSql ) ) );
        DataWriter writer = new CSVWriter ( new File ( "output.csv" ) )
        .setFieldNamesInFirstRow ( true );
        JobTemplate.DEFAULT.transfer ( reader, writer );
        System.out.println ( "Done" );
    }
    public static class LookupTransformer2 extends LookupTransformer {
        public LookupTransformer2 ( FieldList fields, Lookup lookup ) {
            super ( fields, lookup );
        }
        @Override
        protected RecordList noResults ( Record originalRecord, List<?> arguments ) {
            originalRecord.delete();
            return null;
        }
    }
    public static class LookupTransformer3 extends LookupTransformer {
        public LookupTransformer3 ( FieldList fields, Lookup lookup ) {
            super ( fields, lookup );
        }
        @Override
        protected void join ( Record originalRecord, RecordList lookupResult, List<?> lookupArguments ) {
            if ( lookupResult == null || lookupResult.getRecordCount() == 0 ) {
                Record record = new Record();
                record.getField ( "found", true ).setValue ( false );
                lookupResult = new RecordList ( record );
            } else {
                for ( Record record : lookupResult.getRecords() ) {
                    record.getField ( "found", true ).setValue ( true );
                }
            }
            super.join ( originalRecord, lookupResult, lookupArguments );
        }
    }
}
