@ThreadSafe
public abstract class CachedI18NMessageBundle
    extends I18NMessageBundle {
    @GuardedBy ( "lock" )
    private final Map<Locale, FutureTask<MessageSource>> lookups
        = new HashMap<Locale, FutureTask<MessageSource>>();
    private final Lock lock = new ReentrantLock();
    @Override
    protected final List<MessageSource> getSources ( final Locale locale ) {
        FutureTask<MessageSource> task;
        lock.lock();
        try {
            task = lookups.get ( locale );
            if ( task == null ) {
                task = lookupTask ( locale );
                lookups.put ( locale, task );
                task.run();
            }
        } finally {
            lock.unlock();
        }
        try {
            return Arrays.asList ( task.get() );
        } catch ( ExecutionException ignored ) {
            return Collections.emptyList();
        } catch ( InterruptedException  ignored ) {
            return Collections.emptyList();
        }
    }
    protected abstract MessageSource tryAndLookup ( final Locale locale )
    throws IOException;
    @Override
    public final Builder modify() {
        throw new IllegalStateException ( "cached bundles cannot be modified" );
    }
    private FutureTask<MessageSource> lookupTask ( final Locale locale ) {
        final Callable<MessageSource> callable = new Callable<MessageSource>() {
            @Override
            public MessageSource call()
            throws IOException {
                return tryAndLookup ( locale );
            }
        };
        return new FutureTask<MessageSource> ( callable );
    }
}
