import android.annotation.SuppressLint;
import android.content.Context;
import android.content.res.TypedArray;
import android.util.AttributeSet;
import android.util.Log;
import android.view.View;
import android.view.ViewGroup;
import com.cgtag.R;
public class TilesLayout extends ViewGroup {
    private static String   TAG = TilesLayout.class.getName();
    public static class LayoutParams extends ViewGroup.LayoutParams {
        private int _row_span;
        private int _column_span;
        private int _x;
        private int _y;
        private int _width;
        private int _height;
        public int getRowSpan() {
            return _row_span;
        }
        public void setRowSpan ( int row_span ) {
            this._row_span = Math.max ( 1, row_span );
        }
        public int getColumnSpan() {
            return _column_span;
        }
        public void setColumnSpan ( int column_span ) {
            this._column_span = Math.max ( 1, column_span );
        }
        public LayoutParams() {
            super ( 0, 0 );
            Init();
        }
        public LayoutParams ( Context context, AttributeSet attrs ) {
            super ( context, attrs );
            Init();
            TypedArray a = context.getTheme().obtainStyledAttributes ( attrs, R.styleable.TilesLayout_Layout, 0, 0 );
            try {
                setRowSpan ( a.getInt ( R.styleable.TilesLayout_Layout_row_span, 1 ) );
                setColumnSpan ( a.getInt ( R.styleable.TilesLayout_Layout_column_span, 1 ) );
            } finally {
                a.recycle();
            }
        }
        public LayoutParams ( int w, int h ) {
            super ( w, h );
            Init();
        }
        private void Init() {
            _row_span = 1;
            _column_span = 1;
            _x = _y = _width = _height = 0;
        }
    }
    private int _column_width;
    public TilesLayout ( Context context, AttributeSet attrs ) {
        super ( context, attrs );
        TypedArray a = context.getTheme().obtainStyledAttributes ( attrs, R.styleable.TilesLayout, 0, 0 );
        try {
            _column_width = a.getDimensionPixelSize ( R.styleable.TilesLayout_column_width, 200 );
        } finally {
            a.recycle();
        }
    }
    public int getColumnWidth() {
        return this._column_width;
    }
    public void setColumnWidth ( int pColumnWidth ) {
        this._column_width = pColumnWidth;
        this.invalidate();
        this.requestLayout();
    }
    @SuppressLint ( "DrawAllocation" )
    @Override
    protected void onMeasure ( int widthMeasureSpec, int heightMeasureSpec ) {
        Log.d ( TAG, "onMeasure: width - " + MeasureSpec.toString ( widthMeasureSpec ) );
        Log.d ( TAG, "onMeasure: height - " + MeasureSpec.toString ( heightMeasureSpec ) );
        int width = getPaddingLeft() + getPaddingRight() + getSuggestedMinimumWidth();
        width = Math.max ( width, MeasureSpec.getSize ( widthMeasureSpec ) );
        LayoutProcessor processor = new LayoutProcessor ( width );
        final int height = processor.MeasureAllChildren();
        for ( int i = 0, c = getChildCount(); i < c; i++ ) {
            final View child = getChildAt ( i );
            final TilesLayout.LayoutParams params = ( TilesLayout.LayoutParams ) child.getLayoutParams();
            final int cellWidthSpec = MeasureSpec.makeMeasureSpec ( params._width, MeasureSpec.AT_MOST );
            final int cellHeightSpec = MeasureSpec.makeMeasureSpec ( params._height, MeasureSpec.AT_MOST );
            child.measure ( cellWidthSpec, cellHeightSpec );
        }
        setMeasuredDimension ( width, height );
    }
    @Override
    protected int getSuggestedMinimumWidth() {
        return Math.max ( super.getSuggestedMinimumWidth(), this._column_width );
    }
    @Override
    protected int getSuggestedMinimumHeight() {
        return Math.max ( super.getSuggestedMinimumHeight(), this._column_width );
    }
    @Override
    protected void onLayout ( boolean changed, int left, int top, int right, int bottom ) {
        for ( int i = 0, c = getChildCount(); i < c; i++ ) {
            final View child = getChildAt ( i );
            TilesLayout.LayoutParams params = ( TilesLayout.LayoutParams ) child.getLayoutParams();
            child.layout ( params._x, params._y, params._x + params._width, params._y + params._height );
        }
    }
    @Override
    protected boolean checkLayoutParams ( ViewGroup.LayoutParams p ) {
        return p instanceof LayoutParams;
    }
    @Override
    protected ViewGroup.LayoutParams generateDefaultLayoutParams() {
        return new TilesLayout.LayoutParams ( LayoutParams.MATCH_PARENT, LayoutParams.MATCH_PARENT );
    }
    @Override
    public ViewGroup.LayoutParams generateLayoutParams ( AttributeSet attrs ) {
        return new TilesLayout.LayoutParams ( getContext(), attrs );
    }
    @Override
    protected ViewGroup.LayoutParams generateLayoutParams ( ViewGroup.LayoutParams p ) {
        return new TilesLayout.LayoutParams ( p.width, p.height );
    }
    private class LayoutProcessor {
        private int _column_count;
        private int _column_size;
        private int _columns[];
        public LayoutProcessor ( int width ) {
            _column_count = Math.max ( 1, ( ( int ) ( width / _column_width ) ) );
            _column_size = width / _column_count;
            _columns = new int[_column_count];
        }
        private boolean childFits ( View child, int row, int column ) {
            TilesLayout.LayoutParams params = ( TilesLayout.LayoutParams ) child.getLayoutParams();
            for ( int i = column, c = column + Math.min ( _column_count, params.getColumnSpan() ); i < c; i++ ) {
                if ( i == _column_count || _columns[i] > row ) {
                    return false;
                }
            }
            return true;
        }
        private int MeasureChild ( View child, int row, int column ) {
            TilesLayout.LayoutParams params = ( TilesLayout.LayoutParams ) child.getLayoutParams();
            final int col_span = Math.min ( _column_count, params.getColumnSpan() );
            final int row_span = params.getRowSpan();
            params._x = _column_size * column;
            params._y = _column_size * row;
            params._width = _column_size * col_span;
            params._height = _column_size * row_span;
            params._x += 4;
            params._y += 4;
            params._width -= 8;
            params._height -= 8;
            for ( int r = row, rc = row + row_span; r < rc; r++ ) {
                for ( int c = column, cc = Math.min ( column + col_span, _column_count + 1 ); c < cc; c++ ) {
                    _columns[c]++;
                }
            }
            child.setLayoutParams ( params );
            return params._y + params._height;
        }
        public int MeasureAllChildren() {
            final int child_count = getChildCount();
            boolean done[] = new boolean[child_count];
            int offset = 0, row = 0, column = 0, height = 0;
            while ( offset < child_count ) {
                int i;
                for ( i = offset; i < child_count; i++ ) {
                    if ( !done[i] ) {
                        View child = getChildAt ( i );
                        if ( childFits ( child, row, column ) ) {
                            final int bottom = MeasureChild ( child, row, column );
                            height = Math.max ( bottom, height );
                            done[i] = true;
                            break;
                        }
                    }
                }
                while ( offset < child_count && done[offset] ) {
                    offset++;
                }
                column++;
                if ( column == _column_count ) {
                    column = 0;
                    row++;
                }
            }
            return height;
        }
    }
}
