@Module()
public class MyAppModule {
    @Provides ConnectionDictonary connectionDictionary() {
        return new ConnectionDictonary ( System.getProperty ( "some.property" ) );
    }
    @Provides Util util() {
        new Util();
    }
    @Provides @Singleton DataStore store() {
        return new DataStore();
    }
    @Provides @Singleton ConnectionPool pool ( DataStore store, ConnectionDictionary dict ) {
        try {
            return DataStore.connectionPool ( dict, System.getProperty ( "pool.size" ) );
        } catch ( Exception e ) {
            throw new RuntimeException ( "Could not connect to datastore.", e );
        }
    }
}
@Module ( injects = MyRequestEndpoint.class, addsTo = MyAppModule.class )
public class MyRequestModule {
    private Request req;
    public MyRequestModule ( Request req ) {
        this.req = req;
    }
    @Provides @Singleton RequestObject request() {
        return req;
    }
    @Provides @Singleton User user ( ConnectionPool pool, Request req, Util util ) {
        try {
            Connection conn = pool.obtain();
            return util.getUser ( conn, req.get ( "user.id" ), Crypto.hash ( req.get ( "pass" ) ) );
        } catch ( UserNotFoundException e ) {
            return User.UNKNOWN;
        } catch ( Exception e ) {
            throw new RuntimeException ( "Could not obtain a user.", e );
        } finally {
            pool.release();
        }
    }
}
public class MyRequestEndpoint {
    @Inject ConnectionPool pool;
    @Inject Request req;
    public Output performService() {
        try {
            Connection conn = pool.obtain();
        } finally {
            conn.release();
        }
    }
}
public class MyApp {
    public void main ( String ... args ) {
        graph = ObjectGraph.create ( MyAppModule.class );
        new ServiceListener ( graph ).start();
    }
}
public ServiceListener {
    private final appGraph;
    public ServiceListener ( ObjectGraph appGraph ) {
        this.appGraph = appGraph;
    }
    public void serveRequest ( Request req, Response res ) {
        ObjectGraph requestGraph = MyApp.graph().plus ( new MyRequestModule ( req ) );
        Output output = requestGraph.get ( MyRequestEndpoint.class ).performService();
        Util.populateResult ( output, result );
        result.flush();
    }
}
