public class MockInitialContextFactory implements InitialContextFactory {
    private static final ThreadLocal<Context> currentContext = new ThreadLocal<Context>();
    @Override
    public Context getInitialContext ( Hashtable<?, ?> environment ) throws NamingException {
        return currentContext.get();
    }
    public static void setCurrentContext ( Context context ) {
        currentContext.set ( context );
    }
    public static void clearCurrentContext() {
        currentContext.remove();
    }
}
public class MockInitialContextRule implements TestRule {
    private final Context context;
    private MockInitialContextRule ( Context context ) {
        this.context = context;
    }
    @Override
    public Statement apply ( final Statement base, Description description ) {
        return new Statement() {
            @Override
            public void evaluate() throws Throwable {
                System.setProperty ( Context.INITIAL_CONTEXT_FACTORY, MockInitialContextFactory.class.getName() );
                MockInitialContextFactory.setCurrentContext ( context );
                try {
                    base.evaluate();
                } finally {
                    System.clearProperty ( Context.INITIAL_CONTEXT_FACTORY );
                    MockInitialContextFactory.clearCurrentContext();
                }
            }
        };
    }
}
