public class BigMultiPartEntity extends MultipartEntity {
    private ProgressListener listener;
    public BigMultiPartEntity ( final HttpMultipartMode mode, final ProgressListener listener ) {
        super ( mode );
        this.listener = listener;
    }
    public BigMultiPartEntity ( HttpMultipartMode mode, final String boundary, final Charset charset, final ProgressListener listener ) {
        super ( mode, boundary, charset );
        this.listener = listener;
    }
    @Override
    public void writeTo ( final OutputStream outstream ) throws IOException {
        super.writeTo ( new CountingOutputStream ( outstream, this.listener ) );
    }
    public interface ProgressListener {
        void transferred ( int num );
    }
    public class CountingOutputStream extends FilterOutputStream {
        private ProgressListener listener;
        private int transfered;
        public CountingOutputStream ( OutputStream out, ProgressListener listener ) {
            super ( out );
            this.listener = listener;
            this.transfered = 0;
        }
        @Override
        public void write ( byte[] b, int off, int len ) throws IOException {
            this.transfered += len;
            this.listener.transferred ( ( int ) ( ( transfered / ( float ) getContentLength() ) * 100 ) );
            out.write ( b, off, len );
        }
        @Override
        public void write ( int b ) throws IOException {
            this.transfered += b;
            this.listener.transferred ( ( int ) ( ( transfered / ( float ) getContentLength() ) * 100 ) );
            out.write ( b );
        }
    }
}
