public static <T> Type[] resolveActualTypeArgs ( Class<? extends T> offspring, Class<T> base, Type... actualArgs ) {
    assert offspring != null;
    assert base != null;
    assert actualArgs.length == 0 || actualArgs.length == offspring.getTypeParameters().length;
    if ( actualArgs.length == 0 ) {
        actualArgs = offspring.getTypeParameters();
    }
    Map<String, Type> typeVariables = new HashMap<String, Type>();
    for ( int i = 0; i < actualArgs.length; i++ ) {
        TypeVariable<?> typeVariable = ( TypeVariable<?> ) offspring.getTypeParameters() [i];
        typeVariables.put ( typeVariable.getName(), actualArgs[i] );
    }
    List<Type> ancestors = new LinkedList<Type>();
    if ( offspring.getGenericSuperclass() != null ) {
        ancestors.add ( offspring.getGenericSuperclass() );
    }
    for ( Type t : offspring.getGenericInterfaces() ) {
        ancestors.add ( t );
    }
    for ( Type type : ancestors ) {
        if ( type instanceof Class<?> ) {
            Class<?> ancestorClass = ( Class<?> ) type;
            if ( base.isAssignableFrom ( ancestorClass ) ) {
                Type[] result = resolveActualTypeArgs ( ( Class<? extends T> ) ancestorClass, base );
                if ( result != null ) {
                    return result;
                }
            }
        }
        if ( type instanceof ParameterizedType ) {
            ParameterizedType parameterizedType = ( ParameterizedType ) type;
            Type rawType = parameterizedType.getRawType();
            if ( rawType instanceof Class<?> ) {
                Class<?> rawTypeClass = ( Class<?> ) rawType;
                if ( base.isAssignableFrom ( rawTypeClass ) ) {
                    List<Type> resolvedTypes = new LinkedList<Type>();
                    for ( Type t : parameterizedType.getActualTypeArguments() ) {
                        if ( t instanceof TypeVariable<?> ) {
                            Type resolvedType = typeVariables.get ( ( ( TypeVariable<?> ) t ).getName() );
                            resolvedTypes.add ( resolvedType != null ? resolvedType : t );
                        } else {
                            resolvedTypes.add ( t );
                        }
                    }
                    Type[] result = resolveActualTypeArgs ( ( Class<? extends T> ) rawTypeClass, base, resolvedTypes.toArray ( new Type[] {} ) );
                    if ( result != null ) {
                        return result;
                    }
                }
            }
        }
    }
    return offspring.equals ( base ) ? actualArgs : null;
}
