public class SettingsActivity extends PreferenceActivity {
    private static final boolean ALWAYS_SIMPLE_PREFS = false;
    protected static final String TAG = null;
    @TargetApi ( Build.VERSION_CODES.HONEYCOMB )
    @Override
    protected void onCreate ( Bundle savedInstanceState ) {
        super.onCreate ( savedInstanceState );
        setupActionBar();
        if ( Build.VERSION.SDK_INT >= Build.VERSION_CODES.HONEYCOMB ) {
            getFragmentManager().beginTransaction()
            .replace ( android.R.id.content, new GeneralPreferenceFragment() )
            .commit();
        } else {
            setupSimplePreferencesScreen();
            PreferenceManager.getDefaultSharedPreferences ( this ).registerOnSharedPreferenceChangeListener ( mPrefsListener );
        }
    }
    SharedPreferences.OnSharedPreferenceChangeListener mPrefsListener = new SharedPreferences.OnSharedPreferenceChangeListener() {
        @Override
        public void onSharedPreferenceChanged ( final SharedPreferences sharedPreferences, final String key ) {
            Log.v ( TAG, "Setting has been changed!" );
        }
    };
    @TargetApi ( Build.VERSION_CODES.HONEYCOMB )
    private void setupActionBar() {
        if ( Build.VERSION.SDK_INT >= Build.VERSION_CODES.HONEYCOMB ) {
            getActionBar().setDisplayHomeAsUpEnabled ( true );
        }
    }
    @Override
    public boolean onOptionsItemSelected ( MenuItem item ) {
        switch ( item.getItemId() ) {
        case android.R.id.home:
            return true;
        }
        return super.onOptionsItemSelected ( item );
    }
    @Override
    protected void onPostCreate ( Bundle savedInstanceState ) {
        super.onPostCreate ( savedInstanceState );
    }
    private void setupSimplePreferencesScreen() {
        if ( !isSimplePreferences ( this ) ) {
            return;
        }
        addPreferencesFromResource ( R.xml.pref_general );
    }
    @Override
    public boolean onIsMultiPane() {
        return isXLargeTablet ( this ) && !isSimplePreferences ( this );
    }
    private static boolean isXLargeTablet ( Context context ) {
        return ( context.getResources().getConfiguration().screenLayout & Configuration.SCREENLAYOUT_SIZE_MASK ) >= Configuration.SCREENLAYOUT_SIZE_XLARGE;
    }
    private static boolean isSimplePreferences ( Context context ) {
        return ALWAYS_SIMPLE_PREFS
               || Build.VERSION.SDK_INT < Build.VERSION_CODES.HONEYCOMB
               || !isXLargeTablet ( context );
    }
    @Override
    @TargetApi ( Build.VERSION_CODES.HONEYCOMB )
    public void onBuildHeaders ( List<Header> target ) {
        if ( !isSimplePreferences ( this ) ) {
        }
    }
    private static Preference.OnPreferenceChangeListener sBindPreferenceSummaryToValueListener = new Preference.OnPreferenceChangeListener() {
        @Override
        public boolean onPreferenceChange ( Preference preference, Object value ) {
            Log.v ( TAG, "Setting has been changed!" );
            String stringValue = value.toString();
            if ( preference instanceof ListPreference ) {
                ListPreference listPreference = ( ListPreference ) preference;
                int index = listPreference.findIndexOfValue ( stringValue );
                preference.setSummary ( index >= 0 ? listPreference.getEntries() [index] : null );
            } else if ( preference instanceof RingtonePreference ) {
                if ( TextUtils.isEmpty ( stringValue ) ) {
                } else {
                    Ringtone ringtone = RingtoneManager.getRingtone (
                                            preference.getContext(), Uri.parse ( stringValue ) );
                    if ( ringtone == null ) {
                        preference.setSummary ( null );
                    } else {
                        String name = ringtone.getTitle ( preference.getContext() );
                        preference.setSummary ( name );
                    }
                }
            } else {
                preference.setSummary ( stringValue );
            }
            return true;
        }
    };
    private static void bindPreferenceSummaryToValue ( Preference preference ) {
        preference.setOnPreferenceChangeListener ( sBindPreferenceSummaryToValueListener );
        sBindPreferenceSummaryToValueListener.onPreferenceChange ( preference, PreferenceManager.getDefaultSharedPreferences ( preference.getContext() ).getString ( preference.getKey(), "" ) );
    }
    @TargetApi ( Build.VERSION_CODES.HONEYCOMB )
    public static class GeneralPreferenceFragment extends PreferenceFragment {
        @Override
        public void onCreate ( Bundle savedInstanceState ) {
            super.onCreate ( savedInstanceState );
            addPreferencesFromResource ( R.xml.pref_general );
            bindPreferenceSummaryToValue ( findPreference ( "example_list" ) );
            bindPreferenceSummaryToValue ( findPreference ( "fuel_type" ) );
            bindPreferenceSummaryToValue ( findPreference ( "unit_type" ) );
        }
        @Override
        public void onResume() {
            super.onResume();
        }
        public void onPause ( Bundle savedInstanceState ) {
            super.onPause();
            addPreferencesFromResource ( R.xml.pref_general );
        }
    }
}
