package test;
import java.io.*;
import java.util.*;
public class Main {
    private static final String FILE_NAME = "D:/Arne/workspaces/IDEDeluxe/TestParser/bin/test/Main.class";
    private static Map<Integer, String> strings = new HashMap<Integer, String>();
    private static Set<Integer> classes = new HashSet<Integer>();
    private static int indexCorrection = 0;
    public static void main ( String[] args ) throws Exception {
        printReferencedClassesFromFile ( FILE_NAME );
    }
    private static void printReferencedClassesFromFile ( String fileName ) throws IOException {
        printReferencedClassesFromFile ( new File ( fileName ) );
    }
    private static void printReferencedClassesFromFile ( File file ) throws IOException {
        DataInputStream stream = new DataInputStream ( new BufferedInputStream ( new FileInputStream ( file ) ) );
        try {
            saveReferencedClassesFromStream ( stream );
        } finally {
            stream.close();
        }
        printReferencedClasses();
    }
    private static void printReferencedClasses() {
        for ( Integer index : classes ) {
            System.out.println ( strings.get ( index ) );
        }
    }
    private static void saveReferencedClassesFromStream ( DataInputStream stream ) throws IOException {
        skipHeader ( stream );
        saveReferencedClassesFromConstantPool ( stream );
    }
    private static void skipHeader ( DataInputStream dataInputStream ) throws IOException {
        readU4 ( dataInputStream );
        readU2 ( dataInputStream );
        readU2 ( dataInputStream );
    }
    private static void saveReferencedClassesFromConstantPool ( DataInputStream stream ) throws IOException {
        int poolSize = readU2 ( stream );
        for ( int n = 1; n < poolSize; n++ ) {
            savePoolEntryIfIsClassReference ( n, stream );
        }
    }
    private static void savePoolEntryIfIsClassReference ( int index, DataInputStream stream ) throws IOException {
        int tag = readU1 ( stream );
        switch ( tag ) {
        case 1:
            saveStringFromUtf8Entry ( index, stream );
            break;
        case 7:
            saveClassEntry ( stream );
            break;
        case 8:
            readU2 ( stream );
            break;
        case 3:
        case 4:
            readU4 ( stream );
            break;
        case 5:
        case 6:
            readU4 ( stream );
            readU4 ( stream );
            indexCorrection++;
            break;
        case 9:
        case 10:
        case 11:
        case 12:
            readU2 ( stream );
            readU2 ( stream );
            break;
        }
    }
    private static void saveClassEntry ( DataInputStream stream ) throws IOException {
        int nameIndex = readU2 ( stream );
        classes.add ( nameIndex - indexCorrection );
    }
    private static void saveStringFromUtf8Entry ( int index, DataInputStream stream ) throws IOException {
        String content = readString ( stream );
        strings.put ( index, content );
    }
    private static String readString ( DataInputStream stream ) throws IOException {
        return stream.readUTF();
    }
    private static int readU1 ( DataInputStream stream ) throws IOException {
        return stream.readUnsignedByte();
    }
    private static int readU2 ( DataInputStream stream ) throws IOException {
        return stream.readUnsignedShort();
    }
    private static int readU4 ( DataInputStream stream ) throws IOException {
        return stream.readInt();
    }
}
