public abstract class OLSTrendLine implements TrendLine {
    RealMatrix coef = null;
    protected abstract double[] xVector ( double x );
    protected abstract boolean logY();
    @Override
    public void setValues ( double[] y, double[] x ) {
        if ( x.length != y.length ) {
            throw new IllegalArgumentException ( String.format ( "The numbers of y and x values must be equal (%d != %d)", y.length, x.length ) );
        }
        double[][] xData = new double[x.length][];
        for ( int i = 0; i < x.length; i++ ) {
            xData[i] = xVector ( x[i] );
        }
        if ( logY() ) {
            y = Arrays.copyOf ( y, y.length );
            for ( int i = 0; i < x.length; i++ ) {
                y[i] = Math.log ( y[i] );
            }
        }
        OLSMultipleLinearRegression ols = new OLSMultipleLinearRegression();
        ols.setNoIntercept ( true );
        ols.newSampleData ( y, xData );
        coef = MatrixUtils.createColumnRealMatrix ( ols.estimateRegressionParameters() );
    }
    @Override
    public double predict ( double x ) {
        double yhat = coef.preMultiply ( xVector ( x ) ) [0];
        if ( logY() ) {
            yhat = ( Math.exp ( yhat ) );
        }
        return yhat;
    }
}
