import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;
public abstract class HookedFuture<T> implements Future<T> {
    private final Future<T> innerFuture;
    private boolean hookCalled = false;
    public HookedFuture ( Future<T> future ) {
        innerFuture = future;
    }
    protected abstract void futureResultIs ( T result );
    @Override
    public boolean cancel ( boolean mayInterruptIfRunning ) {
        return innerFuture.cancel ( mayInterruptIfRunning );
    }
    @Override
    public boolean isCancelled() {
        return innerFuture.isCancelled();
    }
    @Override
    public boolean isDone() {
        return innerFuture.isDone();
    }
    @Override
    public T get() throws InterruptedException, ExecutionException {
        T result = innerFuture.get();
        if ( !hookCalled ) {
            futureResultIs ( result );
            hookCalled = true;
        }
        return result;
    }
    @Override
    public T get ( long timeout, TimeUnit unit )
    throws InterruptedException, ExecutionException, TimeoutException {
        T result = innerFuture.get ( timeout, unit );
        if ( !hookCalled ) {
            futureResultIs ( result );
            hookCalled = true;
        }
        return result;
    }
}
