import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import com.hp.hpl.jena.query.ParameterizedSparqlString;
import com.hp.hpl.jena.query.QueryExecution;
import com.hp.hpl.jena.query.QueryExecutionFactory;
import com.hp.hpl.jena.query.QuerySolution;
import com.hp.hpl.jena.query.QuerySolutionMap;
import com.hp.hpl.jena.query.ResultSet;
import com.hp.hpl.jena.query.ResultSetFormatter;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.sparql.core.Var;
import com.hp.hpl.jena.sparql.engine.binding.Binding;
public class ReuseResults {
    final static String turtleContent = "" +
                                        "@prefix : <http://example.org/> .\n" +
                                        "\n" +
                                        ":alice :hasName \"Alice\" .\n" +
                                        ":alice :hasAddress \"4222 Clinton Way\" .\n" +
                                        ":herman :hasName \"Herman\".\n" +
                                        ":herman :hasAddress \"1313 Mockingbird Lane\" .\n" +
                                        ":DrWho :hasAddress \"The TARDIS\"" +
                                        "";
    final static Model model = ModelFactory.createDefaultModel()
                               .read ( new ByteArrayInputStream ( turtleContent.getBytes() ), null, "TURTLE" );
    final static String findAlice = "" +
                                    "prefix : <http://example.org/>\n" +
                                    "select ?alice where {\n" +
                                    "  ?alice :hasName \"Alice\" .\n" +
                                    "}\n" +
                                    "";
    final static String findAliceAddress = "" +
                                           "prefix : <http://example.org/>\n" +
                                           "select ?address where {\n" +
                                           "  ?alice :hasAddress ?address .\n" +
                                           "}\n" +
                                           "";
    public static void useInitialBindingsFromQuerySolution() throws IOException {
        System.out.println ( "== useInitialBindingsFromQuerySolution ==" );
        final ResultSet aliceResults = QueryExecutionFactory.create ( findAlice, model ).execSelect();
        final QuerySolution solution = aliceResults.next();
        final ResultSet addressResults = QueryExecutionFactory.create ( findAliceAddress, model, solution ).execSelect();
        ResultSetFormatter.out ( addressResults );
    }
    public static void useParameterizedSPARQLString() {
        System.out.println ( "== useParameterizedSPARQLString ==" );
        final ResultSet aliceResults = QueryExecutionFactory.create ( findAlice, model ).execSelect();
        final QuerySolutionMap map = new QuerySolutionMap();
        map.addAll ( aliceResults.next() );
        final ParameterizedSparqlString pss = new ParameterizedSparqlString ( findAliceAddress );
        System.out.println ( pss.toString() );
        pss.setParams ( map );
        System.out.println ( pss.toString() );
        ResultSetFormatter.out ( QueryExecutionFactory.create ( pss.toString(), model ).execSelect() );
    }
    final static String findNamed = "" +
                                    "prefix : <http://example.org/>\n" +
                                    "select ?person where {\n" +
                                    "  ?person :hasName [] .\n" +
                                    "}\n" +
                                    "";
    final static String findPersonAddress = "" +
                                            "prefix : <http://example.org/>\n" +
                                            "select ?address where { " +
                                            "  ?person :hasAddress ?address .\n" +
                                            "}\n" +
                                            "";
    public static void useValuesFromResultSet() {
        System.out.println ( "\n== useValuesFromResultSet ==" );
        final ResultSet namedResults = QueryExecutionFactory.create ( findNamed, model ).execSelect();
        final QueryExecution qe = QueryExecutionFactory.create ( findPersonAddress, model );
        System.out.println ( "=== Query Before Adding VALUES ===\n" + qe.getQuery() );
        List<Var> variables = new ArrayList<>();
        for ( final String varName : namedResults.getResultVars() ) {
            variables.add ( Var.alloc ( varName ) );
        }
        List<Binding> values = new ArrayList<>();
        while ( namedResults.hasNext() ) {
            values.add ( namedResults.nextBinding() );
        }
        qe.getQuery().setValuesDataBlock ( variables, values );
        System.out.println ( "\n=== Query After Adding VALUES ===\n" + qe.getQuery() );
        ResultSetFormatter.out ( qe.execSelect() );
    }
    public static void main ( String[] args ) throws IOException {
        useInitialBindingsFromQuerySolution();
        useParameterizedSPARQLString();
        useValuesFromResultSet();
    }
}
