public static class XTableTransferHandler extends TransferHandler {
    @Override
    protected Transferable createTransferable ( JComponent c ) {
        if ( ! ( c instanceof JXTable ) ) {
            return null;
        }
        JXTable table = ( JXTable ) c;
        int[] rows;
        int[] cols;
        if ( !table.getRowSelectionAllowed()
                && !table.getColumnSelectionAllowed() ) {
            return null;
        }
        if ( !table.getRowSelectionAllowed() ) {
            int rowCount = table.getRowCount();
            rows = new int[rowCount];
            for ( int counter = 0; counter < rowCount; counter++ ) {
                rows[counter] = counter;
            }
        } else {
            rows = table.getSelectedRows();
        }
        if ( !table.getColumnSelectionAllowed() ) {
            int colCount = table.getColumnCount();
            cols = new int[colCount];
            for ( int counter = 0; counter < colCount; counter++ ) {
                cols[counter] = counter;
            }
        } else {
            cols = table.getSelectedColumns();
        }
        if ( rows == null || cols == null || rows.length == 0
                || cols.length == 0 ) {
            return null;
        }
        StringBuffer plainBuf = new StringBuffer();
        StringBuffer htmlBuf = new StringBuffer();
        htmlBuf.append ( "<html>\n<body>\n<table>\n" );
        for ( int row = 0; row < rows.length; row++ ) {
            htmlBuf.append ( "<tr>\n" );
            for ( int col = 0; col < cols.length; col++ ) {
                String val = table.getStringAt ( row, col );
                plainBuf.append ( val + "\t" );
                htmlBuf.append ( "  <td>" + val + "</td>\n" );
            }
            plainBuf.deleteCharAt ( plainBuf.length() - 1 ).append ( "\n" );
            htmlBuf.append ( "</tr>\n" );
        }
        plainBuf.deleteCharAt ( plainBuf.length() - 1 );
        htmlBuf.append ( "</table>\n</body>\n</html>" );
        return new BasicTransferable ( plainBuf.toString(),
                                       htmlBuf.toString() );
    }
    @Override
    public int getSourceActions ( JComponent c ) {
        return COPY;
    }
}
