public class MyActivity extends Activity implements ScrollAwareGridView.ImmediateScrollListener, AdapterView.OnItemClickListener {
    private static final String TAG = "MyActivity";
    private ImageView mSelectedImage = null;
    private int mSelectedPosition;
    private ScrollAwareGridView mGrid;
    private ImageAdapter mAdapter;
    @Override
    public void onCreate ( Bundle savedInstanceState ) {
        super.onCreate ( savedInstanceState );
        setContentView ( R.layout.main );
        mSelectedImage = ( ImageView ) findViewById ( R.id.selectedImage );
        mGrid = ( ScrollAwareGridView ) findViewById ( R.id.grid );
        if ( null != mGrid ) {
            mAdapter = new ImageAdapter ( this );
            mGrid.setAdapter ( mAdapter );
            mGrid.setImmediateScrollListener ( this );
            mGrid.setOnItemClickListener ( this );
        }
    }
    @Override
    protected void onPause() {
        super.onPause();
    }
    @Override
    protected void onResume() {
        super.onResume();
        mSelectedImage.setImageBitmap ( null );
        mSelectedImage.setVisibility ( View.GONE );
        mSelectedPosition = -1;
    }
    @Override
    public void onImmediateScrollChanged() {
        if ( mSelectedPosition >= 0 ) {
            int firstPosition = mGrid.getFirstVisiblePosition();
            int wantedChild = mSelectedPosition - firstPosition;
            if ( wantedChild < 0 || wantedChild >= mGrid.getChildCount() ) {
                Log.w ( TAG, "Unable to get view for desired position, because it's not being displayed on screen." );
                mSelectedImage.setVisibility ( View.INVISIBLE );
                return;
            } else {
                mSelectedImage.setVisibility ( View.VISIBLE );
            }
            final View selectedView = mGrid.getChildAt ( wantedChild );
            if ( null != selectedView && mSelectedImage.getVisibility() == View.VISIBLE ) {
                final ViewGroup.MarginLayoutParams zoomedImageLayoutParams = ( ViewGroup.MarginLayoutParams ) mSelectedImage.getLayoutParams();
                final Integer thumbnailX = mGrid.getLeft() + selectedView.getLeft() - ( ImageAdapter.HIGHLIGHTED_GRID_ITEM_DIMENSION - ImageAdapter.GRID_ITEM_DIMENSION ) / 2;
                final Integer thumbnailY = mGrid.getTop() + selectedView.getTop() - ( ImageAdapter.HIGHLIGHTED_GRID_ITEM_DIMENSION - ImageAdapter.GRID_ITEM_DIMENSION ) / 2;
                zoomedImageLayoutParams.setMargins ( thumbnailX,
                                                     thumbnailY,
                                                     0,
                                                     0 );
                mSelectedImage.setLayoutParams ( zoomedImageLayoutParams );
            }
        }
    }
    @Override
    public void onItemClick ( final AdapterView<?> parent, final View view, final int position, final long id ) {
        mSelectedPosition = position;
        final Bitmap bm = mAdapter.getImage ( position );
        if ( bm != null ) {
            final ViewGroup.MarginLayoutParams zoomedImageLayoutParams = ( ViewGroup.MarginLayoutParams ) mSelectedImage.getLayoutParams();
            final Integer thumbnailX = mGrid.getLeft() + view.getLeft() - ( ImageAdapter.HIGHLIGHTED_GRID_ITEM_DIMENSION - ImageAdapter.GRID_ITEM_DIMENSION ) / 2;
            final Integer thumbnailY = mGrid.getTop() + view.getTop() - ( ImageAdapter.HIGHLIGHTED_GRID_ITEM_DIMENSION - ImageAdapter.GRID_ITEM_DIMENSION ) / 2;
            zoomedImageLayoutParams.setMargins ( thumbnailX,
                                                 thumbnailY,
                                                 0,
                                                 0 );
            zoomedImageLayoutParams.height = ImageAdapter.HIGHLIGHTED_GRID_ITEM_DIMENSION;
            zoomedImageLayoutParams.width = ImageAdapter.HIGHLIGHTED_GRID_ITEM_DIMENSION;
            mSelectedImage.setImageBitmap ( bm );
            mSelectedImage.setScaleType ( ImageView.ScaleType.CENTER );
            mSelectedImage.setLayoutParams ( zoomedImageLayoutParams );
            mSelectedImage.setVisibility ( View.VISIBLE );
        }
    }
}
