public class WholeFileRecordReader extends RecordReader<NullWritable, Text> {
    private static final Logger LOG = Logger.getLogger ( WholeFileRecordReader.class );
    private final Path mFileToRead;
    private final long mFileLength;
    private final Configuration mConf;
    private boolean mProcessed;
    private final Text mFileText;
    public WholeFileRecordReader ( CombineFileSplit fileSplit, TaskAttemptContext context,
                                   Integer pathToProcess ) {
        mProcessed = false;
        mFileToRead = fileSplit.getPath ( pathToProcess );
        mFileLength = fileSplit.getLength ( pathToProcess );
        mConf = context.getConfiguration();
        assert 0 == fileSplit.getOffset ( pathToProcess );
        if ( LOG.isDebugEnabled() ) {
            LOG.debug ( "FileToRead is: " + mFileToRead.toString() );
            LOG.debug ( "Processing path " + pathToProcess + " out of " + fileSplit.getNumPaths() );
            try {
                FileSystem fs = FileSystem.get ( mConf );
                assert fs.getFileStatus ( mFileToRead ).getLen() == mFileLength;
            } catch ( IOException ioe ) {
            }
        }
        mFileText = new Text();
    }
    @Override
    public void close() throws IOException {
        mFileText.clear();
    }
    @Override
    public NullWritable getCurrentKey() throws IOException, InterruptedException {
        return NullWritable.get();
    }
    @Override
    public Text getCurrentValue() throws IOException, InterruptedException {
        return mFileText;
    }
    @Override
    public float getProgress() throws IOException, InterruptedException {
        return ( mProcessed ) ? ( float ) 1.0 : ( float ) 0.0;
    }
    @Override
    public void initialize ( InputSplit split, TaskAttemptContext context )
    throws IOException, InterruptedException {
    }
    @Override
    public boolean nextKeyValue() throws IOException, InterruptedException {
        if ( !mProcessed ) {
            if ( mFileLength > ( long ) Integer.MAX_VALUE ) {
                throw new IOException ( "File is longer than Integer.MAX_VALUE." );
            }
            byte[] contents = new byte[ ( int ) mFileLength];
            FileSystem fs = mFileToRead.getFileSystem ( mConf );
            FSDataInputStream in = null;
            try {
                in = fs.open ( mFileToRead );
                IOUtils.readFully ( in, contents, 0, contents.length );
                mFileText.set ( contents, 0, contents.length );
            } finally {
                IOUtils.closeStream ( in );
            }
            mProcessed = true;
            return true;
        }
        return false;
    }
}
