import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.EventQueue;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.geom.AffineTransform;
import java.awt.geom.Dimension2D;
import java.awt.image.BufferedImage;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;
import java.io.File;
import java.io.IOException;
import java.text.NumberFormat;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.imageio.ImageIO;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
public class PrintTest {
    public static void main ( String[] args ) {
        new PrintTest();
    }
    public PrintTest() {
        EventQueue.invokeLater ( new Runnable() {
            @Override
            public void run() {
                try {
                    UIManager.setLookAndFeel ( UIManager.getSystemLookAndFeelClassName() );
                } catch ( ClassNotFoundException | InstantiationException | IllegalAccessException | UnsupportedLookAndFeelException ex ) {
                }
                final TestPane imagePane = new TestPane();
                JButton print = new JButton ( "Print" );
                print.addActionListener ( new ActionListener() {
                    @Override
                    public void actionPerformed ( ActionEvent e ) {
                        printComponent ( imagePane );
                    }
                } );
                JFrame frame = new JFrame ( "Testing" );
                frame.setDefaultCloseOperation ( JFrame.EXIT_ON_CLOSE );
                frame.setLayout ( new BorderLayout() );
                frame.add ( imagePane );
                frame.add ( print, BorderLayout.SOUTH );
                frame.setSize ( 200, 200 );
                frame.setLocationRelativeTo ( null );
                frame.setVisible ( true );
            }
        } );
    }
    public class TestPane extends JPanel {
        private BufferedImage bg;
        public TestPane() {
            try {
                bg = ImageIO.read ( new File ( "path/to/a/image" ) );
            } catch ( IOException ex ) {
                Logger.getLogger ( PrintTest.class.getName() ).log ( Level.SEVERE, null, ex );
            }
        }
        @Override
        public Dimension getPreferredSize() {
            return bg == null ? new Dimension ( 200, 200 ) : new Dimension ( bg.getWidth(), bg.getHeight() );
        }
        @Override
        protected void paintComponent ( Graphics g ) {
            super.paintComponent ( g );
            Graphics2D g2d = ( Graphics2D ) g.create();
            if ( bg != null ) {
                int x = ( getWidth() - bg.getWidth() ) / 2;
                int y = ( getHeight() - bg.getHeight() ) / 2;
                g2d.drawImage ( bg, x, y, this );
            }
            g2d.dispose();
        }
    }
    public void printComponent ( Component comp ) {
        PrinterJob pj = PrinterJob.getPrinterJob();
        pj.setJobName ( " Print Component " );
        pj.setPrintable ( new ComponentPrintable ( comp ) );
        if ( !pj.printDialog() ) {
            return;
        }
        try {
            pj.print();
        } catch ( PrinterException ex ) {
            System.out.println ( ex );
        }
    }
    public class ComponentPrintable implements Printable {
        private Component comp;
        private ComponentPrintable ( Component comp ) {
            this.comp = comp;
        }
        @Override
        public int print ( Graphics g, PageFormat pf, int pageNumber )
        throws PrinterException {
            if ( pageNumber > 0 ) {
                return Printable.NO_SUCH_PAGE;
            }
            Dimension compSize = comp.getPreferredSize();
            comp.setSize ( compSize );
            Dimension printSize = new Dimension();
            printSize.setSize ( pf.getImageableWidth(), pf.getImageableHeight() );
            double scaleFactor = getScaleFactorToFit ( compSize, printSize );
            if ( scaleFactor > 1d ) {
                scaleFactor = 1d;
            }
            double scaleWidth = compSize.width * scaleFactor;
            double scaleHeight = compSize.height * scaleFactor;
            Graphics2D g2 = ( Graphics2D ) g.create();
            double x = ( ( pf.getImageableWidth() - scaleWidth ) / 2d ) + pf.getImageableX();
            double y = ( ( pf.getImageableHeight() - scaleHeight ) / 2d ) + pf.getImageableY();
            AffineTransform at = new AffineTransform();
            at.translate ( x, y );
            at.scale ( scaleFactor, scaleFactor );
            g2.transform ( at );
            comp.printAll ( g2 );
            g2.dispose();
            comp.revalidate();
            return Printable.PAGE_EXISTS;
        }
    }
    public static double getScaleFactorToFit ( Dimension original, Dimension toFit ) {
        double dScale = 1d;
        if ( original != null && toFit != null ) {
            double dScaleWidth = getScaleFactor ( original.width, toFit.width );
            double dScaleHeight = getScaleFactor ( original.height, toFit.height );
            dScale = Math.min ( dScaleHeight, dScaleWidth );
        }
        return dScale;
    }
    public static double getScaleFactor ( int iMasterSize, int iTargetSize ) {
        double dScale = 1;
        if ( iMasterSize > iTargetSize ) {
            dScale = ( double ) iTargetSize / ( double ) iMasterSize;
        } else {
            dScale = ( double ) iTargetSize / ( double ) iMasterSize;
        }
        return dScale;
    }
}
