public class BluetoothHDPService extends Service {
    private static final String TAG = "BluetoothHDPService";
    public static final int RESULT_OK = 0;
    public static final int RESULT_FAIL = -1;
    public static final int STATUS_HEALTH_APP_REG = 100;
    public static final int STATUS_HEALTH_APP_UNREG = 101;
    public static final int STATUS_CREATE_CHANNEL = 102;
    public static final int STATUS_DESTROY_CHANNEL = 103;
    public static final int STATUS_READ_DATA = 104;
    public static final int STATUS_READ_DATA_DONE = 105;
    public static final int MSG_REG_CLIENT = 200;
    public static final int MSG_UNREG_CLIENT = 201;
    public static final int MSG_REG_HEALTH_APP = 300;
    public static final int MSG_UNREG_HEALTH_APP = 301;
    public static final int MSG_CONNECT_CHANNEL = 400;
    public static final int MSG_DISCONNECT_CHANNEL = 401;
    public static final int RECEIVED_SYS = 901;
    public static final int RECEIVED_DIA = 902;
    public static final int RECEIVED_PUL = 903;
    private int count;
    private byte[] invoke = {0x00, 0x00};
    private BluetoothHealthAppConfiguration mHealthAppConfig;
    private BluetoothAdapter mBluetoothAdapter;
    private BluetoothHealth mBluetoothHealth;
    private BluetoothDevice mDevice;
    private int mChannelId;
    private Messenger mClient;
    @SuppressLint ( "HandlerLeak" )
    private class IncomingHandler extends Handler {
        @Override
        public void handleMessage ( Message msg ) {
            switch ( msg.what ) {
            case MSG_REG_CLIENT:
                Log.d ( TAG, "Activity client registered" );
                mClient = msg.replyTo;
                break;
            case MSG_UNREG_CLIENT:
                mClient = null;
                break;
            case MSG_REG_HEALTH_APP:
                registerApp ( msg.arg1 );
                break;
            case MSG_UNREG_HEALTH_APP:
                unregisterApp();
                break;
            case MSG_CONNECT_CHANNEL:
                mDevice = ( BluetoothDevice ) msg.obj;
                connectChannel();
                break;
            case MSG_DISCONNECT_CHANNEL:
                mDevice = ( BluetoothDevice ) msg.obj;
                disconnectChannel();
                break;
            default:
                super.handleMessage ( msg );
            }
        }
    }
    final Messenger mMessenger = new Messenger ( new IncomingHandler() );
    @SuppressLint ( "ShowToast" )
    @Override
    public void onCreate() {
        super.onCreate();
        Log.e ( "TEST", "HDPService Created" );
        mBluetoothAdapter = BluetoothAdapter.getDefaultAdapter();
        if ( mBluetoothAdapter == null || !mBluetoothAdapter.isEnabled() ) {
            stopSelf();
            return;
        }
        if ( !mBluetoothAdapter.getProfileProxy ( this, mBluetoothServiceListener,
                BluetoothProfile.HEALTH ) ) {
            Toast.makeText ( this, "bluetooth_health_profile_not_available",
                             Toast.LENGTH_LONG );
            stopSelf();
            return;
        }
    }
    @Override
    public int onStartCommand ( Intent intent, int flags, int startId ) {
        Log.d ( TAG, "BluetoothHDPService is running." );
        return START_STICKY;
    }
    @Override
    public IBinder onBind ( Intent intent ) {
        return mMessenger.getBinder();
    };
    private void registerApp ( int dataType ) {
        Log.e ( TAG, "registerApp()" );
        mBluetoothHealth.registerSinkAppConfiguration ( TAG, dataType, mHealthCallback );
    }
    private void unregisterApp() {
        Log.e ( TAG, "unregisterApp()" );
        mBluetoothHealth.unregisterAppConfiguration ( mHealthAppConfig );
    }
    private void connectChannel() {
        Log.i ( TAG, "connectChannel()" );
        mBluetoothHealth.connectChannelToSource ( mDevice, mHealthAppConfig );
    }
    private void disconnectChannel() {
        Log.i ( TAG, "disconnectChannel()" );
        mBluetoothHealth.disconnectChannel ( mDevice, mHealthAppConfig, mChannelId );
    }
    private final BluetoothProfile.ServiceListener mBluetoothServiceListener =
    new BluetoothProfile.ServiceListener() {
        public void onServiceConnected ( int profile, BluetoothProfile proxy ) {
            if ( profile == BluetoothProfile.HEALTH ) {
                mBluetoothHealth = ( BluetoothHealth ) proxy;
                if ( Log.isLoggable ( TAG, Log.DEBUG ) ) {
                    Log.d ( TAG, "onServiceConnected to profile: " + profile );
                }
            }
        }
        public void onServiceDisconnected ( int profile ) {
            if ( profile == BluetoothProfile.HEALTH ) {
                mBluetoothHealth = null;
            }
        }
    };
    private final BluetoothHealthCallback mHealthCallback = new BluetoothHealthCallback() {
        public void onHealthAppConfigurationStatusChange ( BluetoothHealthAppConfiguration config,
                int status ) {
            if ( status == BluetoothHealth.APP_CONFIG_REGISTRATION_FAILURE ) {
                mHealthAppConfig = null;
                sendMessage ( STATUS_HEALTH_APP_REG, RESULT_FAIL );
            } else if ( status == BluetoothHealth.APP_CONFIG_REGISTRATION_SUCCESS ) {
                mHealthAppConfig = config;
                sendMessage ( STATUS_HEALTH_APP_REG, RESULT_OK );
            } else if ( status == BluetoothHealth.APP_CONFIG_UNREGISTRATION_FAILURE ||
                        status == BluetoothHealth.APP_CONFIG_UNREGISTRATION_SUCCESS ) {
                sendMessage ( STATUS_HEALTH_APP_UNREG,
                              status == BluetoothHealth.APP_CONFIG_UNREGISTRATION_SUCCESS ?
                              RESULT_OK : RESULT_FAIL );
            }
        }
        public void onHealthChannelStateChange ( BluetoothHealthAppConfiguration config,
                BluetoothDevice device, int prevState, int newState, ParcelFileDescriptor fd,
                int channelId ) {
            Log.e ( "Read Thread", "Start If Statements" );
            Log.e ( "Read Thread", "prevState: " + prevState );
            Log.e ( "Read Thread", "newState: " + newState );
            if ( Log.isLoggable ( TAG, Log.DEBUG ) )
                Log.d ( TAG, String.format ( "prevState\t%d ----------> newState\t%d",
                                             prevState, newState ) );
            if ( prevState == BluetoothHealth.STATE_CHANNEL_DISCONNECTED &&
                    newState == BluetoothHealth.STATE_CHANNEL_CONNECTED ) {
                if ( config.equals ( mHealthAppConfig ) ) {
                    mChannelId = channelId;
                    sendMessage ( STATUS_CREATE_CHANNEL, RESULT_OK );
                    Log.e ( "Read Thread", "Read  Start 1" );
                    ( new ReadThread ( fd ) ).start();
                } else {
                    Log.e ( "Read Thread", "Status Create Channel Fail 1" );
                    sendMessage ( STATUS_CREATE_CHANNEL, RESULT_FAIL );
                }
            } else if ( prevState == BluetoothHealth.STATE_CHANNEL_CONNECTING &&
                        newState == BluetoothHealth.STATE_CHANNEL_DISCONNECTED ) {
                sendMessage ( STATUS_CREATE_CHANNEL, RESULT_FAIL );
                Log.e ( "Read Thread", "Status Create Channel Fail 2" );
            } else if ( newState == BluetoothHealth.STATE_CHANNEL_DISCONNECTED ) {
                if ( config.equals ( mHealthAppConfig ) ) {
                    sendMessage ( STATUS_DESTROY_CHANNEL, RESULT_OK );
                    Log.e ( "Read Thread", "Status Disconnect OK" );
                } else {
                    sendMessage ( STATUS_DESTROY_CHANNEL, RESULT_FAIL );
                    Log.e ( "Read Thread", "Status Disconnect FAIL" );
                }
            } else if ( prevState == BluetoothHealth.STATE_CHANNEL_CONNECTING && newState == BluetoothHealth.STATE_CHANNEL_CONNECTED ) {
                if ( config.equals ( mHealthAppConfig ) ) {
                    mChannelId = channelId;
                    sendMessage ( STATUS_CREATE_CHANNEL, RESULT_OK );
                    Log.e ( "Read Thread", "Read  Start 2" );
                    ( new ReadThread ( fd ) ).start();
                } else {
                    sendMessage ( STATUS_CREATE_CHANNEL, RESULT_FAIL );
                    Log.e ( "Read Thread", "Status Create Channel Fail 3" );
                }
            }
        }
    };
    private void sendMessage ( int what, int value ) {
        if ( mClient == null ) {
            Log.d ( TAG, "No clients registered." );
            return;
        }
        try {
            mClient.send ( Message.obtain ( null, what, value, 0 ) );
        } catch ( RemoteException e ) {
            e.printStackTrace();
        }
    }
    public String byte2hex ( byte[] b ) {
        String hs = "";
        String stmp = "";
        for ( int n = 0; n < b.length; n++ ) {
            stmp = ( java.lang.Integer.toHexString ( b[n] & 0XFF ) );
            if ( stmp.length() == 1 ) {
                hs = hs + "0" + stmp;
            } else {
                hs = hs + stmp;
            }
            if ( n < b.length - 1 ) {
                hs = hs + "";
            }
        }
        return hs;
    }
    public static int byteToUnsignedInt ( byte b ) {
        return 0x00 << 24 | b & 0xff;
    }
    public static int toInt ( byte[] bytes ) {
        int ret = 0;
        for ( int i = 0; i < 4 && i < bytes.length; i++ ) {
            ret <<= 8;
            ret |= ( int ) bytes[i] & 0xFF;
        }
        return ret;
    }
    private class ReadThread extends Thread {
        private ParcelFileDescriptor mFd;
        public ReadThread ( ParcelFileDescriptor fd ) {
            super();
            mFd = fd;
        }
        @Override
        public void run() {
            Log.e ( "TEST", "Read Data 1" );
            FileInputStream fis = new FileInputStream ( mFd.getFileDescriptor() );
            final byte data[] = new byte[200];
            Log.i ( TAG, "Read Data 2" );
            try {
                while ( fis.read ( data ) > -1 ) {
                    Log.i ( TAG, "INBOUND" );
                    String test = byte2hex ( data );
                    Log.i ( TAG, test );
                    if ( data[0] != ( byte ) 0x00 ) {
                        if ( data[0] == ( byte ) 0xE2 ) {
                            Log.i ( TAG, "E2 - Association Request" );
                            count = 1;
                            ( new WriteThread ( mFd ) ).start();
                            try {
                                sleep ( 100 );
                            } catch ( InterruptedException e ) {
                                e.printStackTrace();
                            }
                            count = 2;
                            ( new WriteThread ( mFd ) ).start();
                        } else if ( data[0] == ( byte ) 0xE7 ) {
                            Log.i ( TAG, "E7 - Data Given" );
                            if ( data[3] != ( byte ) 0xda ) {
                                invoke[0] = data[6];
                                invoke[1] = data[7];
                                Log.i ( TAG, "E7 - Reading?" );
                                ByteBuffer sys = ByteBuffer.allocate ( 2 );
                                sys.order ( ByteOrder.LITTLE_ENDIAN );
                                sys.put ( data[45] );
                                sys.put ( data[46] );
                                short sysVal = sys.getShort ( 0 );
                                Log.i ( TAG, " Sys - " + sysVal );
                                ByteBuffer dia = ByteBuffer.allocate ( 2 );
                                dia.order ( ByteOrder.LITTLE_ENDIAN );
                                dia.put ( data[47] );
                                dia.put ( data[48] );
                                short diaVal = dia.getShort ( 0 );
                                Log.i ( TAG, " Dia - " + diaVal );
                                sendMessage ( 9919, diaVal );
                                sendMessage ( 9920, sysVal );
                                for ( int i = 0; i < ( data.length - 2 ); i++ ) {
                                    ByteBuffer bb = ByteBuffer.allocate ( 2 );
                                    bb.order ( ByteOrder.LITTLE_ENDIAN );
                                    bb.put ( data[i] );
                                    bb.put ( data[i + 1] );
                                    short shortVal = bb.getShort ( 0 );
                                    Log.i ( TAG, i + " Short Val - " + shortVal );
                                }
                                count = 3;
                                ( new WriteThread ( mFd ) ).start();
                            }
                        } else if ( data[0] == ( byte ) 0xE4 ) {
                        }
                        for ( int i = 0; i < data.length; i++ ) {
                            data[i] = ( byte ) 0x00;
                        }
                    }
                    sendMessage ( STATUS_READ_DATA, 0 );
                }
            } catch ( IOException ioe ) {}
            if ( mFd != null ) {
                try {
                    mFd.close();
                } catch ( IOException e ) {
                }
            }
            sendMessage ( STATUS_READ_DATA_DONE, 0 );
        }
    }
    private class WriteThread extends Thread {
        private ParcelFileDescriptor mFd;
        public WriteThread ( ParcelFileDescriptor fd ) {
            super();
            mFd = fd;
        }
        @Override
        public void run() {
            FileOutputStream fos = new FileOutputStream ( mFd.getFileDescriptor() );
            final byte data_AR[] = new byte[] {         ( byte ) 0xE3, ( byte ) 0x00,
                    ( byte ) 0x00, ( byte ) 0x2C,
                    ( byte ) 0x00, ( byte ) 0x00,
                    ( byte ) 0x50, ( byte ) 0x79,
                    ( byte ) 0x00, ( byte ) 0x26,
                    ( byte ) 0x80, ( byte ) 0x00, ( byte ) 0x00, ( byte ) 0x00,
                    ( byte ) 0x80, ( byte ) 0x00,
                    ( byte ) 0x80, ( byte ) 0x00, ( byte ) 0x00, ( byte ) 0x00,
                    ( byte ) 0x00, ( byte ) 0x00, ( byte ) 0x00, ( byte ) 0x00,
                    ( byte ) 0x80, ( byte ) 0x00, ( byte ) 0x00, ( byte ) 0x00,
                    ( byte ) 0x00, ( byte ) 0x08,
                    ( byte ) 0x3C, ( byte ) 0x5A, ( byte ) 0x37, ( byte ) 0xFF,
                    ( byte ) 0xFE, ( byte ) 0x95, ( byte ) 0xEE, ( byte ) 0xE3,
                    ( byte ) 0x00, ( byte ) 0x00,
                    ( byte ) 0x00, ( byte ) 0x00,
                    ( byte ) 0x00, ( byte ) 0x00,
                    ( byte ) 0x00, ( byte ) 0x00, ( byte ) 0x00, ( byte ) 0x00
                                              };
            final byte data_DR[] = new byte[] {         ( byte ) 0xE7, ( byte ) 0x00,
                    ( byte ) 0x00, ( byte ) 0x12,
                    ( byte ) 0x00, ( byte ) 0x10,
                    ( byte ) 0x00, ( byte ) 0x24,
                    ( byte ) 0x02, ( byte ) 0x01,
                    ( byte ) 0x00, ( byte ) 0x0A,
                    ( byte ) 0x00, ( byte ) 0x00,
                    ( byte ) 0x00, ( byte ) 0x00, ( byte ) 0x00, ( byte ) 0x00,
                    ( byte ) 0x0D, ( byte ) 0x1D,
                    ( byte ) 0x00, ( byte ) 0x00
                                              };
            final byte get_MDS[] = new byte[] {         ( byte ) 0xE7, ( byte ) 0x00,
                    ( byte ) 0x00, ( byte ) 0x0E,
                    ( byte ) 0x00, ( byte ) 0x0C,
                    ( byte ) 0x00, ( byte ) 0x24,
                    ( byte ) 0x01, ( byte ) 0x03,
                    ( byte ) 0x00, ( byte ) 0x06,
                    ( byte ) 0x00, ( byte ) 0x00,
                    ( byte ) 0x00, ( byte ) 0x00,
                    ( byte ) 0x00, ( byte ) 0x00
                                              };
            final byte data_RR[] = new byte[] {         ( byte ) 0xE5, ( byte ) 0x00,
                    ( byte ) 0x00, ( byte ) 0x02,
                    ( byte ) 0x00, ( byte ) 0x00
                                              };
            try {
                Log.i ( TAG, String.valueOf ( count ) );
                if ( count == 1 ) {
                    fos.write ( data_AR );
                    Log.i ( TAG, "Association Responded!" );
                } else if ( count == 2 ) {
                    fos.write ( get_MDS );
                    Log.i ( TAG, "Get MDS object attributes!" );
                } else if ( count == 3 ) {
                    fos.write ( data_DR );
                    Log.i ( TAG, "Data Responsed!" );
                } else if ( count == 4 ) {
                    fos.write ( data_RR );
                    Log.i ( TAG, "Data Released!" );
                }
            } catch ( IOException ioe ) {}
        }
    }
}
