package net.mightypork.rpack.utils;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
public class SimpleConfig {
    public static List<String> listFromFile ( File file ) throws IOException {
        String fileText = FileUtils.fileToString ( file );
        return listFromString ( fileText );
    }
    public static Map<String, String> mapFromFile ( File file ) throws IOException {
        String fileText = FileUtils.fileToString ( file );
        return mapFromString ( fileText );
    }
    public static List<String> listFromString ( String text ) {
        List<String> list = new ArrayList<String>();
        String[] groupsLines = text.split ( "\n" );
        for ( String s : groupsLines ) {
            if ( s.length() == 0 ) {
                continue;
            }
            if ( s.startsWith ( "#" ) || s.startsWith ( "//" ) ) {
                continue;
            }
            if ( s.equalsIgnoreCase ( "NULL" ) ) {
                s = null;
            }
            if ( s != null ) {
                s = s.replace ( "\\n", "\n" );
            }
            list.add ( s );
        }
        return list;
    }
    public static Map<String, String> mapFromString ( String text ) {
        LinkedHashMap<String, String> pairs = new LinkedHashMap<String, String>();
        String[] groupsLines = text.split ( "\n" );
        for ( String s : groupsLines ) {
            if ( s.length() == 0 ) {
                continue;
            }
            if ( s.startsWith ( "#" ) || s.startsWith ( "//" ) ) {
                continue;
            }
            if ( !s.contains ( "=" ) ) {
                continue;
            }
            String[] parts = s.split ( "=" );
            for ( int i = 0; i < parts.length; i++ ) {
                parts[i] = parts[i].trim();
            }
            if ( parts.length == 0 ) {
                Log.w ( "Bad line in config file: " + s );
                continue;
            }
            if ( parts.length == 1 ) {
                parts = new String[] { parts[0], "" };
            }
            if ( parts.length != 2 ) {
                Log.w ( "Bad line in config file: " + s );
                continue;
            }
            if ( parts[0].equalsIgnoreCase ( "NULL" ) ) {
                parts[0] = null;
            }
            if ( parts[1].equalsIgnoreCase ( "NULL" ) ) {
                parts[1] = null;
            }
            if ( parts[0] != null ) {
                parts[0] = parts[0].replace ( "\\n", "\n" );
            }
            if ( parts[1] != null ) {
                parts[1] = parts[1].replace ( "\\n", "\n" );
            }
            pairs.put ( parts[0], parts[1] );
        }
        return pairs;
    }
    public static void mapToFile ( File target, Map<String, String> data ) throws IOException {
        String text = "";
        for ( Entry<String, String> e : data.entrySet() ) {
            if ( text.length() > 0 ) {
                text += "\n";
            }
            String key = e.getKey();
            String value = e.getValue();
            if ( key == null ) {
                key = "NULL";
            }
            if ( value == null ) {
                value = "NULL";
            }
            key = key.replace ( "\n", "\\n" );
            value = value.replace ( "\n", "\\n" );
            text += key + " = " + value;
        }
        FileUtils.stringToFile ( target, text );
    }
    public static void listToFile ( File target, List<String> data ) throws IOException {
        String text = "";
        for ( String s : data ) {
            if ( text.length() > 0 ) {
                text += "\n";
            }
            if ( s == null ) {
                s = "NULL";
            }
            s = s.replace ( "\n", "\\n" );
            text += s;
        }
        FileUtils.stringToFile ( target, text );
    }
}
