private static KeyManager[] getKeyManagers ( Properties props ) throws IOException, GeneralSecurityException {
    String alg = KeyManagerFactory.getDefaultAlgorithm();
    KeyManagerFactory kmFact = KeyManagerFactory.getInstance ( alg );
    FileInputStream fis = new FileInputStream ( props.getProperty ( SSL_KEYSTORE ) );
    logger.info ( "Loaded keystore" );
    KeyStore ks = KeyStore.getInstance ( "jks" );
    String keyStorePassword = props.getProperty ( SSL_KEYSTORE_PASSWORD );
    ks.load ( fis, keyStorePassword.toCharArray() );
    fis.close();
    kmFact.init ( ks, keyStorePassword.toCharArray() );
    KeyManagerFactory dkmFact = KeyManagerFactory.getInstance ( alg );
    dkmFact.init ( null, null );
    X509KeyManager customX509KeyManager = getX509KeyManager ( alg, kmFact );
    X509KeyManager jvmX509KeyManager = getX509KeyManager ( alg, dkmFact );
    KeyManager[] km = { new MultiKeyStoreManager ( jvmX509KeyManager, customX509KeyManager ) };
    logger.debug ( "Number of key managers registered:" + km.length );
    return km;
}
private static X509KeyManager getX509KeyManager ( String algorithm, KeyManagerFactory kmFact )
throws NoSuchAlgorithmException {
    KeyManager[] keyManagers = kmFact.getKeyManagers();
    if ( keyManagers == null || keyManagers.length == 0 ) {
        throw new NoSuchAlgorithmException ( "The default algorithm :" + algorithm + " produced no key managers" );
    }
    X509KeyManager x509KeyManager = null;
    for ( int i = 0; i < keyManagers.length; i++ ) {
        if ( keyManagers[i] instanceof X509KeyManager ) {
            x509KeyManager = ( X509KeyManager ) keyManagers[i];
            break;
        }
    }
    if ( x509KeyManager == null ) {
        throw new NoSuchAlgorithmException ( "The default algorithm :" + algorithm + " did not produce a X509 Key manager" );
    }
    return x509KeyManager;
}
private static void initialiseManager ( Properties props ) throws IOException, GeneralSecurityException {
    SSLContext context = SSLContext.getInstance ( "SSL" );
    context.init ( getKeyManagers ( props ), getTrustManagers ( props ), null );
    SSLContext.setDefault ( context );
}
