class PngImageSourceStream extends ImageSourceStream {
    public PngImageSourceStream ( int width, int height, int frameRate,
                                  Vector<String> images ) {
        super ( width, height, frameRate, images );
        format = new RGBFormat ( new Dimension ( width, height ),
                                 Format.NOT_SPECIFIED, Format.byteArray, frameRate,
                                 24,
                                 1, 2, 3 );
    }
    public void read ( Buffer buf ) throws IOException {
        if ( nextImage >= images.size() ) {
            System.err.println ( "Done reading all images." );
            buf.setEOM ( true );
            buf.setOffset ( 0 );
            buf.setLength ( 0 );
            ended = true;
            return;
        }
        String imageFile = ( String ) images.elementAt ( nextImage );
        nextImage++;
        System.err.println ( "  - reading image file: " + imageFile );
        BufferedImage image = ImageIO.read ( new File ( imageFile ) );
        boolean hasAlpha = image.getColorModel().hasAlpha();
        Dimension size = format.getSize();
        byte[] imageData = convertTo24Bit ( hasAlpha, image.getRaster().getPixels ( 0, 0, size.width, size.height, ( int[] ) null ) );
        buf.setData ( imageData );
        System.err.println ( "    read " + imageData.length + " bytes." );
        buf.setOffset ( 0 );
        buf.setLength ( imageData.length );
        buf.setFormat ( format );
        buf.setFlags ( buf.getFlags() | Buffer.FLAG_KEY_FRAME );
    }
    private void convertIntByteToByte ( int[] src, int srcIndex, byte[] out, int outIndex ) {
        int r = src[srcIndex];
        int g = src[srcIndex + 1];
        int b = src[srcIndex + 2];
        out[outIndex] = ( byte ) ( r & 0xFF );
        out[outIndex + 1] = ( byte ) ( g & 0xFF );
        out[outIndex + 2] = ( byte ) ( b & 0xFF );
    }
    private byte[] convertTo24Bit ( boolean hasAlpha, int[] input ) {
        int dataLength = input.length;
        int newSize = ( hasAlpha ? dataLength * 3 / 4 : dataLength );
        byte[] convertedData = new byte[newSize];
        for ( int i = 0, j = 0; i < dataLength; i += 3, j += 3 ) {
            convertIntByteToByte ( input, i, convertedData, j );
            if ( hasAlpha ) {
                i++;
            }
        }
        return convertedData;
    }
}
