public static RSACryptoServiceProvider DecodeX509PublicKey ( byte[] x509key ) {
    byte[] SeqOID = { 0x2A, 0x86, 0x48, 0x86, 0xF7, 0x0D, 0x01, 0x01, 0x01 };
    MemoryStream ms = new MemoryStream ( x509key );
    BinaryReader reader = new BinaryReader ( ms );
    if ( reader.ReadByte() == 0x30 ) {
        ReadASNLength ( reader );
    } else {
        return null;
    }
    int identifierSize = 0;
    if ( reader.ReadByte() == 0x30 ) {
        identifierSize = ReadASNLength ( reader );
    } else {
        return null;
    }
    if ( reader.ReadByte() == 0x06 ) {
        int oidLength = ReadASNLength ( reader );
        byte[] oidBytes = new byte[oidLength];
        reader.Read ( oidBytes, 0, oidBytes.Length );
        if ( oidBytes.SequenceEqual ( SeqOID ) == false ) {
            return null;
        }
        int remainingBytes = identifierSize - 2 - oidBytes.Length;
        reader.ReadBytes ( remainingBytes );
    }
    if ( reader.ReadByte() == 0x03 ) {
        ReadASNLength ( reader );
        reader.ReadByte();
        if ( reader.ReadByte() == 0x30 ) {
            ReadASNLength ( reader );
            if ( reader.ReadByte() == 0x02 ) {
                int modulusSize = ReadASNLength ( reader );
                byte[] modulus = new byte[modulusSize];
                reader.Read ( modulus, 0, modulus.Length );
                if ( modulus[0] == 0x00 ) {
                    byte[] tempModulus = new byte[modulus.Length - 1];
                    Array.Copy ( modulus, 1, tempModulus, 0, modulus.Length - 1 );
                    modulus = tempModulus;
                }
                if ( reader.ReadByte() == 0x02 ) {
                    int exponentSize = ReadASNLength ( reader );
                    byte[] exponent = new byte[exponentSize];
                    reader.Read ( exponent, 0, exponent.Length );
                    RSACryptoServiceProvider RSA = new RSACryptoServiceProvider();
                    RSAParameters RSAKeyInfo = new RSAParameters();
                    RSAKeyInfo.Modulus = modulus;
                    RSAKeyInfo.Exponent = exponent;
                    RSA.ImportParameters ( RSAKeyInfo );
                    return RSA;
                }
            }
        }
    }
    return null;
}
public static int ReadASNLength ( BinaryReader reader ) {
    int length = reader.ReadByte();
    if ( ( length & 0x00000080 ) == 0x00000080 ) {
        int count = length & 0x0000000f;
        byte[] lengthBytes = new byte[4];
        reader.Read ( lengthBytes, 4 - count, count );
        Array.Reverse ( lengthBytes );
        length = BitConverter.ToInt32 ( lengthBytes, 0 );
    }
    return length;
}
