public class InterceptorRunner extends BlockJUnit4ClassRunner {
    @Retention ( RetentionPolicy.RUNTIME )
    @Target ( ElementType.TYPE )
    public @interface InterceptorClasses {
        public Class<?>[] value();
    }
    public InterceptorRunner ( Class<?> clazz ) throws InitializationError {
        super ( clazz );
    }
    @Override
    public Statement methodInvoker ( FrameworkMethod method, Object test ) {
        InterceptorStatement statement = new InterceptorStatement ( super.methodInvoker ( method, test ) );
        InterceptorClasses annotation = test.getClass().getAnnotation ( InterceptorClasses.class );
        Class<?>[] klasez = annotation.value();
        try {
            for ( Class<?> klaz : klasez ) {
                statement.addInterceptor ( ( Interceptor ) klaz.newInstance() );
            }
        } catch ( IllegalAccessException ilex ) {
            ilex.printStackTrace();
        } catch ( InstantiationException e ) {
            e.printStackTrace();
        }
        return statement;
    }
}
public class InterceptorStatement extends Statement {
    private final Statement invoker;
    private List<Interceptor> interceptors = new ArrayList<Interceptor>();
    public InterceptorStatement ( Statement invoker ) {
        this.invoker = invoker;
    }
    @Override
    public void evaluate() throws Throwable {
        for ( Interceptor interceptor : interceptors ) {
            interceptor.interceptBefore();
        }
        invoker.evaluate();
        for ( Interceptor interceptor : interceptors ) {
            interceptor.interceptAfter();
        }
    }
    public void addInterceptor ( Interceptor interceptor ) {
        interceptors.add ( interceptor );
    }
}
