import java.io.IOException;
import java.io.InputStream;
import java.io.InvalidClassException;
import java.io.ObjectInputStream;
import java.io.ObjectStreamClass;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
public class DecompressibleInputStream extends ObjectInputStream {
    private static Logger logger = LoggerFactory.getLogger ( DecompressibleInputStream.class );
    public DecompressibleInputStream ( InputStream in ) throws IOException {
        super ( in );
    }
    protected ObjectStreamClass readClassDescriptor() throws IOException, ClassNotFoundException {
        ObjectStreamClass resultClassDescriptor = super.readClassDescriptor();
        Class localClass;
        try {
            localClass = Class.forName ( resultClassDescriptor.getName() );
        } catch ( ClassNotFoundException e ) {
            logger.error ( "No local class for " + resultClassDescriptor.getName(), e );
            return resultClassDescriptor;
        }
        ObjectStreamClass localClassDescriptor = ObjectStreamClass.lookup ( localClass );
        if ( localClassDescriptor != null ) {
            final long localSUID = localClassDescriptor.getSerialVersionUID();
            final long streamSUID = resultClassDescriptor.getSerialVersionUID();
            if ( streamSUID != localSUID ) {
                final StringBuffer s = new StringBuffer ( "Overriding serialized class version mismatch: " );
                s.append ( "local serialVersionUID = " ).append ( localSUID );
                s.append ( " stream serialVersionUID = " ).append ( streamSUID );
                Exception e = new InvalidClassException ( s.toString() );
                logger.error ( "Potentially Fatal Deserialization Operation.", e );
                resultClassDescriptor = localClassDescriptor;
            }
        }
        return resultClassDescriptor;
    }
}
