import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
public class KeyBindingExample {
    private DrawingBoard contentPane;
    private int speed;
    private int brakes;
    public KeyBindingExample() {
        speed = 5;
        brakes = 0;
    }
    private void displayGUI() {
        JFrame frame = new JFrame ( "Swing Worker Example" );
        frame.setDefaultCloseOperation ( JFrame.DISPOSE_ON_CLOSE );
        contentPane = new DrawingBoard ( 10, 10, Color.BLUE.darker() );
        addBindingsToBoard();
        frame.setContentPane ( contentPane );
        frame.pack();
        frame.setLocationByPlatform ( true );
        frame.setVisible ( true );
    }
    private void addBindingsToBoard() {
        putBindingsFor ( contentPane, KeyStroke.getKeyStroke ( "UP" ),
                         "UP Arrow Key", brakes, -speed );
        putBindingsFor ( contentPane, KeyStroke.getKeyStroke ( "RIGHT" ),
                         "RIGHT Arrow Key", speed, brakes );
        putBindingsFor ( contentPane, KeyStroke.getKeyStroke ( "DOWN" ),
                         "DOWN Arrow Key", brakes, +speed );
        putBindingsFor ( contentPane, KeyStroke.getKeyStroke ( "LEFT" ),
                         "LEFT Arrow Key", -speed, brakes );
    }
    private void putBindingsFor ( JComponent comp,
                                  KeyStroke keyStroke, String value, final int moveX, final int moveY ) {
        comp.getInputMap().put ( keyStroke, value );
        comp.getActionMap().put ( value, new AbstractAction() {
            @Override
            public void actionPerformed ( ActionEvent ae ) {
                contentPane.setValues ( moveX, moveY );
            }
        } );
    }
    public static void main ( String[] args ) {
        Runnable runnable = new Runnable() {
            @Override
            public void run() {
                new KeyBindingExample().displayGUI();
            }
        };
        EventQueue.invokeLater ( runnable );
    }
}
class DrawingBoard extends JPanel {
    private int x;
    private int y;
    private int width;
    private int height;
    private Color rectColor;
    public DrawingBoard ( int x, int y, Color rColor ) {
        setOpaque ( true );
        this.x = x;
        this.y = y;
        rectColor = rColor;
        width = height = 10;
    }
    public void setValues ( int deltaX, int deltaY ) {
        System.out.format ( "Firstly X : %d\tY : %d%n", x, y );
        repaint ( x, y, width, height );
        x += deltaX;
        y += deltaY;
        if ( ( x + width ) <= getWidth() && ( y + height ) <= getHeight()
                && x >= 0 && y >= 0 ) {
            System.out.format ( "Later X : %d\tY : %d%n", x, y );
            repaint ( x, y, width, height );
        } else {
            x -= deltaX;
            y -= deltaY;
            System.out.format ( "Later X : %d\tY : %d%n", x, y );
            repaint ( x, y, width, height );
        }
    }
    @Override
    public Dimension getPreferredSize() {
        return ( new Dimension ( 100, 100 ) );
    }
    @Override
    protected void paintComponent ( Graphics g ) {
        super.paintComponent ( g );
        g.setColor ( rectColor );
        g.fillRect ( x, y, width, height );
    }
}
