package imageio.byteconversion;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.io.InputStream;
import javax.imageio.ImageIO;
public class BufferedImageToBytes {
    private static void log ( String s ) {
        System.out.println ( s );
    }
    private static String toByteString ( int color ) {
        return Integer.toHexString ( color & 0xFF );
    }
    public static void main ( String[] args ) throws IOException {
        InputStream stream = BufferedImageToBytes.class.getResourceAsStream ( "test.png" );
        BufferedImage image = ImageIO.read ( stream );
        int iw = image.getWidth();
        int ih = image.getHeight();
        log ( "Image loaded succesfully, width=" + iw + " height=" + ih );
        stream.close();
        log ( "Image color model: " + image.getColorModel() );
        log ( "Image sample model: " + image.getSampleModel() );
        log ( "Image raster: " + image.getRaster() );
        int bands = image.getSampleModel().getNumBands();
        log ( "Color bands: " + bands );
        if ( bands != 4 ) {
            throw new RuntimeException ( "The image does not have 4 color bands. Are you sure this is a 32-bit image?" );
        }
        int threshold = 199;
        byte bytes[] = new byte[4 * iw * ih];
        int index = 0;
        for ( int y = 0; y < ih; y++ ) {
            for ( int x = 0; x < iw; x++ ) {
                int pixel = image.getRGB ( x, y );
                int alpha = ( pixel >> 24 ) & 0xFF;
                int red = ( pixel >> 16 ) & 0xFF;
                int green = ( pixel >> 8 ) & 0xFF;
                int blue = pixel & 0xFF;
                if ( red > threshold ) {
                    red = 255;
                } else {
                    red = 0;
                }
                log ( "[" + x + "," + y + "]" + " Converting [" + Integer.toHexString ( pixel )  + "] --> ["
                      + toByteString ( red ) + ", " + toByteString ( green ) + ", "
                      + toByteString ( blue ) + ", " + toByteString ( alpha )
                      + "]" );
                bytes[index++] = ( byte ) red;
                bytes[index++] = ( byte ) green;
                bytes[index++] = ( byte ) blue;
                bytes[index++] = ( byte ) alpha;
            }
        }
    }
}
