public class IdentityTreeSet<T extends Comparable> extends AbstractCollection<T> {
    private static final Object PRESENT = new Object();
    private SortedMap<T, IdentityHashMap<T, Object>> values = new TreeMap<T, IdentityHashMap<T, Object>>();
    @Override
    public Iterator<T> iterator() {
        return new Iterator<T>() {
            Iterator<IdentityHashMap<T, Object>> outerIterator = values.values().iterator();
            IdentityHashMap<T, Object> currentMap = new IdentityHashMap();
            Iterator<T> innerIterator = currentMap.keySet().iterator();
            @Override
            public boolean hasNext() {
                return innerIterator.hasNext() || outerIterator.hasNext();
            }
            @Override
            public T next() {
                if ( innerIterator.hasNext() ) {
                    return innerIterator.next();
                } else {
                    currentMap = outerIterator.next();
                    innerIterator = currentMap.keySet().iterator();
                    return next();
                }
            }
            @Override
            public void remove() {
                innerIterator.remove();
                if ( currentMap.isEmpty() ) {
                    outerIterator.remove();
                }
            }
        };
    }
    @Override
    public int size() {
        int i = 0;
        for ( T t : this ) {
            i++;
        }
        return i;
    }
    @Override
    public boolean add ( T e ) {
        IdentityHashMap<T, Object> entrySet = values.get ( e );
        if ( entrySet == null ) {
            IdentityHashMap<T, Object> newList = new IdentityHashMap<T, Object>();
            newList.put ( e, PRESENT );
            values.put ( e, newList );
            return true;
        } else {
            if ( entrySet.containsKey ( e ) ) {
                return false;
            }
            entrySet.put ( e, PRESENT );
            return true;
        }
    }
    @Override
    public boolean remove ( Object o ) {
        IdentityHashMap<T, Object> entrySet = values.get ( o );
        if ( entrySet == null ) {
            return false;
        } else {
            if ( ! entrySet.containsKey ( o ) ) {
                return false;
            } else {
                entrySet.remove ( o );
                if ( entrySet.isEmpty() ) {
                    values.remove ( o );
                }
                return true;
            }
        }
    }
}
