import android.annotation.SuppressLint;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Bitmap.Config;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Path;
import android.graphics.Rect;
import android.util.AttributeSet;
import android.util.LruCache;
import android.view.MotionEvent;
import android.view.View;
public class MyDrawView extends View {
    public Bitmap  mBitmap;
    public Canvas  mCanvas;
    private Path    mPath;
    private Paint   mBitmapPaint;
    private Paint   mPaint;
    public MyDrawView ( Context c, AttributeSet attrs ) {
        super ( c, attrs );
        mPath = new Path();
        mBitmapPaint = new Paint ( Paint.DITHER_FLAG );
        mPaint = new Paint();
        mPaint.setAntiAlias ( true );
        mPaint.setDither ( true );
        mPaint.setColor ( 0xFF000000 );
        mPaint.setStyle ( Paint.Style.STROKE );
        mPaint.setStrokeJoin ( Paint.Join.ROUND );
        mPaint.setStrokeCap ( Paint.Cap.ROUND );
        mPaint.setStrokeWidth ( 9 );
    }
    @Override
    protected void onSizeChanged ( int w, int h, int oldw, int oldh ) {
        super.onSizeChanged ( w, h, oldw, oldh );
        mBitmap = Bitmap.createBitmap ( w, h, Bitmap.Config.ARGB_8888 );
        mCanvas = new Canvas ( mBitmap );
    }
    @Override
    protected void onDraw ( Canvas canvas ) {
        canvas.drawBitmap ( mBitmap, 0, 0, mBitmapPaint );
        canvas.drawPath ( mPath, mPaint );
    }
    private float mX, mY;
    private static final float TOUCH_TOLERANCE = 4;
    private void touch_start ( float x, float y ) {
        mPath.reset();
        mPath.moveTo ( x, y );
        mX = x;
        mY = y;
    }
    private void touch_move ( float x, float y ) {
        float dx = Math.abs ( x - mX );
        float dy = Math.abs ( y - mY );
        if ( dx >= TOUCH_TOLERANCE || dy >= TOUCH_TOLERANCE ) {
            mPath.quadTo ( mX, mY, ( x + mX ) / 2, ( y + mY ) / 2 );
            mX = x;
            mY = y;
        }
    }
    private void touch_up() {
        mPath.lineTo ( mX, mY );
        mCanvas.drawPath ( mPath, mPaint );
        mPath.reset();
    }
    @Override
    public boolean onTouchEvent ( MotionEvent event ) {
        float x = event.getX();
        float y = event.getY();
        switch ( event.getAction() ) {
        case MotionEvent.ACTION_DOWN:
            touch_start ( x, y );
            invalidate();
            break;
        case MotionEvent.ACTION_MOVE:
            touch_move ( x, y );
            invalidate();
            break;
        case MotionEvent.ACTION_UP:
            touch_up();
            invalidate();
            break;
        }
        return true;
    }
    public Bitmap getBitmap() {
        this.setDrawingCacheEnabled ( true );
        this.buildDrawingCache();
        Bitmap bmp = Bitmap.createBitmap ( this.getDrawingCache() );
        this.setDrawingCacheEnabled ( false );
        return bmp;
    }
    public void clear() {
        mBitmap.eraseColor ( Color.GREEN );
        invalidate();
        System.gc();
    }
}
