public interface Function<T> {
    T execute ( T input );
}
public static <T> List<T> map ( List<T> input, final Function<T> func )
throws InterruptedException, ExecutionException {
    List<Future<T>> futures = new ArrayList<Future<T>> ( input.size() );
    ExecutorService executor = Executors.newFixedThreadPool ( 10 );
    for ( final T item : input ) {
        futures.add ( executor.submit ( new Callable<T>() {
            @Override
            public T call() {
                return func.execute ( item );
            }
        } ) );
    }
    executor.shutdown();
    executor.awaitTermination ( Long.MAX_VALUE, TimeUnit.NANOSECONDS );
    List<T> ret = new ArrayList<T> ( futures.size() );
    for ( Future<T> future : futures ) {
        ret.add ( future.get() );
    }
}
