import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import javax.swing.event.TableModelEvent;
import javax.swing.table.*;
public class ResultSetTableModel extends AbstractTableModel {
    private static final long serialVersionUID = 1L;
    private Connection connection;
    private Statement statement;
    private ResultSet resultSet;
    private ResultSetMetaData metaData;
    private int numberOfRows;
    public ResultSetTableModel ( String driver, String url,
                                 String query ) throws SQLException, ClassNotFoundException {
        Class.forName ( driver );
        connection = DriverManager.getConnection ( url );
        statement = connection.createStatement (
                        ResultSet.TYPE_SCROLL_INSENSITIVE,
                        ResultSet.CONCUR_READ_ONLY );
        setQuery ( query );
    }
    @Override
    public Class<?> getColumnClass ( int column ) {
        try {
            String className = metaData.getColumnClassName ( column + 1 );
            return Class.forName ( className );
        } catch ( Exception exception ) {
            exception.printStackTrace();
        }
        return Object.class;
    }
    @Override
    public int getColumnCount() {
        try {
            return metaData.getColumnCount();
        } catch ( SQLException sqlException ) {
            sqlException.printStackTrace();
        }
        return 0;
    }
    @Override
    public String getColumnName ( int column ) {
        try {
            return metaData.getColumnName ( column + 1 );
        } catch ( SQLException sqlException ) {
            sqlException.printStackTrace();
        }
        return "";
    }
    @Override
    public int getRowCount() {
        return numberOfRows;
    }
    @Override
    public Object getValueAt ( int row, int column ) {
        try {
            resultSet.absolute ( row + 1 );
            return resultSet.getObject ( column + 1 );
        } catch ( SQLException sqlException ) {
            sqlException.printStackTrace();
        }
        fireTableCellUpdated ( row, column );
        return "";
    }
    @Override
    protected void finalize() {
        try {
            statement.close();
            connection.close();
        } catch ( SQLException sqlException ) {
            sqlException.printStackTrace();
        }
    }
    public void setQuery ( String query ) throws SQLException {
        resultSet = statement.executeQuery ( query );
        metaData = resultSet.getMetaData();
        resultSet.last();
        numberOfRows = resultSet.getRow();
        fireTableStructureChanged();
        fireTableChanged ( null );
    }
    public void tableChanged ( TableModelEvent e ) {
        fireTableChanged ( e );
    }
}
