@Controller
@RequestMapping ( "/p/{abbr}/scope" )
public class ScopeController {
    private static final String SHOW_PROJECT_PAGE = "/projects/scope/show";
    private static final Logger log = LoggerFactory.getLogger ( ScopeController.class );
    @Autowired
    private ProjectService projectService;
    @ModelAttribute
    private void initProject ( @PathVariable ( value = "abbr" ) String abbr, Model model ) {
        log.debug ( "loading project for '{}'", abbr );
        model.addAttribute ( "project", projectService.find ( abbr ) );
    }
    @RequestMapping ( method = RequestMethod.GET )
    public String show() throws BindException {
        return SHOW_PROJECT_PAGE;
    }
    @RequestMapping ( method = RequestMethod.POST )
    public String update (
        @Valid @ModelAttribute Project project, BindingResult result,
        RedirectAttributes redirectAttrs )
    throws MethodArgumentNotValidException {
        redirectAttrs.addFlashAttribute ( project );
        try {
            if ( !result.hasErrors() ) {
                projectService.save ( project );
            }
        } catch ( Exception e ) {
            log.error ( e.toString() );
            throw new MethodArgumentNotValidException ( null, result );
        }
        log.debug ( "project '{}' updated", project.getAbbreviation() );
        return SHOW_PROJECT_PAGE;
    }
}
