import javafx.application.Application;
import javafx.beans.property.BooleanProperty;
import javafx.beans.property.SimpleBooleanProperty;
import javafx.event.*;
import javafx.scene.Node;
import javafx.scene.Scene;
import javafx.scene.control.*;
import javafx.scene.effect.BoxBlur;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.*;
import javafx.scene.paint.Color;
import javafx.scene.web.WebView;
import javafx.stage.*;
import javafx.util.Callback;
public class WebViewConfirm extends Application {
    public static void main ( String[] args ) {
        launch ( args );
    }
    @Override public void start ( final Stage primaryStage ) {
        primaryStage.setTitle ( "Modal Confirm Example" );
        final WebView webView = new WebView();
        webView.getEngine().load ( "http://www.w3schools.com/js/tryit.asp?filename=tryjs_confirm" );
        final VBox confirmationResults = new VBox();
        confirmationResults.getStyleClass().add ( "confirmation-results" );
        confirmationResults.setMinWidth ( 150 );
        HBox layout = new HBox();
        layout.getChildren().addAll ( confirmationResults, webView );
        primaryStage.setScene ( new Scene ( layout ) );
        primaryStage.show();
        primaryStage.getScene().getStylesheets().add ( getClass().getResource ( "modal-dialog.css" ).toExternalForm() );
        webView.getEngine().setConfirmHandler ( new Callback<String, Boolean>() {
            @Override public Boolean call ( String msg ) {
                Boolean confirmed = confirm ( primaryStage, msg );
                confirmationResults.getChildren().add ( new Label ( "Confirmed? " + confirmed ) );
                return confirmed;
            }
        } );
    }
    private Boolean confirm ( final Stage parent, String msg ) {
        final BooleanProperty confirmationResult = new SimpleBooleanProperty();
        final Stage dialog = new Stage ( StageStyle.TRANSPARENT );
        dialog.initOwner ( parent );
        dialog.initModality ( Modality.WINDOW_MODAL );
        dialog.setScene (
            new Scene (
                HBoxBuilder.create().styleClass ( "modal-dialog" ).children (
                    LabelBuilder.create().text ( msg ).build(),
        ButtonBuilder.create().text ( "OK" ).defaultButton ( true ).onAction ( new EventHandler<ActionEvent>() {
            @Override public void handle ( ActionEvent actionEvent ) {
                confirmationResult.set ( true );
                parent.getScene().getRoot().setEffect ( null );
                dialog.close();
            }
        } ).build(),
        ButtonBuilder.create().text ( "Cancel" ).cancelButton ( true ).onAction ( new EventHandler<ActionEvent>() {
            @Override public void handle ( ActionEvent actionEvent ) {
                confirmationResult.set ( false );
                parent.getScene().getRoot().setEffect ( null );
                dialog.close();
            }
        } ).build()
                ).build()
        , Color.TRANSPARENT
            )
        );
        final Node root = dialog.getScene().getRoot();
        final Delta dragDelta = new Delta();
        root.setOnMousePressed ( new EventHandler<MouseEvent>() {
            @Override public void handle ( MouseEvent mouseEvent ) {
                dragDelta.x = dialog.getX() - mouseEvent.getScreenX();
                dragDelta.y = dialog.getY() - mouseEvent.getScreenY();
            }
        } );
        root.setOnMouseDragged ( new EventHandler<MouseEvent>() {
            @Override public void handle ( MouseEvent mouseEvent ) {
                dialog.setX ( mouseEvent.getScreenX() + dragDelta.x );
                dialog.setY ( mouseEvent.getScreenY() + dragDelta.y );
            }
        } );
        dialog.getScene().getStylesheets().add ( getClass().getResource ( "modal-dialog.css" ).toExternalForm() );
        parent.getScene().getRoot().setEffect ( new BoxBlur() );
        dialog.showAndWait();
        return confirmationResult.get();
    }
    class Delta {
        double x, y;
    }
}
