package com.marakana.oauth;
import com.marakana.oauth.R;
import oauth.signpost.OAuth;
import oauth.signpost.OAuthConsumer;
import oauth.signpost.OAuthProvider;
import oauth.signpost.basic.DefaultOAuthConsumer;
import oauth.signpost.basic.DefaultOAuthProvider;
import oauth.signpost.exception.OAuthCommunicationException;
import oauth.signpost.exception.OAuthExpectationFailedException;
import oauth.signpost.exception.OAuthMessageSignerException;
import oauth.signpost.exception.OAuthNotAuthorizedException;
import winterwell.jtwitter.OAuthSignpostClient;
import winterwell.jtwitter.Twitter;
import winterwell.jtwitter.TwitterException;
import android.app.Activity;
import android.content.Intent;
import android.content.SharedPreferences;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.util.Log;
import android.view.View;
import android.widget.EditText;
import android.widget.Toast;
public class MainActivity extends Activity {
    private static final String TAG = "OAuthDemo";
    private static final String OAUTH_KEY = "your_OAUTH_KEY";
    private static final String OAUTH_SECRET = "your_OAUTH_SECRET";
    private static final String TWITTER_USER = "your_email";
    private static final String OAUTH_CALLBACK_SCHEME = "x-marakana-oauth-twitter";
    private static final String OAUTH_CALLBACK_URL = OAUTH_CALLBACK_SCHEME + "://callback";
    private OAuthSignpostClient oauthClient;
    private OAuthConsumer mConsumer;
    private OAuthProvider mProvider;
    private Twitter twitter;
    SharedPreferences prefs;
    @Override
    public void onCreate ( Bundle savedInstanceState ) {
        super.onCreate ( savedInstanceState );
        setContentView ( R.layout.activity_main );
        mConsumer = new DefaultOAuthConsumer ( OAUTH_KEY, OAUTH_SECRET );
        mProvider = new DefaultOAuthProvider (
            "https://api.twitter.com/oauth/request_token",
            "https://api.twitter.com/oauth/access_token",
            "https://api.twitter.com/oauth/authorize" );
        prefs = PreferenceManager.getDefaultSharedPreferences ( this );
        String token = prefs.getString ( "token", null );
        String tokenSecret = prefs.getString ( "tokenSecret", null );
        if ( token != null && tokenSecret != null ) {
            mConsumer.setTokenWithSecret ( token, tokenSecret );
            oauthClient = new OAuthSignpostClient ( OAUTH_KEY, OAUTH_SECRET,
                                                    token, tokenSecret );
            twitter = new Twitter ( TWITTER_USER, oauthClient );
        }
    }
    @Override
    public void onNewIntent ( Intent intent ) {
        super.onNewIntent ( intent );
        Log.d ( TAG, "intent: " + intent );
        Uri uri = intent.getData();
        if ( uri != null && uri.getScheme().equals ( OAUTH_CALLBACK_SCHEME ) ) {
            Log.d ( TAG, "callback: " + uri.getPath() );
            String verifier = uri.getQueryParameter ( OAuth.OAUTH_VERIFIER );
            Log.d ( TAG, "verifier: " + verifier );
            new RetrieveAccessTokenTask().execute ( verifier );
        }
    }
    public void onClickAuthorize ( View view ) {
        new OAuthAuthorizeTask().execute();
    }
    public void onClickTweet ( View view ) {
        if ( twitter == null ) {
            Toast.makeText ( this, "Authenticate first", Toast.LENGTH_LONG )
            .show();
            return;
        }
        EditText status = ( EditText ) findViewById ( R.id.status );
        new PostStatusTask().execute ( status.getText().toString() );
    }
    public void onClickGetStatus ( View view ) {
        if ( twitter == null ) {
            Toast.makeText ( this, "Authenticate first", Toast.LENGTH_LONG )
            .show();
            return;
        }
        new GetStatusTask().execute();
    }
    class OAuthAuthorizeTask extends AsyncTask<Void, Void, String> {
        @Override
        protected String doInBackground ( Void... params ) {
            String authUrl;
            String message = null;
            try {
                authUrl = mProvider.retrieveRequestToken ( mConsumer,
                          OAUTH_CALLBACK_URL );
                Intent intent = new Intent ( Intent.ACTION_VIEW,
                                             Uri.parse ( authUrl ) );
                startActivity ( intent );
            } catch ( OAuthMessageSignerException e ) {
                message = "OAuthMessageSignerException";
                e.printStackTrace();
            } catch ( OAuthNotAuthorizedException e ) {
                message = "OAuthNotAuthorizedException";
                e.printStackTrace();
            } catch ( OAuthExpectationFailedException e ) {
                message = "OAuthExpectationFailedException";
                e.printStackTrace();
            } catch ( OAuthCommunicationException e ) {
                message = "OAuthCommunicationException";
                e.printStackTrace();
            }
            return message;
        }
        @Override
        protected void onPostExecute ( String result ) {
            super.onPostExecute ( result );
            if ( result != null ) {
                Toast.makeText ( MainActivity.this, result, Toast.LENGTH_LONG )
                .show();
            }
        }
    }
    class RetrieveAccessTokenTask extends AsyncTask<String, Void, String> {
        @Override
        protected String doInBackground ( String... params ) {
            String message = null;
            String verifier = params[0];
            try {
                Log.d ( TAG, "mConsumer: " + mConsumer );
                Log.d ( TAG, "mProvider: " + mProvider );
                mProvider.retrieveAccessToken ( mConsumer, verifier );
                String token = mConsumer.getToken();
                String tokenSecret = mConsumer.getTokenSecret();
                mConsumer.setTokenWithSecret ( token, tokenSecret );
                Log.d ( TAG, String.format (
                            "verifier: %s, token: %s, tokenSecret: %s", verifier,
                            token, tokenSecret ) );
                prefs.edit().putString ( "token", token )
                .putString ( "tokenSecret", tokenSecret ).commit();
                oauthClient = new OAuthSignpostClient ( OAUTH_KEY, OAUTH_SECRET,
                                                        token, tokenSecret );
                twitter = new Twitter ( "NameTest", oauthClient );
                Log.d ( TAG, "token: " + token );
            } catch ( OAuthMessageSignerException e ) {
                message = "OAuthMessageSignerException";
                e.printStackTrace();
            } catch ( OAuthNotAuthorizedException e ) {
                message = "OAuthNotAuthorizedException";
                e.printStackTrace();
            } catch ( OAuthExpectationFailedException e ) {
                message = "OAuthExpectationFailedException";
                e.printStackTrace();
            } catch ( OAuthCommunicationException e ) {
                message = "OAuthCommunicationException";
                e.printStackTrace();
            }
            return message;
        }
        @Override
        protected void onPostExecute ( String result ) {
            super.onPostExecute ( result );
            if ( result != null ) {
                Toast.makeText ( MainActivity.this, result, Toast.LENGTH_LONG )
                .show();
            }
        }
    }
    class GetStatusTask extends AsyncTask<Void, Void, String> {
        @Override
        protected String doInBackground ( Void... params ) {
            return twitter.getStatus().text;
        }
        @Override
        protected void onPostExecute ( String result ) {
            super.onPostExecute ( result );
            Toast.makeText ( MainActivity.this, result, Toast.LENGTH_LONG ).show();
        }
    }
    class PostStatusTask extends AsyncTask<String, Void, String> {
        @Override
        protected String doInBackground ( String... params ) {
            try {
                twitter.setStatus ( params[0] );
                return "Successfully posted: " + params[0];
            } catch ( TwitterException e ) {
                return "Error connecting to server.";
            }
        }
        @Override
        protected void onPostExecute ( String result ) {
            super.onPostExecute ( result );
            Toast.makeText ( MainActivity.this, result, Toast.LENGTH_LONG ).show();
        }
    }
}
