public class XmlTag {
    private String _tagName;
    private String _tagValue;
    private HashMap<String, String> _tagAttributes;
    private ArrayList<XmlTag> _innerTags;
    void addInnerXmlTag ( final XmlTag tag ) {
        if ( tag == null ) {
            return;
        }
        if ( _innerTags == null ) {
            _innerTags = new ArrayList<XmlTag>();
        }
        _innerTags.add ( tag );
    }
    public String getStringFormat() {
        final StringBuilder sb = new StringBuilder();
        sb.append ( "<" + getTagName() );
        final int numberOfAttributes = getTagAttributes() != null ? getTagAttributes().size() : 0;
        if ( numberOfAttributes != 0 )
            for ( final Entry<String, String> attributeEntry : getTagAttributes().entrySet() ) {
                sb.append ( " " + attributeEntry.getKey() + "=\"" + attributeEntry.getValue() + "\"" );
            }
        final int numberOfInnerTags = getInnerXmlTags() != null ? getInnerXmlTags().size() : 0;
        if ( numberOfInnerTags == 0 ) {
            sb.append ( " />" );
        } else {
            sb.append ( numberOfAttributes == 0 ? ">" : " >" );
            for ( final XmlTag innerTag : getInnerXmlTags() ) {
                sb.append ( innerTag.getStringFormat() );
            }
            sb.append ( "</" + getTagName() + ">" );
        }
        return sb.toString();
    }
    @Override
    public String toString() {
        return getStringFormat();
    }
    public static XmlTag getXmlRootTagOfXmlFileResourceId ( final Context context, final int xmlFileResourceId ) {
        XmlTag currentTag = null, rootTag = null;
        final Stack<XmlTag> tagsStack = new Stack<XmlTag>();
        final Resources res = context.getResources();
        final XmlResourceParser xmlParser = res.getXml ( xmlFileResourceId );
        try {
            xmlParser.next();
            int eventType = xmlParser.getEventType();
            boolean doneParsing = false;
            while ( eventType != XmlPullParser.END_DOCUMENT && !doneParsing ) {
                switch ( eventType ) {
                case XmlPullParser.START_DOCUMENT:
                    break;
                case XmlPullParser.START_TAG:
                    final String xmlTagName = xmlParser.getName();
                    currentTag = new XmlTag();
                    if ( tagsStack.isEmpty() ) {
                        rootTag = currentTag;
                    }
                    tagsStack.push ( currentTag );
                    final int numberOfAttributes = xmlParser.getAttributeCount();
                    if ( numberOfAttributes > 0 ) {
                        final HashMap<String, String> attributes = new HashMap<String, String>();
                        for ( int i = 0; i < numberOfAttributes; ++i ) {
                            final String attrName = xmlParser.getAttributeName ( i );
                            final String attrValue = xmlParser.getAttributeValue ( i );
                            attributes.put ( attrName, attrValue );
                        }
                        currentTag.setTagAttributes ( attributes );
                    }
                    currentTag.setTagName ( xmlTagName );
                    break;
                case XmlPullParser.END_TAG:
                    currentTag = tagsStack.pop();
                    if ( !tagsStack.isEmpty() ) {
                        final XmlTag parentTag = tagsStack.peek();
                        parentTag.addInnerXmlTag ( currentTag );
                        currentTag = parentTag;
                    } else {
                        doneParsing = true;
                    }
                    break;
                case XmlPullParser.TEXT:
                    final String tagValue = xmlParser.getText();
                    if ( currentTag != null ) {
                        currentTag.setTagValue ( tagValue );
                    }
                    break;
                }
                eventType = xmlParser.next();
            }
        } catch ( final Exception e ) {
            rootTag = null;
        }
        return rootTag;
    }
    public HashMap<String, String> getTagAttributes() {
        return _tagAttributes;
    }
    public String getTagName() {
        return _tagName;
    }
    public String getTagValue() {
        return _tagValue;
    }
    public ArrayList<XmlTag> getInnerXmlTags() {
        return _innerTags;
    }
    void setTagAttributes ( final HashMap<String, String> tagAttributes ) {
        _tagAttributes = tagAttributes;
    }
    void setTagName ( final String tagName ) {
        _tagName = tagName;
    }
    void setTagValue ( final String tagValue ) {
        _tagValue = tagValue;
    }
}
