public class TestReport implements Printable {
    private int numberOfPages = 0;
    private int headerLines = 0;
    private int footerLines = 0;
    private int bodyLines = 0;
    private boolean useDefaultFooter = false;
    private int fontSize = 10;
    private Vector header = new Vector();
    private Vector body = new Vector();
    private Vector footer = new Vector();
    public TestReport() {
    }
    public void printReport() {
        headerLines = header.size();
        bodyLines = body.size();
        if ( useDefaultFooter == true ) {
            footerLines = 1;
        } else {
            footerLines = footer.size();
        }
        PrinterJob printerJob = PrinterJob.getPrinterJob();
        PageFormat landscape = printerJob.defaultPage();
        landscape.setOrientation ( PageFormat.LANDSCAPE );
        printerJob.setPrintable ( TestReport.this, landscape );
        printerJob.printDialog();
        try {
            printerJob.print();
        } catch ( Exception PrintException ) {}
    }
    public void addHeaderLine ( String headerLine ) {
        header.addElement ( headerLine );
    }
    public void addBodyLine ( String bodyLine ) {
        body.addElement ( bodyLine );
    }
    public void addFooterLine ( String footerLine ) {
        footer.addElement ( footerLine );
    }
    public void setFontSize ( int fontSize ) {
        this.fontSize = fontSize;
    }
    public void useDefaultFooter() {
        useDefaultFooter = true;
    }
    public int print ( Graphics g, PageFormat pageFormat, int pageIndex ) throws PrinterException {
        Graphics2D g2 = ( Graphics2D ) g;
        g2.setColor ( Color.black );
        Font myFont = new Font ( "Courier", Font.PLAIN, fontSize );
        g2.setFont ( myFont );
        int fontHeight = g2.getFontMetrics().getHeight();
        int fontDescent = g2.getFontMetrics().getDescent();
        double pageHeight = pageFormat.getHeight();
        double pageWidth = pageFormat.getWidth();
        int lineHeight = fontHeight + fontDescent;
        int linesPerPage = ( int ) ( ( pageHeight - 72 - 72 ) / lineHeight );
        int bodyLinesPerPage = linesPerPage - headerLines - footerLines;
        int numberOfPages = ( int ) Math.ceil ( ( double ) bodyLines / bodyLinesPerPage );
        if ( pageIndex >= numberOfPages ) {
            return Printable.NO_SUCH_PAGE;
        }
        int currentY = 72 + lineHeight;
        int currentX = 72;
        for ( int i = 0; i < header.size(); i++ ) {
            g2.drawString ( header.get ( i ), currentX, currentY );
            currentY = currentY + lineHeight;
        }
        for ( int i = bodyLinesPerPage * pageIndex; i < ( bodyLinesPerPage * ( pageIndex + 1 ) ) ; i++ ) {
            if ( i < body.size() ) {
                g2.drawString ( body.get ( i ), currentX, currentY );
            }
            currentY = currentY + lineHeight;
        }
        if ( useDefaultFooter == true ) {
            GregorianCalendar calendar = new GregorianCalendar();
            int todayYear = calendar.get ( GregorianCalendar.YEAR );
            int todayMonth = calendar.get ( GregorianCalendar.MONTH ) + 1;
            int todayDay = calendar.get ( GregorianCalendar.DAY_OF_MONTH );
            String theMonth = "0" + todayMonth;
            String theDay   = "0" + todayDay;
            g2.drawString ( "SYSTEM DATE: " + todayYear + "-" + theMonth.substring
                            ( theMonth.length() - 2 ) + "-" +
                            theDay.substring ( theDay.length() - 2 )
                            + " | PAGE " + ( pageIndex + 1 )
                            + " OF " + numberOfPages, currentX, currentY );
        } else {
            for ( int i = 0; i < footer.size(); i++ ) {
                g2.drawString ( footer.get ( i ), currentX, currentY );
                currentY = currentY + lineHeight;
            }
        }
        return Printable.PAGE_EXISTS;
    }
}
