import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import javax.xml.stream.XMLEventFactory;
import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLEventWriter;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.events.Attribute;
import javax.xml.stream.events.StartElement;
import javax.xml.stream.events.XMLEvent;
public class OptionToAnswer {
    private static XMLInputFactory inFactory = XMLInputFactory.newInstance();
    private static XMLOutputFactory outFactory = XMLOutputFactory.newInstance();
    public static void main ( String[] args ) throws XMLStreamException, IOException {
        Map<String, String> questionAnswers = new HashMap<String, String>();
        questionAnswers.put ( "foobar12", "T1Q1_2" );
        questionAnswers.put ( "foobar23", "T1Q2_3" );
        XMLEventReader xer = inFactory.createXMLEventReader ( new FileInputStream ( "options.xml" ) );
        XMLEventWriter writer = outFactory.createXMLEventWriter ( new FileOutputStream ( "answer.xml" ) );
        XMLEventFactory eventFactory = XMLEventFactory.newInstance();
        XMLEvent whitespaceBeforeOptionStartElement = null;
        while ( xer.hasNext() ) {
            XMLEvent event = xer.nextEvent();
            if ( event.getEventType() == XMLEvent.CHARACTERS && event.asCharacters().isWhiteSpace() &&
                    xer.peek().getEventType() == XMLEvent.START_ELEMENT && xer.peek().asStartElement().getName().getLocalPart().equals ( "option" ) ) {
                whitespaceBeforeOptionStartElement = event;
            } else if ( event.getEventType() == XMLEvent.START_ELEMENT && event.asStartElement().getName().getLocalPart().equals ( "option" ) ) {
                StartElement optionStartElementEvent = event.asStartElement();
                if ( canConvertOptionToAnswer ( optionStartElementEvent, questionAnswers ) ) {
                    convertOptionToAnswer ( whitespaceBeforeOptionStartElement, optionStartElementEvent, xer, writer, eventFactory );
                } else {
                    skipOption ( xer );
                }
                whitespaceBeforeOptionStartElement = null;
            } else {
                writer.add ( event );
            }
        }
        writer.close();
    }
    private static void skipOption ( XMLEventReader xer ) throws XMLStreamException {
        XMLEvent eventWithinOptionElement;
        do {
            eventWithinOptionElement = xer.nextEvent();
        } while ( ! ( eventWithinOptionElement.getEventType() == XMLEvent.END_ELEMENT && eventWithinOptionElement.asEndElement().getName().getLocalPart().equals ( "option" ) ) );
    }
    private static void convertOptionToAnswer (
        XMLEvent whitespaceBeforeOptionStartElement, StartElement optionStartElementEvent,
        XMLEventReader xer,
        XMLEventWriter writer,
        XMLEventFactory eventFactory ) throws XMLStreamException {
        if ( whitespaceBeforeOptionStartElement != null ) {
            writer.add ( whitespaceBeforeOptionStartElement );
        }
        writer.add ( eventFactory.createStartElement ( "", null, "answer" ) );
        Iterator ite = optionStartElementEvent.getAttributes();
        while ( ite.hasNext() ) {
            Attribute attribute = ( Attribute ) ite.next();
            writer.add ( attribute );
        }
        XMLEvent eventWithinOptionElement = xer.nextEvent();
        while ( ! ( eventWithinOptionElement.getEventType() == XMLEvent.END_ELEMENT && eventWithinOptionElement.asEndElement().getName().getLocalPart().equals ( "option" ) ) ) {
            writer.add ( eventWithinOptionElement );
            eventWithinOptionElement = xer.nextEvent();
        }
        writer.add ( eventFactory.createEndElement ( "", null, "answer" ) );
    }
    private static boolean canConvertOptionToAnswer ( StartElement optionStartElementEvent, Map<String, String> questionAnswers ) {
        Iterator ite = optionStartElementEvent.getAttributes();
        while ( ite.hasNext() ) {
            Attribute attr = ( Attribute ) ite.next();
            if ( attr.getName().getLocalPart().equals ( "code" ) && questionAnswers.containsValue ( attr.getValue() ) ) {
                return true;
            }
        }
        return false;
    }
}
