public class Test {
    public static final Message Stop = new Message ( "Stop!" );
    static class Message {
        final String msg;
        public Message ( String msg ) {
            this.msg = msg;
        }
        public String toString() {
            return msg;
        }
    }
    class Worker implements Runnable {
        private volatile boolean stop = false;
        private final BlockingQueue<Message> workQueue;
        public Worker ( BlockingQueue<Message> workQueue ) {
            this.workQueue = workQueue;
        }
        @Override
        public void run() {
            while ( !stop ) {
                try {
                    Message msg = workQueue.poll ( 10, TimeUnit.SECONDS );
                    System.out.println ( "Worker " + Thread.currentThread().getName() + " got message " + msg );
                    if ( msg == Stop ) {
                        stop = true;
                    }
                } catch ( InterruptedException ex ) {
                    stop = true;
                }
            }
        }
    }
    Map<Integer, BlockingQueue<Message>> queues = new HashMap<>();
    public void test() throws InterruptedException {
        List<Thread> threads = new ArrayList<>();
        for ( int i = 0; i < 20; i++ ) {
            BlockingQueue<Message> queue = new ArrayBlockingQueue ( 10 );
            Thread thread = new Thread ( new Worker ( queue ), "Worker-" + i );
            threads.add ( thread );
            queues.put ( i, queue );
            thread.start();
        }
        queues.get ( 5 ).put ( new Message ( "Hello" ) );
        for ( BlockingQueue<Message> q : queues.values() ) {
            q.put ( Stop );
        }
        for ( Thread t : threads ) {
            t.join();
        }
    }
    public static void main ( String args[] ) {
        try {
            new Test().test();
        } catch ( Throwable t ) {
            t.printStackTrace ( System.err );
        }
    }
}
