public class ByJavaScript extends By implements Serializable {
    private final String script;
    public ByJavaScript ( String script ) {
        checkNotNull ( script, "Cannot find elements with a null JavaScript expression." );
        this.script = script;
    }
    @Override
    public List<WebElement> findElements ( SearchContext context ) {
        JavascriptExecutor js = getJavascriptExecutorFromSearchContext ( context );
        Object response = js.executeScript ( script );
        List<WebElement> elements = getElementListFromJsResponse ( response );
        if ( context instanceof WebElement ) {
            filterOutElementsWithoutCommonAncestor ( elements, ( WebElement ) context );
        }
        return elements;
    }
    private static JavascriptExecutor getJavascriptExecutorFromSearchContext ( SearchContext context ) {
        if ( context instanceof JavascriptExecutor ) {
            return ( JavascriptExecutor ) context;
        }
        if ( context instanceof WrapsDriver ) {
            WebDriver driver = ( ( WrapsDriver ) context ).getWrappedDriver();
            checkState ( driver instanceof JavascriptExecutor, "This WebDriver doesn't support JavaScript." );
            return ( JavascriptExecutor ) driver;
        }
        throw new IllegalStateException ( "We can't invoke JavaScript from this context." );
    }
    @SuppressWarnings ( "unchecked" )
    private static List<WebElement> getElementListFromJsResponse ( Object response ) {
        if ( response == null ) {
            return Lists.newArrayList();
        }
        if ( response instanceof WebElement ) {
            return Lists.newArrayList ( ( WebElement ) response );
        }
        if ( response instanceof List ) {
            checkArgument (
                Iterables.all ( ( List<?> ) response, Predicates.instanceOf ( WebElement.class ) ),
                "The JavaScript query returned something that isn't a WebElement." );
            return ( List<WebElement> ) response;
        }
        throw new IllegalArgumentException ( "The JavaScript query returned something that isn't a WebElement." );
    }
    private static void filterOutElementsWithoutCommonAncestor ( List<WebElement> elements, WebElement ancestor ) {
        for ( Iterator<WebElement> iter = elements.iterator(); iter.hasNext(); ) {
            WebElement elem = iter.next();
            while ( !elem.equals ( ancestor ) && !elem.getTagName().equals ( "html" ) ) {
                elem = elem.findElement ( By.xpath ( "./.." ) );
            }
            if ( !elem.equals ( ancestor ) ) {
                iter.remove();
            }
        }
    }
    @Override
    public String toString() {
        return "By.javaScript: \"" + script + "\"";
    }
}
