import java.lang.invoke.MethodHandle;
import java.lang.invoke.MethodHandles;
import java.lang.reflect.Method;
public class TestMethodPerf {
    private static final int ITERATIONS = 50_000_000;
    private static final int WARM_UP = 10;
    public static void main ( String... args ) throws Throwable {
        final int[] dummy = new int[3];
        Method reflected = TestMethodPerf.class
                           .getDeclaredMethod ( "myMethod", int.class, int.class );
        MethodHandle mh = MethodHandles.lookup().unreflect ( reflected );
        for ( int i = 0; i < WARM_UP; i++ ) {
            dummy[0] += testDirect ( dummy[0] );
            dummy[1] += testMH ( dummy[1], mh );
            dummy[2] += testReflection ( dummy[2], reflected );
        }
        long t0 = System.nanoTime();
        dummy[0] += testDirect ( dummy[0] );
        long t1 = System.nanoTime();
        dummy[1] += testMH ( dummy[1], mh );
        long t2 = System.nanoTime();
        dummy[2] += testReflection ( dummy[2], reflected );
        long t3 = System.nanoTime();
        System.out.printf ( "direct: %.2fs, mh: %.2fs, reflection: %.2fs%n",
                            ( t1 - t0 ) * 1e-9, ( t2 - t1 ) * 1e-9, ( t3 - t2 ) * 1e-9 );
        if ( dummy[0] != dummy[1] || dummy[0] != dummy[2] ) {
            throw new AssertionError();
        }
    }
    private static int testMH ( int v, MethodHandle mh ) throws Throwable {
        for ( int i = 0; i < ITERATIONS; i++ ) {
            v += ( int ) mh.invokeExact ( 1000, v );
        }
        return v;
    }
    private static int testReflection ( int v, Method mh ) throws Throwable {
        for ( int i = 0; i < ITERATIONS; i++ ) {
            v += ( int ) mh.invoke ( null, 1000, v );
        }
        return v;
    }
    private static int testDirect ( int v ) {
        for ( int i = 0; i < ITERATIONS; i++ ) {
            v += myMethod ( 1000, v );
        }
        return v;
    }
    public static int myMethod ( int a, int b ) {
        return a < b ? a : b;
    }
}
