public interface Supplier<T> {
    public int size();
    public T get ( int index );
}
public interface PairSupplier<Q, E> extends Supplier<Pair<Q, E>> {
}
public class IterableSupplier<T> implements Iterable<T> {
    final Supplier<T> supplier;
    final AtomicInteger i = new AtomicInteger();
    public IterableSupplier ( Supplier<T> supplier ) {
        this.supplier = supplier;
    }
    @Override
    public Iterator<T> iterator() {
        return new ThreadSafeIterator();
    }
    private class ThreadSafeIterator implements Iterator<T> {
        @Override
        public boolean hasNext() {
            return true;
        }
        private int pickNext() {
            int pick = i.incrementAndGet();
            int actual = pick % supplier.size();
            if ( pick != actual ) {
                i.compareAndSet ( pick, actual );
            }
            return actual;
        }
        @Override
        public T next() {
            return supplier.get ( pickNext() );
        }
        @Override
        public void remove() {
            throw new UnsupportedOperationException ( "Remove not supported." );
        }
    }
}
