import java.awt.Canvas;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.GraphicsConfiguration;
import java.awt.GraphicsEnvironment;
import java.awt.Toolkit;
import java.awt.Transparency;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.image.BufferStrategy;
import java.awt.image.BufferedImage;
import javax.swing.JFrame;
import javax.swing.WindowConstants;
public class Test extends Thread {
    private boolean isRunning = true;
    private Canvas canvas;
    private BufferStrategy strategy;
    private BufferedImage background;
    private Graphics2D backgroundGraphics;
    private Graphics2D graphics;
    private JFrame frame;
    private int width = 320;
    private int height = 240;
    private int scale = 1;
    private GraphicsConfiguration config =
        GraphicsEnvironment.getLocalGraphicsEnvironment()
        .getDefaultScreenDevice()
        .getDefaultConfiguration();
    public final BufferedImage create ( final int width, final int height,
                                        final boolean alpha ) {
        return config.createCompatibleImage ( width, height, alpha
                                              ? Transparency.TRANSLUCENT : Transparency.OPAQUE );
    }
    public Test() {
        frame = new JFrame();
        frame.addWindowListener ( new FrameClose() );
        frame.setDefaultCloseOperation ( WindowConstants.DO_NOTHING_ON_CLOSE );
        frame.setSize ( width * scale, height * scale );
        frame.setVisible ( true );
        canvas = new Canvas ( config );
        canvas.setSize ( width * scale, height * scale );
        frame.add ( canvas, 0 );
        background = create ( width, height, false );
        canvas.createBufferStrategy ( 2 );
        do {
            strategy = canvas.getBufferStrategy();
        } while ( strategy == null );
        start();
    }
    private class FrameClose extends WindowAdapter {
        @Override
        public void windowClosing ( final WindowEvent e ) {
            isRunning = false;
        }
    }
    private Graphics2D getBuffer() {
        if ( graphics == null ) {
            try {
                graphics = ( Graphics2D ) strategy.getDrawGraphics();
            } catch ( IllegalStateException e ) {
                return null;
            }
        }
        return graphics;
    }
    private boolean updateScreen() {
        graphics.dispose();
        graphics = null;
        try {
            strategy.show();
            Toolkit.getDefaultToolkit().sync();
            return ( !strategy.contentsLost() );
        } catch ( NullPointerException e ) {
            return true;
        } catch ( IllegalStateException e ) {
            return true;
        }
    }
    public void run() {
        backgroundGraphics = ( Graphics2D ) background.getGraphics();
        long fpsWait = ( long ) ( 1.0 / 30 * 1000 );
        main: while ( isRunning ) {
            long renderStart = System.nanoTime();
            updateGame();
            do {
                Graphics2D bg = getBuffer();
                if ( !isRunning ) {
                    break main;
                }
                renderGame ( backgroundGraphics );
                if ( scale != 1 ) {
                    bg.drawImage ( background, 0, 0, width * scale, height
                                   * scale, 0, 0, width, height, null );
                } else {
                    bg.drawImage ( background, 0, 0, null );
                }
                bg.dispose();
            } while ( !updateScreen() );
            long renderTime = ( System.nanoTime() - renderStart ) / 1000000;
            try {
                Thread.sleep ( Math.max ( 0, fpsWait - renderTime ) );
            } catch ( InterruptedException e ) {
                Thread.interrupted();
                break;
            }
            renderTime = ( System.nanoTime() - renderStart ) / 1000000;
        }
        frame.dispose();
    }
    public void updateGame() {
    }
    public void renderGame ( Graphics2D g ) {
        g.setColor ( Color.BLACK );
        g.fillRect ( 0, 0, width, height );
    }
    public static void main ( final String args[] ) {
        new Test();
    }
}
