@MappedSuperClass
public class BaseModel {
    @Id @GeneratedValue ( strategy = GenerationType.AUTO )
    protected Long id;
    public Long getId() {
        return id;
    }
    public void setId ( final Long id ) {
        this.id = id;
    }
}
@Entity
@AttributeOverrides ( {
    @AttributeOverride ( name = "id", column = @Column ( name = "word_id" ) )
} )
@Cache ( usage = CacheConcurrencyStrategy.READ_WRITE )
@Table ( name = "words" )
public class Word extends BaseModel {
    @Column ( name = "word_text" )
    private String text;
    @Column ( name = "word_length" )
    private Integer length;
    @ManyToOne
    @JoinColumn ( name = "word_type_id", referencedColumnName = "word_type_id" )
    private WordType type;
    @Column ( name = "word_definition" )
    private String definition;
    @ManyToMany()
    private List<Word> synonyms;
    @ManyToMany()
    private List<Word> antonyms;
}
@MappedSuperClass
public class BaseLookup extends BaseModel {
    private String label;
    private String description;
    private String tag;
}
@Entity
@AttributeOverrides ( {
    @AttributeOverride ( name = "id", column = @Column ( name = "word_type_id" ) ),
    @AttributeOverride ( name = "label", column = @Column ( name = "word_type_label" ) ),
    @AttributeOverride ( name = "description", column = @Column ( name = "word_type_description" ) ),
    @AttributeOverride ( name = "tag", column = @Column ( name = "word_type_tag" ) )
} )
public class WordType extends BaseLookup {
    public WordType ( String label, String description, String tag ) {
        super ( label, description, tag );
    }
}
