package at.rovo.test.httpclient;
import org.eclipse.jetty.security.ConstraintMapping;
import org.eclipse.jetty.security.ConstraintSecurityHandler;
import org.eclipse.jetty.security.HashLoginService;
import org.eclipse.jetty.security.SecurityHandler;
import org.eclipse.jetty.security.authentication.BasicAuthenticator;
import org.eclipse.jetty.server.ConnectionFactory;
import org.eclipse.jetty.server.Connector;
import org.eclipse.jetty.server.HttpConfiguration;
import org.eclipse.jetty.server.HttpConnectionFactory;
import org.eclipse.jetty.server.Server;
import org.eclipse.jetty.server.ServerConnector;
import org.eclipse.jetty.server.SslConnectionFactory;
import org.eclipse.jetty.servlet.ServletContextHandler;
import org.eclipse.jetty.servlet.ServletHolder;
import org.eclipse.jetty.util.security.Constraint;
public class JettyServer {
    private String REALM;
    public static void main ( String[] args ) throws Exception {
        new JettyServer();
    }
    public JettyServer() throws Exception {
        Server server = new Server ( 8080 );
        server.addConnector ( this.getSslChannelConnector ( server ) );
        server.setStopAtShutdown ( true );
        ServletContextHandler context = new ServletContextHandler ( ServletContextHandler.SESSIONS );
        context.setContextPath ( "/" );
        context.setClassLoader ( Thread.currentThread().getContextClassLoader() );
        context.setSecurityHandler ( this.getSecurityHandler() );
        server.setHandler ( context );
        context.addServlet ( new ServletHolder ( new ProcessingServlet() ), "/process" );
        server.start();
        server.join();
    }
    private Connector getSslChannelConnector ( Server server ) {
        try {
            String keyStore = this.getClass().getResource ( "/serverKey.jks" ).toURI().getPath();
            SslConnectionFactory sslConnFactory = new SslConnectionFactory();
            sslConnFactory.getSslContextFactory().setKeyStorePath ( keyStore );
            sslConnFactory.getSslContextFactory().setKeyStorePassword ( "keystorePW" );
            sslConnFactory.getSslContextFactory().setKeyManagerPassword ( "jettyPW" );
            HttpConfiguration config = new HttpConfiguration();
            ConnectionFactory connFactory = new HttpConnectionFactory ( config );
            ServerConnector connector = new ServerConnector ( server, sslConnFactory, connFactory );
            connector.setPort ( 8443 );
            connector.setHost ( "localhost" );
            connector.setIdleTimeout ( 30000 );
            return connector;
        } catch ( Exception e ) {
            e.printStackTrace();
        }
        return null;
    }
    private SecurityHandler getSecurityHandler() throws Exception {
        Constraint constraint = new Constraint ( Constraint.__BASIC_AUTH, "user" );
        constraint.setAuthenticate ( true );
        constraint.setRoles ( new String[] {"user", "admin"} );
        ConstraintMapping cm = new ConstraintMapping();
        cm.setConstraint ( constraint );
        cm.setPathSpec ( "/*" );
        ConstraintSecurityHandler csh = new ConstraintSecurityHandler();
        csh.setAuthenticator ( new BasicAuthenticator() );
        csh.setRealmName ( REALM );
        csh.addConstraintMapping ( cm );
        csh.setLoginService ( getHashLoginService() );
        return csh;
    }
    private HashLoginService getHashLoginService() throws Exception {
        HashLoginService hls = new HashLoginService();
        hls.setName ( REALM );
        hls.setConfig ( this.getClass().getResource ( "/realm.properties" ).toURI().toString() );
        hls.setRefreshInterval ( 0 );
        return hls;
    }
}
