import javafx.application.Application;
import javafx.beans.property.*;
import javafx.event.EventHandler;
import javafx.scene.*;
import javafx.scene.input.MouseEvent;
import javafx.scene.paint.Color;
import javafx.scene.shape.*;
import javafx.stage.Stage;
public class LineManipulator extends Application {
    public static void main ( String[] args ) throws Exception {
        launch ( args );
    }
    @Override public void start ( final Stage stage ) throws Exception {
        DoubleProperty startX = new SimpleDoubleProperty ( 100 );
        DoubleProperty startY = new SimpleDoubleProperty ( 100 );
        DoubleProperty endX   = new SimpleDoubleProperty ( 300 );
        DoubleProperty endY   = new SimpleDoubleProperty ( 200 );
        Anchor start    = new Anchor ( Color.PALEGREEN, startX, startY );
        Anchor end      = new Anchor ( Color.TOMATO,    endX,   endY );
        Line line = new BoundLine ( startX, startY, endX, endY );
        stage.setTitle ( "Line Manipulation Sample" );
        stage.setScene ( new Scene ( new Group ( line, start, end ), 400, 400, Color.ALICEBLUE ) );
        stage.show();
    }
    class BoundLine extends Line {
        BoundLine ( DoubleProperty startX, DoubleProperty startY, DoubleProperty endX, DoubleProperty endY ) {
            startXProperty().bind ( startX );
            startYProperty().bind ( startY );
            endXProperty().bind ( endX );
            endYProperty().bind ( endY );
            setStrokeWidth ( 2 );
            setStroke ( Color.GRAY.deriveColor ( 0, 1, 1, 0.5 ) );
            setStrokeLineCap ( StrokeLineCap.BUTT );
            getStrokeDashArray().setAll ( 10.0, 5.0 );
            setMouseTransparent ( true );
        }
    }
    class Anchor extends Circle {
        Anchor ( Color color, DoubleProperty x, DoubleProperty y ) {
            super ( x.get(), y.get(), 10 );
            setFill ( color.deriveColor ( 1, 1, 1, 0.5 ) );
            setStroke ( color );
            setStrokeWidth ( 2 );
            setStrokeType ( StrokeType.OUTSIDE );
            x.bind ( centerXProperty() );
            y.bind ( centerYProperty() );
            enableDrag();
        }
        private void enableDrag() {
            final Delta dragDelta = new Delta();
            setOnMousePressed ( new EventHandler<MouseEvent>() {
                @Override public void handle ( MouseEvent mouseEvent ) {
                    dragDelta.x = getCenterX() - mouseEvent.getX();
                    dragDelta.y = getCenterY() - mouseEvent.getY();
                    getScene().setCursor ( Cursor.MOVE );
                }
            } );
            setOnMouseReleased ( new EventHandler<MouseEvent>() {
                @Override public void handle ( MouseEvent mouseEvent ) {
                    getScene().setCursor ( Cursor.HAND );
                }
            } );
            setOnMouseDragged ( new EventHandler<MouseEvent>() {
                @Override public void handle ( MouseEvent mouseEvent ) {
                    double newX = mouseEvent.getX() + dragDelta.x;
                    if ( newX > 0 && newX < getScene().getWidth() ) {
                        setCenterX ( newX );
                    }
                    double newY = mouseEvent.getY() + dragDelta.y;
                    if ( newY > 0 && newY < getScene().getHeight() ) {
                        setCenterY ( newY );
                    }
                }
            } );
            setOnMouseEntered ( new EventHandler<MouseEvent>() {
                @Override public void handle ( MouseEvent mouseEvent ) {
                    if ( !mouseEvent.isPrimaryButtonDown() ) {
                        getScene().setCursor ( Cursor.HAND );
                    }
                }
            } );
            setOnMouseExited ( new EventHandler<MouseEvent>() {
                @Override public void handle ( MouseEvent mouseEvent ) {
                    if ( !mouseEvent.isPrimaryButtonDown() ) {
                        getScene().setCursor ( Cursor.DEFAULT );
                    }
                }
            } );
        }
        private class Delta {
            double x, y;
        }
    }
}
