import javafx.application.Application;
import javafx.event.EventHandler;
import javafx.scene.*;
import javafx.scene.input.MouseEvent;
import javafx.scene.paint.*;
import javafx.scene.shape.*;
import javafx.scene.transform.Rotate;
import javafx.stage.Stage;
public class RectangleViewer extends Application {
    double anchorX, anchorY, anchorAngle;
    private PerspectiveCamera addCamera ( Scene scene ) {
        PerspectiveCamera perspectiveCamera = new PerspectiveCamera ( false );
        scene.setCamera ( perspectiveCamera );
        return perspectiveCamera;
    }
    public static void main ( String[] args ) {
        launch ( args );
    }
    @Override
    public void start ( Stage primaryStage ) {
        final MeshView rect = new MeshView (
            new Shape3DRectangle ( 200, 200 )
        );
        rect.setMaterial ( new PhongMaterial ( Color.DARKGREEN ) );
        rect.setRotationAxis ( Rotate.Y_AXIS );
        rect.setTranslateX ( 250 );
        rect.setTranslateY ( 250 );
        rect.setCullFace ( CullFace.NONE );
        final Group root = new Group ( rect );
        final Scene scene = new Scene ( root, 500, 500, true );
        scene.setOnMousePressed ( new EventHandler<MouseEvent>() {
            @Override public void handle ( MouseEvent event ) {
                anchorX = event.getSceneX();
                anchorY = event.getSceneY();
                anchorAngle = rect.getRotate();
            }
        } );
        scene.setOnMouseDragged ( new EventHandler<MouseEvent>() {
            @Override public void handle ( MouseEvent event ) {
                rect.setRotate ( anchorAngle + anchorX - event.getSceneX() );
            }
        } );
        addCamera ( scene );
        primaryStage.setScene ( scene );
        primaryStage.show();
    }
    public class Shape3DRectangle extends TriangleMesh {
        public Shape3DRectangle ( float Width, float Height ) {
            float[] points = {
                -Width / 2,  Height / 2, 0,
                -Width / 2, -Height / 2, 0,
                Width / 2,  Height / 2, 0,
                Width / 2, -Height / 2, 0
            };
            float[] texCoords = {
                1, 1,
                1, 0,
                0, 1,
                0, 0
            };
            int[] faces = {
                2, 3, 0, 2, 1, 0,
                2, 3, 1, 0, 3, 1
            };
            this.getPoints().setAll ( points );
            this.getTexCoords().setAll ( texCoords );
            this.getFaces().setAll ( faces );
        }
    }
}
