import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Set;
public class DynamicClassLoader extends ClassLoader {
    private Set<String> classes;
    public DynamicClassLoader ( Set<String> changedClasses ) {
        this.classes = changedClasses;
    }
    public Class<?> dynamicallyLoadClass ( String name )
    throws ClassNotFoundException {
        classes.add ( name );
        return loadClass ( name );
    }
    @Override
    protected Class<?> findClass ( String s ) throws ClassNotFoundException {
        try {
            byte[] bytes;
            bytes = loadClassData ( s );
            return defineClass ( s, bytes, 0, bytes.length );
        } catch ( IOException e ) {
            e.printStackTrace();
            throw new ClassNotFoundException();
        }
    }
    @Override
    public Class<?> loadClass ( String name ) throws ClassNotFoundException {
        return loadClass ( name, true );
    }
    @Override
    protected Class<?> loadClass ( String name, boolean resolve )
    throws ClassNotFoundException {
        Class<?> clazz = findLoadedClass ( name );
        if ( clazz != null ) {
            return clazz;
        }
        if ( classes.contains ( name ) ) {
            clazz = findClass ( name );
            resolveClass ( clazz );
            return clazz;
        }
        return super.loadClass ( name, resolve );
    }
    protected byte[] loadClassData ( String className ) throws IOException {
        File f = new File ( "bin/"
                            + className.replaceAll ( "\\.", "/" ) + ".class" );
        int size = ( int ) f.length();
        byte buff[] = new byte[size];
        FileInputStream fis = new FileInputStream ( f );
        DataInputStream dis = new DataInputStream ( fis );
        dis.readFully ( buff );
        dis.close();
        return buff;
    }
}
