static boolean[][] board = new boolean[SQUARES][SQUARES];
static int[][] elimination = new int[SQUARES][SQUARES];
static boolean[][] position = new boolean[SQUARES][SQUARES];
static int[] row = new int[8];
static int[] column = new int[8];
public static void main ( String[] args ) {
    Arrays.fill ( row, -1 );
    Arrays.fill ( column, -1 );
    elimSquares();
    fillElim();
    short counter = 0;
    while ( counter < 8 ) {
        placeQueen ( -1, -1 );
        fillElim();
        while ( isFull() && counter < 7 ) {
            backtrack ( counter );
        }
        counter++;
    }
    System.out.println ( row[0] + "/" + column[0] );
    printBoard();
}
public static void printBoard() {
    for ( int i = 0; i < board.length; i++ ) {
        for ( int j = 0; j < board.length; j++ ) {
            if ( board[i][j] && position[i][j] ) {
                System.out.print ( "o " );
            } else if ( board[i][j] ) {
                System.out.print ( "x " );
            } else {
                System.out.print ( "% " );
            }
        }
        System.out.println();
    }
}
public static void fillElim() {
    for ( int i = 0; i < elimination.length; i++ ) {
        for ( int j = 0; j < elimination[i].length; j++ ) {
            elimination[i][j] = openSquares ( i, j );
        }
    }
}
public static int openSquares ( int row, int column ) {
    if ( board[row][column] ) {
        return 0;
    }
    int total = 1 + openHorizontal ( row ) + openVertical ( column ) + openUpSlope ( row, column ) + openDownSlope ( row, column );
    return total;
}
public static int openHorizontal ( int row ) {
    int total = 0;
    for ( boolean b : board[row] ) {
        if ( !b ) {
            total++;
        }
    }
    return total - 1;
}
public static int openVertical ( int column ) {
    int total = 0;
    for ( boolean[] b : board ) {
        if ( !b[column] ) {
            total++;
        }
    }
    return total - 1;
}
public static int openDownSlope ( int x, int y ) {
    int total = 0;
    for ( int i = 0; i < board.length; i++ ) {
        if ( x + i >= 0 && x + i < board.length && y + i >= 0 && y + i < board.length ) {
            if ( !board[x + i][y + i] ) {
                total++;
            }
        }
        if ( x - i >= 0 && x - i < board.length && y - i >= 0 && y - i < board.length ) {
            if ( !board[x - i][y - i] ) {
                total++;
            }
        }
    }
    return total - 2;
}
public static int openUpSlope ( int x, int y ) {
    int total = 0;
    for ( int i = 0; i < board.length; i++ ) {
        if ( x + i >= 0 && x + i < board.length && y - i >= 0 && y - i < board.length ) {
            if ( !board[x + i][y - i] ) {
                total++;
            }
        }
        if ( x - i >= 0 && x - i < board.length && y + i >= 0 && y + i < board.length ) {
            if ( !board[x - i][y + i] ) {
                total++;
            }
        }
    }
    return total - 2;
}
public static boolean isFull() {
    for ( boolean b[] : board ) {
        for ( boolean bb : b ) {
            if ( !bb ) {
                return false;
            }
        }
    }
    return true;
}
public static void placeQueen ( int lastRow, int lastCol ) {
    int[] bestSquare = bestMove ( lastRow, lastCol );
    position[bestSquare[0]][bestSquare[1]] = true;
    elimSquares();
    for ( int i = 0; i < row.length; i++ ) {
        if ( row[i] == -1 ) {
            row[i] = bestSquare[0];
            column[i] = bestSquare[1];
            break;
        }
    }
}
public static int[] bestMove ( int lastRow, int lastCol ) {
    int[] move = { -1, -1};
    if ( lastRow != -1 ) {
        move = dupElimNums ( lastRow, lastCol );
    }
    if ( move[0] == -1 ) {
        move = checkBoard ( lastRow, lastCol );
    }
    return move;
}
public static int[] dupElimNums ( int lastRow, int lastCol ) {
    int limit;
    if ( lastRow == -1 ) {
        limit = -1;
    } else {
        limit = elimination[lastRow][lastCol];
    }
    int[] move = { -1, -1};
    int[] nextSquare = nextSquare ( lastRow, lastCol );
    for ( int i = nextSquare[0]; i < board.length; i++ ) {
        for ( int j = 0; j < board[i].length; j++ ) {
            if ( i >= lastRow && j == 0 ) {
                j = nextSquare[1];
            }
            if ( !board[i][j] && elimination[i][j] == limit ) {
                move[0] = i;
                move[1] = j;
                return move;
            }
        }
    }
    return move;
}
public static int[] nextSquare ( int row, int column ) {
    if ( column < 7 ) {
        column++;
    } else {
        column = 0;
        row++;
    }
    int[] square = {row, column};
    return square;
}
public static int[] checkBoard ( int lastRow, int lastCol ) {
    int low = 100;
    int[] move = { -1, -1};
    int limit;
    if ( lastRow == -1 ) {
        limit = -1;
    } else {
        limit = elimination[lastRow][lastCol];
    }
    for ( int i = 0; i < board.length; i++ ) {
        for ( int j = 0; j < board[i].length; j++ ) {
            if ( !board[i][j] && elimination[i][j] > limit && elimination[i][j] < low ) {
                low = elimination[i][j];
            }
        }
    }
    for ( int i = 0; i < board.length; i++ ) {
        for ( int j = 0; j < board[i].length; j++ ) {
            if ( !board[i][j] && elimination[i][j] == low ) {
                move[0] = i;
                move[1] = j;
                return move;
            }
        }
    }
    return move;
}
public static void elimSquares() {
    resetBoard();
    for ( int r = 0; r < position.length; r++ ) {
        for ( int c = 0; c < position[r].length; c++ ) {
            if ( position[r][c] ) {
                elimHorizontal ( r );
                elimVertical ( c );
                elimUpSlope ( r, c );
                elimDownSlope ( r, c );
            }
        }
    }
    fillElim();
}
public static void elimHorizontal ( int row ) {
    for ( int i = 0; i < board[row].length; i++ ) {
        board[row][i] = true;
    }
}
public static void elimVertical ( int column ) {
    for ( boolean[] b : board ) {
        b[column] = true;
    }
}
public static void elimDownSlope ( int x, int y ) {
    for ( int i = 0; i < board.length; i++ ) {
        if ( x + i >= 0 && x + i < board.length && y + i >= 0 && y + i < board.length ) {
            board[x + i][y + i] = true;
        }
        if ( x - i >= 0 && x - i < board.length && y - i >= 0 && y - i < board.length ) {
            board[x - i][y - i] = true;
        }
    }
}
public static void elimUpSlope ( int x, int y ) {
    for ( int i = 0; i < board.length; i++ ) {
        if ( x + i >= 0 && x + i < board.length && y - i >= 0 && y - i < board.length ) {
            board[x + i][y - i] = true;
        }
        if ( x - i >= 0 && x - i < board.length && y + i >= 0 && y + i < board.length ) {
            board[x - i][y + i] = true;
        }
    }
}
public static void backtrack ( int lastMove ) {
    int lastRow = row[lastMove];
    int lastCol = column[lastMove];
    resetPositions();
    goBack ( lastMove );
    for ( int i = 0; i < row.length; i++ ) {
        if ( row[i] == -1 ) {
            break;
        }
        position[row[i]][column[i]] = true;
        elimSquares();
    }
    while ( !openSpaces ( lastRow, lastCol ) ) {
        backtrack ( lastMove - 1 );
        if ( openSpaces ( lastRow, lastCol ) ) {
            lastRow = -1;
            lastCol = -1;
            break;
        }
    }
    placeQueen ( lastRow, lastCol );
}
public static void resetBoard() {
    for ( boolean[] b : board )
        for ( int i = 0; i < b.length; i++ ) {
            b[i] = false;
        }
}
public static void resetPositions() {
    for ( boolean[] b : position )
        for ( int i = 0; i < b.length; i++ ) {
            b[i] = false;
        }
}
public static void goBack ( int lastMove ) {
    row[lastMove] = -1;
    column[lastMove] = -1;
}
public static boolean openSpaces ( int lastRow, int lastCol ) {
    int limit = elimination[lastRow][lastCol];
    for ( int i = 0; i < board.length; i++ ) {
        for ( int j = 0; j < board[i].length; j++ ) {
            if ( !board[i][j] && elimination[i][j] > limit ) {
                return true;
            }
        }
    }
    return false;
}
