package geojson.sof20181050;
import java.io.IOException;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.concurrent.Phaser;
import java.util.concurrent.atomic.AtomicLong;
import com.allanbank.mongodb.Callback;
import com.allanbank.mongodb.Durability;
import com.allanbank.mongodb.MongoClient;
import com.allanbank.mongodb.MongoCollection;
import com.allanbank.mongodb.MongoFactory;
import com.allanbank.mongodb.StreamCallback;
import com.allanbank.mongodb.bson.Document;
import com.allanbank.mongodb.bson.Element;
import com.allanbank.mongodb.bson.NumericElement;
import com.allanbank.mongodb.bson.builder.BuilderFactory;
import com.allanbank.mongodb.bson.builder.DocumentBuilder;
import com.allanbank.mongodb.builder.Find;
import com.allanbank.mongodb.builder.GeoJson;
import com.mongodb.BasicDBObject;
import com.mongodb.DBObject;
import com.mongodb.MongoClientURI;
import com.mongodb.WriteConcern;
public class ConvertToGeoJSON {
    protected final static MongoClient client;
    protected final static MongoCollection theCollection;
    protected static final String URI;
    static {
        URI = "mongodb://localhost:27017/";
        client = MongoFactory.createClient ( URI );
        theCollection = client.getDatabase ( "db" ).getCollection ( "collection" );
    }
    public static void main ( final String[] args ) throws InterruptedException,
        IOException {
        try {
            final boolean doLegacy = false;
            final boolean doSynchronous = false;
            if ( doLegacy ) {
                doLegacy();
            } else if ( doSynchronous ) {
                doSynchronously();
            } else {
                doAsynchronously();
            }
        } finally {
            client.close();
        }
    }
    protected static void doAsynchronously() throws InterruptedException {
        final Phaser finished = new Phaser ( 1 );
        final AtomicLong updates = new AtomicLong ( 0 );
        final StreamCallback<Document> streamCallback = new StreamCallback<Document>() {
            final Phaser streamFinished = new Phaser ( finished, 1 );
            @Override
            public void callback ( final Document doc ) {
                final Element id = doc.get ( "_id" );
                final NumericElement lat = doc.get ( NumericElement.class,
                                                     "latitude_deg" );
                final NumericElement lon = doc.get ( NumericElement.class,
                                                     "longitude_deg" );
                final DocumentBuilder query = BuilderFactory.start();
                query.add ( id );
                final DocumentBuilder update = BuilderFactory.start();
                update.push ( "$set" ).add (
                    "loc",
                    GeoJson.point ( GeoJson.p ( lon.getDoubleValue(),
                                                lat.getDoubleValue() ) ) );
                final Callback<Long> updateCallback = new Callback<Long>() {
                    final Phaser updateFinished = new Phaser ( streamFinished, 1 );
                    @Override
                    public void callback ( final Long result ) {
                        updates.addAndGet ( result );
                        updateFinished.arriveAndDeregister();
                    }
                    @Override
                    public void exception ( final Throwable thrown ) {
                        System.err.printf ( "Update of {0} failed.\n", id );
                        thrown.printStackTrace();
                        callback ( null );
                    }
                };
                theCollection.updateAsync ( updateCallback, query, update,
                                            Durability.ACK );
            }
            @Override
            public synchronized void done() {
                streamFinished.arriveAndDeregister();
            }
            @Override
            public void exception ( final Throwable thrown ) {
                thrown.printStackTrace();
                done();
            }
        };
        theCollection.streamingFind ( streamCallback, Find.ALL );
        finished.arriveAndAwaitAdvance();
        System.out.printf ( "Updated %d documents asynchronously.%n",
                            updates.get() );
    }
    protected static void doLegacy() throws UnknownHostException {
        final com.mongodb.MongoClient legacyClient = new com.mongodb.MongoClient (
            new MongoClientURI ( URI ) );
        final com.mongodb.DBCollection legacyCollection = legacyClient.getDB (
                    theCollection.getDatabaseName() ).getCollection (
                    theCollection.getName() );
        try {
            int count = 0;
            for ( final DBObject doc : legacyCollection.find() ) {
                final Object id = doc.get ( "_id" );
                final Number lat = ( Number ) doc.get ( "latitude_deg" );
                final Number lon = ( Number ) doc.get ( "longitude_deg" );
                final BasicDBObject query = new BasicDBObject();
                query.append ( "_id", id );
                final ArrayList<Double> coordinates = new ArrayList<>();
                coordinates.add ( lon.doubleValue() );
                coordinates.add ( lat.doubleValue() );
                final BasicDBObject geojson = new BasicDBObject ( "type", "Point" );
                geojson.append ( "coordinates", coordinates );
                final BasicDBObject set = new BasicDBObject ( "loc", geojson );
                final BasicDBObject update = new BasicDBObject ( "$set", set );
                legacyCollection.update ( query, update,  false,
                                          false, WriteConcern.ACKNOWLEDGED );
                count += 1;
            }
            System.out.printf ( "Updated %d documents via the legacy driver.%n",
                                count );
        } finally {
            legacyClient.close();
        }
    }
    protected static void doSynchronously() {
        int count = 0;
        for ( final Document doc : theCollection.find ( Find.ALL ) ) {
            final Element id = doc.get ( "_id" );
            final NumericElement lat = doc.get ( NumericElement.class,
                                                 "latitude_deg" );
            final NumericElement lon = doc.get ( NumericElement.class,
                                                 "longitude_deg" );
            final DocumentBuilder query = BuilderFactory.start();
            query.add ( id );
            final DocumentBuilder update = BuilderFactory.start();
            update.push ( "$set" ).add (
                "loc",
                GeoJson.point ( GeoJson.p ( lon.getDoubleValue(),
                                            lat.getDoubleValue() ) ) );
            theCollection.update ( query, update, Durability.ACK );
            count += 1;
        }
        System.out.printf ( "Updated %d documents synchronously.%n",
                            count );
    }
}
