package org.apache.ant.dotnet.build;
import java.io.File;
import java.util.List;
import org.apache.ant.dotnet.util.CollectionUtils;
import org.w3c.dom.DocumentFragment;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
public class MSBuildTask extends AbstractBuildTask {
    private static final String TARGET = "generated-by-ant";
    private static final String ROOT = "Project";
    private static final String MSBUILD_NS =
        "http://schemas.microsoft.com/developer/msbuild/2003";
    public MSBuildTask() {
        super();
    }
    protected String getExecutable() {
        return "MSBuild.exe";
    }
    protected String[] getBuildfileArguments ( File buildFile ) {
        if ( buildFile != null ) {
            return new String[] {
                       buildFile.getAbsolutePath()
                   };
        } else {
            return new String[0];
        }
    }
    protected String[] getTargetArguments ( List targets ) {
        if ( targets.size() > 0 ) {
            StringBuffer sb = new StringBuffer ( "/target:" );
            sb.append ( CollectionUtils.flattenToString ( targets, ";" ) );
            return new String[] {sb.toString() };
        } else {
            return new String[0];
        }
    }
    protected String[] getPropertyArguments ( List properties ) {
        if ( properties.size() > 0 ) {
            StringBuffer sb = new StringBuffer ( "/property:" );
            sb.append ( CollectionUtils.flattenToString ( properties, ";" ) );
            return new String[] {sb.toString() };
        } else {
            return new String[0];
        }
    }
    protected Element makeTree ( DocumentFragment f ) {
        NodeList nl = f.getChildNodes();
        if ( nl.getLength() == 1
                && nl.item ( 0 ).getNodeType() == Node.ELEMENT_NODE
                && nl.item ( 0 ).getNodeName().equals ( ROOT ) ) {
            return ( Element ) nl.item ( 0 );
        } else {
            Element p = f.getOwnerDocument().createElementNS ( MSBUILD_NS,
                        ROOT );
            p.setAttribute ( "DefaultTargets", TARGET );
            Element t = f.getOwnerDocument().createElementNS ( MSBUILD_NS,
                        "Target" );
            t.setAttribute ( "Name", TARGET );
            p.appendChild ( t );
            t.appendChild ( f );
            return p;
        }
    }
}
