@MappedSuperclass
@Inheritance ( strategy = InheritanceType.SINGLE_TABLE )
@Table ( name = "state" )
public class StateBase {
    @Id
    @Column ( name = "id" )
    private String id = this.getClass().getSimpleName();
    @Column ( name = "name" )
    private String name;
    @Column ( name = "badgeColor" )
    private String badgeColor;
    final public void changeState ( Session session, State newState, StateEntity entity ) {
        if ( newState.equals ( this ) ) {
            return;
        }
        if ( canMove ( session, newState, entity ) ) {
            hook ( session, newState, entity );
            beforeTransition ( session, newState, entity );
            makeTransition ( session, newState, entity );
            afterTransition ( session, newState, entity );
        } else {
            throw new TransitionDeniedException ( "Transition from " + getName() + " to " + newState.getName() + " is denied." );
        }
    }
    protected void beforeTransition ( Session session, State newState, StateEntity entity ) {
        entity.setState ( newState );
    }
    protected void afterTransition ( Session session, State newState, StateEntity entity ) {
        writeHistory ( entity );
        session.merge ( entity );
    }
}
