public class LocationService implements LocationListener {
    boolean isGPSEnabled = false;
    boolean isNetworkEnabled = false;
    boolean canGetLocation = false;
    final static long MIN_TIME_INTERVAL = 60 * 1000L;
    Location location;
    private static final long MIN_DISTANCE_CHANGE_FOR_UPDATES = 0;
    private static final long MIN_TIME_BW_UPDATES = 1;
    protected LocationManager locationManager;
    private CountDownTimer timer = new CountDownTimer ( 5 * 1000, 1000 ) {
        public void onTick ( long millisUntilFinished ) {
        }
        public void onFinish() {
            stopUsingGPS();
        }
    };
    public LocationService() {
        super ( R.id.gps_service_id );
    }
    public void start() {
        if ( Utils.isNetworkAvailable ( context ) ) {
            try {
                timer.start();
                locationManager = ( LocationManager ) context
                                  .getSystemService ( Context.LOCATION_SERVICE );
                isGPSEnabled = locationManager
                               .isProviderEnabled ( LocationManager.GPS_PROVIDER );
                isNetworkEnabled = locationManager
                                   .isProviderEnabled ( LocationManager.NETWORK_PROVIDER );
                this.canGetLocation = true;
                if ( isNetworkEnabled ) {
                    locationManager.requestLocationUpdates (
                        LocationManager.NETWORK_PROVIDER,
                        MIN_TIME_BW_UPDATES,
                        MIN_DISTANCE_CHANGE_FOR_UPDATES, this );
                    Log.d ( "Network", "Network" );
                    if ( locationManager != null ) {
                        Location tempLocation = locationManager
                                                .getLastKnownLocation ( LocationManager.NETWORK_PROVIDER );
                        if ( tempLocation != null
                                && isBetterLocation ( tempLocation,
                                                      location ) ) {
                            location = tempLocation;
                        }
                    }
                }
                if ( isGPSEnabled ) {
                    locationManager.requestSingleUpdate (
                        LocationManager.GPS_PROVIDER, this, null );
                    locationManager.requestLocationUpdates (
                        LocationManager.GPS_PROVIDER, MIN_TIME_BW_UPDATES,
                        MIN_DISTANCE_CHANGE_FOR_UPDATES, this );
                    Log.d ( "GPS Enabled", "GPS Enabled" );
                    if ( locationManager != null ) {
                        Location tempLocation = locationManager
                                                .getLastKnownLocation ( LocationManager.GPS_PROVIDER );
                        if ( tempLocation != null
                                && isBetterLocation ( tempLocation,
                                                      location ) ) {
                            location = tempLocation;
                        }
                    }
                }
            } catch ( Exception e ) {
                onTaskError ( e.getMessage() );
                e.printStackTrace();
            }
        } else {
            onOfflineResponse ( requestData );
        }
    }
    public void stopUsingGPS() {
        if ( locationManager != null ) {
            locationManager.removeUpdates ( LocationService.this );
        }
    }
    public boolean canGetLocation() {
        locationManager = ( LocationManager ) context
                          .getSystemService ( Context.LOCATION_SERVICE );
        isGPSEnabled = locationManager
                       .isProviderEnabled ( LocationManager.GPS_PROVIDER );
        isNetworkEnabled = locationManager
                           .isProviderEnabled ( LocationManager.NETWORK_PROVIDER );
        return isGPSEnabled || isNetworkEnabled;
    }
    @Override
    public void onLocationChanged ( Location location ) {
        if ( location != null
                && isBetterLocation ( location, this.location ) ) {
            this.location = location;
        }
    }
    @Override
    public void onProviderDisabled ( String provider ) {
    }
    @Override
    public void onProviderEnabled ( String provider ) {
    }
    @Override
    public void onStatusChanged ( String provider, int status, Bundle extras ) {
    }
    @Override
    public Object getResponseObject ( Object location ) {
        return location;
    }
    public static boolean isBetterLocation ( Location location,
            Location currentBestLocation ) {
        if ( currentBestLocation == null ) {
            return true;
        }
        long timeDelta = location.getTime() - currentBestLocation.getTime();
        boolean isSignificantlyNewer = timeDelta > MIN_TIME_INTERVAL;
        boolean isSignificantlyOlder = timeDelta < -MIN_TIME_INTERVAL;
        boolean isNewer = timeDelta > 0;
        if ( isSignificantlyNewer ) {
            return true;
        } else if ( isSignificantlyOlder ) {
            return false;
        }
        int accuracyDelta = ( int ) ( location.getAccuracy() - currentBestLocation
                                      .getAccuracy() );
        boolean isLessAccurate = accuracyDelta > 0;
        boolean isMoreAccurate = accuracyDelta < 0;
        boolean isSignificantlyLessAccurate = accuracyDelta > 200;
        boolean isFromSameProvider = isSameProvider ( location.getProvider(),
                                     currentBestLocation.getProvider() );
        if ( isMoreAccurate ) {
            return true;
        } else if ( isNewer && !isLessAccurate ) {
            return true;
        } else if ( isNewer && !isSignificantlyLessAccurate
                    && isFromSameProvider ) {
            return true;
        }
        return false;
    }
}
