package com.example.audiorecord;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import android.app.Activity;
import android.media.AudioFormat;
import android.media.AudioRecord;
import android.media.MediaRecorder;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.View;
import android.widget.Button;
public class MainActivity extends Activity {
    private static final int RECORDER_SAMPLERATE = 44100;
    private static final int RECORDER_CHANNELS = AudioFormat.CHANNEL_IN_MONO;
    private static final int RECORDER_AUDIO_ENCODING = AudioFormat.ENCODING_PCM_16BIT;
    private AudioRecord recorder = null;
    private Thread recordingThread = null;
    private boolean isRecording = false;
    private String floatString = "";
    Calendar c = Calendar.getInstance();
    SimpleDateFormat df = new SimpleDateFormat ( "ddMMMyyyy_HHmmss" );
    String formattedDate = df.format ( c.getTime() );
    @Override
    public void onCreate ( Bundle savedInstanceState ) {
        super.onCreate ( savedInstanceState );
        setContentView ( R.layout.activity_main );
        setButtonHandlers();
        enableButtons ( false );
        int bufferSize = AudioRecord.getMinBufferSize ( RECORDER_SAMPLERATE,
                         RECORDER_CHANNELS, RECORDER_AUDIO_ENCODING );
    }
    private void setButtonHandlers() {
        ( ( Button ) findViewById ( R.id.btnStart ) ).setOnClickListener ( btnClick );
        ( ( Button ) findViewById ( R.id.btnStop ) ).setOnClickListener ( btnClick );
    }
    private void enableButton ( int id, boolean isEnable ) {
        ( ( Button ) findViewById ( id ) ).setEnabled ( isEnable );
    }
    private void enableButtons ( boolean isRecording ) {
        enableButton ( R.id.btnStart, !isRecording );
        enableButton ( R.id.btnStop, isRecording );
    }
    int BufferElements2Rec = 1024;
    int BytesPerElement = 2;
    private void startRecording() {
        final File logFile = new File ( "sdcard/log_" + formattedDate + ".txt" );
        if ( !logFile.exists() ) {
            try {
                logFile.createNewFile();
            } catch ( IOException e ) {
                e.printStackTrace();
            }
        }
        recorder = new AudioRecord ( MediaRecorder.AudioSource.MIC,
                                     RECORDER_SAMPLERATE, RECORDER_CHANNELS,
                                     RECORDER_AUDIO_ENCODING, BufferElements2Rec * BytesPerElement );
        recorder.startRecording();
        isRecording = true;
        recordingThread = new Thread ( new Runnable() {
            public void run() {
                writeAudioDataToFile ( logFile );
            }
        }, "AudioRecorder Thread" );
        recordingThread.start();
    }
    private void writeAudioDataToFile ( File logFile ) {
        short sData[] = new short[BufferElements2Rec];
        while ( isRecording ) {
            recorder.read ( sData, 0, BufferElements2Rec );
            String sDataString = "";
            double sDataMax = 0;
            for ( int i = 0; i < sData.length; i++ ) {
                if ( Math.abs ( sData[i] ) >= sDataMax ) {
                    sDataMax = Math.abs ( sData[i] );
                };
                sDataString += "\n" + sDataMax;
            }
            try {
                BufferedWriter buf = new BufferedWriter ( new FileWriter ( logFile, true ) );
                buf.append ( sDataString );
                buf.newLine();
                buf.close();
            } catch ( IOException e ) {
                e.printStackTrace();
            }
        }
    }
    private void stopRecording() {
        if ( null != recorder ) {
            isRecording = false;
            recorder.stop();
            recorder.release();
            recorder = null;
            recordingThread = null;
        }
    }
    private View.OnClickListener btnClick = new View.OnClickListener() {
        public void onClick ( View v ) {
            switch ( v.getId() ) {
            case R.id.btnStart: {
                enableButtons ( true );
                startRecording();
                break;
            }
            case R.id.btnStop: {
                enableButtons ( false );
                stopRecording();
                break;
            }
            }
        }
    };
    @Override
    public boolean onKeyDown ( int keyCode, KeyEvent event ) {
        if ( keyCode == KeyEvent.KEYCODE_BACK ) {
            finish();
        }
        return super.onKeyDown ( keyCode, event );
    }
    @Override
    public boolean onCreateOptionsMenu ( Menu menu ) {
        getMenuInflater().inflate ( R.menu.main, menu );
        return true;
    }
}
