package be.pbeckers.javafxguiupdater;
import java.util.concurrent.ConcurrentLinkedQueue;
import javafx.application.Platform;
import javafx.beans.property.Property;
import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
public abstract class GUIUpdater {
    private static  ConcurrentLinkedQueue<PropertyUpdater<?>>   dirtyPropertyUpdaters   =   new ConcurrentLinkedQueue<>();
    private static  Updater                                     updater                 =   new Updater();
    private static  boolean                                     isUpdating              =   false;
    public static <T> void bind ( Property<T> property, ObservableValue<T> observable ) {
        PropertyUpdater<T>  propertyUpdater = new PropertyUpdater<> ( property, observable );
        observable.addListener ( propertyUpdater );
    }
    public static <T> void unbind ( Property<T> property, ObservableValue<T> observable ) {
        PropertyUpdater<T>  tmpPropertyUpdater = new PropertyUpdater<> ( property, observable );
        observable.removeListener ( tmpPropertyUpdater );
    }
    private static synchronized void scheduleUpdate ( PropertyUpdater<?> updater ) {
        GUIUpdater.dirtyPropertyUpdaters.add ( updater );
        if ( !GUIUpdater.isUpdating ) {
            GUIUpdater.isUpdating = true;
            Platform.runLater ( GUIUpdater.updater );
        }
    }
    private static class PropertyUpdater<T> implements ChangeListener<T> {
        private boolean             isDirty     =   false;
        private Property<T>         property    =   null;
        private ObservableValue<T>  observable  =   null;
        public PropertyUpdater ( Property<T> property, ObservableValue<T> observable ) {
            this.property = property;
            this.observable = observable;
        }
        @Override
        public synchronized void changed ( ObservableValue<? extends T> observable, T oldValue, T newValue ) {
            if ( !this.isDirty ) {
                this.isDirty = true;
                GUIUpdater.scheduleUpdate ( this );
            }
        }
        public synchronized void update() {
            T value = this.observable.getValue();
            this.property.setValue ( value );
            this.isDirty = false;
        }
        @Override
        public boolean equals ( Object otherObj ) {
            PropertyUpdater<?>  otherUpdater = ( PropertyUpdater<?> ) otherObj;
            if ( otherObj == null ) {
                return false;
            } else {
                return ( this.property == otherUpdater.property ) && ( this.observable == otherUpdater.observable );
            }
        }
    }
    private static class Updater implements Runnable {
        @Override
        public void run() {
            synchronized ( GUIUpdater.class ) {
                while ( !GUIUpdater.dirtyPropertyUpdaters.isEmpty() ) {
                    PropertyUpdater<?>  curUpdater = GUIUpdater.dirtyPropertyUpdaters.poll();
                    curUpdater.update();
                }
                GUIUpdater.isUpdating = false;
            }
        }
    }
}
