public abstract class GuardedInteceptor implements MethodInterceptor {
    private boolean executionInProgress = false;
    public Object invoke ( MethodInvocation invocation ) throws Throwable {
        if ( executionInProgress ) {
            return invocation.proceed();
        }
        executionInProgress = true;
        Object returnValue = null;
        try {
            runBefore();
            returnValue = invocation.proceed();
            runAfter();
        } finally {
            executionInProgress = false;
        }
        return returnValue;
    }
    protected abstract void runBefore();
    protected abstract void runAfter();
}
