private static final int HARD_CACHE_CAPACITY = 10;
private static final int DELAY_BEFORE_PURGE = 10 * 1000;
private final HashMap<String, Bitmap> sHardBitmapCache =
new LinkedHashMap<String, Bitmap> ( HARD_CACHE_CAPACITY / 2, 0.75f, true ) {
    @Override
    protected boolean removeEldestEntry ( LinkedHashMap.Entry<String, Bitmap> eldest ) {
        if ( size() > HARD_CACHE_CAPACITY ) {
            sSoftBitmapCache.put ( eldest.getKey(), new SoftReference<Bitmap> ( eldest.getValue() ) );
            return true;
        } else {
            return false;
        }
    }
};
private final static ConcurrentHashMap<String, SoftReference<Bitmap>> sSoftBitmapCache =
    new ConcurrentHashMap<String, SoftReference<Bitmap>> ( HARD_CACHE_CAPACITY / 2 );
private final Handler purgeHandler = new Handler();
private final Runnable purger = new Runnable() {
    public void run() {
        clearCache();
    }
};
private void addBitmapToCache ( String url, Bitmap bitmap ) {
    if ( bitmap != null ) {
        synchronized ( sHardBitmapCache ) {
            sHardBitmapCache.put ( url, bitmap );
        }
    }
}
private Bitmap getBitmapFromCache ( String url ) {
    synchronized ( sHardBitmapCache ) {
        final Bitmap bitmap = sHardBitmapCache.get ( url );
        if ( bitmap != null ) {
            sHardBitmapCache.remove ( url );
            sHardBitmapCache.put ( url, bitmap );
            return bitmap;
        }
    }
    SoftReference<Bitmap> bitmapReference = sSoftBitmapCache.get ( url );
    if ( bitmapReference != null ) {
        final Bitmap bitmap = bitmapReference.get();
        if ( bitmap != null ) {
            return bitmap;
        } else {
            sSoftBitmapCache.remove ( url );
        }
    }
    return null;
}
public void clearCache() {
    sHardBitmapCache.clear();
    sSoftBitmapCache.clear();
}
