public class MainActivity extends Activity {
    private DrawerLayout mDrawerLayout;
    private ListView mDrawerList;
    private ActionBarDrawerToggle mDrawerToggle;
    private CharSequence mDrawerTitle;
    private CharSequence mTitle;
    private String[] navMenuTitles;
    private TypedArray navMenuIcons;
    private ArrayList<NavDrawerItem> navDrawerItems;
    private NavDrawerListAdapter adapter;
    private HomeFragment homeFragment;
    @Override
    protected void onCreate ( Bundle savedInstanceState ) {
        super.onCreate ( savedInstanceState );
        setContentView ( R.layout.activity_main );
        mTitle = mDrawerTitle = getTitle();
        navMenuTitles = getResources().getStringArray ( R.array.nav_drawer_items );
        navMenuIcons = getResources()
                       .obtainTypedArray ( R.array.nav_drawer_icons );
        mDrawerLayout = ( DrawerLayout ) findViewById ( R.id.drawer_layout );
        mDrawerList = ( ListView ) findViewById ( R.id.list_slidermenu );
        navDrawerItems = new ArrayList<NavDrawerItem>();
        navDrawerItems.add ( new NavDrawerItem ( navMenuTitles[0], navMenuIcons.getResourceId ( 0, -1 ) ) );
        navDrawerItems.add ( new NavDrawerItem ( navMenuTitles[1], navMenuIcons.getResourceId ( 1, -1 ), true, "22" ) );
        navDrawerItems.add ( new NavDrawerItem ( navMenuTitles[2], navMenuIcons.getResourceId ( 2, -1 ) ) );
        navMenuIcons.recycle();
        mDrawerList.setOnItemClickListener ( new SlideMenuClickListener() );
        adapter = new NavDrawerListAdapter ( getApplicationContext(),
                                             navDrawerItems );
        mDrawerList.setAdapter ( adapter );
        getActionBar().setDisplayHomeAsUpEnabled ( true );
        getActionBar().setHomeButtonEnabled ( true );
        mDrawerToggle = new ActionBarDrawerToggle ( this, mDrawerLayout,
                R.drawable.ic_drawer,
                R.string.app_name,
                R.string.app_name
        ) {
            public void onDrawerClosed ( View view ) {
                getActionBar().setTitle ( mTitle );
                invalidateOptionsMenu();
            }
            public void onDrawerOpened ( View drawerView ) {
                getActionBar().setTitle ( mDrawerTitle );
                invalidateOptionsMenu();
            }
        };
        mDrawerLayout.setDrawerListener ( mDrawerToggle );
        if ( savedInstanceState == null ) {
            displayView ( 0 );
            homeFragment = new HomeFragment();
            FragmentManager fragmentManager = getFragmentManager();
            fragmentManager.beginTransaction()
            .replace ( R.id.frame_container, homeFragment ).commit();
            mDrawerLayout.closeDrawer ( mDrawerList );
        }
    }
    private class SlideMenuClickListener implements
        ListView.OnItemClickListener {
        @Override
        public void onItemClick ( AdapterView<?> parent, View view, int position,
                                  long id ) {
            displayView ( position );
        }
    }
    @Override
    public boolean onOptionsItemSelected ( MenuItem item ) {
        if ( mDrawerToggle.onOptionsItemSelected ( item ) ) {
            return true;
        }
        switch ( item.getItemId() ) {
        case R.id.action_settings:
            return true;
        default:
            return super.onOptionsItemSelected ( item );
        }
    }
    @Override
    public boolean onPrepareOptionsMenu ( Menu menu ) {
        boolean drawerOpen = mDrawerLayout.isDrawerOpen ( mDrawerList );
        return super.onPrepareOptionsMenu ( menu );
    }
    private void displayView ( int position ) {
        switch ( position ) {
        case 0:
            homeFragment.clearText();
            break;
        case 1:
            break;
        case 2:
            break;
        default:
            break;
        }
    }
    @Override
    public void setTitle ( CharSequence title ) {
        mTitle = title;
        getActionBar().setTitle ( mTitle );
    }
    @Override
    protected void onPostCreate ( Bundle savedInstanceState ) {
        super.onPostCreate ( savedInstanceState );
        mDrawerToggle.syncState();
    }
    @Override
    public void onConfigurationChanged ( Configuration newConfig ) {
        super.onConfigurationChanged ( newConfig );
        mDrawerToggle.onConfigurationChanged ( newConfig );
    }
}
