#include <iostream>
#include <vector>
#include <opencv2/highgui/highgui.hpp>
#include <opencv2/imgproc/imgproc.hpp>
double angle ( cv::Point pt1, cv::Point pt2, cv::Point pt0 ) {
    double dx1 = pt1.x - pt0.x;
    double dy1 = pt1.y - pt0.y;
    double dx2 = pt2.x - pt0.x;
    double dy2 = pt2.y - pt0.y;
    return ( dx1 * dx2 + dy1 * dy2 ) / sqrt ( ( dx1 * dx1 + dy1 * dy1 ) * ( dx2 * dx2 + dy2 * dy2 ) + 1e-10 );
}
void findSquares ( const cv::Mat & image, std::vector<std::vector<cv::Point> > & squares ) {
    cv::Mat pyr, timg;
    cv::pyrDown ( image, pyr, cv::Size ( image.cols / 2, image.rows / 2 ) );
    cv::pyrUp ( pyr, timg, image.size() );
    cv::Canny ( timg, timg, 0, 50, 5 );
    cv::dilate ( timg, timg, cv::Mat(), cv::Point ( -1, -1 ) );
    std::vector<std::vector<cv::Point> > contours;
    cv::findContours ( timg, contours, CV_RETR_LIST, CV_CHAIN_APPROX_SIMPLE );
    for ( size_t i = 0; i < contours.size(); i++ ) {
        std::vector<cv::Point> approx;
        cv::approxPolyDP ( cv::Mat ( contours[i] ), approx, cv::arcLength ( cv::Mat ( contours[i] ), true ) * 0.02, true );
        if ( approx.size() == 4 &&
                fabs ( cv::contourArea ( cv::Mat ( approx ) ) ) > 1000 &&
                cv::isContourConvex ( cv::Mat ( approx ) ) ) {
            double maxCosine = 0;
            for ( int j = 2; j < 5; j++ ) {
                double cosine = fabs ( angle ( approx[j % 4], approx[j - 2], approx[j - 1] ) );
                maxCosine = MAX ( maxCosine, cosine );
            }
            if ( maxCosine < 0.3 ) {
                squares.push_back ( approx );
            }
        }
    }
}
void drawSquares ( cv::Mat & image, const std::vector<std::vector<cv::Point> > & squares ) {
    for ( size_t i = 0; i < squares.size(); i++ ) {
        const cv::Point * p = & squares[i][0];
        int n = ( int ) squares[i].size();
        cv::polylines ( image, & p, & n, 1, true, cv::Scalar ( 0, 255, 0 ), 2, CV_AA );
    }
    cv::imshow ( "drawSquares", image );
}
