package checkboxdemo;
import javafx.application.Application;
import javafx.beans.property.*;
import javafx.beans.value.ObservableValue;
import javafx.collections.FXCollections;
import javafx.event.*;
import javafx.geometry.*;
import javafx.scene.Scene;
import javafx.scene.control.*;
import javafx.scene.control.cell.PropertyValueFactory;
import javafx.scene.image.Image;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.*;
import javafx.stage.*;
import javafx.util.Callback;
public class TableViewWithAddButtonExample extends Application {
    public static void main ( String[] args ) {
        launch ( args );
    }
    @Override public void start ( final Stage stage ) {
        stage.setTitle ( "People" );
        final TableView<Person> table = new TableView<> (
            FXCollections.observableArrayList (
                new Person ( "Jacob", "Smith" ),
                new Person ( "Isabella", "Johnson" ),
                new Person ( "Ethan", "Williams" ),
                new Person ( "Emma", "Jones" ),
                new Person ( "Michael", "Brown" )
            )
        );
        TableColumn<Person, String> firstNameCol = new TableColumn<> ( "First Name" );
        firstNameCol.setCellValueFactory ( new PropertyValueFactory ( "firstName" ) );
        TableColumn<Person, String> lastNameCol = new TableColumn<> ( "Last Name" );
        lastNameCol.setCellValueFactory ( new PropertyValueFactory ( "lastName" ) );
        TableColumn<Person, Boolean> actionCol = new TableColumn<> ( "Action" );
        actionCol.setSortable ( false );
        actionCol.setCellValueFactory ( new Callback<TableColumn.CellDataFeatures<Person, Boolean>, ObservableValue<Boolean>>() {
            @Override public ObservableValue<Boolean> call ( TableColumn.CellDataFeatures<Person, Boolean> features ) {
                return new SimpleBooleanProperty ( features.getValue() != null );
            }
        } );
        actionCol.setCellFactory ( new Callback<TableColumn<Person, Boolean>, TableCell<Person, Boolean>>() {
            @Override public TableCell<Person, Boolean> call ( TableColumn<Person, Boolean> personBooleanTableColumn ) {
                return new AddPersonCell ( stage, table );
            }
        } );
        table.getColumns().setAll ( firstNameCol, lastNameCol, actionCol );
        table.setColumnResizePolicy ( TableView.CONSTRAINED_RESIZE_POLICY );
        stage.setScene ( new Scene ( table ) );
        stage.show();
    }
    private class AddPersonCell extends TableCell<Person, Boolean> {
        final Button addButton       = new Button ( "Add" );
        final StackPane paddedButton = new StackPane();
        final DoubleProperty buttonY = new SimpleDoubleProperty();
        AddPersonCell ( final Stage stage, final TableView table ) {
            paddedButton.setPadding ( new Insets ( 3 ) );
            paddedButton.getChildren().add ( addButton );
            addButton.setOnMousePressed ( new EventHandler<MouseEvent>() {
                @Override public void handle ( MouseEvent mouseEvent ) {
                    buttonY.set ( mouseEvent.getScreenY() );
                }
            } );
            addButton.setOnAction ( new EventHandler<ActionEvent>() {
                @Override public void handle ( ActionEvent actionEvent ) {
                    showAddPersonDialog ( stage, table, buttonY.get() );
                    table.getSelectionModel().select ( getTableRow().getIndex() );
                }
            } );
        }
        @Override protected void updateItem ( Boolean item, boolean empty ) {
            super.updateItem ( item, empty );
            if ( !empty ) {
                setContentDisplay ( ContentDisplay.GRAPHIC_ONLY );
                setGraphic ( paddedButton );
            }
        }
    }
    private void showAddPersonDialog ( Stage parent, final TableView<Person> table, double y ) {
        final Stage dialog = new Stage();
        dialog.setTitle ( "New Person" );
        dialog.initOwner ( parent );
        dialog.initModality ( Modality.WINDOW_MODAL );
        dialog.initStyle ( StageStyle.UTILITY );
        dialog.setX ( parent.getX() + parent.getWidth() );
        dialog.setY ( y );
        GridPane grid = new GridPane();
        final TextField firstNameField = new TextField();
        final TextField lastNameField = new TextField();
        grid.addRow ( 0, new Label ( "First Name" ), firstNameField );
        grid.addRow ( 1, new Label ( "Last Name" ), lastNameField );
        grid.setHgap ( 10 );
        grid.setVgap ( 10 );
        GridPane.setHgrow ( firstNameField, Priority.ALWAYS );
        GridPane.setHgrow ( lastNameField, Priority.ALWAYS );
        Button ok = new Button ( "OK" );
        ok.setDefaultButton ( true );
        Button cancel = new Button ( "Cancel" );
        cancel.setCancelButton ( true );
        ok.disableProperty().bind ( firstNameField.textProperty().isEqualTo ( "" ).or ( lastNameField.textProperty().isEqualTo ( "" ) ) );
        ok.setOnAction ( new EventHandler<ActionEvent>() {
            @Override public void handle ( ActionEvent actionEvent ) {
                int nextIndex = table.getSelectionModel().getSelectedIndex() + 1;
                table.getItems().add ( nextIndex, new Person ( firstNameField.getText(), lastNameField.getText() ) );
                table.getSelectionModel().select ( nextIndex );
                dialog.close();
            }
        } );
        cancel.setOnAction ( new EventHandler<ActionEvent>() {
            @Override public void handle ( ActionEvent actionEvent ) {
                dialog.close();
            }
        } );
        HBox buttons = HBoxBuilder.create().spacing ( 10 ).children ( ok, cancel ).alignment ( Pos.CENTER_RIGHT ).build();
        VBox layout = new VBox ( 10 );
        layout.getChildren().addAll ( grid, buttons );
        layout.setPadding ( new Insets ( 5 ) );
        dialog.setScene ( new Scene ( layout ) );
        dialog.show();
    }
}
