package com.weebly.foxgenesis.src;
import java.net.*;
import java.io.*;
public final class ChatClient {
    private Socket socket = null;
    private DataOutputStream streamOut = null;
    private ChatClientThread client = null;
    private String serverName = "localhost";
    private int serverPort = -1;
    private final ChatReciever output;
    public ChatClient ( ChatReciever chatReciever ) {
        output = chatReciever;
        serverPort = chatReciever.getPort();
        serverName = chatReciever.getHost();
        connect ( serverName, serverPort );
    }
    private void connect ( String serverName, int serverPort ) {
        output.handleLog ( "Establishing connection. Please wait ..." );
        try {
            socket = new Socket ( serverName, serverPort );
            output.handle ( "Connected to chat server" );
            open();
        } catch ( UnknownHostException uhe ) {
            output.handleError ( "Host unknown: " + uhe.getMessage() );
        } catch ( IOException ioe ) {
            output.handleError ( "Unexpected exception: " + ioe.getMessage() );
        }
    }
    public void send ( String msg ) throws IOException {
        streamOut.writeUTF ( msg );
        streamOut.flush();
    }
    public void handle ( String msg ) {
        output.handle ( msg );
    }
    private void open() throws IOException {
        streamOut = new DataOutputStream ( socket.getOutputStream() );
        client = new ChatClientThread ( this, socket );
    }
    public void close() throws IOException {
        if ( streamOut != null ) {
            streamOut.close();
        }
        if ( socket    != null ) {
            socket.close();
        }
    }
    @SuppressWarnings ( "deprecation" )
    public void stop() {
        if ( client != null ) {
            client.stop();
        }
        client = null;
    }
    public boolean isConnected() {
        return client == null ? ( false ) : ( true );
    }
}
