public class HelloPhysics extends SimpleApplication {
    public static void main ( String args[] ) {
        HelloPhysics app = new HelloPhysics();
        app.start();
    }
    private BulletAppState bulletAppState;
    Material wall_mat;
    Material stone_mat;
    Material floor_mat;
    Node removables;
    private static final Box    box;
    private RigidBodyControl    ball_phy;
    private static final Sphere sphere;
    private RigidBodyControl    floor_phy;
    private static final Box    floor;
    private static Collection<RigidBodyControl> objectsThatNeedWaking = new HashSet<RigidBodyControl>();
    private static final float brickLength = 0.48f;
    private static final float brickWidth  = 0.24f;
    private static final float brickHeight = 0.12f;
    static {
        sphere = new Sphere ( 32, 32, 0.4f, true, false );
        sphere.setTextureMode ( TextureMode.Projected );
        box = new Box ( Vector3f.ZERO, brickLength, brickHeight, brickWidth );
        box.scaleTextureCoordinates ( new Vector2f ( 1f, .5f ) );
        floor = new Box ( Vector3f.ZERO, 10f, 0.1f, 5f );
        floor.scaleTextureCoordinates ( new Vector2f ( 3, 6 ) );
    }
    @Override
    public void simpleInitApp() {
        bulletAppState = new BulletAppState();
        stateManager.attach ( bulletAppState );
        cam.setLocation ( new Vector3f ( 0, 4f, 6f ) );
        cam.lookAt ( new Vector3f ( 2, 2, 0 ), Vector3f.UNIT_Y );
        inputManager.addMapping ( "shoot",
                                  new MouseButtonTrigger ( MouseInput.BUTTON_LEFT ) );
        inputManager.addListener ( actionListener, "shoot" );
        removables = new Node ( "Removables" );
        rootNode.attachChild ( removables );
        initMaterials();
        initWall();
        initFloor();
        initKeys();
        initCrossHairs();
    }
    private void initKeys() {
        inputManager.addMapping ( "Shoot",
                                  new KeyTrigger ( KeyInput.KEY_SPACE ),
                                  new MouseButtonTrigger ( MouseInput.BUTTON_LEFT ) );
        inputManager.addListener ( actionListener, "Shoot" );
    }
    public void initMaterials() {
        wall_mat = new Material ( assetManager, "Common/MatDefs/Misc/Unshaded.j3md" );
        TextureKey key = new TextureKey ( "Textures/Terrain/BrickWall/BrickWall.jpg" );
        key.setGenerateMips ( true );
        Texture tex = assetManager.loadTexture ( key );
        wall_mat.setTexture ( "ColorMap", tex );
        stone_mat = new Material ( assetManager, "Common/MatDefs/Misc/Unshaded.j3md" );
        TextureKey key2 = new TextureKey ( "Textures/Terrain/Rock/Rock.PNG" );
        key2.setGenerateMips ( true );
        Texture tex2 = assetManager.loadTexture ( key2 );
        stone_mat.setTexture ( "ColorMap", tex2 );
        floor_mat = new Material ( assetManager, "Common/MatDefs/Misc/Unshaded.j3md" );
        TextureKey key3 = new TextureKey ( "Textures/Terrain/Pond/Pond.jpg" );
        key3.setGenerateMips ( true );
        Texture tex3 = assetManager.loadTexture ( key3 );
        tex3.setWrap ( WrapMode.Repeat );
        floor_mat.setTexture ( "ColorMap", tex3 );
    }
    public void initFloor() {
        Geometry floor_geo = new Geometry ( "Floor", floor );
        floor_geo.setMaterial ( floor_mat );
        floor_geo.setLocalTranslation ( 0, -0.1f, 0 );
        this.rootNode.attachChild ( floor_geo );
        floor_phy = new RigidBodyControl ( 0.0f );
        floor_geo.addControl ( floor_phy );
        bulletAppState.getPhysicsSpace().add ( floor_phy );
    }
    public void initWall() {
        float startpt = brickLength / 4;
        float height = 0;
        for ( int j = 0; j < 15; j++ ) {
            for ( int i = 0; i < 6; i++ ) {
                Vector3f vt =
                    new Vector3f ( i * brickLength * 2 + startpt, brickHeight + height, 0 );
                makeBrick ( vt );
            }
            startpt = -startpt;
            height += 2 * brickHeight;
        }
    }
    public void makeBrick ( Vector3f loc ) {
        Geometry brick_geo = new Geometry ( "brick", box );
        brick_geo.setMaterial ( wall_mat );
        brick_geo.setLocalTranslation ( loc );
        RigidBodyControl brick_phy = new RigidBodyControl ( 2f );
        objectsThatNeedWaking.add ( brick_phy );
        brick_geo.addControl ( brick_phy );
        bulletAppState.getPhysicsSpace().add ( brick_phy );
        removables.attachChild ( brick_geo );
    }
    public void makeCannonBall() {
        Geometry ball_geo = new Geometry ( "cannon ball", sphere );
        ball_geo.setMaterial ( stone_mat );
        rootNode.attachChild ( ball_geo );
        ball_geo.setLocalTranslation ( cam.getLocation() );
        ball_phy = new RigidBodyControl ( 1f );
        ball_geo.addControl ( ball_phy );
        bulletAppState.getPhysicsSpace().add ( ball_phy );
        ball_phy.setLinearVelocity ( cam.getDirection().mult ( 25 ) );
    }
    private ActionListener actionListener = new ActionListener() {
        public void onAction ( String name, boolean keyPressed, float tpf ) {
            if ( name.equals ( "Shoot" ) && !keyPressed ) {
                CollisionResults results = new CollisionResults();
                Ray ray = new Ray ( cam.getLocation(), cam.getDirection() );
                removables.collideWith ( ray, results );
                System.out.println ( "----- Collisions? " + results.size() + "-----" );
                for ( int i = 0; i < results.size(); i++ ) {
                    float dist = results.getCollision ( i ).getDistance();
                    Vector3f pt = results.getCollision ( i ).getContactPoint();
                    String hit = results.getCollision ( i ).getGeometry().getName();
                    System.out.println ( "* Collision #" + i );
                    System.out.println ( "  You shot " + hit + " at " + pt + ", " + dist + " wu away." );
                }
                if ( results.size() > 0 ) {
                    Geometry closest = results.getClosestCollision().getGeometry();
                    bulletAppState.getPhysicsSpace().remove ( closest.getControl ( 0 ) );
                    objectsThatNeedWaking.remove ( closest.getControl ( 0 ) );
                    removables.detachChild ( closest );
                    bulletAppState.getPhysicsSpace().clearForces();
                    bulletAppState.getPhysicsSpace().applyGravity();
                    System.out.println ( "removing a " + closest.getName() );
                    for ( RigidBodyControl wakeMeUp : objectsThatNeedWaking ) {
                        wakeMeUp.activate();
                    }
                }
            }
        }
    };
    protected void initCrossHairs() {
        guiNode.detachAllChildren();
        guiFont = assetManager.loadFont ( "Interface/Fonts/Default.fnt" );
        BitmapText ch = new BitmapText ( guiFont, false );
        ch.setSize ( guiFont.getCharSet().getRenderedSize() * 2 );
        ch.setText ( "+" );
        ch.setLocalTranslation (
            settings.getWidth() / 2 - guiFont.getCharSet().getRenderedSize() / 3 * 2,
            settings.getHeight() / 2 + ch.getLineHeight() / 2, 0 );
        guiNode.attachChild ( ch );
    }
}
