public class MainActivity extends FragmentActivity implements OnClickListener {
    private GoogleMap myMap;
    Polyline line;
    Context context;
    Location location;
    boolean check_provider_enabled = false;
    LatLng startLatLng = new LatLng ( 30.707104, 76.690749 );
    LatLng endLatLng = new LatLng ( 30.721419, 76.730017 );
    public void onCreate ( Bundle bd ) {
        super.onCreate ( bd );
        setContentView ( R.layout.activity_main );
        context = MainActivity.this;
        myMap = ( ( SupportMapFragment ) getSupportFragmentManager()
                  .findFragmentById ( R.id.map ) ).getMap();
        myMap.setMyLocationEnabled ( true );
        myMap.moveCamera ( CameraUpdateFactory.newLatLng ( startLatLng ) );
        myMap.animateCamera ( CameraUpdateFactory.zoomTo ( 12 ) );
        myMap.getUiSettings().setZoomControlsEnabled ( false );
        LocationManager service = ( LocationManager ) getSystemService ( LOCATION_SERVICE );
        boolean enabled = service.isProviderEnabled ( LocationManager.GPS_PROVIDER );
        location = service.getLastKnownLocation ( LocationManager.GPS_PROVIDER );
        if ( !enabled ) {
            Intent intent = new Intent ( Settings.ACTION_LOCATION_SOURCE_SETTINGS );
            startActivity ( intent );
            Toast.makeText ( getApplicationContext(), "Enable GPS servcies to use this app.", Toast.LENGTH_LONG ).show();
        } else {
            try {
                final LatLng PERTH = new LatLng ( -31.90, 115.86 );
                Marker perth = myMap.addMarker ( new MarkerOptions()
                                                 .position ( PERTH )
                                                 .anchor ( ( float ) 0.5, ( float ) 0.5 )
                                                 .rotation ( ( float ) 90.0 ) );
                String urlTopass = makeURL ( startLatLng.latitude,
                                             startLatLng.longitude, endLatLng.latitude,
                                             endLatLng.longitude );
            } catch ( Exception e ) {
                e.printStackTrace();
            }
        }
    }
    private class connectAsyncTask extends AsyncTask<Void, Void, String> {
        private ProgressDialog progressDialog;
        String url;
        connectAsyncTask ( String urlPass ) {
            url = urlPass;
        }
        @Override
        protected void onPreExecute() {
            super.onPreExecute();
            progressDialog = new ProgressDialog ( context );
            progressDialog.setMessage ( "Fetching route, Please wait..." );
            progressDialog.setIndeterminate ( true );
            progressDialog.show();
        }
        @Override
        protected String doInBackground ( Void... params ) {
            JSONParser jParser = new JSONParser();
            String json = jParser.getJSONFromUrl ( url );
            return json;
        }
        @Override
        protected void onPostExecute ( String result ) {
            super.onPostExecute ( result );
            progressDialog.hide();
            if ( result != null ) {
                drawPath ( result );
            }
        }
    }
    public String makeURL ( double sourcelat, double sourcelog, double destlat,
                            double destlog ) {
        StringBuilder urlString = new StringBuilder();
        urlString.append ( "http://maps.googleapis.com/maps/api/directions/json" );
        urlString.append ( "?origin=" );
        urlString.append ( Double.toString ( sourcelat ) );
        urlString.append ( "," );
        urlString.append ( Double.toString ( sourcelog ) );
        urlString.append ( "&destination=" );
        urlString.append ( Double.toString ( destlat ) );
        urlString.append ( "," );
        urlString.append ( Double.toString ( destlog ) );
        urlString.append ( "&sensor=false&mode=driving&alternatives=true" );
        return urlString.toString();
    }
    public class JSONParser {
        InputStream is = null;
        JSONObject jObj = null;
        String json = "";
        public JSONParser() {
        }
        public String getJSONFromUrl ( String url ) {
            try {
                DefaultHttpClient httpClient = new DefaultHttpClient();
                HttpPost httpPost = new HttpPost ( url );
                HttpResponse httpResponse = httpClient.execute ( httpPost );
                HttpEntity httpEntity = httpResponse.getEntity();
                is = httpEntity.getContent();
            } catch ( UnsupportedEncodingException e ) {
                e.printStackTrace();
            } catch ( ClientProtocolException e ) {
                e.printStackTrace();
            } catch ( IOException e ) {
                e.printStackTrace();
            }
            try {
                BufferedReader reader = new BufferedReader (
                    new InputStreamReader ( is, "iso-8859-1" ), 8 );
                StringBuilder sb = new StringBuilder();
                String line = null;
                while ( ( line = reader.readLine() ) != null ) {
                    sb.append ( line + "\n" );
                }
                json = sb.toString();
                is.close();
            } catch ( Exception e ) {
                Log.e ( "Buffer Error", "Error converting result " + e.toString() );
            }
            return json;
        }
    }
    public void drawPath ( String result ) {
        if ( line != null ) {
            myMap.clear();
        }
        myMap.addMarker ( new MarkerOptions().position ( endLatLng ).icon (
                              BitmapDescriptorFactory.fromResource ( R.drawable.ic_launcher ) ) );
        myMap.addMarker ( new MarkerOptions().position ( startLatLng ).icon (
                              BitmapDescriptorFactory.fromResource ( R.drawable.ic_launcher ) ) );
        try {
            final JSONObject json = new JSONObject ( result );
            JSONArray routeArray = json.getJSONArray ( "routes" );
            JSONObject routes = routeArray.getJSONObject ( 0 );
            JSONObject overviewPolylines = routes
                                           .getJSONObject ( "overview_polyline" );
            String encodedString = overviewPolylines.getString ( "points" );
            List<LatLng> list = decodePoly ( encodedString );
            PolylineOptions options = new PolylineOptions().width ( 5 ).color ( Color.BLUE ).geodesic ( true );
            for ( int z = 0; z < list.size(); z++ ) {
                LatLng point = list.get ( z );
                options.add ( point );
            }
            line = myMap.addPolyline ( options );
        } catch ( Exception e ) {
            e.printStackTrace();
        }
    }
    private List<LatLng> decodePoly ( String encoded ) {
        List<LatLng> poly = new ArrayList<LatLng>();
        int index = 0, len = encoded.length();
        int lat = 0, lng = 0;
        while ( index < len ) {
            int b, shift = 0, result = 0;
            do {
                b = encoded.charAt ( index++ ) - 63;
                result |= ( b & 0x1f ) << shift;
                shift += 5;
            } while ( b >= 0x20 );
            int dlat = ( ( result & 1 ) != 0 ? ~ ( result >> 1 ) : ( result >> 1 ) );
            lat += dlat;
            shift = 0;
            result = 0;
            do {
                b = encoded.charAt ( index++ ) - 63;
                result |= ( b & 0x1f ) << shift;
                shift += 5;
            } while ( b >= 0x20 );
            int dlng = ( ( result & 1 ) != 0 ? ~ ( result >> 1 ) : ( result >> 1 ) );
            lng += dlng;
            LatLng p = new LatLng ( ( ( ( double ) lat / 1E5 ) ),
                                    ( ( ( double ) lng / 1E5 ) ) );
            poly.add ( p );
        }
        return poly;
    }
    @Override
    public void onClick ( View arg0 ) {
    }
}
