import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;
import java.io.File;
import javax.imageio.ImageIO;
import javax.swing.*;
public class Paint {
    public static void main ( String[] args ) {
        PaintWindow frame = new PaintWindow();
        frame.setDefaultCloseOperation ( JFrame.EXIT_ON_CLOSE );
        frame.setResizable ( false );
        frame.setVisible ( true );
    }
}
class PaintWindow extends JFrame {
    public PaintWindow() {
        setTitle ( " Paint" );
        setSize ( 700, 600 );
        panel = new JPanel();
        drawPad = new PadDraw();
        panel.setPreferredSize ( new Dimension ( 100, 68 ) );
        Container content = this.getContentPane();
        content.setLayout ( new BorderLayout() );
        content.add ( panel, BorderLayout.WEST );
        content.add ( drawPad, BorderLayout.CENTER );
        makeColorButton ( Color.MAGENTA );
        makeColorButton ( Color.BLUE );
        makeColorButton ( Color.CYAN );
        makeColorButton ( Color.GREEN );
        makeColorButton ( Color.YELLOW );
        makeColorButton ( Color.ORANGE );
        makeColorButton ( Color.RED );
        makeColorButton ( Color.PINK );
        makeColorButton ( Color.LIGHT_GRAY );
        makeColorButton ( Color.GRAY );
        makeColorButton ( Color.DARK_GRAY );
        makeColorButton ( Color.BLACK );
        JButton eraserButton = new JButton ( "Eraser" );
        eraserButton.addActionListener ( new ActionListener() {
            public void actionPerformed ( ActionEvent e ) {
                drawPad.changeColor ( Color.white );
            }
        } );
        panel.add ( eraserButton );
        JButton clearButton = new JButton ( "Clear" );
        clearButton.addActionListener ( new ActionListener() {
            public void actionPerformed ( ActionEvent e ) {
                drawPad.clear();
            }
        } );
        panel.add ( clearButton );
        JButton brush1Button = new JButton ( "Brush 1" );
        brush1Button.addActionListener ( new ActionListener() {
            public void actionPerformed ( ActionEvent e ) {
                PadDraw.stroke = 10;
            }
        } );
        panel.add ( brush1Button );
        JButton brush2Button = new JButton ( "Brush 2" );
        brush2Button.addActionListener ( new ActionListener() {
            public void actionPerformed ( ActionEvent e ) {
                PadDraw.stroke = 15;
            }
        } );
        panel.add ( brush2Button );
        JButton saveButton = new JButton ( "Save As" );
        saveButton.addActionListener ( new ActionListener() {
            public void actionPerformed ( ActionEvent e ) {
                BufferedImage image = new BufferedImage ( drawPad.getWidth(), drawPad.getHeight(), BufferedImage.TYPE_INT_RGB );
                try {
                    ImageIO.write ( image, "png", new File ( "C:/.../Desktop/Example.png" ) );
                } catch ( Exception e2 ) {
                    e2.printStackTrace();
                }
            }
        } );
        panel.add ( saveButton );
    }
    public void makeColorButton ( final Color color ) {
        JButton tempButton = new JButton();
        tempButton.setBackground ( color );
        tempButton.setPreferredSize ( new Dimension ( 20, 20 ) );
        panel.add ( tempButton );
        tempButton.addActionListener ( new ActionListener() {
            public void actionPerformed ( ActionEvent e ) {
                drawPad.changeColor ( color );
            }
        } );
    }
    private JPanel panel;
    private PadDraw drawPad;
}
class PadDraw extends JComponent {
    static int stroke = 5;
    Image image;
    Graphics2D graphics2D;
    int currentX, currentY, oldX, oldY;
    public PadDraw() {
        setDoubleBuffered ( false );
        addMouseListener ( new MouseAdapter() {
            public void mousePressed ( MouseEvent e ) {
                oldX = e.getX();
                oldY = e.getY();
            }
        } );
        addMouseMotionListener ( new MouseMotionAdapter() {
            public void mouseDragged ( MouseEvent e ) {
                currentX = e.getX();
                currentY = e.getY();
                int nBsize;
                nBsize = stroke;
                graphics2D.fillOval ( ( oldX - ( nBsize / 2 ) ), ( oldY - ( nBsize / 2 ) ), nBsize, nBsize );
                repaint();
                oldX = currentX;
                oldY = currentY;
            }
        } );
    }
    public void paintComponent ( Graphics g ) {
        if ( image == null ) {
            image = createImage ( getSize().width, getSize().height );
            graphics2D = ( Graphics2D ) image.getGraphics();
            graphics2D.setRenderingHint ( RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON );
            clear();
        }
        g.drawImage ( image, 0, 0, null );
    }
    public void clear() {
        graphics2D.setPaint ( Color.white );
        graphics2D.fillRect ( 0, 0, getSize().width, getSize().height );
        graphics2D.setPaint ( Color.black );
        repaint();
    }
    public void changeColor ( Color theColor ) {
        graphics2D.setPaint ( theColor );
        repaint();
    }
}
