private MessageSource messageSource;
@Autowired
public RestErrorHandler ( @Qualifier ( "messageSource" ) MessageSource messageSource ) {
    this.messageSource = messageSource;
}
@ExceptionHandler ( MethodArgumentNotValidException.class )
@ResponseStatus ( HttpStatus.BAD_REQUEST )
@ResponseBody
public ValidationError processValidationError ( MethodArgumentNotValidException ex ) {
    BindingResult result = ex.getBindingResult();
    List<FieldError> fieldErrors = result.getFieldErrors();
    return this.processFieldErrors ( fieldErrors );
}
private ValidationError processFieldErrors ( List<FieldError> fieldErrors ) {
    ValidationError dto = new ValidationError();
    for ( FieldError fieldError : fieldErrors ) {
        String localizedErrorMessage = this.resolveLocalizedErrorMessage ( fieldError );
        dto.getErrors().put ( fieldError.getField(), localizedErrorMessage );
    }
    return dto;
}
private String resolveLocalizedErrorMessage ( FieldError fieldError ) {
    Locale currentLocale = LocaleContextHolder.getLocale();
    String localizedErrorMessage = this.messageSource.getMessage ( fieldError, currentLocale );
    return localizedErrorMessage;
}
