import java.awt.*;
import javax.swing.*;
import java.awt.image.*;
import java.awt.event.*;
import java.util.concurrent.atomic.*;
class DoubleBuffer {
    public static void main ( String[] args ) {
        EventQueue.invokeLater ( new Runnable() {
            @Override
            public void run() {
                new DoubleBuffer();
            }
        } );
    }
    static final int WIDTH = 640, HEIGHT = 480;
    static final double COS_PI_D_4 = Math.cos ( Math.PI / 4.0 );
    static BufferedImage newCompatibleImage() {
        return GraphicsEnvironment
               .getLocalGraphicsEnvironment()
               .getDefaultScreenDevice()
               .getDefaultConfiguration()
               .createCompatibleImage (
                   WIDTH, HEIGHT, Transparency.TRANSLUCENT
               );
    }
    BufferedImage front = newCompatibleImage();
    BufferedImage back = newCompatibleImage();
    final JFrame frame = new JFrame();
    final JPanel panel = new JPanel() {
        @Override
        public void paintComponent ( Graphics g ) {
            super.paintComponent ( g );
            g.drawImage ( front, 0, 0, null );
        }
    };
    final MouseAdapter onClick = new MouseAdapter() {
        @Override
        public void mousePressed ( MouseEvent me ) {
            beginPaint ( me.getPoint() );
        }
    };
    final Object monitor = new Object();
    final AtomicBoolean painting = new AtomicBoolean();
    DoubleBuffer() {
        panel.setPreferredSize ( new Dimension ( WIDTH, HEIGHT ) );
        panel.setBackground ( Color.WHITE );
        panel.addMouseListener ( onClick );
        frame.setContentPane ( panel );
        frame.pack();
        frame.setResizable ( false );
        frame.setLocationRelativeTo ( null );
        frame.setDefaultCloseOperation ( JFrame.EXIT_ON_CLOSE );
        frame.setVisible ( true );
    }
    void beginPaint ( Point pt ) {
        if ( painting.compareAndSet ( false, true ) ) {
            new PaintTask ( pt ).execute();
        }
    }
    void swapBuffers() {
        BufferedImage swap = front;
        front = back;
        back = swap;
    }
    class PaintTask
        extends SwingWorker<Void, Void> {
        final Point pt;
        PaintTask ( Point pt ) {
            this.pt = pt;
        }
        @Override
        public Void doInBackground() {
            synchronized ( monitor ) {
                Graphics2D g2D = back.createGraphics();
                g2D.setRenderingHint (
                    RenderingHints.KEY_ANTIALIASING,
                    RenderingHints.VALUE_ANTIALIAS_ON
                );
                g2D.setRenderingHint (
                    RenderingHints.KEY_STROKE_CONTROL,
                    RenderingHints.VALUE_STROKE_PURE
                );
                g2D.setBackground ( new Color ( 0, true ) );
                g2D.clearRect ( 0, 0, WIDTH, HEIGHT );
                float depth = ( float ) (
                                  Math.pow ( 2.0, 7.0 + ( int ) ( Math.random() * 3.0 ) )
                              );
                int h = ( int ) ( Math.random() * depth );
                int r = 1;
                int c;
                do {
                    g2D.setColor ( new Color (
                                       Color.HSBtoRGB ( h / depth, 1f, 1f )
                                   ) );
                    int x = pt.x - r;
                    int y = pt.y - r;
                    int d = r * 2;
                    g2D.drawOval ( x, y, d, d );
                    r++;
                    h++;
                    c = ( int ) ( r * COS_PI_D_4 );
                } while (
                    0 <= pt.x - c || pt.x + c < WIDTH
                    || 0 <= pt.y - c || pt.y + c < HEIGHT
                );
                g2D.dispose();
                back.flush();
                return ( Void ) null;
            }
        }
        @Override
        public void done() {
            synchronized ( monitor ) {
                swapBuffers();
            }
            painting.set ( false );
            panel.repaint();
        }
    }
}
