import android.app.Activity;
import android.graphics.Color;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.os.Bundle;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.widget.RadioGroup;
import android.widget.RadioGroup.OnCheckedChangeListener;
import android.widget.Toast;
public class SensorTestActivity extends Activity implements SensorEventListener {
    private SensorManager sensorManager;
    private boolean color = false;
    private View view;
    private long lastUpdate;
    static boolean enableSensor = true;
    @Override
    public void onCreate ( Bundle savedInstanceState ) {
        super.onCreate ( savedInstanceState );
        requestWindowFeature ( Window.FEATURE_NO_TITLE );
        getWindow().setFlags ( WindowManager.LayoutParams.FLAG_FULLSCREEN,
                               WindowManager.LayoutParams.FLAG_FULLSCREEN );
        setContentView ( R.layout.main );
        view = findViewById ( R.id.label );
        view.setBackgroundColor ( Color.GREEN );
        sensorManager = ( SensorManager ) getSystemService ( SENSOR_SERVICE );
        sensorManager.registerListener ( this,
                                         sensorManager.getDefaultSensor ( Sensor.TYPE_ACCELEROMETER ),
                                         SensorManager.SENSOR_DELAY_NORMAL );
        lastUpdate = System.currentTimeMillis();
        RadioGroup sensorSettings = ( RadioGroup ) findViewById ( R.id.sensorSettings );
        sensorSettings.setOnCheckedChangeListener ( new OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged ( RadioGroup group, int checkedId ) {
                switch ( checkedId ) {
                case R.id.enable:
                    enableSensor = true;
                    break;
                case R.id.disable:
                    enableSensor = false;
                    break;
                }
            }
        } );
    }
    @Override
    public void onSensorChanged ( SensorEvent event ) {
        if ( event.sensor.getType() == Sensor.TYPE_ACCELEROMETER ) {
            if ( enableSensor ) {
                getAccelerometer ( event );
            }
        }
    }
    private void getAccelerometer ( SensorEvent event ) {
        float[] values = event.values;
        float x = values[0];
        float y = values[1];
        float z = values[2];
        float accelationSquareRoot = ( x * x + y * y + z * z )
                                     / ( SensorManager.GRAVITY_EARTH * SensorManager.GRAVITY_EARTH );
        long actualTime = System.currentTimeMillis();
        if ( accelationSquareRoot >= 2 ) {
            if ( actualTime - lastUpdate < 200 ) {
                return;
            }
            lastUpdate = actualTime;
            Toast.makeText ( this, "Device has shaken", Toast.LENGTH_SHORT )
            .show();
            if ( color ) {
                view.setBackgroundColor ( Color.GREEN );
            } else {
                view.setBackgroundColor ( Color.RED );
            }
            color = !color;
        }
    }
    @Override
    public void onAccuracyChanged ( Sensor sensor, int accuracy ) {
    }
    @Override
    protected void onResume() {
        super.onResume();
        sensorManager.registerListener ( this,
                                         sensorManager.getDefaultSensor ( Sensor.TYPE_ACCELEROMETER ),
                                         SensorManager.SENSOR_DELAY_NORMAL );
    }
    @Override
    protected void onPause() {
        super.onPause();
        sensorManager.unregisterListener ( this );
    }
}
