public class GPSManager extends Service implements android.location.LocationListener {
    protected static final String   TAG                             = GPSManager.class.getSimpleName();
    protected Context               mContext;
    protected static GPSManager     mInstance;
    protected static final long     MIN_DISTANCE_CHANGE_FOR_UPDATES = 0;
    protected static final long     MIN_TIME_BW_UPDATES             = 1000 * 30 * 1;
    protected LocationManager       mLocationManager;
    boolean                         mIsGPSEnabled                   = false;
    boolean                         mIsNetworkEnabled               = false;
    boolean                         mCanGetLocation                 = false;
    public Location                 mLocation;
    double                          mLatitude;
    double                          mLongitude;
    public GPSManager() {
        mContext = null;
    }
    public static GPSManager getInstance() {
        if ( mInstance == null ) {
            mInstance = new GPSManager();
        }
        return mInstance;
    }
    public void set_mContext ( Context context ) {
        this.mContext = context;
    }
    public Location get_mLocation ( Context context ) {
        mContext = context;
        try {
            mLocationManager = ( LocationManager ) mContext.getSystemService ( LOCATION_SERVICE );
            mIsGPSEnabled = mLocationManager.isProviderEnabled ( LocationManager.GPS_PROVIDER );
            mIsNetworkEnabled = mLocationManager.isProviderEnabled ( LocationManager.NETWORK_PROVIDER );
            if ( !mIsGPSEnabled && !mIsNetworkEnabled ) {
            } else {
                this.mCanGetLocation = true;
                if ( mIsNetworkEnabled ) {
                    mLocationManager.requestLocationUpdates ( LocationManager.NETWORK_PROVIDER, MIN_TIME_BW_UPDATES, MIN_DISTANCE_CHANGE_FOR_UPDATES, this );
                    Log.d ( "Network", "Network" );
                    if ( mLocationManager != null ) {
                        mLocation = mLocationManager.getLastKnownLocation ( LocationManager.NETWORK_PROVIDER );
                        if ( mLocation != null ) {
                            mLatitude = mLocation.getLatitude();
                            mLongitude = mLocation.getLongitude();
                        }
                    }
                }
                if ( mIsGPSEnabled ) {
                    if ( mLocation == null ) {
                        mLocationManager.requestLocationUpdates ( LocationManager.GPS_PROVIDER, MIN_TIME_BW_UPDATES, MIN_DISTANCE_CHANGE_FOR_UPDATES, this );
                        Log.d ( "GPS Enabled", "GPS Enabled" );
                        if ( mLocationManager != null ) {
                            mLocation = mLocationManager.getLastKnownLocation ( LocationManager.GPS_PROVIDER );
                            if ( mLocation != null ) {
                                mLatitude = mLocation.getLatitude();
                                mLongitude = mLocation.getLongitude();
                            }
                        }
                    }
                }
            }
        } catch ( Exception e ) {
        }
        return mLocation;
    }
    public void stopUsingGPS() {
        if ( mLocationManager != null ) {
            mLocationManager.removeUpdates ( GPSManager.this );
        }
    }
    public double get_mLatitude() {
        if ( mLocation != null ) {
            mLatitude = mLocation.getLatitude();
        }
        return mLatitude;
    }
    public double get_mLongitude() {
        if ( mLocation != null ) {
            mLongitude = mLocation.getLongitude();
        }
        return mLongitude;
    }
    public boolean canGetLocation() {
        return this.mCanGetLocation;
    }
    public void showSettingsAlert ( String title, String msg ) {
        AlertDialog.Builder alertDialog = new AlertDialog.Builder ( mContext );
        alertDialog.setTitle ( title );
        alertDialog.setMessage ( msg );
        alertDialog.setPositiveButton ( "Paramètres", new DialogInterface.OnClickListener() {
            public void onClick ( DialogInterface dialog, int which ) {
                Intent intent = new Intent ( Settings.ACTION_LOCATION_SOURCE_SETTINGS );
                mContext.startActivity ( intent );
            }
        } );
        alertDialog.setNegativeButton ( "Annuler", new DialogInterface.OnClickListener() {
            public void onClick ( DialogInterface dialog, int which ) {
                dialog.cancel();
            }
        } );
        alertDialog.show();
    }
    @Override
    public void onLocationChanged ( Location location ) {
    }
    @Override
    public void onProviderDisabled ( String provider ) {
    }
    @Override
    public void onProviderEnabled ( String provider ) {
    }
    @Override
    public void onStatusChanged ( String provider, int status, Bundle extras ) {
    }
    @Override
    public IBinder onBind ( Intent arg0 ) {
        return null;
    }
}
