public void onCreate ( Bundle savedInstanceState ) {
    super.onCreate ( savedInstanceState );
    setContentView ( R.layout.main );
    recipient = ( EditText ) this.findViewById ( R.id.toET );
    textMessage = ( EditText ) this.findViewById ( R.id.chatET );
    listview = ( ListView ) this.findViewById ( R.id.listMessages );
    setListAdapter();
    Button send = ( Button ) this.findViewById ( R.id.sendBtn );
    send.setOnClickListener ( new View.OnClickListener() {
        public void onClick ( View view ) {
            String to = recipient.getText().toString();
            String text = textMessage.getText().toString();
            Log.i ( "XMPPChatDemoActivity", "Sending text " + text + " to " + to );
            Message msg = new Message ( to, Message.Type.chat );
            msg.setBody ( text );
            if ( xmpp != null ) {
                xmpp.sendPacket ( msg );
                messages.add ( xmpp.getUser() + ":" );
                messages.add ( text );
                setListAdapter();
            }
        }
    } );
    connect();
}
public void setConnection ( XMPPConnection connection ) {
    this.xmpp = connection;
    if ( xmpp != null ) {
        PacketFilter filter = new MessageTypeFilter ( Message.Type.chat );
        xmpp.addPacketListener ( new PacketListener() {
            @Override
            public void processPacket ( Packet packet ) {
                Message message = ( Message ) packet;
                if ( message.getBody() != null ) {
                    String fromName = StringUtils.parseBareAddress ( message.getFrom() );
                    Log.i ( "XMPPChatDemoActivity", "Text Recieved " + message.getBody() + " from " + fromName );
                    messages.add ( fromName + ":" );
                    messages.add ( message.getBody() );
                    mHandler.post ( new Runnable() {
                        public void run() {
                            setListAdapter();
                        }
                    } );
                }
            }
        }, filter );
    }
}
private void setListAdapter() {
    ArrayAdapter<String> adapter = new ArrayAdapter<String> ( this,
            R.layout.listitem, messages );
    listview.setAdapter ( adapter );
}
public void connect() {
    final ProgressDialog dialog = ProgressDialog.show ( this,
                                  "Connecting...", "Please wait...", false );
    Thread t = new Thread ( new Runnable() {
        @Override
        public void run() {
            ConnectionConfiguration connConfig = new ConnectionConfiguration (
                HOST, PORT, SERVICE );
            XMPPConnection xmpp = new XMPPConnection ( connConfig );
            try {
                xmpp.connect();
                Log.i ( "XMPPChatDemoActivity",
                        "Connected to " + xmpp.getHost() );
            } catch ( XMPPException ex ) {
                Log.e ( "XMPPChatDemoActivity", "Failed to connect to "
                        + xmpp.getHost() );
                Log.e ( "XMPPChatDemoActivity", ex.toString() );
                setConnection ( null );
            }
            try {
                xmpp.login ( USERNAME, PASSWORD );
                Log.i ( "XMPPChatDemoActivity",
                        "Logged in as " + xmpp.getUser() );
                Presence presence = new Presence ( Presence.Type.available );
                xmpp.sendPacket ( presence );
                setConnection ( xmpp );
                Roster roster = xmpp.getRoster();
                Collection<RosterEntry> entries = roster.getEntries();
                for ( RosterEntry entry : entries ) {
                    Log.d ( "XMPPChatDemoActivity",
                            "--------------------------------------" );
                    Log.d ( "XMPPChatDemoActivity", "RosterEntry " + entry );
                    Log.d ( "XMPPChatDemoActivity",
                            "User: " + entry.getUser() );
                    Log.d ( "XMPPChatDemoActivity",
                            "Name: " + entry.getName() );
                    Log.d ( "XMPPChatDemoActivity",
                            "Status: " + entry.getStatus() );
                    Log.d ( "XMPPChatDemoActivity",
                            "Type: " + entry.getType() );
                    Presence entryPresence = roster.getPresence ( entry.getUser() );
                    Log.d ( "XMPPChatDemoActivity", "Presence Status: "
                            + entryPresence.getStatus() );
                    Log.d ( "XMPPChatDemoActivity", "Presence Type: "
                            + entryPresence.getType() );
                    Presence.Type type = entryPresence.getType();
                    if ( type == Presence.Type.available ) {
                        Log.d ( "XMPPChatDemoActivity", "Presence AVIALABLE" );
                    }
                    Log.d ( "XMPPChatDemoActivity", "Presence : "
                            + entryPresence );
                }
            } catch ( XMPPException ex ) {
                Log.e ( "XMPPChatDemoActivity", "Failed to log in as "
                        + USERNAME );
                Log.e ( "XMPPChatDemoActivity", ex.toString() );
                setConnection ( null );
            }
            dialog.dismiss();
        }
    } );
    t.start();
    dialog.show();
}
