public static float[] unpack (
    byte[] bytes,
    long[] transfer,
    float[] samples,
    int bvalid,
    AudioFormat fmt
) {
    if ( fmt.getEncoding() != AudioFormat.Encoding.PCM_SIGNED
            && fmt.getEncoding() != AudioFormat.Encoding.PCM_UNSIGNED ) {
        return samples;
    }
    final int bitsPerSample = fmt.getSampleSizeInBits();
    final int bytesPerSample = bitsPerSample / 8;
    final int normalBytes = normalBytesFromBits ( bitsPerSample );
    if ( fmt.isBigEndian() ) {
        for ( int i = 0, k = 0, b; i < bvalid; i += normalBytes, k++ ) {
            transfer[k] = 0L;
            int least = i + normalBytes - 1;
            for ( b = 0; b < normalBytes; b++ ) {
                transfer[k] |= ( bytes[least - b] & 0xffL ) << ( 8 * b );
            }
        }
    } else {
        for ( int i = 0, k = 0, b; i < bvalid; i += normalBytes, k++ ) {
            transfer[k] = 0L;
            for ( b = 0; b < normalBytes; b++ ) {
                transfer[k] |= ( bytes[i + b] & 0xffL ) << ( 8 * b );
            }
        }
    }
    final long fullScale = ( long ) Math.pow ( 2.0, bitsPerSample - 1 );
    if ( fmt.getEncoding() == AudioFormat.Encoding.PCM_SIGNED ) {
        final long signMask = -1L << bitsPerSample - 1L;
        for ( int i = 0; i < transfer.length; i++ ) {
            if ( ( transfer[i] & signMask ) != 0L ) {
                transfer[i] |= signMask;
            }
        }
    } else {
        for ( int i = 0; i < transfer.length; i++ ) {
            transfer[i] -= fullScale;
        }
    }
    for ( int i = 0; i < transfer.length; i++ ) {
        samples[i] = ( float ) transfer[i] / ( float ) fullScale;
    }
    return samples;
}
public static int normalBytesFromBits ( int bitsPerSample ) {
    return bitsPerSample + 7 >> 3;
}
