class SequenceListException extends Exception {
    SequenceListException() {
        super();
    }
    SequenceListException ( String s ) {
        super ( s );
    }
}
public class SequenceList {
    protected class Node {
        public Node ( Object o ) {
            this ( o, null, null );
        }
        public Node ( Object o, Node p, Node n ) {
            datum = o;
            prev = p;
            next = n;
        }
        protected Object datum;
        protected Node next;
        protected Node prev;
    }
    private Node listHead;
    private Node listTail;
    public SequenceList() {
        listHead = null;
        listTail = null;
    }
    public void insertFirst ( Object o ) {
        if ( listHead == null ) {
            listHead = new Node ( o );
            listTail = listHead;
        } else {
            listHead.prev = new Node ( o, null, listHead );
            listHead = listHead.prev;
        }
    }
    public void insertLast ( Object o ) {
        if ( listHead == null ) {
            listHead = new Node ( o );
            listTail = listHead;
        } else {
            listTail.next = new Node ( o, listTail, null );
            listTail = listTail.next;
        }
    }
    public void insert ( Object o, int index ) throws SequenceListException {
        if ( index < 0 ) {
            throw new SequenceListException ( "Indexed Element out of Range" );
        }
        if ( listHead == null ) {
            if ( index == 0 ) {
                listHead = new Node ( o );
                listTail = listHead;
            } else {
                throw new SequenceListException ( "Indexed element is out of range" );
            }
            return;
        }
        if ( index == 0 ) {
            insertFirst ( o );
            return;
        }
        Node nodePointer = listHead;
        int i = 1;
        while ( i < index ) {
            nodePointer = nodePointer.next;
            i += 1;
            if ( nodePointer == null ) {
                throw new SequenceListException ( "Indexed Element out of Range" );
            }
        }
        Node newNode = new Node ( o, nodePointer, nodePointer.next );
        if ( nodePointer.next != null ) {
            nodePointer.next.prev = newNode;
        }
        nodePointer.next = newNode;
        if ( nodePointer == listTail ) {
            listTail = newNode;
        }
    }
    public void deleteFirst() throws SequenceListException {
        if ( listHead == null ) {
            throw new SequenceListException ( "Sequence Underflow" );
        }
        if ( listHead.next == null ) {
            listHead = null;
            listTail = null;
            return;
        }
        listHead = listHead.next;
        listHead.prev = null;
    }
    public void deleteLast() throws SequenceListException {
        if ( listHead == null ) {
            throw new SequenceListException ( "Sequence Underflow" );
        }
        if ( listHead.next == null ) {
            listHead = null;
            listTail = null;
            return;
        }
        listTail = listTail.prev;
        listTail.next = null;
    }
    public void delete ( int index ) throws SequenceListException {
        if ( listHead == null ) {
            throw new SequenceListException ( "Sequence Underflow" );
        }
        if ( index < 0 ) {
            throw new SequenceListException ( "Indexed Element out of Range" );
        }
        if ( listHead.next == null ) {
            if ( index == 0 ) {
                listHead = null;
                listTail = null;
            } else {
                throw new SequenceListException ( "Indexed element is out of range." );
            }
            return;
        }
        if ( index == 0 ) {
            deleteFirst();
            return;
        }
        Node nodePointer = listHead;
        int i = 1;
        while ( i < index ) {
            nodePointer = nodePointer.next;
            i += 1;
            if ( nodePointer.next == null ) {
                throw new SequenceListException ( "Indexed Element out of Range" );
            }
        }
        if ( nodePointer.next == listTail ) {
            listTail = nodePointer;
        }
        if ( nodePointer.next.next != null ) {
            nodePointer.next.next.prev = nodePointer;
        }
        nodePointer.next = nodePointer.next.next;
    }
    public Object first() throws SequenceListException {
        if ( listHead != null ) {
            return listHead.datum;
        } else {
            throw new SequenceListException ( "Indexed Element out of Range" );
        }
    }
    public Object last() throws SequenceListException {
        if ( listTail != null ) {
            return listTail.datum;
        } else {
            throw new SequenceListException ( "Indexed Element out of Range" );
        }
    }
    public Object element ( int index ) throws SequenceListException {
        if ( index < 0 ) {
            throw new SequenceListException ( "Indexed Element out of Range" );
        }
        Node nodePointer = listHead;
        int i = 0;
        while ( i < index ) {
            if ( nodePointer.next == null ) {
                throw new SequenceListException ( "Indexed Element out of Range" );
            } else {
                nodePointer = nodePointer.next;
                i += 1;
            }
        }
        return nodePointer.datum;
    }
    public boolean empty() {
        return ( listHead == null );
    }
    public int size() {
        Node nodePointer = listHead;
        int size = 0;
        while ( nodePointer != null ) {
            size += 1;
            nodePointer = nodePointer.next;
        }
        return size;
    }
    public void clear() {
        listHead = null;
        listTail = null;
    }
    public Node lastNode() {
        return listTail;
    }
    public Node firstNode() {
        return listHead;
    }
}
