import java.awt.*;
import java.util.HashMap;
import java.util.Map;
import javax.swing.*;
class TwoColumnLayoutWithHeader {
    public static JComponent getTwoColumnLayout (
        JLabel[] labels,
        JComponent[] fields,
        boolean addMnemonics ) {
        if ( labels.length != fields.length ) {
            String s = labels.length + " labels supplied for "
                       + fields.length + " fields!";
            throw new IllegalArgumentException ( s );
        }
        JComponent panel = new JPanel();
        GroupLayout layout = new GroupLayout ( panel );
        panel.setLayout ( layout );
        layout.setAutoCreateGaps ( true );
        GroupLayout.SequentialGroup hGroup = layout.createSequentialGroup();
        GroupLayout.Group yLabelGroup = layout.createParallelGroup ( GroupLayout.Alignment.TRAILING );
        hGroup.addGroup ( yLabelGroup );
        GroupLayout.Group yFieldGroup = layout.createParallelGroup();
        hGroup.addGroup ( yFieldGroup );
        layout.setHorizontalGroup ( hGroup );
        GroupLayout.SequentialGroup vGroup = layout.createSequentialGroup();
        layout.setVerticalGroup ( vGroup );
        int p = GroupLayout.PREFERRED_SIZE;
        for ( JLabel label : labels ) {
            yLabelGroup.addComponent ( label );
        }
        for ( Component field : fields ) {
            yFieldGroup.addComponent ( field, p, p, p );
        }
        for ( int ii = 0; ii < labels.length; ii++ ) {
            vGroup.addGroup ( layout.createParallelGroup().
                              addComponent ( labels[ii] ).
                              addComponent ( fields[ii], p, p, p ) );
        }
        if ( addMnemonics ) {
            addMnemonics ( labels, fields );
        }
        return panel;
    }
    private final static void addMnemonics (
        JLabel[] labels,
        JComponent[] fields ) {
        Map<Character, Object> m = new HashMap<Character, Object>();
        for ( int ii = 0; ii < labels.length; ii++ ) {
            labels[ii].setLabelFor ( fields[ii] );
            String lwr = labels[ii].getText().toLowerCase();
            for ( int jj = 0; jj < lwr.length(); jj++ ) {
                char ch = lwr.charAt ( jj );
                if ( m.get ( ch ) == null && Character.isLetterOrDigit ( ch ) ) {
                    m.put ( ch, ch );
                    labels[ii].setDisplayedMnemonic ( ch );
                    break;
                }
            }
        }
    }
    public static JComponent getTwoColumnLayout (
        String[] labelStrings,
        JComponent[] fields ) {
        JLabel[] labels = new JLabel[labelStrings.length];
        for ( int ii = 0; ii < labels.length; ii++ ) {
            labels[ii] = new JLabel ( labelStrings[ii] );
        }
        return getTwoColumnLayout ( labels, fields );
    }
    public static JComponent getTwoColumnLayout (
        JLabel[] labels,
        JComponent[] fields ) {
        return getTwoColumnLayout ( labels, fields, true );
    }
    public static String getProperty ( String name ) {
        return name + ": \t"
               + System.getProperty ( name )
               + System.getProperty ( "line.separator" );
    }
    public static void main ( String[] args ) {
        Runnable r = new Runnable() {
            @Override
            public void run() {
                JComponent[] components = {
                    new JTextField ( 15 ),
                    new JTextField ( 10 ),
                    new JTextField ( 8 ),
                    new JSpinner ( new SpinnerNumberModel ( 1, 0, 10, 1 ) ),
                    new JSpinner ( new SpinnerNumberModel ( 9.95, 0d, 100d, .01 ) ),
                    new JSpinner ( new SpinnerNumberModel ( 9.95, 0d, 1000d, .01 ) ),
                    new JSpinner ( new SpinnerNumberModel ( 9.95, 0d, 100d, .01 ) ),
                    new JSpinner ( new SpinnerNumberModel ( 9.95, 0d, 1000d, .01 ) ),
                    new JSpinner ( new SpinnerNumberModel ( 9.95, 0d, 100d, .01 ) ),
                    new JSpinner ( new SpinnerNumberModel ( 9.95, 0d, 1000d, .01 ) )
                };
                String[] labels = {
                    "Product Name:",
                    "Product Unit Name:",
                    "Purchase Date:",
                    "Quantity:",
                    "Price Per Unit:",
                    "Total Price:",
                    "Discount:",
                    "Total:",
                    "VAT:",
                    "Grand Total:"
                };
                JComponent labelsAndFields = getTwoColumnLayout ( labels, components );
                JComponent orderForm = new JPanel ( new BorderLayout ( 5, 5 ) );
                orderForm.add ( new JLabel ( "Purchase Form", SwingConstants.CENTER ),
                                BorderLayout.PAGE_START );
                orderForm.add ( labelsAndFields, BorderLayout.CENTER );
                JOptionPane.showMessageDialog ( null, orderForm );
            }
        };
        SwingUtilities.invokeLater ( r );
    }
}
