public static void validate ( XmlObject xmlObject, SchemaTypeSystem schemas, QName xmlObjectType ) throws XmlException {
    Node node = xmlObject.getDomNode();
    String elemName = node.getLocalName();
    String elemNamespace = node.getNamespaceURI();
    String typeName = xmlObjectType.getLocalPart();
    String typeNamespace = xmlObjectType.getNamespaceURI();
    String schemasKey = elemName + "@" + elemNamespace + "," + typeName + "@" + typeNamespace;
    SchemaTypeSystem loader = schemas;
    SchemaGlobalElement schemaElem = loader.findElement ( new QName ( elemNamespace, elemName ) );
    if ( schemaElem != null ) {
        QName elemQName = schemaElem.getType().getName();
        if ( !elemQName.getLocalPart().equals ( typeName ) || !elemQName.getNamespaceURI().equals ( typeNamespace ) )
            throw new IllegalArgumentException ( "Requested type " + typeName + "@" + typeNamespace
                                                 + " of xmlObject is different from its actual type " + elemQName.getLocalPart() + "@" + elemQName.getNamespaceURI() + " in provided schema(s)" );
    } else {
        String helperSchema = ""
                              + "<xs:schema attributeFormDefault=\"unqualified\" elementFormDefault=\"qualified\"\n"
                              + "  targetNamespace=\"" + elemNamespace + "\"\n"
                              + "  xmlns=\"" + elemNamespace + "\"\n"
                              + "  xmlns:t=\"" + typeNamespace + "\"\n"
                              + "  xmlns:xs=\"http://www.w3.org/2001/XMLSchema\">\n"
                              + "\n"
                              + "    <xs:element name=\"" + elemName + "\" type=\"t:" + typeName + "\"/>\n"
                              + "\n"
                              + "</xs:schema>\n";
        loader = XmlBeans.compileXsd ( schemas,
                                       new XmlObject[] {
                                           XmlObject.Factory.parse ( helperSchema, new XmlOptions().setLoadLineNumbers().setLoadMessageDigest()
                                                   .setDocumentSourceName ( schemasKey ) )
                                       }, null,
                                       new XmlOptions().setErrorListener ( null ).setCompileDownloadUrls().setCompileNoPvrRule() );
    }
    validate ( xmlObject, loader );
}
