public boolean authenticate ( Request request,
                              Response response,
                              LoginConfig config )
throws IOException {
    Principal principal = request.getUserPrincipal();
    String ssoId = ( String ) request.getNote ( Constants.REQ_SSOID_NOTE );
    if ( principal != null ) {
        if ( log.isDebugEnabled() ) {
            log.debug ( "Already authenticated '" + principal.getName() + "'" );
        }
        if ( ssoId != null ) {
            associate ( ssoId, request.getSessionInternal ( true ) );
        }
        return ( true );
    }
    if ( ssoId != null ) {
        if ( log.isDebugEnabled() )
            log.debug ( "SSO Id " + ssoId + " set; attempting " +
                        "reauthentication" );
        if ( reauthenticateFromSSO ( ssoId, request ) ) {
            return true;
        }
    }
    String username = null;
    String password = null;
    MessageBytes authorization =
        request.getCoyoteRequest().getMimeHeaders()
        .getValue ( "authorization" );
    if ( authorization != null ) {
        authorization.toBytes();
        ByteChunk authorizationBC = authorization.getByteChunk();
        if ( authorizationBC.startsWithIgnoreCase ( "basic ", 0 ) ) {
            authorizationBC.setOffset ( authorizationBC.getOffset() + 6 );
            CharChunk authorizationCC = authorization.getCharChunk();
            Base64.decode ( authorizationBC, authorizationCC );
            int colon = authorizationCC.indexOf ( ':' );
            if ( colon < 0 ) {
                username = authorizationCC.toString();
            } else {
                char[] buf = authorizationCC.getBuffer();
                username = new String ( buf, 0, colon );
                password = new String ( buf, colon + 1,
                                        authorizationCC.getEnd() - colon - 1 );
            }
            authorizationBC.setOffset ( authorizationBC.getOffset() - 6 );
        }
        principal = context.getRealm().authenticate ( username, password );
        if ( principal != null ) {
            register ( request, response, principal, Constants.BASIC_METHOD,
                       username, password );
            return ( true );
        }
    }
    MessageBytes authenticate =
        response.getCoyoteResponse().getMimeHeaders()
        .addValue ( AUTHENTICATE_BYTES, 0, AUTHENTICATE_BYTES.length );
    CharChunk authenticateCC = authenticate.getCharChunk();
    authenticateCC.append ( "Basic realm=\"" );
    if ( config.getRealmName() == null ) {
        authenticateCC.append ( request.getServerName() );
        authenticateCC.append ( ':' );
        authenticateCC.append ( Integer.toString ( request.getServerPort() ) );
    } else {
        authenticateCC.append ( config.getRealmName() );
    }
    authenticateCC.append ( '\"' );
    authenticate.toChars();
    response.sendError ( HttpServletResponse.SC_UNAUTHORIZED );
    return ( false );
}
