import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridLayout;
import java.awt.image.BufferedImage;
import java.util.Random;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;
public class AlphaMapping {
    public static void main ( String[] args ) {
        SwingUtilities.invokeLater ( new Runnable() {
            @Override
            public void run() {
                createAndShowGUI();
            }
        } );
    }
    private static BufferedImage createImage ( int w, int h ) {
        BufferedImage image = new BufferedImage ( w, h,
                BufferedImage.TYPE_INT_ARGB );
        Graphics g = image.getGraphics();
        Random random = new Random ( 0 );
        for ( int i = 0; i < 200; i++ ) {
            int x = random.nextInt ( w );
            int y = random.nextInt ( h );
            Color c = new Color (
                random.nextInt ( 255 ),
                random.nextInt ( 255 ),
                random.nextInt ( 255 ) );
            g.setColor ( c );
            g.fillOval ( x - 20, y - 20, 40, 40 );
        }
        g.dispose();
        return image;
    }
    private static BufferedImage createMask ( int w, int h ) {
        BufferedImage image = new BufferedImage ( w, h,
                BufferedImage.TYPE_INT_ARGB );
        Graphics2D g = image.createGraphics();
        g.setColor ( new Color ( 0, 0, 0, 255 ) );
        g.fillRect ( 0, 0, w, h );
        g.setComposite ( AlphaComposite.DstAtop );
        g.setColor ( new Color ( 0, 0, 0, 0 ) );
        g.fillRect ( w / 3, h / 3, w / 3, h / 3 );
        g.dispose();
        return image;
    }
    private static BufferedImage createMaskedImage (
        BufferedImage image, BufferedImage mask ) {
        BufferedImage maskedImage = new BufferedImage (
            image.getWidth(), image.getHeight(),
            BufferedImage.TYPE_INT_ARGB );
        Graphics2D g = maskedImage.createGraphics();
        g.drawImage ( image, 0, 0, null );
        g.setComposite ( AlphaComposite.DstOut );
        g.drawImage ( mask, 0, 0, null );
        g.dispose();
        return maskedImage;
    }
    private static void createAndShowGUI() {
        JFrame f = new JFrame();
        f.setDefaultCloseOperation ( JFrame.EXIT_ON_CLOSE );
        int w = 400;
        int h = 400;
        BufferedImage image = createImage ( w, h );
        BufferedImage mask = createMask ( w, h );
        BufferedImage maskedImage = createMaskedImage ( image, mask );
        JPanel panel = new JPanel ( new GridLayout ( 1, 0 ) );
        panel.add ( new JLabel ( new ImageIcon ( image ) ) );
        panel.add ( new JLabel ( new ImageIcon ( mask ) ) );
        panel.add ( new JLabel ( new ImageIcon ( maskedImage ) ) );
        f.getContentPane().add ( panel );
        f.pack();
        f.setLocationRelativeTo ( null );
        f.setVisible ( true );
    }
}
