public class AddToDoActivity extends Activity {
    private static final int SEVEN_DAYS = 604800000;
    private static final String TAG = "Lab-UserInterface";
    private static String timeString;
    private static String dateString;
    private static TextView dateView;
    private static TextView timeView;
    private Date mDate;
    private RadioGroup mPriorityRadioGroup;
    private RadioGroup mStatusRadioGroup;
    private EditText mTitleText;
    private RadioButton mDefaultStatusButton;
    private RadioButton mDefaultPriorityButton;
    @Override
    protected void onCreate ( Bundle savedInstanceState ) {
        super.onCreate ( savedInstanceState );
        setContentView ( R.layout.add_todo );
        mTitleText = ( EditText ) findViewById ( R.id.title );
        mDefaultStatusButton = ( RadioButton ) findViewById ( R.id.statusNotDone );
        mDefaultPriorityButton = ( RadioButton ) findViewById ( R.id.medPriority );
        mPriorityRadioGroup = ( RadioGroup ) findViewById ( R.id.priorityGroup );
        mStatusRadioGroup = ( RadioGroup ) findViewById ( R.id.statusGroup );
        dateView = ( TextView ) findViewById ( R.id.date );
        timeView = ( TextView ) findViewById ( R.id.time );
        setDefaultDateTime();
        final Button datePickerButton = ( Button ) findViewById ( R.id.date_picker_button );
        datePickerButton.setOnClickListener ( new OnClickListener() {
            @Override
            public void onClick ( View v ) {
                showDatePickerDialog();
            }
        } );
        final Button timePickerButton = ( Button ) findViewById ( R.id.time_picker_button );
        timePickerButton.setOnClickListener ( new OnClickListener() {
            @Override
            public void onClick ( View v ) {
                showTimePickerDialog();
            }
        } );
        final Button cancelButton = ( Button ) findViewById ( R.id.cancelButton );
        cancelButton.setOnClickListener ( new OnClickListener() {
            @Override
            public void onClick ( View v ) {
                log ( "Entered cancelButton.OnClickListener.onClick()" );
                finish();
            }
        } );
        final Button resetButton = ( Button ) findViewById ( R.id.resetButton );
        resetButton.setOnClickListener ( new OnClickListener() {
            @Override
            public void onClick ( View v ) {
                log ( "Entered resetButton.OnClickListener.onClick()" );
                setDefaultDateTime();
                mTitleText.setText ( "" );
                mDefaultStatusButton.setChecked ( true );
                mDefaultPriorityButton.setChecked ( true );
            }
        } );
        final Button submitButton = ( Button ) findViewById ( R.id.submitButton );
        submitButton.setOnClickListener ( new OnClickListener() {
            @Override
            public void onClick ( View v ) {
                log ( "Entered submitButton.OnClickListener.onClick()" );
                Priority priority = getPriority();
                Status status = getStatus();
                String titleString = mTitleText.getText().toString();
                String fullDate = dateString + " " + timeString;
                Intent data = new Intent();
                ToDoItem.packageIntent ( data, titleString, priority, status, fullDate );
                setResult ( Activity.RESULT_OK, data );
                finish();
            }
        } );
    }
    private void setDefaultDateTime() {
        mDate = new Date();
        mDate = new Date ( mDate.getTime() + SEVEN_DAYS );
        Calendar c = Calendar.getInstance();
        c.setTime ( mDate );
        setDateString ( c.get ( Calendar.DAY_OF_MONTH ), c.get ( Calendar.MONTH ),
                        c.get ( Calendar.YEAR ) );
        dateView.setText ( dateString );
        setTimeString ( c.get ( Calendar.HOUR_OF_DAY ), c.get ( Calendar.MINUTE ),
                        c.get ( Calendar.MILLISECOND ) );
        timeView.setText ( timeString );
    }
    private static void setDateString ( int dayOfMonth, int monthOfYear, int year ) {
        monthOfYear++;
        String mon = "" + monthOfYear;
        String day = "" + dayOfMonth;
        if ( monthOfYear < 10 ) {
            mon = "0" + monthOfYear;
        }
        if ( dayOfMonth < 10 ) {
            day = "0" + dayOfMonth;
        }
        dateString = year + "-" + mon + "-" + day;
    }
    private static void setTimeString ( int hourOfDay, int minute, int mili ) {
        String hour = "" + hourOfDay;
        String min = "" + minute;
        if ( hourOfDay < 10 ) {
            hour = "0" + hourOfDay;
        }
        if ( minute < 10 ) {
            min = "0" + minute;
        }
        timeString = hour + ":" + min + ":00";
    }
    private Priority getPriority() {
        switch ( mPriorityRadioGroup.getCheckedRadioButtonId() ) {
        case R.id.lowPriority: {
            return Priority.LOW;
        }
        case R.id.highPriority: {
            return Priority.HIGH;
        }
        default: {
            return Priority.MED;
        }
        }
    }
    private Status getStatus() {
        switch ( mStatusRadioGroup.getCheckedRadioButtonId() ) {
        case R.id.statusDone: {
            return Status.DONE;
        }
        default: {
            return Status.NOTDONE;
        }
        }
    }
    public static class DatePickerFragment extends DialogFragment implements
        DatePickerDialog.OnDateSetListener {
        @Override
        public Dialog onCreateDialog ( Bundle savedInstanceState ) {
            final Calendar c = Calendar.getInstance();
            int day = c.get ( Calendar.DAY_OF_MONTH );
            int month = c.get ( Calendar.MONTH );
            int year = c.get ( Calendar.YEAR );
            return new DatePickerDialog ( getActivity(), this, year, month, day );
        }
        @Override
        public void onDateSet ( DatePicker view, int dayOfMonth, int monthOfYear,
                                int year ) {
            setDateString ( dayOfMonth, monthOfYear, year );
            dateView.setText ( dateString );
        }
    }
    public static class TimePickerFragment extends DialogFragment implements
        TimePickerDialog.OnTimeSetListener {
        @Override
        public Dialog onCreateDialog ( Bundle savedInstanceState ) {
            final Calendar c = Calendar.getInstance();
            int hour = c.get ( Calendar.HOUR_OF_DAY );
            int minute = c.get ( Calendar.MINUTE );
            return new TimePickerDialog ( getActivity(), this, hour, minute,
                                          true );
        }
        public void onTimeSet ( TimePicker view, int hourOfDay, int minute ) {
            setTimeString ( hourOfDay, minute, 0 );
            timeView.setText ( timeString );
        }
    }
    private void showDatePickerDialog() {
        DialogFragment newFragment = new DatePickerFragment();
        newFragment.show ( getFragmentManager(), "datePicker" );
    }
    private void showTimePickerDialog() {
        DialogFragment newFragment = new TimePickerFragment();
        newFragment.show ( getFragmentManager(), "timePicker" );
    }
    private void log ( String msg ) {
        try {
            Thread.sleep ( 500 );
        } catch ( InterruptedException e ) {
            e.printStackTrace();
        }
        Log.i ( TAG, msg );
    }
}
