import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.EventListener;
import java.util.EventObject;
import java.util.HashMap;
import java.util.HashSet;
import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.JTree;
import javax.swing.event.EventListenerList;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeSelectionModel;
import javax.swing.tree.TreeCellRenderer;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
public class JCheckBoxTree extends JTree {
    private static final long serialVersionUID = -4194122328392241790L;
    JCheckBoxTree selfPointer = this;
    private class CheckedNode {
        boolean isSelected;
        boolean hasChildren;
        boolean allChildrenSelected;
        public CheckedNode ( boolean isSelected_, boolean hasChildren_, boolean allChildrenSelected_ ) {
            isSelected = isSelected_;
            hasChildren = hasChildren_;
            allChildrenSelected = allChildrenSelected_;
        }
    }
    HashMap<TreePath, CheckedNode> nodesCheckingState;
    HashSet<TreePath> checkedPaths = new HashSet<TreePath>();
    protected EventListenerList listenerList = new EventListenerList();
    public class CheckChangeEvent extends EventObject {
        private static final long serialVersionUID = -8100230309044193368L;
        public CheckChangeEvent ( Object source ) {
            super ( source );
        }
    }
    public interface CheckChangeEventListener extends EventListener {
        public void checkStateChanged ( CheckChangeEvent event );
    }
    public void addCheckChangeEventListener ( CheckChangeEventListener listener ) {
        listenerList.add ( CheckChangeEventListener.class, listener );
    }
    public void removeCheckChangeEventListener ( CheckChangeEventListener listener ) {
        listenerList.remove ( CheckChangeEventListener.class, listener );
    }
    void fireCheckChangeEvent ( CheckChangeEvent evt ) {
        Object[] listeners = listenerList.getListenerList();
        for ( int i = 0; i < listeners.length; i++ ) {
            if ( listeners[i] == CheckChangeEventListener.class ) {
                ( ( CheckChangeEventListener ) listeners[i + 1] ).checkStateChanged ( evt );
            }
        }
    }
    public void setModel ( TreeModel newModel ) {
        super.setModel ( newModel );
        resetCheckingState();
    }
    public TreePath[] getCheckedPaths() {
        return checkedPaths.toArray ( new TreePath[checkedPaths.size()] );
    }
    public boolean isSelectedPartially ( TreePath path ) {
        CheckedNode cn = nodesCheckingState.get ( path );
        return cn.isSelected && cn.hasChildren && !cn.allChildrenSelected;
    }
    private void resetCheckingState() {
        nodesCheckingState = new HashMap<TreePath, CheckedNode>();
        checkedPaths = new HashSet<TreePath>();
        DefaultMutableTreeNode node = ( DefaultMutableTreeNode ) getModel().getRoot();
        if ( node == null ) {
            return;
        }
        addSubtreeToCheckingStateTracking ( node );
    }
    private void addSubtreeToCheckingStateTracking ( DefaultMutableTreeNode node ) {
        TreeNode[] path = node.getPath();
        TreePath tp = new TreePath ( path );
        CheckedNode cn = new CheckedNode ( false, node.getChildCount() > 0, false );
        nodesCheckingState.put ( tp, cn );
        for ( int i = 0 ; i < node.getChildCount() ; i++ ) {
            addSubtreeToCheckingStateTracking ( ( DefaultMutableTreeNode ) tp.pathByAddingChild ( node.getChildAt ( i ) ).getLastPathComponent() );
        }
    }
    private class CheckBoxCellRenderer extends JPanel implements TreeCellRenderer {
        private static final long serialVersionUID = -7341833835878991719L;
        JCheckBox checkBox;
        public CheckBoxCellRenderer() {
            super();
            this.setLayout ( new BorderLayout() );
            checkBox = new JCheckBox();
            add ( checkBox, BorderLayout.CENTER );
            setOpaque ( false );
        }
        @Override
        public Component getTreeCellRendererComponent ( JTree tree, Object value,
                boolean selected, boolean expanded, boolean leaf, int row,
                boolean hasFocus ) {
            DefaultMutableTreeNode node = ( DefaultMutableTreeNode ) value;
            Object obj = node.getUserObject();
            TreePath tp = new TreePath ( node.getPath() );
            CheckedNode cn = nodesCheckingState.get ( tp );
            if ( cn == null ) {
                return this;
            }
            checkBox.setSelected ( cn.isSelected );
            checkBox.setText ( obj.toString() );
            checkBox.setOpaque ( cn.isSelected && cn.hasChildren && ! cn.allChildrenSelected );
            return this;
        }
    }
    public JCheckBoxTree() {
        super();
        this.setToggleClickCount ( 0 );
        CheckBoxCellRenderer cellRenderer = new CheckBoxCellRenderer();
        this.setCellRenderer ( cellRenderer );
        DefaultTreeSelectionModel dtsm = new DefaultTreeSelectionModel() {
            private static final long serialVersionUID = -8190634240451667286L;
            public void setSelectionPath ( TreePath path ) {
            }
            public void addSelectionPath ( TreePath path ) {
            }
            public void removeSelectionPath ( TreePath path ) {
            }
            public void setSelectionPaths ( TreePath[] pPaths ) {
            }
        };
        this.addMouseListener ( new MouseListener() {
            public void mouseClicked ( MouseEvent arg0 ) {
                TreePath tp = selfPointer.getPathForLocation ( arg0.getX(), arg0.getY() );
                if ( tp == null ) {
                    return;
                }
                boolean checkMode = ! nodesCheckingState.get ( tp ).isSelected;
                checkSubTree ( tp, checkMode );
                updatePredecessorsWithCheckMode ( tp, checkMode );
                fireCheckChangeEvent ( new CheckChangeEvent ( new Object() ) );
                selfPointer.repaint();
            }
            public void mouseEntered ( MouseEvent arg0 ) {
            }
            public void mouseExited ( MouseEvent arg0 ) {
            }
            public void mousePressed ( MouseEvent arg0 ) {
            }
            public void mouseReleased ( MouseEvent arg0 ) {
            }
        } );
        this.setSelectionModel ( dtsm );
    }
    protected void updatePredecessorsWithCheckMode ( TreePath tp, boolean check ) {
        TreePath parentPath = tp.getParentPath();
        if ( parentPath == null ) {
            return;
        }
        CheckedNode parentCheckedNode = nodesCheckingState.get ( parentPath );
        DefaultMutableTreeNode parentNode = ( DefaultMutableTreeNode ) parentPath.getLastPathComponent();
        parentCheckedNode.allChildrenSelected = true;
        parentCheckedNode.isSelected = false;
        for ( int i = 0 ; i < parentNode.getChildCount() ; i++ ) {
            TreePath childPath = parentPath.pathByAddingChild ( parentNode.getChildAt ( i ) );
            CheckedNode childCheckedNode = nodesCheckingState.get ( childPath );
            if ( ! childCheckedNode.allChildrenSelected ) {
                parentCheckedNode.allChildrenSelected = false;
            }
            if ( childCheckedNode.isSelected ) {
                parentCheckedNode.isSelected = true;
            }
        }
        if ( parentCheckedNode.isSelected ) {
            checkedPaths.add ( parentPath );
        } else {
            checkedPaths.remove ( parentPath );
        }
        updatePredecessorsWithCheckMode ( parentPath, check );
    }
    protected void checkSubTree ( TreePath tp, boolean check ) {
        CheckedNode cn = nodesCheckingState.get ( tp );
        cn.isSelected = check;
        DefaultMutableTreeNode node = ( DefaultMutableTreeNode ) tp.getLastPathComponent();
        for ( int i = 0 ; i < node.getChildCount() ; i++ ) {
            checkSubTree ( tp.pathByAddingChild ( node.getChildAt ( i ) ), check );
        }
        cn.allChildrenSelected = check;
        if ( check ) {
            checkedPaths.add ( tp );
        } else {
            checkedPaths.remove ( tp );
        }
    }
}
