public class FadingImageView extends ImageView {
    private FadeSide mFadeSide;
    private Context c;
    public enum FadeSide {
        RIGHT_SIDE, LEFT_SIDE
    }
    public FadingImageView ( Context c, AttributeSet attrs, int defStyle ) {
        super ( c, attrs, defStyle );
        this.c = c;
        init();
    }
    public FadingImageView ( Context c, AttributeSet attrs ) {
        super ( c, attrs );
        this.c = c;
        init();
    }
    public FadingImageView ( Context c ) {
        super ( c );
        this.c = c;
        init();
    }
    private void init() {
        this.setHorizontalFadingEdgeEnabled ( true );
        this.setEdgeLength ( 14 );
        this.setFadeDirection ( FadeSide.RIGHT_SIDE );
    }
    public void setFadeDirection ( FadeSide side ) {
        this.mFadeSide = side;
    }
    public void setEdgeLength ( int length ) {
        this.setFadingEdgeLength ( getPixels ( length ) );
    }
    @Override
    protected float getLeftFadingEdgeStrength() {
        return mFadeSide.equals ( FadeSide.LEFT_SIDE ) ? 1.0f : 0.0f;
    }
    @Override
    protected float getRightFadingEdgeStrength() {
        return mFadeSide.equals ( FadeSide.RIGHT_SIDE ) ? 1.0f : 0.0f;
    }
    @Override
    public boolean hasOverlappingRendering() {
        return true;
    }
    @Override
    public boolean onSetAlpha ( int alpha ) {
        return false;
    }
    private int getPixels ( int dipValue ) {
        Resources r = c.getResources();
        return ( int ) TypedValue.applyDimension ( TypedValue.COMPLEX_UNIT_DIP,
                dipValue, r.getDisplayMetrics() );
    }
}
