public class InstallCert {
    public static void main ( String[] args ) throws Exception {
        String httpsUrl = null;
        String host;
        int port;
        char[] passphrase;
        if ( args.length == 0 ) {
            httpsUrl = "https://www.google.com/";
            port = 443;
            host = "www.google.com";
            passphrase = "changeit".toCharArray();
        } else if ( ( args.length == 1 ) || ( args.length == 2 ) ) {
            String[] c = args[0].split ( ":" );
            host = c[0];
            port = ( c.length == 1 ) ? 443 : Integer.parseInt ( c[1] );
            String p = ( args.length == 1 ) ? "changeit" : args[1];
            passphrase = p.toCharArray();
        } else {
            System.out.println ( "Usage: java InstallCert [:port] [passphrase]" );
            return;
        }
        File file = new File ( "jssecacerts" );
        if ( file.isFile() == false ) {
            char SEP = File.separatorChar;
            File dir = new File ( System.getProperty ( "java.home" ) + SEP + "lib"
                                  + SEP + "security" );
            file = new File ( dir, "jssecacerts" );
            if ( file.isFile() == false ) {
                file = new File ( dir, "cacerts" );
            }
        }
        System.out.println ( "Loading KeyStore " + file + "..." );
        InputStream in = new FileInputStream ( file );
        KeyStore ks = KeyStore.getInstance ( KeyStore.getDefaultType() );
        ks.load ( in, passphrase );
        in.close();
        SSLContext context = SSLContext.getInstance ( "TLS" );
        TrustManagerFactory tmf = TrustManagerFactory
                                  .getInstance ( TrustManagerFactory.getDefaultAlgorithm() );
        tmf.init ( ks );
        X509TrustManager defaultTrustManager = ( X509TrustManager ) tmf
                                               .getTrustManagers() [0];
        SavingTrustManager tm = new SavingTrustManager ( defaultTrustManager );
        context.init ( null, new TrustManager[] { tm }, null );
        SSLSocketFactory factory = context.getSocketFactory();
        System.out.println ( "Opening connection to " + host + ":" + port + "..." );
        URL url = new URL ( httpsUrl );
        Proxy p = new Proxy ( Proxy.Type.SOCKS, new InetSocketAddress ( "176.9.240.109", 1080 ) );
        HttpsURLConnection conn = ( HttpsURLConnection ) url.openConnection ( p );
        conn.setSSLSocketFactory ( factory );
        conn.setConnectTimeout ( 3000 );
        conn.setReadTimeout ( 3000 );
        conn.setRequestProperty ( "User-Agent", "Mozilla/5.0 (Windows NT 6.0; WOW64; rv:19.0) Gecko/20100101 Firefox/19.0" );
        try {
            System.out.println ( "Starting SSL handshake..." );
            conn.connect();
            System.out.println();
            System.out.println ( "No errors, certificate is already trusted" );
        } catch ( SSLException e ) {
            System.out.println();
            e.printStackTrace ( System.out );
        } finally {
            conn.disconnect();
        }
        X509Certificate[] chain = tm.chain;
        if ( chain == null ) {
            System.out.println ( "Could not obtain server certificate chain" );
            return;
        }
        BufferedReader reader = new BufferedReader ( new InputStreamReader (
                    System.in ) );
        System.out.println();
        System.out.println ( "Server sent " + chain.length + " certificate(s):" );
        System.out.println();
        MessageDigest sha1 = MessageDigest.getInstance ( "SHA1" );
        MessageDigest md5 = MessageDigest.getInstance ( "MD5" );
        for ( int i = 0; i < chain.length; i++ ) {
            X509Certificate cert = chain[i];
            System.out.println ( " " + ( i + 1 ) + " Subject "
                                 + cert.getSubjectDN() );
            System.out.println ( "   Issuer  " + cert.getIssuerDN() );
            sha1.update ( cert.getEncoded() );
            System.out.println ( "   sha1    " + toHexString ( sha1.digest() ) );
            md5.update ( cert.getEncoded() );
            System.out.println ( "   md5     " + toHexString ( md5.digest() ) );
            System.out.println();
        }
        System.out
        .println ( "Enter certificate to add to trusted keystore or 'q' to quit: [1]" );
        String line = reader.readLine().trim();
        int k;
        try {
            k = ( line.length() == 0 ) ? 0 : Integer.parseInt ( line ) - 1;
        } catch ( NumberFormatException e ) {
            System.out.println ( "KeyStore not changed" );
            return;
        }
        X509Certificate cert = chain[k];
        String alias = host + "-" + ( k + 1 );
        ks.setCertificateEntry ( alias, cert );
        System.out.println();
        System.out.println ( cert );
        System.out.println();
        System.out
        .println ( "Added certificate to keystore 'jssecacerts' using alias '"
                   + alias + "'" );
    }
    private static final char[] HEXDIGITS = "0123456789abcdef".toCharArray();
    private static String toHexString ( byte[] bytes ) {
        StringBuilder sb = new StringBuilder ( bytes.length * 3 );
        for ( int b : bytes ) {
            b &= 0xff;
            sb.append ( HEXDIGITS[b >> 4] );
            sb.append ( HEXDIGITS[b & 15] );
            sb.append ( ' ' );
        }
        return sb.toString();
    }
    private static class SavingTrustManager implements X509TrustManager {
        private final X509TrustManager tm;
        private X509Certificate[] chain;
        SavingTrustManager ( X509TrustManager tm ) {
            this.tm = tm;
        }
        public X509Certificate[] getAcceptedIssuers() {
            throw new UnsupportedOperationException();
        }
        public void checkClientTrusted ( X509Certificate[] chain, String authType )
        throws CertificateException {
            throw new UnsupportedOperationException();
        }
        public void checkServerTrusted ( X509Certificate[] chain, String authType )
        throws CertificateException {
            this.chain = chain;
            tm.checkServerTrusted ( chain, authType );
        }
    }
}
