import java.util.AbstractList;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
public class Sorting {
    public static void main ( String[] args ) {
        final List<Integer> list = Arrays.asList ( 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 3, 3, 3, 3, 3 );
        System.out.println ( "Input: " + list );
        System.out.println ( "Solution1 (regroup): " + regroup ( list, 3 ) );
        System.out.println ( "Solution2 (regroup2): " + regroup2 ( list, 5 ) );
        System.out.println ( "Solution3 (groupSize constructor): " + new GroupedList<Integer> ( list, 5 ) );
        System.out.println ( "Solution3 (typeCount constructor): " + new GroupedList<Integer> ( 3, list ) );
        System.out.println ( "Demonstration of Solution3 emptyList constructor for groupSize 5" );
        final GroupedList<Integer> groupedList = new GroupedList<Integer> ( 5 );
        final List<List<Integer>> oneOfEachType = groupedList.getOneOfEachType();
        System.out.println ( "Ungrouped: " + groupedList );
        System.out.println ( "Grouped:" + oneOfEachType );
        groupedList.addAll ( Arrays.asList ( 11, 12, 13, 14, 15 ) );
        groupedList.addAll ( Arrays.asList ( 21, 22, 23, 24, 25 ) );
        groupedList.add ( 31 );
        groupedList.add ( 32 );
        groupedList.add ( 33 );
        groupedList.add ( 34 );
        groupedList.add ( 35 );
        System.out.println ( "After adding 3 groups" );
        System.out.println ( "Ungrouped: " + groupedList );
        System.out.println ( "Grouped:" + oneOfEachType );
        groupedList.addAll ( Arrays.asList ( 41, 42, 43, 44, 45 ) );
        System.out.println ( "After adding 4th group" );
        System.out.println ( "Ungrouped: " + groupedList );
        System.out.println ( "Grouped:" + oneOfEachType );
    }
    private static <T> List<T> regroup ( final List<T> list, final int typeCount ) {
        final int groupSize = list.size() / typeCount;
        final ArrayList<T> result = new ArrayList<T> ( list.size() );
        for ( int i = 0; i < groupSize; i++ ) {
            for ( int j = 0; j < typeCount; j++ ) {
                result.add ( list.get ( i + j * groupSize ) );
            }
        }
        return result;
    }
    private static <T> List<T> regroup2 ( final List<T> list, final int groupSize ) {
        final ArrayList<T> result = new ArrayList<T> ( list.size() );
        for ( int i = 0; i < groupSize; i++ ) {
            for ( int j = 0; i + j * groupSize < list.size(); j++ ) {
                result.add ( list.get ( i + j * groupSize ) );
            }
        }
        return result;
    }
    public static class GroupedList<T> extends AbstractList<T> {
        private final List<T> list;
        private final int     groupSize;
        public GroupedList ( final int groupSize ) {
            this.list = new ArrayList<T>();
            this.groupSize = groupSize;
        }
        public GroupedList ( final List<T> list, final int groupSize ) {
            this.list = list;
            this.groupSize = groupSize;
        }
        public GroupedList ( final int typeCount, final List<T> list ) {
            this.list = list;
            this.groupSize = list.size() / typeCount;
        }
        public int getGroupSize() {
            return groupSize;
        }
        public int getTypeCount() {
            return list.size() / groupSize;
        }
        @Override
        public boolean add ( T t ) {
            return list.add ( t );
        }
        @Override
        public T get ( int index ) {
            final int realIndex = getRealIndex ( index );
            return list.get ( realIndex );
        }
        @Override
        public T set ( int index, T element ) {
            final int realIndex = getRealIndex ( index );
            if ( realIndex < list.size() ) {
                return list.set ( realIndex, element );
            } else {
                throw new IndexOutOfBoundsException ( "Adding to this List-implementation is not possible" );
            }
        }
        private int getRealIndex ( int index ) {
            final int typeCount = getTypeCount();
            if ( typeCount * groupSize != list.size() ) {
                throw new IllegalStateException ( "There is an incomplete group. Can't establish list order." );
            }
            final int offsetInGroup = index / typeCount;
            final int groupStart = ( index % typeCount ) * groupSize;
            return offsetInGroup + groupStart;
        }
        @Override
        public int size() {
            return list.size();
        }
        public List<List<T>> getOneOfEachType() {
            return new AbstractList<List<T>>() {
                @Override
                public List<T> get ( int index ) {
                    return getOneOfEachType ( index );
                }
                @Override
                public int size() {
                    return groupSize;
                }
            };
        }
        public List<T> getOneOfEachType ( final int groupNumber ) {
            return new AbstractList<T>() {
                @Override
                public T get ( int index ) {
                    return list.get ( getRealIndex ( index ) );
                }
                @Override
                public T set ( int index, T element ) {
                    return list.set ( getRealIndex ( index ), element );
                }
                private int getRealIndex ( int index ) {
                    final int typeCount = getTypeCount();
                    if ( index >= typeCount ) {
                        throw new IndexOutOfBoundsException();
                    }
                    if ( typeCount * groupSize != list.size() ) {
                        throw new IllegalStateException ( "There is an incomplete group. Can't establish list order." );
                    }
                    return index * groupSize + groupNumber;
                }
                @Override
                public int size() {
                    return getTypeCount();
                }
            };
        }
    }
}
