import java.awt.Dimension;
import java.awt.EventQueue;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.HeadlessException;
import java.awt.RenderingHints;
import java.awt.Transparency;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import javax.imageio.ImageIO;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
public class SimpleBackground {
    public static void main ( String[] args ) {
        new SimpleBackground();
    }
    public SimpleBackground() {
        EventQueue.invokeLater ( new Runnable() {
            @Override
            public void run() {
                try {
                    UIManager.setLookAndFeel ( UIManager.getSystemLookAndFeelClassName() );
                } catch ( ClassNotFoundException | InstantiationException | IllegalAccessException | UnsupportedLookAndFeelException ex ) {
                }
                try {
                    BackgroundPane background = new BackgroundPane();
                    background.setBackground ( ImageIO.read ( new File ( "/path/to/your/image/on/your/disk" ) ) );
                    JFrame frame = new JFrame ( "Testing" );
                    frame.setDefaultCloseOperation ( JFrame.EXIT_ON_CLOSE );
                    frame.setContentPane ( background );
                    frame.setLayout ( new GridBagLayout() );
                    GridBagConstraints gbc = new GridBagConstraints();
                    gbc.gridwidth = GridBagConstraints.REMAINDER;
                    frame.add ( new JLabel ( "Hello world" ), gbc );
                    frame.add ( new JLabel ( "I'm on top" ), gbc );
                    frame.add ( new JButton ( "Clickity-clackity" ), gbc );
                    frame.pack();
                    frame.setLocationRelativeTo ( null );
                    frame.setVisible ( true );
                } catch ( IOException exp ) {
                    exp.printStackTrace();
                }
            }
        } );
    }
    public class BackgroundPane extends JPanel {
        private BufferedImage img;
        private BufferedImage scaled;
        public BackgroundPane() {
        }
        @Override
        public Dimension getPreferredSize() {
            return img == null ? super.getPreferredSize() : new Dimension ( img.getWidth(), img.getHeight() );
        }
        public void setBackground ( BufferedImage value ) {
            if ( value != img ) {
                this.img = value;
                repaint();
            }
        }
        @Override
        public void invalidate() {
            super.invalidate();
            if ( getWidth() > img.getWidth() || getHeight() > img.getHeight() ) {
                scaled = getScaledInstanceToFill ( img, getSize() );
            } else {
                scaled = img;
            }
        }
        @Override
        protected void paintComponent ( Graphics g ) {
            super.paintComponent ( g );
            if ( scaled != null ) {
                int x = ( getWidth() - scaled.getWidth() ) / 2;
                int y = ( getHeight() - scaled.getHeight() ) / 2;
                g.drawImage ( scaled, x, y, this );
            }
        }
    }
    public static BufferedImage getScaledInstanceToFill ( BufferedImage img, Dimension size ) {
        double scaleFactor = getScaleFactorToFill ( img, size );
        return getScaledInstance ( img, scaleFactor );
    }
    public static double getScaleFactorToFill ( BufferedImage img, Dimension size ) {
        double dScale = 1;
        if ( img != null ) {
            int imageWidth = img.getWidth();
            int imageHeight = img.getHeight();
            double dScaleWidth = getScaleFactor ( imageWidth, size.width );
            double dScaleHeight = getScaleFactor ( imageHeight, size.height );
            dScale = Math.max ( dScaleHeight, dScaleWidth );
        }
        return dScale;
    }
    public static double getScaleFactor ( int iMasterSize, int iTargetSize ) {
        double dScale = ( double ) iTargetSize / ( double ) iMasterSize;
        return dScale;
    }
    public static BufferedImage getScaledInstance ( BufferedImage img, double dScaleFactor ) {
        return getScaledInstance ( img, dScaleFactor, RenderingHints.VALUE_INTERPOLATION_BILINEAR, true );
    }
    protected static BufferedImage getScaledInstance ( BufferedImage img, double dScaleFactor, Object hint, boolean bHighQuality ) {
        BufferedImage imgScale = img;
        int iImageWidth = ( int ) Math.round ( img.getWidth() * dScaleFactor );
        int iImageHeight = ( int ) Math.round ( img.getHeight() * dScaleFactor );
        if ( dScaleFactor <= 1.0d ) {
            imgScale = getScaledDownInstance ( img, iImageWidth, iImageHeight, hint, bHighQuality );
        } else {
            imgScale = getScaledUpInstance ( img, iImageWidth, iImageHeight, hint, bHighQuality );
        }
        return imgScale;
    }
    protected static BufferedImage getScaledDownInstance ( BufferedImage img,
            int targetWidth,
            int targetHeight,
            Object hint,
            boolean higherQuality ) {
        int type = ( img.getTransparency() == Transparency.OPAQUE )
                   ? BufferedImage.TYPE_INT_RGB : BufferedImage.TYPE_INT_ARGB;
        BufferedImage ret = ( BufferedImage ) img;
        if ( targetHeight > 0 || targetWidth > 0 ) {
            int w, h;
            if ( higherQuality ) {
                w = img.getWidth();
                h = img.getHeight();
            } else {
                w = targetWidth;
                h = targetHeight;
            }
            do {
                if ( higherQuality && w > targetWidth ) {
                    w /= 2;
                    if ( w < targetWidth ) {
                        w = targetWidth;
                    }
                }
                if ( higherQuality && h > targetHeight ) {
                    h /= 2;
                    if ( h < targetHeight ) {
                        h = targetHeight;
                    }
                }
                BufferedImage tmp = new BufferedImage ( Math.max ( w, 1 ), Math.max ( h, 1 ), type );
                Graphics2D g2 = tmp.createGraphics();
                g2.setRenderingHint ( RenderingHints.KEY_INTERPOLATION, hint );
                g2.drawImage ( ret, 0, 0, w, h, null );
                g2.dispose();
                ret = tmp;
            } while ( w != targetWidth || h != targetHeight );
        } else {
            ret = new BufferedImage ( 1, 1, type );
        }
        return ret;
    }
    protected static BufferedImage getScaledUpInstance ( BufferedImage img,
            int targetWidth,
            int targetHeight,
            Object hint,
            boolean higherQuality ) {
        int type = BufferedImage.TYPE_INT_ARGB;
        BufferedImage ret = ( BufferedImage ) img;
        int w, h;
        if ( higherQuality ) {
            w = img.getWidth();
            h = img.getHeight();
        } else {
            w = targetWidth;
            h = targetHeight;
        }
        do {
            if ( higherQuality && w < targetWidth ) {
                w *= 2;
                if ( w > targetWidth ) {
                    w = targetWidth;
                }
            }
            if ( higherQuality && h < targetHeight ) {
                h *= 2;
                if ( h > targetHeight ) {
                    h = targetHeight;
                }
            }
            BufferedImage tmp = new BufferedImage ( w, h, type );
            Graphics2D g2 = tmp.createGraphics();
            g2.setRenderingHint ( RenderingHints.KEY_INTERPOLATION, hint );
            g2.drawImage ( ret, 0, 0, w, h, null );
            g2.dispose();
            ret = tmp;
            tmp = null;
        } while ( w != targetWidth || h != targetHeight );
        return ret;
    }
}
