public class VersionNumberHandler implements SOAPHandler<SOAPMessageContext> {
    private static final String LoggerName = "ClientSideLogger";
    private Logger logger;
    private final boolean log_p = true;
    public VersionNumberHandler() {
        logger = Logger.getLogger ( LoggerName );
    }
    public boolean handleMessage ( SOAPMessageContext ctx ) {
        if ( log_p ) {
            logger.info ( "handleMessage" );
        }
        Boolean request_p = ( Boolean ) ctx
                            .get ( MessageContext.MESSAGE_OUTBOUND_PROPERTY );
        if ( request_p ) {
            String versionNumber = "v1.0";
            try {
                SOAPMessage msg = ctx.getMessage();
                SOAPEnvelope env = msg.getSOAPPart().getEnvelope();
                SOAPHeader hdr = env.getHeader();
                if ( hdr == null ) {
                    hdr = env.addHeader();
                }
                QName qname = new QName ( "http://ticket.example.com/",
                                          "versionnumber" );
                SOAPHeaderElement helem = hdr.addHeaderElement ( qname );
                helem.setActor ( SOAPConstants.URI_SOAP_ACTOR_NEXT );
                helem.setMustUnderstand ( true );
                helem.addTextNode ( versionNumber );
                msg.saveChanges();
                msg.writeTo ( System.out );
            } catch ( SOAPException e ) {
                System.err.println ( e );
            } catch ( IOException e ) {
                System.err.println ( e );
            }
        }
        return true;
    }
    public boolean handleFault ( SOAPMessageContext ctx ) {
        if ( log_p ) {
            logger.info ( "handleFault" );
        }
        try {
            ctx.getMessage().writeTo ( System.out );
        } catch ( SOAPException e ) {
            System.err.println ( e );
        } catch ( IOException e ) {
            System.err.println ( e );
        }
        return true;
    }
    public Set<QName> getHeaders() {
        if ( log_p ) {
            logger.info ( "getHeaders" );
        }
        return null;
    }
    public void close ( MessageContext messageContext ) {
        if ( log_p ) {
            logger.info ( "close" );
        }
    }
