package fibonacci.bench;
import fibonacci.mdl.ExplicitLocking;
import fibonacci.mdl.FibonacciGenerator;
import fibonacci.mdl.IntrinsicLocking;
import fibonacci.mdl.LockFree;
import fibonacci.mdl.STM;
import org.openjdk.jmh.annotations.BenchmarkMode;
import org.openjdk.jmh.annotations.GenerateMicroBenchmark;
import org.openjdk.jmh.annotations.Level;
import org.openjdk.jmh.annotations.Measurement;
import org.openjdk.jmh.annotations.Mode;
import org.openjdk.jmh.annotations.Scope;
import org.openjdk.jmh.annotations.Setup;
import org.openjdk.jmh.annotations.State;
import org.openjdk.jmh.annotations.Warmup;
import java.math.BigInteger;
@BenchmarkMode ( Mode.SingleShotTime )
@Warmup ( iterations = 100, batchSize = JmhBench.BATCH_SIZE )
@Measurement ( iterations = 100, batchSize = JmhBench.BATCH_SIZE )
@State ( Scope.Benchmark )
public class JmhBench {
    public static final int BATCH_SIZE = 50000;
    private FibonacciGenerator explicitLock;
    private IntrinsicLocking intrinsicLock;
    private LockFree lockFree;
    private STM stm;
    @Setup ( Level.Iteration )
    public void setup() {
        explicitLock = new ExplicitLocking();
        intrinsicLock = new IntrinsicLocking();
        lockFree = new LockFree();
        stm = new STM();
    }
    @GenerateMicroBenchmark
    public BigInteger stm() {
        return stm.next();
    }
    @GenerateMicroBenchmark
    public BigInteger explicitLock() {
        return explicitLock.next();
    }
    @GenerateMicroBenchmark
    public BigInteger intrinsicLock() {
        return intrinsicLock.next();
    }
    @GenerateMicroBenchmark
    public BigInteger lockFree() {
        return lockFree.next();
    }
}
