private class DrawingPanel extends JPanel implements
    MouseListener, MouseMotionListener {
    private BufferedImage bufferedImage;
    private int xPos, yPos;
    private DrawingPanel() {
        addMouseListener ( this );
        addMouseMotionListener ( this );
    }
    private void validateImage() {
        if ( bufferedImage == null ) {
            bufferedImage = new BufferedImage (
                getWidth(), getHeight(),
                BufferedImage.TYPE_INT_ARGB );
            Graphics g = bufferedImage.getGraphics();
            g.setColor ( getBackground() );
            g.fillRect ( 0, 0, getWidth(), getHeight() );
            g.dispose();
        }
        if ( bufferedImage.getWidth() != getWidth() ||
                bufferedImage.getHeight() != getHeight() ) {
            BufferedImage newBufferedImage = new BufferedImage (
                getWidth(), getHeight(),
                BufferedImage.TYPE_INT_ARGB );
            Graphics g = newBufferedImage.getGraphics();
            g.setColor ( getBackground() );
            g.fillRect ( 0, 0, getWidth(), getHeight() );
            g.drawImage ( bufferedImage, 0, 0, null );
            g.dispose();
            bufferedImage = newBufferedImage;
        }
    }
    @Override
    protected void paintComponent ( Graphics g ) {
        super.paintComponent ( g );
        validateImage();
        g.drawImage ( bufferedImage, 0, 0, null );
    }
    @Override
    public void mousePressed ( MouseEvent me ) {
        xPos = me.getX();
        yPos = me.getY();
    }
    @Override
    public void mouseDragged ( MouseEvent me ) {
        int x = me.getX(), y = me.getY();
        validateImage();
        Graphics g = bufferedImage.getGraphics();
        g.setColor ( Color.BLACK );
        g.drawOval ( xPos, yPos, 30, 30 );
        repaint();
        xPos = x;
        yPos = y;
    }
    public void loadDrawing ( BufferedImage bi ) {
        JOptionPane.showMessageDialog ( null, new JLabel ( new ImageIcon ( bi ) ) );
        System.out.println ( "w:" + bi.getWidth() + " h:" + bi.getHeight() );
    }
    public BufferedImage getScreenShot() {
        BufferedImage image = new BufferedImage (
            getWidth(), getHeight(), BufferedImage.TYPE_INT_RGB );
        Graphics g = image.getGraphics();
        g.drawImage ( bufferedImage, 0, 0, null );
        g.dispose();
        return image;
    }
    @Override
    public void mouseClicked ( MouseEvent me ) {
    }
    @Override
    public void mouseReleased ( MouseEvent me ) {
    }
    @Override
    public void mouseEntered ( MouseEvent me ) {
    }
    @Override
    public void mouseExited ( MouseEvent me ) {
    }
    @Override
    public void mouseMoved ( MouseEvent me ) {
    }
}
