public class Array2UniqueID {
    private final double min;
    private final double max;
    private final double prec;
    private final int length;
    public Array2UniqueID ( double min, double max, double prec, int length ) {
        this.min = min;
        this.max = max;
        this.prec = prec;
        this.length = length;
    }
    public Array2UniqueID ( int length ) {
        this ( -Double.MAX_VALUE, Double.MAX_VALUE, Double.MIN_VALUE, length );
    }
    public BigDecimal createUniqueID ( double[] array ) {
        if ( array.length != length ) {
            throw new IllegalArgumentException ( "Array length must be "
                                                 + length + " but was " + array.length );
        }
        for ( double d : array ) {
            if ( d < min || d > max ) {
                throw new IllegalArgumentException ( "Each element of the array"
                                                     + " must be in the range [" + min + ", " + max + "]" );
            }
        }
        double range = max - min;
        BigDecimal maxNums = BigDecimal.valueOf ( range )
                             .divide ( BigDecimal.valueOf ( prec ) )
                             .add ( BigDecimal.ONE );
        BigDecimal id = BigDecimal.valueOf ( 0 );
        for ( int i = 0; i < array.length; i++ ) {
            BigDecimal num = BigDecimal.valueOf ( array[i] )
                             .divide ( BigDecimal.valueOf ( prec ) )
                             .multiply ( maxNums ).pow ( i );
            id = id.add ( num );
        }
        return id;
    }
