public class GPSTracker extends Service implements LocationListener {
    private final Context mContext;
    boolean isGPSEnabled = false;
    boolean isNetworkEnabled = false;
    boolean canGetLocation = false;
    Location location;
    double latitude;
    double longitude;
    private static final long MIN_DISTANCE_CHANGE_FOR_UPDATES = 10;
    private static final long MIN_TIME_BW_UPDATES = 1000 * 60 * 1;
    protected LocationManager locationManager;
    public GPSTracker ( Context context ) {
        this.mContext = context;
        getLocation();
    }
    public Location getLocation() {
        try {
            locationManager = ( LocationManager ) mContext.getSystemService ( LOCATION_SERVICE );
            isGPSEnabled = locationManager.isProviderEnabled ( LocationManager.GPS_PROVIDER );
            isNetworkEnabled = locationManager.isProviderEnabled ( LocationManager.NETWORK_PROVIDER );
            if ( !isGPSEnabled && !isNetworkEnabled ) {
            } else {
                this.canGetLocation = true;
                if ( isNetworkEnabled ) {
                    locationManager.requestLocationUpdates (
                        LocationManager.NETWORK_PROVIDER,
                        MIN_TIME_BW_UPDATES,
                        MIN_DISTANCE_CHANGE_FOR_UPDATES, this );
                    Log.d ( "Network", "Network" );
                    if ( locationManager != null ) {
                        location = locationManager.getLastKnownLocation ( LocationManager.NETWORK_PROVIDER );
                        updateGPSCoordinates();
                    }
                }
                if ( isGPSEnabled ) {
                    if ( location == null ) {
                        locationManager.requestLocationUpdates (
                            LocationManager.GPS_PROVIDER,
                            MIN_TIME_BW_UPDATES,
                            MIN_DISTANCE_CHANGE_FOR_UPDATES, this );
                        Log.d ( "GPS Enabled", "GPS Enabled" );
                        if ( locationManager != null ) {
                            location = locationManager.getLastKnownLocation ( LocationManager.GPS_PROVIDER );
                            updateGPSCoordinates();
                        }
                    }
                }
            }
        } catch ( Exception e ) {
            Log.e ( "Error : Location", "Impossible to connect to LocationManager", e );
        }
        return location;
    }
    public void updateGPSCoordinates() {
        if ( location != null ) {
            latitude = location.getLatitude();
            longitude = location.getLongitude();
        }
    }
    public void stopUsingGPS() {
        if ( locationManager != null ) {
            locationManager.removeUpdates ( GPSTracker.this );
        }
    }
    public double getLatitude() {
        if ( location != null ) {
            latitude = location.getLatitude();
        }
        return latitude;
    }
    public double getLongitude() {
        if ( location != null ) {
            longitude = location.getLongitude();
        }
        return longitude;
    }
    public boolean canGetLocation() {
        return this.canGetLocation;
    }
    public void showSettingsAlert() {
        AlertDialog.Builder alertDialog = new AlertDialog.Builder ( mContext );
        alertDialog.setTitle ( R.string.GPSAlertDialogTitle );
        alertDialog.setMessage ( R.string.GPSAlertDialogMessage );
        alertDialog.setPositiveButton ( R.string.settings, new DialogInterface.OnClickListener() {
            @Override
            public void onClick ( DialogInterface dialog, int which ) {
                Intent intent = new Intent ( Settings.ACTION_LOCATION_SOURCE_SETTINGS );
                mContext.startActivity ( intent );
            }
        } );
        alertDialog.setNegativeButton ( R.string.cancel, new DialogInterface.OnClickListener() {
            @Override
            public void onClick ( DialogInterface dialog, int which ) {
                dialog.cancel();
            }
        } );
        alertDialog.show();
    }
    public List<Address> getGeocoderAddress ( Context context ) {
        if ( location != null ) {
            Geocoder geocoder = new Geocoder ( context, Locale.ENGLISH );
            try {
                List<Address> addresses = geocoder.getFromLocation ( latitude, longitude, 1 );
                return addresses;
            } catch ( IOException e ) {
                Log.e ( "Error : Geocoder", "Impossible to connect to Geocoder", e );
            }
        }
        return null;
    }
    public String getAddressLine ( Context context ) {
        List<Address> addresses = getGeocoderAddress ( context );
        if ( addresses != null && addresses.size() > 0 ) {
            Address address = addresses.get ( 0 );
            String addressLine = address.getAddressLine ( 0 );
            return addressLine;
        } else {
            return null;
        }
    }
    public String getLocality ( Context context ) {
        List<Address> addresses = getGeocoderAddress ( context );
        if ( addresses != null && addresses.size() > 0 ) {
            Address address = addresses.get ( 0 );
            String locality = address.getLocality();
            return locality;
        } else {
            return null;
        }
    }
    public String getPostalCode ( Context context ) {
        List<Address> addresses = getGeocoderAddress ( context );
        if ( addresses != null && addresses.size() > 0 ) {
            Address address = addresses.get ( 0 );
            String postalCode = address.getPostalCode();
            return postalCode;
        } else {
            return null;
        }
    }
    public String getCountryName ( Context context ) {
        List<Address> addresses = getGeocoderAddress ( context );
        if ( addresses != null && addresses.size() > 0 ) {
            Address address = addresses.get ( 0 );
            String countryName = address.getCountryName();
            return countryName;
        } else {
            return null;
        }
    }
    @Override
    public void onLocationChanged ( Location location ) {
    }
    @Override
    public void onProviderDisabled ( String provider ) {
    }
    @Override
    public void onProviderEnabled ( String provider ) {
    }
    @Override
    public void onStatusChanged ( String provider, int status, Bundle extras ) {
    }
    @Override
    public IBinder onBind ( Intent intent ) {
        return null;
    }
}
