import java.awt.BorderLayout;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.UIManager;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeSelectionModel;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JButton;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;
public class TestTree extends JPanel {
    private JTree tree;
    private DefaultTreeModel model;
    private JButton btnAdd;
    private int childCount;
    public TestTree() {
        super ( new BorderLayout() );
        DefaultMutableTreeNode top = new DefaultMutableTreeNode ( "The Java Series" );
        createNodes ( top );
        model = new DefaultTreeModel ( top );
        tree = new JTree ( model );
        tree.getSelectionModel().setSelectionMode ( TreeSelectionModel.SINGLE_TREE_SELECTION );
        JScrollPane treeView = new JScrollPane ( tree );
        add ( treeView );
        btnAdd = new JButton ( "Add" );
        btnAdd.addActionListener ( new ActionListener() {
            @Override
            public void actionPerformed ( ActionEvent e ) {
                TreePath treePath = tree.getSelectionPath();
                if ( treePath != null ) {
                    DefaultMutableTreeNode node = ( DefaultMutableTreeNode ) treePath.getLastPathComponent();
                    DefaultMutableTreeNode child = new DefaultMutableTreeNode ( "Child " + ( ++childCount ) );
                    model.insertNodeInto ( child, node, node.getChildCount() );
                }
            }
        } );
        add ( btnAdd, BorderLayout.SOUTH );
    }
    private class BookInfo {
        public String bookName;
        public BookInfo ( String book ) {
            bookName = book;
        }
        public String toString() {
            return bookName;
        }
    }
    private void createNodes ( DefaultMutableTreeNode top ) {
        DefaultMutableTreeNode category = null;
        DefaultMutableTreeNode book = null;
        category = new DefaultMutableTreeNode ( "Books for Java Programmers" );
        top.add ( category );
        book = new DefaultMutableTreeNode ( new BookInfo ( "The Java Tutorial: A Short Course on the Basics" ) );
        category.add ( book );
        book = new DefaultMutableTreeNode ( new BookInfo ( "The Java Tutorial Continued: The Rest of the JDK" ) );
        category.add ( book );
        book = new DefaultMutableTreeNode ( new BookInfo ( "The JFC Swing Tutorial: A Guide to Constructing GUIs" ) );
        category.add ( book );
        book = new DefaultMutableTreeNode ( new BookInfo ( "Effective Java Programming Language Guide" ) );
        category.add ( book );
        book = new DefaultMutableTreeNode ( new BookInfo ( "The Java Programming Language" ) );
        category.add ( book );
        book = new DefaultMutableTreeNode ( new BookInfo ( "The Java Developers Almanac" ) );
        category.add ( book );
        category = new DefaultMutableTreeNode ( "Books for Java Implementers" );
        top.add ( category );
        book = new DefaultMutableTreeNode ( new BookInfo ( "The Java Virtual Machine Specification" ) );
        category.add ( book );
        book = new DefaultMutableTreeNode ( new BookInfo ( "The Java Language Specification" ) );
        category.add ( book );
    }
    private static void createAndShowGUI() {
        try {
            UIManager.setLookAndFeel ( UIManager.getSystemLookAndFeelClassName() );
        } catch ( ClassNotFoundException | InstantiationException | IllegalAccessException | UnsupportedLookAndFeelException ex ) {
            ex.printStackTrace();
        }
        JFrame frame = new JFrame ( "TreeDemo" );
        frame.setDefaultCloseOperation ( JFrame.EXIT_ON_CLOSE );
        frame.add ( new TestTree() );
        frame.pack();
        frame.setVisible ( true );
    }
    public static void main ( String[] args ) {
        javax.swing.SwingUtilities.invokeLater ( new Runnable() {
            public void run() {
                createAndShowGUI();
            }
        } );
    }
}
