package net.finmath.tests.marketdata.curves;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import org.junit.Test;
import net.finmath.marketdata.model.curves.Curve;
import net.finmath.marketdata.model.curves.CurveInterface;
import net.finmath.optimizer.LevenbergMarquardt;
import net.finmath.optimizer.SolverException;
public class CurveTest {
    private static NumberFormat numberFormat = new DecimalFormat ( "0.0000" );
    public static void main ( String[] args ) throws SolverException, CloneNotSupportedException {
        ( new CurveTest() ).testCurveFitting();
    }
    @Test
    public void testCurveFitting() throws SolverException, CloneNotSupportedException {
        Curve.CurveBuilder curveBuilder = new Curve.CurveBuilder();
        curveBuilder.setInterpolationMethod ( Curve.InterpolationMethod.LINEAR );
        curveBuilder.setExtrapolationMethod ( Curve.ExtrapolationMethod.LINEAR );
        curveBuilder.setInterpolationEntity ( Curve.InterpolationEntity.VALUE );
        curveBuilder.addPoint ( -1.0  , 1.0  , false );
        curveBuilder.addPoint ( 0.0  , 1.0  , false );
        curveBuilder.addPoint ( 0.5   , 2.0  , true );
        curveBuilder.addPoint ( 0.75  , 2.0  , true );
        curveBuilder.addPoint ( 1.0  , 2.0  , true );
        curveBuilder.addPoint ( 2.2  , 2.0  , true );
        curveBuilder.addPoint ( 3.0  , 2.0  , true );
        final Curve curve = curveBuilder.build();
        final double[] givenTimes   = { 0.0,  0.5, 0.75, 1.0, 1.5, 1.75, 2.5 };
        final double[] givenValues  = { 3.5, 12.3, 13.2, 7.5, 5.5, 2.9,  4.4 };
        LevenbergMarquardt optimizer = new LevenbergMarquardt (
            curve.getParameter()     ,
            givenValues              ,
            100,
            Runtime.getRuntime().availableProcessors()
        ) {
            @Override
            public void setValues ( double[] parameters, double[] values ) throws SolverException {
                CurveInterface curveGuess = null;
                try {
                    curveGuess = curve.getCloneForParameter ( parameters );
                } catch ( CloneNotSupportedException e ) {
                    throw new SolverException ( e );
                }
                for ( int valueIndex = 0; valueIndex < values.length; valueIndex++ ) {
                    values[valueIndex] = curveGuess.getValue ( givenTimes[valueIndex] );
                }
            }
        };
        optimizer.run();
        CurveInterface fittedCurve = curve.getCloneForParameter ( optimizer.getBestFitParameters() );
        for ( double time = -2.0; time < 5.0; time += 0.1 ) {
            System.out.println ( numberFormat.format ( time ) + "\t" + numberFormat.format ( fittedCurve.getValue ( time ) ) );
        }
        double errorSum = 0.0;
        for ( int pointIndex = 0; pointIndex < givenTimes.length; pointIndex++ ) {
            errorSum += fittedCurve.getValue ( givenTimes[pointIndex] ) - givenValues[pointIndex];
        }
        System.out.println ( "Mean deviation: " + errorSum );
        org.junit.Assert.assertTrue ( Math.abs ( errorSum - -2.5 ) < 1E-5 );
    }
}
