import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.table.*;
public class TablePopupEditor extends DefaultCellEditor
    implements TableCellEditor {
    private PopupDialog popup;
    private String currentText = "";
    private JButton editorComponent;
    public TablePopupEditor() {
        super ( new JTextField() );
        setClickCountToStart ( 1 );
        editorComponent = new JButton();
        editorComponent.setBackground ( Color.white );
        editorComponent.setBorderPainted ( false );
        editorComponent.setContentAreaFilled ( false );
        editorComponent.setFocusable ( false );
        popup = new PopupDialog();
    }
    public Object getCellEditorValue() {
        return currentText;
    }
    public Component getTableCellEditorComponent (
        JTable table, Object value, boolean isSelected, int row, int column ) {
        SwingUtilities.invokeLater ( new Runnable() {
            public void run() {
                popup.setText ( currentText );
                Point p = editorComponent.getLocationOnScreen();
                popup.setLocation ( p.x, p.y + editorComponent.getSize().height );
                popup.show();
                fireEditingStopped();
            }
        } );
        currentText = value.toString();
        editorComponent.setText ( currentText );
        return editorComponent;
    }
    class PopupDialog extends JDialog implements ActionListener {
        private JTextArea textArea;
        public PopupDialog() {
            super ( ( Frame ) null, "Change Description", true );
            textArea = new JTextArea ( 5, 20 );
            textArea.setLineWrap ( true );
            textArea.setWrapStyleWord ( true );
            KeyStroke keyStroke = KeyStroke.getKeyStroke ( "ENTER" );
            textArea.getInputMap().put ( keyStroke, "none" );
            JScrollPane scrollPane = new JScrollPane ( textArea );
            getContentPane().add ( scrollPane );
            JButton cancel = new JButton ( "Cancel" );
            cancel.addActionListener ( this );
            JButton ok = new JButton ( "Ok" );
            ok.setPreferredSize ( cancel.getPreferredSize() );
            ok.addActionListener ( this );
            JPanel buttons = new JPanel();
            buttons.add ( ok );
            buttons.add ( cancel );
            getContentPane().add ( buttons, BorderLayout.SOUTH );
            pack();
            getRootPane().setDefaultButton ( ok );
        }
        public void setText ( String text ) {
            textArea.setText ( text );
        }
        public void actionPerformed ( ActionEvent e ) {
            if ( "Ok".equals ( e.getActionCommand() ) ) {
                currentText = textArea.getText();
            }
            textArea.requestFocusInWindow();
            setVisible ( false );
        }
    }
    private static void createAndShowUI() {
        String[] columnNames = {"Item", "Description"};
        Object[][] data = {
            {"Item 1", "Description of Item 1"},
            {"Item 2", "Description of Item 2"},
            {"Item 3", "Description of Item 3"}
        };
        JTable table = new JTable ( data, columnNames );
        table.getColumnModel().getColumn ( 1 ).setPreferredWidth ( 300 );
        table.setPreferredScrollableViewportSize ( table.getPreferredSize() );
        JScrollPane scrollPane = new JScrollPane ( table );
        TablePopupEditor popupEditor = new TablePopupEditor();
        table.getColumnModel().getColumn ( 1 ).setCellEditor ( popupEditor );
        JFrame frame = new JFrame ( "Popup Editor Test" );
        frame.setDefaultCloseOperation ( JFrame.EXIT_ON_CLOSE );
        frame.add ( new JTextField(), BorderLayout.NORTH );
        frame.add ( scrollPane );
        frame.pack();
        frame.setLocationRelativeTo ( null );
        frame.setVisible ( true );
    }
    public static void main ( String[] args ) {
        EventQueue.invokeLater ( new Runnable() {
            public void run() {
                createAndShowUI();
            }
        } );
    }
}
