@Component
public class PathUrlAuthenticationFailureHandler extends SimpleUrlAuthenticationFailureHandler {
    private final PathTokens tokens;
    @Autowired
    public PathUrlAuthenticationFailureHandler ( PathTokens tokens ) {
        super();
        this.tokens = tokens;
    }
    @Override
    public void onAuthenticationFailure ( HttpServletRequest request, HttpServletResponse response,
                                          AuthenticationException exception ) throws IOException, ServletException {
        setDefaultFailureUrl ( getFailureUrlFromPath ( request ) );
        super.onAuthenticationFailure ( request, response, exception );
    }
    private String getFailureUrlFromPath ( HttpServletRequest request ) {
        String refererUrl = request.getHeader ( "Referer" );
        if ( tokens.isTokenInPath ( refererUrl ) ) {
            return "/" + tokens.getTokenFromPath ( refererUrl ) + "_signin?error=1";
        }
        throw new PathTokenNotFoundException ( "Token not found in referer URL " + refererUrl + " when retrieving failureUrl for login form" );
    }
}
