public class SettingsActivity extends PreferenceActivity {
    private static final String STATE_CUR_HEADER_POS = "Current Position";
    private static final String STATE_HEADERS_LIST   = "Headers List";
    private int mCurPos = AdapterView.INVALID_POSITION;
    private ArrayList<Header> mHeaders;
    @Override
    public void onBuildHeaders ( List<Header> target ) {
        loadHeadersFromResource ( R.xml.preference, target );
        mHeaders = ( ArrayList<Header> ) target;
    }
    @Override
    protected void onCreate ( Bundle savedInstanceState ) {
        super.onCreate ( savedInstanceState );
        if ( onIsMultiPane() && onIsHidingHeaders() ) {
            finish();
        }
    }
    @Override
    public boolean onIsMultiPane() {
        return getResources().getBoolean ( R.bool.pref_prefer_dual_pane );
    }
    @Override
    protected void onListItemClick ( ListView l, View v, int position, long id ) {
        super.onListItemClick ( l, v, position, id );
        mCurPos = position;
    }
    @Override
    protected void onRestoreInstanceState ( Bundle state ) {
        super.onRestoreInstanceState ( state );
        mHeaders = state.getParcelableArrayList ( STATE_HEADERS_LIST );
        mCurPos = state.getInt ( STATE_CUR_HEADER_POS );
        if ( mHeaders != null ) {
            if ( mCurPos != AdapterView.INVALID_POSITION ) {
                switchToHeader ( mHeaders.get ( mCurPos ) );
            } else {
                switchToHeader ( onGetInitialHeader() );
            }
        }
    }
    @Override
    protected void onSaveInstanceState ( Bundle outState ) {
        super.onSaveInstanceState ( outState );
        if ( mHeaders != null && mHeaders.size() > 0 ) {
            outState.putInt ( STATE_CUR_HEADER_POS, mCurPos );
            outState.putParcelableArrayList ( STATE_HEADERS_LIST, mHeaders );
        }
    }
}
