package objtest;
import java.io.IOException;
import java.io.InputStream;
import java.nio.BufferUnderflowException;
import java.nio.ByteBuffer;
import java.util.ArrayDeque;
import java.util.Queue;
import kokuks.KokuKS;
public class FramedInputStream extends InputStream {
    public static final int INITIAL_BUFFER_SIZE = 8 << 1;
    public static final int FRAME_HEADER_1 = 0xBEEFFACE;
    public static final int FRAME_HEADER_2 = 0xFACEBEEF;
    public static final byte[] HEADER_BYTES = new byte[4 * 2];
    protected static final byte[] CURR_HEADER_BUFF = new byte[HEADER_BYTES.length];
    static {
        ByteBuffer b = ByteBuffer.allocateDirect ( 8 );
        b.putInt ( FRAME_HEADER_1 );
        b.putInt ( FRAME_HEADER_2 );
        ByteBuffer b2 = ( ByteBuffer ) b.flip();
        b2.get ( HEADER_BYTES, 0, 4 );
        b2.get ( HEADER_BYTES, 3, 4 );
    }
    protected int     size         = 0;
    protected int     chain        = 0;
    protected boolean inFrame      = false;
    protected boolean readingSize  = false;
    protected int     sizePos      = 0;
    protected int dbgput = 0;
    protected ByteBuffer        bb  = ByteBuffer.allocateDirect ( INITIAL_BUFFER_SIZE );
    protected Queue<ByteBuffer> bbq = new ArrayDeque<ByteBuffer>();
    protected ByteBuffer        currBuff = null;
    protected final boolean     recoverFromError;
    public FramedInputStream ( boolean recoverFromError ) {
        this.recoverFromError = recoverFromError;
    }
    public FramedInputStream() {
        this ( true );
    }
    protected boolean ensureFramebufferCapacity ( int min ) {
        int mymin = 1 << min;
        if ( mymin <= bb.capacity() ) {
            return false;
        }
        int num = bb.capacity();
        while ( num < mymin ) {
            num <<= 1;
        }
        ByteBuffer bb2 = ByteBuffer.allocateDirect ( num );
        int bb_pos = bb.position();
        bb.rewind();
        bb2.put ( bb );
        bb = bb2;
        if ( KokuKS.DEBUG_MODE ) {
            System.out.println ( "modified buffer size to: " + num );
        }
        return true;
    }
    public boolean isRecoverFromError() {
        return recoverFromError;
    }
    @Override
    public int read() throws IOException {
        if ( currBuff == null || !currBuff.hasRemaining() ) {
            return -1;
        }
        byte b = currBuff.get();
        return b;
    }
    public void putBuffer ( ByteBuffer source ) {
        ensureFramebufferCapacity ( bb.capacity() + source.remaining() );
        while ( source.hasRemaining() ) {
            putByte ( source.get() );
        }
    }
    public boolean checkCompleteFrame() {
        return !bbq.isEmpty();
    }
    @Override
    public int available() throws IOException {
        return currBuff != null ? currBuff.remaining() : 0;
    }
    public int read ( byte[] data ) {
        if ( currBuff == null || !currBuff.hasRemaining() ) {
            return -1;
        }
        if ( data.length > currBuff.remaining() ) {
            throw new BufferUnderflowException();
        }
        currBuff.get ( data );
        return data.length;
    }
    public boolean nextFrame() {
        ByteBuffer bbf = bbq.poll();
        if ( bbf != null ) {
            currBuff = ByteBuffer.allocateDirect ( bbf.limit() );
            currBuff.put ( bbf ).flip();
            bbf.rewind();
            currBuff.rewind();
            currBuff.position ( 1 );
            return true;
        }
        return false;
    }
    public void putByte ( byte b ) {
        if ( recoverFromError || !inFrame ) {
            if ( b == HEADER_BYTES[chain++] ) {
                if ( chain >= ( HEADER_BYTES.length ) ) {
                    if ( KokuKS.DEBUG_MODE ) {
                        System.out.println ( "got header!" + ( inFrame ? " (recovered)" : "" ) );
                    }
                    inFrame = true;
                    sizePos = 0;
                    size = 0;
                    readingSize = true;
                    chain = 0;
                    bb.clear();
                }
            } else {
                chain = 0;
            }
        }
        if ( inFrame ) {
            if ( readingSize ) {
                size += ( b & 0xFF ) << ( ( 8 * 3 ) - ( 8 * sizePos ) );
                sizePos++;
                if ( sizePos >= 4 ) {
                    readingSize = false;
                    sizePos = 0;
                    ensureFramebufferCapacity ( size );
                    bb.clear();
                    bb.limit ( size );
                }
            } else {
                bb.put ( b );
                if ( !bb.hasRemaining() ) {
                    bb.flip();
                    ByteBuffer newbuf = ByteBuffer.allocateDirect ( bb.limit() );
                    newbuf.put ( bb ).flip();
                    bbq.offer ( newbuf );
                    inFrame = false;
                    readingSize = false;
                    size = 0;
                    sizePos = 0;
                    chain = 0;
                    bb.clear();
                    if ( KokuKS.DEBUG_MODE ) {
                        System.out.println ( "FIS: complete object" );
                    }
                }
            }
        }
    }
}
