package misc;
import java.io.OutputStream;
import java.nio.BufferOverflowException;
import java.nio.ByteBuffer;
import org.apache.mina.core.buffer.IoBuffer;
public class ByteBufferOutputStream extends OutputStream {
    public ByteBufferOutputStream () {
        _buffer = IoBuffer.allocate ( INITIAL_BUFFER_SIZE );
    }
    public IoBuffer getBuffer () {
        return _buffer;
    }
    public IoBuffer flip () {
        return _buffer.flip();
    }
    public void reset () {
        _buffer.clear();
    }
    @Override
    public void write ( int b ) {
        try {
            _buffer.put ( ( byte ) b );
        } catch ( BufferOverflowException boe ) {
            expand ( 1 );
            _buffer.put ( ( byte ) b );
        }
    }
    @Override
    public void write ( byte[] b, int off, int len ) {
        if ( ( off < 0 ) || ( off > b.length ) || ( len < 0 ) ||
                ( ( off + len ) > b.length ) || ( ( off + len ) < 0 ) ) {
            throw new IndexOutOfBoundsException();
        } else if ( len == 0 ) {
            return;
        }
        try {
            _buffer.put ( b, off, len );
        } catch ( BufferOverflowException boe ) {
            expand ( len );
            _buffer.put ( b, off, len );
        }
    }
    protected final void expand ( int needed ) {
        _buffer.expand ( needed );
    }
    protected IoBuffer _buffer;
    protected static final int INITIAL_BUFFER_SIZE = 32;
}
