package demo;
import com.vividsolutions.jts.geom.Coordinate;
import java.util.ArrayList;
import java.util.List;
public class CatmullRom {
    public static List<Coordinate> interpolate ( List<Coordinate> coordinates, int pointsPerSegment )
    throws Exception {
        List<Coordinate> vertices = new ArrayList<>();
        for ( Coordinate c : coordinates ) {
            vertices.add ( new Coordinate ( c.x, c.y ) );
        }
        if ( pointsPerSegment < 2 ) {
            throw new Exception ( "The pointsPerSegment parameter must be greater than 2, since 2 points is just the linear segment." );
        }
        if ( vertices.size() < 3 ) {
            return vertices;
        }
        boolean isClosed = vertices.get ( 0 ).x == vertices.get ( vertices.size() - 1 ).x
                           && vertices.get ( 0 ).y == vertices.get ( vertices.size() - 1 ).y;
        if ( isClosed ) {
            Coordinate p2 = new Coordinate ( vertices.get ( 1 ) );
            Coordinate pn1 = new Coordinate ( vertices.get ( vertices.size() - 2 ) );
            vertices.add ( 0, pn1 );
            vertices.add ( p2 );
        } else {
            double dx = vertices.get ( 1 ).x - vertices.get ( 0 ).x;
            double dy = vertices.get ( 1 ).y - vertices.get ( 0 ).y;
            double x1 = vertices.get ( 0 ).x - dx;
            double y1 = vertices.get ( 0 ).y - dy;
            Coordinate start = new Coordinate ( x1, y1 );
            int n = vertices.size() - 1;
            dx = vertices.get ( n ).x - vertices.get ( n - 1 ).x;
            dy = vertices.get ( n ).y - vertices.get ( n - 1 ).y;
            double xn = vertices.get ( n ).x + dx;
            double yn = vertices.get ( n ).y + dy;
            Coordinate end = new Coordinate ( xn, yn );
            vertices.add ( 0, start );
            vertices.add ( end );
        }
        List<Coordinate> result = new ArrayList<>();
        for ( int i = 0; i < vertices.size() - 3; i++ ) {
            List<Coordinate> points = interpolate ( vertices, i, pointsPerSegment );
            if ( result.size() > 0 ) {
                points.remove ( 0 );
            }
            result.addAll ( points );
        }
        return result;
    }
    public static List<Coordinate> interpolate ( List<Coordinate> points, int index, int pointsPerSegment ) {
        List<Coordinate> result = new ArrayList<>();
        double[] x = new double[4];
        double[] y = new double[4];
        double[] time = new double[4];
        for ( int i = 0; i < 4; i++ ) {
            x[i] = points.get ( index + i ).x;
            y[i] = points.get ( index + i ).y;
            time[i] = i;
        }
        double tstart;
        double tend;
        double total = 0;
        for ( int i = 1; i < 4; i++ ) {
            double dx = x[i] - x[i - 1];
            double dy = y[i] - y[i - 1];
            total += Math.pow ( dx * dx + dy * dy, .25 );
            time[i] = total;
        }
        tstart = time[1];
        tend = time[2];
        int segments = pointsPerSegment - 1;
        result.add ( points.get ( index + 1 ) );
        for ( int i = 1; i < segments; i++ ) {
            double xi = interpolate ( x, time, tstart + ( i * ( tend - tstart ) ) / segments );
            double yi = interpolate ( y, time, tstart + ( i * ( tend - tstart ) ) / segments );
            result.add ( new Coordinate ( xi, yi ) );
        }
        result.add ( points.get ( index + 2 ) );
        return result;
    }
    public static double interpolate ( double[] p, double[] time, double t ) {
        double L01 = p[0] * ( time[1] - t ) / ( time[1] - time[0] ) + p[1] * ( t - time[0] ) / ( time[1] - time[0] );
        double L12 = p[1] * ( time[2] - t ) / ( time[2] - time[1] ) + p[2] * ( t - time[1] ) / ( time[2] - time[1] );
        double L23 = p[2] * ( time[3] - t ) / ( time[3] - time[2] ) + p[3] * ( t - time[2] ) / ( time[3] - time[2] );
        double L012 = L01 * ( time[2] - t ) / ( time[2] - time[0] ) + L12 * ( t - time[0] ) / ( time[2] - time[0] );
        double L123 = L12 * ( time[3] - t ) / ( time[3] - time[1] ) + L23 * ( t - time[1] ) / ( time[3] - time[1] );
        double C12 = L012 * ( time[2] - t ) / ( time[2] - time[1] ) + L123 * ( t - time[1] ) / ( time[2] - time[1] );
        return C12;
    }
}
