public abstract class JsonNumEquivalence
    extends Equivalence<JsonNode> {
    protected abstract boolean doNumEquivalent ( final JsonNode a, final JsonNode b );
    protected abstract int doNumHash ( final JsonNode t );
    @Override
    protected final boolean doEquivalent ( final JsonNode a, final JsonNode b ) {
        if ( a.isNumber() && b.isNumber() ) {
            return doNumEquivalent ( a, b );
        }
        final NodeType typeA = NodeType.getNodeType ( a );
        final NodeType typeB = NodeType.getNodeType ( b );
        if ( typeA != typeB ) {
            return false;
        }
        if ( !a.isContainerNode() ) {
            return a.equals ( b );
        }
        if ( a.size() != b.size() ) {
            return false;
        }
        return typeA == NodeType.ARRAY ? arrayEquals ( a, b ) : objectEquals ( a, b );
    }
    @Override
    protected final int doHash ( final JsonNode t ) {
        if ( t.isNumber() ) {
            return doNumHash ( t );
        }
        if ( !t.isContainerNode() ) {
            return t.hashCode();
        }
        int ret = 0;
        if ( t.size() == 0 ) {
            return ret;
        }
        if ( t.isArray() ) {
            for ( final JsonNode element : t ) {
                ret = 31 * ret + doHash ( element );
            }
            return ret;
        }
        final Iterator<Map.Entry<String, JsonNode>> iterator = t.fields();
        Map.Entry<String, JsonNode> entry;
        while ( iterator.hasNext() ) {
            entry = iterator.next();
            ret = 31 * ret
                  + ( entry.getKey().hashCode() ^ doHash ( entry.getValue() ) );
        }
        return ret;
    }
    private boolean arrayEquals ( final JsonNode a, final JsonNode b ) {
        final int size = a.size();
        for ( int i = 0; i < size; i++ )
            if ( !doEquivalent ( a.get ( i ), b.get ( i ) ) ) {
                return false;
            }
        return true;
    }
    private boolean objectEquals ( final JsonNode a, final JsonNode b ) {
        final Set<String> keys = Sets.newHashSet ( a.fieldNames() );
        final Set<String> set = Sets.newHashSet ( b.fieldNames() );
        if ( !set.equals ( keys ) ) {
            return false;
        }
        for ( final String key : keys )
            if ( !doEquivalent ( a.get ( key ), b.get ( key ) ) ) {
                return false;
            }
        return true;
    }
}
