import java.awt.BorderLayout;
import javax.swing.DefaultCellEditor;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import java.awt.Component;
import java.awt.Dimension;
import javax.swing.event.CellEditorListener;
import javax.swing.event.ChangeEvent;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableModel;
public class TableRenderDemo extends JPanel {
    public TableRenderDemo() {
        super ( new BorderLayout() );
        JTable table = new JTable ( new MyTableModel() );
        table.setPreferredScrollableViewportSize ( new Dimension ( 500, 70 ) );
        table.setFillsViewportHeight ( true );
        JScrollPane scrollPane = new JScrollPane ( table );
        initColumnSizes ( table );
        setUpSportColumn ( table, table.getColumnModel().getColumn ( 2 ) );
        add ( scrollPane );
    }
    private void initColumnSizes ( JTable table ) {
        MyTableModel model = ( MyTableModel ) table.getModel();
        model.addTableModelListener ( new TableModelListener() {
            @Override
            public void tableChanged ( TableModelEvent e ) {
                int type = e.getType();
                switch ( type ) {
                case TableModelEvent.UPDATE:
                    if ( e.getFirstRow() - e.getLastRow() == 0 ) {
                        TableModel model = ( TableModel ) e.getSource();
                        int row = e.getFirstRow();
                        int col = e.getColumn();
                        System.out.println ( "Update " + row + "x" + col + " = " + model.getValueAt ( row, col ) );
                    }
                    break;
                }
            }
        } );
        TableColumn column = null;
        Component comp = null;
        int headerWidth = 0;
        int cellWidth = 0;
        Object[] longValues = model.longValues;
        TableCellRenderer headerRenderer
            = table.getTableHeader().getDefaultRenderer();
        for ( int i = 0; i < 5; i++ ) {
            column = table.getColumnModel().getColumn ( i );
            comp = headerRenderer.getTableCellRendererComponent (
                       null, column.getHeaderValue(),
                       false, false, 0, 0 );
            headerWidth = comp.getPreferredSize().width;
            comp = table.getDefaultRenderer ( model.getColumnClass ( i ) ).
                   getTableCellRendererComponent (
                       table, longValues[i],
                       false, false, 0, i );
            cellWidth = comp.getPreferredSize().width;
            column.setPreferredWidth ( Math.max ( headerWidth, cellWidth ) );
        }
    }
    public void setUpSportColumn ( final JTable table,
                                   TableColumn sportColumn ) {
        JComboBox comboBox = new JComboBox();
        comboBox.addItem ( "Snowboarding" );
        comboBox.addItem ( "Rowing" );
        comboBox.addItem ( "Knitting" );
        comboBox.addItem ( "Speed reading" );
        comboBox.addItem ( "Pool" );
        comboBox.addItem ( "None of the above" );
        TableCellEditor editor = new DefaultCellEditor ( comboBox );
        editor.addCellEditorListener ( new CellEditorListener() {
            @Override
            public void editingStopped ( ChangeEvent e ) {
                System.out.println ( "Editing stopped" );
                TableCellEditor editor = ( TableCellEditor ) e.getSource();
                Object value = editor.getCellEditorValue();
                System.out.println ( "Selected value = " + value );
            }
            @Override
            public void editingCanceled ( ChangeEvent e ) {
            }
        } );
        sportColumn.setCellEditor ( editor );
        DefaultTableCellRenderer renderer
            = new DefaultTableCellRenderer();
        renderer.setToolTipText ( "Click for combo box" );
        sportColumn.setCellRenderer ( renderer );
    }
    class MyTableModel extends AbstractTableModel {
        private String[] columnNames = {"First Name",
                                        "Last Name",
                                        "Sport",
                                        "# of Years",
                                        "Vegetarian"
                                       };
        private Object[][] data = {
            {
                "Kathy", "Smith",
                "Snowboarding", new Integer ( 5 ), new Boolean ( false )
            },
            {
                "John", "Doe",
                "Rowing", new Integer ( 3 ), new Boolean ( true )
            },
            {
                "Sue", "Black",
                "Knitting", new Integer ( 2 ), new Boolean ( false )
            },
            {
                "Jane", "White",
                "Speed reading", new Integer ( 20 ), new Boolean ( true )
            },
            {
                "Joe", "Brown",
                "Pool", new Integer ( 10 ), new Boolean ( false )
            }
        };
        public final Object[] longValues = {"Jane", "Kathy",
                                            "None of the above",
                                            new Integer ( 20 ), Boolean.TRUE
                                           };
        @Override
        public int getColumnCount() {
            return columnNames.length;
        }
        @Override
        public int getRowCount() {
            return data.length;
        }
        @Override
        public String getColumnName ( int col ) {
            return columnNames[col];
        }
        @Override
        public Object getValueAt ( int row, int col ) {
            return data[row][col];
        }
        @Override
        public Class getColumnClass ( int c ) {
            return getValueAt ( 0, c ).getClass();
        }
        @Override
        public boolean isCellEditable ( int row, int col ) {
            if ( col < 2 ) {
                return false;
            } else {
                return true;
            }
        }
        @Override
        public void setValueAt ( Object value, int row, int col ) {
            data[row][col] = value;
            fireTableCellUpdated ( row, col );
        }
    }
    private static void createAndShowGUI() {
        JFrame frame = new JFrame ( "TableRenderDemo" );
        frame.setDefaultCloseOperation ( JFrame.EXIT_ON_CLOSE );
        TableRenderDemo newContentPane = new TableRenderDemo();
        newContentPane.setOpaque ( true );
        frame.setContentPane ( newContentPane );
        frame.pack();
        frame.setVisible ( true );
    }
    public static void main ( String[] args ) {
        javax.swing.SwingUtilities.invokeLater ( new Runnable() {
            public void run() {
                createAndShowGUI();
            }
        } );
    }
}
