public class ForkJoinSort {
    private static final int LIST_SIZE = 10000;
    private static final int SORT_THRESHOLD = 10;
    private static ForkJoinPool forkJoinPool = new ForkJoinPool();
    public static class MergeSortTask extends RecursiveTask<List<Integer>> {
        private final List<Integer> victim;
        public MergeSortTask ( List<Integer> victim ) {
            this.victim = victim;
        }
        @Override
        protected List<Integer> compute() {
            if ( victim.size() < SORT_THRESHOLD ) {
                Collections.sort ( victim );
                return victim;
            }
            MergeSortTask leftTask = new MergeSortTask ( victim.subList ( 0, victim.size() / 2 ) );
            MergeSortTask rightTask = new MergeSortTask ( victim.subList ( victim.size() / 2, victim.size() ) );
            forkJoinPool.submit ( leftTask );
            forkJoinPool.submit ( rightTask );
            return merge ( leftTask.join(), rightTask.join() );
        }
        public List<Integer> merge ( List<Integer> left, List<Integer> right ) {
            List<Integer> result = new ArrayList<Integer> ( left.size() + right.size() );
            Iterator<Integer> leftIterator = left.iterator();
            Iterator<Integer> rightIterator = right.iterator();
            Integer fromLeft = null;
            Integer fromRight = null;
            while ( leftIterator.hasNext() || rightIterator.hasNext() ) {
                fromLeft = fromLeft == null ? leftIterator.hasNext() ? leftIterator.next() : null : fromLeft;
                fromRight = fromRight == null ? rightIterator.hasNext() ? rightIterator.next() : null : fromRight;
                if ( fromLeft != null && ( fromRight == null || fromLeft <= fromRight ) ) {
                    result.add ( fromLeft );
                    fromLeft = null;
                } else if ( fromRight != null && ( fromLeft == null || fromRight <= fromLeft ) ) {
                    result.add ( fromRight );
                    fromRight = null;
                }
            }
            return result;
        }
    }
    public static void main ( String[] args ) throws Exception {
        SecureRandom random = new SecureRandom();
        List<Integer> victim = new ArrayList<Integer> ( LIST_SIZE );
        for ( int i = 0; i < LIST_SIZE; ++i ) {
            victim.add ( random.nextInt() );
        }
        long timeMark = System.currentTimeMillis();
        MergeSortTask task = new MergeSortTask ( victim );
        forkJoinPool.submit ( task );
        List<Integer> probablySorted = task.get();
        timeMark = System.currentTimeMillis() - timeMark;
        for ( int i = 0; i < probablySorted.size() - 1; ++i ) {
            if ( probablySorted.get ( i ) > probablySorted.get ( i + 1 ) ) {
                throw new IllegalStateException ( "Sorting failed :(" );
            }
        }
        System.out.println ( "Sorting " + LIST_SIZE + " random numbers using merge sort algorithm in " + Runtime.getRuntime().availableProcessors() + " threads took " + timeMark + " ms." );
    }
}
