public class Sector {
    private final int SIZE;
    private int[][] allocationTable;
    private Set<SubSector> subSectors = new HashSet<>();
    public Sector ( int size ) {
        SIZE = size;
        allocationTable = new int[SIZE][SIZE];
    }
    public boolean isAllocated ( int x, int y ) {
        return ( allocationTable[x][y] != 0 );
    }
    public int getNonAllocatedSpaceAt ( int x, int y ) {
        int space = Math.min ( SIZE - x, SIZE - y );
        for ( int i = x; i < space + x ; i++ ) {
            for ( int j = y; j < space + y; j++ ) {
                if ( isAllocated ( i, j ) ) {
                    return Math.max ( i - x, j - y );
                }
            }
        }
        return space;
    }
    public SubSector getSubSectorAt ( int x, int y ) {
        for ( SubSector s : subSectors )
            if ( s.number == allocationTable[x][y] ) {
                return s;
            }
        return null;
    }
    public SubSector getSubSector ( int subSector ) {
        for ( SubSector s : subSectors )
            if ( s.number == subSector ) {
                return s;
            }
        return null;
    }
    public boolean allocate ( int x, int y, int size, int subSector ) {
        if ( getNonAllocatedSpaceAt ( x, y ) < size || !subSectors.add ( new SubSector ( x, y, size, subSector ) ) ) {
            return false;
        }
        for ( int i = x; i < size + x; i++ ) {
            for ( int j = y; j < size + y; j++ ) {
                allocationTable[i][j] = subSector;
            }
        }
        return true;
    }
    public boolean deallocate ( int subSector ) {
        return remove ( getSubSector ( subSector ) );
    }
    public boolean deallocate ( int x, int y ) {
        return remove ( getSubSectorAt ( x, y ) );
    }
    private boolean remove ( SubSector sub ) {
        if ( !subSectors.remove ( sub ) ) {
            return false;
        }
        for ( int i = sub.x; i < sub.size + sub.x; i++ ) {
            for ( int j = sub.y; j < sub.size + sub.y; j++ ) {
                allocationTable[i][j] = 0;
            }
        }
        return true;
    }
    public int[][] getAllocationTable() {
        return allocationTable;
    }
    public int getSize() {
        return SIZE;
    }
    public Set<SubSector> getSubSectors() {
        return subSectors;
    }
    public class SubSector {
        public final int x;
        public final int y;
        public final int size;
        public final int number;
        private SubSector ( int x, int y, int size, int number ) {
            this.x = x;
            this.y = y;
            this.size = size;
            this.number = number;
        }
    }
}
