typedef std::map<std::pair<int, int>, std::pair<size_t, int> > EdgeMap;
static bool Get_EdgeID ( size_t & eid, int a, int b, EdgeMap & edges ) {
    std::pair<int, int> e ( std::min ( a, b ), std::max ( a, b ) );
    EdgeMap::iterator it = edges.find ( e );
    if ( it == edges.end() ) {
        edges[e] = std::make_pair ( eid = edges.size(), 1 );
        return true;
    } else {
        eid = ( * it ).second.first;
        ++ ( * it ).second.second;
        return false;
    }
}
void CatClark ( std::vector<std::vector<int> > & src_quads, std::vector<float> & src_verts ) {
    const static float vpw[4] = {9.0f, 3.0f, 1.0f, 3.0f};
    const static float epw[4] = {3.0f, 3.0f, 1.0f, 1.0f};
    std::vector<std::vector<int> > dst_quads ( src_quads.size() * 4, std::vector<int> ( 4 ) );
    std::vector<float> dst_verts ( src_verts.size() + src_quads.size() * 3, 0 );
    EdgeMap edges;
    std::vector<int> face_valences ( src_verts.size() / 3, 0 );
    const size_t off_vp = src_quads.size(), off_ep = off_vp + src_verts.size() / 3;
    for ( size_t j = 0; j < off_vp; ++ j ) {
        assert ( src_quads[j].size() == 4 );
        size_t eid[4];
        for ( int k = 0; k < 4; ++ k ) {
            int quad[4];
            for ( int i = 0; i < 4; ++ i ) {
                quad[i] = src_quads[j][ ( i + k ) & 3];
            }
            if ( Get_EdgeID ( eid[k], quad[0], quad[1], edges ) ) {
                dst_verts.insert ( dst_verts.end(), 3, .0f );
            }
            ++ face_valences[quad[0]];
            for ( int n = 0; n < 3; ++ n ) {
                dst_verts[j * 3 + n] += 0.25f * src_verts[quad[0] * 3 + n];
            }
            for ( int i = 0; i < 4; ++ i ) {
                for ( int n = 0; n < 3; ++ n ) {
                    dst_verts[ ( off_vp + quad[0] ) * 3 + n] += vpw[i] * src_verts[quad[i] * 3 + n];
                    dst_verts[ ( off_ep + eid[k] ) * 3 + n] += epw[i] * src_verts[quad[i] * 3 + n];
                }
            }
        }
        for ( int k = 0; k < 4; ++ k ) {
            dst_quads[4 * j + k][0] = j;
            dst_quads[4 * j + k][4] = off_ep + eid[ ( 3 + k ) & 3];
            dst_quads[4 * j + k][5] = off_ep + eid[ ( 0 + k ) & 3];
            dst_quads[4 * j + k][6] = off_vp + src_quads[j][k];
        }
    }
    for ( size_t j = 0, n = src_verts.size() / 3; j < n; ++ j ) {
        for ( int n = 0; n < 3; ++ n ) {
            dst_verts[ ( off_vp + j ) * 3 + n] *= 0.0625f / float ( face_valences[j] );
        }
    }
    for ( EdgeMap::const_iterator it = edges.begin(), e = edges.end(); it != e; ++ it ) {
        size_t j = ( * it ).second.first;
        float rvalence = 0.1250f / float ( ( * it ).second.second );
        for ( int n = 0; n < 3; ++ n ) {
            dst_verts[ ( off_ep + j ) * 3 + n] *= rvalence;
        }
    }
    dst_quads.swap ( src_quads );
    dst_verts.swap ( src_verts );
}
