import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.EventQueue;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.geom.Area;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import javax.imageio.ImageIO;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
public class ShowArea {
    public static void main ( String[] args ) {
        new ShowArea();
    }
    public ShowArea() {
        EventQueue.invokeLater ( new Runnable() {
            @Override
            public void run() {
                try {
                    UIManager.setLookAndFeel ( UIManager.getSystemLookAndFeelClassName() );
                } catch ( ClassNotFoundException | InstantiationException | IllegalAccessException | UnsupportedLookAndFeelException ex ) {
                    ex.printStackTrace();
                }
                JFrame frame = new JFrame ( "Testing" );
                frame.setDefaultCloseOperation ( JFrame.EXIT_ON_CLOSE );
                frame.add ( new TestPane() );
                frame.pack();
                frame.setLocationRelativeTo ( null );
                frame.setVisible ( true );
            }
        } );
    }
    public class TestPane extends JPanel {
        private BufferedImage img;
        public TestPane() {
            try {
                img = ImageIO.read ( new File ( "sample.png" ) );
                Rectangle bounds = new Rectangle ( 0, 0, img.getWidth(), img.getHeight() );
                Rectangle clip = new Rectangle ( 150, 10, 100, 100 );
                Area area = new Area ( bounds );
                area.subtract ( new Area ( clip ) );
                Graphics2D g2d = img.createGraphics();
                g2d.setColor ( Color.BLACK );
                g2d.setComposite ( AlphaComposite.SrcOver.derive ( 0.5f ) );
                g2d.fill ( area );
                g2d.dispose();
            } catch ( IOException ex ) {
                ex.printStackTrace();
            }
        }
        @Override
        public Dimension getPreferredSize() {
            return img == null ? new Dimension ( 200, 200 ) :  new Dimension ( img.getWidth(), img.getHeight() );
        }
        @Override
        protected void paintComponent ( Graphics g ) {
            super.paintComponent ( g );
            Graphics2D g2d = ( Graphics2D ) g.create();
            int x = ( getWidth() - img.getWidth() ) / 2;
            int y = ( getHeight() - img.getHeight() ) / 2;
            g2d.drawImage ( img, x, y, this );
            g2d.dispose();
        }
    }
}
