class JavaStreamBuf : public std::streambuf {
    std::vector<char> buff;
    unsigned int size;
    jobject handle;
    JNIEnv * env;
    public:
    JavaStreamBuf ( JNIEnv * env, jobject  cppstream, unsigned int buffsize = 50 ) {
        handle = env->NewGlobalRef ( cppstream );
        this->env = env;
        this->size = size;
        this->setbuf ( 0, 0 );
    }
    virtual int_type overflow ( int_type in  = traits_type::eof() ) {
        if ( in == std::ios::traits_type::eof() || buff.size() == size ) {
            this->std::streambuf::overflow ( in );
            if ( in != EOF ) {
                buff.push_back ( in );
            }
            jbyteArray o = env->NewByteArray ( buff.size() );
            env->SetByteArrayRegion ( o, 0, buff.size(), ( jbyte * ) & buff[0] );
            jmethodID id = env->GetMethodID ( env->GetObjectClass ( handle ), "push", "([B)V" );
            env->CallVoidMethod ( handle, id, o );
            if ( in == EOF ) {
                env->CallVoidMethod ( handle, id, NULL );
            }
            buff.clear();
        } else {
            buff.push_back ( in );
        }
        return in;
    }
    virtual ~JavaStreamBuf() {
        overflow();
        env->DeleteGlobalRef ( handle );
    }
    package jx;
    import java.io.ByteArrayInputStream;
    import java.io.IOException;
    import java.io.InputStream;
    import java.io.InterruptedIOException;
    import java.nio.ByteBuffer;
    import java.util.ArrayList;
    import java.util.Arrays;
    import java.util.List;
    public class CPPStream extends InputStream {
        List<Byte> data = new ArrayList<Byte>();
        int off = 0;
        private boolean endflag = false;
        public void push ( byte[] d ) {
            synchronized ( data ) {
                if ( d == null ) {
                    this.endflag = true;
                } else {
                    for ( int i = 0; i < d.length; ++i ) {
                        data.add ( d[i] );
                    }
                }
            }
        }
        @Override
        public int read() throws IOException {
            synchronized ( data ) {
                while ( data.isEmpty() && !endflag ) {
                    try {
                        data.wait();
                    } catch ( InterruptedException e ) {
                        throw new InterruptedIOException();
                    }
                }
            }
            if ( endflag ) {
                return -1;
            } else {
                return data.remove ( 0 );
            }
        }
    }
