import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.net.URL;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.imageio.ImageIO;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;
import javax.swing.Timer;
public class DrawImageDemo extends JPanel {
    private static final int SPRITE_ROWS = 5;
    private static final int SPRITE_COLUMNS = 2;
    private static final int DELAY = 120;
    private AnimationSprite animationSprite;
    public DrawImageDemo() {
        animationSprite = new AnimationSprite ( getImage(), DrawImageDemo.this );
        Timer timer = new Timer ( DELAY, new ActionListener() {
            public void actionPerformed ( ActionEvent e ) {
                animationSprite.moveDestinationCoords();
                animationSprite.moveSourceCoords();
                repaint();
            }
        } );
        timer.start();
    }
    private BufferedImage getImage() {
        BufferedImage image = null;
        try {
            image = ImageIO.read ( new URL ( "http://i.stack.imgur.com/DG2IN.png" ) );
        } catch ( IOException ex ) {
            Logger.getLogger ( DrawImageDemo.class.getName() ).log ( Level.SEVERE, null, ex );
        }
        return image;
    }
    @Override
    protected void paintComponent ( Graphics g ) {
        super.paintComponent ( g );
        animationSprite.drawSprite ( g );
    }
    @Override
    public Dimension getPreferredSize() {
        return new Dimension ( 660, 300 );
    }
    private class AnimationSprite {
        private static final int SPRITE_ROWS = 4;
        private static final int SPRITE_COLUMNS = 6;
        private static final int DX_INCREMENT = 15;
        private int SRC_HEIGHT;
        private int SRC_WIDTH;
        private BufferedImage sprite;
        private JComponent surface;
        private int dx1, dy1, dx2, dy2;
        private int sx1, sy1, sx2, sy2;
        public AnimationSprite ( BufferedImage sprite, JComponent surface ) {
            this.sprite = sprite;
            this.surface = surface;
            SRC_HEIGHT = sprite.getHeight() / SPRITE_ROWS;
            SRC_WIDTH = sprite.getWidth() / SPRITE_COLUMNS;
            dy1 = 100;
            dy2 = 100 + SRC_HEIGHT;
            dx1 = 0;
            dx2 = 0 + SRC_WIDTH;
            sx1 = 0;
            sy1 = 0;
            sx2 = 0 + SRC_WIDTH;
            sy2 = 0 + SRC_HEIGHT;
        }
        public void drawSprite ( Graphics g ) {
            g.drawImage ( sprite, dx1, dy1, dx2, dy2, sx1, sy1, sx2, sy2, surface );
        }
        public void moveDestinationCoords() {
            if ( dx1 > surface.getWidth() ) {
                dx1 = 0 - SRC_WIDTH;
                dx2 = 0;
            } else {
                dx1 += DX_INCREMENT;
                dx2 += DX_INCREMENT;
            }
        }
        public void moveSourceCoords() {
            if ( sx1 >= sprite.getWidth() - SRC_WIDTH - 5 ) {
                sx1 = 0;
                sx2 = sx1 + SRC_WIDTH;
                if ( sy1 >= SRC_HEIGHT - 5 ) {
                    sy1 = 0;
                    sy2 = sy1 + SRC_HEIGHT;
                } else {
                    sy1 += SRC_HEIGHT;
                    sy2 = sy1 + SRC_HEIGHT;
                }
            } else {
                sx1 += SRC_WIDTH;
                sx2 = sx1 + SRC_WIDTH;
            }
        }
    }
    public static void main ( String[] args ) {
        SwingUtilities.invokeLater ( new Runnable() {
            public void run() {
                JFrame frame = new JFrame();
                frame.add ( new DrawImageDemo() );
                frame.setDefaultCloseOperation ( JFrame.EXIT_ON_CLOSE );
                frame.pack();
                frame.setLocationRelativeTo ( null );
                frame.setVisible ( true );
            }
        } );
    }
}
