import org.eclipse.swt.SWT;
import org.eclipse.swt.events.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.program.Program;
import org.eclipse.swt.widgets.*;
public class ShowPrograms {
    public void run() {
        Display display = new Display();
        Shell shell = new Shell ( display );
        shell.setText ( "Show Programs" );
        createContents ( shell );
        shell.open();
        while ( !shell.isDisposed() ) {
            if ( !display.readAndDispatch() ) {
                display.sleep();
            }
        }
        display.dispose();
    }
    private void createContents ( Shell shell ) {
        shell.setLayout ( new GridLayout ( 2, false ) );
        new Label ( shell, SWT.NONE ).setText ( "Extension:" );
        Combo extensionsCombo = new Combo ( shell, SWT.BORDER | SWT.READ_ONLY );
        extensionsCombo.setLayoutData ( new GridData ( GridData.FILL_HORIZONTAL ) );
        new Label ( shell, SWT.NONE ).setText ( "Program:" );
        final Label programName = new Label ( shell, SWT.NONE );
        programName.setLayoutData ( new GridData ( GridData.FILL_HORIZONTAL ) );
        String[] extensions = Program.getExtensions();
        for ( int i = 0, n = extensions.length; i < n; i++ ) {
            extensionsCombo.add ( extensions[i] );
        }
        extensionsCombo.addSelectionListener ( new SelectionAdapter() {
            public void widgetSelected ( SelectionEvent event ) {
                Combo combo = ( Combo ) event.widget;
                Program program = Program.findProgram ( combo.getText() );
                programName.setText ( program == null ? "(None)" : program.getName() );
            }
        } );
        List allPrograms = new List ( shell, SWT.SINGLE | SWT.BORDER | SWT.H_SCROLL
                                      | SWT.V_SCROLL );
        GridData data = new GridData ( GridData.FILL_BOTH );
        data.horizontalSpan = 2;
        allPrograms.setLayoutData ( data );
        Program[] programs = Program.getPrograms();
        for ( int i = 0, n = programs.length; i < n; i++ ) {
            String name = programs[i].getName();
            allPrograms.add ( name );
            allPrograms.setData ( name, programs[i] );
        }
        new Label ( shell, SWT.NONE ).setText ( "Data File:" );
        final Text dataFile = new Text ( shell, SWT.BORDER );
        dataFile.setLayoutData ( new GridData ( GridData.FILL_HORIZONTAL ) );
        allPrograms.addMouseListener ( new MouseAdapter() {
            public void mouseDoubleClick ( MouseEvent event ) {
                List list = ( List ) event.widget;
                if ( list.getSelectionCount() > 0 ) {
                    String programName = list.getSelection() [0];
                    Program program = ( Program ) list.getData ( programName );
                    program.execute ( dataFile.getText() );
                }
            }
        } );
        Button launch = new Button ( shell, SWT.PUSH );
        data = new GridData ( GridData.FILL_HORIZONTAL );
        data.horizontalSpan = 2;
        launch.setLayoutData ( data );
        launch.setText ( "Use Program.launch() instead of Program.execute()" );
        launch.addSelectionListener ( new SelectionAdapter() {
            public void widgetSelected ( SelectionEvent event ) {
                Program.launch ( dataFile.getText() );
            }
        } );
    }
    public static void main ( String[] args ) {
        new ShowPrograms().run();
    }
}
