import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.NoSuchElementException;
import java.util.StringTokenizer;
import stackoverflow.objload.ObjectLoader.LineParser;
import stackoverflow.objload.ObjectLoader.Material;
import stackoverflow.objload.ObjectLoader.MtlLineParser;
import stackoverflow.objload.ObjectLoader.ObjLineParser;
public class ObjectLoaderTest {
    public static void main ( String[] args ) {
        ObjectLoader objLoader = new ObjectLoader();
        try {
            PolygonGroup object = objLoader.loadObject ( "res/", "coffeCup.obj" );
        } catch ( IOException e ) {
            e.printStackTrace();
        }
    }
}
class PolygonGroup {
    public PolygonGroup ( String name ) {
    }
    public PolygonGroup() {
    }
    public void setFilename ( String filename ) {
    }
    public void addPolygonGroup ( PolygonGroup currentGroup ) {
    }
}
class Vector3D {
    public Vector3D ( float parseFloat, float parseFloat2, float parseFloat3 ) {
    }
}
class ObjectLoader {
    public static class Material {
        public File sourceFile;
    }
    protected interface LineParser {
        public void parseLine ( String line ) throws IOException, NumberFormatException, NoSuchElementException;
    }
    protected File path;
    protected List vertices;
    protected Material currentMaterial;
    protected HashMap materials;
    protected List lights;
    protected float ambientLightIntensity;
    protected HashMap parsers;
    private PolygonGroup object;
    private PolygonGroup currentGroup;
    public ObjectLoader() {
        materials = new HashMap();
        vertices = new ArrayList();
        parsers = new HashMap();
        parsers.put ( "obj", new ObjLineParser() );
        parsers.put ( "mtl", new MtlLineParser() );
        currentMaterial = null;
        setLights ( new ArrayList(), 1 );
    }
    public void setLights ( List lights, float ambientLightIntensity ) {
        this.lights = lights;
        this.ambientLightIntensity = ambientLightIntensity;
    }
    public PolygonGroup loadObject ( String parent, String filename ) throws IOException {
        object = new PolygonGroup();
        object.setFilename ( filename );
        path = new File ( parent );
        vertices.clear();
        currentGroup = object;
        parseFile ( filename );
        return object;
    }
    protected Vector3D getVector ( String indexStr ) {
        int index = Integer.parseInt ( indexStr );
        if ( index < 0 ) {
            index = vertices.size() + index + 1;
        }
        return ( Vector3D ) vertices.get ( index - 1 );
    }
    protected void parseFile ( String filename ) throws IOException {
        File file = new File ( path, filename );
        System.out.println ( "Reading " + file + ", exists " + file.exists() );
        BufferedReader reader = new BufferedReader ( new FileReader ( file ) );
        LineParser parser = null;
        int extIndex = filename.lastIndexOf ( '.' );
        if ( extIndex != -1 ) {
            String ext = filename.substring ( extIndex + 1 );
            parser = ( LineParser ) parsers.get ( ext.toLowerCase() );
        }
        if ( parser == null ) {
            parser = ( LineParser ) parsers.get ( "obj" );
        }
        while ( true ) {
            String line = reader.readLine();
            if ( line == null ) {
                reader.close();
                return;
            }
            line = line.trim();
            if ( line.length() > 0 && !line.startsWith ( "#" ) ) {
                try {
                    parser.parseLine ( line );
                } catch ( NumberFormatException ex ) {
                    throw new IOException ( ex.getMessage() );
                } catch ( NoSuchElementException ex ) {
                    throw new IOException ( ex.getMessage() );
                }
            }
        }
    }
    protected class ObjLineParser implements LineParser {
        public void parseLine ( String line ) throws IOException, NumberFormatException, NoSuchElementException {
            StringTokenizer tokenizer = new StringTokenizer ( line );
            String command = tokenizer.nextToken();
            if ( command.equals ( "v" ) ) {
                vertices.add ( new Vector3D ( Float.parseFloat ( tokenizer.nextToken() ), Float.parseFloat ( tokenizer.nextToken() ), Float.parseFloat ( tokenizer.nextToken() ) ) );
            } else if ( command.equals ( "f" ) ) {
                List currVertices = new ArrayList();
                while ( tokenizer.hasMoreTokens() ) {
                    String indexStr = tokenizer.nextToken();
                    int endIndex = indexStr.indexOf ( '/' );
                    if ( endIndex != -1 ) {
                        indexStr = indexStr.substring ( 0, endIndex );
                    }
                    currVertices.add ( getVector ( indexStr ) );
                }
                Vector3D[] array = new Vector3D[currVertices.size()];
                currVertices.toArray ( array );
            } else if ( command.equals ( "g" ) ) {
                if ( tokenizer.hasMoreTokens() ) {
                    String name = tokenizer.nextToken();
                    currentGroup = new PolygonGroup ( name );
                } else {
                    currentGroup = new PolygonGroup();
                }
                object.addPolygonGroup ( currentGroup );
            } else if ( command.equals ( "mtllib" ) ) {
                String name = tokenizer.nextToken();
                parseFile ( name );
            } else if ( command.equals ( "usemtl" ) ) {
                String name = tokenizer.nextToken();
                currentMaterial = ( Material ) materials.get ( name );
                if ( currentMaterial == null ) {
                    System.out.println ( "no material: " + name );
                }
            } else {
            }
        }
    }
    protected class MtlLineParser implements LineParser {
        public void parseLine ( String line ) throws NoSuchElementException {
            StringTokenizer tokenizer = new StringTokenizer ( line );
            String command = tokenizer.nextToken();
            if ( command.equals ( "newmtl" ) ) {
                String name = tokenizer.nextToken();
                currentMaterial = ( Material ) materials.get ( name );
                if ( currentMaterial == null ) {
                    currentMaterial = new Material();
                    materials.put ( name, currentMaterial );
                }
            } else if ( command.equals ( "map_Kd" ) ) {
                String name = tokenizer.nextToken();
                File file = new File ( path, name );
                if ( !file.equals ( currentMaterial.sourceFile ) ) {
                    currentMaterial.sourceFile = file;
                }
            } else {
            }
        }
    }
}
