import java.awt.*;
import java.awt.geom.*;
import javax.swing.*;
import javax.swing.border.AbstractBorder;
class TextBubbleBorder extends AbstractBorder {
    private Color color;
    private int thickness = 4;
    private int radii = 8;
    private int pointerSize = 7;
    private Insets insets = null;
    private BasicStroke stroke = null;
    private int strokePad;
    private int pointerPad = 4;
    RenderingHints hints;
    TextBubbleBorder (
        Color color ) {
        this ( color, 4, 8, 7 );
    }
    TextBubbleBorder (
        Color color, int thickness, int radii, int pointerSize ) {
        this.thickness = thickness;
        this.radii = radii;
        this.pointerSize = pointerSize;
        this.color = color;
        stroke = new BasicStroke ( thickness );
        strokePad = thickness / 2;
        hints = new RenderingHints (
            RenderingHints.KEY_ANTIALIASING,
            RenderingHints.VALUE_ANTIALIAS_ON );
        int pad = radii + strokePad;
        int bottomPad = pad + pointerSize + strokePad;
        insets = new Insets ( pad, pad, bottomPad, pad );
    }
    @Override
    public Insets getBorderInsets ( Component c ) {
        return insets;
    }
    @Override
    public Insets getBorderInsets ( Component c, Insets insets ) {
        return getBorderInsets ( c );
    }
    @Override
    public void paintBorder (
        Component c,
        Graphics g,
        int x, int y,
        int width, int height ) {
        Graphics2D g2 = ( Graphics2D ) g;
        int bottomLineY = height - thickness - pointerSize;
        RoundRectangle2D.Double bubble = new RoundRectangle2D.Double (
            0 + strokePad,
            0 + strokePad,
            width - thickness,
            bottomLineY,
            radii,
            radii
        );
        Polygon pointer = new Polygon();
        pointer.addPoint (
            strokePad + radii + pointerPad,
            bottomLineY );
        pointer.addPoint (
            strokePad + radii + pointerPad + pointerSize,
            bottomLineY );
        pointer.addPoint (
            strokePad + radii + pointerPad + ( pointerSize / 2 ),
            height - strokePad );
        Area area = new Area ( bubble );
        area.add ( new Area ( pointer ) );
        g2.setRenderingHints ( hints );
        Area spareSpace = new Area ( new Rectangle ( 0, 0, width, height ) );
        spareSpace.subtract ( area );
        g2.setClip ( spareSpace );
        g2.clearRect ( 0, 0, width, height );
        g2.setClip ( null );
        g2.setColor ( color );
        g2.setStroke ( stroke );
        g2.draw ( area );
    }
    public static void main ( String[] args ) {
        SwingUtilities.invokeLater ( new Runnable() {
            public void run() {
                JTextField o = new JTextField (
                    "The quick brown fox jumps over the lazy dog!" );
                o.setBorder ( new TextBubbleBorder ( Color.MAGENTA.darker(), 2, 4, 0 ) );
                JOptionPane.showMessageDialog ( null, o );
            }
        } );
    }
}
