import android.os.Build;
import android.support.v4.app.FragmentActivity;
import android.view.View;
import android.view.animation.TranslateAnimation;
import android.widget.FrameLayout;
public class ActivitySlider {
    private final FragmentActivity activity;
    private final View content;
    public ActivitySlider ( FragmentActivity activity ) {
        this.activity = activity;
        this.content = ( View ) activity.findViewById ( android.R.id.content ).getParent();
    }
    public void slideTo ( int x, int y ) {
        final int currentX = getActivityPositionX();
        final int currentY = getActivityPositionY();
        setActivityPosition ( x, y );
        TranslateAnimation animation = new TranslateAnimation ( currentX - x, 0, currentY - y, 0 );
        animation.setDuration ( 500 );
        this.content.startAnimation ( animation );
    }
    public void setActivityPosition ( int x, int y ) {
        if ( Build.VERSION.SDK_INT >= Build.VERSION_CODES.HONEYCOMB ) {
            FrameLayout.LayoutParams contentParams = ( FrameLayout.LayoutParams ) this.content.getLayoutParams();
            contentParams.setMargins ( x, y, -x, -y );
            this.content.setLayoutParams ( contentParams );
        } else {
            this.content.setPadding ( x, y, -x, -y );
        }
    }
    public int getActivityPositionX() {
        if ( Build.VERSION.SDK_INT >= Build.VERSION_CODES.HONEYCOMB ) {
            FrameLayout.LayoutParams contentParams = ( FrameLayout.LayoutParams ) this.content.getLayoutParams();
            return contentParams.leftMargin;
        } else {
            return this.content.getPaddingLeft();
        }
    }
    public int getActivityPositionY() {
        if ( Build.VERSION.SDK_INT >= Build.VERSION_CODES.HONEYCOMB ) {
            FrameLayout.LayoutParams contentParams = ( FrameLayout.LayoutParams ) this.content.getLayoutParams();
            return contentParams.topMargin;
        } else {
            return this.content.getPaddingTop();
        }
    }
}
