import android.graphics.Rect;
import android.os.Build;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentActivity;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentTransaction;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.view.animation.Animation;
import android.view.animation.TranslateAnimation;
import android.widget.FrameLayout;
import at.test.app.R;
import at.test.app.helper.LayoutHelper;
public class ActivitySlider {
    private final FragmentActivity activity;
    private final View content;
    private final FrameLayout menuContainer;
    public ActivitySlider ( FragmentActivity activity ) {
        this.activity = activity;
        this.content = ( View ) activity.findViewById ( android.R.id.content ).getParent();
        ViewGroup parent = ( ViewGroup ) this.content.getParent();
        this.menuContainer = new FrameLayout ( this.activity );
        this.menuContainer.setId ( R.id.flMenuContainer );
        this.menuContainer.setVisibility ( View.GONE );
        parent.addView ( this.menuContainer );
    }
    public <T extends Fragment> void setMenuFragment ( Class<T> cls ) {
        Fragment fragment = Fragment.instantiate ( this.activity, cls.getName() );
        setMenuFragment ( fragment );
    }
    public void setMenuFragment ( Fragment fragment ) {
        FragmentManager manager = this.activity.getSupportFragmentManager();
        FragmentTransaction transaction = manager.beginTransaction();
        transaction.replace ( R.id.flMenuContainer, fragment );
        transaction.commit();
    }
    public boolean isMenuVisible() {
        return this.menuContainer.getVisibility() == View.VISIBLE;
    }
    public void showMenu ( int dpWidth ) {
        final int menuWidth = LayoutHelper.dpToPixel ( this.activity, dpWidth );
        slideTo ( menuWidth, 0 );
        Rect rectangle = new Rect();
        Window window = this.activity.getWindow();
        window.getDecorView().getWindowVisibleDisplayFrame ( rectangle );
        final int statusBarHeight = rectangle.top;
        FrameLayout.LayoutParams fragmentParams = new FrameLayout.LayoutParams ( menuWidth, ViewGroup.LayoutParams.MATCH_PARENT );
        fragmentParams.setMargins ( 0, statusBarHeight, 0, 0 );
        this.menuContainer.setLayoutParams ( fragmentParams );
        if ( !isMenuVisible() ) {
            this.menuContainer.setVisibility ( View.VISIBLE );
            TranslateAnimation animation = new TranslateAnimation ( -menuWidth, 0, 0, 0 );
            animation.setDuration ( 500 );
            this.menuContainer.startAnimation ( animation );
        }
    }
    public void hideMenu() {
        if ( isMenuVisible() ) {
            slideTo ( 0, 0 );
            final int menuWidth = this.menuContainer.getWidth();
            TranslateAnimation menuAnimation = new TranslateAnimation ( 0, -menuWidth, 0, 0 );
            menuAnimation.setDuration ( 500 );
            menuAnimation.setAnimationListener ( new Animation.AnimationListener() {
                @Override
                public void onAnimationStart ( Animation animation ) {
                }
                @Override
                public void onAnimationEnd ( Animation animation ) {
                    menuContainer.setVisibility ( View.GONE );
                }
                @Override
                public void onAnimationRepeat ( Animation animation ) {
                }
            } );
            this.menuContainer.startAnimation ( menuAnimation );
        }
    }
    public void slideTo ( int x, int y ) {
        final int currentX = getActivityPositionX();
        final int currentY = getActivityPositionY();
        setActivityPosition ( x, y );
        TranslateAnimation animation = new TranslateAnimation ( currentX - x, 0, currentY - y, 0 );
        animation.setDuration ( 500 );
        this.content.startAnimation ( animation );
    }
    public void setActivityPosition ( int x, int y ) {
        if ( Build.VERSION.SDK_INT >= Build.VERSION_CODES.HONEYCOMB ) {
            FrameLayout.LayoutParams contentParams = ( FrameLayout.LayoutParams ) this.content.getLayoutParams();
            contentParams.setMargins ( x, y, -x, -y );
            this.content.setLayoutParams ( contentParams );
        } else {
            this.content.setPadding ( x, y, -x, -y );
        }
    }
    public int getActivityPositionX() {
        if ( Build.VERSION.SDK_INT >= Build.VERSION_CODES.HONEYCOMB ) {
            FrameLayout.LayoutParams contentParams = ( FrameLayout.LayoutParams ) this.content.getLayoutParams();
            return contentParams.leftMargin;
        } else {
            return this.content.getPaddingLeft();
        }
    }
    public int getActivityPositionY() {
        if ( Build.VERSION.SDK_INT >= Build.VERSION_CODES.HONEYCOMB ) {
            FrameLayout.LayoutParams contentParams = ( FrameLayout.LayoutParams ) this.content.getLayoutParams();
            return contentParams.topMargin;
        } else {
            return this.content.getPaddingTop();
        }
    }
}
