private int WIDTH = 450;
private int HEIGHT = 100;
private WaveformPanel wfp;
private ArrayList<Line2D.Float> lines;
public WaveformFrame() {
    super();
    setSize ( WIDTH, HEIGHT + 20 );
    setTitle ( "Waveform Frame" );
    setName ( "Main FRAME" );
    wfp = new WaveformPanel();
    wfp.setSize ( WIDTH, HEIGHT );
    wfp.setName ( "Waveform PANEL" );
    add ( wfp );
}
public void updateWave ( byte[] pcmdata ) {
    wfp.updateWave ( pcmdata );
}
class WaveformPanel extends JPanel {
    byte[] pcmdata = null;
    public void updateWave ( byte[] pcmdata ) {
        this.pcmdata = pcmdata;
        doDrawing ( getGraphics() );
        repaint();
    }
    private void doDrawing ( Graphics g ) {
        Graphics2D g2d = ( Graphics2D ) g;
        int HEIGHT = getHeight();
        int WIDTH = getWidth();
        if ( pcmdata == null ) {
            g2d.drawLine ( 0, HEIGHT / 2, WIDTH, HEIGHT / 2 );
            return;
        }
        BackgroundExecutor.get().execute ( new WaveformTask ( WIDTH, HEIGHT ) );
        if ( lines != null ) {
            drawLines ( g2d );
        }
    }
    class WaveformTask implements Runnable {
        Graphics2D g2d;
        int HEIGHT;
        int WIDTH;
        public WaveformTask ( int width, int height ) {
            HEIGHT = height;
            WIDTH = width;
        }
        @Override
        public void run() {
            calcLine2d();
        }
        void calcLine2d() {
            float scale = ( float ) HEIGHT / 65536;
            int nlines = pcmdata.length / 4;
            float lineLen = ( float ) WIDTH / ( nlines - 1 );
            ArrayList<Line2D.Float> lines = new ArrayList<Line2D.Float>();
            for ( int i = 0; i < nlines - 4; i += 4 ) {
                int sample0 = Utils.getSixteenBitSample ( pcmdata[i + 1], pcmdata[i] );
                int sample1 = Utils.getSixteenBitSample ( pcmdata[i + 3], pcmdata[i + 2] );
                int val0 = ( int ) ( sample0 * scale ) + HEIGHT / 2;
                int val1 = ( int ) ( sample1 * scale ) + HEIGHT / 2;
                int diffx0 = Math.round ( lineLen * i );
                int diffx1 = Math.round ( lineLen * i + 1 );
                lines.add ( new Line2D.Float ( diffx0, val0, diffx1, val1 ) );
            }
            synchronized ( this ) {
                WaveformFrame.this.lines = lines;
            }
        }
    }
    synchronized void drawLines ( Graphics2D g2d ) {
        for ( Line2D.Float line : lines ) {
            g2d.draw ( line );
        }
        repaint();
    }
    @Override
    public void paint ( Graphics g ) {
        super.paint ( g );
        doDrawing ( g );
    }
}
private void log ( String line ) {
    System.out.println ( "GUI out] " + line );
}
