private static final class EnumTypeAdapter<T extends Enum<T>> extends TypeAdapter<T> {
    private final Map<String, T> nameToConstant = new HashMap<String, T>();
    private final Map<T, String> constantToName = new HashMap<T, String>();
    public EnumTypeAdapter ( Class<T> classOfT ) {
        try {
            String classPrefix = classOfT.getName() + ".";
            for ( T constant : classOfT.getEnumConstants() ) {
                String name = constant.name();
                SerializedName annotation = classOfT.getField ( name ).getAnnotation ( SerializedName.class );
                if ( annotation != null ) {
                    name = annotation.value();
                } else {
                    name = classPrefix + name;
                }
                nameToConstant.put ( name, constant );
                constantToName.put ( constant, name );
            }
        } catch ( NoSuchFieldException e ) {
            throw new AssertionError();
        }
    }
    public T read ( JsonReader in ) throws IOException {
        if ( in.peek() == JsonToken.NULL ) {
            in.nextNull();
            return null;
        }
        return nameToConstant.get ( in.nextString() );
    }
    public void write ( JsonWriter out, T value ) throws IOException {
        out.value ( value == null ? null : constantToName.get ( value ) );
    }
}
