private static final double MAX_ULP = 1.9958403095347198E292;
@Override
public double ulp ( double d ) {
    if ( Double.isNaN ( d ) ) {
        return Double.NaN;
    }
    if ( Double.isInfinite ( d ) ) {
        return Double.POSITIVE_INFINITY;
    }
    if ( d == 0.0 ) {
        return Double.MIN_VALUE;
    }
    d = Math.abs ( d );
    if ( d == Double.MAX_VALUE ) {
        return MAX_ULP;
    }
    return nextAfter ( d, Double.MAX_VALUE ) - d;
}
@Override
public double copySign ( double x, double y ) {
    return com.codename1.util.MathUtil.copysign ( x, y );
}
private boolean isSameSign ( double x, double y ) {
    return copySign ( x, y ) == x;
}
@Override
public double nextAfter ( final double start, final double direction ) {
    if ( Double.isNaN ( start ) || Double.isNaN ( direction ) ) {
        return Double.NaN;
    }
    if ( start == direction ) {
        return direction;
    }
    final double absStart = Math.abs ( start );
    final double absDir = Math.abs ( direction );
    final boolean toZero = !isSameSign ( start, direction ) || absDir < absStart;
    if ( toZero ) {
        if ( absStart == Double.MIN_VALUE ) {
            return copySign ( 0.0, start );
        }
        if ( Double.isInfinite ( absStart ) ) {
            return copySign ( Double.MAX_VALUE, start );
        }
        return copySign ( Double.longBitsToDouble ( Double.doubleToLongBits ( absStart ) - 1L ), start );
    } else {
        if ( start == 0.0 ) {
            return copySign ( Double.MIN_VALUE, direction );
        }
        if ( absStart == Double.MAX_VALUE ) {
            return copySign ( Double.POSITIVE_INFINITY, start );
        }
        return copySign ( Double.longBitsToDouble ( Double.doubleToLongBits ( absStart ) + 1L ), start );
    }
}
