package Earth;
import gov.nasa.worldwind.*;
import gov.nasa.worldwind.avlist.AVKey;
import gov.nasa.worldwind.awt.WorldWindowGLCanvas;
import java.awt.BorderLayout;
import org.openide.awt.ActionID;
import org.openide.awt.ActionReference;
import org.openide.windows.TopComponent;
import org.openide.util.NbBundle.Messages;
import gov.nasa.worldwind.layers.*;
import gov.nasa.worldwind.layers.Earth.*;
import java.awt.Dimension;
import javax.swing.JInternalFrame;
@TopComponent.Description (
    preferredID = "EarthTopComponent",
    persistenceType = TopComponent.PERSISTENCE_ALWAYS
)
@TopComponent.Registration ( mode = "explorer", openAtStartup = false )
@ActionID ( category = "Window", id = "Earth.EarthTopComponent" )
@ActionReference ( path = "Menu/Window" )
@TopComponent.OpenActionRegistration (
    displayName = "#CTL_EarthAction",
    preferredID = "EarthTopComponent"
)
@Messages ( {
    "CTL_EarthAction=Earth",
    "CTL_EarthTopComponent=Earth Window",
    "HINT_EarthTopComponent=This is the Earth window"
} )
public final class EarthTopComponent extends TopComponent {
    public EarthTopComponent() {
        initcomp();
        setName ( Bundle.CTL_EarthTopComponent() );
        setToolTipText ( Bundle.HINT_EarthTopComponent() );
    }
    private LayerList layerlist;
    private WorldWindowGLCanvas canvas;
    private Model model;
    private OptionSideBar sidebar;
    private JInternalFrame layersPanel;
    private void initcomp() {
        canvas = new WorldWindowGLCanvas();
        sidebar = new OptionSideBar();
        model = ( Model ) WorldWind.createConfigurationComponent ( AVKey.MODEL_CLASS_NAME );
        loadLayers();
        model.setLayers ( layerlist );
        canvas.setModel ( model );
        setLayout ( new BorderLayout() );
        layersPanel = new JInternalFrame ( "Layers Panel" );
        layersPanel.setIconifiable ( true );
        layersPanel.setSize ( new Dimension ( 150, 250 ) );
        layersPanel.add ( sidebar.getSideBar() );
        layersPanel.setVisible ( true );
        setLayout ( new BorderLayout() );
        canvas.setSize ( new Dimension ( 500, 550 ) );
        canvas.setVisible ( true );
        add ( layersPanel, BorderLayout.CENTER );
        add ( canvas, BorderLayout.CENTER );
    }
    private void loadLayers() {
        layerlist = new LayerList();
        layerlist.add ( new StarsLayer() );
        layerlist.add ( new SkyGradientLayer() );
        layerlist.add ( new BMNGWMSLayer() );
        layerlist.add ( new CountryBoundariesLayer() );
        layerlist.add ( new MSVirtualEarthLayer() );
        layerlist.add ( new NASAWFSPlaceNameLayer() );
        layerlist.add ( new LatLonGraticuleLayer() );
    }
}
