import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.event.ActionEvent;
import java.awt.geom.Area;
import java.awt.geom.Ellipse2D;
import java.awt.geom.GeneralPath;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.net.URL;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.imageio.ImageIO;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ActionMap;
import javax.swing.InputMap;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
public class SquaresInArea extends JPanel {
    private static final int INC_MOVE = 5;
    BufferedImage imageMask;
    Area imageArea;
    Rectangle2D pacmanRect;
    public SquaresInArea() {
        try {
            imageMask = ImageIO.read ( new URL ( "http://i.stack.imgur.com/oBwNs.png" ) );
        } catch ( IOException ex ) {
            Logger.getLogger ( SquaresInArea.class.getName() ).log ( Level.SEVERE, null, ex );
        }
        imageArea = getOutline ( Color.BLACK, imageMask );
        pacmanRect = new Rectangle2D.Double ( 15, 15, 25, 25 );
        InputMap im = getInputMap ( JComponent.WHEN_IN_FOCUSED_WINDOW );
        ActionMap am = getActionMap();
        im.put ( KeyStroke.getKeyStroke ( "LEFT" ), "leftAction" );
        am.put ( "leftAction", leftAction );
        im.put ( KeyStroke.getKeyStroke ( "RIGHT" ), "rightAction" );
        am.put ( "rightAction", rightAction );
        im.put ( KeyStroke.getKeyStroke ( "DOWN" ), "downAction" );
        am.put ( "downAction", downAction );
        im.put ( KeyStroke.getKeyStroke ( "UP" ), "upAction" );
        am.put ( "upAction", upAction );
    }
    Action leftAction = new AbstractAction() {
        @Override
        public void actionPerformed ( ActionEvent e ) {
            Rectangle2D newRect = pacmanRect.getBounds2D();
            double x = newRect.getX() - INC_MOVE;
            double y = newRect.getY();
            double width = newRect.getWidth();
            double height = newRect.getHeight();
            newRect.setRect ( x, y, width, height );
            if ( !isIntersect ( imageArea, newRect ) ) {
                pacmanRect.setRect ( x, y, width, height );
                repaint();
            }
        }
    };
    Action rightAction = new AbstractAction() {
        @Override
        public void actionPerformed ( ActionEvent e ) {
            Rectangle2D newRect = pacmanRect.getBounds2D();
            double x = newRect.getX() + INC_MOVE;
            double y = newRect.getY();
            double width = newRect.getWidth();
            double height = newRect.getHeight();
            newRect.setRect ( x, y, width, height );
            if ( !isIntersect ( imageArea, newRect ) ) {
                pacmanRect.setRect ( x, y, width, height );
                repaint();
            }
        }
    };
    Action downAction = new AbstractAction() {
        @Override
        public void actionPerformed ( ActionEvent e ) {
            Rectangle2D newRect = pacmanRect.getBounds2D();
            double x = newRect.getX();
            double y = newRect.getY() + INC_MOVE;
            double width = newRect.getWidth();
            double height = newRect.getHeight();
            newRect.setRect ( x, y, width, height );
            if ( !isIntersect ( imageArea, newRect ) ) {
                pacmanRect.setRect ( x, y, width, height );
                repaint();
            }
        }
    };
    Action upAction = new AbstractAction() {
        @Override
        public void actionPerformed ( ActionEvent e ) {
            Rectangle2D newRect = pacmanRect.getBounds2D();
            double x = newRect.getX();
            double y = newRect.getY() - INC_MOVE;
            double width = newRect.getWidth();
            double height = newRect.getHeight();
            newRect.setRect ( x, y, width, height );
            if ( !isIntersect ( imageArea, newRect ) ) {
                pacmanRect.setRect ( x, y, width, height );
                repaint();
            }
        }
    };
    private boolean isIntersect ( Shape imageArea, Rectangle2D rect ) {
        return imageArea.intersects ( rect );
    }
    @Override
    protected void paintComponent ( Graphics g ) {
        super.paintComponent ( g );
        Graphics2D g2 = ( Graphics2D ) g;
        g2.setColor ( Color.red );
        g2.fill ( imageArea );
        Ellipse2D pacman = new Ellipse2D.Double ( pacmanRect.getX(), pacmanRect.getY(),
                pacmanRect.getWidth(), pacmanRect.getHeight() );
        g2.setColor ( Color.BLUE );
        g2.fill ( pacman );
    }
    @Override
    public Dimension getPreferredSize() {
        return imageMask == null ? new Dimension ( 300, 300 )
               : new Dimension ( imageMask.getWidth(), imageMask.getHeight() );
    }
    private Area getOutline ( Color target, BufferedImage bi ) {
        GeneralPath gp = new GeneralPath();
        boolean cont = false;
        int targetRGB = target.getRGB();
        for ( int xx = 0; xx < bi.getWidth(); xx++ ) {
            for ( int yy = 0; yy < bi.getHeight(); yy++ ) {
                if ( bi.getRGB ( xx, yy ) == targetRGB ) {
                    if ( cont ) {
                        gp.lineTo ( xx, yy );
                        gp.lineTo ( xx, yy + 1 );
                        gp.lineTo ( xx + 1, yy + 1 );
                        gp.lineTo ( xx + 1, yy );
                        gp.lineTo ( xx, yy );
                    } else {
                        gp.moveTo ( xx, yy );
                    }
                    cont = true;
                } else {
                    cont = false;
                }
            }
            cont = false;
        }
        gp.closePath();
        return new Area ( gp );
    }
    public static void main ( String[] args ) {
        SwingUtilities.invokeLater ( new Runnable() {
            public void run() {
                JFrame frame = new JFrame();
                frame.add ( new SquaresInArea() );
                frame.setDefaultCloseOperation ( JFrame.EXIT_ON_CLOSE );
                frame.pack();
                frame.setLocationRelativeTo ( null );
                frame.setVisible ( true );
            }
        } );
    }
}
