import javax.tools.*;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.jar.JarEntry;
import java.util.jar.JarOutputStream;
import java.util.jar.Manifest;
import java.net.URL;
import java.net.URLClassLoader;
import java.lang.reflect.Method;
public class CompileJarLoadReflect {
    public static int BUFFER_SIZE = 10 * 1024;
    public void compile ( String[] files ) throws Exception {
        for ( String f : files ) {
            compile ( f + ".java" );
        }
        System.out.println ( files.length + " files successfully compiled" );
    }
    protected void compile ( String f ) throws Exception {
        JavaCompiler compiler = ToolProvider.getSystemJavaCompiler();
        int compilationResult = compiler.run ( null, null, null, f );
        if ( compilationResult == 0 ) {
            System.out.println ( f + " compilation is successful" );
        } else {
            throw new Exception ( "Compilation error at file " + f );
        }
    }
    public void jar ( String jarName, String[] tobeJared ) throws Exception {
        File archiveFile = new File ( jarName );
        byte buffer[] = new byte[BUFFER_SIZE];
        FileOutputStream stream = new FileOutputStream ( archiveFile );
        JarOutputStream out = new JarOutputStream ( stream, new Manifest() );
        for ( String name : tobeJared ) {
            File f = new File ( name + ".class" );
            if ( f == null || !f.exists() || f.isDirectory() ) {
                throw new Exception ( "Jar problem at file " + name );
            }
            JarEntry jarAdd = new JarEntry ( f.getName() );
            jarAdd.setTime ( f.lastModified() );
            out.putNextEntry ( jarAdd );
            FileInputStream in = new FileInputStream ( f );
            while ( true ) {
                int nRead = in.read ( buffer, 0, buffer.length );
                if ( nRead <= 0 ) {
                    break;
                }
                out.write ( buffer, 0, nRead );
            }
            in.close();
        }
        out.close();
        stream.close();
    }
    public void load ( String jarName, String[] clazzes ) throws Exception {
        File file  = new File ( jarName );
        URL url = file.toURL();
        URL[] urls = new URL[] {url};
        ClassLoader cl = new URLClassLoader ( urls );
        for ( String c : clazzes ) {
            System.out.println ( "Class " + c );
            Class cls = cl.loadClass ( c );
            Method[] methods = cls.getDeclaredMethods();
            for ( Method m : methods ) {
                System.out.println ( "\t" + m.getName() );
            }
        }
    }
    public static void main ( String[] args ) {
        String jarName = "output.jar";
        try {
            CompileJarLoadReflect cjlr = new CompileJarLoadReflect();
            cjlr.compile ( args );
            cjlr.jar ( jarName, args );
            cjlr.load ( jarName, args );
        } catch ( Exception ex ) {
            ex.printStackTrace();
        }
    }
}
