@Configuration
@ComponentScan
@EnableResourceServer
@Import ( {SecurityConfig.class} )
public class OAuth2ServerConfig {
    @Configuration
    @EnableAuthorizationServer
    protected static class OAuth2Config extends AuthorizationServerConfigurerAdapter {
        @Autowired
        private AuthenticationManager authenticationManager;
        @Autowired
        @Qualifier ( "restDataSource" )
        private DataSource datasource;
        @Bean
        public JwtAccessTokenConverter accessTokenConverter() {
            return new JwtAccessTokenConverter();
        }
        @Bean
        public CustomJwtTokenStore tokenStore() {
            return new CustomJwtTokenStore();
        }
        @Override
        public void configure ( AuthorizationServerSecurityConfigurer oauthServer ) throws Exception {
            oauthServer.tokenKeyAccess ( "isAnonymous() || hasAuthority('ROLE_TRUSTED_CLIENT')" ).checkTokenAccess (
                "hasAuthority('ROLE_TRUSTED_CLIENT')" );
        }
        @Override
        public void configure ( AuthorizationServerEndpointsConfigurer endpoints ) throws Exception {
            endpoints.authenticationManager ( authenticationManager ).tokenStore ( tokenStore() ).accessTokenConverter ( accessTokenConverter() );
        }
        @Override
        public void configure ( ClientDetailsServiceConfigurer clients ) throws Exception {
            clients.inMemory()
            .withClient ( "my-trusted-client" )
            .authorizedGrantTypes ( "password", "authorization_code", "refresh_token", "implicit" )
            .authorities ( "ROLE_CLIENT", "ROLE_TRUSTED_CLIENT" )
            .scopes ( "read", "write", "trust" )
            .accessTokenValiditySeconds ( 60 )
            .and()
            .withClient ( "my-client-with-registered-redirect" )
            .authorizedGrantTypes ( "authorization_code" )
            .authorities ( "ROLE_CLIENT" )
            .scopes ( "read", "trust" )
            .redirectUris ( "http://anywhere?key=value" )
            .and()
            .withClient ( "my-client-with-secret" )
            .authorizedGrantTypes ( "client_credentials", "password" )
            .authorities ( "ROLE_CLIENT", "ROLE_TRUSTED_CLIENT" )
            .scopes ( "read", "write" )
            .secret ( "secret" );
        }
    }
}
