public class HttpSpnegoConnection {
    public static void main ( String[] args ) throws ClientProtocolException, IOException {
        String domain = "yourdomain.com";
        String kdc =  "thekdc.yourdomain.ch";
        String url = "http://somewebsite:8080/myresource/";
        boolean skipPortAtKerberosDatabaseLookup = true;
        File krb5Config = createKrb5Configuration ( domain, kdc );
        System.out.println ( "Created config '" + krb5Config.getAbsolutePath() + "':" );
        System.out.println ( IOUtils.toString ( new FileReader ( krb5Config ) ) );
        System.setProperty ( "java.security.krb5.conf", krb5Config.toURI().toString() );
        System.setProperty ( "sun.security.krb5.debug", "true" );
        System.setProperty ( "javax.security.auth.useSubjectCredsOnly", "false" );
        Lookup<AuthSchemeProvider> authSchemeRegistry = RegistryBuilder.<AuthSchemeProvider>create()
                .register ( AuthSchemes.SPNEGO, new SPNegoSchemeFactory ( skipPortAtKerberosDatabaseLookup ) )
                .build();
        CloseableHttpClient client = HttpClients.custom().setDefaultAuthSchemeRegistry ( authSchemeRegistry ).build();
        HttpClientContext context = HttpClientContext.create();
        BasicCredentialsProvider credentialsProvider = new BasicCredentialsProvider();
        Credentials useJaasCreds = new Credentials() {
            public String getPassword() {
                return null;
            }
            public Principal getUserPrincipal() {
                return null;
            }
        };
        credentialsProvider.setCredentials ( new AuthScope ( null, -1, null ), useJaasCreds );
        context.setCredentialsProvider ( credentialsProvider );
        HttpGet httpget = new HttpGet ( url );
        CloseableHttpResponse response = client.execute ( httpget, context );
        String responseString = IOUtils.toString ( response.getEntity().getContent() );
        System.out.println ( "HTTP Response:" );
        System.out.println ( responseString );
    }
    private static File createKrb5Configuration ( String domain, String kdc ) throws IOException {
        File tempFile = File.createTempFile ( "krb5", "kdc" );
        ArrayList<String> lines = new ArrayList<>();
        lines.add ( "[libdefaults]" );
        lines.add ( "\tdefault_realm = " + domain );
        lines.add ( "[realms]" );
        lines.add ( "\t" + domain + " = {" );
        lines.add ( "\t\tkdc = " + kdc );
        lines.add ( "\t\tadmin_server = " + kdc );
        lines.add ( "\t}" );
        FileWriter writer = new FileWriter ( tempFile );
        IOUtils.writeLines ( lines, System.lineSeparator(), writer );
        IOUtils.closeQuietly ( writer );
        return tempFile;
    }
}
