public class BitPattern implements Iterable<BigInteger> {
    private static final BigInteger ONE = BigInteger.ONE;
    private static final BigInteger TWO = ONE.add ( ONE );
    private final int bits;
    private final BigInteger stop;
    private final boolean not;
    public BitPattern ( int bits, int max, boolean not ) {
        this.bits = bits;
        this.stop = TWO.pow ( max );
        this.not = not;
    }
    public BitPattern ( int bits, int max ) {
        this ( bits, max, false );
    }
    @Override
    public Iterator<BigInteger> iterator() {
        return new BitPatternIterator();
    }
    private class BitPatternIterator implements Iterator<BigInteger> {
        BigInteger next = TWO.pow ( bits ).subtract ( ONE );
        BigInteger last;
        @Override
        public boolean hasNext() {
            if ( next == null ) {
                BigInteger t = last.or ( last.subtract ( BigInteger.ONE ) );
                BigInteger notT = t.not();
                next = t.add ( ONE ).or ( notT.and ( notT.negate() ).subtract ( ONE ).shiftRight ( last.getLowestSetBit() + 1 ) );
                if ( next.compareTo ( stop ) >= 0 ) {
                    next = null;
                }
            }
            return next != null;
        }
        @Override
        public BigInteger next() {
            last = hasNext() ? next : null;
            next = null;
            return not ? last.not() : last;
        }
        @Override
        public void remove() {
            throw new UnsupportedOperationException ( "Not supported." );
        }
        @Override
        public String toString() {
            return next != null ? next.toString ( 2 ) : last != null ? last.toString ( 2 ) : "";
        }
    }
    public static void main ( String[] args ) {
        int bits = 1;
        int max = 3;
        System.out.println ( "BitPattern(" + bits + ", " + max + ")" );
        for ( BigInteger i : new BitPattern ( bits, max ) ) {
            System.out.println ( i.toString ( 2 ) );
        }
    }
}
