import java.awt.Color;
import java.awt.Dimension;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.MultipleGradientPaint.CycleMethod;
import java.awt.RadialGradientPaint;
import java.awt.Shape;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.geom.RoundRectangle2D;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;
public class RectangularGradientTest {
    public static void main ( String[] args ) {
        SwingUtilities.invokeLater ( new Runnable() {
            @Override
            public void run() {
                createAndShowGUI();
            }
        } );
    }
    private static void createAndShowGUI() {
        JFrame f = new JFrame();
        f.setDefaultCloseOperation ( JFrame.EXIT_ON_CLOSE );
        f.getContentPane().add ( new RectangularGradientTestPanel() );
        f.pack();
        f.setLocationRelativeTo ( null );
        f.setVisible ( true );
    }
}
class RectangularGradientTestPanel extends JPanel {
    @Override
    protected void paintComponent ( Graphics gr ) {
        super.paintComponent ( gr );
        Graphics2D g = ( Graphics2D ) gr;
        Rectangle2D r = new Rectangle2D.Double ( 100, 100, 200, 100 );
        draw ( g, r, 75 );
        Shape rr = new RoundRectangle2D.Double ( 80, 80, 240, 140, 20, 20 );
        g.setColor ( Color.BLACK );
        g.fill ( rr );
    }
    @Override
    public Dimension getPreferredSize() {
        if ( isPreferredSizeSet() ) {
            return super.getPreferredSize();
        }
        return new Dimension ( 400, 300 );
    }
    private static void draw ( Graphics2D g, Rectangle2D r, double s ) {
        Color c0 = new Color ( 255, 0, 0 );
        Color c1 = new Color ( 255, 0, 0, 0 );
        double x0 = r.getMinX();
        double y0 = r.getMinY();
        double x1 = r.getMaxX();
        double y1 = r.getMaxY();
        double w = r.getWidth();
        double h = r.getHeight();
        g.setPaint ( new GradientPaint (
                         new Point2D.Double ( x0, y0 ), c0,
                         new Point2D.Double ( x0 - s, y0 ), c1 ) );
        g.fill ( new Rectangle2D.Double ( x0 - s, y0, s, h ) );
        g.setPaint ( new GradientPaint (
                         new Point2D.Double ( x1, y0 ), c0,
                         new Point2D.Double ( x1 + s, y0 ), c1 ) );
        g.fill ( new Rectangle2D.Double ( x1, y0, s, h ) );
        g.setPaint ( new GradientPaint (
                         new Point2D.Double ( x0, y0 ), c0,
                         new Point2D.Double ( x0, y0 - s ), c1 ) );
        g.fill ( new Rectangle2D.Double ( x0, y0 - s, w, s ) );
        g.setPaint ( new GradientPaint (
                         new Point2D.Double ( x0, y1 ), c0,
                         new Point2D.Double ( x0, y1 + s ), c1 ) );
        g.fill ( new Rectangle2D.Double ( x0, y1, w, s ) );
        float fractions[] = new float[] { 0.0f, 1.0f };
        Color colors[] = new Color[] { c0, c1 };
        g.setPaint ( new RadialGradientPaint (
                         new Rectangle2D.Double ( x0 - s, y0 - s, s + s, s + s ),
                         fractions, colors, CycleMethod.NO_CYCLE ) );
        g.fill ( new Rectangle2D.Double ( x0 - s, y0 - s, s, s ) );
        g.setPaint ( new RadialGradientPaint (
                         new Rectangle2D.Double ( x1 - s, y0 - s, s + s, s + s ),
                         fractions, colors, CycleMethod.NO_CYCLE ) );
        g.fill ( new Rectangle2D.Double ( x1, y0 - s, s, s ) );
        g.setPaint ( new RadialGradientPaint (
                         new Rectangle2D.Double ( x0 - s, y1 - s, s + s, s + s ),
                         fractions, colors, CycleMethod.NO_CYCLE ) );
        g.fill ( new Rectangle2D.Double ( x0 - s, y1, s, s ) );
        g.setPaint ( new RadialGradientPaint (
                         new Rectangle2D.Double ( x1 - s, y1 - s, s + s, s + s ),
                         fractions, colors, CycleMethod.NO_CYCLE ) );
        g.fill ( new Rectangle2D.Double ( x1, y1, s, s ) );
    }
}
