public enum LockRegistry {
    INSTANCE;
    private Map<String, ReadWriteLock> lockMap = new HashMap<String, ReadWriteLock>();
    private Lock registryLock = new ReentrantLock();
    public enum LockType {
        READ, WRITE
    }
    public void acquire ( String fileName, LockType type ) {
        ReadWriteLock lock = retrieveLock ( fileName );
        switch ( type ) {
        case READ:
            lock.readLock().lock();
            break;
        case WRITE:
            lock.writeLock().lock();
            break;
        default:
            break;
        }
    }
    public void release ( String fileName, LockType type ) {
        ReadWriteLock lock = retrieveLock ( fileName );
        switch ( type ) {
        case READ:
            lock.readLock().unlock();
            break;
        case WRITE:
            lock.writeLock().unlock();
            break;
        default:
            break;
        }
    }
    private ReadWriteLock retrieveLock ( String fileName ) {
        ReadWriteLock newLock = null;
        try {
            registryLock.lock();
            newLock = lockMap.get ( fileName );
            if ( newLock == null ) {
                newLock = new ReentrantReadWriteLock();
                lockMap.put ( fileName, newLock );
            }
        } finally {
            registryLock.unlock();
        }
        return newLock;
    }
}
