package com.example.mapdemo;
import com.google.android.gms.common.ConnectionResult;
import com.google.android.gms.common.GooglePlayServicesClient.ConnectionCallbacks;
import com.google.android.gms.common.GooglePlayServicesClient.OnConnectionFailedListener;
import com.google.android.gms.location.LocationClient;
import com.google.android.gms.location.LocationListener;
import com.google.android.gms.location.LocationRequest;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.GoogleMap.OnMyLocationButtonClickListener;
import com.google.android.gms.maps.SupportMapFragment;
import android.location.Location;
import android.os.Bundle;
import android.support.v4.app.FragmentActivity;
import android.view.View;
import android.widget.TextView;
import android.widget.Toast;
public class MyLocationDemoActivity extends FragmentActivity
    implements
    ConnectionCallbacks,
    OnConnectionFailedListener,
    LocationListener,
    OnMyLocationButtonClickListener {
    private GoogleMap mMap;
    private LocationClient mLocationClient;
    private TextView mMessageView;
    private static final LocationRequest REQUEST = LocationRequest.create()
            .setInterval ( 5000 )
            .setFastestInterval ( 16 )
            .setPriority ( LocationRequest.PRIORITY_HIGH_ACCURACY );
    @Override
    protected void onCreate ( Bundle savedInstanceState ) {
        super.onCreate ( savedInstanceState );
        setContentView ( R.layout.my_location_demo );
        mMessageView = ( TextView ) findViewById ( R.id.message_text );
    }
    @Override
    protected void onResume() {
        super.onResume();
        setUpMapIfNeeded();
        setUpLocationClientIfNeeded();
        mLocationClient.connect();
    }
    @Override
    public void onPause() {
        super.onPause();
        if ( mLocationClient != null ) {
            mLocationClient.disconnect();
        }
    }
    private void setUpMapIfNeeded() {
        if ( mMap == null ) {
            mMap = ( ( SupportMapFragment ) getSupportFragmentManager().findFragmentById ( R.id.map ) )
                   .getMap();
            if ( mMap != null ) {
                mMap.setMyLocationEnabled ( true );
                mMap.setOnMyLocationButtonClickListener ( this );
            }
        }
    }
    private void setUpLocationClientIfNeeded() {
        if ( mLocationClient == null ) {
            mLocationClient = new LocationClient (
                getApplicationContext(),
                this,
                this );
        }
    }
    public void showMyLocation ( View view ) {
        if ( mLocationClient != null && mLocationClient.isConnected() ) {
            String msg = "Location = " + mLocationClient.getLastLocation();
            Toast.makeText ( getApplicationContext(), msg, Toast.LENGTH_SHORT ).show();
        }
    }
    @Override
    public void onLocationChanged ( Location location ) {
        mMessageView.setText ( "Location = " + location );
    }
    @Override
    public void onConnected ( Bundle connectionHint ) {
        mLocationClient.requestLocationUpdates (
            REQUEST,
            this );
    }
    @Override
    public void onDisconnected() {
    }
    @Override
    public void onConnectionFailed ( ConnectionResult result ) {
    }
    @Override
    public boolean onMyLocationButtonClick() {
        Toast.makeText ( this, "MyLocation button clicked", Toast.LENGTH_SHORT ).show();
        return false;
    }
}
