package com.ges.util;
import javax.swing.*;
import java.awt.*;
import java.util.*;
import java.awt.event.*;
public class RubberBandSelect extends Component {
    private Point startPoint = null;
    private Point endPoint = null;
    private Graphics hostGraphics = null;
    private Component hostComponent = null;
    private Color bandColor = Color.ORANGE.darker().darker();
    private boolean started = false;
    private boolean eraseSomething = false;
    private int startX, endX, startY, endY = 0;
    private Rectangle boundingBox;
    private StringBuilder QuadrantMessage = null;
    private HashSet<RubberBandListener> listeners =
        new HashSet<RubberBandListener>();
    private int width = 0;
    private int height = 0;
    public RubberBandSelect ( Component c ) {
        hostComponent = c;
        hostGraphics = c.getGraphics();
    }
    public void addListener ( RubberBandListener l ) {
        listeners.add ( l );
    }
    public void paint ( Graphics g ) {
        draw();
    }
    public void erase() {
        if ( eraseSomething ) {
            draw();
            eraseSomething = false;
        }
    }
    private void draw() {
        hostGraphics = hostComponent.getGraphics();
        if ( hostGraphics != null ) {
            try {
                erase();
                drawRubberBand();
                eraseSomething = false;
            } finally {
            }
        }
    }
    private void drawRubberBand() {
        if ( !started ) {
            return;
        }
        hostGraphics = hostComponent.getGraphics();
        if ( hostGraphics == null ) {
            return;
        }
        if ( startPoint == null || endPoint == null ) {
            return;
        }
        hostGraphics.setColor ( bandColor );
        if ( endX > startX && endY > startY ) {
            boundingBox = new Rectangle ( startX, startY, endX - startX, endY - startY );
            hostGraphics.drawRect ( startX, startY, endX - startX, endY - startY );
            QuadrantMessage = new StringBuilder ( "Drawing in Q - IV X1=" );
            width = endX - startX;
            height = endY - startY;
        } else if ( endX < startX && endY < startY ) {
            boundingBox = new Rectangle ( endX, endY, startX - endX, startY - endY );
            hostGraphics.drawRect ( endX, endY, startX - endX, startY - endY );
            QuadrantMessage = new StringBuilder ( "Drawing in Q - II X1=" );
            width = startX - endX;
            height = startY - endY;
        } else if ( endX > startX && endY < startY ) {
            boundingBox = new Rectangle ( startX, endY, endX - startX, startY - endY );
            hostGraphics.drawRect ( startX, endY, endX - startX, startY - endY );
            QuadrantMessage = new StringBuilder ( "Drawing in Q - I X1=" );
            width = endX - startX;
            height = startY - endY;
        } else if ( endX < startX && endY > startY ) {
            boundingBox = new Rectangle ( endX, startY, startX - endX, endY - startY );
            hostGraphics.drawRect ( endX, startY, startX - endX, endY - startY );
            QuadrantMessage = new StringBuilder ( "Drawing in Q - III X1=" );
            width = startX - endX;
            height = endY - startY;
        }
    }
    public void assignToCompoent ( Component c ) {
        this.hostComponent = c;
        hostGraphics = c.getGraphics();
    }
    public void update ( Graphics g ) {
        drawRubberBand();
    }
    public Point getStartPoint() {
        return startPoint;
    }
    public void setStartPoint ( Point startPoint ) {
        this.startPoint = startPoint;
        startX = ( int ) startPoint.getX();
        startY = ( int ) startPoint.getY();
        QuadrantMessage = new StringBuilder();
    }
    public Point getEndPoint() {
        return endPoint;
    }
    public void setEndPoint ( Point endPoint ) {
        this.endPoint = endPoint;
        clear();
        endX = ( int ) endPoint.getX();
        endY = ( int ) endPoint.getY();
        draw();
        notifyListeners();
        started = true;
    }
    public Color getBandColor() {
        return bandColor;
    }
    public void setBandColor ( Color bandColor ) {
        this.bandColor = bandColor;
    }
    public void setForeground ( Color color ) {
        this.bandColor = color;
    }
    private void clear() {
        hostGraphics = hostComponent.getGraphics();
        if ( hostGraphics == null ) {
            return;
        }
        try {
            drawRubberBand();
        } finally {
        }
    }
    public void breakBand() {
        startPoint = null;
        endPoint = null;
        started = false;
        boundingBox = new Rectangle ( 0, 0, 0, 0 );
        if ( hostGraphics != null ) {
            hostGraphics.dispose();
        }
        clear();
        hostComponent.repaint();
    }
    public boolean isStarted() {
        return started;
    }
    public void notifyListeners() {
        Iterator<RubberBandListener> it = listeners.iterator();
        while ( it.hasNext() ) {
            it.next().notifyBounds ( boundingBox );
        }
    }
    public void redraw ( Graphics g ) {
        if ( startPoint == null || endPoint == null ) {
            return;
        }
        g.setColor ( bandColor );
        if ( endX > startX && endY > startY ) {
            boundingBox = new Rectangle ( startX, startY, endX - startX, endY - startY );
            g.drawRect ( startX, startY, endX - startX, endY - startY );
            QuadrantMessage = new StringBuilder ( "Drawing in Q - IV X1=" );
            width = endX - startX;
            height = endY - startY;
        } else if ( endX < startX && endY < startY ) {
            boundingBox = new Rectangle ( endX, endY, startX - endX, startY - endY );
            g.drawRect ( endX, endY, startX - endX, startY - endY );
            QuadrantMessage = new StringBuilder ( "Drawing in Q - II X1=" );
            width = startX - endX;
            height = startY - endY;
        } else if ( endX > startX && endY < startY ) {
            boundingBox = new Rectangle ( startX, endY, endX - startX, startY - endY );
            g.drawRect ( startX, endY, endX - startX, startY - endY );
            QuadrantMessage = new StringBuilder ( "Drawing in Q - I X1=" );
            width = endX - startX;
            height = startY - endY;
        } else if ( endX < startX && endY > startY ) {
            boundingBox = new Rectangle ( endX, startY, startX - endX, endY - startY );
            g.drawRect ( endX, startY, startX - endX, endY - startY );
            QuadrantMessage = new StringBuilder ( "Drawing in Q - III X1=" );
            width = startX - endX;
            height = endY - startY;
        }
    }
    public Rectangle getBoundingBox() {
        return boundingBox;
    }
}
