package snippet;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;
public class SessionAttributes implements Comparable<SessionAttributes> {
    private final String userName;
    private final String sessionState;
    public SessionAttributes ( String userName, String sessionState ) {
        if ( userName == null ) {
            throw new NullPointerException ( "userName must not be null" );
        }
        if ( sessionState == null ) {
            throw new NullPointerException ( "sessionState must not be null" );
        }
        this.userName = userName;
        this.sessionState = sessionState;
    }
    public String getUserName() {
        return userName;
    }
    public String getSessionState() {
        return sessionState;
    }
    @Override
    public int compareTo ( SessionAttributes o ) {
        if ( this == o ) {
            return 0;
        }
        if ( o == null ) {
            return 1;
        }
        int diff = userName.compareToIgnoreCase ( o.userName );
        if ( diff != 0 ) {
            return diff;
        }
        diff = sessionState.compareToIgnoreCase ( o.sessionState );
        return diff;
    }
    public String toString() {
        return "\n User Name : " + this.getUserName() + " Session State : " + getSessionState() + "\n";
    }
    public boolean equals ( Object o ) {
        if ( o == this ) {
            return true;
        }
        if ( ! ( o instanceof SessionAttributes ) ) {
            return false;
        }
        SessionAttributes that = ( SessionAttributes ) o;
        return userName.equalsIgnoreCase ( that.userName ) && sessionState.equalsIgnoreCase ( sessionState );
    }
    public int hashCode() {
        System.out.println ( " in hashcode " );
        int hash = 1;
        hash = hash * 17 + this.getUserName().toUpperCase().hashCode();
        hash = hash * 31 + this.getSessionState().toUpperCase().hashCode();
        System.out.println ( " hash=>" + hash );
        return hash;
    }
    public static void main ( String[] args ) {
        List<SessionAttributes> expectedSessionList = new ArrayList<SessionAttributes>();
        SessionAttributes user11 = new SessionAttributes ( "postureuser1", "STARTED" );
        SessionAttributes user12 = new SessionAttributes ( "postureuser1", "DISCONNECTED" );
        SessionAttributes user13 = new SessionAttributes ( "postureuser5", "STARTED" );
        expectedSessionList.add ( user11 );
        expectedSessionList.add ( user12 );
        expectedSessionList.add ( user13 );
        System.out.println ( "expectedSessionList: " + expectedSessionList );
        List<SessionAttributes> actualSessionList = new ArrayList<SessionAttributes>();
        SessionAttributes user3 = new SessionAttributes ( "postureuser1", "STARTED" );
        SessionAttributes user4 = new SessionAttributes ( "postureuser1", "DISCONNECTED" );
        SessionAttributes user5 = new SessionAttributes ( "postureuser2", "DISCONNECTED" );
        actualSessionList.add ( user3 );
        actualSessionList.add ( user4 );
        actualSessionList.add ( user5 );
        System.out.println ( "actualSessionList: " + actualSessionList );
        Set<SessionAttributes> removeDups = new TreeSet<SessionAttributes>();
        boolean b1 = removeDups.add ( user11 );
        boolean b2 = removeDups.add ( user12 );
        boolean b3 = removeDups.add ( user13 );
        boolean b4 = removeDups.add ( user3 );
        boolean b5 = removeDups.add ( user4 );
        boolean b6 = removeDups.add ( user5 );
        System.out.println ( " Set--" + removeDups );
        actualSessionList.removeAll ( expectedSessionList );
        System.out.println ( "actualSessionList after removeAll: " + actualSessionList );
    }
}
