#ifndef SHA_1_H_INCLUDED
#define SHA_1_H_INCLUDED
#include <algorithm>
#include <vector>
#include <string>
#include <assert.h>
#include <iostream>
#include <sstream>
#include <iomanip>
#if defined(_MSC_VER) && _MSC_VER < 1600
typedef unsigned int uint32_t;
typedef unsigned __int64 uint64_t;
#else
#include <stdint.h>
#endif
namespace crypto {
    namespace {
        struct ternary_operator {
            virtual uint32_t operator() ( uint32_t x, uint32_t y, uint32_t z ) = 0;
        };
    }
    class sha1 {
        static const size_t hash_size = 5;
        static const size_t min_pad = 64;
        static const size_t block_bits = 512;
        static const size_t block_bytes = block_bits / 8;
        static const size_t block_words = block_bytes / 4;
        std::vector<uint32_t> K;
        std::vector<uint32_t> H;
        std::vector<uint32_t> W;
        std::vector<ternary_operator * > fs;
        uint32_t a, b, c, d, e, T;
        static const size_t block_size = 16;
        static const size_t bytes_per_word = 4;
        size_t total_size;
        void hash_block ( std::vector<uint32_t> const & block );
        static std::string pad ( std::string const & input, size_t size );
        std::vector<uint32_t> make_block ( std::string const & in );
        public:
        sha1();
        std::vector<uint32_t> operator() ( std::istream & in );
        std::vector<uint32_t> operator() ( std::string const & input );
        friend std::ostream & operator<< ( std::ostream & os, sha1 const & s );
    };
}
#endif
