@Override
protected void onHandleIntent ( Intent intent ) {
    Bundle extras = intent.getExtras();
    GoogleCloudMessaging gcm = GoogleCloudMessaging.getInstance ( this );
    String messageType = gcm.getMessageType ( intent );
    if ( !extras.isEmpty() ) {
        if ( GoogleCloudMessaging.MESSAGE_TYPE_SEND_ERROR.equals ( messageType ) ) {
            Log.i ( Consts.TAG, "onHandleIntent: message error" );
        } else if ( GoogleCloudMessaging.MESSAGE_TYPE_DELETED.equals ( messageType ) ) {
            Log.i ( Consts.TAG, "onHandleIntent: message deleted" );
        } else if ( GoogleCloudMessaging.MESSAGE_TYPE_MESSAGE.equals ( messageType ) ) {
            String subId = intent.getStringExtra ( GCM_KEY_SUBID );
            Log.i ( Consts.TAG, "onHandleIntent: subId: " + subId );
            String[] tokens = subId.split ( ":" );
            String typeId = tokens[1];
            if ( GCM_TYPEID_QUERY.equals ( typeId ) ) {
                Intent messageIntent = new Intent ( BROADCAST_ON_MESSAGE );
                messageIntent.putExtras ( intent );
                messageIntent.putExtra ( "token", tokens[2] );
                boolean isReceived = LocalBroadcastManager.getInstance ( this ).sendBroadcast ( messageIntent );
                if ( !isReceived ) {
                    Log.i ( Consts.TAG, "A message has been recieved but no broadcast was handled." );
                    generateNotification ( this, intent, tokens[2] );
                } else {
                    Log.i ( Consts.TAG, "A message has been recieved, broadcasted and handled." );
                }
            }
        }
    }
    GCMBroadcastReceiver.completeWakefulIntent ( intent );
}
public static void generateNotification ( Context context, Intent intent, String message ) {
    HashMap data = new HashMap();
    for ( String key : intent.getExtras().keySet() ) {
        Log.d ( Consts.TAG, "Message key: " + key + " value: " + intent.getExtras().getString ( key ) );
        String eventKey = key.startsWith ( "data." ) ? key.substring ( 5 ) : key;
        data.put ( eventKey, intent.getExtras().getString ( key ) );
    }
    CharSequence contentTitle = ( CharSequence ) data.get ( "updatedBy" );
    if ( contentTitle == null ) {
        contentTitle = "New Message";
    }
    CharSequence contentText = ( CharSequence ) data.get ( "message" );
    if ( contentText == null ) {
        contentText = "";
    }
    CharSequence userId = ( CharSequence ) data.get ( "updatedBy" );
    Bitmap iconBitmap = getUserIcon ( context, userId.toString() );
    if ( iconBitmap == null ) {
        iconBitmap = BitmapFactory.decodeResource ( context.getResources(), R.drawable.ic_launcher );
    }
    Intent resultIntent = new Intent ( context, GuestbookActivity.class );
    TaskStackBuilder stackBuilder = TaskStackBuilder.create ( context );
    stackBuilder.addParentStack ( IntroductionActivity.class );
    stackBuilder.addNextIntent ( resultIntent );
    PendingIntent resultPendingIntent = stackBuilder.getPendingIntent ( 0, PendingIntent.FLAG_UPDATE_CURRENT );
    Notification.Builder mBuilder = new Notification.Builder ( context );
    mBuilder.setContentIntent ( resultPendingIntent );
    Notification notification = mBuilder
                                .setContentTitle ( contentTitle )
                                .setContentText ( contentText )
                                .setSmallIcon ( R.drawable.notification_icon )
                                .setLargeIcon ( iconBitmap )
                                .setTicker ( contentTitle + ": " + contentText )
                                .setWhen ( System.currentTimeMillis() )
                                .setAutoCancel ( true )
                                .build();
    int notifyID = 1;
    String contentID = ( String ) data.get ( "id" );
    if ( contentID != null ) {
        notifyID = Integer.parseInt ( contentID );
    }
    NotificationManager mNotificationManager = ( NotificationManager ) context.getSystemService ( Context.NOTIFICATION_SERVICE );
    mNotificationManager.notify ( notifyID, notification );
}
