public class WordCount {
    public static class Map extends Mapper<LongWritable, Text, Text, IntWritable> {
        private final static IntWritable one = new IntWritable ( 1 );
        private Text word = new Text();
        private Set<String> stopWords;
        protected void setup ( Context context ) throws IOException, InterruptedException {
            Configuration conf = context.getConfiguration();
            stopWords = new HashSet<String>();
            for ( String word : conf.get ( "stop.words" ).split ( "," ) ) {
                stopWords.add ( word );
            }
        }
        public void map ( LongWritable key, Text value, Context context ) throws IOException, InterruptedException {
            String line = value.toString();
            StringTokenizer tokenizer = new StringTokenizer ( line );
            while ( tokenizer.hasMoreTokens() ) {
                String token = tokenizer.nextToken();
                if ( stopWords.contains ( token ) ) {
                    continue;
                }
                word.set ( tokenizer.nextToken() );
                context.write ( word, one );
            }
        }
    }
    public static class Reduce extends Reducer<Text, IntWritable, Text, IntWritable> {
        public void reduce ( Text key, Iterable<IntWritable> values, Context context )
        throws IOException, InterruptedException {
            int sum = 0;
            for ( IntWritable val : values ) {
                sum += val.get();
            }
            context.write ( key, new IntWritable ( sum ) );
        }
    }
    public static void main ( String[] args ) throws Exception {
        Configuration conf = new Configuration();
        conf.set ( "stop.words", "the, a, an, be, but, can" );
        Job job = new Job ( conf, "wordcount" );
        job.setOutputKeyClass ( Text.class );
        job.setOutputValueClass ( IntWritable.class );
        job.setMapperClass ( Map.class );
        job.setReducerClass ( Reduce.class );
        job.setInputFormatClass ( TextInputFormat.class );
        job.setOutputFormatClass ( TextOutputFormat.class );
        FileInputFormat.addInputPath ( job, new Path ( args[0] ) );
        FileOutputFormat.setOutputPath ( job, new Path ( args[1] ) );
        job.waitForCompletion ( true );
    }
}
