public class MySingleton
    implements IConfigurationObserver {
    private static volatile MySingleton instance;
    private final Configuration configuration;
    private volatile ParsedConfiguration currentConfig;
    private MySingleton ( Configuration configuration ) {
        this.configuration = configuration;
        parseConfiguration();
        configuration.addObserver ( this );
    }
    public static MySingleton getInstance ( Configuration configuration ) {
        MySingleton syncInstance = instance;
        if ( syncInstance == null ) {
            synchronized ( MySingleton.class ) {
                syncInstance = instance;
                if ( syncInstance == null ) {
                    syncInstance = new MySingleton ( configuration );
                    instance = syncInstance;
                }
            }
        }
        return syncInstance;
    }
    private void parseConfiguration() {
        currentConfig = configuration.getNewParsedConfiguration();
    }
    public String buildSentence() {
        ParsedConfiguration configInUse = currentConfig;
        StringBuilder strBuilder = new StringBuilder();
        strBuilder.append ( configInUse.getFirstParam() );
        strBuilder.append ( " - " );
        strBuilder.append ( configInUse.getSecondParam() );
        return strBuilder.toString();
    }
    @Override
    public void onConfigurationUpdated() {
        parseConfiguration();
    }
}
