public class FrameTest {
    public static void main ( String[] args ) {
        new FrameTest().createAndShow();
    }
    private JFrame frame;
    private BufferStrategy bufferStrategy;
    private Rectangle screenBounds;
    private GraphicsDevice myScreen;
    private boolean isRunning = false;
    private void createAndShow() {
        try {
            GraphicsEnvironment env = GraphicsEnvironment.getLocalGraphicsEnvironment();
            myScreen = env.getDefaultScreenDevice();
            GraphicsConfiguration gConf = myScreen.getDefaultConfiguration();
            screenBounds = gConf.getBounds();
            frame = new JFrame ( gConf );
            frame.setDefaultCloseOperation ( JFrame.DISPOSE_ON_CLOSE );
            frame.setUndecorated ( true );
            frame.addKeyListener ( new KeyAdapter() {
                @Override
                public void keyPressed ( KeyEvent e ) {
                    super.keyPressed ( e );
                    goExit();
                }
            } );
            frame.setVisible ( true );
            myScreen.setFullScreenWindow ( frame );
            bufferStrategy = null;
            BufferCapabilities bcaps = gConf.getBufferCapabilities();
            int amountBuffer = 0;
            if ( bcaps.isMultiBufferAvailable() ) {
                amountBuffer = 3;
            } else {
                amountBuffer = 2;
            }
            frame.createBufferStrategy ( amountBuffer, bcaps );
            bufferStrategy = frame.getBufferStrategy();
            isRunning = true;
            startRendering();
            startUniverse();
            System.out.println ( "created a buffer with " + amountBuffer + " pages" );
        } catch ( HeadlessException e ) {
        } catch ( AWTException e ) {
        }
    }
    private double xpos = 0;
    private double ypos = 0;
    private double xCenter = 0;
    private double yCenter = 0;
    private double radius = 100;
    private double dAngle = 0;
    private double angle = ( 2d * Math.PI / 360d );
    private void startUniverse() {
        Runnable r = new Runnable() {
            @Override
            public void run() {
                while ( isRunning ) {
                    xCenter = screenBounds.getCenterX();
                    yCenter = screenBounds.getCenterY();
                    dAngle = dAngle + angle;
                    xpos = xCenter + radius * ( Math.sin ( dAngle ) );
                    ypos = yCenter + radius * ( Math.cos ( dAngle ) );
                    try {
                        Thread.sleep ( 10 );
                    } catch ( InterruptedException e ) {
                        e.printStackTrace();
                    }
                }
            }
        };
        Thread t = new Thread ( r );
        t.setDaemon ( true );
        t.start();
    }
    private void startRendering() {
        Runnable r = new Runnable() {
            @Override
            public void run() {
                while ( isRunning ) {
                    Graphics g = null;
                    try {
                        g = bufferStrategy.getDrawGraphics();
                        render ( g );
                    } finally {
                        if ( g != null ) {
                            g.dispose();
                        }
                    }
                    bufferStrategy.show();
                }
            }
        };
        Thread t = new Thread ( r );
        t.setDaemon ( true );
        t.start();
    }
    protected void render ( Graphics g ) {
        g.setColor ( Color.WHITE );
        int x = 0;
        int y = 0;
        int width = screenBounds.width;
        int height = screenBounds.width;
        g.fillRect ( x, y, width, height );
        g.setColor ( Color.BLACK );
        int x_a = ( int ) xpos;
        int y_a = ( int ) ypos;
        int x_b = ( int ) xCenter;
        int y_b = ( int ) yCenter;
        g.drawLine ( x_a, y_a, x_b, y_b );
    }
    protected void goExit() {
        isRunning = false;
        frame.setVisible ( false );
        frame.dispose();
        myScreen.setFullScreenWindow ( null );
    }
}
