import java.awt.Color;
import java.awt.Dimension;
import java.awt.GradientPaint;
import java.io.File;
import java.io.IOException;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.CategoryPlot;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.renderer.category.BarRenderer;
import org.jfree.data.category.CategoryDataset;
import org.jfree.data.category.DefaultCategoryDataset;
import org.jfree.ui.ApplicationFrame;
import org.jfree.ui.RefineryUtilities;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;
public class BarChartDemo extends ApplicationFrame {
    public BarChartDemo ( final String title ) throws Exception {
        super ( title );
        final CategoryDataset dataset = createDataset();
        final JFreeChart chart = createChart ( dataset );
        final ChartPanel chartPanel = new ChartPanel ( chart );
        chartPanel.setPreferredSize ( new Dimension ( 500, 270 ) );
        setContentPane ( chartPanel );
    }
    private CategoryDataset createDataset() throws Exception {
        final DefaultCategoryDataset dataset = new DefaultCategoryDataset();
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        DocumentBuilder db = dbf.newDocumentBuilder();
        Document document = db.parse ( new File ( "src/input.xml" ) );
        NodeList nodeList = document.getElementsByTagName ( "result" );
        for ( int x = nodeList.getLength() - 1; x >= 0; x-- ) {
            dataset.addValue ( 10.0, nodeList.item ( x ).getAttributes().getNamedItem ( "state" ).getNodeValue(), nodeList.item ( x ).getAttributes().getNamedItem ( "key" ).getNodeValue() );
        }
        return dataset;
    }
    private JFreeChart createChart ( final CategoryDataset dataset ) {
        final JFreeChart chart = ChartFactory.createBarChart (
                                     "Bar Chart Demo",
                                     "key",
                                     "number",
                                     dataset,
                                     PlotOrientation.HORIZONTAL,
                                     true,
                                     true,
                                     false
                                 );
        chart.setBackgroundPaint ( Color.white );
        final CategoryPlot plot = chart.getCategoryPlot();
        plot.setBackgroundPaint ( Color.lightGray );
        plot.setDomainGridlinePaint ( Color.white );
        plot.setRangeGridlinePaint ( Color.white );
        final NumberAxis rangeAxis = ( NumberAxis ) plot.getRangeAxis();
        rangeAxis.setStandardTickUnits ( NumberAxis.createIntegerTickUnits() );
        final BarRenderer renderer = ( BarRenderer ) plot.getRenderer();
        renderer.setDrawBarOutline ( false );
        final GradientPaint gp0 = new GradientPaint (
            0.0f, 0.0f, Color.blue,
            0.0f, 0.0f, Color.lightGray
        );
        final GradientPaint gp1 = new GradientPaint (
            0.0f, 0.0f, Color.green,
            0.0f, 0.0f, Color.lightGray
        );
        final GradientPaint gp2 = new GradientPaint (
            0.0f, 0.0f, Color.red,
            0.0f, 0.0f, Color.lightGray
        );
        renderer.setSeriesPaint ( 0, gp0 );
        renderer.setSeriesPaint ( 1, gp1 );
        renderer.setSeriesPaint ( 2, gp2 );
        final CategoryAxis domainAxis = plot.getDomainAxis();
        domainAxis.setCategoryLabelPositions (
            CategoryLabelPositions.createUpRotationLabelPositions ( Math.PI / 6.0 )
        );
        return chart;
    }
    public static void main ( final String[] args ) throws Exception {
        final BarChartDemo demo = new BarChartDemo ( "Barchart" );
        demo.pack();
        RefineryUtilities.centerFrameOnScreen ( demo );
        demo.setVisible ( true );
    }
}
