public class DateTimeGapFinder {
    public List<Interval> findGaps ( List<Interval> existingIntervals, Interval searchInterval ) {
        List<Interval> gaps = new ArrayList<Interval>();
        DateTime searchStart = searchInterval.getStart();
        DateTime searchEnd = searchInterval.getEnd();
        if ( hasNoOverlap ( existingIntervals, searchInterval, searchStart, searchEnd ) ) {
            gaps.add ( searchInterval );
            return gaps;
        }
        List<Interval> subExistingList = removeNoneOverlappingIntervals ( existingIntervals, searchInterval );
        DateTime subEarliestStart = subExistingList.get ( 0 ).getStart();
        DateTime subLatestStop = subExistingList.get ( subExistingList.size() - 1 ).getEnd();
        if ( searchStart.isBefore ( subEarliestStart ) ) {
            gaps.add ( new Interval ( searchStart, subEarliestStart ) );
        }
        gaps.addAll ( getExistingIntervalGaps ( subExistingList ) );
        if ( searchEnd.isAfter ( subLatestStop ) ) {
            gaps.add ( new Interval ( subLatestStop, searchEnd ) );
        }
        return gaps;
    }
    private List<Interval> getExistingIntervalGaps ( List<Interval> existingList ) {
        List<Interval> gaps = new ArrayList<Interval>();
        Interval current = existingList.get ( 0 );
        for ( int i = 1; i < existingList.size(); i++ ) {
            Interval next = existingList.get ( i );
            Interval gap = current.gap ( next );
            if ( gap != null ) {
                gaps.add ( gap );
            }
            current = next;
        }
        return gaps;
    }
    private List<Interval> removeNoneOverlappingIntervals ( List<Interval> existingIntervals, Interval searchInterval ) {
        List<Interval> subExistingList = new ArrayList<Interval>();
        for ( Interval interval : existingIntervals ) {
            if ( interval.overlaps ( searchInterval ) ) {
                subExistingList.add ( interval );
            }
        }
        return subExistingList;
    }
    private boolean hasNoOverlap ( List<Interval> existingIntervals, Interval searchInterval, DateTime searchStart, DateTime searchEnd ) {
        DateTime earliestStart = existingIntervals.get ( 0 ).getStart();
        DateTime latestStop = existingIntervals.get ( existingIntervals.size() - 1 ).getEnd();
        if ( searchEnd.isBefore ( earliestStart ) || searchStart.isAfter ( latestStop ) ) {
            return true;
        }
        return false;
    }
