public class DisallowNullDeserializer<T> extends StdDeserializer<T> implements ResolvableDeserializer {
    private final JsonDeserializer<?> delegateDeserializer;
    public DisallowNullDeserializer ( Class<T> clazz, JsonDeserializer<?> delegateDeserializer ) {
        super ( clazz );
        this.delegateDeserializer = delegateDeserializer;
    }
    @Override
    public T deserialize ( JsonParser jp, DeserializationContext ctxt ) throws IOException, JsonProcessingException {
        T out = ( T ) delegateDeserializer.deserialize ( jp, ctxt );
        Map<String, Object> propertyMap = mapper.convertValue ( out, Map.class );
        for ( Object property : propertyMap.values() )
            if ( property == null ) {
                throw ctxt.mappingException ( "Can not map JSON null values." );
            }
        return out;
    }
    @Override
    public void resolve ( DeserializationContext ctxt ) throws JsonMappingException {
        ( ( ResolvableDeserializer ) delegateDeserializer ).resolve ( ctxt );
    }
}
