public class YourClass {
    private final String name;
    private final String description;
    private YourClass ( String name, String description ) {
        this.name = name;
        this.description = description;
    }
    public String getName() {
        return name;
    }
    public String getDescription() {
        return description;
    }
    @Override
    public int hashCode() {
        return name.hashCode();
    }
    @Override
    public boolean equals ( Object o ) {
        if ( o == this ) {
            return true;
        }
        if ( ! ( o instanceof YourClass ) ) {
            return false;
        }
        YourClass other = ( YourClass ) o;
        return this.name.equals ( other.getName() );
    }
    private static class YourClassFlyweight {
        static final Map<String, YourClass> cache =
            Collections.synchronizedMap (
                new WeakHashMap<String, YourClass>() );
        private YourClassFlyweight() { }
    }
    public static YourClass create ( String name, String description ) {
        if ( YourClassFlyweight.cache.containsKey ( name ) ) {
            return YourClassFlyweight.cache.get ( name );
        }
        synchronized ( YourClassFlyweight.cache ) {
            YourClass newInstance = new YourClass ( name, description );
            YourClassFlyweight.cache.put ( name, newInstance );
            return newInstance;
        }
    }
}
