protected final byte CPMEOF = 26;
protected final int MAXERRORS = 10;
protected final int SECSIZE = 128;
protected final int SENTIMOUT = 30;
protected final int SLEEP   = 30;
protected final byte    SOH = 1;
protected final byte    EOT = 4;
protected final byte    ACK = 6;
protected final byte    NAK = 0x15;
protected InputStream inStream;
protected OutputStream outStream;
protected PrintWriter errStream;
public TModem ( InputStream is, OutputStream os, PrintWriter errs ) {
    inStream = is;
    outStream = os;
    errStream = errs;
}
public TModem() {
    inStream = System.in;
    outStream = System.out;
    errStream = new PrintWriter ( System.err );
}
public static void main ( String[] argv ) throws
    Exception, IOException, InterruptedException {
    if ( argv.length != 2 ) {
        usage();
    }
    if ( argv[0].charAt ( 0 ) != '-' ) {
        usage();
    }
    TModem tm = new TModem( );
    tm.setStandalone ( true );
    boolean OK = false;
    switch ( argv[0].charAt ( 1 ) ) {
    case 'r':
        OK = tm.receive ( argv[1] );
        break;
    case 's':
        OK = tm.send ( argv[1] );
        break;
    default:
        usage();
    }
    System.out.print ( OK ? "Done OK" : "Failed" );
    System.exit ( 0 );
}
protected static void usage() {
    System.err.println ( "usage: TModem -r/-s file" );
    System.exit ( 1 );
}
protected boolean standalone = false;
public void setStandalone ( boolean is ) {
    standalone = is;
}
public boolean isStandalone() {
    return standalone;
}
protected boolean gotChar;
class IOTimer extends Thread {
    String message;
    long milliseconds;
    IOTimer ( long sec, String mesg ) {
        milliseconds = 1000 * sec;
        message = mesg;
    }
    public void run() {
        try {
            Thread.sleep ( milliseconds );
        } catch ( InterruptedException e ) {
            e.printStackTrace();
        }
        if ( !gotChar ) {
            errStream.println ( "Timed out waiting for " + message );
        }
        die ( 1 );
    }
}
public boolean send ( String tfile ) throws Exception, IOException, InterruptedException {
    Parameters param;
    param = new Parameters();
    param.setPort ( "COM1" );
    param.setBaudRate ( "115200" );
    param.setParity ( "N" );
    param.setByteSize ( "8" );
    Com com = new Com ( param );
    char checksum, index, blocknumber, errorcount;
    byte character;
    byte[] sector = new byte[SECSIZE];
    int nbytes;
    DataInputStream foo;
    foo = new DataInputStream ( new FileInputStream ( tfile ) );
    errStream.println ( "file open, ready to send" );
    System.out.println ( "file open, ready to send" );
    errorcount = 0;
    blocknumber = 1;
    gotChar = false;
    new IOTimer ( SENTIMOUT, "NAK to start send" ).start();
    do {
        character = getchar ( com );
        gotChar = true;
        if ( character != NAK && errorcount < MAXERRORS ) {
            ++errorcount;
        }
    } while ( character != NAK && errorcount < MAXERRORS );
    errStream.println ( "transmission beginning" );
    System.out.println ( "transmission beginning" );
    if ( errorcount == MAXERRORS ) {
        xerror();
    }
    while ( ( nbytes = inStream.read ( sector ) ) != 0 ) {
        if ( nbytes < SECSIZE ) {
            sector[nbytes] = CPMEOF;
        }
        errorcount = 0;
        while ( errorcount < MAXERRORS ) {
            errStream.println ( "{" + blocknumber + "} " );
            System.out.println ( "{" + blocknumber + "} " );
            putchar ( com,  SOH );
            putchar ( com,  blocknumber );
            putchar ( com,  ~blocknumber );
            checksum = 0;
            for ( index = 0; index < SECSIZE; index++ ) {
                putchar ( com, sector[index] );
                checksum += sector[index];
            }
            putchar ( com, checksum );
            if ( getchar ( com ) != ACK ) {
                ++errorcount;
            } else {
                break;
            }
        }
        if ( errorcount == MAXERRORS ) {
            xerror();
        }
        ++blocknumber;
    }
    boolean isAck = false;
    while ( !isAck ) {
        putchar ( com, EOT );
        isAck = getchar ( com ) == ACK;
    }
    errStream.println ( "Transmission complete." );
    return true;
}
public boolean receive ( String tfile ) throws Exception {
    Parameters param;
    param = new Parameters();
    param.setPort ( "COM1" );
    param.setBaudRate ( "115200" );
    param.setParity ( "N" );
    param.setByteSize ( "8" );
    Com com = new Com ( param );
    char checksum, index, blocknumber, errorcount;
    byte character;
    byte[] sector = new byte[SECSIZE];
    DataOutputStream foo;
    foo = new DataOutputStream ( new FileOutputStream ( tfile ) );
    System.out.println ( "you have " + SLEEP + " seconds..." );
    gotChar = false;
    new IOTimer ( SLEEP, "receive from remote" ).start();
    errStream.println ( "Starting receive..." );
    putchar ( com, NAK );
    errorcount = 0;
    blocknumber = 1;
    rxLoop:
    do {
        character = getchar ( com );
        gotChar = true;
        if ( character != EOT ) {
            try {
                byte not_ch;
                if ( character != SOH ) {
                    errStream.println ( "Not SOH" );
                    if ( ++errorcount < MAXERRORS ) {
                        continue rxLoop;
                    } else {
                        xerror();
                    }
                }
                character = getchar ( com );
                not_ch = ( byte ) ( ~getchar ( com ) );
                errStream.println ( "[" +  character + "] " );
                if ( character != not_ch ) {
                    errStream.println ( "Blockcounts not ~" );
                    ++errorcount;
                    continue rxLoop;
                }
                if ( character != blocknumber ) {
                    errStream.println ( "Wrong blocknumber" );
                    ++errorcount;
                    continue rxLoop;
                }
                checksum = 0;
                for ( index = 0; index < SECSIZE; index++ ) {
                    sector[index] = getchar ( com );
                    checksum += sector[index];
                }
                if ( checksum != getchar ( com ) ) {
                    errStream.println ( "Bad checksum" );
                    errorcount++;
                    continue rxLoop;
                }
                putchar ( com, ACK );
                blocknumber++;
                try {
                    foo.write ( sector );
                } catch ( IOException e ) {
                    errStream.println ( "write failed, blocknumber " + blocknumber );
                }
            } finally {
                if ( errorcount != 0 ) {
                    putchar ( com, NAK );
                }
            }
        }
    } while ( character != EOT );
    foo.close();
    putchar ( com, ACK );
    putchar ( com, ACK );
    putchar ( com, ACK );
    errStream.println ( "Receive Completed." );
    return true;
}
protected byte getchar ( Com com ) throws Exception {
    return ( byte ) com.receiveSingleDataInt();
}
protected void putchar ( Com com, int c ) throws Exception {
    com.sendSingleData ( c );
}
protected void xerror() {
    errStream.println ( "too many errors...aborting" );
    die ( 1 );
}
protected void die ( int how ) {
    if ( standalone ) {
        System.exit ( how );
    } else {
        System.out.println ( ( "Error code " + how ) );
    }
}
