import java.awt.Canvas;
import java.awt.Graphics2D;
import java.awt.Dimension;
import java.awt.image.BufferStrategy;
import java.applet.Applet;
public abstract class AppletGameCore extends Applet implements Runnable {
    private BufferStrategy bufferStrategy;
    private Canvas drawArea;
    private boolean stopped = false;
    private int x = 0;
    public void init() {
        Thread t = new Thread ( this );
        drawArea = new Canvas();
        setIgnoreRepaint ( true );
        t.start();
    }
    public void destroy() {
        stopped = true;
        super.destroy();
    }
    public void update() {
        if ( !bufferStrategy.contentsLost() ) {
            bufferStrategy.show();
        }
    }
    public BufferStrategy getBufferStrategy() {
        return bufferStrategy;
    }
    public void createBufferStrategy ( int numBuffers ) {
        drawArea.createBufferStrategy ( numBuffers );
    }
    public abstract void draw ( Graphics2D g );
    public void update ( Graphics2D g ) {
        g.setColor ( g.getBackground() );
        g.fillRect ( 0, 0, getWidth(), getHeight() );
    }
    public abstract void update ( long time );
    public Graphics2D getGraphics() {
        return ( Graphics2D ) bufferStrategy.getDrawGraphics();
    }
    public void run() {
        drawArea.setSize ( new Dimension ( getWidth(), getHeight() ) );
        add ( drawArea );
        createBufferStrategy ( 2 );
        bufferStrategy = drawArea.getBufferStrategy();
        long startTime = System.currentTimeMillis();
        long currTime = startTime;
        while ( !stopped ) {
            long elapsedTime = System.currentTimeMillis() - currTime;
            currTime += elapsedTime;
            update();
            update ( elapsedTime );
            Graphics2D g = getGraphics();
            update ( g );
            draw ( g );
            g.dispose();
        }
    }
}
