package numberexpression;
public class NumberExpression {
    private final NumberRange[] ranges;
    private final int min, max;
    public NumberExpression ( String pattern ) {
        String[] parts = pattern.toLowerCase().split ( ",", -1 );
        ranges = new NumberRange[parts.length];
        int min = Integer.MAX_VALUE, max = 0;
        for ( int i = 0; i < ranges.length; i++ ) {
            String part = parts[i];
            try {
                if ( part.equals ( "*" ) ) {
                    ranges[i] = new NumberRange ( 0, Integer.MAX_VALUE, 0, 1 );
                } else if ( part.matches ( "\\*/\\d+" ) ) {
                    ranges[i] = new NumberRange ( 0, Integer.MAX_VALUE, 0, Integer.parseInt ( part.substring ( 2 ) ) );
                } else if ( part.matches ( "\\d+" ) ) {
                    int value = Integer.parseInt ( part );
                    ranges[i] = new NumberRange ( value, value, 0, 1 );
                } else if ( part.matches ( "\\d*-\\d*" ) ) {
                    String[] limits = part.split ( "-", -1 );
                    int from = limits[0].length() == 0 ? 0 : Integer.parseInt ( limits[0] );
                    int to = limits[1].length() == 0 ? Integer.MAX_VALUE : Integer.parseInt ( limits[1] );
                    if ( to < from ) {
                        throw new IllegalArgumentException ( "Invalid pattern: " + part );
                    }
                    ranges[i] = new NumberRange ( from, to, 0, 1 );
                } else if ( part.matches ( "\\d*-\\d*/\\d+" ) ) {
                    String[] rangeAndModulus = part.split ( "/", -1 );
                    String[] limits = rangeAndModulus[0].split ( "-", -1 );
                    int from = limits[0].length() == 0 ? 0 : Integer.parseInt ( limits[0] );
                    int to = limits[1].length() == 0 ? Integer.MAX_VALUE : Integer.parseInt ( limits[1] );
                    int modulus = Integer.parseInt ( rangeAndModulus[1] );
                    if ( to < from ) {
                        throw new IllegalArgumentException ( "Invalid pattern: " + part );
                    }
                    ranges[i] = new NumberRange ( from, to, from % modulus, modulus );
                } else if ( part.matches ( "\\d*-\\d*[eo]" ) ) {
                    String[] limits = part.substring ( 0, part.length() - 1 ).split ( "-", -1 );
                    int from = limits[0].length() == 0 ? 0 : Integer.parseInt ( limits[0] );
                    int to = limits[1].length() == 0 ? Integer.MAX_VALUE : Integer.parseInt ( limits[1] );
                    if ( to < from ) {
                        throw new IllegalArgumentException ( "Invalid pattern: " + part );
                    }
                    ranges[i] = new NumberRange ( from, to, part.charAt ( part.length() - 1 ) == 'o' ? 1 : 0, 2 );
                } else {
                    throw new IllegalArgumentException ( "Invalid pattern: " + part );
                }
                max = Math.max ( max, ranges[i].getMax() );
                min = Math.min ( min, ranges[i].getMin() );
            } catch ( NumberFormatException ex ) {
                throw new IllegalArgumentException ( "Invalid pattern: " + part );
            }
        }
        this.max = max;
        this.min = min;
    }
    public boolean matches ( int number ) {
        if ( number < min || number > max ) {
            return false;
        }
        for ( int i = 0; i < ranges.length; i++ ) {
            if ( ranges[i].matches ( number ) ) {
                return true;
            }
        }
        return false;
    }
    public int getMinimum() {
        return min;
    }
    public int getMaximum() {
        return max;
    }
    private static class NumberRange {
        private final int min, max, remainder, modulus;
        NumberRange ( int min, int max, int remainder, int modulus ) {
            this.min = min;
            this.max = max;
            this.remainder = remainder;
            this.modulus = modulus;
        }
        boolean matches ( int number ) {
            return number >= min && number <= max && number % modulus == remainder;
        }
        int getMin() {
            return min;
        }
        int getMax() {
            return max;
        }
    }
}
