class SurfaceHolderCallback implements SurfaceHolder.Callback {
    private static final int IMAGE_WIDTH = 512;
    private static final int IMAGE_HEIGHT = 384;
    private static final String ORIENTATION = "orientation";
    private static final String ROTATION = "rotation";
    private static final String PORTRAIT = "portrait";
    private static final String LANDSCAPE = "landscape";
    public void surfaceCreated ( SurfaceHolder holder ) {
        try {
            camera = Camera.open();
        } catch ( Exception e ) {
            finish();
            return;
        }
        Parameters p = camera.getParameters();
        p.setPictureSize ( IMAGE_WIDTH, IMAGE_HEIGHT );
        camera.getParameters().setRotation ( 90 );
        Camera.Size s = p.getSupportedPreviewSizes().get ( 0 );
        p.setPreviewSize ( s.width, s.height );
        p.setPictureFormat ( PixelFormat.JPEG );
        p.set ( "flash-mode", "auto" );
        camera.setParameters ( p );
        try {
            camera.setPreviewDisplay ( surfaceHolder );
        } catch ( Throwable ignored ) {
            Log.e ( APP, "set preview error.", ignored );
        }
    }
    public void surfaceChanged ( SurfaceHolder holder, int format, int width,
                                 int height ) {
        if ( isPreviewRunning ) {
            camera.stopPreview();
        }
        try {
            camera.startPreview();
        } catch ( Exception e ) {
            Log.d ( APP, "Cannot start preview", e );
        }
        isPreviewRunning = true;
    }
    public void surfaceDestroyed ( SurfaceHolder arg0 ) {
        if ( isPreviewRunning && camera != null ) {
            if ( camera != null ) {
                camera.stopPreview();
                camera.release();
                camera = null;
            }
            isPreviewRunning = false;
        }
    }
}
