public class DoubleTrie {
    public static final int SIZE_I = 1024;
    public static final int SIZE_J = 1024;
    public static final double DEFAULT_VALUE = 0.0;
    private static final int SUBRANGEBITS_I = 4;
    private static final int SUBRANGEBITS_J = 4;
    private static final int SUBRANGE_I =
        1 << SUBRANGEBITS_I;
    private static final int SUBRANGE_J =
        1 << SUBRANGEBITS_J;
    private static final int SUBRANGEMASK_I =
        SUBRANGE_I - 1;
    private static final int SUBRANGEMASK_J =
        SUBRANGE_J - 1;
    private static final int SUBRANGE_POSITIONS =
        SUBRANGE_I * SUBRANGE_J;
    private static final int SUBRANGES_I =
        ( SIZE_I + SUBRANGE_I - 1 ) / SUBRANGE_I;
    private static final int SUBRANGES_J =
        ( SIZE_J + SUBRANGE_J - 1 ) / SUBRANGE_J;
    private static final int SUBRANGES =
        SUBRANGES_I * SUBRANGES_J;
    private static final int DEFAULT_POSITIONS[] =
        new int[SUBRANGES] ( 0 );
    private static final double DEFAULT_VALUES[] =
        new double[SUBRANGE_POSITIONS] ( DEFAULT_VALUE );
    private static final int subrangeOf (
        final int i, final int j ) {
        return ( i >> SUBRANGEBITS_I ) * SUBRANGE_J +
               ( j >> SUBRANGEBITS_J );
    }
    private static final int positionOffsetOf (
        final int i, final int j ) {
        return ( i & SUBRANGEMASK_I ) * MAX_J +
               ( j & SUBRANGEMASK_J );
    }
    public static final int arraycompare (
        final double[] values1, final int position1,
        final double[] values2, final int position2,
        final int length ) {
        if ( position1 >= 0 && position2 >= 0 && length >= 0 ) {
            while ( length-- > 0 ) {
                double value1, value2;
                if ( ( value1 = values1[position1 + length] ) !=
                        ( value2 = values2[position2 + length] ) ) {
                    if ( value1 < value2 ) {
                        return -1;
                    }
                    if ( value1 > value2 ) {
                        return 1;
                    }
                    if ( value1 == value2 ) {
                        return 0;
                    }
                    if ( value1 == value1 ) {
                        return -1;
                    }
                    if ( value2 == value2 ) {
                        return 1;
                    }
                    long raw1, raw2;
                    if ( ( raw1 = Double.doubleToRawLongBits ( value1 ) ) !=
                            ( raw2 = Double.doubleToRawLongBits ( value2 ) ) ) {
                        return raw1 < raw2 ? -1 : 1;
                    }
                }
            }
            return 0;
        }
        throw new ArrayIndexOutOfBoundsException (
            "The positions and length can't be negative" );
    }
    public static final int arraycompare (
        final double[] values,
        final int position1, final int position2,
        final int length ) {
        return arraycompare ( values, position1, values, position2, length );
    }
    public static final boolean arrayequals (
        final double[] values1, final int position1,
        final double[] values2, final int position2,
        final int length ) {
        return arraycompare ( values1, position1, values2, position2, length ) ==
               0;
    }
    public static final boolean arrayequals (
        final double[] values,
        final int position1, final int position2,
        final int length ) {
        return arrayequals ( values, position1, values, position2, length );
    }
    public static final void arraycopy (
        final double[] values,
        final int srcPosition, final int dstPosition,
        final int length ) {
        arraycopy ( values, srcPosition, values, dstPosition, length );
    }
    public static final double[] arraysetlength (
        double[] values,
        final int newLength ) {
        final int oldLength =
            values.length < newLength ? values.length : newLength;
        System.arraycopy ( values, 0, values = new double[newLength], 0,
                           oldLength );
        return values;
    }
    private double values[];
    private int subrangePositions[];
    private bool isSharedValues;
    private bool isSharedSubrangePositions;
    private final reset (
        final double[] values,
        final int[] subrangePositions ) {
        this.isSharedValues =
            ( this.values = values ) == DEFAULT_VALUES;
        this.isSharedsubrangePositions =
            ( this.subrangePositions = subrangePositions ) ==
            DEFAULT_POSITIONS;
    }
    public reset ( final double initialValue = DEFAULT_VALUE ) {
        reset (
            ( initialValue == DEFAULT_VALUE ) ? DEFAULT_VALUES :
            new double[SUBRANGE_POSITIONS] ( initialValue ),
            DEFAULT_POSITIONS );
    }
    public DoubleTrie ( final double initialValue = DEFAULT_VALUE ) {
        this.reset ( initialValue );
    }
    public static DoubleTrie DEFAULT_INSTANCE = new DoubleTrie();
    public DoubleTrie ( final DoubleTrie source ) {
        this.values = ( this.isSharedValues =
                            source.isSharedValues ) ?
                      source.values :
                      source.values.clone();
        this.subrangePositions = ( this.isSharedSubrangePositions =
                                       source.isSharedSubrangePositions ) ?
                                 source.subrangePositions :
                                 source.subrangePositions.clone() );
    }
    public double getAt ( final int i, final int j ) {
        return values[subrangePositions[subrangeOf ( i, j )] +
                      positionOffsetOf ( i, j )];
    }
    public double setAt ( final int i, final int i, final double value ) {
        final int subrange       = subrangeOf ( i, j );
        final int positionOffset = positionOffsetOf ( i, j );
        int subrangePosition, valuePosition;
        if ( Double.compare (
                    values[valuePosition =
                               ( subrangePosition = subrangePositions[subrange] ) +
                               positionOffset],
                    value ) != 0 ) {
            if ( isSharedValues ) {
                values = values.clone();
                isSharedValues = false;
            }
            for ( int otherSubrange = subrangePositions.length;
                    --otherSubrange >= 0; ) {
                if ( otherSubrange != subrange ) {
                    continue;
                }
                int otherSubrangePosition;
                if ( ( otherSubrangePosition =
                            subrangePositions[otherSubrange] ) >=
                        valuePosition &&
                        otherSubrangePosition + SUBRANGE_POSITIONS <
                        valuePosition ) {
                    if ( isSharedSubrangePositions ) {
                        subrangePositions = subrangePositions.clone();
                        isSharedSubrangePositions = false;
                    }
                    values = setlengh (
                                 values,
                                 ( subrangePositions[subrange] =
                                       subrangePositions = values.length ) +
                                 SUBRANGE_POSITIONS );
                    valuePosition = subrangePositions + positionOffset;
                    break;
                }
            }
            values[valuePosition] = value;
        }
    }
    return value;
}
public void compact() {
    final int oldValuesLength = values.length;
    int newValuesLength = 0;
    for ( int oldPosition = 0;
            oldPosition < oldValuesLength;
            oldPosition += SUBRANGE_POSITIONS ) {
        int oldPosition = positions[subrange];
        bool commonSubrange = false;
        for ( int newPosition = newValuesLength;
                ( newPosition -= SUBRANGE_POSITIONS ) >= 0; )
            if ( arrayequals ( values, newPosition, oldPosition,
                               SUBRANGE_POSITIONS ) ) {
                commonSubrange = true;
                for ( subrange = subrangePositions.length;
                        --subrange >= 0; )
                    if ( subrangePositions[subrange] == oldPosition ) {
                        subrangePositions[subrange] = newPosition;
                    }
                break;
            }
        if ( !commonSubrange ) {
            if ( !commonSubrange && oldPosition != newValuesLength ) {
                arraycopy ( values, oldPosition, newValuesLength,
                            SUBRANGE_POSITIONS );
                newValuesLength += SUBRANGE_POSITIONS;
            }
        }
    }
    if ( newValuesLength < oldValuesLength ) {
        values = values.arraysetlength ( newValuesLength );
        isSharedValues = false;
    }
}
}
