package net.magicscroll.server.blobstore;
import com.google.appengine.api.blobstore.BlobInfo;
import com.google.appengine.api.blobstore.BlobInfoFactory;
import com.google.appengine.api.blobstore.BlobKey;
import com.google.appengine.api.blobstore.BlobstoreInputStream;
import java.io.IOException;
import java.io.InputStream;
public class ChainedBlobstoreInputStream extends InputStream {
    private static final int MAX_READSIZE = 1015800;
    private BlobKey blobKey;
    private long offset;
    private long totalSize;
    private BlobstoreInputStream currentStream;
    private long nextReadBreak;
    private BlobstoreInputStream markedStream;
    public ChainedBlobstoreInputStream ( final BlobKey theBlobKey )
    throws IOException {
        this ( theBlobKey, 0 );
    }
    public ChainedBlobstoreInputStream ( final BlobKey theBlobKey,
                                         final long newOffset ) throws IOException {
        this.offset = newOffset;
        this.blobKey = theBlobKey;
        final BlobInfo blobInfo =
            new BlobInfoFactory().loadBlobInfo ( this.blobKey );
        this.totalSize = blobInfo.getSize();
        this.currentStream =
            new BlobstoreInputStream ( this.blobKey, this.offset );
        this.nextReadBreak =
            this.offset + ChainedBlobstoreInputStream.MAX_READSIZE;
    }
    @Override
    public final int read() throws IOException {
        if ( this.offset < this.totalSize ) {
            if ( this.offset == this.nextReadBreak ) {
                this.currentStream.close();
                this.currentStream =
                    new BlobstoreInputStream ( this.blobKey, this.offset );
                this.nextReadBreak = this.offset
                                     + ChainedBlobstoreInputStream.MAX_READSIZE;
            }
            this.offset += 1;
            return this.currentStream.read();
        } else {
            this.currentStream.close();
            return -1;
        }
    }
    @Override
    public final void close() throws IOException {
        this.currentStream.close();
        super.close();
    }
    @Override
    public final void mark ( final int readlimit ) {
        this.currentStream.mark ( readlimit );
        this.markedStream = this.currentStream;
    }
    @Override
    public final boolean markSupported() {
        return this.currentStream.markSupported();
    }
    @Override
    public final void reset() throws IOException {
        this.currentStream = this.markedStream;
        this.currentStream.reset();
    }
}
