package persistence;
import java.sql.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
public class DatabaseUtils {
    private static final String DEFAULT_DRIVER = "";
    private static final String DEFAULT_URL = "";
    private static final String DEFAULT_USERNAME = "";
    private static final String DEFAULT_PASSWORD = "";
    public static void main ( String[] args ) {
        String driver = ( ( args.length > 0 ) ? args[0] : DEFAULT_DRIVER );
        String url = ( ( args.length > 1 ) ? args[1] : DEFAULT_URL );
        String username = ( ( args.length > 2 ) ? args[2] : DEFAULT_USERNAME );
        String password = ( ( args.length > 3 ) ? args[3] : DEFAULT_PASSWORD );
        Connection connection = null;
        try {
            connection = createConnection ( driver, url, username, password );
            DatabaseMetaData meta = connection.getMetaData();
            System.out.println ( meta.getDatabaseProductName() );
            System.out.println ( meta.getDatabaseProductVersion() );
        } catch ( Exception e ) {
            e.printStackTrace();
        } finally {
            close ( connection );
        }
    }
    public static Connection createConnection ( String driver, String url, String username, String password ) throws ClassNotFoundException, SQLException {
        Class.forName ( driver );
        if ( ( username == null ) || ( password == null ) || ( username.trim().length() == 0 ) || ( password.trim().length() == 0 ) ) {
            return DriverManager.getConnection ( url );
        } else {
            return DriverManager.getConnection ( url, username, password );
        }
    }
    public static void close ( Connection connection ) {
        try {
            if ( connection != null ) {
                connection.close();
            }
        } catch ( SQLException e ) {
            e.printStackTrace();
        }
    }
    public static void close ( Statement st ) {
        try {
            if ( st != null ) {
                st.close();
            }
        } catch ( SQLException e ) {
            e.printStackTrace();
        }
    }
    public static void close ( ResultSet rs ) {
        try {
            if ( rs != null ) {
                rs.close();
            }
        } catch ( SQLException e ) {
            e.printStackTrace();
        }
    }
    public static void rollback ( Connection connection ) {
        try {
            if ( connection != null ) {
                connection.rollback();
            }
        } catch ( SQLException e ) {
            e.printStackTrace();
        }
    }
    public static List<Map<String, Object>> map ( ResultSet rs ) throws SQLException {
        List<Map<String, Object>> results = new ArrayList<Map<String, Object>>();
        try {
            if ( rs != null ) {
                ResultSetMetaData meta = rs.getMetaData();
                int numColumns = meta.getColumnCount();
                while ( rs.next() ) {
                    Map<String, Object> row = new HashMap<String, Object>();
                    for ( int i = 1; i <= numColumns; ++i ) {
                        String name = meta.getColumnName ( i );
                        Object value = rs.getObject ( i );
                        row.put ( name, value );
                    }
                    results.add ( row );
                }
            }
        } finally {
            close ( rs );
        }
        return results;
    }
}
