public static <T> T getManagedBean ( String managedBeanKey, Class<T> clazz )
throws IllegalArgumentException {
    Validate.notNull ( managedBeanKey );
    Validate.isTrue ( !managedBeanKey.isEmpty() );
    Validate.notNull ( clazz );
    FacesContext facesContext = FacesContext.getCurrentInstance();
    if ( facesContext == null ) {
        return null;
    }
    Validate.notNull ( facesContext.getApplication() );
    ELResolver resolver = facesContext.getApplication().getELResolver();
    Validate.notNull ( resolver );
    ELContext elContext = facesContext.getELContext();
    Validate.notNull ( elContext );
    Object managedBean = resolver.getValue (
                             elContext, null, managedBeanKey );
    if ( !elContext.isPropertyResolved() ) {
        throw new IllegalArgumentException (
            "No managed bean found for key: " + managedBeanKey );
    }
    if ( managedBean == null ) {
        return null;
    } else {
        if ( clazz.isInstance ( managedBean ) ) {
            return clazz.cast ( managedBean );
        } else {
            throw new IllegalArgumentException (
                "Managed bean is not of type [" + clazz.getName() +
                "] | Actual type is: [" + managedBean.getClass().getName() +
                "]" );
        }
    }
}
