import java.lang.reflect.InvocationHandler;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
public class DynamicProxy {
    public static class DelegatingInvocationHandler
        implements InvocationHandler {
        private final Object delegate;
        public DelegatingInvocationHandler ( final Object delegate ) {
            this.delegate = delegate;
        }
        @Override
        public Object invoke ( final Object proxy, final Method m,
                               final Object[] args ) throws Throwable {
            Object result;
            try {
                result = m.invoke ( delegate, args );
            } catch ( final InvocationTargetException e ) {
                throw e.getTargetException();
            } catch ( final Exception e ) {
                throw new RuntimeException ( "unexpected invocation exception: "
                                             + e.getMessage() );
            }
            return result;
        }
    }
    public static <T> T createSelectiveProxy ( final T delegate,
            final Class<T> requiredInterface,
            final Class<?>... moreInterfaces ) {
        if ( delegate == null ) {
            throw new IllegalArgumentException (
                "The delegate object cannot be null" );
        }
        return createProxy ( new DelegatingInvocationHandler ( delegate ),
                             requiredInterface, moreInterfaces );
    }
    @SuppressWarnings ( "unchecked" )
    public static <T> T createProxy ( final InvocationHandler invocationHandler,
                                      final Class<T> requiredInterface,
                                      final Class<?>... moreInterfaces ) {
        if ( invocationHandler == null ) {
            throw new IllegalArgumentException (
                "The invocation handler cannot be null" );
        }
        final int size = ( moreInterfaces != null ? moreInterfaces.length : 0 );
        final Class<?>[] interfaces = new Class<?>[size + 1];
        interfaces[0] = requiredInterface;
        System.arraycopy ( moreInterfaces, 0, interfaces, 1, moreInterfaces.length );
        return ( T ) Proxy.newProxyInstance ( invocationHandler.getClass()
                                              .getClassLoader(), interfaces, invocationHandler );
    }
}
