public class DynamicProxyDemo {
    private interface A {
        void methodA();
    }
    private interface B {
        void methodB();
    }
    private static class Foo implements A, B {
        public void methodA() {
            System.out.println ( "A" );
        }
        public void methodB() {
            System.out.println ( "B" );
        }
    }
    private DynamicProxyDemo() {}
    public static void main ( final String[] args ) {
        final Foo foo = new Foo();
        final A a = DynamicProxy.createSelectiveProxy ( foo, A.class );
        final B b = DynamicProxy.createSelectiveProxy ( foo, B.class );
        final A ab = DynamicProxy.createSelectiveProxy ( foo, A.class, B.class );
        System.out.println ( "\n *** Call a method A.methodA() on proxy 'a'" );
        a.methodA();
        try {
            System.out.println ( "\n *** Call a method B.methodB() on proxy 'a' (throws exception)" );
            ( ( B ) a ).methodB();
        } catch ( final Exception ex ) {
            ex.printStackTrace ( System.out );
        }
        System.out.println ( "\n *** Call a method B.methodB() on proxy 'b'" );
        b.methodB();
        try {
            System.out.println ( "\n *** Call a method A.methodA() on proxy 'b' (throws exception)" );
            ( ( A ) b ).methodA();
        } catch ( final Exception ex ) {
            ex.printStackTrace ( System.out );
        }
        System.out.println ( "\n *** Call a method A.methodA() on proxy 'ab'" );
        ab.methodA();
        System.out.println ( "\n *** Call a method B.methodB() on proxy 'ab'" );
        ( ( B ) ab ).methodB();
        try {
            System.out.println ( "\n *** Call a method 'A' of class 'Foo' on proxy 'a' (throws exception)" );
            ( ( Foo ) a ).methodA();
        } catch ( final Exception ex ) {
            ex.printStackTrace ( System.out );
        }
        try {
            System.out.println ( "\n *** Call a method 'B' of class 'Foo' on proxy 'b' (throws exception)" );
            ( ( Foo ) b ).methodB();
        } catch ( final Exception ex ) {
            ex.printStackTrace ( System.out );
        }
        try {
            System.out.println ( "\n *** Call a method B.methodB() on proxy 'a' (throws exception)" );
            ( ( B ) a ).methodB();
        } catch ( final Exception ex ) {
            ex.printStackTrace ( System.out );
        }
        try {
            System.out.println ( "\n *** Call a method A.methodA() on proxy 'b' (throws exception)" );
            ( ( A ) b ).methodA();
        } catch ( final Exception ex ) {
            ex.printStackTrace ( System.out );
        }
    }
}
