import javax.swing.JComponent;
import javax.swing.RepaintManager;
import javax.swing.SwingUtilities;
public class CheckThreadViolationRepaintManager extends RepaintManager {
    private boolean completeCheck = true;
    public synchronized void addInvalidComponent ( JComponent component ) {
        checkThreadViolations ( component );
        super.addInvalidComponent ( component );
    }
    public boolean isCompleteCheck() {
        return completeCheck;
    }
    public void setCompleteCheck ( boolean completeCheck ) {
        this.completeCheck = completeCheck;
    }
    public void addDirtyRegion ( JComponent component, int x, int y, int w, int h ) {
        checkThreadViolations ( component );
        super.addDirtyRegion ( component, x, y, w, h );
    }
    private void checkThreadViolations ( JComponent c ) {
        if ( !SwingUtilities.isEventDispatchThread() && ( completeCheck || c.isShowing() ) ) {
            Exception exception = new Exception ( "EDT Violation" );
            boolean repaint = false;
            boolean fromSwing = false;
            StackTraceElement[] stackTrace = exception.getStackTrace();
            for ( StackTraceElement st : stackTrace ) {
                if ( repaint && st.getClassName().startsWith ( "javax.swing." ) ) {
                    fromSwing = true;
                }
                if ( "repaint".equals ( st.getMethodName() ) ) {
                    repaint = true;
                }
            }
            if ( repaint && !fromSwing ) {
                return;
            }
            exception.printStackTrace();
        }
    }
}
