package monthly.schedule;
import java.util.Timer;
import java.util.TimerTask;
import java.util.Date;
import java.util.Calendar;
public class MonthlyTimer {
    private final Runnable whatToDo;
    private final int dayOfMonth;
    private final int hourOfDay;
    private Timer current = new Timer();
    public void cancelCurrent() {
        current.cancel();
        current.purge();
    }
    public static MonthlyTimer schedule ( Runnable runnable, int dayOfMonth, int hourOfDay ) {
        return new MonthlyTimer ( runnable, dayOfMonth, hourOfDay );
    }
    private MonthlyTimer ( Runnable runnable, int day, int hour ) {
        this.whatToDo = runnable;
        this.dayOfMonth = day;
        this.hourOfDay = hour;
        schedule();
    }
    private void schedule() {
        cancelCurrent();
        current = new Timer();
        current.schedule ( new TimerTask() {
            public void run() {
                try {
                    whatToDo.run();
                } finally {
                    schedule();
                }
            }
        } , nextDate() );
    }
    private Date nextDate() {
        Calendar runDate = Calendar.getInstance();
        runDate.set ( Calendar.DAY_OF_MONTH, dayOfMonth );
        runDate.set ( Calendar.HOUR_OF_DAY, hourOfDay );
        runDate.set ( Calendar.MINUTE, 0 );
        runDate.add ( Calendar.MONTH, 1 );
        return runDate.getTime();
    }
}
class UseIt {
    public static void main ( String [] args ) {
        int the1st = 1;
        int at16hrs = 16;
        MonthlyTimer t = MonthlyTimer.schedule ( new Runnable() {
            public void run() {
                System.out.println ( "Hola" );
            }
        }, the1st, at16hrs );
        t.cancelCurrent();
    }
}
