import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import javax.swing.*;
import javax.swing.text.JTextComponent;
@SuppressWarnings ( "serial" )
public class TestScrollingArea extends JPanel {
    private static final String UP = "Up";
    private static final String DOWN = "Down";
    private JTextArea area = new JTextArea ( 20, 40 );
    private JScrollPane scrollPane = new JScrollPane ( area );
    public TestScrollingArea() {
        area.setEditable ( false );
        area.setFocusable ( false );
        area.setWrapStyleWord ( true );
        area.setLineWrap ( true );
        add ( scrollPane );
        for ( int i = 0; i < 10; i++ ) {
            for ( int j = 0; j < 100; j++ ) {
                area.append ( "abcdefg " );
            }
        }
        int scrollableIncrement = area.getScrollableUnitIncrement ( scrollPane.getVisibleRect(),
                                  SwingConstants.VERTICAL, 1 );
        int condition = JTextComponent.WHEN_IN_FOCUSED_WINDOW;
        InputMap inMap = area.getInputMap ( condition );
        ActionMap actMap = area.getActionMap();
        inMap.put ( KeyStroke.getKeyStroke ( KeyEvent.VK_UP, 0 ), UP );
        inMap.put ( KeyStroke.getKeyStroke ( KeyEvent.VK_DOWN, 0 ), DOWN );
        actMap.put ( UP, new UpDownAction ( UP, scrollPane.getVerticalScrollBar().getModel(),
                                            scrollableIncrement ) );
        actMap.put ( DOWN, new UpDownAction ( DOWN, scrollPane.getVerticalScrollBar().getModel(),
                                              scrollableIncrement ) );
    }
    private class UpDownAction extends AbstractAction {
        private BoundedRangeModel vScrollBarModel;
        private int scrollableIncrement;
        public UpDownAction ( String name, BoundedRangeModel model, int scrollableIncrement ) {
            super ( name );
            this.vScrollBarModel = model;
            this.scrollableIncrement = scrollableIncrement;
        }
        @Override
        public void actionPerformed ( ActionEvent ae ) {
            String name = getValue ( AbstractAction.NAME ).toString();
            int value = vScrollBarModel.getValue();
            if ( name.equals ( UP ) ) {
                value -= scrollableIncrement;
                vScrollBarModel.setValue ( value );
            } else if ( name.equals ( DOWN ) ) {
                value += scrollableIncrement;
                vScrollBarModel.setValue ( value );
            }
        }
    }
    private static void createAndShowUI() {
        JFrame frame = new JFrame ( "TestScrollingArea" );
        frame.getContentPane().add ( new TestScrollingArea() );
        frame.setDefaultCloseOperation ( JFrame.EXIT_ON_CLOSE );
        frame.pack();
        frame.setLocationRelativeTo ( null );
        frame.setVisible ( true );
    }
    public static void main ( String[] args ) {
        java.awt.EventQueue.invokeLater ( new Runnable() {
            public void run() {
                createAndShowUI();
            }
        } );
    }
}
