public class Animals {
    private Animals() {
    }
    interface Animal {
        void accept ( final AnimalProcessor visitor );
    }
    interface AnimalProcessor {
        void visitTailed ( final TailedAnimal tailedAnimal );
        void visitHorned ( final HornedAnimal hornedAnimal );
    }
    interface TailedAnimal extends Animal {
        void moveTail();
    }
    interface HornedAnimal extends Animal {
        void hitWithHorns();
    }
    static class Dog implements TailedAnimal {
        public void moveTail() {
        }
        public void accept ( final AnimalProcessor visitor ) {
            visitor.visitTailed ( this );
        }
    }
    static class Cat implements TailedAnimal {
        public void moveTail() {
        }
        public void accept ( final AnimalProcessor visitor ) {
            visitor.visitTailed ( this );
        }
    }
    static class Ram implements HornedAnimal {
        public void hitWithHorns() {
        }
        public void accept ( final AnimalProcessor visitor ) {
            visitor.visitHorned ( this );
        }
    }
    static class Rhinoceros implements HornedAnimal, TailedAnimal {
        public void hitWithHorns() {
        }
        public void moveTail() {
        }
        public void accept ( final AnimalProcessor visitor ) {
            visitor.visitTailed ( this );
            visitor.visitHorned ( this );
        }
    }
    public static void main ( String[] args ) {
        Collection<Animal> animals = new ArrayList<Animal> ( Arrays.asList ( new Dog(), new Cat(), new Rhinoceros() ) );
        for ( final Animal animal : animals ) {
            animal.accept ( new AnimalProcessor() {
                public void visitTailed ( final TailedAnimal tailedAnimal ) {
                }
                public void visitHorned ( final HornedAnimal hornedAnimal ) {
                }
            } );
        }
    }
}
