public class OutlinedTextView extends TextView {
    private static final float OUTLINE_PROPORTION = 0.1f;
    private final Paint mStrokePaint = new Paint();
    private final Rect mTextBounds = new Rect();
    private int mOutlineColor = Color.TRANSPARENT;
    public OutlinedTextView ( Context context ) {
        super ( context );
        this.setupPaint();
    }
    public OutlinedTextView ( Context context, AttributeSet attrs ) {
        super ( context, attrs );
        this.setupPaint();
        this.setupAttributes ( context, attrs );
    }
    public OutlinedTextView ( Context context, AttributeSet attrs, int defStyle ) {
        super ( context, attrs, defStyle );
        this.setupPaint();
        this.setupAttributes ( context, attrs );
    }
    @Override
    protected void onDraw ( Canvas canvas ) {
        final float textSize = super.getTextSize();
        final String text = super.getText().toString();
        mStrokePaint.setColor ( mOutlineColor );
        mStrokePaint.setStrokeWidth ( textSize * OUTLINE_PROPORTION );
        mStrokePaint.setTextSize ( textSize );
        mStrokePaint.setFlags ( super.getPaintFlags() );
        mStrokePaint.setTypeface ( super.getTypeface() );
        super.getPaint().getTextBounds ( text, 0, text.length(), mTextBounds );
        canvas.drawText ( text,
                          super.getWidth() * 0.5f, ( super.getHeight() + mTextBounds.height() ) * 0.5f,
                          mStrokePaint );
        super.onDraw ( canvas );
    }
    private final void setupPaint() {
        mStrokePaint.setAntiAlias ( true );
        mStrokePaint.setStyle ( Paint.Style.STROKE );
        mStrokePaint.setTextAlign ( Paint.Align.CENTER );
    }
    private final void setupAttributes ( Context context, AttributeSet attrs ) {
        final TypedArray array = context.obtainStyledAttributes ( attrs,
                                 R.styleable.OutlinedTextView );
        mOutlineColor = array.getColor (
                            R.styleable.OutlinedTextView_outlineColor, 0x00000000 );
        array.recycle();
        super.setGravity ( Gravity.CENTER_HORIZONTAL );
    }
}
