package difflib.annotate;
import java.util.*;
import difflib.*;
public class Annotate<R> {
    private final List<R> revisions;
    private final int[] lineNumbers;
    private R currentRevision;
    private final List<Object> currentLines;
    private final List<Integer> currentLineMap;
    public Annotate ( R revision, List<?> targetLines ) {
        revisions = new ArrayList<R>();
        lineNumbers = new int[targetLines.size()];
        currentRevision = revision;
        currentLines = new ArrayList<Object> ( targetLines );
        currentLineMap = new ArrayList<Integer>();
        for ( int i = 0; i < lineNumbers.length; i++ ) {
            lineNumbers[i] = -1;
            revisions.add ( null );
            currentLineMap.add ( i );
        }
    }
    public boolean areLinesUnannotated() {
        for ( int i = 0; i < lineNumbers.length; i++ ) {
            if ( lineNumbers[i] == -1 || revisions.get ( i ) == null ) {
                return true;
            }
        }
        return false;
    }
    public void addRevision ( R revision, List<?> lines ) {
        Patch patch = DiffUtils.diff ( currentLines, lines );
        int lineOffset = 0;
        for ( Delta d : patch.getDeltas() ) {
            Chunk original = d.getOriginal();
            Chunk revised = d.getRevised();
            int pos = original.getPosition() + lineOffset;
            for ( int i = 0; i < original.getSize(); i++ ) {
                int origLine = currentLineMap.remove ( pos );
                currentLines.remove ( pos );
                if ( origLine != -1 ) {
                    lineNumbers[origLine] = original.getPosition() + i;
                    revisions.set ( origLine, currentRevision );
                }
            }
            for ( int i = 0; i < revised.getSize(); i++ ) {
                currentLines.add ( pos + i, revised.getLines().get ( i ) );
                currentLineMap.add ( pos + i, -1 );
            }
            lineOffset += revised.getSize() - original.getSize();
        }
        currentRevision = revision;
        if ( !currentLines.equals ( lines ) ) {
            throw new RuntimeException ( "Patch application failed" );
        }
    }
    public List<R> getAnnotatedRevisions() {
        return Collections.unmodifiableList ( revisions );
    }
    public int[] getAnnotatedLineNumbers() {
        return ( int[] ) lineNumbers.clone();
    }
}
