import java.awt.*;
import java.awt.datatransfer.*;
import java.awt.dnd.*;
import java.util.*;
import java.util.List;
import javax.swing.*;
import javax.swing.tree.*;
public class TreeDragAndDrop {
    private JScrollPane getContent() {
        JTree tree = new JTree();
        tree.setDragEnabled ( true );
        tree.setDropMode ( DropMode.ON_OR_INSERT );
        tree.setTransferHandler ( new TreeTransferHandler() );
        tree.getSelectionModel().setSelectionMode (
            TreeSelectionModel.CONTIGUOUS_TREE_SELECTION );
        expandTree ( tree );
        return new JScrollPane ( tree );
    }
    private void expandTree ( JTree tree ) {
        DefaultMutableTreeNode root =
            ( DefaultMutableTreeNode ) tree.getModel().getRoot();
        Enumeration e = root.breadthFirstEnumeration();
        while ( e.hasMoreElements() ) {
            DefaultMutableTreeNode node =
                ( DefaultMutableTreeNode ) e.nextElement();
            if ( node.isLeaf() ) {
                continue;
            }
            int row = tree.getRowForPath ( new TreePath ( node.getPath() ) );
            tree.expandRow ( row );
        }
    }
    public static void main ( String[] args ) {
        JFrame f = new JFrame();
        f.setDefaultCloseOperation ( JFrame.EXIT_ON_CLOSE );
        f.add ( new TreeDragAndDrop().getContent() );
        f.setSize ( 400, 400 );
        f.setLocation ( 200, 200 );
        f.setVisible ( true );
    }
}
class TreeTransferHandler extends TransferHandler {
    DataFlavor nodesFlavor;
    DataFlavor[] flavors = new DataFlavor[1];
    DefaultMutableTreeNode[] nodesToRemove;
    public TreeTransferHandler() {
        try {
            String mimeType = DataFlavor.javaJVMLocalObjectMimeType +
                              ";class=\"" +
                              javax.swing.tree.DefaultMutableTreeNode[].class.getName() +
                              "\"";
            nodesFlavor = new DataFlavor ( mimeType );
            flavors[0] = nodesFlavor;
        } catch ( ClassNotFoundException e ) {
            System.out.println ( "ClassNotFound: " + e.getMessage() );
        }
    }
    public boolean canImport ( TransferHandler.TransferSupport support ) {
        if ( !support.isDrop() ) {
            return false;
        }
        support.setShowDropLocation ( true );
        if ( !support.isDataFlavorSupported ( nodesFlavor ) ) {
            return false;
        }
        JTree.DropLocation dl =
            ( JTree.DropLocation ) support.getDropLocation();
        JTree tree = ( JTree ) support.getComponent();
        int dropRow = tree.getRowForPath ( dl.getPath() );
        int[] selRows = tree.getSelectionRows();
        for ( int i = 0; i < selRows.length; i++ ) {
            if ( selRows[i] == dropRow ) {
                return false;
            }
        }
        int action = support.getDropAction();
        if ( action == MOVE ) {
            return haveCompleteNode ( tree );
        }
        TreePath dest = dl.getPath();
        DefaultMutableTreeNode target =
            ( DefaultMutableTreeNode ) dest.getLastPathComponent();
        TreePath path = tree.getPathForRow ( selRows[0] );
        DefaultMutableTreeNode firstNode =
            ( DefaultMutableTreeNode ) path.getLastPathComponent();
        if ( firstNode.getChildCount() > 0 &&
                target.getLevel() < firstNode.getLevel() ) {
            return false;
        }
        return true;
    }
    private boolean haveCompleteNode ( JTree tree ) {
        int[] selRows = tree.getSelectionRows();
        TreePath path = tree.getPathForRow ( selRows[0] );
        DefaultMutableTreeNode first =
            ( DefaultMutableTreeNode ) path.getLastPathComponent();
        int childCount = first.getChildCount();
        if ( childCount > 0 && selRows.length == 1 ) {
            return false;
        }
        for ( int i = 1; i < selRows.length; i++ ) {
            path = tree.getPathForRow ( selRows[i] );
            DefaultMutableTreeNode next =
                ( DefaultMutableTreeNode ) path.getLastPathComponent();
            if ( first.isNodeChild ( next ) ) {
                if ( childCount > selRows.length - 1 ) {
                    return false;
                }
            }
        }
        return true;
    }
    protected Transferable createTransferable ( JComponent c ) {
        JTree tree = ( JTree ) c;
        TreePath[] paths = tree.getSelectionPaths();
        if ( paths != null ) {
            List<DefaultMutableTreeNode> copies =
                new ArrayList<DefaultMutableTreeNode>();
            List<DefaultMutableTreeNode> toRemove =
                new ArrayList<DefaultMutableTreeNode>();
            DefaultMutableTreeNode node =
                ( DefaultMutableTreeNode ) paths[0].getLastPathComponent();
            DefaultMutableTreeNode copy = copy ( node );
            copies.add ( copy );
            toRemove.add ( node );
            for ( int i = 1; i < paths.length; i++ ) {
                DefaultMutableTreeNode next =
                    ( DefaultMutableTreeNode ) paths[i].getLastPathComponent();
                if ( next.getLevel() < node.getLevel() ) {
                    break;
                } else if ( next.getLevel() > node.getLevel() ) {
                    copy.add ( copy ( next ) );
                } else {
                    copies.add ( copy ( next ) );
                    toRemove.add ( next );
                }
            }
            DefaultMutableTreeNode[] nodes =
                copies.toArray ( new DefaultMutableTreeNode[copies.size()] );
            nodesToRemove =
                toRemove.toArray ( new DefaultMutableTreeNode[toRemove.size()] );
            return new NodesTransferable ( nodes );
        }
        return null;
    }
    private DefaultMutableTreeNode copy ( TreeNode node ) {
        return new DefaultMutableTreeNode ( node );
    }
    protected void exportDone ( JComponent source, Transferable data, int action ) {
        if ( ( action & MOVE ) == MOVE ) {
            JTree tree = ( JTree ) source;
            DefaultTreeModel model = ( DefaultTreeModel ) tree.getModel();
            for ( int i = 0; i < nodesToRemove.length; i++ ) {
                model.removeNodeFromParent ( nodesToRemove[i] );
            }
        }
    }
    public int getSourceActions ( JComponent c ) {
        return COPY_OR_MOVE;
    }
    public boolean importData ( TransferHandler.TransferSupport support ) {
        if ( !canImport ( support ) ) {
            return false;
        }
        DefaultMutableTreeNode[] nodes = null;
        try {
            Transferable t = support.getTransferable();
            nodes = ( DefaultMutableTreeNode[] ) t.getTransferData ( nodesFlavor );
        } catch ( UnsupportedFlavorException ufe ) {
            System.out.println ( "UnsupportedFlavor: " + ufe.getMessage() );
        } catch ( java.io.IOException ioe ) {
            System.out.println ( "I/O error: " + ioe.getMessage() );
        }
        JTree.DropLocation dl =
            ( JTree.DropLocation ) support.getDropLocation();
        int childIndex = dl.getChildIndex();
        TreePath dest = dl.getPath();
        DefaultMutableTreeNode parent =
            ( DefaultMutableTreeNode ) dest.getLastPathComponent();
        JTree tree = ( JTree ) support.getComponent();
        DefaultTreeModel model = ( DefaultTreeModel ) tree.getModel();
        int index = childIndex;
        if ( childIndex == -1 ) {
            index = parent.getChildCount();
        }
        for ( int i = 0; i < nodes.length; i++ ) {
            model.insertNodeInto ( nodes[i], parent, index++ );
        }
        return true;
    }
    public String toString() {
        return getClass().getName();
    }
    public class NodesTransferable implements Transferable {
        DefaultMutableTreeNode[] nodes;
        public NodesTransferable ( DefaultMutableTreeNode[] nodes ) {
            this.nodes = nodes;
        }
        public Object getTransferData ( DataFlavor flavor )
        throws UnsupportedFlavorException {
            if ( !isDataFlavorSupported ( flavor ) ) {
                throw new UnsupportedFlavorException ( flavor );
            }
            return nodes;
        }
        public DataFlavor[] getTransferDataFlavors() {
            return flavors;
        }
        public boolean isDataFlavorSupported ( DataFlavor flavor ) {
            return nodesFlavor.equals ( flavor );
        }
    }
}
