package stackoverflow_4615895;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.MessageProducer;
import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueSession;
import javax.jms.Session;
public class Dispatcher {
    private static long QUEUE_WAIT_TIME = 1000;
    private boolean mStop = false;
    private QueueConnectionFactory mFactory;
    private String mSourceQueueName;
    private String[] mConsumerQueueNames;
    public Dispatcher (
        QueueConnectionFactory factory,
        String sourceQueue,
        String[] consumerQueues ) {
        mFactory = factory;
        mSourceQueueName = sourceQueue;
        mConsumerQueueNames = consumerQueues;
    }
    public void start() {
        Thread thread = new Thread ( new Runnable() {
            public void run() {
                Dispatcher.this.run();
            }
        } );
        thread.setName ( "Queue Dispatcher" );
        thread.start();
    }
    public void stop() {
        mStop = true;
    }
    private void run() {
        QueueConnection connection = null;
        MessageProducer producer = null;
        MessageConsumer consumer = null;
        QueueSession session = null;
        try {
            connection = mFactory.createQueueConnection();
            session = connection.createQueueSession ( false, Session.DUPS_OK_ACKNOWLEDGE );
            Queue sourceQueue = session.createQueue ( mSourceQueueName );
            consumer = session.createConsumer ( sourceQueue );
            producer = session.createProducer ( null );
            Queue[] destinationQueues = new Queue[mConsumerQueueNames.length];
            for ( int index = 0; index < mConsumerQueueNames.length; ++index ) {
                destinationQueues[index] = session.createQueue ( mConsumerQueueNames[index] );
            }
            connection.start();
            while ( !mStop ) {
                Message m = consumer.receive ( QUEUE_WAIT_TIME );
                if ( m == null ) {
                    continue;
                }
                for ( Queue q : destinationQueues ) {
                    producer.send ( q, m );
                }
            }
        } catch ( JMSException ex ) {
        } finally {
            if ( producer != null ) {
                try {
                    producer.close();
                } catch ( JMSException ex ) {
                }
            }
            if ( consumer != null ) {
                try {
                    consumer.close();
                } catch ( JMSException ex ) {
                }
            }
            if ( session != null ) {
                try {
                    session.close();
                } catch ( JMSException ex ) {
                }
            }
            if ( connection != null ) {
                try {
                    connection.close();
                } catch ( JMSException ex ) {
                }
            }
        }
    }
}
