public class Thrift {
    private static abstract class ThriftRequest {
        private void go ( final Thrift thrift, final AsyncClient cli ) {
            on ( cli );
            thrift.ret ( cli );
        }
        public abstract void on ( AsyncClient cli );
    }
    private final ConcurrentLinkedQueue<AsyncClient>   instances = new ConcurrentLinkedQueue<AsyncClient>();
    private final ConcurrentLinkedQueue<ThriftRequest> requests  = new ConcurrentLinkedQueue<ThriftRequest>();
    private Executor                                 exe       = null;
    public Thrift (
        final int clients,
        final int clients_per_message_processing_thread,
        final String host,
        final int port ) throws IOException {
        TProtocolFactory proto_fac = new TProtocolFactory() {
            @Override
            public TProtocol getProtocol ( final TTransport trans ) {
                return new TBinaryProtocol ( trans );
            }
        };
        Factory fac = null;
        for ( int i = 0; i < clients; i++ ) {
            if ( fac == null || i % clients_per_message_processing_thread == 0 ) {
                fac = new AsyncClient.Factory (
                    new TAsyncClientManager(),
                    proto_fac );
            }
            instances.add ( fac.getAsyncClient ( new TNonblockingSocket (
                    host,
                    port ) ) );
        }
    }
    public Thrift ( Executor exe,
                    final int clients,
                    final int clients_per_message_processing_thread,
                    final String host,
                    final int port ) throws IOException {
        this ( clients, clients_per_message_processing_thread, host, port );
        this.exe = exe;
    }
    public void
    req ( final ThriftRequest req ) {
        final AsyncClient cli;
        synchronized ( instances ) {
            cli = instances.poll();
        }
        if ( cli != null ) {
            if ( exe != null ) {
                exe.execute ( new Runnable() {
                    @Override
                    public void run() {
                        req.go ( Thrift.this, cli );
                    }
                } );
            } else {
                req.go ( this, cli );
            }
            return;
        }
        requests.add ( req );
    }
    private void ret ( final AsyncClient cli ) {
        final ThriftRequest req;
        synchronized ( requests ) {
            req = requests.poll();
        }
        if ( req != null ) {
            if ( exe != null ) {
                exe.execute ( new Runnable() {
                    @Override
                    public void run() {
                        req.go ( Thrift.this, cli );
                    }
                } );
            } else {
                req.go ( this, cli );
            }
            return;
        }
        instances.add ( cli );
    }
}
