import java.util.concurrent.*;
public class ConcurrentTimer {
    public static void main ( String[] args ) {
        try {
            Runnable action = new Runnable() {
                public void run() {
                    System.out.println ( "Thread Running" );
                }
            };
            time ( 3, action );
        } catch ( InterruptedException e ) {
            e.printStackTrace();
        }
    }
    private ConcurrentTimer() { }
    public static long time ( int concurrency,
                              final Runnable action ) throws InterruptedException {
        Executor executor = Executors.newFixedThreadPool ( concurrency );
        final CountDownLatch ready = new CountDownLatch ( concurrency );
        final CountDownLatch start = new CountDownLatch ( 1 );
        final CountDownLatch done = new CountDownLatch ( concurrency );
        for ( int i = 0; i < concurrency; i++ ) {
            executor.execute ( new Runnable() {
                public void run() {
                    ready.countDown();
                    try {
                        start.await();
                        action.run();
                    } catch ( InterruptedException e ) {
                        Thread.currentThread().interrupt();
                    } finally {
                        done.countDown();
                    }
                }
            } );
        }
        ready.await();
        long startNanos = System.nanoTime();
        start.countDown();
        done.await();
        return System.nanoTime() - startNanos;
    }
}
