import java.awt.image.BufferedImage;
import java.io.File;
import javax.imageio.ImageIO;
public class Convolution {
    public static void main ( String[] args ) throws Exception {
        File inputFile = new File ( "apple.jpg" );
        BufferedImage bufferedImage = ImageIO.read ( inputFile );
        int w = bufferedImage.getWidth();
        int h = bufferedImage.getHeight();
        System.out.println ( "w=" + w + ", h=" + h );
        int[] image = new int[w * h];
        bufferedImage.getRGB ( 0, 0, w, h, image, 0, w );
        for ( int y = 0; y < h; y++ ) {
            for ( int x = 0; x < w; x++ ) {
                int idx = ( y * w ) + x;
                int p = image[idx];
                int r = p & 0x00FF0000 >> 16;
                int g = p & 0x0000FF >> 8;
                int b = p & 0x000000FF;
                image[idx] = ( int ) ( ( r + g + b ) / 3.0 );
            }
        }
        int convolutionSize = 3;
        int[][] convolution = { { 0, -1, 0 }, { -1, 4, -1 }, { 0, -1, 0 } };
        int[] newImage = new int[w * h];
        for ( int y = 1; y + 1 < h; y++ ) {
            for ( int x = 1; x + 1 < w; x++ ) {
                int idx = ( y * w ) + x;
                for ( int cy = 0; cy < convolutionSize; cy++ ) {
                    for ( int cx = 0; cx < convolutionSize; cx++ ) {
                        int cIdx = ( ( ( y - 1 ) + cy ) * w )
                                   + ( ( x - 1 ) + cx );
                        newImage[idx] += convolution[cy][cx] * image[cIdx];
                    }
                }
                if ( newImage[idx] < 0 ) {
                    newImage[idx] = -newImage[idx];
                } else {
                    newImage[idx] = 0;
                }
                if ( newImage[idx] > 0 ) {
                    newImage[idx] = 120 - newImage[idx];
                } else {
                    newImage[idx] = 255;
                }
            }
        }
        for ( int y = 0; y < h; y++ ) {
            for ( int x = 0; x < w; x++ ) {
                int idx = ( y * w ) + x;
                int p = newImage[idx];
                newImage[idx] = 0xFF000000 | ( p << 16 ) | ( p << 8 ) | p;
            }
        }
        bufferedImage.setRGB ( 0, 0, w, h, newImage, 0, w );
        ImageIO.write ( bufferedImage, "png", new File ( "new_apple.png" ) );
    }
}
