import java.awt.*;
import java.util.*;
import java.awt.event.*;
import javax.swing.*;
public class TextAreaTab extends JFrame {
    public TextAreaTab() {
        Container contentPane = getContentPane();
        contentPane.setLayout ( new BoxLayout ( contentPane, BoxLayout.Y_AXIS ) );
        contentPane.add ( nullTraversalKeys() );
        contentPane.add ( writeYourOwnAction() );
        contentPane.add ( useKeyListener() );
        contentPane.add ( addTraversalKeys() );
    }
    private JComponent nullTraversalKeys() {
        JTextArea textArea = new JTextArea ( 3, 30 );
        textArea.setComponentOrientation ( ComponentOrientation.RIGHT_TO_LEFT );
        textArea.setText ( "Null Traversal Keys\n2\n3\n4\n5\n6\n7\n8\n9" );
        JScrollPane scrollPane = new JScrollPane ( textArea );
        textArea.setFocusTraversalKeys ( KeyboardFocusManager.FORWARD_TRAVERSAL_KEYS, null );
        textArea.setFocusTraversalKeys ( KeyboardFocusManager.BACKWARD_TRAVERSAL_KEYS, null );
        return scrollPane;
    }
    private JComponent writeYourOwnAction() {
        JTextArea textArea = new JTextArea ( 3, 30 );
        textArea.setText ( "Write Your Own Tab Actions\n2\n3\n4\n5\n6\n7\n8\n9" );
        JScrollPane scrollPane = new JScrollPane ( textArea );
        InputMap im = textArea.getInputMap();
        KeyStroke tab = KeyStroke.getKeyStroke ( "TAB" );
        textArea.getActionMap().put ( im.get ( tab ), new TabAction ( true ) );
        KeyStroke shiftTab = KeyStroke.getKeyStroke ( "shift TAB" );
        im.put ( shiftTab, shiftTab );
        textArea.getActionMap().put ( im.get ( shiftTab ), new TabAction ( false ) );
        return scrollPane;
    }
    private JComponent useKeyListener() {
        JTextArea textArea = new JTextArea ( 3, 30 );
        textArea.setText ( "Use Key Listener\n2\n3\n4\n5\n6\n7\n8\n9" );
        JScrollPane scrollPane = new JScrollPane ( textArea );
        scrollPane.getVerticalScrollBar().setFocusable ( false );
        textArea.addKeyListener ( new KeyAdapter() {
            public void keyPressed ( KeyEvent e ) {
                if ( e.getKeyCode() == KeyEvent.VK_TAB ) {
                    e.consume();
                    KeyboardFocusManager.
                    getCurrentKeyboardFocusManager().focusNextComponent();
                }
                if ( e.getKeyCode() == KeyEvent.VK_TAB
                        &&  e.isShiftDown() ) {
                    e.consume();
                    KeyboardFocusManager.
                    getCurrentKeyboardFocusManager().focusPreviousComponent();
                }
            }
        } );
        return scrollPane;
    }
    private JComponent addTraversalKeys() {
        JTextArea textArea = new JTextArea ( 3, 30 );
        textArea.setText ( "Add Traversal Keys\n2\n3\n4\n5\n6\n7\n8\n9" );
        JScrollPane scrollPane = new JScrollPane ( textArea );
        scrollPane.getVerticalScrollBar().setFocusable ( false );
        Set set = new HashSet ( textArea.getFocusTraversalKeys (
                                    KeyboardFocusManager.FORWARD_TRAVERSAL_KEYS ) );
        set.add ( KeyStroke.getKeyStroke ( "TAB" ) );
        textArea.setFocusTraversalKeys (
            KeyboardFocusManager.FORWARD_TRAVERSAL_KEYS, set );
        set = new HashSet ( textArea.getFocusTraversalKeys (
                                KeyboardFocusManager.BACKWARD_TRAVERSAL_KEYS ) );
        set.add ( KeyStroke.getKeyStroke ( "shift TAB" ) );
        textArea.setFocusTraversalKeys (
            KeyboardFocusManager.BACKWARD_TRAVERSAL_KEYS, set );
        return scrollPane;
    }
    class TabAction extends AbstractAction {
        private boolean forward;
        public TabAction ( boolean forward ) {
            this.forward = forward;
        }
        public void actionPerformed ( ActionEvent e ) {
            if ( forward ) {
                tabForward();
            } else {
                tabBackward();
            }
        }
        private void tabForward() {
            final KeyboardFocusManager manager =
                KeyboardFocusManager.getCurrentKeyboardFocusManager();
            manager.focusNextComponent();
            SwingUtilities.invokeLater ( new Runnable() {
                public void run() {
                    if ( manager.getFocusOwner() instanceof JScrollBar ) {
                        manager.focusNextComponent();
                    }
                }
            } );
        }
        private void tabBackward() {
            final KeyboardFocusManager manager =
                KeyboardFocusManager.getCurrentKeyboardFocusManager();
            manager.focusPreviousComponent();
            SwingUtilities.invokeLater ( new Runnable() {
                public void run() {
                    if ( manager.getFocusOwner() instanceof JScrollBar ) {
                        manager.focusPreviousComponent();
                    }
                }
            } );
        }
    }
    public static void main ( String[] args ) {
        TextAreaTab frame = new TextAreaTab();
        frame.setDefaultCloseOperation ( EXIT_ON_CLOSE );
        frame.pack();
        frame.setLocationRelativeTo ( null );
        frame.setVisible ( true );
    }
}
