package de.fencing_game.paul.examples;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;
import javax.swing.*;
public class MethodAdapter implements ActionListener {
    private Object target;
    private Method method;
    private boolean takesArgument;
    public MethodAdapter ( Object o, String mName ) {
        this ( o, findMethod ( o, mName ) );
    }
    public MethodAdapter ( Object o, Method m ) {
        Class<?>[] params = m.getParameterTypes();
        if ( !m.getDeclaringClass().isInstance ( o ) ) {
            throw new IllegalArgumentException ( "wrong target object" );
        }
        if ( params.length > 1 ) {
            throw new IllegalArgumentException ( "too many arguments" );
        }
        if ( params.length == 1 &&
                ! params[0].isAssignableFrom ( ActionEvent.class ) ) {
            throw new IllegalArgumentException ( "method not compatible: " + m );
        }
        this.target = o;
        this.method = m;
        this.takesArgument = params.length > 0;
    }
    private static Method findMethod ( Object o, String mName ) {
        Class<?> c = o.getClass();
        Class<?> eventClass = ActionEvent.class;
        while ( eventClass != null ) {
            try {
                return c.getMethod ( mName, ActionEvent.class );
            } catch ( NoSuchMethodException ex ) {}
            eventClass = eventClass.getSuperclass();
        }
        try {
            return c.getMethod ( mName );
        } catch ( NoSuchMethodException ex ) {
            throw new IllegalArgumentException ( "No fitting method named '" +
                                                 mName + "' on this object " + o +
                                                 " of class " + c.getName() );
        }
    }
    public void actionPerformed ( ActionEvent event ) {
        try {
            if ( takesArgument ) {
                method.invoke ( target, event );
            } else {
                method.invoke ( target );
            }
        } catch ( Exception e ) {
            if ( e instanceof InvocationTargetException ) {
                Throwable t = e.getCause();
                if ( t instanceof Error ) {
                    throw ( Error ) t;
                }
                e = ( Exception ) t;
            }
            if ( e instanceof RuntimeException ) {
                throw ( RuntimeException ) e;
            }
            throw new RuntimeException ( e );
        }
    }
    public static void main ( String[] params ) {
        JFrame f = new JFrame ( "Test" );
        JButton b = new JButton ( "close" );
        b.addActionListener ( new MethodAdapter ( f, "dispose" ) );
        f.getContentPane().add ( b );
        f.setVisible ( true );
    }
}
