@Service
public final class DataPushServiceImpl implements DataPushService {
    private static final Logger LOG = Logger.getLogger ( DataPushServiceImpl.class );
    private boolean doPushViaTransaction ( final AbstractDto dto ) throws DsPushException {
        if ( LOG.isDebugEnabled() ) {
            LOG.debug ( "Sending message through DataServiceTransaction (see userId field) : " + dto.toString() );
        }
        DataServiceTransaction dtx = null;
        boolean isOwnerOfTx = false;
        boolean isSent = false;
        try {
            dtx = DataServiceTransaction.getCurrentDataServiceTransaction();
            if ( dtx == null ) {
                isOwnerOfTx = true;
                dtx = DataServiceTransaction.begin ( "_messageBroker", false );
            }
            isSent = this.doTransactionSend ( dto, dtx );
        } catch ( Exception e ) {
            LOG.error ( "Could not send the creation to LCDS", e );
            if ( isOwnerOfTx ) {
                dtx.rollback();
            }
        } finally {
            try {
                if ( isOwnerOfTx && ( dtx != null ) ) {
                    dtx.commit();
                }
            } catch ( Exception e ) {
                LOG.error ( "Could not send the creation to LCDS (@commit of the DataServiceTransaction)", e );
            }
        }
        return isSent;
    }
    private boolean doTransactionSend ( final AbstractDto dto, final DataServiceTransaction dtx ) {
        boolean isSent = false;
        if ( dto == null ) {
            LOG.error ( "The given DTO is null! Nothing happens" );
        } else {
            try {
                dtx.createItem ( FlexUtils.DESTINATION_NAME__POC_DS, dto );
                isSent = true;
            } catch ( Exception e ) {
                LOG.error ( "Could not send the creation to LCDS for DTO " + dto.toString(), e );
            } finally {
                if ( LOG.isDebugEnabled() ) {
                    LOG.debug ( "DTO : " + dto.toString() + "\n sent : " + String.valueOf ( isSent ) );
                }
            }
        }
        return isSent;
    }
    @Transactional ( rollbackFor = DsPushException.class )
    public boolean pushNewDTo ( final AbstractDto dto, final Long subscriberId ) throws DsPushException {
        return this.doPushViaTransaction ( dto );
    }
}
