public class CustomHttpServletRequestWrapper extends HttpServletRequestWrapper {
    private static final Logger logger = Logger.getLogger ( CustomHttpServletRequestWrapper.class );
    private final String body;
    public CustomHttpServletRequestWrapper ( HttpServletRequest request ) {
        super ( request );
        StringBuilder stringBuilder = new StringBuilder();
        BufferedReader bufferedReader = null;
        try {
            InputStream inputStream = request.getInputStream();
            if ( inputStream != null ) {
                bufferedReader = new BufferedReader ( new InputStreamReader ( inputStream ) );
                String line = "";
                while ( ( line = bufferedReader.readLine() ) != null ) {
                    stringBuilder.append ( line ).append ( "\n" );
                }
            } else {
                stringBuilder.append ( "" );
            }
        } catch ( IOException ex ) {
            logger.error ( "Error reading the request body..." );
        } finally {
            if ( bufferedReader != null ) {
                try {
                    bufferedReader.close();
                } catch ( IOException ex ) {
                    logger.error ( "Error closing bufferedReader..." );
                }
            }
        }
        body = stringBuilder.toString();
    }
    @Override
    public ServletInputStream getInputStream() throws IOException {
        final StringReader reader = new StringReader ( body );
        ServletInputStream inputStream = new ServletInputStream() {
            public int read() throws IOException {
                return reader.read();
            }
        };
        return inputStream;
    }
}
