import com.sun.jna.*;
import com.sun.jna.ptr.PointerByReference;
import java.sql.Connection;
public class JNATest {
    public interface JLibspotify extends Library {
        int sp_session_create ( sp_session_config config, PointerByReference sess );
        int sp_session_login ( sp_session session, String username, String password );
    }
    public static class sp_session extends PointerType {
        public sp_session ( Pointer address ) {
            super ( address );
        }
        public sp_session() {
            super();
        }
    }
    public static class sp_session_config extends Structure {
        public int api_version = 7;
        public String cache_location = ".";
        public String settings_location = ".";
        public Pointer application_key;
        public int application_key_size;
        public String user_agent = "jspotify";
        public sp_session_callbacks callbacks;
        public Pointer userdata;
        public boolean compress_playlists;
        public boolean dont_save_metadata_for_playlists;
        public boolean initially_unload_playlists;
    }
    public interface LoggedIn extends Callback {
        public void logged_in ( sp_session session, int error );
    }
    public interface LoggedOut extends Callback {
        public void logged_out ( sp_session session, int error );
    }
    public interface ConnectionError extends Callback {
        public void connection_error ( sp_session session, int error );
    }
    public static class sp_session_callbacks extends Structure implements Structure.ByReference {
        public LoggedIn logged_in;
        public LoggedOut logged_out;
        public Callback metadata_updated;
        public ConnectionError connection_error;
        public Callback message_to_user;
        public Callback notify_main_thread;
        public Callback music_delivery;
        public Callback play_token_lost;
        public Callback log_message;
        public Callback end_of_track;
        public Callback streaming_error;
        public Callback userinfo_updated;
        public Callback start_playback;
        public Callback stop_playback;
        public Callback get_audio_buffer_stats;
    }
    private static final char[] APP_KEY = { ;
    public static void main ( String[] args ) throws InterruptedException {
        JLibspotify lib = ( JLibspotify ) Native.loadLibrary ( "spotify", JLibspotify.class );
        sp_session_config cfg = new sp_session_config();
        Pointer ptr = new Memory ( APP_KEY.length );
        ptr.write ( 0, toBytes ( APP_KEY ), 0, APP_KEY.length );
        cfg.application_key = ptr;
        cfg.application_key_size = APP_KEY.length;
        sp_session_callbacks sessCallbacks = new sp_session_callbacks();
        LoggedIn loggedInCallback = new LoggedIn() {
            public void logged_in ( sp_session session, int error ) {
                System.out.println ( "logged_in() called" );
            }
        };
        ConnectionError connectionErrorCallback = new ConnectionError() {
            public void connection_error ( sp_session session, int error ) {
                System.out.println ( "connection_error() called" );
            }
        };
        LoggedOut loggedOutCallback = new LoggedOut() {
            public void logged_out ( sp_session session, int error ) {
                System.out.println ( "logged_out() called" );
            }
        };
        sessCallbacks.logged_in = loggedInCallback;
        sessCallbacks.connection_error = connectionErrorCallback;
        sessCallbacks.logged_out = loggedOutCallback;
        sessCallbacks.log_message = new Callback() {
            public void callback ( sp_session session, String message ) {
                System.out.println ( "log_message() called:" + message );
            }
        };
        cfg.callbacks = sessCallbacks;
        PointerByReference sessionPbr = new PointerByReference();
        int error_id = lib.sp_session_create ( cfg, sessionPbr );
        System.out.println ( "sp_session_create returned " + error_id );
        sp_session mySession = new sp_session ( sessionPbr.getValue() );
        error_id = lib.sp_session_login ( mySession, "foo", "bar" );
        System.out.println ( "sp_session_login returned " + error_id );
        Thread.sleep ( 1000 );
    }
    public static byte[] toBytes ( char[] key ) {
        byte[] b = new byte[key.length];
        for ( int i = 0; i < key.length; i++ ) {
            if ( key[i] > 127 ) {
                b[i] = ( byte ) ( key[i] - 256 );
            } else {
                b[i] = ( byte ) key[i];
            }
        }
        return b;
    }
                                          }
