package stackoverflow.fight;
public class Fight {
    private int roundCount = 1;
    private final Opponent opponentOne;
    private final Opponent opponentTwo;
    public Fight ( Opponent opponentOne, Opponent opponentTwo ) {
        super();
        this.opponentOne = opponentOne;
        this.opponentTwo = opponentTwo;
    }
    public boolean isOngoing() {
        return opponentOne.isAlive() && opponentTwo.isAlive();
    }
    private void attack ( Opponent a, Opponent b ) {
        a.attack ( b );
    }
    private void roundBasedSuccess() {
        if ( opponentOne.isAlive() && roundCount == 2 ) {
            System.out
            .print ( "\tThis is encouraging. Goodman has lasted past round 2!" );
            return;
        }
        if ( opponentOne.isAlive() && roundCount % 3 == 0 ) {
            System.out
            .print ( "\tGoodman is as strong as Samson. He has lasted round "
                     + roundCount + " and still looks strong." );
            opponentOne.recover ( 10 );
        }
    }
    public void fight() {
        roundBasedSuccess();
        switch ( Double.valueOf ( Math.random() * 2 ).intValue() ) {
        case 0:
            System.out
            .println ( "\t" + opponentOne.getName() + " strikes first." );
            attack ( opponentOne, opponentTwo );
            attack ( opponentTwo, opponentOne );
            break;
        case 1:
            System.out.println ( "\t " + opponentTwo.getName()
                                 + " strikes first." );
            attack ( opponentTwo, opponentOne );
            attack ( opponentOne, opponentTwo );
            break;
        }
        if ( opponentOne.isAlive() ) {
            System.out.println ( "\t" + opponentOne.getName() + " has "
                                 + opponentOne.getCurrentHitpoints()
                                 + " points left. Not bad, Man! " );
        }
        if ( opponentTwo.isAlive() ) {
            System.out.println ( "\tWait, " + opponentTwo.getName()
                                 + " has a total of " + opponentTwo.getCurrentHitpoints()
                                 + " points and is still in the game" );
        }
        System.out.println ( "This is the end of round " + roundCount++ + " " );
    }
    public void rest() {
        System.out.println ( "r: Opponents should rest (regaining 2 hitpoints" );
        opponentOne.recover ( 2 );
        opponentTwo.recover ( 2 );
    }
    public void status() {
        System.out.println ( "\tChecking the status of the game: Round "
                             + getRoundCount() + ": " + opponentOne.getName() + " has "
                             + opponentOne.getCurrentHitpoints() + " points and "
                             + opponentTwo.getName() + " has "
                             + opponentTwo.getCurrentHitpoints() + " points\n" );
    }
    public int getRoundCount() {
        return roundCount;
    }
    public String getTitle() {
        return opponentOne.getName() + " vs. " + opponentTwo.getName()
               + " - Round " + roundCount + ": ";
    }
    public void end() {
        if ( isOngoing() ) {
            return;
        }
        if ( !opponentOne.isAlive() ) {
            System.out.println ( "\t" + opponentOne.getName()
                                 + "  you are out of the game! " + opponentTwo.getName()
                                 + " will take over the village. This is sad!" );
            System.out.println ( "Game Over!" );
        } else if ( !opponentTwo.isAlive() ) {
            System.out.println ( "\t" + opponentOne.getName()
                                 + " has been victorious" );
            System.out.println ( "\t" + opponentTwo.getName()
                                 + " is dead. The people live!!!!" );
            System.out.println ( "Game Over!" );
        }
    }
    public void buy ( Opponent opponent, Weapon weapon ) {
        if ( opponent.isMonster() ) {
            System.out.println ( "\tMonsters cannot buy weapons!" );
            return;
        }
        if ( opponent.getCurrentMoney() < weapon.getCost() ) {
            System.out.println ( "\t" + opponent.getName() + " has only " + opponent.getCurrentMoney() + " left. " + weapon.getName() + " costs " + weapon.getCost() + "!" );
            return;
        }
        opponent.setWeapon ( weapon );
    }
}
