package com.example;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Pattern;
import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletResponse;
public final class RegexFilter implements Filter {
    private List<Pattern> searchPatterns;
    private List<String> replaceStrings;
    public void init ( FilterConfig filterConfig ) {
        Map<String, String> patternMap = new HashMap<String, String>();
        Enumeration<String> names = ( Enumeration<String> ) filterConfig.getInitParameterNames();
        while ( names.hasMoreElements() ) {
            String name = names.nextElement();
            if ( name.startsWith ( "search" ) ) {
                patternMap.put ( name.substring ( 6 ), filterConfig.getInitParameter ( name ) );
            }
        }
        this.searchPatterns = new ArrayList<Pattern> ( patternMap.size() );
        this.replaceStrings = new ArrayList<String> ( patternMap.size() );
        names = ( Enumeration<String> ) filterConfig.getInitParameterNames();
        while ( names.hasMoreElements() ) {
            String name = names.nextElement();
            if ( name.startsWith ( "replace" ) ) {
                String searchString = patternMap.get ( name.substring ( 7 ) );
                if ( searchString != null ) {
                    this.searchPatterns.add ( Pattern.compile ( searchString ) );
                    this.replaceStrings.add ( filterConfig.getInitParameter ( name ) );
                }
            }
        }
    }
    public void doFilter ( ServletRequest request, ServletResponse response, FilterChain chain ) throws IOException, ServletException {
        PrintWriter out = response.getWriter();
        CharResponseWrapper wrapper = new CharResponseWrapper ( ( HttpServletResponse ) response );
        chain.doFilter ( request, wrapper );
        String modifiedHtml = wrapper.toString();
        for ( int i = 0; i < this.searchPatterns.size(); i++ ) {
            modifiedHtml = this.searchPatterns.get ( i ).matcher ( modifiedHtml ).replaceAll ( this.replaceStrings.get ( i ) );
        }
        response.setContentLength ( modifiedHtml.getBytes().length );
        out.write ( modifiedHtml );
        out.close();
    }
    public void destroy() {
        this.searchPatterns = null;
        this.replaceStrings = null;
    }
}
