public abstract class LazyInit<T> {
    private volatile T field;
    @edu.umd.cs.findbugs.annotations.SuppressWarnings ( "UG_SYNC_SET_UNSYNC_GET" )
    public T get () {
        T result = field;
        if ( result == null ) {
            synchronized ( this ) {
                result = field;
                if ( result == null ) {
                    field = result = computeValue ();
                }
            }
        }
        return result;
    }
    protected abstract T computeValue ();
    public synchronized void set ( T value ) {
        field = value;
    }
    public boolean hasValue() {
        return field != null;
    }
}
