public void doRecordingDemo() {
    DataSource[] sources = new DataSource[2];
    sources[0] = Manager.createDataSource ( audioDevice.getLocator() );
    sources[1] = Manager.createDataSource ( videoDevice.getLocator() );
    DataSource source = Manager.createMergingDataSource ( sources );
    Processor processor = Manager.createProcessor ( source );
    processor.configure();
    waitForState ( processor, Processor.Configured );
    processor.setContentDescriptor ( new FileTypeDescriptor ( FileTypeDescriptor.QUICKTIME ) );
    for ( TrackControl control : processor.getTrackControls() ) {
        if ( control.getFormat() instanceof VideoFormat || control.getFormat() instanceof AudioFormat ) {
            if ( control.getFormat() instanceof AudioFormat ) {
                control.setFormat ( new AudioFormat ( AudioFormat.GSM ) );
            }
            if ( control.getFormat() instanceof VideoFormat ) {
                VideoFormat desiredVideoFormat = null;
                Dimension targetDimension = new Dimension ( 352, 288 );
                VideoFormat[] desiredFormats = new VideoFormat[] {new H263Format(), new JPEGFormat(), new RGBFormat(), new YUVFormat() };
                for ( VideoFormat checkFormat : desiredFormats ) {
                    List<VideoFormat> candidates = new LinkedList<VideoFormat>();
                    for ( Format format : control.getSupportedFormats() ) {
                        if ( format.isSameEncoding ( checkFormat ) ) {
                            candidates.add ( ( VideoFormat ) format );
                        }
                    }
                    if ( !candidates.isEmpty() ) {
                        desiredVideoFormat = candidates.get ( 0 );
                        for ( VideoFormat format : candidates ) {
                            if ( targetDimension.equals ( format.getSize() ) ) {
                                desiredVideoFormat = format;
                                break;
                            }
                        }
                    }
                    if ( desiredVideoFormat != null ) {
                        break;
                    }
                }
                if ( desiredVideoFormat != null ) {
                    control.setFormat ( desiredVideoFormat );
                }
            }
            control.setEnabled ( true );
            System.out.println ( "Enabled track: " + control + " (" + control.getFormat() + ")" );
        }
    }
    processor.realize();
    waitForState ( processor, Processor.Realized );
    DataSource dataOutput = processor.getDataOutput();
    File answerFile = new File ( "recording.mov" );
    MediaLocator dest = new MediaLocator ( answerFile.toURI().toURL() );
    DataSink answerSink = Manager.createDataSink ( dataOutput, dest );
    processor.start();
    answerSink.open();
    answerSink.start();
    Thread.sleep ( 10000 );
    try {
        processor.stop();
        processor.close();
        try {
            Thread.sleep ( 1000 );
        } catch ( InterruptedException ex ) {
            Logger.getLogger ( getClass().getName() ).log ( Level.SEVERE, null, ex );
        }
        try {
            answerSink.stop();
        } catch ( IOException ex ) {
            Logger.getLogger ( getClass().getName() ).log ( Level.SEVERE, null, ex );
        }
    } finally {
        try {
            answerSink.close();
        } catch ( Exception ex ) {
            Logger.getLogger ( getClass().getName() ).log ( Level.SEVERE, null, ex );
        }
        try {
            processor.deallocate();
        } catch ( Exception ex ) {
            Logger.getLogger ( getClass().getName() ).log ( Level.SEVERE, null, ex );
        }
    }
}
private void waitForState ( Player player, int state ) {
    if ( player.getState() == state ) {
        return;
    }
    long startTime = new Date().getTime();
    long timeout = 10 * 1000;
    final Object waitListener = new Object();
    ControllerListener cl = new ControllerListener() {
        @Override
        public void controllerUpdate ( ControllerEvent ce ) {
            synchronized ( waitListener ) {
                waitListener.notifyAll();
            }
        }
    };
    try {
        player.addControllerListener ( cl );
        synchronized ( waitListener ) {
            while ( player.getState() != state && new Date().getTime() - startTime < timeout ) {
                try {
                    waitListener.wait ( 500 );
                } catch ( InterruptedException ex ) {
                    Logger.getLogger ( getClass().getName() ).log ( Level.SEVERE, null, ex );
                }
            }
        }
    } finally {
        player.removeControllerListener ( cl );
    }
}
