import java.awt.*;
import javax.swing.*;
import javax.swing.event.*;
public class SplitPaneDemo2 extends JFrame
    implements ListSelectionListener {
    private JLabel label;
    public SplitPaneDemo2() {
        super ( "SplitPaneDemo2" );
        SplitPaneDemo splitPaneDemo = new SplitPaneDemo();
        JSplitPane top = splitPaneDemo.getSplitPane();
        splitPaneDemo.getImageList().addListSelectionListener ( this );
        top.setBorder ( null );
        label = new JLabel ( "Click on an image name in the list.",
                             JLabel.CENTER );
        JSplitPane splitPane = new JSplitPane ( JSplitPane.VERTICAL_SPLIT,
                                                top, label );
        splitPane.setOneTouchExpandable ( true );
        splitPane.setDividerLocation ( 180 );
        top.setMinimumSize ( new Dimension ( 100, 50 ) );
        label.setMinimumSize ( new Dimension ( 100, 30 ) );
        getContentPane().add ( splitPane );
    }
    public void valueChanged ( ListSelectionEvent e ) {
        if ( e.getValueIsAdjusting() ) {
            return;
        }
        JList theList = ( JList ) e.getSource();
        if ( theList.isSelectionEmpty() ) {
            label.setText ( "Nothing selected." );
        } else {
            int index = theList.getSelectedIndex();
            label.setText ( "Selected image number " + index );
        }
    }
    private static void createAndShowGUI() {
        JFrame frame = new SplitPaneDemo2();
        frame.setDefaultCloseOperation ( JFrame.EXIT_ON_CLOSE );
        frame.pack();
        frame.setVisible ( true );
    }
    public static void main ( String[] args ) {
        javax.swing.SwingUtilities.invokeLater ( new Runnable() {
            public void run() {
                createAndShowGUI();
            }
        } );
    }
}
class SplitPaneDemo extends JPanel
    implements ListSelectionListener {
    private JLabel picture;
    private JList list;
    private JSplitPane splitPane;
    private String[] imageNames = {"Bird", "Cat", "Dog", "Rabbit", "Pig", "dukeWaveRed",
                                   "kathyCosmo", "lainesTongue", "left", "middle", "right", "stickerface"
                                  };
    public SplitPaneDemo() {
        list = new JList ( imageNames );
        list.setSelectionMode ( ListSelectionModel.SINGLE_SELECTION );
        list.setSelectedIndex ( 0 );
        list.addListSelectionListener ( this );
        JScrollPane listScrollPane = new JScrollPane ( list );
        picture = new JLabel();
        picture.setFont ( picture.getFont().deriveFont ( Font.ITALIC ) );
        picture.setHorizontalAlignment ( JLabel.CENTER );
        JScrollPane pictureScrollPane = new JScrollPane ( picture );
        splitPane = new JSplitPane ( JSplitPane.HORIZONTAL_SPLIT,
                                     listScrollPane, pictureScrollPane );
        splitPane.setOneTouchExpandable ( true );
        splitPane.setDividerLocation ( 150 );
        Dimension minimumSize = new Dimension ( 100, 50 );
        listScrollPane.setMinimumSize ( minimumSize );
        pictureScrollPane.setMinimumSize ( minimumSize );
        splitPane.setPreferredSize ( new Dimension ( 400, 200 ) );
        updateLabel ( imageNames[list.getSelectedIndex()] );
    }
    public void valueChanged ( ListSelectionEvent e ) {
        JList list = ( JList ) e.getSource();
        updateLabel ( imageNames[list.getSelectedIndex()] );
    }
    protected void updateLabel ( String name ) {
        ImageIcon icon = createImageIcon ( "images/" + name + ".gif" );
        picture.setIcon ( icon );
        if ( icon != null ) {
            picture.setText ( null );
        } else {
            picture.setText ( "Image not found" );
        }
    }
    public JList getImageList() {
        return list;
    }
    public JSplitPane getSplitPane() {
        return splitPane;
    }
    protected static ImageIcon createImageIcon ( String path ) {
        java.net.URL imgURL = SplitPaneDemo.class.getResource ( path );
        if ( imgURL != null ) {
            return new ImageIcon ( imgURL );
        } else {
            System.err.println ( "Couldn't find file: " + path );
            return null;
        }
    }
    private static void createAndShowGUI() {
        JFrame frame = new JFrame ( "SplitPaneDemo" );
        frame.setDefaultCloseOperation ( JFrame.EXIT_ON_CLOSE );
        SplitPaneDemo splitPaneDemo = new SplitPaneDemo();
        frame.getContentPane().add ( splitPaneDemo.getSplitPane() );
        frame.pack();
        frame.setVisible ( true );
    }
    public static void main ( String[] args ) {
        javax.swing.SwingUtilities.invokeLater ( new Runnable() {
            public void run() {
                createAndShowGUI();
            }
        } );
    }
}
