public static <T> Iterator<T> concatenate ( final Iterator<T> ... sources ) {
    if ( sources.length == 0 ) {
        return new Iterator<T>() {
            @Override public boolean hasNext() {
                return false;
            }
            @Override public T next() {
                throw new NoSuchElementException ( "end of iteration" );
            }
            @Override public void remove() {
                throw new IllegalStateException ( "no previous element" );
            }
        };
    }
    return new Iterator<T>() {
        Iterator<Iterator<T>> sourcesIterator = Arrays.asList ( sources ).iterator();
        Iterator<T> currentIterator = sourcesIterator.next();
        @Override
        public boolean hasNext() {
            if ( currentIterator.hasNext() ) {
                return true;
            } else {
                if ( sourcesIterator.hasNext() ) {
                    currentIterator = sourcesIterator.next();
                    return hasNext();
                } else {
                    return false;
                }
            }
        }
        @Override
        public T next() {
            if ( hasNext() ) {
                return currentIterator.next();
            } else {
                throw new NoSuchElementException ( "end of iteration" );
            }
        }
        @Override
        public void remove() {
            currentIterator.remove();
        }
    };
}
@SuppressWarnings ( "unchecked" )
public static <T> Iterable<T> concatenate ( final Iterable<T> ... sources ) {
    return new Iterable<T>() {
        @Override
        public Iterator<T> iterator() {
            final Iterator[] iteratorsArrays = new Iterator[sources.length];
            for ( int i = 0; i < sources.length; i++ ) {
                iteratorsArrays[i] = sources[i].iterator();
            }
            return concatenate ( iteratorsArrays );
        }
    };
}
