import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
public class TimeBoxedWriter extends Writer {
    private static DateFormat FORMAT = new SimpleDateFormat ( "yyyyDDDHHmm" );
    private static final int TIME_BOX = 120000;
    public static void main ( String[] args ) throws IOException {
        Writer w = new TimeBoxedWriter ( new File ( "." ), "test" );
        for ( int i = 0; i < 500; i++ ) {
            w.write ( "testing " + i + "\n" );
            try {
                Thread.sleep ( 1000 );
            } catch ( InterruptedException ie ) {}
        }
        w.close();
    }
    private File dir_;
    private long stamp_ = 0;
    private String stem_;
    private Writer writer_ = null;
    public TimeBoxedWriter ( File dir, String stem ) {
        dir_ = dir;
        stem_ = stem;
    }
    @Override
    public void close() throws IOException {
        synchronized ( lock ) {
            if ( writer_ != null ) {
                writer_.close();
                writer_ = null;
            }
        }
    }
    @Override
    public void flush() throws IOException {
        synchronized ( lock ) {
            if ( writer_ != null ) {
                writer_.flush();
            }
        }
    }
    private void rollover() throws IOException {
        synchronized ( lock ) {
            long now = System.currentTimeMillis();
            if ( ( stamp_ + TIME_BOX ) < now ) {
                if ( writer_ != null ) {
                    writer_.flush();
                    writer_.close();
                }
                stamp_ = TIME_BOX * ( System.currentTimeMillis() / TIME_BOX );
                String time = FORMAT.format ( new Date ( stamp_ ) );
                writer_ = new FileWriter ( new File ( dir_, stem_ + "." + time
                                                      + ".txt" ) );
            }
        }
    }
    @Override
    public void write ( char[] cbuf, int off, int len ) throws IOException {
        synchronized ( lock ) {
            rollover();
            writer_.write ( cbuf, off, len );
        }
    }
}
