package com.rorlig;
import java.io.IOException;
import java.util.StringTokenizer;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.IntWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.mapreduce.Job;
import org.apache.hadoop.mapreduce.Mapper;
import org.apache.hadoop.mapreduce.Reducer;
import org.apache.hadoop.mapreduce.lib.input.FileInputFormat;
import org.apache.hadoop.mapreduce.lib.output.FileOutputFormat;
public class WordCountJob {
    public static class TokenizerMapper
        extends Mapper<Object, Text, Text, IntWritable> {
        private final static IntWritable one = new IntWritable ( 1 );
        private Text word = new Text();
        public void map ( Object key, Text value, Context context
                        ) throws IOException, InterruptedException {
            StringTokenizer itr = new StringTokenizer ( value.toString() );
            while ( itr.hasMoreTokens() ) {
                word.set ( itr.nextToken() );
                context.write ( word, one );
            }
        }
    }
    public static class IntSumReducer
        extends Reducer<Text, IntWritable, Text, IntWritable> {
        private IntWritable result = new IntWritable();
        public void reduce ( Text key, Iterable<IntWritable> values,
                             Context context
                           ) throws IOException, InterruptedException {
            int sum = 0;
            for ( IntWritable val : values ) {
                sum += val.get();
            }
            result.set ( sum );
            context.write ( key, result );
        }
    }
    public static void main ( String[] args ) throws Exception {
        Configuration conf = new Configuration();
        if ( args.length != 3 ) {
            System.err.println ( "Usage: wordcount <in> <out> <iterations>" );
            System.exit ( 2 );
        }
        int iterations = new Integer ( args[2] );
        Path inPath = new Path ( args[0] );
        Path outPath =  null;
        for ( int i = 0; i < iterations; ++i ) {
            outPath = new Path ( args[1] + i );
            Job job = new Job ( conf, "word count" );
            job.setJarByClass ( WordCountJob.class );
            job.setMapperClass ( TokenizerMapper.class );
            job.setCombinerClass ( IntSumReducer.class );
            job.setReducerClass ( IntSumReducer.class );
            job.setOutputKeyClass ( Text.class );
            job.setOutputValueClass ( IntWritable.class );
            FileInputFormat.addInputPath ( job, inPath );
            FileOutputFormat.setOutputPath ( job, outPath );
            job.waitForCompletion ( true );
            inPath = outPath;
        }
    }
}
