import java.util.*;
public class myListGraph {
    protected String[] names;
    protected StringLinkList[] Edges;
    protected int numVertices;
    protected int numEdges;
    public myListGraph( ) {
        this ( 1 );
    }
    public myListGraph ( int capacity ) {
        names = new String[capacity];
        Edges = new StringLinkList[capacity];
        for ( int i = 0 ; i < capacity ; i ++ ) {
            Edges[i] = new StringLinkList();
        }
    }
    public int numberOfVertices() {
        return numVertices;
    }
    public int numberOfEdges() {
        return numEdges;
    }
    public int getIndex ( String vertex ) {
        for ( int i = 0; i < numVertices; i++ )
            if ( vertex.equals ( names[i] ) ) {
                return i;
            }
        return -1;
    }
    protected String[] resize ( String[] array, int newSize ) {
        String[] temp = new String[newSize];
        int smallerSize = newSize;
        if ( array.length < smallerSize ) {
            smallerSize = array.length;
        }
        for ( int i = 0; i < smallerSize; i++ ) {
            temp[i] = array[i];
        }
        return temp;
    }
    protected StringLinkList[] resize ( StringLinkList[] array, int newSize ) {
        StringLinkList[] temp = new StringLinkList[newSize];
        int smallerSize = newSize;
        if ( array.length < smallerSize ) {
            smallerSize = array.length;
        }
        for ( int i = 0; i < smallerSize; i++ ) {
            temp[i] = array[i];
        }
        for ( int i = smallerSize ; i < temp.length ; i ++ ) {
            temp [i] = new StringLinkList();
        }
        return temp;
    }
    public void addVertex ( String newVertex ) {
        if ( getIndex ( newVertex ) != -1 ) {
            System.out.print ( "addVertex: " );
            System.out.print ( newVertex );
            System.out.println ( " failed -- vertex already exists." );
            return;
        }
        if ( names.length == numVertices ) {
            names = resize ( names, 2 * numVertices + 1 );
            Edges = resize ( Edges, 2 * numVertices + 1 );
        }
        names[numVertices++] = newVertex;
    }
    public void addEdge ( String vertex1, String vertex2 ) {
        int i = getIndex ( vertex1 );
        if ( i == -1 ) {
            System.out.print ( "addEdge failed: " );
            System.out.print ( vertex1 );
            System.out.println ( " does not exist." );
            return;
        }
        int j = getIndex ( vertex2 );
        if ( j == -1 ) {
            System.out.print ( "addEdge failed: " );
            System.out.print ( vertex2 );
            System.out.println ( " does not exist." );
            return;
        }
        Edges[i].insertFirst ( names[j] );
        Edges[j].insertFirst ( names[i] );
        numEdges++;
    }
    public void printAdjacencyList ( String vertex ) {
        int i = getIndex ( vertex );
        if ( i == -1 ) {
            System.out.print ( "addEdge failed: " );
            System.out.print ( vertex );
            System.out.println ( " does not exist." );
            return;
        }
        System.out.print ( vertex + " is connected to " );
        Edges [i].displayList();
    }
    public String[] getNeighbors ( String vertex ) {
        int source = getIndex ( vertex );
        if ( source == -1 ) {
            System.out.print ( "getNeighbors failed: Vertex " );
            System.out.print ( vertex );
            System.out.println ( " does not exist." );
            return null;
        }
        return Edges[source].copyIntoArray();
    }
    public int[] getNeighbors ( int index ) {
        if ( ( index < 0 ) || ( index >= numVertices ) ) {
            System.out.print ( "getNeighbors failed: Index" );
            System.out.print ( index );
            System.out.println ( " is out of bounds." );
            return null;
        }
        String[] nbrNames = getNeighbors ( names[index] );
        int[] nbrIndices = new int[nbrNames.length];
        for ( int i = 0; i < nbrIndices.length; i++ ) {
            nbrIndices[i] = getIndex ( nbrNames[i] );
        }
        return nbrIndices;
    }
    public int degree ( String vertex ) {
        int i = getIndex ( vertex );
        if ( i == -1 ) {
            System.out.print ( "In degree: " );
            System.out.print ( vertex );
            System.out.println ( " does not exist." );
            return -1;
        }
        return degree ( i );
    }
    public int degree ( int index ) {
        return Edges[index].size();
    }
    public boolean isEdge ( String vertex1, String vertex2 ) {
        int i = getIndex ( vertex1 );
        if ( i == -1 ) {
            System.out.print ( "isEdge failed: " );
            System.out.print ( vertex1 );
            System.out.println ( " does not exist." );
            return false;
        }
        int j = getIndex ( vertex2 );
        if ( j == -1 ) {
            System.out.print ( "isEdge failed: " );
            System.out.print ( vertex2 );
            System.out.println ( " does not exist." );
            return false;
        }
        return ( Edges[i].find ( vertex2 ) != null );
    }
    public double clusteringCoefficient ( int i ) {
        String[] temp = Edges[i].copyIntoArray();
        int edgesInNbd = 0;
        for ( int j = 0; j < temp.length; j++ )
            for ( int k = 0; k < j; k++ )
                if ( isEdge ( temp[j], temp[k] ) ) {
                    edgesInNbd++;
                }
        if ( temp.length <= 1 ) {
            return 1;
        } else {
            return edgesInNbd * 2.0 / ( temp.length * ( temp.length - 1 ) );
        }
    }
    public double clusteringCoefficient ( String v ) {
        int i = getIndex ( v );
        if ( i == -1 ) {
            System.out.print ( "clusteringCoefficient failed: " );
            System.out.print ( v );
            System.out.println ( " does not exist." );
            return 0;
        }
        return clusteringCoefficient ( i );
    }
    public double clusteringCoefficient() {
        double sum = 0;
        for ( int i = 0; i < numVertices; i++ ) {
            sum =  sum + clusteringCoefficient ( i );
        }
        return sum / numVertices;
    }
    public boolean isConnected() {
        int[] bfsTree = breadthFirstSearch ( names[0] );
        for ( int i = 0; i < bfsTree.length; i++ )
            if ( bfsTree[i] == -1 ) {
                return false;
            }
        return true;
    }
    public String[][] connectedComponents() {
        String[][] cc = new String[numVertices][];
        boolean[] visited = new boolean[numVertices];
        for ( int i = 0; i < numVertices; i++ ) {
            visited[i] = false;
        }
        int numVisited = 0;
        int numComponents = 0;
        while ( numVisited < numVertices ) {
            int source;
            for ( source = 0; source < numVisited; source++ )
                if ( !visited[source] ) {
                    break;
                }
            int[] bfsTree = breadthFirstSearch ( names[source] );
            int countNewVisited = 0;
            for ( int i = 0; i < numVertices; i++ )
                if ( bfsTree[i] != -1 ) {
                    countNewVisited++;
                }
            cc[numComponents] = new String[countNewVisited];
            countNewVisited = 0;
            for ( int i = 0; i < numVertices; i++ )
                if ( bfsTree[i] != -1 ) {
                    cc[numComponents][countNewVisited++] = names[i];
                    visited[i] = true;
                }
            numVisited = numVisited + countNewVisited;
            numComponents++;
        }
        String[][] newCC = new String[numComponents][];
        for ( int i = 0; i < numComponents; i++ ) {
            newCC[i] = cc[i];
        }
        return newCC;
    }
    public String[] shortestPath ( String source, String dest ) {
        int sourceIndex = getIndex ( source );
        if ( sourceIndex == -1 ) {
            System.out.print ( "breadthFirstSearch failed: " );
            System.out.print ( source );
            System.out.println ( " does not exist." );
            return null;
        }
        int destIndex = getIndex ( dest );
        if ( destIndex == -1 ) {
            System.out.print ( "breadthFirstSearch failed: " );
            System.out.print ( dest );
            System.out.println ( " does not exist." );
            return null;
        }
        int[] bfsTree = breadthFirstSearch ( destIndex );
        if ( bfsTree[sourceIndex] == -1 ) {
            return null;
        }
        String[] path = new String[numVertices];
        path[0] = names[sourceIndex];
        int currentIndex = sourceIndex;
        int pathLength = 0;
        while ( currentIndex != bfsTree[currentIndex] ) {
            currentIndex = bfsTree[currentIndex];
            pathLength++;
            path[pathLength] = names[currentIndex];
        }
        String[] newPath = new String[pathLength + 1];
        for ( int i = 0; i < newPath.length; i++ ) {
            newPath[i] = path[i];
        }
        return newPath;
    }
    public int[] breadthFirstSearch ( String source ) {
        int sourceIndex = getIndex ( source );
        if ( sourceIndex == -1 ) {
            System.out.print ( "breadthFirstSearch failed: " );
            System.out.print ( source );
            System.out.println ( " does not exist." );
            return null;
        }
        return breadthFirstSearch ( sourceIndex );
    }
    public int[] breadthFirstSearch ( int sourceIndex ) {
        int[] bfsTree = new int[numVertices];
        for ( int i = 0; i < numVertices; i++ ) {
            bfsTree[i] = -1;
        }
        bfsTree[sourceIndex] = sourceIndex;
        boolean[] visited = new boolean[numVertices];
        for ( int i = 0; i < numVertices; i++ ) {
            visited[i] = false;
        }
        visited[sourceIndex] = true;
        Queue Q = new Queue ( numVertices );
        Q.enqueue ( sourceIndex );
        while ( !Q.isEmpty() ) {
            int current = Q.dequeue();
            int[] neighbors = getNeighbors ( current );
            for ( int i = 0; i < neighbors.length; i++ ) {
                int currentNeighbor = neighbors[i];
                if ( !visited[currentNeighbor] ) {
                    visited[currentNeighbor] = true;
                    Q.enqueue ( currentNeighbor );
                    bfsTree[currentNeighbor] = current;
                }
            }
        }
        return bfsTree;
    }
}
