import org.apache.poi.hssf.usermodel.*;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.util.CellRangeAddress;
import java.io.FileInputStream;
import java.io.FileOutputStream;
public class RowCopy {
    public static void main ( String[] args ) throws Exception {
        HSSFWorkbook workbook = new HSSFWorkbook ( new FileInputStream ( "c:/input.xls" ) );
        HSSFSheet sheet = workbook.getSheet ( "Sheet1" );
        copyRow ( workbook, sheet, 0, 1 );
        FileOutputStream out = new FileOutputStream ( "c:/output.xls" );
        workbook.write ( out );
        out.close();
    }
    private static void copyRow ( HSSFWorkbook workbook, HSSFSheet worksheet, int sourceRowNum, int destinationRowNum ) {
        HSSFRow newRow = worksheet.getRow ( destinationRowNum );
        HSSFRow sourceRow = worksheet.getRow ( sourceRowNum );
        if ( newRow != null ) {
            worksheet.shiftRows ( destinationRowNum, worksheet.getLastRowNum(), 1 );
        } else {
            newRow = worksheet.createRow ( destinationRowNum );
        }
        for ( int i = 0; i < sourceRow.getLastCellNum(); i++ ) {
            HSSFCell oldCell = sourceRow.getCell ( i );
            HSSFCell newCell = newRow.createCell ( i );
            if ( oldCell == null ) {
                newCell = null;
                continue;
            }
            HSSFCellStyle newCellStyle = workbook.createCellStyle();
            newCellStyle.cloneStyleFrom ( oldCell.getCellStyle() );
            ;
            newCell.setCellStyle ( newCellStyle );
            if ( oldCell.getCellComment() != null ) {
                newCell.setCellComment ( oldCell.getCellComment() );
            }
            if ( oldCell.getHyperlink() != null ) {
                newCell.setHyperlink ( oldCell.getHyperlink() );
            }
            newCell.setCellType ( oldCell.getCellType() );
            switch ( oldCell.getCellType() ) {
            case Cell.CELL_TYPE_BLANK:
                newCell.setCellValue ( oldCell.getStringCellValue() );
                break;
            case Cell.CELL_TYPE_BOOLEAN:
                newCell.setCellValue ( oldCell.getBooleanCellValue() );
                break;
            case Cell.CELL_TYPE_ERROR:
                newCell.setCellErrorValue ( oldCell.getErrorCellValue() );
                break;
            case Cell.CELL_TYPE_FORMULA:
                newCell.setCellFormula ( oldCell.getCellFormula() );
                break;
            case Cell.CELL_TYPE_NUMERIC:
                newCell.setCellValue ( oldCell.getNumericCellValue() );
                break;
            case Cell.CELL_TYPE_STRING:
                newCell.setCellValue ( oldCell.getRichStringCellValue() );
                break;
            }
        }
        for ( int i = 0; i < worksheet.getNumMergedRegions(); i++ ) {
            CellRangeAddress cellRangeAddress = worksheet.getMergedRegion ( i );
            if ( cellRangeAddress.getFirstRow() == sourceRow.getRowNum() ) {
                CellRangeAddress newCellRangeAddress = new CellRangeAddress ( newRow.getRowNum(),
                        ( newRow.getRowNum() +
                          ( cellRangeAddress.getLastRow() - cellRangeAddress.getFirstRow()
                          ) ),
                        cellRangeAddress.getFirstColumn(),
                        cellRangeAddress.getLastColumn() );
                worksheet.addMergedRegion ( newCellRangeAddress );
            }
        }
    }
}
