import com.steadystate.css.parser.CSSOMParser
import org.htmlcleaner.CleanerProperties
import org.htmlcleaner.HtmlCleaner;
import org.htmlcleaner.PrettyHtmlSerializer
import org.htmlcleaner.SimpleHtmlSerializer
import org.htmlcleaner.TagNode
import org.jsoup.Jsoup
import org.jsoup.nodes.Document
import org.jsoup.safety.Cleaner
import org.jsoup.safety.Whitelist
import org.jsoup.select.Elements
import org.w3c.css.sac.InputSource
import org.w3c.dom.css.CSSRule
import org.w3c.dom.css.CSSRuleList
import org.w3c.dom.css.CSSStyleDeclaration
import org.w3c.dom.css.CSSStyleSheet
import org.w3c.tidy.Tidy
class HtmlCleanerService {
    static transactional = true
    def cleanHTML ( def html ) {
        html = soupClean ( html );
        html = tidyClean ( html );
        return html;
    }
    def tidyClean ( def html ) {
        Tidy tidy = new Tidy()
        tidy.setAsciiChars ( true )
        tidy.setDropEmptyParas ( true )
        tidy.setDropProprietaryAttributes ( true )
        tidy.setPrintBodyOnly ( true )
        tidy.setEncloseText ( true )
        tidy.setJoinStyles ( true )
        tidy.setLogicalEmphasis ( true )
        tidy.setQuoteMarks ( true )
        tidy.setHideComments ( true )
        tidy.setWraplen ( 120 )
        tidy.setMakeClean ( true )
        tidy.setDropFontTags ( true )
        tidy.setWord2000 ( true )
        tidy.setMakeBare ( true )
        tidy.setRepeatedAttributes ( org.w3c.tidy.Configuration.KEEP_FIRST )
        def reader = new StringReader ( html );
        def writer = new StringWriter();
        tidy.setShowWarnings ( false )
        tidy.setErrout ( new PrintWriter ( new StringWriter() ) )
        tidy.parse ( reader, writer );
        return writer.toString()
    }
    def soupClean ( def html ) {
        Document dirty = Jsoup.parseBodyFragment ( html );
        Cleaner cleaner = new Cleaner ( createWhitelist() );
        Document clean = cleaner.clean ( dirty );
        Elements styledNodes = clean.select ( "[style]" );
        styledNodes.each { element ->
                           def style = element.attr ( "style" );
                           def tag = element.tagName().toLowerCase()
                                     def newstyle = ""
                                             CSSOMParser parser = new CSSOMParser();
                           InputSource is = new InputSource ( new StringReader ( style ) )
        CSSStyleDeclaration styledeclaration = parser.parseStyleDeclaration ( is )
                                               boolean hasProps = false
        for ( int i = 0; i < styledeclaration.getLength(); i++ ) {
        def propname = styledeclaration.item ( i )
                           def propval = styledeclaration.getPropertyValue ( propname )
                                         propval = propval ? propval.trim() : ""
            if ( ["padding-left", "text-decoration", "text-align", "font-weight", "font-style"].contains ( propname ) ) {
                newstyle = newstyle + propname + ": " + propval + ";"
                           hasProps = true
            }
            if ( tag.equals ( "table" ) && propname.equals ( "width" ) ) {
                if ( propval.endsWith ( "%" ) ) {
                    propval = propval.replaceAll ( ~"[^0-9]", "" )
                              propval = Math.min ( 100, propval.toInteger() )
                } else {
                    propval = propval.replaceAll ( ~"[^0-9]", "" )
                              propval = Math.min ( 100, ( int ) ( propval.toInteger() / 740 ) * 100 )
                }
                element.attr ( "width", propval + "%" )
            }
        }
        if ( hasProps ) {
        element.attr ( "style", newstyle )
        } else {
            element.removeAttr ( "style" )
        }
                         }
        return clean.body().html();
    }
    def createWhitelist() {
        Whitelist wl = new Whitelist();
        wl.addTags (
            "br", "div", "p", "pre", "span", "blockquote", "q", "hr",
            "h1", "h2", "h3", "h4", "h5", "h6",
            "u", "strike", "s", "strong", "sub", "sup", "em", "i", "b",
            "ul", "ol", "li", "ol",
            "table", "tbody", "td", "tfoot", "th", "thead", "tr",
        );
        ["span", "div", "p", "table", "ul", "ol", "pre", "td", "th"].each { tag ->
                ["style", "padding-left", "text-decoration", "text-align", "font-weight", "font-style"].each { attr ->
                        wl.addAttributes ( tag, attr )
                                                                                                             }
                                                                          }
        ["td", "th"].each { tag ->
                            ["colspan", "rowspan", "width"].each { attr ->
                                    wl.addAttributes ( tag, attr )
                                                                 }
                          }
        wl.addAttributes ( "table", "width", "style", "cellpadding" )
        return wl
    }
}
