public class LousyServlet extends HttpServlet {
    private Properties supportedLanguages;
    private Locale requestLocale = ( Locale ) supportedLanguages.get ( "DEFAULT" );
    public LousyServlet() {
        supportedLanguages = new Properties();
        supportedLanguages.put ( "DEFAULT", Locale.US );
        supportedLanguages.put ( "de-DEFAULT", Locale.GERMANY );
        supportedLanguages.put ( "de_AT", new Locale ( "de", "AT" ) );
        supportedLanguages.put ( "de_CH", new Locale ( "de", "CH" ) );
        supportedLanguages.put ( "ja_JP", Locale.JAPAN );
    }
    @Override
    protected void doGet ( HttpServletRequest request,
                           HttpServletResponse response ) throws ServletException, IOException {
        detectLocale ( request );
        super.doGet ( request, response );
    }
    @Override
    protected void doPost ( HttpServletRequest request, HttpServletResponse response )
    throws ServletException, IOException {
        detectLocale ( request );
        super.doPost ( request, response );
    }
    private void detectLocale ( HttpServletRequest request ) {
        Enumeration locales = request.getLocales();
        while ( locales.hasMoreElements() ) {
            Locale locale = ( Locale ) locales.nextElement();
            if ( supportedLanguages.contains ( locale ) ) {
                requestLocale = locale;
                break;
            }
        }
    }
    public String getLanguage() {
        return requestLocale.getDisplayLanguage();
    }
}
