import com.puppycrawl.tools.checkstyle.api.Check;
import com.puppycrawl.tools.checkstyle.api.DetailAST;
import com.puppycrawl.tools.checkstyle.api.TokenTypes;
public class IfBracesCheck extends Check {
    @Override
    public int[] getDefaultTokens() {
        return new int[] {
                   TokenTypes.LITERAL_ELSE,
                   TokenTypes.LITERAL_IF,
               };
    }
    @Override
    public void visitToken ( DetailAST aAST ) {
        final DetailAST slistAST = aAST.findFirstToken ( TokenTypes.SLIST );
        if ( aAST.getType() == TokenTypes.LITERAL_ELSE ) {
            DetailAST ifToken = aAST.findFirstToken ( TokenTypes.LITERAL_IF );
            if ( ifToken == null ) {
                if ( slistAST == null ) {
                    log ( aAST.getLineNo(), "ifBracesElse", aAST.getText() );
                }
            } else {
                if ( ifToken.findFirstToken ( TokenTypes.SLIST ) == null ) {
                    log ( aAST.getLineNo(), "ifBracesConditional", ifToken.getText(), aAST.getText() + " " + ifToken.getText() );
                }
            }
        } else if ( aAST.getType() == TokenTypes.LITERAL_IF ) {
            if ( slistAST != null ) {
                return;
            }
            final int[] conditionals = {
                TokenTypes.LITERAL_DO,
                TokenTypes.LITERAL_ELSE,
                TokenTypes.LITERAL_FOR,
                TokenTypes.LITERAL_IF,
                TokenTypes.LITERAL_WHILE,
                TokenTypes.LITERAL_SWITCH,
            };
            for ( int conditional : conditionals ) {
                DetailAST conditionalAST = aAST.findFirstToken ( conditional );
                if ( conditionalAST != null ) {
                    log ( aAST.getLineNo(), "ifBracesConditional", aAST.getText(), conditionalAST.getText() );
                    return;
                }
            }
        }
    }
}
