@Entity
@Table ( name = "USER" )
@Access ( AccessType.FIELD )
public class User implements UserDetails, Serializable {
    private static final long serialVersionUID = 2L;
    @Id
    @Column ( name = "USER_ID", updatable = false, nullable = false )
    @GeneratedValue ( strategy = GenerationType.IDENTITY )
    private long id;
    @Column ( name = "USERNAME" )
    private String username;
    @Column ( name = "PASSWORD" )
    private String password;
    @Column ( name = "NAME" )
    private String name;
    @Column ( name = "EMAIL" )
    private String email;
    @Column ( name = "LOCKED" )
    private boolean locked;
    @OneToMany ( cascade = CascadeType.ALL, fetch = FetchType.EAGER, targetEntity = Role.class )
    @JoinTable ( name = "USER_ROLE", joinColumns = { @JoinColumn ( name = "USER_ID" ) }, inverseJoinColumns = { @JoinColumn ( name = "ROLE_ID" ) } )
    private Set<Role> roles;
    @Override
    public GrantedAuthority[] getAuthorities() {
        List<GrantedAuthorityImpl> list = new ArrayList<GrantedAuthorityImpl> ( 0 );
        for ( Role role : roles ) {
            list.add ( new GrantedAuthorityImpl ( role.getRole() ) );
        }
        return ( GrantedAuthority[] ) list.toArray ( new GrantedAuthority[list.size()] );
    }
    @Override
    public boolean isAccountNonExpired() {
        return true;
    }
    @Override
    public boolean isAccountNonLocked() {
        return !isLocked();
    }
    @Override
    public boolean isCredentialsNonExpired() {
        return true;
    }
    @Override
    public boolean isEnabled() {
        return true;
    }
    public long getId() {
        return id;
    }
    public void setId ( long id ) {
        this.id = id;
    }
    @Override
    public String getUsername() {
        return username;
    }
    public void setUsername ( String username ) {
        this.username = username;
    }
    @Override
    public String getPassword() {
        return password;
    }
    public void setPassword ( String password ) {
        this.password = password;
    }
    public String getName() {
        return name;
    }
    public void setName ( String name ) {
        this.name = name;
    }
    public String getEmail() {
        return email;
    }
    public void setEmail ( String email ) {
        this.email = email;
    }
    public boolean isLocked() {
        return locked;
    }
    public void setLocked ( boolean locked ) {
        this.locked = locked;
    }
    public Set<Role> getRoles() {
        return roles;
    }
    public void setRoles ( Set<Role> roles ) {
        this.roles = roles;
    }
}
