class DependencyStack {
    private final Map<String, DependencyDataCollection> dependencies;
    {
        dependencies = new LinkedHashMap<String, DependencyDataCollection>();
    }
    public void add ( final DependencyDataCollection ddc ) {
        if ( ddc == null ) {
            throw new IllegalArgumentException ( "ddc cannot be null" );
        }
        dependencies.put ( ddc.getSessionID(), ddc );
    }
    public int size() {
        return dependencies.size();
    }
    public void show() {
        show ( System.out );
    }
    public void show ( final OutputStream out ) {
        show ( new OutputStreamWriter ( out ) );
    }
    public void show ( final Writer writer ) {
        show ( new PrintWriter ( writer ) );
    }
    public void show ( final PrintWriter writer ) {
        for ( final DependencyDataCollection ddc : dependencies.values() ) {
            writer.println ( ddc.getSNo()            + "   " +
                             ddc.getSessionID()      + "   " +
                             ddc.getNoOfDependency() + "   " +
                             ddc.getNoOfRejection()  + "   " +
                             ddc.getTotalValue() );
        }
    }
    public int returnIndexOfSession ( final String sessionID ) {
        final DependencyDataCollection ddc;
        final int                      retVal;
        if ( sessionID == null ) {
            throw new IllegalArgumentException ( "sessionID cannot be null" );
        }
        ddc = dependencies.get ( sessionID );
        if ( ddc == null ) {
            retVal = -1;
        } else {
            retVal = ddc.getSNo();
        }
        return ( retVal );
    }
    public static void main ( final String[] args ) {
        DependencyStack ds = new DependencyStack();
        ds.add ( new DependencyDataCollection ( 1, "a", 0, 0, 0 ) );
        ds.add ( new DependencyDataCollection ( 1, "a", 0, 0, 0 ) );
        ds.show();
    }
}
