import java.io.File;
import org.netbeans.Main;
public class MultipleInstancesMain {
    private static final String NETBEANS_USER = "netbeans.user";
    private static final String USERDIR_ARG = "--userdir";
    private static final File MAIN_DIR = getMainDir();
    private static final String INSTANCE_DIR = "instance";
    private static final String NETBEANS_SUBDIR = "netbeans";
    private static final String LOCKFILE = "lock";
    private static final int MAX_DIR_COUNT = 999;
    private static final String PAD = "000";
    private static final int PAD_LENGTH = PAD.length();
    private static final long LOCKFILE_MAX_AGE_IN_MILLIS = 1000L * 60L * 60L * 24L * 30L;
    public static void main ( String[] args ) throws Exception {
        String[] userDir = new String[2];
        userDir[0] = USERDIR_ARG;
        userDir[1] = getNetbeansDir();
        deleteDefaultNetbeansDir();
        System.clearProperty ( "netbeans.mainclass" );
        startNetbeans ( args, userDir );
    }
    private static void startNetbeans ( String[] oldArgs, String[] newArgs )  throws Exception {
        String[] args = new String[oldArgs.length + newArgs.length];
        for ( int i = 0; i < oldArgs.length; i++ ) {
            args[i] = oldArgs[i];
        }
        for ( int i = 0; i < newArgs.length; i++ ) {
            args[oldArgs.length + i] = newArgs[i];
        }
        Main.main ( args );
    }
    private static String getNetbeansDir() {
        for ( int i = 1; i <= MAX_DIR_COUNT; i++ ) {
            File instanceDir = getSuffixedInstanceDir ( i );
            if ( isLockFileFree ( instanceDir ) ) {
                File dirToUse = new File ( instanceDir, NETBEANS_SUBDIR );
                return dirToUse.getAbsolutePath();
            }
        }
        String errorMessage = String.format ( "Unable to find Netbeans userdir, %s dirs checked in '%s'",
                                              MAX_DIR_COUNT, MAIN_DIR.getAbsolutePath() );
        throw new RuntimeException ( errorMessage );
    }
    private static File getSuffixedInstanceDir ( int count ) {
        String suffix = PAD + count;
        suffix = suffix.substring ( suffix.length() - PAD_LENGTH );
        File suffixedDir = new File ( MAIN_DIR, INSTANCE_DIR + "_" + suffix );
        return suffixedDir;
    }
    private static boolean isLockFileFree ( File instanceDir ) {
        File netbeansDir = new File ( instanceDir, NETBEANS_SUBDIR );
        File lockFile = new File ( netbeansDir, LOCKFILE );
        if ( lockFile.exists() ) {
            return deleteLockFileIfOldEnough ( lockFile );
        } else {
            return true;
        }
    }
    private static boolean deleteLockFileIfOldEnough ( File lockFile ) {
        long currentTime = System.currentTimeMillis();
        long fileCreated = lockFile.lastModified();
        long ageInMillis = currentTime - fileCreated;
        if ( ageInMillis > LOCKFILE_MAX_AGE_IN_MILLIS ) {
            return lockFile.delete();
        } else {
            return false;
        }
    }
    private static File getMainDir() {
        String defaultNetbeansDir = System.getProperty ( NETBEANS_USER );
        File mainDir = new File ( defaultNetbeansDir ).getParentFile();
        return mainDir;
    }
    private static void deleteDefaultNetbeansDir() {
        File defaultNetbeansDir = new File ( System.getProperty ( NETBEANS_USER ) );
        Thread t = new Thread ( new DirectoryDeleter ( defaultNetbeansDir ), "NetbeansDirDeleter" );
        t.start();
    }
    private static class DirectoryDeleter implements Runnable {
        private static final long SLEEP_TIME = 3000;
        private final File dirToDelete;
        DirectoryDeleter ( File dirToDelete ) {
            this.dirToDelete = dirToDelete;
        }
        public void run() {
            while ( !deleteDirOrFile ( dirToDelete ) ) {
                try {
                    Thread.sleep ( SLEEP_TIME );
                } catch ( InterruptedException e ) {
                    continue;
                }
            }
        }
        private boolean deleteDirOrFile ( File dirFile ) {
            if ( dirFile.isDirectory() ) {
                for ( File f : dirFile.listFiles() ) {
                    boolean deleted = deleteDirOrFile ( f );
                    if ( !deleted ) {
                        return false;
                    }
                }
            }
            return dirFile.delete();
        }
    }
}
