protected List<Rectangle> findHighlightAreas ( JLabel object, int width,
        int height ) {
    String text = object.getText();
    insets = object.getInsets ( insets );
    viewR.x = 0 + insets.left;
    viewR.y = 0 + insets.bottom;
    viewR.width = width - insets.right;
    viewR.height = height - insets.top;
    iconR.x = iconR.y = 0;
    textR.x = textR.y = 0;
    FontMetrics fm = object.getFontMetrics ( object.getFont() );
    String clippedText = SwingUtilities.layoutCompoundLabel ( object, fm,
                         text, object.getIcon(), object.getVerticalAlignment(), object
                         .getHorizontalAlignment(), object
                         .getVerticalTextPosition(), object
                         .getHorizontalTextPosition(), viewR, iconR, textR,
                         object.getIconTextGap() );
    int xOffset = calculateXOffset ( object, viewR, textR );
    String textToSearch = clippedText;
    if ( !object.getText().equals ( clippedText ) ) {
        textToSearch = clippedText.substring ( 0, clippedText.length() - 3 );
    }
    return createHighlightAreas ( textToSearch, fm, xOffset, height );
}
protected List<Rectangle> createHighlightAreas ( String text, FontMetrics fm,
        int xOffset, int height ) {
    SearchPredicate predicate = ( SearchPredicate ) getHighlightPredicate();
    Matcher matcher = predicate.getPattern().matcher ( text );
    List<Rectangle> highlightAreas = null;
    int startFrom = 0;
    while ( startFrom < text.length() && matcher.find ( startFrom ) ) {
        if ( highlightAreas == null ) {
            highlightAreas = new ArrayList<Rectangle>();
        }
        int start = matcher.start();
        int end = matcher.end();
        if ( start == end ) {
            break;
        }
        startFrom = end;
        int highlightx;
        int highlightWidth;
        if ( start == 0 ) {
            highlightx = textR.x + xOffset;
        } else {
            String strToStart = text.substring ( 0, start );
            highlightx = textR.x + fm.stringWidth ( strToStart ) + xOffset;
        }
        String highlightText = text.substring ( start, end );
        highlightWidth = fm.stringWidth ( highlightText );
        highlightAreas.add ( new Rectangle ( highlightx, 0, highlightWidth,
                                             height ) );
    }
    if ( highlightAreas == null ) {
        highlightAreas = Collections.emptyList();
    } else {
        coalesceHighlightAreas ( highlightAreas );
    }
    return highlightAreas;
}
protected void coalesceHighlightAreas ( List<Rectangle> highlightAreas ) {
    Collections.sort ( highlightAreas, X_AXIS_RECTANGLE_COMPARATOR );
    int i = 0;
    while ( i < highlightAreas.size() - 1 ) {
        Rectangle r1 = highlightAreas.get ( i );
        Rectangle r2 = highlightAreas.get ( i + 1 );
        if ( r1.x + r1.width == r2.x ) {
            r1.width += r2.width;
            highlightAreas.remove ( i + 1 );
        } else {
            i++;
        }
    }
}
protected int calculateXOffset ( JLabel component, Rectangle viewR,
                                 Rectangle textR ) {
    int horizAlignment = component.getHorizontalAlignment();
    boolean leftToRight = component.getComponentOrientation()
                          .isLeftToRight();
    if ( horizAlignment == SwingConstants.LEFT
            || ( horizAlignment == SwingConstants.LEADING && leftToRight )
            || ( horizAlignment == SwingConstants.TRAILING && !leftToRight ) ) {
        return 0;
    } else if ( horizAlignment == SwingConstants.RIGHT
                || ( horizAlignment == SwingConstants.TRAILING && !leftToRight )
                || ( horizAlignment == SwingConstants.LEADING && leftToRight ) ) {
        return viewR.width - textR.width;
    } else if ( horizAlignment == SwingConstants.CENTER ) {
        return ( viewR.width - textR.width ) / 2;
    }
    throw new AssertionError ( "Unknown horizonal alignment "
                               + horizAlignment );
}
