public interface TextRenderer {
    public enum FontSize { LARGE, SMALL };
    public void render ( String text, int x, int y, FontSize size );
}
public class ScreenDimensionAwareTextRendererFactory {
    public static TextRenderer getTextRenderer ( Graphics gc ) {
        Font baseFont = Font.getDefault();
        int fontWidth = ScreenDimension.getFontWidth();
        return fontWidth == 320
               ? new DefaultTextRenderer ( gc, baseFont.derive ( Font.PLAIN, 10, Ui.UNITS_pt ), baseFont.derive ( Font.PLAIN, 6, Ui.UNITS_pt ) )
               : new DefaultTextRenderer ( gc, baseFont.derive ( Font.PLAIN, 12, Ui.UNITS_pt ), baseFont.derive ( Font.PLAIN, 8, Ui.UNITS_pt ) );
    }
}
public class DefaultTextRenderer implements TextRenderer {
    protected Map<FontSize, Font> fontSizeMap = new HashMap<FontSize, Font>();
    protected Graphics gc;
    public DefaultTextRenderer ( Graphics gc, Font largeFont, Font smallFont ) {
        this.gc = gc;
        fontSizeMap.put ( LARGE, largeFont );
        fontSizeMap.put ( SMALL, smallFont );
    }
    public void render ( String text, int x, int y, FontSize size ) {
        Font font = fontSizeMap.get ( size )
                    int pixelWidth = font.getAdvance ( text );
        gc.setFont ( font );
        gc.drawText ( text, Math.round ( x - ( pixelWidth / 2 ) ), y );
    }
}
public void paint ( Graphics graphics ) {
    super.paint ( graphics );
    String strScore = String.valueOf ( score );
    TextRenderer textRenderer = ScreenDimensionAwareTextRendererFactory.getTextRenderer ( graphics );
    int middleX = Math.round ( this.backgroundBitmap.getWidth() / 2 );
    textRenderer.render ( strScore, middleX, 10, TextRenderer.TextSize.LARGE );
    textRenderer.render ( judgeName, middleX, this.backgroundBitmap.getHeight(), TextRenderer.TextSize.SMALL );
}
