public class DBSessionFilter implements Filter {
    private static final Logger log = Logger.getLogger ( DBSessionFilter.class );
    private SessionFactory sf;
    @Override
    public void destroy() {
    }
    @Override
    public void doFilter ( ServletRequest request, ServletResponse response,
                           FilterChain chain ) throws IOException, ServletException {
        try {
            log.debug ( "Starting a database transaction" );
            sf.getCurrentSession().beginTransaction();
            chain.doFilter ( request, response );
            log.debug ( "Committing the database transaction" );
            sf.getCurrentSession().getTransaction().commit();
        } catch ( StaleObjectStateException staleEx ) {
            log.error ( "This interceptor does not implement optimistic concurrency control!" );
            log.error ( "Your application will not work until you add compensation actions!" );
            throw staleEx;
        } catch ( Throwable ex ) {
            ex.printStackTrace();
            try {
                if ( sf.getCurrentSession().getTransaction().isActive() ) {
                    log.debug ( "Trying to rollback database transaction after exception" );
                    sf.getCurrentSession().getTransaction().rollback();
                }
            } catch ( Throwable rbEx ) {
                log.error ( "Could not rollback transaction after exception!", rbEx );
            }
            throw new ServletException ( ex );
        }
    }
    @Override
    public void init ( FilterConfig arg0 ) throws ServletException {
        log.debug ( "Initializing filter..." );
        log.debug ( "Obtaining SessionFactory from static HibernateUtil singleton" );
        sf = HibernateUtils.getSessionFactory();
    }
