public static KeyStore createCertificateAuthority ( String keyAlgorithm, int keyLength, char[] storePassword, char[] keyPassword, String alias,
        String signatureAlgorithm, String issuer, Date validFrom, Date validTo )
throws GeneralSecurityException {
    String subject = issuer;
    String subjectAltName = null;
    String subjectIPAddress = null;
    KeyPair keyPair = SecurityUtils.generateKeyPair ( keyAlgorithm, keyLength );
    X509Certificate x509Certificate = CertificateUtils.generateV3Certificate (
                                          new X500Principal ( issuer ), new X500Principal ( subject ),
                                          false, false, subjectAltName, subjectIPAddress,
                                          keyPair.getPublic(), keyPair.getPrivate(), validFrom, validTo, signatureAlgorithm );
    x509Certificate.checkValidity ( new Date() );
    x509Certificate.verify ( keyPair.getPublic() );
    X509Certificate[] chain = new X509Certificate[1];
    chain[0] = x509Certificate;
    KeyStore keyStoreSigningKey = SecurityUtils.createKeyStore ( storePassword );
    keyStoreSigningKey.setKeyEntry ( alias, keyPair.getPrivate(), keyPassword, chain );
    return keyStoreSigningKey;
}
public static KeyPair generateKeyPair ( String algorithm, int keySize ) throws NoSuchAlgorithmException {
    KeyPairGenerator kpg = KeyPairGenerator.getInstance ( algorithm );
    kpg.initialize ( keySize );
    return kpg.generateKeyPair();
}
public static KeyStore createKeyStore ( char[] storePassword ) throws KeyStoreException, NoSuchAlgorithmException, CertificateException {
    KeyStore keyStore = KeyStore.getInstance ( KeyStore.getDefaultType() );
    try {
        keyStore.load ( null, storePassword );
    } catch ( IOException e ) {
        throw new KeyStoreException ( e );
    }
    return keyStore;
}
