@implementation QTKitCaptureDecompressedVideoOutput
- ( QTKitCaptureDecompressedVideoOutput * ) initWithEnv: ( JNIEnv * ) env javaObject: ( jobject ) javaObjectRef {
    [super init];
    ( * env )->GetJavaVM ( env, & g_vm );
    objectRef = ( * env )->NewGlobalRef ( env, javaObjectRef );
    return self;
}
- ( void ) captureOutput: ( QTCaptureOutput * ) captureOutput didOutputVideoFrame: ( CVImageBufferRef ) videoFrame withSampleBuffer: ( QTSampleBuffer * ) sampleBuffer fromConnection: ( QTCaptureConnection * ) connection {
    JNIEnv * env;
    ( * g_vm )->AttachCurrentThread ( g_vm, ( void * * ) & env, NULL );
    void * rawData = [sampleBuffer bytesForAllSamples];
    int length = [sampleBuffer lengthForAllSamples];
    QTFormatDescription * formatDescription = [sampleBuffer formatDescription];
    QTTime duration = [sampleBuffer duration];
    float frameDuration = duration.timeValue / duration.timeScale;
    float fps = 1 / frameDuration;
    jint format = [formatDescription formatType];
    NSValue * pixelSize = [formatDescription attributeForKey:QTFormatDescriptionVideoEncodedPixelsSizeAttribute];
    NSSize size = [pixelSize sizeValue];
    jint width = size.width;
    jint height = size.height;
    switch ( format ) {
    case kCVPixelFormatType_1Monochrome:
    case kCVPixelFormatType_2Indexed:
    case kCVPixelFormatType_4Indexed:
    case kCVPixelFormatType_8Indexed:
    case kCVPixelFormatType_1IndexedGray_WhiteIsZero:
    case kCVPixelFormatType_2IndexedGray_WhiteIsZero:
    case kCVPixelFormatType_4IndexedGray_WhiteIsZero:
    case kCVPixelFormatType_8IndexedGray_WhiteIsZero:
    case kCVPixelFormatType_422YpCbCr8:
    case kCVPixelFormatType_4444YpCbCrA8:
    case kCVPixelFormatType_4444YpCbCrA8R:
    case kCVPixelFormatType_444YpCbCr8:
    case kCVPixelFormatType_420YpCbCr8Planar:
    case kCVPixelFormatType_422YpCbCr_4A_8BiPlanar:
    case kCVPixelFormatType_24RGB:
    case kCVPixelFormatType_24BGR:
    default: {
        if ( byteFrameData == nil || ( * env )->GetArrayLength ( env, byteFrameData ) < length ) {
            if ( byteFrameData != nil ) {
                ( * env )->DeleteGlobalRef ( env, byteFrameData );
                byteFrameData = nil;
            }
            byteFrameData = ( * env )->NewGlobalRef ( env, ( * env )->NewByteArray ( env, length ) );
        }
        if ( byteFrameData ) {
            ( * env )->SetByteArrayRegion ( env, byteFrameData, 0, length, rawData );
            jclass classRef = ( * env )->GetObjectClass ( env, objectRef );
            jmethodID methodId = ( * env )->GetMethodID ( env, classRef, "pushFrame", "([BIIIF)V" );
            ( * env )->CallVoidMethod ( env, objectRef, methodId, byteFrameData, format, width, height, fps );
        }
        break;
    }
    case kCVPixelFormatType_16BE555:
    case kCVPixelFormatType_16LE555:
    case kCVPixelFormatType_16LE5551:
    case kCVPixelFormatType_16BE565:
    case kCVPixelFormatType_16LE565:
    case kCVPixelFormatType_16Gray:
    case kCVPixelFormatType_422YpCbCr16:
    case kCVPixelFormatType_422YpCbCr10:
    case kCVPixelFormatType_444YpCbCr10: {
        if ( shortFrameData == nil || ( * env )->GetArrayLength ( env, shortFrameData ) < length / 2 ) {
            if ( shortFrameData != nil ) {
                ( * env )->DeleteGlobalRef ( env, shortFrameData );
                shortFrameData = nil;
            }
            shortFrameData = ( * env )->NewGlobalRef ( env, ( * env )->NewShortArray ( env, length / 2 ) );
        }
        if ( shortFrameData ) {
            ( * env )->SetShortArrayRegion ( env, shortFrameData, 0, length / 2, rawData );
            jclass classRef = ( * env )->GetObjectClass ( env, objectRef );
            jmethodID methodId = ( * env )->GetMethodID ( env, classRef, "pushFrame", "([SIIIF)V" );
            ( * env )->CallVoidMethod ( env, objectRef, methodId, shortFrameData, format, width, height, fps );
        }
        break;
    }
    case kCVPixelFormatType_32ABGR:
    case kCVPixelFormatType_32AlphaGray:
    case kCVPixelFormatType_32ARGB:
    case kCVPixelFormatType_32BGRA:
    case kCVPixelFormatType_32RGBA: {
        if ( intFrameData == nil || ( * env )->GetArrayLength ( env, intFrameData ) < length / 4 ) {
            if ( intFrameData != nil ) {
                ( * env )->DeleteGlobalRef ( env, intFrameData );
                intFrameData = nil;
            }
            intFrameData = ( * env )->NewGlobalRef ( env, ( * env )->NewIntArray ( env, length / 4 ) );
        }
        if ( intFrameData ) {
            ( * env )->SetByteArrayRegion ( env, intFrameData, 0, length / 4, rawData );
            jclass classRef = ( * env )->GetObjectClass ( env, objectRef );
            jmethodID methodId = ( * env )->GetMethodID ( env, classRef, "pushFrame", "([IIIIF)V" );
            ( * env )->CallVoidMethod ( env, objectRef, methodId, intFrameData, format, width, height, fps );
        }
        break;
    }
    }
    ( * g_vm )->DetachCurrentThread ( g_vm );
}
- ( void ) dealloc {
    JNIEnv * env;
    ( * g_vm )->AttachCurrentThread ( g_vm, ( void * * ) & env, NULL );
    if ( objectRef != nil ) {
        ( * env )->DeleteGlobalRef ( env, objectRef );
        objectRef = nil;
    }
    if ( byteFrameData != nil ) {
        ( * env )->DeleteGlobalRef ( env, byteFrameData );
        byteFrameData = nil;
    }
    if ( shortFrameData != nil ) {
        ( * env )->DeleteGlobalRef ( env, shortFrameData );
        shortFrameData = nil;
    }
    if ( intFrameData != nil ) {
        ( * env )->DeleteGlobalRef ( env, intFrameData );
        intFrameData = nil;
    }
    ( * g_vm )->DetachCurrentThread ( g_vm );
    g_vm = nil;
    [super dealloc];
}
@end
