import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.EventQueue;
import java.awt.event.ActionEvent;
import java.util.*;
import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import org.jfree.chart.*;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.axis.NumberTickUnit;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYItemRenderer;
import org.jfree.data.xy.XYDataset;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;
public class ScatterAdd extends JFrame {
    private static final int N = 8;
    private static final String title = "Scatter Add Demo";
    private static final Random rand = new Random();
    private XYSeries added = new XYSeries ( "Added" );
    public ScatterAdd ( String s ) {
        super ( s );
        final ChartPanel chartPanel = createDemoPanel();
        this.add ( chartPanel, BorderLayout.CENTER );
        JPanel control = new JPanel();
        control.add ( new JButton ( new AbstractAction ( "Add" ) {
            @Override
            public void actionPerformed ( ActionEvent e ) {
                for ( int i = 0; i < N; i++ ) {
                    added.add ( rand.nextDouble(), rand.nextDouble() );
                }
            }
        } ) );
        this.add ( control, BorderLayout.SOUTH );
    }
    private ChartPanel createDemoPanel() {
        JFreeChart jfreechart = ChartFactory.createScatterPlot (
                                    title, "X", "Y", createSampleData(),
                                    PlotOrientation.VERTICAL, true, true, false );
        XYPlot xyPlot = ( XYPlot ) jfreechart.getPlot();
        xyPlot.setDomainCrosshairVisible ( true );
        xyPlot.setRangeCrosshairVisible ( true );
        XYItemRenderer renderer = xyPlot.getRenderer();
        renderer.setSeriesPaint ( 0, Color.blue );
        NumberAxis domain = ( NumberAxis ) xyPlot.getDomainAxis();
        domain.setRange ( 0.00, 1.00 );
        domain.setTickUnit ( new NumberTickUnit ( 0.1 ) );
        domain.setVerticalTickLabels ( true );
        NumberAxis range = ( NumberAxis ) xyPlot.getRangeAxis();
        range.setRange ( 0.0, 1.0 );
        range.setTickUnit ( new NumberTickUnit ( 0.1 ) );
        return new ChartPanel ( jfreechart );
    }
    private XYDataset createSampleData() {
        XYSeriesCollection xySeriesCollection = new XYSeriesCollection();
        XYSeries series = new XYSeries ( "Random" );
        for ( int i = 0; i < N * N; i++ ) {
            double x = rand.nextDouble();
            double y = rand.nextDouble();
            series.add ( x, y );
        }
        xySeriesCollection.addSeries ( series );
        xySeriesCollection.addSeries ( added );
        return xySeriesCollection;
    }
    public static void main ( String args[] ) {
        EventQueue.invokeLater ( new Runnable() {
            @Override
            public void run() {
                ScatterAdd demo = new ScatterAdd ( title );
                demo.setDefaultCloseOperation ( JFrame.EXIT_ON_CLOSE );
                demo.pack();
                demo.setLocationRelativeTo ( null );
                demo.setVisible ( true );
            }
        } );
    }
}
