public final class ExcelDocumentConverter {
    public static XSSFWorkbook convertWorkbookHSSFToXSSF ( HSSFWorkbook source ) {
        XSSFWorkbook retVal = new XSSFWorkbook();
        for ( int i = 0; i < source.getNumberOfSheets(); i++ ) {
            XSSFSheet xssfSheet = retVal.createSheet();
            HSSFSheet hssfsheet = source.getSheetAt ( i );
            copySheets ( hssfsheet, xssfSheet );
        }
        return retVal;
    }
    public static void copySheets ( HSSFSheet source, XSSFSheet destination ) {
        copySheets ( source, destination, true );
    }
    public static void copySheets ( HSSFSheet source, XSSFSheet destination, boolean copyStyle ) {
        int maxColumnNum = 0;
        Map<Integer, HSSFCellStyle> styleMap = ( copyStyle ) ? new HashMap<Integer, HSSFCellStyle>() : null;
        for ( int i = source.getFirstRowNum(); i <= source.getLastRowNum(); i++ ) {
            HSSFRow srcRow = source.getRow ( i );
            XSSFRow destRow = destination.createRow ( i );
            if ( srcRow != null ) {
                copyRow ( source, destination, srcRow, destRow, styleMap );
                if ( srcRow.getLastCellNum() > maxColumnNum ) {
                    maxColumnNum = srcRow.getLastCellNum();
                }
            }
        }
        for ( int i = 0; i <= maxColumnNum; i++ ) {
            destination.setColumnWidth ( i, source.getColumnWidth ( i ) );
        }
    }
    public static void copyRow ( HSSFSheet srcSheet, XSSFSheet destSheet, HSSFRow srcRow, XSSFRow destRow,
                                 Map<Integer, HSSFCellStyle> styleMap ) {
        Set<CellRangeAddressWrapper> mergedRegions = new TreeSet<CellRangeAddressWrapper>();
        destRow.setHeight ( srcRow.getHeight() );
        for ( int j = srcRow.getFirstCellNum(); j <= srcRow.getLastCellNum(); j++ ) {
            HSSFCell oldCell = srcRow.getCell ( j );
            XSSFCell newCell = destRow.getCell ( j );
            if ( oldCell != null ) {
                if ( newCell == null ) {
                    newCell = destRow.createCell ( j );
                }
                copyCell ( oldCell, newCell, styleMap );
                CellRangeAddress mergedRegion = getMergedRegion ( srcSheet, srcRow.getRowNum(),
                                                ( short ) oldCell.getColumnIndex() );
                if ( mergedRegion != null ) {
                    CellRangeAddress newMergedRegion = new CellRangeAddress ( mergedRegion.getFirstRow(),
                            mergedRegion.getLastRow(), mergedRegion.getFirstColumn(), mergedRegion.getLastColumn() );
                    CellRangeAddressWrapper wrapper = new CellRangeAddressWrapper ( newMergedRegion );
                    if ( isNewMergedRegion ( wrapper, mergedRegions ) ) {
                        mergedRegions.add ( wrapper );
                        destSheet.addMergedRegion ( wrapper.range );
                    }
                }
            }
        }
    }
    public static void copyCell ( HSSFCell oldCell, XSSFCell newCell, Map<Integer, HSSFCellStyle> styleMap ) {
        if ( styleMap != null ) {
            int stHashCode = oldCell.getCellStyle().hashCode();
            HSSFCellStyle sourceCellStyle = styleMap.get ( stHashCode );
            XSSFCellStyle destnCellStyle = newCell.getCellStyle();
            if ( sourceCellStyle == null ) {
                sourceCellStyle = oldCell.getSheet().getWorkbook().createCellStyle();
            }
            destnCellStyle.cloneStyleFrom ( oldCell.getCellStyle() );
            styleMap.put ( stHashCode, sourceCellStyle );
            newCell.setCellStyle ( destnCellStyle );
        }
        switch ( oldCell.getCellType() ) {
        case HSSFCell.CELL_TYPE_STRING:
            newCell.setCellValue ( oldCell.getStringCellValue() );
            break;
        case HSSFCell.CELL_TYPE_NUMERIC:
            newCell.setCellValue ( oldCell.getNumericCellValue() );
            break;
        case HSSFCell.CELL_TYPE_BLANK:
            newCell.setCellType ( HSSFCell.CELL_TYPE_BLANK );
            break;
        case HSSFCell.CELL_TYPE_BOOLEAN:
            newCell.setCellValue ( oldCell.getBooleanCellValue() );
            break;
        case HSSFCell.CELL_TYPE_ERROR:
            newCell.setCellErrorValue ( oldCell.getErrorCellValue() );
            break;
        case HSSFCell.CELL_TYPE_FORMULA:
            newCell.setCellFormula ( oldCell.getCellFormula() );
            break;
        default:
            break;
        }
    }
    public static CellRangeAddress getMergedRegion ( HSSFSheet sheet, int rowNum, short cellNum ) {
        for ( int i = 0; i < sheet.getNumMergedRegions(); i++ ) {
            CellRangeAddress merged = sheet.getMergedRegion ( i );
            if ( merged.isInRange ( rowNum, cellNum ) ) {
                return merged;
            }
        }
        return null;
    }
    private static boolean isNewMergedRegion ( CellRangeAddressWrapper newMergedRegion,
            Set<CellRangeAddressWrapper> mergedRegions ) {
        return !mergedRegions.contains ( newMergedRegion );
    }
