private static class BitmapCache {
    private final int mCacheSize;
    private final String mBitmapLocation;
    private LinkedHashMap<String, Bitmap> mBitmapCache;
    public BitmapCache ( int cacheSize, String bitmapLocation ) {
        mCacheSize = cacheSize;
        mBitmapLocation = bitmapLocation;
        mBitmapCache = new LinkedHashMap<String, Bitmap> ( mCacheSize + 1 ) {
            private static final long serialVersionUID = -4156123801558395154L;
            @Override
            protected boolean removeEldestEntry (
                java.util.Map.Entry<String, Bitmap> eldest ) {
                return size() > mCacheSize;
            };
            @Override
            public Bitmap remove ( Object key ) {
                Bitmap bmp = super.get ( key );
                bmp.recycle();
                return super.remove ( key );
            }
        };
    }
    public Bitmap getBitmap ( String bitmapFilename ) {
        Bitmap ret = mBitmapCache.get ( bitmapFilename );
        if ( ret == null ) {
            ret = BitmapFactory.decodeFile ( new File ( mBitmapLocation, bitmapFilename ).getAbsolutePath() );
            mBitmapCache.put ( bitmapFilename, ret );
        }
        return ret;
    }
    public void clear() {
        if ( mBitmapCache != null ) {
            for ( Bitmap bitmap : mBitmapCache.values() ) {
                if ( bitmap != null ) {
                    bitmap.recycle();
                }
            }
            mBitmapCache.clear();
        }
    }
}
