public static BigDecimal ln ( BigDecimal x, int scale ) {
    if ( x.signum() <= 0 ) {
        throw new IllegalArgumentException ( "x <= 0" );
    }
    int magnitude = x.toString().length() - x.scale() - 1;
    if ( magnitude < 3 ) {
        return lnNewton ( x, scale );
    } else {
        BigDecimal root = intRoot ( x, magnitude, scale );
        BigDecimal lnRoot = lnNewton ( root, scale );
        return BigDecimal.valueOf ( magnitude ).multiply ( lnRoot )
               .setScale ( scale, BigDecimal.ROUND_HALF_EVEN );
    }
}
private static BigDecimal lnNewton ( BigDecimal x, int scale ) {
    int        sp1 = scale + 1;
    BigDecimal n   = x;
    BigDecimal term;
    BigDecimal tolerance = BigDecimal.valueOf ( 5 )
                           .movePointLeft ( sp1 );
    do {
        BigDecimal eToX = exp ( x, sp1 );
        term = eToX.subtract ( n )
               .divide ( eToX, sp1, BigDecimal.ROUND_DOWN );
        x = x.subtract ( term );
        Thread.yield();
    } while ( term.compareTo ( tolerance ) > 0 );
    return x.setScale ( scale, BigDecimal.ROUND_HALF_EVEN );
}
