public final class IntegerPerm {
    private final static int DEFAULT_KEY = 0x6CFB18E2;
    private final static int LOW_16_MASK = 0xFFFF;
    private final static int HALF_SHIFT = 16;
    private final static int NUM_ROUNDS = 4;
    private int mKey;
    private int[] mRoundKeys = new int[NUM_ROUNDS];
    public IntegerPerm() {
        this ( DEFAULT_KEY );
    }
    public IntegerPerm ( int key ) {
        setKey ( key );
    }
    public void setKey ( int newKey ) {
        assert ( NUM_ROUNDS == 4 ) : "NUM_ROUNDS is not 4";
        mKey = newKey;
        mRoundKeys[0] = mKey & LOW_16_MASK;
        mRoundKeys[1] = ~ ( mKey & LOW_16_MASK );
        mRoundKeys[2] = mKey >>> HALF_SHIFT;
        mRoundKeys[3] = ~ ( mKey >>> HALF_SHIFT );
    }
    public int getKey() {
        return mKey;
    }
    public int encipher ( int plain ) {
        int rhs = plain & LOW_16_MASK;
        int lhs = plain >>> HALF_SHIFT;
        for ( int i = 0; i < NUM_ROUNDS; ++i ) {
            if ( i > 0 ) {
                final int temp = lhs;
                lhs = rhs;
                rhs = temp;
            }
            rhs ^= F ( lhs, i );
        }
        return ( lhs << HALF_SHIFT ) + ( rhs & LOW_16_MASK );
    }
    public int decipher ( int cypher ) {
        int rhs = cypher & LOW_16_MASK;
        int lhs = cypher >>> HALF_SHIFT;
        for ( int i = 0; i < NUM_ROUNDS; ++i ) {
            if ( i > 0 ) {
                final int temp = lhs;
                lhs = rhs;
                rhs = temp;
            }
            rhs ^= F ( lhs, NUM_ROUNDS - 1 - i );
        }
        return ( lhs << HALF_SHIFT ) + ( rhs & LOW_16_MASK );
    }
    private int F ( int num, int round ) {
        num ^= mRoundKeys[round];
        num *= num;
        return ( num >>> HALF_SHIFT ) ^ ( num & LOW_16_MASK );
    }
}
