@Override
public String createImage ( String origFilename ) {
    BlobKey blobKey = null;
    String modifiedURL = null;
    try {
        FileService fileService = FileServiceFactory.getFileService();
        AppEngineFile file1 = fileService.createNewBlobFile ( "image/png" );
        boolean lock = true;
        FileWriteChannel writeChannel = fileService.openWriteChannel ( file1, lock );
        OutputStream os = Channels.newOutputStream ( writeChannel );
        URL url = new URL ( escapeHTML ( origFilename ) );
        PngReader pngr     = new PngReader ( url.openStream() );
        PngWriter pngw = new PngWriter ( os, pngr.imgInfo );
        int channels = pngr.imgInfo.channels;
        if ( channels < 3 ) {
            throw new RuntimeException ( "Only for truecolour images" );
        }
        for ( int row = 0; row < pngr.imgInfo.rows; row++ ) {
            ImageLine l1 = pngr.readRow ( row );
            for ( int j = 0; j < pngr.imgInfo.cols; j++ ) {
                l1.scanline[j * channels] = 250;
            }
            pngw.writeRow ( l1 );
        }
        pngr.end();
        pngw.end();
        os.close();
        writeChannel.closeFinally();
        blobKey = fileService.getBlobKey ( file1 );
        ImagesService imagesService = ImagesServiceFactory.getImagesService();
        modifiedURL = imagesService.getServingUrl ( blobKey );
    } catch ( IOException e ) {
        e.printStackTrace();
    }
    return  modifiedURL;
}
public static final String escapeHTML ( String s ) {
    StringBuffer sb = new StringBuffer();
    int n = s.length();
    for ( int i = 0; i < n; i++ ) {
        char c = s.charAt ( i );
        switch ( c ) {
        case '|':
            sb.append ( "%7C" );
            break;
        case ' ':
            sb.append ( "%20" );
            break;
        default:
            sb.append ( c );
            break;
        }
    }
    return sb.toString();
}
