import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;
public class DragLayout implements LayoutManager, java.io.Serializable {
    public DragLayout() {
    }
    @Override
    public void addLayoutComponent ( String name, Component comp ) {}
    @Override
    public void removeLayoutComponent ( Component component ) {
    }
    @Override
    public Dimension minimumLayoutSize ( Container parent ) {
        synchronized ( parent.getTreeLock() ) {
            return preferredLayoutSize ( parent );
        }
    }
    @Override
    public Dimension preferredLayoutSize ( Container parent ) {
        synchronized ( parent.getTreeLock() ) {
            return getLayoutSize ( parent );
        }
    }
    private Dimension getLayoutSize ( Container parent ) {
        Insets parentInsets = parent.getInsets();
        int x = parentInsets.left;
        int y = parentInsets.top;
        int width = 0;
        int height = 0;
        for ( Component component : parent.getComponents() ) {
            if ( component.isVisible() ) {
                Point p = component.getLocation();
                Dimension d = component.getPreferredSize();
                x = Math.min ( x, p.x );
                y = Math.min ( y, p.y );
                width = Math.max ( width, p.x + d.width );
                height = Math.max ( height, p.y + d.height );
            }
        }
        if ( x < parentInsets.left ) {
            width += parentInsets.left - x;
        }
        if ( y < parentInsets.top ) {
            height += parentInsets.top - y;
        }
        width += parentInsets.right;
        height += parentInsets.bottom;
        Dimension d = new Dimension ( width, height );
        return d;
    }
    @Override
    public void layoutContainer ( Container parent ) {
        synchronized ( parent.getTreeLock() ) {
            Insets parentInsets = parent.getInsets();
            int x = parentInsets.left;
            int y = parentInsets.top;
            for ( Component component : parent.getComponents() ) {
                if ( component.isVisible() ) {
                    Point location = component.getLocation();
                    x = Math.min ( x, location.x );
                    y = Math.min ( y, location.y );
                }
            }
            x = ( x < parentInsets.left ) ? parentInsets.left - x : 0;
            y = ( y < parentInsets.top ) ? parentInsets.top - y : 0;
            for ( Component component : parent.getComponents() ) {
                if ( component.isVisible() ) {
                    Point p = component.getLocation();
                    Dimension d = component.getPreferredSize();
                    component.setBounds ( p.x + x, p.y + y, d.width, d.height );
                }
            }
        }
    }
    public String toString() {
        return "["
               + getClass().getName()
               + "]";
    }
    public static void main ( String[] args ) {
        ComponentMover cm = new ComponentMover();
        cm.setEdgeInsets ( new Insets ( -100, -100, -100, -100 ) );
        cm.setAutoLayout ( true );
        JPanel panel = new JPanel ( new DragLayout() );
        panel.setBorder ( new MatteBorder ( 10, 10, 10, 10, Color.YELLOW ) );
        createLabel ( cm, panel, "North", 150, 0 );
        createLabel ( cm, panel, "West", 0, 100 );
        createLabel ( cm, panel, "East", 300, 100 );
        createLabel ( cm, panel, "South", 150, 200 );
        createLabel ( cm, panel, "Center", 150, 100 );
        JFrame frame = new JFrame();
        frame.setDefaultCloseOperation ( JFrame.EXIT_ON_CLOSE );
        frame.add ( new JScrollPane ( panel ) );
        frame.pack();
        frame.setLocationRelativeTo ( null );
        frame.setVisible ( true );
    }
    public static void createLabel ( ComponentMover cm, JPanel panel, String text, int x, int y ) {
        JLabel label = new JLabel ( text );
        label.setOpaque ( true );
        label.setBackground ( Color.ORANGE );
        label.setLocation ( x, y );
        panel.add ( label );
        cm.registerComponent ( label );
    }
}
