import java.util.Random;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.atomic.AtomicInteger;
class Worker implements Runnable {
    private final AtomicInteger sum;
    private final CountDownLatch latch;
    public Worker ( AtomicInteger sum, CountDownLatch latch ) {
        this.sum = sum;
        this.latch = latch;
    }
    @Override
    public void run() {
        Random random = new Random();
        try {
            Thread.sleep ( random.nextInt ( 5000 ) + 5000 );
        } catch ( InterruptedException e ) {
            e.printStackTrace();
        }
        int x = random.nextInt ( 500 );
        System.out.println ( "Adding " + x + " to sum" );
        sum.addAndGet ( x );
        latch.countDown();
    }
}
class Program {
    public static void main ( String[] args ) {
        final int N = 39;
        AtomicInteger sum = new AtomicInteger();
        CountDownLatch latch = new CountDownLatch ( N );
        System.out.println ( "Starting " + N + " workers" );
        for ( int i = 0; i < N; i++ ) {
            Worker worker = new Worker ( sum, latch );
            new Thread ( worker ).start();
        }
        try {
            latch.await();
        } catch ( InterruptedException e ) {
            e.printStackTrace();
        }
        double average = ( double ) sum.get() / ( double ) N;
        System.out.println ( "    Sum: " + sum.get() );
        System.out.println ( "Workers: " + N );
        System.out.println ( "Average: " + average );
    }
}
