@SuppressWarnings ( "unchecked" )
public class FileWorker extends SwingWorker<Void, PropertyChangeEvent> {
    public static final String FILE_DELETED = StandardWatchEventKinds.ENTRY_DELETE.name();
    public static final String FILE_CREATED = StandardWatchEventKinds.ENTRY_CREATE.name();
    public static final String FILE_MODIFIED = StandardWatchEventKinds.ENTRY_MODIFY.name();
    private Path directory;
    private WatchService watcher;
    public FileWorker ( File file ) throws IOException {
        directory = file.toPath();
        watcher = FileSystems.getDefault().newWatchService();
        directory.register ( watcher, ENTRY_CREATE, ENTRY_DELETE, ENTRY_MODIFY );
    }
    @Override
    protected Void doInBackground() throws Exception {
        for ( ;; ) {
            WatchKey key;
            try {
                key = watcher.take();
            } catch ( InterruptedException x ) {
                return null;
            }
            for ( WatchEvent<?> event : key.pollEvents() ) {
                WatchEvent.Kind<?> kind = event.kind();
                if ( kind == OVERFLOW ) {
                    continue;
                }
                publish ( createChangeEvent ( ( WatchEvent<Path> ) event, key ) );
            }
            boolean valid = key.reset();
            if ( !valid ) {
                break;
            }
        }
        return null;
    }
    protected PropertyChangeEvent createChangeEvent ( WatchEvent<Path> event, WatchKey key ) {
        Path name = event.context();
        Path child = directory.resolve ( name );
        PropertyChangeEvent e = new PropertyChangeEvent ( this, event.kind().name(), null, child.toFile() );
        return e;
    }
    @Override
    protected void process ( List<PropertyChangeEvent> chunks ) {
        super.process ( chunks );
        for ( PropertyChangeEvent event : chunks ) {
            getPropertyChangeSupport().firePropertyChange ( event );
        }
    }
}
