public class XMLInputStream extends FilterInputStream {
    private static final int MIN_LENGTH = 2;
    StringBuilder red = new StringBuilder();
    StringBuilder pushBack = new StringBuilder();
    int given = 0;
    int pulled = 0;
    public XMLInputStream ( InputStream in ) {
        super ( in );
    }
    public int length() {
        try {
            StringBuilder s = read ( MIN_LENGTH );
            pushBack.append ( s );
            return s.length();
        } catch ( IOException ex ) {
            log.warning ( "Oops ", ex );
        }
        return 0;
    }
    private StringBuilder read ( int n ) throws IOException {
        boolean eof = false;
        StringBuilder s = new StringBuilder ( n );
        while ( s.length() < n && !eof ) {
            if ( pushBack.length() == 0 ) {
                eof = readIntoPushBack();
            }
            if ( pushBack.length() > 0 ) {
                s.append ( pushBack.charAt ( 0 ) );
                pushBack.deleteCharAt ( 0 );
            }
        }
        return s;
    }
    private boolean readIntoPushBack() throws IOException {
        boolean eof = false;
        int ch = in.read();
        if ( ch >= 0 ) {
            if ( ! ( pulled == 0 && isWhiteSpace ( ch ) ) ) {
                pulled += 1;
                if ( ch == '&' ) {
                    readAmpersand();
                } else {
                    pushBack.append ( ( char ) ch );
                }
            }
        } else {
            eof = true;
        }
        return eof;
    }
    private void readAmpersand() throws IOException {
        StringBuilder reference = new StringBuilder();
        int ch;
        for ( ch = in.read(); isAlphaNumeric ( ch ); ch = in.read() ) {
            reference.append ( ( char ) ch );
        }
        if ( ch == ';' ) {
            String code = hash ( reference );
            if ( code != null ) {
                pushBack.append ( code );
            } else {
                throw new IOException ( "Invalid/Unknown reference '&" + reference + ";'" );
            }
        } else {
            pushBack.append ( "&amp;" ).append ( reference ).append ( ( char ) ch );
        }
    }
    private void given ( CharSequence s, int wanted, int got ) {
        red.append ( s );
        given += got;
        log.finer ( "Given: [" + wanted + "," + got + "]-" + s );
    }
    @Override
    public int read() throws IOException {
        StringBuilder s = read ( 1 );
        given ( s, 1, 1 );
        return s.length() > 0 ? s.charAt ( 0 ) : -1;
    }
    @Override
    public int read ( byte[] data, int offset, int length ) throws IOException {
        int n = 0;
        StringBuilder s = read ( length );
        for ( int i = 0; i < Math.min ( length, s.length() ); i++ ) {
            data[offset + i] = ( byte ) s.charAt ( i );
            n += 1;
        }
        given ( s, length, n );
        return n > 0 ? n : -1;
    }
    @Override
    public String toString() {
        String s = red.toString();
        String h = "";
        if ( s.length() < 8 ) {
            Separator sep = new Separator ( " " );
            for ( int i = 0; i < s.length(); i++ ) {
                h += sep.sep() + Integer.toHexString ( s.charAt ( i ) );
            }
        }
        return "[" + given + "]-\"" + s + "\"" + ( h.length() > 0 ? " (" + h + ")" : "" );
    }
    private boolean isWhiteSpace ( int ch ) {
        switch ( ch ) {
        case ' ':
        case '\r':
        case '\n':
        case '\t':
            return true;
        }
        return false;
    }
    private boolean isAlphaNumeric ( int ch ) {
        return ( 'a' <= ch && ch <= 'z' )
               || ( 'A' <= ch && ch <= 'Z' )
               || ( '0' <= ch && ch <= '9' );
    }
    static String hash ( CharSequence s ) {
        Integer code = specials.get ( s.toString() );
        if ( code != null ) {
            return "&#" + code + ";";
        }
        return null;
    }
}
