import java.io.BufferedReader;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Stack;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import javax.xml.stream.*;
interface XMLNodeFoundNotifier {
    abstract void nodeFound ( StringBuilder node, XMLStackFilter withFilter );
}
abstract class XMLHandler implements XMLNodeFoundNotifier {
}
interface XMLStackFilter {
    abstract boolean isRelevant ( Stack fullPath );
}
class XMLRegexFilter implements XMLStackFilter {
    Pattern relevantExpression;
    XMLRegexFilter ( String filterRules ) {
        relevantExpression = Pattern.compile ( filterRules );
    }
    @Override
    public boolean isRelevant ( Stack fullPath ) {
        String stackPath = XMLParser.StackToString ( fullPath );
        Matcher m = relevantExpression.matcher ( stackPath );
        return  m.matches();
    }
}
public class XMLParser {
    HashMap<XMLStackFilter, XMLNodeFoundNotifier> filterHandler;
    HashMap<Integer, Integer> feedingStreams;
    Stack<HashMap> currentStack;
    String filePath;
    XMLParser() {
        currentStack   = new <HashMap>Stack();
        filterHandler  = new <XMLStackFilter, XMLNodeFoundNotifier> HashMap();
        feedingStreams = new <Integer, Integer>HashMap();
    }
    public void addFilterWithHandler ( XMLStackFilter f, XMLNodeFoundNotifier h ) {
        filterHandler.put ( f, h );
    }
    public void setFilePath ( String filePath ) {
        this.filePath = filePath;
    }
    public static String StackToString ( Stack<HashMap> s ) {
        int k = s.size();
        if ( k == 0 ) {
            return null;
        }
        StringBuilder out = new StringBuilder();
        out.append ( s.get ( 0 ).get ( "tag" ) );
        for ( int x = 1; x < k; ++x ) {
            HashMap node = s.get ( x );
            out.append ( '/' ).append ( node.get ( "tag" ) );
        }
        return out.toString();
    }
    private StringBuilder getChunk ( int from, int to ) throws Exception {
        int length = to - from;
        FileReader f = new FileReader ( filePath );
        BufferedReader br = new BufferedReader ( f );
        br.skip ( from );
        char[] readb = new char[length];
        br.read ( readb, 0, length );
        StringBuilder b = new StringBuilder();
        b.append ( readb );
        return b;
    }
    public HashMap XSRNode2HashMap ( XMLStreamReader xsr ) {
        HashMap h = new HashMap();
        ArrayList attributes = new ArrayList();
        for ( int i = 0; i < xsr.getAttributeCount(); i++ ) {
            String[] s = new String[2];
            s[0] = xsr.getAttributeName ( i ).toString();
            s[1] = xsr.getAttributeValue ( i );
            attributes.add ( s );
        }
        h.put ( "tag", xsr.getName() );
        h.put ( "attr", attributes );
        return h;
    }
    public void parse() throws Exception {
        FileReader f         = new FileReader ( filePath );
        XMLInputFactory xif  = XMLInputFactory.newInstance();
        XMLStreamReader xsr  = xif.createXMLStreamReader ( f );
        Location previousLoc = xsr.getLocation();
        while ( xsr.hasNext() ) {
            switch ( xsr.next() ) {
            case XMLStreamConstants.START_ELEMENT:
                currentStack.add ( XSRNode2HashMap ( xsr ) );
                for ( XMLStackFilter filter : filterHandler.keySet() ) {
                    if ( filter.isRelevant ( currentStack ) ) {
                        feedingStreams.put ( currentStack.hashCode(), new Integer ( previousLoc.getCharacterOffset() ) );
                    }
                }
                previousLoc = xsr.getLocation();
                break;
            case XMLStreamConstants.END_ELEMENT:
                Integer stream = null;
                if ( ( stream = feedingStreams.get ( currentStack.hashCode() ) ) != null ) {
                    for ( XMLStackFilter filter : filterHandler.keySet() ) {
                        if ( filter.isRelevant ( currentStack ) ) {
                            XMLNodeFoundNotifier h = filterHandler.get ( filter );
                            StringBuilder aChunk = getChunk ( stream.intValue(), xsr.getLocation().getCharacterOffset() );
                            h.nodeFound ( aChunk, filter );
                        }
                    }
                    feedingStreams.remove ( currentStack.hashCode() );
                }
                previousLoc = xsr.getLocation();
                currentStack.pop();
                break;
            default:
                break;
            }
        }
    }
}
