package com.broken.keyboard;
import android.app.ActionBar;
import android.app.Activity;
import android.app.Fragment;
import android.app.FragmentManager;
import android.os.Bundle;
import android.util.AttributeSet;
import android.util.Log;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.view.inputmethod.InputMethodManager;
import android.app.FragmentTransaction;
import android.app.ActionBar.Tab;
import android.content.Context;
public class KeyboardTestActivity extends Activity {
    public static class ActionBarTabManager {
        public static interface TabChangeListener {
            public abstract void onTabSelected ( String tag );
            public abstract void onTabSelectedPost ( String tag );
            public abstract void onTabReselected ( String tag );
            public abstract void onTabUnselected ( String tag );
        }
        Activity mActivity = null;
        ActionBar mActionBar = null;
        FragmentManager mFragmentManager = null;
        TabChangeListener mListener = null;
        View mContainer = null;
        Runnable mTabSelectedPostRunnable = null;
        public ActionBarTabManager ( Activity activity, int containerId, TabChangeListener listener ) {
            mActivity = activity;
            if ( mActivity == null ) {
                throw new RuntimeException ( "ActionBarTabManager requires a valid activity!" );
            }
            mActionBar = mActivity.getActionBar();
            if ( mActionBar == null ) {
                throw new RuntimeException ( "ActionBarTabManager requires an activity with an ActionBar." );
            }
            mContainer = activity.findViewById ( containerId );
            if ( mContainer == null ) {
                throw new RuntimeException ( "ActionBarTabManager requires a valid container (FrameLayout, preferably)." );
            }
            mListener = listener;
            mFragmentManager = mActivity.getFragmentManager();
            mActionBar.setNavigationMode ( ActionBar.NAVIGATION_MODE_TABS );
        }
        private class TabSelectedPostRunnable implements Runnable {
            String mTag = null;
            public TabSelectedPostRunnable ( String tag ) {
                mTag = tag;
            }
            @Override
            public void run() {
                if ( mListener != null ) {
                    mListener.onTabSelectedPost ( mTag );
                }
            }
        }
        private class TabListener implements ActionBar.TabListener {
            private Fragment mFragment = null;
            private String mTag = null;
            public TabListener ( Fragment fragment, String tag ) {
                mFragment = fragment;
                mTag = tag;
            }
            private boolean post ( Runnable runnable ) {
                return mContainer.post ( runnable );
            }
            @Override
            public void onTabReselected ( Tab tab, FragmentTransaction ft ) {
                if ( mListener != null ) {
                    mListener.onTabReselected ( mTag );
                }
            }
            @Override
            public void onTabSelected ( Tab tab, FragmentTransaction ft ) {
                mFragmentManager.beginTransaction()
                .replace ( mContainer.getId(), mFragment, mTag )
                .commit();
                if ( mListener != null ) {
                    mListener.onTabSelected ( mTag );
                }
                post ( new TabSelectedPostRunnable ( mTag ) );
            }
            @Override
            public void onTabUnselected ( Tab tab, FragmentTransaction ft ) {
                mFragmentManager.beginTransaction()
                .remove ( mFragment )
                .commit();
                if ( mListener != null ) {
                    mListener.onTabUnselected ( mTag );
                }
            }
        }
        public void addTab ( String title, Fragment fragment, String tag ) {
            mActionBar.addTab ( mActionBar.newTab()
                                .setText ( title )
                                .setTabListener ( new TabListener ( fragment, tag ) ) );
        }
    }
    public static class MyView extends View {
        public void toggleKeyboard() {
            ( ( InputMethodManager ) getContext().getSystemService ( Context.INPUT_METHOD_SERVICE ) ).toggleSoftInput ( 0, 0 );
        }
        public MyView ( Context context ) {
            super ( context );
        }
        public MyView ( Context context, AttributeSet attrs ) {
            super ( context, attrs );
        }
        public MyView ( Context context, AttributeSet attrs, int defStyle ) {
            super ( context, attrs, defStyle );
        }
        @Override
        public boolean onKeyDown ( int keyCode, KeyEvent event ) {
            Log.i ( "BDBG", "Key (" + keyCode + ") went down in the custom view!" );
            return true;
        }
        @Override
        public boolean onTouchEvent ( MotionEvent event ) {
            if ( ( event.getAction() & MotionEvent.ACTION_MASK ) == MotionEvent.ACTION_DOWN ) {
                toggleKeyboard();
            }
            return super.onTouchEvent ( event );
        }
    }
    public class MyFragment extends Fragment {
        @Override
        public View onCreateView ( LayoutInflater inflater, ViewGroup container,
                                   Bundle savedInstanceState ) {
            View v = inflater.inflate ( R.layout.my_fragment, container, false );
            return v;
        }
    }
    public class MyTabChangeListener implements ActionBarTabManager.TabChangeListener {
        public void onTabReselected ( String tag ) { }
        public void onTabSelected ( String tag ) { }
        public void onTabSelectedPost ( String tag ) {
            View view = findViewById ( R.id.myview );
            if ( view == null ) {
                throw new RuntimeException ( "Tab with tag of (\"" + tag + "\") should have the view we're looking for, but doesn't!" );
            }
            view.requestFocus();
        }
        public void onTabUnselected ( String tag ) { }
    }
    ActionBarTabManager mActionBarTabManager = null;
    MyTabChangeListener mListener = new MyTabChangeListener();
    @Override
    public void onCreate ( Bundle savedInstanceState ) {
        super.onCreate ( savedInstanceState );
        setContentView ( R.layout.main );
        mActionBarTabManager = new ActionBarTabManager ( this, R.id.actionbar_content, mListener );
        getActionBar().setDisplayShowTitleEnabled ( false );
        mActionBarTabManager.addTab ( "Tab 1", new MyFragment(), "Frag1" );
        mActionBarTabManager.addTab ( "Tab 2", new MyFragment(), "Frag2" );
    }
}
