import java.awt.*;
import java.util.Vector;
public class MultiBorderLayout extends BorderLayout {
    Vector northList = new Vector();
    Vector southList = new Vector();
    Vector westList = new Vector();
    Vector eastList = new Vector();
    Vector centerList = new Vector();
    public MultiBorderLayout() {
        super();
    }
    public MultiBorderLayout ( int hgap, int vgap ) {
        super ( hgap, vgap );
    }
    public void addLayoutComponent ( String name, Component comp ) {
        synchronized ( comp.getTreeLock() ) {
            if ( name == null ) {
                name = "Center";
            }
            if ( "Center".equals ( name ) ) {
                centerList.add ( comp );
            } else if ( "North".equals ( name ) ) {
                northList.insertElementAt ( comp, 0 );
            } else if ( "South".equals ( name ) ) {
                southList.add ( comp );
            } else if ( "East".equals ( name ) ) {
                eastList.add ( comp );
            } else if ( "West".equals ( name ) ) {
                westList.add ( comp );
            } else {
                throw new IllegalArgumentException ( "cannot add to layout: unknown constraint: " + name );
            }
        }
    }
    public void removeLayoutComponent ( Component comp ) {
        synchronized ( comp.getTreeLock() ) {
            southList.remove ( comp );
            northList.remove ( comp );
            centerList.remove ( comp );
            westList.remove ( comp );
            eastList.remove ( comp );
        }
    }
    public Dimension minimumLayoutSize ( Container target ) {
        synchronized ( target.getTreeLock() ) {
            Dimension dim = new Dimension ( 0, 0 );
            Component c;
            if ( eastList.size() > 0 ) {
                for ( int i = 0; i < eastList.size(); i++ ) {
                    c = ( Component ) eastList.get ( i );
                    if ( !c.isVisible() ) {
                        continue;
                    }
                    Dimension d = c.getMinimumSize();
                    dim.width += d.width + this.getHgap();
                    dim.height = Math.max ( d.height, dim.height );
                }
            }
            if ( westList.size() > 0 ) {
                for ( int i = 0; i < westList.size(); i++ ) {
                    c = ( Component ) westList.get ( i );
                    if ( !c.isVisible() ) {
                        continue;
                    }
                    Dimension d = c.getMinimumSize();
                    dim.width += d.width + this.getHgap();
                    dim.height = Math.max ( d.height, dim.height );
                }
            }
            if ( centerList.size() > 0 ) {
                for ( int i = 0; i < centerList.size(); i++ ) {
                    c = ( Component ) centerList.get ( i );
                    if ( !c.isVisible() ) {
                        continue;
                    }
                    Dimension d = c.getMinimumSize();
                    dim.width += d.width;
                    dim.height = Math.max ( d.height, dim.height );
                }
            }
            if ( northList.size() > 0 ) {
                for ( int i = 0; i < northList.size(); i++ ) {
                    c = ( Component ) northList.get ( i );
                    if ( !c.isVisible() ) {
                        continue;
                    }
                    Dimension d = c.getMinimumSize();
                    dim.width = Math.max ( d.width, dim.width );
                    dim.height += d.height + this.getVgap();
                }
            }
            if ( southList.size() > 0 ) {
                for ( int i = 0; i < southList.size(); i++ ) {
                    c = ( Component ) southList.get ( i );
                    if ( !c.isVisible() ) {
                        continue;
                    }
                    Dimension d = c.getMinimumSize();
                    dim.width = Math.max ( d.width, dim.width );
                    dim.height += d.height + this.getVgap();
                }
            }
            Insets insets = target.getInsets();
            dim.width += insets.left + insets.right;
            dim.height += insets.top + insets.bottom;
            return dim;
        }
    }
    public Dimension prefferedLayoutSize ( Container target ) {
        synchronized ( target.getTreeLock() ) {
            Dimension dim = new Dimension ( 0, 0 );
            Component c;
            if ( eastList.size() > 0 ) {
                for ( int i = 0; i < eastList.size(); i++ ) {
                    c = ( Component ) eastList.get ( i );
                    if ( !c.isVisible() ) {
                        continue;
                    }
                    Dimension d = c.getPreferredSize();
                    dim.width += d.width + this.getHgap();
                    dim.height = Math.max ( d.height, dim.height );
                }
            }
            if ( westList.size() > 0 ) {
                for ( int i = 0; i < westList.size(); i++ ) {
                    c = ( Component ) westList.get ( i );
                    if ( !c.isVisible() ) {
                        continue;
                    }
                    Dimension d = c.getPreferredSize();
                    dim.width += d.width + this.getHgap();
                    dim.height = Math.max ( d.height, dim.height );
                }
            }
            if ( centerList.size() > 0 ) {
                for ( int i = 0; i < centerList.size(); i++ ) {
                    c = ( Component ) centerList.get ( i );
                    if ( !c.isVisible() ) {
                        continue;
                    }
                    Dimension d = c.getPreferredSize();
                    dim.width += d.width;
                    dim.height = Math.max ( d.height, dim.height );
                }
            }
            if ( northList.size() > 0 ) {
                for ( int i = 0; i < northList.size(); i++ ) {
                    c = ( Component ) northList.get ( i );
                    if ( !c.isVisible() ) {
                        continue;
                    }
                    Dimension d = c.getPreferredSize();
                    dim.width = Math.max ( d.width, dim.width );
                    dim.height += d.height + this.getVgap();
                }
            }
            if ( southList.size() > 0 ) {
                for ( int i = 0; i < southList.size(); i++ ) {
                    c = ( Component ) southList.get ( i );
                    if ( !c.isVisible() ) {
                        continue;
                    }
                    Dimension d = c.getPreferredSize();
                    dim.width = Math.max ( d.width, dim.width );
                    dim.height += d.height + this.getVgap();
                }
            }
            Insets insets = target.getInsets();
            dim.width += insets.left + insets.right;
            dim.height += insets.top + insets.bottom;
            return dim;
        }
    }
    public void layoutContainer ( Container target ) {
        synchronized ( target.getTreeLock() ) {
            Insets insets = target.getInsets();
            int top = insets.top;
            int bottom = target.getHeight() - insets.bottom;
            int left = insets.left;
            int right = target.getWidth() - insets.right;
            Component c;
            if ( northList.size() > 0 ) {
                for ( int i = 0; i < northList.size(); i++ ) {
                    c = ( Component ) northList.get ( i );
                    if ( !c.isVisible() ) {
                        continue;
                    }
                    Dimension d = c.getPreferredSize();
                    c.setSize ( right - left, d.height );
                    c.setBounds ( left, top, right - left, c.getHeight() );
                    top += d.height;
                }
            }
            if ( southList.size() > 0 ) {
                for ( int i = 0; i < southList.size(); i++ ) {
                    c = ( Component ) southList.get ( i );
                    if ( !c.isVisible() ) {
                        continue;
                    }
                    Dimension d = c.getPreferredSize();
                    c.setSize ( right - left, d.height );
                    c.setBounds ( left, bottom - d.height, right - left, c.getHeight() );
                    bottom -= d.height;
                }
            }
            if ( eastList.size() > 0 ) {
                for ( int i = 0; i < eastList.size(); i++ ) {
                    c = ( Component ) eastList.get ( i );
                    if ( !c.isVisible() ) {
                        continue;
                    }
                    Dimension d = c.getPreferredSize();
                    c.setSize ( d.width, bottom - top );
                    c.setBounds ( right - d.width, top, c.getWidth(), bottom - top );
                    right -= d.width;
                }
            }
            if ( westList.size() > 0 ) {
                for ( int i = 0; i < westList.size(); i++ ) {
                    c = ( Component ) westList.get ( i );
                    if ( !c.isVisible() ) {
                        continue;
                    }
                    Dimension d = c.getPreferredSize();
                    c.setSize ( d.width, bottom - top );
                    c.setBounds ( left, top, c.getWidth(), bottom - top );
                    left += d.width;
                }
            }
            if ( centerList.size() > 0 ) {
                for ( int i = 0; i < centerList.size(); i++ ) {
                    c = ( Component ) centerList.get ( i );
                    if ( !c.isVisible() ) {
                        continue;
                    }
                    c.setBounds ( left, top, right - left, bottom - top );
                }
            }
        }
    }
}
