import java.io.*;
public class RandomFileOutputStream
    extends OutputStream {
    protected RandomAccessFile              randomFile;
    protected boolean                       sync;
    public RandomFileOutputStream ( String fnm ) throws IOException {
        this ( fnm, false );
    }
    public RandomFileOutputStream ( String fnm, boolean syn ) throws IOException {
        this ( new File ( fnm ), syn );
    }
    public RandomFileOutputStream ( File fil ) throws IOException {
        this ( fil, false );
    }
    public RandomFileOutputStream ( File fil, boolean syn ) throws IOException {
        super();
        File                                par;
        fil = fil.getAbsoluteFile();
        if ( ( par = fil.getParentFile() ) != null ) {
            IoUtil.createDir ( par );
        }
        randomFile = new RandomAccessFile ( fil, "rw" );
        sync = syn;
    }
    public void write ( int val ) throws IOException {
        randomFile.write ( val );
        if ( sync ) {
            randomFile.getFD().sync();
        }
    }
    public void write ( byte[] val ) throws IOException {
        randomFile.write ( val );
        if ( sync ) {
            randomFile.getFD().sync();
        }
    }
    public void write ( byte[] val, int off, int len ) throws IOException {
        randomFile.write ( val, off, len );
        if ( sync ) {
            randomFile.getFD().sync();
        }
    }
    public void flush() throws IOException {
        if ( sync ) {
            randomFile.getFD().sync();
        }
    }
    public void close() throws IOException {
        randomFile.close();
    }
    public long getFilePointer() throws IOException {
        return randomFile.getFilePointer();
    }
    public void setFilePointer ( long pos ) throws IOException {
        randomFile.seek ( pos );
    }
    public long getFileSize() throws IOException {
        return randomFile.length();
    }
    public void setFileSize ( long len ) throws IOException {
        randomFile.setLength ( len );
    }
    public FileDescriptor getFD() throws IOException {
        return randomFile.getFD();
    }
}
