static class TreeNodeEditor extends AbstractCellEditor implements TreeCellEditor {
    private static final long serialVersionUID = 1L;
    private JButton button1;
    private JButton button2;
    private JPanel panel1;
    private DefaultTreeCellRenderer defaultRenderer;
    private Object editorValue;
    public TreeNodeEditor() {
        super();
        panel1 = new JPanel();
        defaultRenderer = new DefaultTreeCellRenderer();
        button1 = new JButton ( "DELETE" );
        button1.setOpaque ( true );
        button1.setIcon ( new ImageIcon ( "trash.png" ) );
        button1.setCursor ( Cursor.getPredefinedCursor ( Cursor.HAND_CURSOR ) );
        button2 = new JButton ( "UPLOAD" );
        button2.setOpaque ( true );
        button2.setIcon ( new ImageIcon ( "app_up.png" ) );
        button2.setCursor ( Cursor.getPredefinedCursor ( Cursor.HAND_CURSOR ) );
        button2.setAction ( createAction ( "upload", "UPLOAD" ) );
        button1.setAction ( createAction ( "delete", "DELETE" ) );
        panel1.add ( defaultRenderer );
        panel1.add ( button1 );
        panel1.add ( button2 );
    }
    private Action createAction ( final String actionCommand, String display ) {
        Action action = new AbstractAction ( display ) {
            @Override
            public void actionPerformed ( ActionEvent e ) {
                stopEditing ( actionCommand );
            }
        };
        return action;
    }
    protected void stopEditing ( String actionCommand ) {
        editorValue = actionCommand;
        stopCellEditing();
    }
    @Override
    public Component getTreeCellEditorComponent ( JTree tree, Object value,
            boolean isSelected, boolean expanded, boolean leaf, int row ) {
        defaultRenderer.getTreeCellRendererComponent ( tree, value,
                isSelected, expanded, leaf, row, true );
        return panel1;
    }
    private void reset() {
        editorValue = null;
    }
    @Override
    public boolean shouldSelectCell ( EventObject anEvent ) {
        reset();
        if ( anEvent instanceof MouseEvent ) {
            redirect ( ( MouseEvent ) anEvent );
        }
        return false;
    }
    private void redirect ( final MouseEvent anEvent ) {
        SwingUtilities.invokeLater ( new Runnable() {
            @Override
            public void run() {
                MouseEvent ev = SwingUtilities.convertMouseEvent ( anEvent.getComponent(), anEvent, panel1 );
                panel1.dispatchEvent ( ev );
            }
        } );
    }
    @Override
    public Object getCellEditorValue() {
        return editorValue;
    }
}
