public class RetryTest {
    public class Retry implements TestRule {
        private int retryCount;
        public Retry ( int retryCount ) {
            this.retryCount = retryCount;
        }
        public Statement apply ( Statement base, Description description ) {
            return statement ( base, description );
        }
        private Statement statement ( final Statement base, final Description description ) {
            return new Statement() {
                @Override
                public void evaluate() throws Throwable {
                    Throwable caughtThrowable = null;
                    for ( int i = 0; i < retryCount; i++ ) {
                        try {
                            base.evaluate();
                            return;
                        } catch ( Throwable t ) {
                            caughtThrowable = t;
                            System.err.println ( description.getDisplayName() + ": run " + ( i + 1 ) + " failed" );
                        }
                    }
                    System.err.println ( description.getDisplayName() + ": giving up after " + retryCount + " failures" );
                    throw caughtThrowable;
                }
            };
        }
    }
    @Rule
    public Retry retry = new Retry ( 3 );
    @Test
    public void test1() {
    }
    @Test
    public void test2() {
        Object o = null;
        o.equals ( "foo" );
    }
}
