package com.joshuawitter.maps;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.ArrayList;
import android.app.Activity;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Matrix;
import android.graphics.Paint;
import android.graphics.Point;
import android.graphics.RectF;
import android.util.Log;
import com.google.android.maps.GeoPoint;
import com.google.android.maps.MapView;
import com.google.android.maps.Overlay;
import com.google.android.maps.Projection;
import com.joshuawitter.Base64;
public class UsersOverlay extends Overlay {
    private static final String TAG = "Josh-UsersOverlay";
    private Activity m_caller;
    private UserOnMap m_selectedUser;
    private ArrayList<UserOnMap> m_Users;
    public void setUsers ( ArrayList<UserOnMap> users ) {
        this.m_Users = users;
    }
    public UsersOverlay ( Activity caller ) {
        super();
        m_caller = caller;
        m_Users = new ArrayList<UserOnMap> ( 0 );
    }
    @Override
    public boolean onTap ( GeoPoint geoPoint, MapView mapView ) {
        UserOnMap user = getTappedUser ( mapView, geoPoint );
        if ( null != user ) {
            m_selectedUser = ( m_selectedUser != user ) ? user : null;
        } else {
            m_selectedUser = null;
        }
        return super.onTap ( geoPoint, mapView );
    }
    @Override
    public void draw ( Canvas canvas, MapView mapView, boolean shadow ) {
        drawUserIcons ( canvas, mapView );
        super.draw ( canvas, mapView, shadow );
    }
    private void drawUserIcons ( Canvas canvas, MapView mapView ) {
        for ( UserOnMap user : m_Users ) {
            drawUserIcon ( canvas, mapView, user );
        }
        if ( null != m_selectedUser ) {
            drawUserBubble ( canvas, mapView, m_selectedUser );
        }
    }
    private void drawUserBubble ( Canvas canvas, MapView mapView, UserOnMap user ) {
        Projection projection = mapView.getProjection();
        Point pt = new Point();
        projection.toPixels ( user.getLocation(), pt );
        Bitmap bubbleBitmap = BitmapFactory
                              .decodeResource ( m_caller.getResources(),
                                                R.drawable.user_bubble );
        int height = bubbleBitmap.getHeight();
        int width = bubbleBitmap.getWidth();
        int xValue = pt.x - ( width / 2 );
        int yValue = pt.y - height - 20;
        canvas.drawBitmap ( bubbleBitmap, xValue, yValue, null );
        Paint paint = new Paint();
        paint.setStyle ( Paint.Style.FILL );
        paint.setAntiAlias ( true );
        paint.setTextSize ( 24 );
        canvas.drawText ( "Current Location", xValue + 30, yValue + 33, paint );
    }
    private void drawUserIcon ( Canvas canvas, MapView mapView,
                                UserOnMap user ) {
        Projection projection = mapView.getProjection();
        Point pt = new Point();
        projection.toPixels ( user.getLocation(), pt );
        Bitmap icon = BitmapFactory.decodeResource ( m_caller.getResources(),
                      user.getIconResource() );
        int height = icon.getHeight();
        int width = icon.getWidth();
        int xValue = pt.x - ( width / 2 );
        int yValue = pt.y - height;
        canvas.drawBitmap ( icon, xValue, yValue, null );
    }
    private UserOnMap getTappedUser ( MapView mapView, GeoPoint tapPoint ) {
        Point tapScreenCoordinates = new Point();
        mapView.getProjection().toPixels ( tapPoint, tapScreenCoordinates );
        for ( UserOnMap user : m_Users ) {
            if ( user == m_selectedUser ) {
                if ( hitUserBubble ( mapView, tapPoint, user ) ) {
                    Log.d ( TAG, "User " + user.getLastName() + " was tapped (bubble)" );
                    return user;
                }
            }
            if ( hitUserIcon ( mapView, tapPoint, user ) ) {
                Log.d ( TAG, "User " + user.getLastName() + " was tapped (icon)" );
                return user;
            }
        }
        return null;
    }
    private boolean hitUserIcon ( MapView mapView, GeoPoint tapPoint,
                                  UserOnMap user ) {
        Point tapScreenCoordinates = new Point();
        mapView.getProjection().toPixels ( tapPoint, tapScreenCoordinates );
        Point iconScreenCoordinates = new Point();
        mapView.getProjection().toPixels ( user.getLocation(),
                                           iconScreenCoordinates );
        Bitmap icon = BitmapFactory.decodeResource ( m_caller.getResources(),
                      user.getIconResource() );
        int xValue = 50;
        int yValue = 50;
        RectF iconRectangle = new RectF();
        iconRectangle.set ( -xValue, -yValue, xValue, yValue );
        iconRectangle.offset ( iconScreenCoordinates.x, iconScreenCoordinates.y );
        return ( iconRectangle.contains ( tapScreenCoordinates.x,
                                          tapScreenCoordinates.y ) );
    }
    private boolean hitUserBubble ( MapView mapView, GeoPoint tapPoint,
                                    UserOnMap user ) {
        Point tapScreenCoordinates = new Point();
        mapView.getProjection().toPixels ( tapPoint, tapScreenCoordinates );
        Point iconScreenCoordinates = new Point();
        mapView.getProjection().toPixels ( user.getLocation(),
                                           iconScreenCoordinates );
        Bitmap bubble = BitmapFactory.decodeResource ( m_caller.getResources(),
                        R.drawable.backup_map_current_location );
        int xValue = bubble.getWidth() * 2;
        int yValue = bubble.getHeight() * 2;
        RectF iconRectangle = new RectF();
        iconRectangle.set ( -xValue, -yValue, xValue, yValue );
        iconRectangle.offset ( iconScreenCoordinates.x, iconScreenCoordinates.y );
        return ( iconRectangle.contains ( tapScreenCoordinates.x,
                                          tapScreenCoordinates.y ) );
    }
}
