public class NettyServer implements Processor {
    private final ChannelGroup channelGroup = new DefaultChannelGroup();
    private NioServerSocketChannelFactory serverSocketChannelFactory = null;
    private final ExecutorService executor = Executors.newCachedThreadPool();
    private String listenAddress = "0.0.0.0";
    private int listenPort = 51501;
    @Override
    public void process ( Exchange exchange ) throws Exception {
        byte[] bytes = ( byte[] ) exchange.getIn().getBody();
        sendMessage ( bytes );
    }
    public synchronized void sendMessage ( byte[] message ) {
        ChannelBuffer cb = ChannelBuffers.copiedBuffer ( message );
        this.channelGroup.write ( cb );
    }
    private class NettyServerHandler extends SimpleChannelUpstreamHandler {
        @Override
        public void channelOpen ( ChannelHandlerContext ctx, ChannelStateEvent e ) throws Exception {
            super.channelOpen ( ctx, e );
            NettyServer.this.channelGroup.add ( e.getChannel() );
        }
        @Override
        public void channelConnected ( ChannelHandlerContext ctx, ChannelStateEvent e ) throws Exception {
            super.channelConnected ( ctx, e );
        }
        @Override
        public void messageReceived ( ChannelHandlerContext ctx, MessageEvent e ) {
        }
        @Override
        public void exceptionCaught ( ChannelHandlerContext ctx, ExceptionEvent e ) {
        }
    }
    private class PublishSocketServerPipelineFactory implements ChannelPipelineFactory {
        @Override
        public ChannelPipeline getPipeline() throws Exception {
            return Channels.pipeline ( new NettyServerHandler() );
        }
    }
    public void init() {
        try {
            this.serverSocketChannelFactory = new NioServerSocketChannelFactory ( this.executor, this.executor );
            final ServerBootstrap serverBootstrap = new ServerBootstrap ( this.serverSocketChannelFactory );
            serverBootstrap.setPipelineFactory ( new PublishSocketServerPipelineFactory() );
            serverBootstrap.setOption ( "reuseAddress", true );
            final InetSocketAddress listenSocketAddress = new InetSocketAddress ( this.listenAddress, this.listenPort );
            this.channelGroup.add ( serverBootstrap.bind ( listenSocketAddress ) );
        } catch ( Exception e ) {
        }
    }
    public void destroy() {
        try {
            this.channelGroup.close();
            this.serverSocketChannelFactory.releaseExternalResources();
            this.executor.shutdown();
        } catch ( Exception e ) {
        }
    }
    public void setListenAddress ( String listenAddress ) {
        this.listenAddress = listenAddress;
    }
    public void setListenPort ( int listenPort ) {
        this.listenPort = listenPort;
    }
