import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.junit.Test;
public class SimpleLoggerTest {
    private void logMessages ( Logger logger ) {
        logger.warning ( getLoggerName ( logger ) + ": warning message" );
        logger.info ( getLoggerName ( logger ) + ": info message" );
        logger.fine ( getLoggerName ( logger ) + ": fine message" );
    }
    private String getLoggerName ( Logger logger ) {
        String loggerName = logger.getName();
        if ( loggerName.isEmpty() ) {
            return "[root logger]";
        }
        return loggerName;
    }
    private void listHandlerLevels ( Logger logger ) {
        for ( Handler handler : logger.getHandlers() ) {
            logger.info ( getLoggerName ( logger ) + ": handler level = " + handler.getLevel() );
        }
        Logger parentLogger = logger.getParent();
        if ( null != parentLogger ) {
            for ( Handler handler : parentLogger.getHandlers() ) {
                logger.info ( "parent logger handler (" + getLoggerName ( parentLogger ) + "): handler level = " + handler.getLevel() );
            }
        }
    }
    private void setHandlerLevels ( Logger logger, Level level ) {
        for ( Handler handler : logger.getHandlers() ) {
            handler.setLevel ( level );
        }
        Logger parentLogger = logger.getParent();
        if ( null != parentLogger ) {
            for ( Handler handler : parentLogger.getHandlers() ) {
                handler.setLevel ( level );
            }
        }
    }
    @Test
    public void testLoggingLevel() {
        Logger myLogger = Logger.getLogger ( SimpleLoggerTest.class.getName() );
        Logger rootLogger = myLogger.getParent();
        listHandlerLevels ( myLogger );
        listHandlerLevels ( rootLogger );
        logMessages ( myLogger );
        logMessages ( rootLogger );
        myLogger.setLevel ( Level.ALL );
        rootLogger.setLevel ( Level.WARNING );
        listHandlerLevels ( myLogger );
        listHandlerLevels ( rootLogger );
        logMessages ( myLogger );
        logMessages ( rootLogger );
        setHandlerLevels ( myLogger, Level.FINE );
        listHandlerLevels ( myLogger );
        listHandlerLevels ( rootLogger );
        logMessages ( myLogger );
        logMessages ( rootLogger );
    }
}
