using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
namespace TickPicker {
    class TickPicker
    {
        double tickLow;
        double tickHigh;
        double tickUnit;
        double tickCount;
        static double[] targetUnits = { 1, 2, 2.5, 5, 10 };
        static int[] roundFactors = { 1, 5, 4, 2, 1 };
        static double[] targetLogs = arrayLog ( targetUnits );
        TickPicker ( double low, double high, int tickCountGoal ) {
            double range = high - low;
            double divGoal = range / ( tickCountGoal - 2 );
            double logGoal = Math.Log10 ( divGoal );
            double powerFactor = Math.Floor ( logGoal );
            logGoal = logGoal - powerFactor;
            int closestIndex = findClosest ( targetLogs, logGoal );
            tickUnit = targetUnits[closestIndex] * ( Math.Pow ( 10, powerFactor ) );
            int roundFactor = roundFactors[closestIndex];
            tickLow = Math.Floor ( low / tickUnit / roundFactor ) * tickUnit * roundFactor;
            tickHigh = Math.Ceiling ( high / tickUnit / roundFactor ) * tickUnit * roundFactor;
            tickCount = ( tickHigh - tickLow ) / tickUnit;
        }
        static double[] arrayLog ( double[] inputs ) {
            double[] retVal = new double[inputs.Length];
            int x = 0;
            foreach ( double input in inputs ) {
                retVal[x] = Math.Log10 ( inputs[x] );
                x++;
            }
            return retVal;
        }
        static int findClosest ( double[] candidates, double input ) {
            int low = 0;
            for ( int i = 1; i < candidates.Length && input > candidates[i]; i++ ) {
                low = i;
            }
            int high = low + 1;
            return candidates[high] - input < input - candidates[low] ? high : low;
        }
        static void testPicker ( double low, double high, int tickCountGoal ) {
            TickPicker picker = new TickPicker ( low, high, tickCountGoal );
            System.Console.WriteLine ( "[{0}:{1}]/{2} gives [{3}:{4}] with {5} ticks of {6} units each.", low, high, tickCountGoal, picker.tickLow, picker.tickHigh, picker.tickCount, picker.tickUnit );
        }
        static void Main ( string[] args ) {
            testPicker ( 4.7, 39.2, 13 );
            testPicker ( 4.7, 39.2, 16 );
            testPicker ( 4.7, 39.2, 19 );
            testPicker ( 4.7, 39.2, 21 );
            testPicker ( 4.7, 39.2, 24 );
            testPicker ( 1967, 2011, 20 );
            testPicker ( 1967, 2011, 10 );
            testPicker ( 2.71, 3.14, 5 );
            testPicker ( .0568, 13, 20 );
        }
    }
}
