public class LdapAuthenticationToken extends AbstractAuthenticationToken {
    private static final long serialVersionUID = -5040340622950665401L;
    private Authentication auth;
    transient private InitialLdapContext context;
    private List<GrantedAuthority> authorities = new ArrayList<GrantedAuthority>();
    public LdapAuthenticationToken ( Authentication auth, GrantedAuthority defaultAuthority ) {
        this.auth = auth;
        if ( auth.getAuthorities() != null ) {
            this.authorities.addAll ( Arrays.asList ( auth.getAuthorities() ) );
        }
        if ( defaultAuthority != null ) {
            this.authorities.add ( defaultAuthority );
        }
        super.setAuthenticated ( true );
    }
    public LdapAuthenticationToken ( Authentication auth, String defaultAuthority ) {
        this ( auth, new GrantedAuthorityImpl ( defaultAuthority ) );
    }
    public GrantedAuthority[] getAuthorities() {
        GrantedAuthority[] authoritiesArray = this.authorities.toArray ( new GrantedAuthority[0] );
        return authoritiesArray;
    }
    public void addAuthority ( GrantedAuthority authority ) {
        this.authorities.add ( authority );
    }
    public Object getCredentials() {
        return auth.getCredentials();
    }
    public Object getPrincipal() {
        return auth.getPrincipal();
    }
    public InitialLdapContext getContext() {
        return context;
    }
    public void setContext ( InitialLdapContext context ) {
        this.context = context;
    }
}
