public class TaskPaneView extends JScrollPane {
    private transient ExplorerManager manager;
    JXTaskPaneContainer taskpanecontainer = new JXTaskPaneContainer();
    transient Listener managerListener;
    transient PropertyChangeListener wlpc;
    transient boolean listenerActive;
    Font labelFont = new Font ( "Segoe UI", Font.BOLD, 14 );
    Painter backgroundPainter = new MattePainter ( Color.white );
    public TaskPaneView() {
        setViewportView ( taskpanecontainer );
    }
    public void setBackground ( Painter background ) {
        this.backgroundPainter = background;
    }
    public void setLabelFont ( Font labelFont ) {
        this.labelFont = labelFont;
    }
    @Override
    public void addNotify() {
        super.addNotify();
        ExplorerManager em = ExplorerManager.find ( this );
        if ( em != manager ) {
            if ( manager != null ) {
                manager.removePropertyChangeListener ( wlpc );
            }
            manager = em;
            manager.addPropertyChangeListener ( wlpc = WeakListeners.propertyChange ( managerListener, manager ) );
            Node root = manager.getExploredContext();
            setRootNode ( root );
        } else {
            if ( !listenerActive && ( manager != null ) ) {
                manager.addPropertyChangeListener ( wlpc = WeakListeners.propertyChange ( managerListener, manager ) );
            }
        }
    }
    @Override
    public void removeNotify() {
        super.removeNotify();
        listenerActive = false;
        if ( manager != null ) {
            manager.removePropertyChangeListener ( wlpc );
        }
    }
    private void setRootNode ( Node root ) {
        taskpanecontainer.removeAll();
        System.out.println ( "root node set " + root );
        Node[] children = root.getChildren().getNodes();
        for ( int i = 0; i < children.length; i++ ) {
            Node node = children[i];
            JXTaskPane taskPane = new JXTaskPane();
            taskPane.setName ( node.getName() );
            taskPane.setCollapsed ( true );
            taskPane.setTitle ( node.getDisplayName() );
            taskPane.setIcon ( new ImageIcon ( node.getIcon ( BeanInfo.ICON_COLOR_16x16 ) ) );
            Action [] actions = node.getActions ( true );
            for ( int j = 0; j < actions.length; j++ ) {
                Action action = actions[j];
                taskPane.add ( action );
            }
            taskpanecontainer.add ( taskPane );
        }
    }
    private final class Listener implements PropertyChangeListener {
        public void propertyChange ( PropertyChangeEvent evt ) {
            if ( ExplorerManager.PROP_EXPLORED_CONTEXT.equals ( evt.getPropertyName() ) ) {
                setRootNode ( manager.getExploredContext() );
                return;
            }
        }
    }
}
