import java.io.Closeable;
import java.io.IOException;
import java.util.LinkedList;
import java.util.List;
public abstract class AutoFileCloser {
    protected abstract void doWork() throws Throwable;
    private List<Closeable> closeables_ = new LinkedList<Closeable>();
    protected final <T extends Closeable> T autoClose ( T closeable ) {
        closeables_.add ( 0, closeable );
        return closeable;
    }
    public AutoFileCloser() {
        Throwable pending = null;
        try {
            doWork();
        } catch ( Throwable throwable ) {
            pending = throwable;
        } finally {
            for ( Closeable closeable : closeables_ ) {
                if ( closeable != null ) {
                    try {
                        closeable.close();
                    } catch ( Throwable throwable ) {
                        if ( pending == null ) {
                            pending = throwable;
                        }
                    }
                }
            }
            if ( pending != null ) {
                if ( pending instanceof RuntimeException ) {
                    throw ( RuntimeException ) pending;
                } else {
                    throw new RuntimeException ( pending );
                }
            }
        }
    }
}
