public class Hello implements DiscoveryListener, PipeMsgListener {
    public static void main ( String[] args ) throws PeerGroupException, IOException {
        Logger.getLogger ( "net.jxta" ).setLevel ( Level.ALL );
        int port = 9000 + new Random().nextInt ( 100 );
        Hello hello = new Hello ( port );
        hello.start();
        hello.fetch_advertisements();
    }
    private String peer_name;
    private PeerID peer_id;
    private File conf;
    private NetworkManager manager;
    public Hello ( int port ) {
        peer_name = "Peer " + new Random().nextInt ( 1000000 );
        peer_id = IDFactory.newPeerID ( PeerGroupID.defaultNetPeerGroupID, peer_name.getBytes() );
        conf = new File ( "." + System.getProperty ( "file.separator" ) + peer_name );
        try {
            manager = new NetworkManager (
                NetworkManager.ConfigMode.ADHOC,
                peer_name, conf.toURI() );
        } catch ( IOException e ) {
            e.printStackTrace();
        }
        NetworkConfigurator configurator;
        try {
            configurator = manager.getConfigurator();
            configurator.setTcpPort ( port );
            configurator.setTcpEnabled ( true );
            configurator.setTcpIncoming ( true );
            configurator.setTcpOutgoing ( true );
            configurator.setUseMulticast ( true );
            configurator.setPeerID ( peer_id );
        } catch ( IOException e ) {
            e.printStackTrace();
        }
    }
    private static final String subgroup_name = "Make sure this is spelled the same everywhere";
    private static final String subgroup_desc = "...";
    private static final PeerGroupID subgroup_id = IDFactory.newPeerGroupID ( PeerGroupID.defaultNetPeerGroupID, subgroup_name.getBytes() );
    private static final String unicast_name = "This must be spelled the same too";
    private static final String multicast_name = "Or else you will get the wrong PipeID";
    private static final String service_name = "And dont forget it like i did a million times";
    private PeerGroup subgroup;
    private PipeService pipe_service;
    private PipeID unicast_id;
    private PipeID multicast_id;
    private PipeID service_id;
    private DiscoveryService discovery;
    private ModuleSpecAdvertisement mdadv;
    public void start() throws PeerGroupException, IOException {
        PeerGroup net_group = manager.startNetwork();
        ModuleImplAdvertisement mAdv = null;
        try {
            mAdv = net_group.getAllPurposePeerGroupImplAdvertisement();
        } catch ( Exception ex ) {
            System.err.println ( ex.toString() );
        }
        subgroup = net_group.newGroup ( subgroup_id, mAdv, subgroup_name, subgroup_desc );
        if ( Module.START_OK != subgroup.startApp ( new String[0] ) ) {
            System.err.println ( "Cannot start child peergroup" );
        }
        unicast_id = IDFactory.newPipeID ( subgroup.getPeerGroupID(), unicast_name.getBytes() );
        multicast_id = IDFactory.newPipeID ( subgroup.getPeerGroupID(), multicast_name.getBytes() );
        pipe_service = subgroup.getPipeService();
        pipe_service.createInputPipe ( get_advertisement ( unicast_id, false ), this );
        pipe_service.createInputPipe ( get_advertisement ( multicast_id, true ), this );
        discovery = subgroup.getDiscoveryService();
        discovery.addDiscoveryListener ( this );
        ModuleClassAdvertisement mcadv = ( ModuleClassAdvertisement )
                                         AdvertisementFactory.newAdvertisement ( ModuleClassAdvertisement.getAdvertisementType() );
        mcadv.setName ( "STACK-OVERFLOW:HELLO" );
        mcadv.setDescription ( "Tutorial example to use JXTA module advertisement Framework" );
        ModuleClassID mcID = IDFactory.newModuleClassID();
        mcadv.setModuleClassID ( mcID );
        discovery.publish ( mcadv );
        discovery.remotePublish ( mcadv );
        mdadv = ( ModuleSpecAdvertisement )
                AdvertisementFactory.newAdvertisement ( ModuleSpecAdvertisement.getAdvertisementType() );
        mdadv.setName ( "STACK-OVERFLOW:HELLO" );
        mdadv.setVersion ( "Version 1.0" );
        mdadv.setCreator ( "sun.com" );
        mdadv.setModuleSpecID ( IDFactory.newModuleSpecID ( mcID ) );
        mdadv.setSpecURI ( "http://www.jxta.org/Ex1" );
        service_id = IDFactory.newPipeID ( subgroup.getPeerGroupID(), service_name.getBytes() );
        PipeAdvertisement pipeadv = get_advertisement ( service_id, false );
        mdadv.setPipeAdvertisement ( pipeadv );
        discovery.publish ( mdadv );
        discovery.remotePublish ( mdadv );
        pipe_service.createInputPipe ( pipeadv, this );
    }
    private static PipeAdvertisement get_advertisement ( PipeID id, boolean is_multicast ) {
        PipeAdvertisement adv = ( PipeAdvertisement ) AdvertisementFactory.
                                newAdvertisement ( PipeAdvertisement.getAdvertisementType() );
        adv.setPipeID ( id );
        if ( is_multicast ) {
            adv.setType ( PipeService.PropagateType );
        } else {
            adv.setType ( PipeService.UnicastType );
        }
        adv.setName ( "This however" );
        adv.setDescription ( "does not really matter" );
        return adv;
    }
    @Override public void discoveryEvent ( DiscoveryEvent event ) {
        String found_peer_id = "urn:jxta:" + event.getSource().toString().substring ( 7 );
        send_to_peer ( "Hello", found_peer_id );
    }
    private void send_to_peer ( String message, String found_peer_id ) {
        PipeAdvertisement adv = get_advertisement ( unicast_id, false );
        Set<PeerID> ps = new HashSet<PeerID>();
        try {
            ps.add ( ( PeerID ) IDFactory.fromURI ( new URI ( found_peer_id ) ) );
        } catch ( URISyntaxException e ) {
            e.printStackTrace();
        }
        OutputPipe sender = null;
        try {
            sender = pipe_service.createOutputPipe ( adv, ps, 10000 );
        } catch ( IOException e ) {
            e.printStackTrace();
        }
        Message msg = new Message();
        MessageElement fromElem = null;
        MessageElement msgElem = null;
        try {
            fromElem = new ByteArrayMessageElement ( "From", null, peer_id.toString().getBytes ( "ISO-8859-1" ), null );
            msgElem = new ByteArrayMessageElement ( "Msg", null, message.getBytes ( "ISO-8859-1" ), null );
        } catch ( UnsupportedEncodingException e ) {
            e.printStackTrace();
        }
        msg.addMessageElement ( fromElem );
        msg.addMessageElement ( msgElem );
        try {
            sender.send ( msg );
        } catch ( IOException e ) {
            e.printStackTrace();
        }
    }
    @Override public void pipeMsgEvent ( PipeMsgEvent event ) {
        try {
            Message msg = event.getMessage();
            byte[] msgBytes = msg.getMessageElement ( "Msg" ).getBytes ( true );
            byte[] fromBytes = msg.getMessageElement ( "From" ).getBytes ( true );
            String from = new String ( fromBytes );
            String message = new String ( msgBytes );
            System.out.println ( message + " says " + from );
        } catch ( Exception e ) {
            e.printStackTrace();
        }
    }
    private void fetch_advertisements() {
        new Thread ( "fetch advertisements thread" ) {
            public void run() {
                while ( true ) {
                    discovery.getRemoteAdvertisements ( null, DiscoveryService.ADV, "Name", "STACK-OVERFLOW:HELLO", 1, null );
                    try {
                        sleep ( 10000 );
                    } catch ( InterruptedException e ) {}
                }
            }
        } .start();
    }
}
