public class AnnotationServletModelAttributeResolver extends ServletModelAttributeMethodProcessor {
    private ConcurrentMap<Class<?>, Map<String, String>> definitionsCache = Maps.newConcurrentMap();
    public AnnotationServletModelAttributeResolver ( boolean annotationNotRequired ) {
        super ( annotationNotRequired );
    }
    @Override
    public boolean supportsParameter ( MethodParameter parameter ) {
        if ( parameter.getParameterType().isAnnotationPresent ( SupportsAnnotationParameterResolution.class ) ) {
            return true;
        }
        return false;
    }
    @Override
    protected void bindRequestParameters ( WebDataBinder binder, NativeWebRequest request ) {
        ServletRequest servletRequest = request.getNativeRequest ( ServletRequest.class );
        ServletRequestDataBinder servletBinder = ( ServletRequestDataBinder ) binder;
        bind ( servletRequest, servletBinder );
    }
    @SuppressWarnings ( "unchecked" )
    public void bind ( ServletRequest request, ServletRequestDataBinder binder ) {
        Map<String, ?> propertyValues = parsePropertyValues ( request, binder );
        MutablePropertyValues mpvs = new MutablePropertyValues ( propertyValues );
        MultipartRequest multipartRequest = WebUtils.getNativeRequest ( request, MultipartRequest.class );
        if ( multipartRequest != null ) {
            bindMultipart ( multipartRequest.getMultiFileMap(), mpvs );
        }
        String attr = HandlerMapping.URI_TEMPLATE_VARIABLES_ATTRIBUTE;
        mpvs.addPropertyValues ( ( Map<String, String> ) request.getAttribute ( attr ) );
        binder.bind ( mpvs );
    }
    private Map<String, ?> parsePropertyValues ( ServletRequest request, ServletRequestDataBinder binder ) {
        Map<String, Object> params = Maps.newTreeMap();
        Assert.notNull ( request, "Request must not be null" );
        Enumeration<?> paramNames = request.getParameterNames();
        Map<String, String> parameterMappings = getParameterMappings ( binder );
        while ( paramNames != null && paramNames.hasMoreElements() ) {
            String paramName = ( String ) paramNames.nextElement();
            String[] values = request.getParameterValues ( paramName );
            String fieldName = parameterMappings.get ( paramName );
            if ( fieldName == null ) {
                fieldName = paramName;
            }
            if ( values == null || values.length == 0 ) {
            } else if ( values.length > 1 ) {
                params.put ( fieldName, values );
            } else {
                params.put ( fieldName, values[0] );
            }
        }
        return params;
    }
    private Map<String, String> getParameterMappings ( ServletRequestDataBinder binder ) {
        Class<?> targetClass = binder.getTarget().getClass();
        Map<String, String> map = definitionsCache.get ( targetClass );
        if ( map == null ) {
            Field[] fields = targetClass.getDeclaredFields();
            map = Maps.newHashMapWithExpectedSize ( fields.length );
            for ( Field field : fields ) {
                CommandParameter annotation = field.getAnnotation ( CommandParameter.class );
                if ( annotation != null && !annotation.value().isEmpty() ) {
                    map.put ( annotation.value(), field.getName() );
                }
            }
            definitionsCache.putIfAbsent ( targetClass, map );
            return map;
        } else {
            return map;
        }
    }
    protected void bindMultipart ( Map<String, List<MultipartFile>> multipartFiles, MutablePropertyValues mpvs ) {
        for ( Map.Entry<String, List<MultipartFile>> entry : multipartFiles.entrySet() ) {
            String key = entry.getKey();
            List<MultipartFile> values = entry.getValue();
            if ( values.size() == 1 ) {
                MultipartFile value = values.get ( 0 );
                if ( !value.isEmpty() ) {
                    mpvs.add ( key, value );
                }
            } else {
                mpvs.add ( key, values );
            }
        }
    }
}
