public class PdfMergeHelper {
    public static void concatPDFs ( List<InputStream> streamOfPDFFiles, OutputStream outputStream, boolean paginate ) {
        Document document = new Document();
        try {
            List<InputStream> pdfs = streamOfPDFFiles;
            List<PdfReader> readers = new ArrayList<PdfReader>();
            int totalPages = 0;
            Iterator<InputStream> iteratorPDFs = pdfs.iterator();
            while ( iteratorPDFs.hasNext() ) {
                InputStream pdf = iteratorPDFs.next();
                PdfReader pdfReader = new PdfReader ( pdf );
                readers.add ( pdfReader );
                totalPages += pdfReader.getNumberOfPages();
            }
            PdfWriter writer = PdfWriter.getInstance ( document, outputStream );
            document.open();
            BaseFont bf = BaseFont.createFont ( BaseFont.HELVETICA, BaseFont.CP1252, BaseFont.NOT_EMBEDDED );
            PdfContentByte cb = writer.getDirectContent();
            PdfImportedPage page;
            int currentPageNumber = 0;
            int pageOfCurrentReaderPDF = 0;
            Iterator<PdfReader> iteratorPDFReader = readers.iterator();
            while ( iteratorPDFReader.hasNext() ) {
                PdfReader pdfReader = iteratorPDFReader.next();
                while ( pageOfCurrentReaderPDF < pdfReader.getNumberOfPages() ) {
                    document.newPage();
                    pageOfCurrentReaderPDF++;
                    currentPageNumber++;
                    page = writer.getImportedPage ( pdfReader, pageOfCurrentReaderPDF );
                    cb.addTemplate ( page, 0, 0 );
                    if ( paginate ) {
                        cb.beginText();
                        cb.setFontAndSize ( bf, 9 );
                        cb.showTextAligned ( PdfContentByte.ALIGN_CENTER, "" + currentPageNumber + " of " + totalPages,
                                             520, 5, 0 );
                        cb.endText();
                    }
                }
                pageOfCurrentReaderPDF = 0;
            }
            outputStream.flush();
            document.close();
            outputStream.close();
        } catch ( Exception e ) {
            e.printStackTrace();
        } finally {
            if ( document.isOpen() ) {
                document.close();
            }
            try {
                if ( outputStream != null ) {
                    outputStream.close();
                }
            } catch ( IOException ioe ) {
                ioe.printStackTrace();
            }
        }
    }
}
