package LWJGLTest2;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.awt.image.DataBufferByte;
import java.awt.image.DataBufferInt;
import static org.lwjgl.opengl.GL11.*;
import static org.lwjgl.util.glu.GLU.*;
import java.io.IOException;
import java.net.URL;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;
import java.nio.IntBuffer;
import java.util.logging.FileHandler;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.imageio.ImageIO;
import javax.swing.ImageIcon;
import org.lwjgl.BufferUtils;
import org.lwjgl.LWJGLException;
import org.lwjgl.input.Keyboard;
import org.lwjgl.input.Mouse;
import org.lwjgl.opengl.Display;
import org.lwjgl.opengl.DisplayMode;
import org.lwjgl.util.glu.GLU;
import org.newdawn.slick.opengl.Texture;
import org.newdawn.slick.opengl.TextureLoader;
import org.newdawn.slick.util.ResourceLoader;
public class LWJGLTest2 {
    public static final int WIDTH = 640;
    public static final int HEIGHT = 480;
    public static final Logger LOGGER = Logger.getLogger ( LWJGLTest2.class.getName() );
    float xrot;
    float yrot;
    float xspeed;
    float yspeed;
    float zpos = -5f;
    boolean lp;
    boolean fp;
    boolean light;
    final float[] AMBIENT = {.5f, .5f, .5f, 1f};
    final float[] DIFFUSE = {1f, 1f, 1f, 1f};
    final float[] LPOSITION = {0f, 0f, 2f, 1f};
    int filter;
    BufferedImage textureImage;
    Texture texture[] = new Texture[3];
    static {
        try {
            LOGGER.addHandler ( new FileHandler ( "errors.log", true ) );
        } catch ( IOException ex ) {
            LOGGER.log ( Level.WARNING, ex.toString(), ex );
        }
    }
    public static void main ( String[] args ) {
        LWJGLTest2 main = null;
        try {
            main = new LWJGLTest2();
            main.create();
            main.run();
        } catch ( Exception ex ) {
            LOGGER.log ( Level.SEVERE, ex.toString(), ex );
        } finally {
            if ( main != null ) {
                main.destroy();
            }
        }
    }
    public void create() throws LWJGLException {
        Display.setDisplayMode ( new DisplayMode ( WIDTH, HEIGHT ) );
        Display.setFullscreen ( false );
        Display.setTitle ( "LWJGL Test" );
        Display.create();
        Keyboard.create();
        Mouse.setGrabbed ( false );
        Mouse.create();
        initGL();
        resizeGL();
    }
    public void destroy() {
        Mouse.destroy();
        Keyboard.destroy();
        Display.destroy();
    }
    public void initGL() {
        try {
            loadTextures();
        } catch ( IOException ex ) {
            System.err.println ( ex );
            System.exit ( 0 );
        }
        glEnable ( GL_TEXTURE_2D );
        glShadeModel ( GL_SMOOTH );
        glClearColor ( 0f, 0f, 0f, 0.5f );
        glClearDepth ( 1f );
        glEnable ( GL_DEPTH_TEST );
        glDepthFunc ( GL_LEQUAL );
        glHint ( GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST );
        ByteBuffer scratch = ByteBuffer.allocateDirect ( 16 );
        scratch.order ( ByteOrder.nativeOrder() );
        glLight ( GL_LIGHT1, GL_AMBIENT, ( FloatBuffer ) scratch.asFloatBuffer().put ( AMBIENT ).flip() );
        glLight ( GL_LIGHT1, GL_DIFFUSE, ( FloatBuffer ) scratch.asFloatBuffer().put ( AMBIENT ).flip() );
        glLight ( GL_LIGHT1, GL_POSITION, ( FloatBuffer ) scratch.asFloatBuffer().put ( AMBIENT ).flip() );
        glEnable ( GL_LIGHT1 );
    }
    public void resizeGL() {
        glViewport ( 0, 0, WIDTH, HEIGHT );
        glMatrixMode ( GL_PROJECTION );
        glLoadIdentity();
        gluPerspective ( 45f, ( float ) WIDTH / ( float ) HEIGHT, 0.1f, 100f );
        glMatrixMode ( GL_MODELVIEW );
        glLoadIdentity();
    }
    public void run() {
        while ( !Display.isCloseRequested() && !Keyboard.isKeyDown ( Keyboard.KEY_ESCAPE ) ) {
            if ( Display.isVisible() ) {
                checkInput();
                render();
                Display.update();
            }
        }
    }
    public void checkInput() {
        if ( Keyboard.isKeyDown ( Keyboard.KEY_L ) && !lp ) {
            lp = true;
            light = !light;
            if ( light ) {
                glEnable ( GL_LIGHTING );
            } else {
                glDisable ( GL_LIGHTING );
            }
        }
        if ( !Keyboard.isKeyDown ( Keyboard.KEY_L ) ) {
            lp = false;
        }
        if ( Keyboard.isKeyDown ( Keyboard.KEY_F ) && !fp ) {
            fp = true;
            filter = ( filter + 1 ) % 3;
        }
        if ( !Keyboard.isKeyDown ( Keyboard.KEY_F ) ) {
            fp = false;
        }
        if ( Keyboard.isKeyDown ( Keyboard.KEY_PRIOR ) ) {
            zpos -= 0.02f;
        }
        if ( Keyboard.isKeyDown ( Keyboard.KEY_NEXT ) ) {
            zpos += 0.02f;
        }
        if ( Keyboard.isKeyDown ( Keyboard.KEY_UP ) ) {
            xspeed -= 0.001f;
        }
        if ( Keyboard.isKeyDown ( Keyboard.KEY_DOWN ) ) {
            xspeed += 0.001f;
        }
        if ( Keyboard.isKeyDown ( Keyboard.KEY_RIGHT ) ) {
            yspeed -= 0.001f;
        }
        if ( Keyboard.isKeyDown ( Keyboard.KEY_LEFT ) ) {
            yspeed += 0.001f;
        }
    }
    public void render() {
        glClear ( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
        glLoadIdentity();
        glTranslatef ( 0f, 0f, zpos );
        glRotatef ( xrot, 1f, 0f, 0f );
        glRotatef ( yrot, 0f, 1f, 0f );
        texture[filter].bind();
        glBegin ( GL_QUADS );
        glNormal3f ( 0f, 0f, 1f );
        glTexCoord2f ( 0.0f, 0.0f );
        glVertex3f ( -1.0f, -1.0f,  1.0f );
        glTexCoord2f ( 1.0f, 0.0f );
        glVertex3f ( 1.0f, -1.0f,  1.0f );
        glTexCoord2f ( 1.0f, 1.0f );
        glVertex3f ( 1.0f,  1.0f,  1.0f );
        glTexCoord2f ( 0.0f, 1.0f );
        glVertex3f ( -1.0f,  1.0f,  1.0f );
        glNormal3f ( 0f, 0f, -1f );
        glTexCoord2f ( 1.0f, 0.0f );
        glVertex3f ( -1.0f, -1.0f, -1.0f );
        glTexCoord2f ( 1.0f, 1.0f );
        glVertex3f ( -1.0f,  1.0f, -1.0f );
        glTexCoord2f ( 0.0f, 1.0f );
        glVertex3f ( 1.0f,  1.0f, -1.0f );
        glTexCoord2f ( 0.0f, 0.0f );
        glVertex3f ( 1.0f, -1.0f, -1.0f );
        glNormal3f ( 0f, 1f, 0f );
        glTexCoord2f ( 0.0f, 1.0f );
        glVertex3f ( -1.0f,  1.0f, -1.0f );
        glTexCoord2f ( 0.0f, 0.0f );
        glVertex3f ( -1.0f,  1.0f,  1.0f );
        glTexCoord2f ( 1.0f, 0.0f );
        glVertex3f ( 1.0f,  1.0f,  1.0f );
        glTexCoord2f ( 1.0f, 1.0f );
        glVertex3f ( 1.0f,  1.0f, -1.0f );
        glNormal3f ( 0f, -1f, 0f );
        glTexCoord2f ( 1.0f, 1.0f );
        glVertex3f ( -1.0f, -1.0f, -1.0f );
        glTexCoord2f ( 0.0f, 1.0f );
        glVertex3f ( 1.0f, -1.0f, -1.0f );
        glTexCoord2f ( 0.0f, 0.0f );
        glVertex3f ( 1.0f, -1.0f,  1.0f );
        glTexCoord2f ( 1.0f, 0.0f );
        glVertex3f ( -1.0f, -1.0f,  1.0f );
        glNormal3f ( 1f, 0f, 0f );
        glTexCoord2f ( 1.0f, 0.0f );
        glVertex3f ( 1.0f, -1.0f, -1.0f );
        glTexCoord2f ( 1.0f, 1.0f );
        glVertex3f ( 1.0f,  1.0f, -1.0f );
        glTexCoord2f ( 0.0f, 1.0f );
        glVertex3f ( 1.0f,  1.0f,  1.0f );
        glTexCoord2f ( 0.0f, 0.0f );
        glVertex3f ( 1.0f, -1.0f,  1.0f );
        glNormal3f ( -1f, 0f, 0f );
        glTexCoord2f ( 0.0f, 0.0f );
        glVertex3f ( -1.0f, -1.0f, -1.0f );
        glTexCoord2f ( 1.0f, 0.0f );
        glVertex3f ( -1.0f, -1.0f,  1.0f );
        glTexCoord2f ( 1.0f, 1.0f );
        glVertex3f ( -1.0f,  1.0f,  1.0f );
        glTexCoord2f ( 0.0f, 1.0f );
        glVertex3f ( -1.0f,  1.0f, -1.0f );
        glEnd();
        xrot += xspeed;
        yrot += yspeed;
    }
    public void loadTextures() throws IOException {
        texture[0] = TextureLoader.getTexture ( "BMP", this.getClass().getResourceAsStream ( "Crate.bmp" ), true, GL_NEAREST );
        texture[1] = TextureLoader.getTexture ( "BMP", this.getClass().getResourceAsStream ( "Crate.bmp" ), true, GL_LINEAR );
        texture[2] = TextureLoader.getTexture ( "BMP", this.getClass().getResourceAsStream ( "Crate.bmp" ), true );
        texture[2].bind();
        org.lwjgl.opengl.GL30.glGenerateMipmap ( GL_TEXTURE_2D );
    }
}
