class CloseableBlockingQueue<E> extends ArrayBlockingQueue<E> {
    private volatile boolean closed = false;
    private final Container<Thread> blocked;
    public CloseableBlockingQueue ( int queueLength ) {
        super ( queueLength );
        blocked = new Container<Thread> ( queueLength );
    }
    @Override
    public void put ( E e ) throws InterruptedException {
        if ( !closed ) {
            Thread t = Thread.currentThread();
            Container.Node<Thread> n = blocked.add ( t );
            try {
                super.put ( e );
            } finally {
                blocked.remove ( n, t );
            }
        }
    }
    @Override
    public E poll() {
        E it = null;
        if ( !closed ) {
            it = super.poll();
        }
        return it;
    }
    @Override
    public E poll ( long l, TimeUnit tu ) throws InterruptedException {
        E it = null;
        if ( !closed ) {
            it = super.poll ( l, tu );
        }
        return it;
    }
    boolean isClosed() {
        return closed;
    }
    void close() {
        closed = true;
        for ( Thread t : blocked ) {
            t.interrupt();
        }
    }
    @Override
    public String toString() {
        return blocked.toString();
    }
}
