public List<Class<?>> getTestableClasses ( String packageName, ClassLoader loader ) {
    System.out.println ( "Exploring package: " + packageName );
    List<Class<?>> ret = new ArrayList<Class<?>>();
    List<File> directories = new ArrayList<File>();
    if ( loader == null ) {
        loader = Thread.currentThread().getContextClassLoader();
    }
    String path = packageName.replace ( '.', '/' );
    try {
        Enumeration<URL> res = loader.getResources ( path );
        while ( res.hasMoreElements() ) {
            String dirPath = URLDecoder.decode ( res.nextElement()
                                                 .getPath(), "UTF-8" );
            File dir = new File ( dirPath );
            for ( File file : dir.listFiles() ) {
                if ( file.isDirectory() ) {
                    ret.addAll ( getTestableClasses ( packageName + '.' + file.getName(), loader ) );
                }
            }
        }
    } catch ( IOException e ) {
        System.out.println ( "Failed to load resources for [" + packageName + ']' );
    }
    URL tmp = loader.getResource ( path );
    System.out.println ( tmp );
    if ( tmp == null ) {
        return ret;
    }
    File currDir = new File ( tmp.getPath() );
    for ( String classFile : currDir.list() ) {
        if ( classFile.endsWith ( ".class" ) ) {
            try {
                Class<?> add = Class.forName ( packageName + '.' +
                                               classFile.substring ( 0, classFile.length() - 6 ) );
                if ( add.isAnnotationPresent ( UnitTestable.class ) ) {
                    ret.add ( add );
                } else {
                    System.out.println ( add.getName() + " is not a UnitTestable class" );
                }
            } catch ( NoClassDefFoundError e ) {
                System.out.println ( "We have found class [" + classFile +
                                     "], and couldn't load it." );
            } catch ( ClassNotFoundException e ) {
                System.out.println ( "We could not find class [" + classFile + ']' );
            }
        }
    }
    return ret;
}
