import java.awt.*;
import java.awt.event.*;
import java.applet.*;
public class GameOfLifeApplet extends Applet implements ActionListener {
    private Button nextButton = null;
    private Ecosystem ecosystem;
    public void init() {
        add ( getGui() );
    }
    public Component getGui() {
        Panel gui = new Panel ( new BorderLayout ( 3, 3 ) );
        ecosystem = new Ecosystem();
        gui.add ( ecosystem, BorderLayout.CENTER );
        nextButton = new Button ( "Next Stage" );
        Panel p = new Panel ( new FlowLayout() );
        p.add ( nextButton );
        gui.add ( p, BorderLayout.SOUTH );
        nextButton.addActionListener ( this );
        return gui;
    }
    public void actionPerformed ( ActionEvent evt ) {
        if ( evt.getSource() == nextButton ) {
            System.out.println ( "I got clicked!" );
            ecosystem.nextIteration();
            ecosystem.repaint();
        }
    }
    public static void main ( String[] args ) {
        GameOfLifeApplet gola = new GameOfLifeApplet();
        javax.swing.JOptionPane.showMessageDialog ( null, gola.getGui() );
    }
}
class Ecosystem extends Panel implements MouseListener {
    private GameOfLife game = new GameOfLife();
    private int xCo, yCo;
    private int diff_x, diff_y;
    private int size = 300;
    Ecosystem() {
        diff_x = diff_y = 600 / game.getGridSize();
        setPreferredSize ( new Dimension ( size, size ) );
        addMouseListener ( this );
    }
    public void nextIteration() {
        game.nextIteration();
    }
    private void drawEmptyGrid ( Graphics g ) {
        g.setColor ( Color.white );
        g.fillRect ( 0, 0, size, size );
        g.setColor ( Color.black );
        for ( int i = 0; i < game.getGridSize(); ++i ) {
            g.drawLine ( 0, i * diff_x, size, i * diff_x );
            g.drawLine ( i * diff_x, 0, i * diff_x, size );
        }
        g.setColor ( Color.white );
    }
    public void paint ( Graphics g ) {
        drawEmptyGrid ( g );
        g.setColor ( Color.red );
        for ( int i = 0; i < game.getGridSize(); ++i ) {
            for ( int j = 0; j < game.getGridSize(); ++j ) {
                if ( game.grid[i][j] ) {
                    g.fillRect ( i * diff_x, j * diff_y, diff_x, diff_y );
                }
            }
        }
        g.setColor ( Color.white );
    }
    public void mouseClicked ( MouseEvent me ) {
        Point point = me.getPoint();
        xCo = ( int ) point.getX();
        yCo = ( int ) point.getY();
        int x_init = xCo / diff_x;
        int y_init = yCo / diff_y;
        System.out.println ( x_init + "x" + y_init );
        game.grid[x_init][y_init] = true;
        repaint();
    }
    public void mousePressed ( MouseEvent me ) {}
    public void mouseReleased ( MouseEvent me ) {}
    public void mouseEntered ( MouseEvent me ) {}
    public void mouseExited ( MouseEvent me ) {}
}
class GameOfLife {
    private final int GRID_SIZE = 60;
    public boolean [][] grid = new boolean[GRID_SIZE][GRID_SIZE];
    public GameOfLife() {
        for ( int i = 0; i < GRID_SIZE; ++i ) {
            for ( int j = 0; j < GRID_SIZE; ++j ) {
                grid[i][j] = false;
            }
        }
    }
    public int getGridSize() {
        return GRID_SIZE;
    }
    public int getLiveNeighbors ( int i, int j ) {
        int neighbors = 0;
        for ( int tmp_i = i - 1; tmp_i <= i + 1; ++tmp_i ) {
            for ( int tmp_j = j - 1; tmp_j <= j + 1; ++tmp_j ) {
                if ( tmp_i < 0 || tmp_i >= GRID_SIZE || tmp_j < 0 || tmp_j >= GRID_SIZE ) {
                } else {
                    if ( grid[tmp_i][tmp_j] ) {
                        neighbors++;
                    }
                }
            }
        }
        return neighbors;
    }
    public void nextIteration() {
        boolean [][] newGrid = new boolean[GRID_SIZE][GRID_SIZE];
        for ( int i = 0; i < GRID_SIZE; ++i ) {
            for ( int j = 0; j < GRID_SIZE; ++j ) {
                newGrid[i][j] = grid[i][j];
            }
        }
        for ( int i = 0; i < GRID_SIZE; ++i ) {
            for ( int j = 0; j < GRID_SIZE; ++j ) {
                int my_neighbors = getLiveNeighbors ( i, j );
                if ( !newGrid[i][j] && my_neighbors == 3 ) {
                    grid[i][j] = true;
                } else if ( newGrid[i][j] && ( my_neighbors == 2 || my_neighbors == 3 ) ) {
                    grid[i][j] = true;
                } else {
                    grid[i][j] = false;
                }
            }
        }
        System.out.println ( "Change of assignment" );
    }
}
