public class DownloaderInputStream extends InputStream {
    public static interface IDownloadListener {
        public void onComplete ( final byte[] buf, final int offset, final int length );
    }
    private final class ByteArrayOutputStreamX extends ByteArrayOutputStream {
        public ByteArrayOutputStreamX ( final int size ) {
            super ( size );
        }
        public byte[] getBuffer() {
            return buf;
        }
    }
    private final class Downloader extends Object implements Runnable {
        private final InputStream is;
        public Downloader ( final InputStream is ) {
            this.is = is;
        }
        public void run() {
            int read = 0;
            byte[] buf = new byte[16 * 1024];
            try {
                while ( ( read = is.read ( buf ) ) != -1 ) {
                    if ( read > 0 ) {
                        content.write ( buf, 0, read );
                        downloadedBytes += read;
                    } else {
                        Thread.sleep ( 50 );
                    }
                }
            } catch ( Exception e ) {
                e.printStackTrace();
            }
            listener.onComplete ( content.getBuffer(), 0  , downloadedBytes );
        }
    }
    private final int contentLength;
    private final IDownloadListener listener;
    private ByteArrayOutputStreamX content;
    private volatile int downloadedBytes;
    private volatile int readBytes;
    public DownloaderInputStream ( final int contentLength, final InputStream is, final IDownloadListener listener ) {
        this.contentLength = contentLength;
        this.listener = listener;
        this.content = new ByteArrayOutputStreamX ( contentLength );
        this.downloadedBytes = 0;
        this.readBytes = 0;
        new Thread ( new Downloader ( is ) ).start();
    }
    public int getDownloadedBytes() {
        return downloadedBytes;
    }
    public int getReadBytes() {
        return readBytes;
    }
    @Override
    public int available() throws IOException {
        return downloadedBytes - readBytes;
    }
    @Override
    public int read() throws IOException {
        return 0;
    }
    @Override
    public int read ( byte[] b, int off, int len ) throws IOException {
        if ( readBytes == contentLength ) {
            return -1;
        }
        int tr = 0;
        while ( ( tr = Math.min ( downloadedBytes - readBytes, len ) ) == 0 ) {
            try {
                Thread.sleep ( 100 );
            } catch ( Exception e ) {
            }
        }
        byte[] buf = content.getBuffer();
        System.arraycopy ( buf, readBytes, b, off, tr );
        readBytes += tr;
        return tr;
    }
    @Override
    public long skip ( long n ) throws IOException {
        return n;
    }
}
