import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SwingUtilities;
public class Test {
    public static class WrapLayout extends FlowLayout {
        private Dimension preferredLayoutSize;
        public WrapLayout() {
            super();
        }
        public WrapLayout ( int align ) {
            super ( align );
        }
        public WrapLayout ( int align, int hgap, int vgap ) {
            super ( align, hgap, vgap );
        }
        @Override
        public Dimension preferredLayoutSize ( Container target ) {
            return layoutSize ( target, true );
        }
        @Override
        public Dimension minimumLayoutSize ( Container target ) {
            Dimension minimum = layoutSize ( target, false );
            minimum.width -= getHgap() + 1;
            return minimum;
        }
        private Dimension layoutSize ( Container target, boolean preferred ) {
            synchronized ( target.getTreeLock() ) {
                int targetWidth = target.getSize().width;
                if ( targetWidth == 0 ) {
                    targetWidth = Integer.MAX_VALUE;
                }
                int hgap = getHgap();
                int vgap = getVgap();
                Insets insets = target.getInsets();
                int horizontalInsetsAndGap = insets.left + insets.right + hgap * 2;
                int maxWidth = targetWidth - horizontalInsetsAndGap;
                Dimension dim = new Dimension ( 0, 0 );
                int rowWidth = 0;
                int rowHeight = 0;
                int nmembers = target.getComponentCount();
                for ( int i = 0; i < nmembers; i++ ) {
                    Component m = target.getComponent ( i );
                    if ( m.isVisible() ) {
                        Dimension d = preferred ? m.getPreferredSize() : m.getMinimumSize();
                        if ( rowWidth + d.width > maxWidth ) {
                            addRow ( dim, rowWidth, rowHeight );
                            rowWidth = 0;
                            rowHeight = 0;
                        }
                        if ( rowWidth != 0 ) {
                            rowWidth += hgap;
                        }
                        rowWidth += d.width;
                        rowHeight = Math.max ( rowHeight, d.height );
                    }
                }
                addRow ( dim, rowWidth, rowHeight );
                dim.width += horizontalInsetsAndGap;
                dim.height += insets.top + insets.bottom + vgap * 2;
                Container scrollPane = SwingUtilities.getAncestorOfClass ( JScrollPane.class, target );
                if ( scrollPane != null ) {
                    dim.width -= hgap + 1;
                }
                return dim;
            }
        }
        private void addRow ( Dimension dim, int rowWidth, int rowHeight ) {
            dim.width = Math.max ( dim.width, rowWidth );
            if ( dim.height > 0 ) {
                dim.height += getVgap();
            }
            dim.height += rowHeight;
        }
    }
    public static class MyTabbedPane extends JPanel {
        private JPanel buttonPanel;
        private JPanel currentview;
        private Tab currentTab;
        private class Tab {
            String name;
            JComponent component;
        }
        private List<Tab> tabs = new ArrayList<Tab>();
        public MyTabbedPane() {
            super ( new BorderLayout() );
            buttonPanel = new JPanel ( new WrapLayout() );
            currentview = new JPanel();
            add ( buttonPanel, BorderLayout.NORTH );
            add ( currentview );
        }
        public void addTab ( String name, JComponent tabView, int index ) {
            if ( index < 0 || index > tabs.size() ) {
                throw new IllegalArgumentException ( "Index out of bounds" );
            }
            final Tab tab = new Tab();
            tab.component = tabView;
            tab.name = name;
            tabs.add ( index, tab );
            JButton b = new JButton ( name );
            b.addActionListener ( new ActionListener() {
                @Override
                public void actionPerformed ( ActionEvent e ) {
                    setCurrentTab ( tab );
                }
            } );
            buttonPanel.add ( b, index );
            buttonPanel.validate();
        }
        public void removeTab ( int i ) {
            Tab tab = tabs.remove ( i );
            if ( tab == currentTab ) {
                if ( tabs.size() > 0 ) {
                    if ( i < tabs.size() ) {
                        setCurrentTab ( tabs.get ( i ) );
                    } else {
                        setCurrentTab ( tabs.get ( i - 1 ) );
                    }
                } else {
                    setCurrentTab ( null );
                }
            }
            buttonPanel.remove ( index );
        }
        void setCurrentTab ( final Tab tab ) {
            if ( currentTab == tab ) {
                return;
            }
            if ( currentTab != null ) {
                currentview.remove ( currentTab.component );
            }
            if ( tab != null ) {
                currentview.add ( tab.component );
            }
            currentTab = tab;
            currentview.validate();
        }
    }
    public static void main ( String[] args ) {
        JFrame frame = new JFrame();
        MyTabbedPane tabbedPane = new MyTabbedPane();
        for ( int i = 0; i < 100; i++ ) {
            tabbedPane.addTab ( "Button " + ( i + 1 ), new JLabel ( "Dummy Label " + ( i + 1 ) ), i );
        }
        frame.add ( tabbedPane );
        frame.pack();
        frame.setSize ( new Dimension ( 1000, 800 ) );
        frame.setDefaultCloseOperation ( JFrame.EXIT_ON_CLOSE );
        frame.setVisible ( true );
    }
}
