private final class ChildExecutor implements Executor, Runnable {
    private final Queue<Runnable> tasks = QueueFactory
                                          .createQueue ( Runnable.class );
    private final AtomicBoolean isRunning = new AtomicBoolean();
    public void execute ( Runnable command ) {
        tasks.add ( command );
        if ( !isRunning.get() ) {
            doUnorderedExecute ( this );
        } else {
        }
    }
    public void run() {
        boolean acquired = false;
        if ( isRunning.compareAndSet ( false, true ) ) {
            acquired = true;
            try {
                Thread thread = Thread.currentThread();
                for ( ;; ) {
                    final Runnable task = tasks.poll();
                    if ( task == null ) {
                        break;
                    }
                    boolean ran = false;
                    beforeExecute ( thread, task );
                    try {
                        task.run();
                        ran = true;
                        onAfterExecute ( task, null );
                    } catch ( RuntimeException e ) {
                        if ( !ran ) {
                            onAfterExecute ( task, e );
                        }
                        throw e;
                    }
                }
            } finally {
                isRunning.set ( false );
            }
        }
        if ( acquired && !isRunning.get() && tasks.peek() != null ) {
            doUnorderedExecute ( this );
        }
    }
}
