private void map ( Object mapFrom, Object mapTo ) {
    setMapFilter ( mapFrom, mapTo );
    for ( Method sMethod : filterMap.keySet() ) {
        try {
            Method gMethod = filterMap.get ( sMethod );
            Object attValue = gMethod.invoke ( mapFrom, new Object[0] );
            Class<?> fromType = gMethod.getReturnType();
            Class<?> toType   = sMethod.getParameterTypes() [0];
            if ( !toType.isAssignableFrom ( fromType ) && Objects.isNotNull ( attValue ) ) {
                attValue = parseType ( attValue, toType );
            }
            sMethod.invoke ( mapTo, attValue );
        } catch ( IllegalArgumentException e ) {
            Logger.getLogger ( Constants.APP_LOGGER ).fine ( "Exception in DataFormMappingService.map: "
                    + "IllegalArgumentException " + e.getMessage() );
            continue;
        } catch ( IllegalAccessException e ) {
            Logger.getLogger ( Constants.APP_LOGGER ).fine ( "Exception in DataFormMappingService.map: "
                    + "IllegalAccessException " + e.getMessage() );
            continue;
        } catch ( InvocationTargetException e ) {
            Logger.getLogger ( Constants.APP_LOGGER ).fine ( "Exception in DataFormMappingService.map: "
                    + "InvocationTargetException " + e.getMessage() );
            continue;
        }
    }
}
private Object parseType ( Object attValue, Class<?> type ) {
    Object newAttValue = null;
    if ( isLiteral ( type ) ) {
        newAttValue = attValue.toString();
    } else if ( isByte ( type ) ) {
        newAttValue = Byte.valueOf ( attValue.toString() );
    } else if ( isInt ( type ) ) {
        newAttValue = Integer.valueOf ( attValue.toString() );
    } else if ( isShort ( type ) ) {
        newAttValue = Short.valueOf ( attValue.toString() );
    } else if ( isLong ( type ) ) {
        newAttValue = Long.valueOf ( attValue.toString() );
    } else if ( isFloat ( type ) ) {
        newAttValue = Float.valueOf ( attValue.toString() );
    } else if ( isDouble ( type ) ) {
        newAttValue = Double.valueOf ( attValue.toString() );
    } else if ( isBoolean ( type ) ) {
        newAttValue = Boolean.valueOf ( attValue.toString() );
    }
    return newAttValue;
}
