private class MidiSender extends Thread {
    private boolean _raiseStop = false;
    private Object _sendLock = new Object();
    private LinkedList<ByteBuffer> _outputQueue = new LinkedList<ByteBuffer>();
    public void queue ( ByteBuffer buffer ) {
        synchronized ( _sendLock ) {
            _outputQueue.add ( buffer );
            _sendLock.notify();
        }
    }
    public void raiseStop() {
        synchronized ( this ) {
            _raiseStop = true;
        }
        synchronized ( _sendLock ) {
            _sendLock.notify();
        }
    }
    public void run() {
        while ( true ) {
            synchronized ( this ) {
                if ( _raiseStop ) {
                    return;
                }
            }
            ByteBuffer currentBuffer = null;
            synchronized ( _sendLock ) {
                if ( !_outputQueue.isEmpty() ) {
                    currentBuffer = _outputQueue.removeFirst();
                }
            }
            while ( currentBuffer != null ) {
                int transferred = _connection.bulkTransfer ( _outPort, currentBuffer.array(), currentBuffer.position(), 100 );
                if ( transferred < 0 ) {
                    Log.w ( _tag, "Failed to send MIDI packet" );
                }
                synchronized ( _sendLock ) {
                    if ( !_outputQueue.isEmpty() ) {
                        currentBuffer = _outputQueue.removeFirst();
                    } else {
                        currentBuffer = null;
                    }
                }
            }
            synchronized ( _sendLock ) {
                try {
                    _sendLock.wait();
                } catch ( InterruptedException e ) {
                }
            }
        }
    }
}
