public class RestClient {
    public enum RequestMethod {
        GET,
        POST
    }
    private ArrayList <NameValuePair> params;
    private ArrayList <NameValuePair> headers;
    private String url;
    private int responseCode;
    private String message;
    private String response;
    public String getResponse() {
        return response;
    }
    public String getErrorMessage() {
        return message;
    }
    public int getResponseCode() {
        return responseCode;
    }
    public RestClient ( String url ) {
        this.url = url;
        params = new ArrayList<NameValuePair>();
        headers = new ArrayList<NameValuePair>();
    }
    public void AddParam ( String name, String value ) {
        params.add ( new BasicNameValuePair ( name, value ) );
    }
    public void AddHeader ( String name, String value ) {
        headers.add ( new BasicNameValuePair ( name, value ) );
    }
    public void Execute ( RequestMethod method ) throws Exception {
        switch ( method ) {
        case GET: {
            String combinedParams = "";
            if ( !params.isEmpty() ) {
                combinedParams += "?";
                for ( NameValuePair p : params ) {
                    String paramString = p.getName() + "=" + URLEncoder.encode ( p.getValue(), "UTF-8" );
                    if ( combinedParams.length() > 1 ) {
                        combinedParams  +=  "&" + paramString;
                    } else {
                        combinedParams += paramString;
                    }
                }
            }
            HttpGet request = new HttpGet ( url + combinedParams );
            for ( NameValuePair h : headers ) {
                request.addHeader ( h.getName(), h.getValue() );
            }
            executeRequest ( request, url );
            break;
        }
        case POST: {
            HttpPost request = new HttpPost ( url );
            for ( NameValuePair h : headers ) {
                request.addHeader ( h.getName(), h.getValue() );
            }
            JSONObject jo = new JSONObject();
            if ( !params.isEmpty() ) {
                for ( int i = 0; i < params.size(); i++ ) {
                    jo.put ( params.get ( i ).getName(), params.get ( i ).getValue() );
                }
                StringEntity se = new StringEntity ( jo.toString() );
                se.setContentType ( "text/xml" );
                se.setContentEncoding ( new BasicHeader ( HTTP.CONTENT_TYPE, "application/json" ) );
                request.setEntity ( se );
            }
            executeRequest ( request, url );
            break;
        }
        }
    }
    private void executeRequest ( HttpUriRequest request, String url ) {
        HttpClient client = HttpClientFactory.getThreadSafeClient();
        HttpResponse httpResponse;
        try {
            httpResponse = client.execute ( request );
            responseCode = httpResponse.getStatusLine().getStatusCode();
            message = httpResponse.getStatusLine().getReasonPhrase();
            HttpEntity entity = httpResponse.getEntity();
            if ( entity != null ) {
                InputStream instream = entity.getContent();
                response = convertStreamToString ( instream );
                instream.close();
            }
        } catch ( ClientProtocolException e )  {
            client.getConnectionManager().shutdown();
            e.printStackTrace();
        } catch ( IOException e ) {
            client.getConnectionManager().shutdown();
            e.printStackTrace();
        }
    }
    private static String convertStreamToString ( InputStream is ) {
        BufferedReader reader = new BufferedReader ( new InputStreamReader ( is ) );
        StringBuilder sb = new StringBuilder();
        String line = null;
        try {
            while ( ( line = reader.readLine() ) != null ) {
                sb.append ( line + "\n" );
            }
        } catch ( IOException e ) {
            e.printStackTrace();
        } finally {
            try {
                is.close();
            } catch ( IOException e ) {
                e.printStackTrace();
            }
        }
        return sb.toString();
    }
}
